* SUBROUTINE UA0JS3             ALL SYSTEMS                 98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION
* WHICH IS DEFINED AS A SUM OF SQUARES.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  RU  X(NF)  VECTORS OF VARIABLES.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RI  XO(NF)  SAVED VECTORS OF VARIABLES.
*  RO  AG(MA)  SPARSE JACOBIAN MATRIX.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFO(NA)  SAVED VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IG(NF) VECTOR DEFINING GROUPS CONTAINING JACOBIAN COLUMNS.
*  II  IVAR  INDEX OF GROUP CONTAINING JACOBIAN COLUMNS.
*  II  IND  GROUP SELECTION INDICATOR.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*
* COMMON DATA :
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S  UYPRO1  PROLOGUE.
*  S  UYEPI1  EPILOGUE.
*  S  UYSET0  STATUS DEFINITION.
*  S  UXVSET  INITIATION OF A VECTOR.
*  S  UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*
      SUBROUTINE UA0JS3(NF,NA,MA,X,XN,XO,AG,IAG,JAG,AF,AFO,IG,IVAR,IND,
     & ETA1)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,MA,IAG(NA+1),JAG(MA),IG(NF),IVAR,IND
      REAL*8 X(NF),XN(NF),XO(NF),AG(MA),AF(NA),AFO(NA),ETA1
      INTEGER I,J,KA
      REAL*8 ETA,XSTEP
      IF (TSS(NS).NE.'UAGX') CALL UYPRO1('UAGX',0)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      IND=1
      DO 2 I=1,NF
      X(I)=XO(I)
      IF (IG(I).EQ.IVAR) THEN
      IND=0
      IF (NORMF.GT.0) THEN
      XSTEP=XN(I)
      ELSE
      XSTEP=ONE
      ENDIF
      XSTEP=ETA*MAX(ABS(X(I)),XSTEP)*SIGN(ONE,X(I))
      X(I)=X(I)+XSTEP
      ENDIF
    2 CONTINUE
      IF (IND.NE.0) GO TO 6
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    3 DO 5 KA=1,NA
      DO 4 J=IAG(KA),IAG(KA+1)-1
      I=ABS(JAG(J))
      IF (IG(I).EQ.IVAR) THEN
      AG(J)=(AF(KA)-AFO(KA))/(X(I)-XO(I))
      ENDIF
    4 CONTINUE
    5 CONTINUE
    6 CALL UYEPI1(0)
      RETURN
      END
