* SUBROUTINE UA1DQ2             ALL SYSTEMS                 98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION
* WHICH IS DEFINED AS A SUM OF SQUARES.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF MESH POINTS.
*  II  NE  DECLARED NUMBER OF DIFFERENTIAL EQUATIONS.
*  II  MEL  ACTUAL NUMBER OF DIFFERENTIAL EQUATIONS.
*  IU  KA  INDEX OF THE APPROXIMATED FUNCTION.
*  IU  KE  INDEX OF THE DIFFERENTIAL EQUATION.
*  RI  AT(NA)  MESH POINTS.
*  RO  AY(NA*NE)  FIELD OF SOLUTION VECTORS IN THE MESH POINTS.
*  RU  YA(MEL)  SOLUTION VECTOR.
*  RI  EY(NE)  REQUIRED TERMINAL SOLUTION VECTOR.
*  RI  EW(NE)  VECTOR WITH WEIGHT COEFFICIENTS.
*  RO  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GE(NF)  GRADIENT OF THE RIGHT HAND SIDE FUNCTION.
*  RI  ETA9  MAXIMUM VALUE LESS THEN INFINITY.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RU  TA  VALUE OF THE INTEGRATION PARAMETER.
*  RI  TAMAX  MAXIMUM VALUE OF THE INTEGRATION PARAMETER.
*  RU  HE  ACTUAL STEPSIZE.
*  II  MED  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  II  MED1  CHOICE OF STEP SELECTION.
*  II  MED2  NUMBER OF RESETING SOLUTIONS.
*  IO  IER  ERROR INDICATOR.
*
* COMMON DATA :
*  II  KD  DEGREE OF REQUIRED DERVATIVES.
*  II  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  II  KDF  DEGREE OF ANALYTICALLY COMPUTED DERVATIVES.
*  II  KDA  DEGREE OF ANALYTICALLY COMPUTED DERVATIVES.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S  UYPRO1  PROLOGUE.
*  S  UYEPI1  EPILOGUE.
*  S  UYSET0  STATUS DEFINITION.
*  S  UXVCOP  COPY OF A VECTOR.
*  S  UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  S  UXVNEG  COPY OF A VECTOR WITH CHANGE OF THE SIGN.
*  S  UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE UA1DQ2(NF,NA,NE,MEL,KA,KE,AT,AY,YA,EY,EW,G,GE,ETA9,
     & F,TA,TAMAX,HE,MED,MED1,MED2,IER)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,NE,MEL,KA,KE,MED,MED1,MED2,IER
      REAL*8 AT(NA),AY(NA*NE),YA(*),EY(NE),EW(NE),G(NF),GE(NF),
     & ETA9,F,TA,TAMAX,HE
      REAL*8 TEMP
      INTEGER KDS,MEDS
      SAVE KDS,MEDS,TEMP
      IF (TSS(NS).NE.'UAXX') CALL UYPRO1('UAXX',-1)
      GO TO (1,2,6,8) ISP(NS)
    1 CONTINUE
      MEDS=MED
      KDS=KD
      IF (KD.LT.0) GO TO 9
      IF (LD.GE.0) GO TO 4
      MED=MEDS
      MED1=1
      MED2=0
      IF (KDA.LT.0) THEN
      MEL=NE
      ELSE
      MEL=NE+1
      YA(MEL)=ZERO
      ENDIF
      TA=AT(1)
      IF (KDY.LT.0) CALL UXVCOP(NE,AY,YA)
      KD=0
      HE=ZERO
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      IF (IER.LT.0) THEN
      F=ETA9
      ITERM=MAX(-96,-90+IER)
      CALL UOERR1('UA1DQ2',-ITERM)
      TDXX=TEDX
      GO TO 9
      ENDIF
      IF (ITERM.LT.0) ITERM=0
      KD=KDS
      IF (MED.EQ.1) NA=KA-1
      IF (KDA.LT.0) THEN
      F=ZERO
      ELSE
      F=YA(MEL)
      ENDIF
      IF (KDF.LT.0) GO TO 4
      DO 3 KE=1,NE
      F=F+HALF*EW(KE)*(YA(KE)-EY(KE))**2
    3 CONTINUE
    4 IF (KD.LT.1) GO TO 9
      IF (LD.GE.1) GO TO 9
      MED=0
      IF (MEDS.GT.0) THEN
      MED1=2
      MED2=1
      ELSE
      MED1=1
      MED2=0
      HE=ZERO
      ENDIF
      MEL=2*NE+NF
      IF (KDF.LT.0) THEN
      CALL UXVSET(NE,ZERO,YA(NE+1))
      ELSE
      DO 5 KE=1,NE
      YA(NE+KE)=-EW(KE)*(YA(KE)-EY(KE))
    5 CONTINUE
      ENDIF
      CALL UXVSET(NF,ZERO,YA(2*NE+1))
      TA=TAMAX
      TEMP=TAMAX
      TAMAX=AT(1)
      CALL UYSET0(1,3)
      RETURN
    6 CONTINUE
      CALL UXVNEG(NF,YA(2*NE+1),G)
      TAMAX=TEMP
      IF (KDY.LE.0) GO TO 9
      KE=1
    7 IF (KE.GT.NE) GO TO 9
      CALL UYSET0(2,4)
      RETURN
    8 CONTINUE
      CALL UXVDIR(NF,-YA(NE+KE),GE,G,G)
      KE=KE+1
      GO TO 7
    9 CALL UYEPI1(-1)
      MED=MEDS
      LD=KD
      RETURN
      END
