* SUBROUTINE UA1MX4             ALL SYSTEMS                 05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION
* WHICH IS DEFINED AS A SUM OF SQUARES.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  KA  INDEX OF THE APPROXIMATED FUNCTION.
*  II  KAP  OPTIMUM INDEX OF THE APPROXIMATED FUNCTION.
*  RO  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  FA  VALUE OF THE SELECTED FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*
* COMMON DATA :
*  II  KD  DEGREE OF REQUIRED DERVATIVES.
*  IO  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  IU  NAV  NUMBER OF APPROXIMATED FUNCTION VALUES COMPUTED.
*  IU  NAG  NUMBER OF APPROXIMATED FUNCTION GRADIENTS COMPUTED.
*  IU  NFV  NUMBER OF OBJECTIVE FUNCTION VALUES COMPUTED.
*  IU  NFG  NUMBER OF OBJECTIVE FUNCTION GRADIENTS COMPUTED.
*  II  IEXT  TYPE OF EXTREMUM. IEXT<0-MINIMAX. IEXT=0-CHEBYSHEV
*         APPROXIMATION, IEXT>0-MAXIMIN.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S  UYPRO1  PROLOGUE.
*  S  UYEPI1  EPILOGUE.
*  S  UYSET0  STATUS DEFINITION.
*  S  UXVCOP  COPYING OF A VECTOR.
*  S  UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*
      SUBROUTINE UA1MX4(NF,NA,NAL,MA,KA,KAP,G,AG,IAG,JAG,FA,F)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NA,NAL,MA,KA,KAP,IAG(NA+1),JAG(MA)
      REAL*8 G(NF),AG(MA),FA,F
      REAL*8 FTEMP,FVAL
      INTEGER J,JP,K,KK,L
      SAVE FVAL
      SAVE KK
      IF (TSS(NS).NE.'UAXX') CALL UYPRO1('UAXX',-1)
      GO TO (1,3,5) ISP(NS)
    1 CONTINUE
      IF (KD.LE.LD) GO TO 8
      IF (KD.GE.1.AND.LD.LT.1) CALL UXVSET(NF,ZERO,G)
      KA=1
    2 IF (KA.GT.NA) GO TO 7
      IF (LD.GE.0) GO TO 6
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
      IF (IEXT.EQ.0.AND.FA.GE.ZERO.OR.IEXT.LT.0) THEN
      FTEMP= FA
      K= 1
      ELSE
      FTEMP=-FA
      K=-1
      ENDIF
      IF (KA.EQ.1) THEN
      FVAL=FTEMP
      KAP=KA
      KK=K
      ELSE IF (FVAL.LT.FTEMP) THEN
      FVAL=FTEMP
      KAP=KA
      KK=K
      ENDIF
      IF (KD.LT.1) GO TO 6
      IF (LD.GE.1) GO TO 6
      CALL UYSET0(2,3)
      RETURN
    5 CONTINUE
    6 KA=KA+1
      GO TO 2
    7 IF (KD.GE.0.AND.LD.LT.0) F=FVAL
      IF (KD.GE.1.AND.LD.LT.1) THEN
      K=IAG(KAP)
      L=IAG(KAP+1)-K
      DO 9 J=1,L
      JP=ABS(JAG(K))
      IF (KK.GE.0) THEN
      G(JP)= AG(K)
      ELSE
      G(JP)=-AG(K)
      ENDIF
      K=K+1
    9 CONTINUE
      ENDIF
      NFV=NAV/(NA-NAL)
      NFG=NAG/(NA-NAL)
      LD=KD
    8 CALL UYEPI1(-1)
      RETURN
      END
