* SUBROUTINE UCEND1             ALL SYSTEMS                   98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE SPARSE SCHUR COMPLEMENT FOR THE KARUSH-KUHN-TUCKER
* MATRIX WITH THE DIAGONAL HESSIAN BLOCK.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINT FUNCTIONS.
*  II  MC  NUMBER OF NONZERO ELEMENTS IN THE FIELD CG.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE SCHUR COMPLEMENT.
*  II  ND  ACTUAL NUMBER OF DENSE ROWS.
*  RO  B(M)  ELEMENTS OF THE SPARSE MATRIX B.
*  II  IB(NC+1)  POINTERS OF THE DIAGONAL ELEMENTS OF B.
*  II  JB(M)  INDICES OF THE NONZERO ELEMENTS OF B.
*  II  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  II  KCG(NF)  INDICATION OF DENSE ROWS.
*  RO  CGD(NC*ND)  ELEMENTS OF DENSE ROWS.
*  RI  DD(ND)  DENSE PART OF DIAGONAL WEIGHTING MATRIX.
*  RI  D(NF)  DIAGONAL WEIGHTING MATRIX.
*
      SUBROUTINE UCEND1(NF,NC,MC,M,ND,B,IB,JB,CG,ICG,JCG,KCG,CGD,DD,D)
      INTEGER NF,NC,MC,M,ND,IB(NC+1),JB(M),ICG(NC+1),JCG(MC),KCG(NF)
      REAL*8 B(M),CG(MC),CGD(NC*ND),DD(ND),D(NF)
      INTEGER I1,I2,II,K,KK,K1,K2,L,LL,L2,KC
      IF (ND.GT.0) CALL UXVSET(NC*ND,0.0D 0,CGD)
      DO 4 KC=1,NC
      I1=IB(KC)
      I2=IB(KC+1)-1
      K1=ICG(KC)
      K2=ICG(KC+1)-1
      DO 2 II=I1,I2
      B(II)=0.0D 0
      L=JB(II)
      KK=K1
      LL=ICG(L)
      L2=ICG(L+1)-1
      K=JCG(KK)
      L=JCG(LL)
    1 IF (KCG(ABS(K)).NE.0.OR.K.LE.0.OR.K.LT.L) THEN
      IF (KK.GE.K2) GO TO 2
      KK=KK+1
      K=JCG(KK)
      GO TO 1
      ELSE IF (KCG(ABS(L)).NE.0.OR.L.LE.0.OR.L.LT.K) THEN
      IF (LL.GE.L2) GO TO 2
      LL=LL+1
      L=JCG(LL)
      GO TO 1
      ELSE
      B(II)=B(II)+D(K)*CG(KK)*CG(LL)
      IF(KK.GE.K2.OR.LL.GE.L2) GO TO 2
      KK=KK+1
      LL=LL+1
      K=JCG(KK)
      L=JCG(LL)
      GO TO 1
      ENDIF
    2 CONTINUE
      IF (ND.GT.0) THEN
      DO 3 KK=K1,K2
      K=JCG(KK)
      IF (K.GT.0) THEN
      LL=KCG(K)
      IF (LL.GT.0) THEN
      CGD((LL-1)*NC+KC)=CG(KK)
      DD(LL)=1.0D 0/D(K)
      ENDIF
      ENDIF
    3 CONTINUE
      ENDIF
    4 CONTINUE
      RETURN
      END
