* SUBROUTINE UCISD1             ALL SYSTEMS                   98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE SPARSE SCHUR COMPLEMENT STRUCTURE FOR THE
* KARUSH-KUHN-TUCKER MATRIX WITH THE UNIT HESSIAN BLOCK.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  II  NC  NUMBER OF CONSTRAINT FUNCTIONS.
*  II  MC  NUMBER OF NONZERO ELEMENTS IN THE FIELD CG.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE SCHUR COMPLEMENT.
*  IO  ND  ACTUAL NUMBER OF DENSE ROWS.
*  IO  MD  MAXIMUM NUMBER OF DENSE ROWS.
*  IO  MDE  MINIMUM NUMBER OF NONZERO ELEMENTS IN DENSE ROWS.
*  IO  IB(NF+1)  POINTERS OF DIAGONAL ELEMENTS OF THE SPARSE SCHUR
*         COMPLEMENT.
*  IO  JB(M)  COLUMN INDICES OF NONZERO ELEMENTS OF THE SPARSE SCHUR
*         COMPLEMENT.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  II  ICG(NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  IO  KCG(NF)  INDICATION OF DENSE ROWS.
*  IO  ITERM  TERMINATION INDICATOR.
*
      SUBROUTINE UCISD1(NF,NCA,NC,MC,M,ND,MD,MDE,IB,JB,ICA,ICG,JCG,KCG,
     & ITERM)
      INTEGER NF,NCA,NC,MC,M,ND,MD,MDE,IB(NCA+1),JB(M),ICA(NCA),
     & ICG(NC+1),JCG(MC),KCG(NF),ITERM
      INTEGER I,J,K,KK,K1,K2,L,LL,L1,L2,MM
      CALL UXVINS(NF,0,KCG)
      DO 8 I=1,NCA
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KCG(K)=KCG(K)+1
      ELSE
      K1=ICG(K)
      K2=ICG(K+1)-1
      DO 7 KK=K1,K2
      K=ABS(JCG(KK))
      KCG(K)=KCG(K)+1
    7 CONTINUE
      ENDIF
    8 CONTINUE
      ND=0
      DO 9 K=1,NF
      IF (KCG(K).GE.MDE) THEN
      ND=ND+1
      KCG(K)=ND
      ELSE
      KCG(K)=0
      ENDIF
    9 CONTINUE
      IF (ND.GT.MD) THEN
      CALL UOERR2('UCISD1',85,ND,MD)
      ITERM=-85
      RETURN
      ENDIF
      MM=0
      IB(1)=1
      DO 6 I=1,NCA
      K=ICA(I)
      MM=MM+1
      JB(MM)=I
      IF (K.LT.0) THEN
      IF (KCG(ABS(K)).EQ.0) THEN
      K=-K
      DO 2 J=I+1,NCA
      L=ICA(J)
      IF (L.GT.0) THEN
      L1=ICG(L)
      L2=ICG(L+1)-1
      DO 1 LL=L1,L2
      IF (ABS(JCG(LL)).EQ.K) THEN
      MM=MM+1
      JB(MM)=J
      GO TO 2
      ENDIF
    1 CONTINUE
      ENDIF
    2 CONTINUE
      ENDIF
      ELSE IF (K.GT.0) THEN
      K1=ICG(K)
      K2=ICG(K+1)-1
      DO 5 J=I+1,NCA
      L=ICA(J)
      IF (L.LT.0) THEN
      IF (KCG(ABS(L)).EQ.0) THEN
      L=-L
      DO 3 KK=K1,K2
      IF (ABS(JCG(KK)).EQ.L) THEN
      MM=MM+1
      JB(MM)=J
      GO TO 5
      ENDIF
    3 CONTINUE
      ENDIF
      ELSE IF (L.GT.0) THEN
      KK=K1
      LL=ICG(L)
      L2=ICG(L+1)-1
      K=ABS(JCG(KK))
      L=ABS(JCG(LL))
    4 IF (KCG(K).NE.0.OR.K.LT.L) THEN
      IF (KK.GE.K2) GO TO 5
      KK=KK+1
      K=ABS(JCG(KK))
      GO TO 4
      ELSE IF (KCG(L).NE.0.OR.L.LT.K) THEN
      IF (LL.GE.L2) GO TO 5
      LL=LL+1
      L=ABS(JCG(LL))
      GO TO 4
      ELSE
      MM=MM+1
      JB(MM)=J
      ENDIF
      ENDIF
    5 CONTINUE
      ENDIF
      IB(I+1)=MM+1
    6 CONTINUE
      RETURN
      END
