* SUBROUTINE UCITRS                ALL SYSTEMS                03/12/01
C PORTABILITY : ALL SYSTEMS
C 03/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF A SCALING FACTOR FOR THE BOUNDARY STEP USING
* A POSITIVE SLACK VECTOR.
*
* PARAMETERS :
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  II  IX(NX)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  IO  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  D(NCA)  DIAGONAL SCALING MATRIX.
*  RI  DEL  MAXIMUM STEPSIZE.
*  RI  X(NCA)  INTERIOR VECTOR.
*  RI  Y(NCA)  EXTERIOR VECTOR.
*  RU  ALF  SCALING FACTOR FOR THE BOUNDARY STEP.
*  II  INT  CHOICE OF THE INTERIOR VECTOR. INTERIOR VECTOR IS EQUAL
*         TO ZERO IF INT=0.
*  II  INP  POSITIVITY OF SLACK CONSTRAINTS. SLACK VARIABLES ARE
*         POSITIVE IF INP=0.
*  IO  INF  INDICATION. INF=0-ALL SLACK CONSTRAINTS ARE INACTIVE.
*         INF=1-AT LEAST ONE SLACK CONSTRAINT IS ACTIVE.
*
      SUBROUTINE UCITRS(NX,NC,NCA,IX,XSL,XSU,IC,CSL,CSU,ICA,D,DEL,X,Y,
     & ALF,INT,INP,INF)
      INTEGER NX,NC,NCA,IX(NX),IC(NC),ICA(NCA),INT,INP,INF
      REAL*8 XSL(NX),XSU(NX),CSL(NC),CSU(NC),D(NCA),DEL,X(NCA),Y(NCA),
     & ALF
      REAL*8 VINT,VDIF,ALFO
      INTEGER I,K,IPOM
      REAL*8 TAU
      PARAMETER (TAU=0.99D 0)
      IPOM= 1
      IF (ALF.LT.0.0D 0) THEN
      IPOM=-1
      ALF=-ALF
      ENDIF
      ALFO=ALF
      INF=0
      DO 1 I=1,NCA
      IF (INT.EQ.0) THEN
      VINT=0.0D 0
      ELSE
      VINT=X(I)
      ENDIF
      IF (INT.GE.0) THEN
      VDIF=Y(I)
      ELSE
      VDIF=Y(I)-VINT
      ENDIF
      IF (IPOM.LT.0) VDIF=-VDIF
      IF (VDIF.GT.0.0D 0) THEN
      ALF=MIN(ALF,( DEL-VINT)/VDIF)
      ELSE IF (VDIF.LT.0.0D 0) THEN
      ALF=MIN(ALF,(-DEL-VINT)/VDIF)
      ENDIF
      IF (INP.EQ.0) THEN
      IF (ALF.LT.ALFO) THEN
      ALFO=ALF
      INF=0
      ENDIF
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      IF (IX(K).EQ.-1.OR.IX(K).EQ.-3) THEN
      IF (VDIF.GT.0.0D 0) THEN
      ALF=MIN(ALF,-(VINT-TAU*XSL(K)/D(I))/VDIF)
      ENDIF
      ELSE IF (IX(K).EQ.-2.OR.IX(K).EQ.-4) THEN
      IF (VDIF.LT.0.0D 0) THEN
      ALF=MIN(ALF,-(VINT+TAU*XSU(K)/D(I))/VDIF)
      ENDIF
      ENDIF
      ELSE
      IF (IC(K).EQ.-1.OR.IC(K).EQ.-3) THEN
      IF (VDIF.GT.0.0D 0) THEN
      ALF=MIN(ALF,-(VINT-TAU*CSL(K)/D(I))/VDIF)
      ENDIF
      ELSE IF (IC(K).EQ.-2.OR.IC(K).EQ.-4) THEN
      IF (VDIF.LT.0.0D 0) THEN
      ALF=MIN(ALF,-(VINT+TAU*CSU(K)/D(I))/VDIF)
      ENDIF
      ENDIF
      ENDIF
      IF (ALF.LT.ALFO) THEN
      INF=1
      ALF=ALFO
      ENDIF
      ENDIF
    1 CONTINUE
      IF (IPOM.LT.0) ALF=-ALF
      RETURN
      END
