* SUBROUTINE UEVDP5                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF THE INITIAL VALUE PROBLEM FOR A SET OF THE FIRST ORDER
* ORDINARY DIFFERENTIAL EQUATIONS USING THE FIFTH ORDER DORMAN AND
* PRINCE INTEGRATION FORMULA.
*
* PARAMETERS :
*  II  NA  NUMBER OF MESH POINTS.
*  II  NAL  NUMBER OF MESH POINTS.
*  II  NE  DECLARED NUMBER OF DIFFERENTIAL EQUATIONS.
*  II  MEL  ACTUAL NUMBER OF DIFFERENTIAL EQUATIONS.
*  IU  KA  INDEX OF THE MESH POINT.
*  RI  AT(NA)  MESH POINTS.
*  RO  AY(NA*NE)  FIELD OF SOLUTION VECTORS IN THE MESH POINTS.
*  RU  YA(MEL)  SOLUTION VECTOR.
*  RA  YAO(MEL)  OLD SOLUTION VECTOR.
*  RI  EF(MEL)  RIGHT HAND SIDE VECTOR.
*  RA  ES(11*MEL)  AUXILIARY VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  RI  TET1  LOVER PARAMETER FOR STEPSIZE SELECTION.
*  RI  TET2  UPPER PARAMETER FOR STEPSIZE SELECTION.
*  RI  TET3  SAFETY FACTOR FOR STEPSIZE PREDICTION.
*  RI  BET3  PARAMETER FOR STEPSIZE CONTROL.
*  RU  TA  VALUE OF THE INTEGRATION PARAMETER.
*  RU  TAO  OLD VALUE OF THE INTEGRATION PARAMETER.
*  RI  TAMAX  MAXIMUM VALUE OF THE INTEGRATION PARAMETER.
*  RU  HE  ACTUAL STEPSIZE.
*  RU  HEMAX  MAXIMUM STEPSIZE.
*  RI  FMAX  UPPER BOUND FOR SOLUTION NORM.
*  RO  ERR  INTEGRATION ERROR.
*  II  MED  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  II  MED1  CHOICE OF STEP SELECTION.
*  II  MED2  NUMBER OF RESETING SOLUTIONS.
*  II  MED3  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  IO  NACC  NUMBER OF ACCEPTED INTEGRATION STEPS.
*  IO  NREJ  NUMBER OF REJECTED INTEGRATION STEPS.
*  IO  IER  ERROR INDICATOR. IER=0-SOLUTION OBTAINED. IER=-1-SYSTEM
*         IS UNSTABLE. IER=-2-MAXIMUM NUMBER OF STEPS EXCEEDED.
*         IER=-3-TOO SMALL STEP REQUIRED. IER=-4-SYSTEM IS PROBABLY
*         STIFF. IER=-5-JACOBIAN MATRIX IS SINGULAR. IER=-1-LACK
*         OF SPACE FOR OUTPUT DATA.
*
* COMMON DATA :
*  IO  NSTP  ACTUAL  NUMBER OF INTEGRATION STEPS.
*  II  MSTP  MAXIMUM NUMBER OF INTEGRATION STEPS.
*  RI  TOLR  RELATIVE PRECISION.
*  RI  TOLA  ABSOLUTE PRECISION.
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UNVDP5  INTERPOLATION IN MESH POINTS.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVMED  MEDIAN OF TWO VECTORS.
*
* METHOD :
* E.HAIRER, S.P.NORSETT, G.WANNER: SOLVING ORDINARY DIFFERENTIAL
* EQUATIONS I. SPRINGER SERIES IN COMPUTATIONAL MATHEMATICS.
* SPRINGER VERLAG, BERLIN 1980.
*
      SUBROUTINE UEVDP5(NA,NAL,NE,MEL,KA,AT,AY,YA,YAO,EF,ES,ETA0,TET1,
     & TET2,TET3,BET3,TA,TAO,TAMAX,HE,HEMAX,FMAX,ERR,MED,MED1,MED2,MED3,
     & NACC,NREJ,IER)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NA,NAL,NE,MEL,KA,MED,MED1,MED2,MED3,NACC,NREJ,IER
      REAL*8 AT(*),AY(*),YA(MEL),YAO(MEL),EF(MEL),ES(*),ETA0,
     & TET1,TET2,TET3,BET3,TA,TAO,TAMAX,HE,HEMAX,FMAX,ERR
      LOGICAL REJECT,LAST
      INTEGER NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,NE11,
     & I,IASTI,IORD,NONSTI,NEGPOS
      REAL*8 FACOLD,FAC,FAC11,POSNEG,HLAMB,DNF,DNY,
     & SK,DER2,DER12,HE1,HNEW,STNUM,STDEN,YD0,YDIFF,BSPL
      REAL*8 C2,C3,C4,C5,A21,A31,A32,A41,A42,A43,A51,A52,A53,A54,A61,
     & A62,A63,A64,A65,A71,A73,A74,A75,A76,E1,E3,E4,E5,E6,E7,D1,D3,D4,
     & D5,D6,D7,UXVMAX
      SAVE REJECT,LAST
      SAVE NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,NE11,
     & IASTI,IORD,NONSTI,NEGPOS
      SAVE FACOLD,POSNEG,HLAMB,DNF,HNEW
C
C     PARAMETERS FOR THE FIFTH ORDER DORMAN-PRINCE FORMULA
C
      PARAMETER (C2=0.2D 0,C3=0.3D 0,C4=0.8D 0,C5=8.D 0/9.D 0)
      PARAMETER (A21=0.2D 0,A31=3.D 0/40.D 0,A32=9.D 0/40.D 0,
     & A41=44.D 0/45.D 0,A42=-56.D 0/15.D 0,A43=32.D 0/9.D 0,
     & A51=19372.D 0/6561.D 0,A52=-25360.D 0/2187.D 0,
     & A53=64448.D 0/6561.D 0,A54=-212.D 0/729.D 0,
     & A61=9017.D 0/3168.D 0,A62=-355.D 0/33.D 0,
     & A63=46732.D 0/5247.D 0,A64=49.D 0/176.D 0,
     & A65=-5103.D 0/18656.D 0,A71=35.D 0/384.D 0,
     & A73=500.D 0/1113.D 0,A74=125.D 0/192.D 0,
     & A75=-2187.D 0/6784.D 0,A76=11.D 0/84.D 0)
      PARAMETER (E1=71.D 0/57600.D 0,E3=-71.D 0/16695.D 0,
     & E4=71.D 0/1920.D 0,E5=-17253.D 0/339200.D 0,
     & E6=22.D 0/525.D 0,E7=-1.D 0/40.D 0)
      PARAMETER (D1=-12715105075.D 0/11282082432.D 0,
     & D3=87487479700.D 0/32700410799.D 0,
     & D4=-10690763975.D 0/1880347072.D 0,
     & D5=701980252875.D 0/199316789632.D 0,
     & D6=-1453857185.D 0/822651844.D 0,
     & D7=69997945.D 0/29380423.D 0)
      IF (TSS(NS).NE.'UEDX') CALL UYPRO1('UEDX',-1)
      GO TO (1,2,3,4,5,6,7,8,9) ISP(NS)
    1 CONTINUE
      NE1=0
      NE2=MEL
      NE3=2*MEL
      NE4=3*MEL
      NE5=4*MEL
      NE6=5*MEL
      NE7=6*MEL
      NE8=7*MEL
      NE9=8*MEL
      NE10=9*MEL
      NE11=10*MEL
      NSTP=0
      IASTI=0
      NONSTI=0
      HLAMB=ZERO
      FACOLD=1.D-4
      POSNEG=SIGN(ONE,TAMAX-TA)
      IF (MED1.EQ.1) THEN
      KA=2
      ELSE
      NEGPOS=POSNEG
      IF (NEGPOS.GT.0) KA=1
      IF (NEGPOS.LT.0) KA=NA
      ENDIF
      TAO=TA
C
C     INITIALISATION
C
      CALL UXVCOP(MEL,YA,YAO)
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE1+1))
      HEMAX=ABS(HEMAX)
      IORD=5
      IF (MED1.EQ.2.OR.HE.NE.ZERO) GO TO 24
C
C     INITIAL STEPSIZE
C
      DNF=ZERO
      DNY=ZERO
      DO 21 I=1,MEL
      SK=TOLA+TOLR*ABS(YAO(I))
      DNF=DNF+(ES(NE1+I)/SK)**2
      DNY=DNY+(YAO(I)/SK)**2
  21  CONTINUE
      IF (DNF.LE.1.D-10.OR.DNY.LE.1.D-10) THEN
         HE=1.0D-6
      ELSE
         HE=SQRT(DNY/DNF)*0.01D 0
      END IF
      HE=MIN(HE,HEMAX)
      HE=SIGN(HE,POSNEG)
      DO 22 I=1,MEL
      YA(I)=YAO(I)+HE*ES(NE1+I)
  22  CONTINUE
      TA=TAO+HE
      CALL UYSET0(1,3)
      RETURN
    3 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE2+1))
      DER2=ZERO
      DO 23 I=1,MEL
      SK=TOLA+TOLR*ABS(YAO(I))
      DER2=DER2+((ES(NE2+I)-ES(NE1+I))/SK)**2
  23  CONTINUE
      DER2=SQRT(DER2)/HE
      DER12=MAX(ABS(DER2),SQRT(DNF))
      IF (DER12.LE.1.D-15) THEN
         HE1=MAX(1.0D-6,ABS(HE)*1.0D-3)
      ELSE
         HE1=(0.01D 0/DER12)**(ONE/IORD)
      END IF
      HE=MIN(100.0D 0*ABS(HE),HE1,HEMAX)
      HE=SIGN(HE,POSNEG)
  24  CONTINUE
      REJECT=.FALSE.
      IF (MED.NE.0) CALL UNVDP5(NA,NAL,NE,MEL,KA,AT,AY,YAO,TA,TAO,HE,
     & IER,-MED)
C
C     BASIC INTEGRATION STEP
C
  25  CONTINUE
      IF (UXVMAX(NE,YA).GE.FMAX) GO TO 41
      IF (MED1.EQ.2) THEN
      IER=0
      KA=KA+NEGPOS
      IF (KA.LT.1.OR.KA.GT.NA) GO TO 42
      HE=AT(KA)-TAO
      ELSE
      IF (NSTP.GT.MSTP) GO TO 40
      IF (0.1D 0*ABS(HE).LE.ABS(TAO)*ETA0) GO TO 39
      IF ((TAO+1.01D 0*HE-TAMAX)*POSNEG.GT.ZERO) THEN
        HE=TAMAX-TAO
        LAST=.TRUE.
      ELSE
        LAST=.FALSE.
      END IF
      END IF
      NSTP=NSTP+1
C
C     THE FIRST 6 STAGES
C
      DO 26 I=1,MEL
      YA(I)=YAO(I)+HE*A21*ES(NE1+I)
  26  CONTINUE
      TA=TAO+C2*HE
      CALL UYSET0(1,4)
      RETURN
    4 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE2+1))
      DO 27 I=1,MEL
      YA(I)=YAO(I)+HE*(A31*ES(NE1+I)+A32*ES(NE2+I))
  27  CONTINUE
      TA=TAO+C3*HE
      CALL UYSET0(1,5)
      RETURN
    5 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE3+1))
      DO 28 I=1,MEL
      YA(I)=YAO(I)+HE*(A41*ES(NE1+I)+A42*ES(NE2+I)+A43*ES(NE3+I))
  28  CONTINUE
      TA=TAO+C4*HE
      CALL UYSET0(1,6)
      RETURN
    6 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE4+1))
      DO 29 I=1,MEL
      YA(I)=YAO(I)+HE*(A51*ES(NE1+I)+A52*ES(NE2+I)+A53*ES(NE3+I)+
     & A54*ES(NE4+I))
  29  CONTINUE
      TA=TAO+C5*HE
      CALL UYSET0(1,7)
      RETURN
    7 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE5+1))
      DO 30 I=1,MEL
      YA(I)=YAO(I)+HE*(A61*ES(NE1+I)+A62*ES(NE2+I)+A63*ES(NE3+I)+
     & A64*ES(NE4+I)+A65*ES(NE5+I))
  30  CONTINUE
      TA=TAO+HE
      CALL UYSET0(1,8)
      RETURN
    8 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE6+1))
      DO 31 I=1,MEL
      YA(I)=YAO(I)+HE*(A71*ES(NE1+I)+A73*ES(NE3+I)+A74*ES(NE4+I)+
     & A75*ES(NE5+I)+A76*ES(NE6+I))
  31  CONTINUE
      CALL UYSET0(1,9)
      RETURN
    9 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE2+1))
      IF (MED.GE.2) THEN
      IF (HE*(TA-AT(KA)).GE.ZERO) THEN
        DO 32 I=1,NE
        ES(NE11+I)=HE*(D1*ES(NE1+I)+D3*ES(NE3+I)+D4*ES(NE4+I)+
     &                  D5*ES(NE5+I)+D6*ES(NE6+I)+D7*ES(NE2+I))
  32    CONTINUE
      END IF
      END IF
      DO 33 I=1,MEL
      ES(NE4+I)=HE*(E1*ES(NE1+I)+E3*ES(NE3+I)+E4*ES(NE4+I)+
     &               E5*ES(NE5+I)+E6*ES(NE6+I)+E7*ES(NE2+I))
  33  CONTINUE
      IF (MED1.EQ.2) THEN
        NACC=NACC+1
        DO 43 I=1,MEL
        ES(NE1+I)=ES(NE2+I)
        YAO(I)=YA(I)
  43    CONTINUE
        IF (MED2.EQ.1) CALL UXVCOP(NE,AY((KA-1)*NE+1),YAO)
        IF (MED2.EQ.2) CALL UXVMED(NE,AY((KA-1)*NE+1),YAO,YAO)
        CALL UNVDP5(NA,NAL,NE,MEL,KA,AT,AY,YAO,TA,TAO,HE,IER,3)
        TAO=AT(KA)
        GO TO 25
      ENDIF
C
C     ERROR ESTIMATION
C
      ERR=ZERO
      DO 34 I=1,MEL
      SK=TOLA+TOLR*MAX(ABS(YAO(I)),ABS(YA(I)))
  34  ERR=ERR+(ES(NE4+I)/SK)**2
      ERR=SQRT(ERR/MEL)
      FAC11=ERR**(0.2D 0-BET3*0.75D 0)
      FAC=FAC11/FACOLD**BET3
      FAC=MAX(ONE/TET2,MIN(ONE/TET1,FAC/TET3))
      HNEW=HE/FAC
      IF(ERR.LE.ONE)THEN
C
C     STEP IS ACCEPTED
C
        FACOLD=MAX(ERR,1.0D-4)
        NACC=NACC+1
C
C     STIFFNESS DETECTION
C
        IF (MOD(NACC,MED3).EQ.0.OR.IASTI.GT.0) THEN
          STNUM=ZERO
          STDEN=ZERO
          DO 35 I=1,MEL
          STNUM=STNUM+(ES(NE2+I)-ES(NE6+I))**2
          STDEN=STDEN+(YA(I)-ES(I))**2
  35        CONTINUE
          IF (STDEN.GT.ZERO) HLAMB=ABS(HE)*SQRT(STNUM/STDEN)
          IF (HLAMB.GT.3.25D 0) THEN
            NONSTI=0
            IASTI=IASTI+1
            IF (IASTI.EQ.15) GO TO 38
          ELSE
            NONSTI=NONSTI+1
            IF (NONSTI.EQ.6) IASTI=0
          END IF
        END IF
        IF (MED.GE.2) THEN
        IF (HE*(TA-AT(KA)).GE.ZERO) THEN
          DO 36 I=1,NE
          YD0=YAO(I)
          YDIFF=YA(I)-YD0
          BSPL=HE*ES(NE1+I)-YDIFF
          ES(NE7+I)=YAO(I)
          ES(NE8+I)=YDIFF
          ES(NE9+I)=BSPL
          ES(NE10+I)=-HE*ES(NE2+I)+YDIFF-BSPL
  36      CONTINUE
        END IF
        END IF
        DO 37 I=1,MEL
        ES(NE1+I)=ES(NE2+I)
        YAO(I)=YA(I)
  37    CONTINUE
        IF (MED.EQ.1) CALL UNVDP5(NA,NAL,NE,MEL,KA,AT,AY,YAO,TA,TAO,HE,
     &      IER,MED)
        IF (MED.GE.2) THEN
        IF (HE*(TA-AT(KA)).GE.ZERO) THEN
        CALL UNVDP5(NA,NAL,NE,MEL,KA,AT,AY,ES(NE7+1),TA,TAO,HE,IER,MED)
        END IF
        END IF
        IF (IER.EQ.-11) GO TO 42
        TAO=TA
C
C     NORMAL EXIT
C
        IF (LAST) THEN
          HE=HNEW
          IER=0
          GO TO 42
        END IF
        IF(ABS(HNEW).GT.HEMAX)HNEW=POSNEG*HEMAX
        IF(REJECT)HNEW=POSNEG*MIN(ABS(HNEW),ABS(HE))
        REJECT=.FALSE.
      ELSE
C
C     STEP IS REJECTED
C
        HNEW=HE/MIN(ONE/TET1,FAC11/TET3)
        REJECT=.TRUE.
        IF(NACC.GE.1)NREJ=NREJ+1
      END IF
      HE=HNEW
      GO TO 25
C
C     FAIL EXIT
C
  38  IER=-4
      TEDX='STIFF EQ'
      GO TO 42
  39  IER=-3
      TEDX='SMALL ST'
      GO TO 42
  40  IER=-2
      TEDX='STEP LIM'
      GO TO 42
  41  IER=-1
      TEDX='UNSTABLE'
  42  CALL UYEPI1(-1)
      RETURN
      END
