* SUBROUTINE UKLINM                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SPARSE QP INPUT TRANSFORMATION MAIN.
*
* PARAMETERS :
*  IU  NF ACTUAL NUMBER OF VARIABLES.
*  IU  NFL INITIAL NUMBER OF VARIABLES.
*  IU  NC ACTUAL NUMBER OF CONSTRAINTS.
*  IU  NCL INITIAL NUMBER OF CONSTRAINTS.
*  II  N NUMBER OF COLUMNS.
*  II  M NUMBER OF ROWS.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  MCOLS DECLARED NUMBER OF COLUMNS.
*  II  NZ  SIZE OF VECTORS CNR/INR.
*  II  NZA  SIZE OF ARRAYS A/RNR.
*  II  NAU SIZE OF VECTORS AI/AU.
*  II  NUP SIZE OF ARRAYS LI,LJ,CM.
*  IO  NEL  NUMBER OF ELEMENTARY ELIMINATION MATRICES STORED IN THE
*           TRIPLE: LI,LJ,CM.
*  IO  IBEG  BEGINNING OF THE ROW SCHEME OF THE FACTOR U IN CNR.
*  IO  IBGR  BEGINNING OF THE COLUMN SCHEME IN CNR.
*  IO  IENDLU  END OF THE ROW INDICES OF THE FACTOR U IN CNR.
*  IO  RNR(NZA) ROW INDICES OF CONSTRAINT MATRIX.
*  IO  PRNR(MCOLS+1) POINTER VECTOR TO RNR.
*  IA  WN01(MCOLS) AUXILIARY VECTOR.
*  IA  WN02(MCOLS) AUXILIARY VECTOR.
*  RA  WR01(MCOLS) AUXILIARY VECTOR.
*  RA  WR02(MCOLS) AUXILIARY VECTOR.
*  IU  BCOL(MCOLS) VECTOR THAT IS FILLED WITH THE INDICES OF THE
*                  BASIS ON THE OUTPUT - WHEN MFP1 > 0, IT CAN BE
*                  GIVEN AS THE INPUT ALONG WITH SOME SOLUTION VECTOR.
*  IU  TYPE(MCOLS) TYPES OF THE VARIABLES-ON INPUT:
*                                     =1 FOR LOWER BOUNDED VARIABLE.
*                                     =2 FOR UPPER BOUNDED VARIABLE.
*                                     =3 FOR LOWER AND UPPER BOUNDED
*                                        VARIABLE.
*                                     =5 FOR FIXED VARIABLE.
*                                     =0,6 FOR FREE VARIABLES.
*                                         ON OUTPUT:
*                                    =1 FOR NONBASIC LOWER BOUNDED
*                                    =2 FOR NONBASIC UPPER BOUNDED
*                                    =3 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS LOWER BOUND
*                                    =4 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS UPPER BOUND
*                                    =5 FOR NONBASIC FIXED VARIABLE
*                                    =6 FOR NONBASIC FREE VARIABLE
*                                    =-1 FOR BASIC LOWER BOUNDED
*                                    =-2 FOR BASIC UPPER BOUNDED
*                                    =-3 FOR BASIC LOWER AND UPPER
*                     BOUNDED VARIABLE
*                                    =-6 FOR BASIC FREE VARIABLE.
*  IU  TYPC(MROWS) TYPES OF THE MATRIX ROWS:
*                                     =1 FOR LOWER BOUNDED ROW.
*                                     =2 FOR UPPER BOUNDED ROW.
*                                     =3 FOR LOWER AND UPPER BOUNDED ROW
*                                        WITH NON-EQUAL BOUNDS.
*                                     =4 FOR LOWER AND UPPER BOUNDED ROW
*                                        WITH EQUAL BOUNDS.
*  II  PINR(MROWS) VECTOR SHOWING BEGINNINGS OF THE INPUT MATRIX
*           ROWS.
*  II  INR(NZ) INPUT MATRIX COLUMN INDICES - ON OUTPUT USUALLY
*           REWRITED BY CNR.
*  IO  PERMR(MROWS) INPUT ROW PERMUTATION VECTOR.
*  IO  IPRMC(MCOLS) INPUT COLUMN INVERSE PERMUTATION VECTOR.
*  IU  LENR(MROWS) LENGTHS OF THE ROWS OF THE INPUT STRUCTURE INR/AI,
*           THEN LENGTHS OF THE ROWS OF THE U-FACTOR.
*  IO  PTR(MROWS)  POINTER VECTOR OF THE ROWS OF THE FACTORS U.
*  IO  LENL(MROWS) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  IO  PTL(MROWS) POINTER VECTOR OF THE COLUMNS OF THE FACTOR L.
*  IO  LENC(MROWS) LENGTHS OF THE COLUMNS OF U IF ( ONLY IF IFL.LT.0).
*  IO  PTC(MROWS) POINTER VECTOR OF THE COLUMN SCHEME OF THE
*             FACTOR U (ONLY IF IFL.LT.0).
*  IO  CNR(NZ) CONTAINS THE INDICES OF THE COLUMN AND ROW SCHEME
*              FOR THE FACTORS OF THE SPARSE MATRIX.
*  IO  ORDR(MCOLS) ORDER OF THE ROWS THAT WILL BE FURTHER UPDATED.
*  IO  ORDC(MCOLS) ORDER OF THE COLUMNS THAT WILL BE FURTHER UPDATED.
*  IO  ORDL(MCOLS) ORDER OF THE ROWS OF THIS REFACTORIZATION.
*  IO  RIGHT(MROWS) SIZE OF THE ROW SPIKE USED DURING COLUMNS EXCHANGE.
*  IO  UP(MROWS) SIZE OF THE COLUMN SPIKE USED DURING COLUMN EXCHANGE.
*  RU  X(MCOLS) SOLUTION VECTOR.
*  RO  C(MCOLS) COST VECTOR.
*  RO  AU(NAU) ELEMENTS OF THE CONSTRAINT MATRIX.
*  RO  A(NZA) NUMERICAL VALUES OF THE CONSTRAINT MATRIX
*             CORRESPONDING TO RNR.
*  RI  AI(NAU) INPUT MATRIX VALUES ON OUTPUT USUALLY REWRITED BY AU.
*  RU  LBC(MROWS) LOWER BOUNDS ON MATRIX ROWS.
*  RI  UBC(MROWS) UPPER BOUNDS ON MATRIX ROWS.
*  RI  LB(MCOLS) LOWER BOUNDS ON VARIABLES.
*  RI  UB(MCOLS) UPPER BOUNDS ON VARIABLES.
*  RO  DEL1  MULTIPLIER BOUND FOR PIVOT LOOK-UP.
*  RO  DEL2  RELATIVE BOUND FOR NEGLECTING SMALL ELEMENTS WHEN
*            CONSTRUCTING INITIAL BASIS.
*  RO  EPS7  BOUND FOR NEGLECTING SMALL SPARSE MATRIX ELEMENTS.
*  RI  EPS9  TOLERANCE ON MULTIPLIERS.
*  RO  DMAX  MINIMUM ELEMENT OF THE U-DIAGONAL (EARLIER MNU).
*  RO  GMAX  ELEMENT WITH THE MAXIMUM ABSOLUTE VALUE OF THE FACTOR U.
*  RO  UMAX  MAXIMUM ELEMENT OF THE U-DIAGONAL (EARLIER MXU).
*  II  MFP  TYPE OF FEASIBLE POINT: =0 WHEN NO FFEASIBLE/NONFEASIBLE
*           SOLUTION IS INITIALLY GIVEN,  >0 WHEN SOME SOLUTION IS
*           GIVEN AND WE WANT AT LEAST A FEASIBLE ONE.
*  II  MFP1 >0 WHEN BASIS IS SPECIFIED ON INPUT, =0 OTHERWISE.
*  II  MLP PARAMETER CONTROLLING NUMBER OF COLUMNS TO BE TESTED
*             ON INPUT INTO THE INITIAL BASIS:
*             =2 WHEN BASIS IS TO INCLUDE ONLY SLACKS AND ARTIFICIALS,
*             =3 WHEN VARIABLES OF NONLINEAR CONSTRAINTS ARE TO BE
*             EXCLUDED, =4 WHEN VARIABLES NONLINER IN OBJECTIVE FUNCTION
*             ARE TO BE EXCLUDED.
*  IU  MLP1  ACTUAL NUMBER OF DEGENERATE ITERATIONS.
*  IO  NREF  NUMBER OF REFACTORIZATIONS.
*  IO  KINP NUMBER OF ARTIFICIALS.
*  IO  LINP NUMBER OF BASIC VARIABLES SET IN UKLBAS.
*  IO  MINP NUMBER OF SLACKS.
*  IO  KOLD  NUMBER OF UPDATES AFTER LATEST REFACTORIZATION.
*  IO  IER  ERROR INDICATOR.
*  IO  ITERL TERMINATION INDICATOR
*
      SUBROUTINE UKLINM(NF,NFL,NC,NCL,N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,
     & NEL,IBEG,IBGR,IENDLU,RNR,PRNR,WN01,WN02,WR01,WR02,BCOL,TYPE,
     & TYPC,PINR,INR,PERMR,IPRMC,LENR,PTR,LENL,PTL,LENC,PTC,CNR,
     & ORDR,ORDC,ORDL,RIGHT,UP,X,C,AU,A,AI,LBC,UBC,LB,UB,DEL1,DEL2,
     & EPS7,EPS9,DMAX,GMAX,UMAX,MFP,MFP1,MLP,MLP1,NREF,KINP,LINP,MINP,
     & KOLD,IER,ITERL)
      INTEGER NF,NFL,NC,NCL,N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,IBEG,
     & IBGR,IENDLU,RNR(NZA),PRNR(MCOLS+1),WN01(MCOLS),WN02(MCOLS),
     & BCOL(MCOLS),TYPE(MCOLS),TYPC(MROWS),PINR(MROWS),INR(NZ),
     & PERMR(MROWS),IPRMC(MCOLS),LENR(MROWS),PTR(MROWS),
     & LENL(MROWS),PTL(MROWS),LENC(MROWS),PTC(MROWS),CNR(NZ),
     & ORDR(MROWS),ORDC(MROWS),ORDL(MROWS),RIGHT(MROWS),UP(MROWS),
     & MFP,MFP1,MLP,MLP1,NREF,KINP,LINP,MINP,KOLD,IER,ITERL
      REAL*8 WR01(MCOLS),WR02(MCOLS),X(MCOLS),C(MCOLS),AU(NAU),A(NZA),
     & AI(NAU),LBC(MROWS),UBC(MROWS),LB(MCOLS),UB(MCOLS),DEL1,DEL2,EPS7,
     & EPS9,DMAX,GMAX,UMAX
      INTEGER I,J,NFDUMM,NCDUMM,NPINR,L,K,P12,LACK,JOB
      REAL*8 STFL,GRFL,GSFL
C
C FICTIVE DIMENSION
C
      INTEGER LI(1),LJ(1)
      REAL*8 CM(1)
      PARAMETER(GRFL=1.0D 0,GSFL=1.0D 16)
      NUP=1
C
C NCL,NFL SPECIFIED
C
      NCL=NC
      NFL=NF
      NEL=0
C
C SET CONSTANTS
C
      GMAX=0
      CALL UKSGLF(STFL,DEL1,DEL2,EPS7,EPS9,MLP1)
      NREF=0
      KOLD=0
      ITERL=0
      IER=0
C
C SET NBCOL - NUMBER OF BASICS DETERMINED SO FAR
C
      IF(MFP1.EQ.0) THEN
        LINP=0
      ELSE
        LINP=NC
      END IF
C
C SET NPINR - FOR USE IN UOKIT1 ONLY
C
      NPINR=0
      DO 60 I=1,NC
        NPINR=NPINR+LENR(I)
   60 CONTINUE
      IF (NPINR.EQ.0) THEN
      L=1
      DO 70 I=1,NC
        K=PINR(I+1)
        LENR(I)=K-L
        NPINR=NPINR+LENR(I)
        L=K
   70 CONTINUE
      ENDIF
      NFDUMM=NF
      NCDUMM=NC
      CALL UKLTS3(NF,NC,MROWS,MCOLS,LENR,TYPE,LB,UB,C,TYPC,LBC,UBC,
     & PERMR,IPRMC,IER)
      IF(IER.EQ.14) THEN
        ITERL=-9
        CALL UOERR1('UKLINM',37)
        RETURN
      ELSEIF(IER.EQ.15) THEN
        ITERL=-10
        CALL UOERR1('UKLINM',38)
        RETURN
      ELSEIF(IER.EQ.16) THEN
        ITERL=-11
        CALL UOERR1('UKLINM',39)
        RETURN
      ELSEIF(IER.EQ.17) THEN
        ITERL=-12
        CALL UOERR1('UKLINM',40)
        RETURN
      ENDIF
      NFDUMM=NFDUMM-NF
      NCDUMM=NCDUMM-NC
C
C OUTPUT PROLOGUE
C
      CALL UOKIT1(NF,NC,'UKLINM',NFDUMM,NCDUMM,NPINR)
C
C SET NF,NC - AFTER EXCLUSION OF THE EMPTY ROWS/COLUMNS
C
      CALL UKLTSF(NF,NC,N,M,MROWS,MCOLS,NZ,NZA,PINR,INR,LENR,RNR,PRNR,
     & A,AI,LB,TYPE,LBC,UBC,TYPC,WN01,WN02,PERMR,IPRMC,MINP,IER)
C
C OUTPUT OF PERMUTATION VECTORS
C
      CALL UOKIT3(NF,M,MCOLS,WN01,IPRMC)
C
C PERMR,IPRMC ARE NOW USED FOR OTHER PURPOSES USING OUTER SUBSTITUTION
C
C MAIN CYCLE
C
  100 CALL UKLBAS(MCOLS,NF,M,NZA,RNR,PRNR,A,WN01,TYPE,BCOL,DEL1,DEL2,
     & MLP,KINP,LINP,MINP)
      CALL UKLART(MCOLS,NF,N,M,NZA,RNR,PRNR,A,WN01,LB,UB,TYPE,BCOL,C,
     & LINP,MINP)
      CALL UKLNBS(N,M,MROWS,MCOLS,NZA,RNR,PRNR,A,WR01,X,LB,UB,TYPE,MFP)
      P12=0
      CALL UXVSET(N,0.0D 0,WR02)
      CALL UXSGFD(M,M,NZ,NZA,NAU,NEL,P12,N,PTL,LENL,PTC,LENC,PTR,
     &            LENR,CNR,ORDC,ORDR,ORDL,RNR,PRNR,BCOL,RIGHT,WR02,
     &            A,AU,STFL,GMAX,UMAX,DMAX,IBEG,IBGR,IENDLU,LACK,
     &            GSFL,EPS7,ITERL,0)
      NREF=NREF+1
C
C SET VECTOR UP
C
C      DO 150 I=1,N
      DO 150 I=1,M
        UP(I)=WR02(I)
  150 CONTINUE
C
C TEST MLP
C
      IF(ITERL.NE.0) THEN
         IF(MLP.NE.2) THEN
            MLP=2
            GO TO 100
         ELSE
            CALL UOERR1('UKLINM',ITERL+20)
            IF(ITERL.EQ.1.OR.ITERL.EQ.6) THEN
              ITERL=-3
            ELSEIF(ITERL.EQ.2.OR.ITERL.EQ.7) THEN
              ITERL=-4
            ELSE
              ITERL=-5
            ENDIF
            GO TO 500
         END IF
      END IF
C
C RIGHT-HAND SIDE IS ADDED TO THE CONTRIBUTIONS
C
      DO 290 I=1,M
      WR01(I)=WR01(I)+LBC(I)
 290  CONTINUE
C
C BASIC PART OF SOLUTION IS COMPUTED
C
      JOB=0
      CALL UXSGFB(M,NZ,NAU,NEL,NUP,PTL,LENL,CNR,PTR,
     *            LENR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,WR01,WR02,JOB)
      DO 300 I=1,M
      J=BCOL(I)
        X(J)=WR01(I)
 300  CONTINUE
C
C UP IS PASSED OVER USING WN02
C
      DO 310 I=1,M
        WN02(I)=WR01(I)
 310  CONTINUE
C
C CHANGE MFP
C
      IF(MFP.EQ.0) MFP=1
  500 CONTINUE
C
C OUTPUT EPILOGUE
C
      CALL UOKIT2(N,M,MROWS,MCOLS,IBEG,PRNR,LENR,TYPE,BCOL,X,C,LB,UB,
     & LBC,KINP,MINP)
      RETURN
      END
