* SUBROUTINE UKQGLR                 ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* RELAXING DIRECTION IS FOUND.IF SOLUTION IS OPTIMAL - THEN
* ON OUTPUT IS IFL SET TO 1.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF ROWS OF THE MATRIX OF CONSTRAINTS.
*  II  MCOLS DECLARED NUMBER OF VARIABLES.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NZA SIZE OF THE INDEX ARRAY RNR
*  II  NAU SIZE OF THE ARRAY AU.
*  II  NUP SIZE OF THE ARRAYS LI,LJ,CM.
*  II  NEL NUMBER OF L-UPDATES IN THE ARRAYS LI,LJ,CM.
*  II  PTL(MROWS) POINTER VECTOR TO THE COLUMNS OF THE FACTOR L.
*  II  LENL(MROWS) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  II  CNR(NZ) CONTAINS THE INDICES OF THE COLUMN AND ROW SCHEME OF
*              THE FACTORS OF THE SPARSE MATRIX.
*  II  PTR(MROWS)  POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  II  LENR(MROWS) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  ORDC(MROWS) PERMUTATION OF THE COLUMNS.
*  II  ORDR(MROWS) PERMUTATION OF THE ROWS.
*  II  ORDL(MROWS) PERMUTATION OF THE ROWS AFTER THE LATEST REFACTORIZATION.
*  II  PRNR(MCOLS+1) POINTER VECTOR FOR RNR.
*  II  RNR(NZA) COLUMNS OF THE MATRIX OF CONSTRAINTS.
*  II  LI(NUP) ROW INDICES OF THE ELEMENTARY FACTORS.
*  II  LJ(NUP) COLUMN INDICES OF THE ELEMENTARY FACTORS.
*  RI  CM(NUP) NUMERICAL VALUES OF THE NONDIAGONAL ELEMENTS
*              OF THE ELEMENTARY FACTORS.
*  RI  A(NZA) NUMERICAL VALUES OF THE MATRIX OF CONSTRAINTS.
*  RI  AU(NAU) ELEMENTS OF THE FACTORS OF THE PROCESSED BASIS.
*  II  TYPE(MCOLS) TYPES OF THE VARIABLES:
*                                    =1 FOR NONBASIC LOWER BOUNDED
*                                    =2 FOR NONBASIC UPPER BOUNDED
*                                    =3 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS LOWER BOUND
*                                    =4 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS UPPER BOUND
*                                    =5 FOR NONBASIC FIXED VARIABLE
*                                    =6 FOR NONBASIC FREE VARIABLE
*                                    =-1 FOR BASIC LOWER BOUNDED
*                                    =-2 FOR BASIC UPPER BOUNDED
*                                    =-3 FOR BASIC LOWER AND UPPER
*                     BOUNDED VARIABLE
*                                    =-6 FOR BASIC FREE VARIABLE.
*  II  BCOL(MCOLS) NUMBERS OF THE BASIC ,SUPERBASIC AND NONBASIC COLUMNS.
*  IA  WN01(MCOLS) AUXILIARY VECTOR
*  RA  WR01(MCOLS) AUXILIARY VECTOR
*  RA  WR02(MCOLS) AUXILIARY VECTOR.
*  RI  LAM(MCOLS) VECTOR OF REDUCED GRADIENTS/REDUCED COSTS
*  RI  S(MCOLS) DIRECTION VECTOR.
*  RI  C(MCOLS) GRADIENT VECTOR.
*  RI  LB(MCOLS) LOWER BOUNDS FOR SOLUTION.
*  RI  UB(MCOLS) UPPER BOUNDS FOR SOLUTION.
*  RI  X(MCOLS)  SOLUTION VECTOR.
*  II  INBCOL(MCOLS) VECTOR WITH COMPONENTS :
*                INBCOL(K)=I WHEN EXISTS K=BCOL(I) AND I<=LINP,
*                INBCOL(K)=0 OTHERWISE.
*  IO  NSB NUMBER OF SUPERBASICS AND BASICS SHIFTED AT THE BOUND,
*          WHEN STEP XMAX IS TAKEN.
*  IO  NBB NUMBER OF BASICS SHIFTED AT THE BOUND, WHEN STEP
*          OF THE SIZE XMAX IS TAKEN.
*  RI  ALF1  LOWER BOUND FOR STEPSIZE PARAMETER.
*  RI  ALF2  UPPER BOUND FOR STEPSIZE PARAMETER.
*  RI  EPS9  ZERO TOLERANCE.
*  RO  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RO  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RU  DMAX   MINIMUM ELEMENT OF THE U-DIAGONAL.
*  RI  XMAX  MAXIMUM STEPSIZE.
*  RI  GNORM  EUCLIDEAN NORM OF THE GRADIENT.
*  RI  SNORM
*  RI  XNORM  MAXIMUM NORM OF THE DIRECTION VECTOR.
*  RI  TO  PARAMETER SIGNALIZING INCREASED ACCURACY FOR POSSIBLE
*            LAST UPDATE : =1  FOR POSSIBLE LAST UPDATE
*                          =0  OTHERWISE.
*  RU  EL  NORM OF THE LAGRANGE MULTIPLIERS OF THE SUPERBASIS
*              IS COMPUTED.
*  II  MQP DEGENERACY RULE:IF DGFL.LE.0 THEN FTRAN IS TOO COSTLY,
*          =1 WHEN NO OPTIMIZATION OF NON-BASIC PART OF P IS TRIED,
*          =2 IN THE OPPOSITE CASE TO THE FORMER ONE.
*  IU  MLP1 NUMBER OF ZERO PIVOTS.
*  IO  INEW WHEN PNORM=0 AND ANTICYCLING RULE HAS BEEN USED - IT
*           DENOTES THE COLUMN GIVEN INTO THE BASIS.
*  IO  IOLD WHEN PNORM=0 AND ANTICYCLING RULE HAS BEEN USED - IT
*           DENOTES OUT-OF-THE-BASIS COLUMN ELSE : =0.
*  IU  KINP DEGENERACY INFORMATION PARAMETER.
*  II  LINP NUMBER OF SUPERBASICS AND BASICS.
*  IO  ITERD TERMINATION INDICATOR:=1 FOR OPTIMUM SOLUTION OF THE
*            PROBLEM, =2 FOR OPTIMUM SOLUTION OF THE THIRD PHASE
*            (WITH DECREASED ACCURACY USING TO = 0), =3 FOR ANTICYCLING
*            EXCHANGE, =4 TO SIGNALIZE DEGENERATE PIVOT, =5 OTHERWISE.
*  IO  IER  =0  FOR NON-OPTIMAL SOLUTION
*           =1  FOR OPTIMAL SOLUTION.
*
      SUBROUTINE UKQGLR(N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,PTL,LENL,
     & CNR,PTR,LENR,ORDC,ORDR,ORDL,PRNR,RNR,LI,LJ,CM,A,AU,TYPE,BCOL,
     & WN01,WR01,WR02,LAM,S,C,LB,UB,X,INBCOL,NSB,NBB,ALF1,ALF2,EPS9,
     & RMIN,RMAX,DMAX,XMAX,GNORM,SNORM,XNORM,TO,EL,MQP,MLP1,INEW,IOLD,
     & KINP,LINP,ITERD,IER)
      INTEGER N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,PTL(MROWS),
     & LENL(MROWS),CNR(NZ),PTR(MROWS),LENR(MROWS),ORDC(MROWS),
     & ORDR(MROWS),ORDL(MROWS),PRNR(MCOLS+1),RNR(NZA),LI(NUP),
     & LJ(NUP),TYPE(MCOLS),BCOL(MCOLS),WN01(MCOLS),INBCOL(MCOLS),
     & NSB,NBB,MQP,MLP1,INEW,IOLD,KINP,LINP,ITERD,IER
      REAL*8 CM(NUP),A(NZA),AU(NAU),WR01(MCOLS),WR02(MCOLS),LAM(MCOLS),
     & S(MCOLS),C(MCOLS),LB(MCOLS),UB(MCOLS),X(MCOLS),ALF1,ALF2,EPS9,
     & RMIN,RMAX,DMAX,XMAX,GNORM,SNORM,XNORM,TO,EL
      INTEGER OPFL,SPAN,NSOLD,TYPV
      REAL*8 TEMP,TK,MULT,ETAP,EPS13,EPS19
      INTEGER IBLAND,JBLAND,NUMP,JSTRT,JSTOP,ISTRT,ISTOP,JOB,IND
      INTEGER I,J,K,L,J1,K1,KA
      PARAMETER(OPFL=30)
      PARAMETER(EPS19=1.0D-9)
C
C     INITIALIZE VARIABLES
C
      IER=0
      IOLD=0
      EL=0.0D 0
      IF(LINP.GT.M) THEN
        EPS13=1.0D-7
      ELSE
        EPS13=1.0D-10
      ENDIF
C
C     SET SPAN : =M FOR COMPLETE DIRECTION, =LINP FOR THE PARTIAL ONE.
C
      IF(TO.EQ.1.0D 0) THEN
        SPAN=M
      ELSE
        SPAN=LINP
      END IF
C
C     IBLAND IS A VARIABLE WHICH STORES INDEX OF SOME SUPERBASIC
C     VARIABLE TAKEN WITH RESPECT TO THE ANTICYCLING RULE.
C
      IBLAND=0
C
C     NUMP COUNTS NONZERO ELEMENTS IN NONBASIC PART OF P
C
      NUMP=0
C
C     ETAP IS PARAMETER THAT CONTROLS SPECIAL PROCESSING
C     OF P IN HIGHLY DEGENERATE SITUATIONS.
C
      ETAP=1.0D 0
      DO 50 I=1,N
        S(I)=0.0D 0
50    CONTINUE
C
C     TEST WHETHER THE NUMBER OF ITERATIONS SURPASS SOME LIMIT
C
      IF(MLP1.GT.((N*OPFL)/100+1)) THEN
        KINP=1
      END IF
C
C     NORM INITIALIZING
C
      XNORM=0.0D 0
C
C     CYCLE OF NON-BASICS
C
      DO 300 I=LINP+1,N
      MULT=0.0D 0
      J=BCOL(I)
      JSTRT=PRNR(J)
      JSTOP=PRNR(J+1)-1
      DO 200 J1=JSTRT,JSTOP
        K=RNR(J1)
        MULT=MULT+A(J1)*LAM(K)
200   CONTINUE
C
C     REDUCE COST EVALUATING
C
      MULT=C(J)-MULT
      LAM(I)=MULT
      TYPV=TYPE(J)
C
C     EXCLUDE FIXED VARIABLES
C
      IF(TYPV.EQ.5) THEN
        GO TO 250
      ELSEIF(TYPV.EQ.3.OR.TYPV.EQ.4.) THEN
        IF(ABS(LB(J)-UB(J)).LE.EPS9) THEN
          GO TO 250
        END IF
      END IF
C
C TEST WHETHER THE REDUCED COST CORRESPONDS TO OPTIMAL SOLUTION.
C NEGATIVE GRADIENT DIRECTION IS PROJECTED.
C
         IF(TYPV.EQ.2.OR.TYPV.EQ.4) THEN
C
C     WE ARE AT THE UPPER BOUND
C
           MULT=MIN(0.0D 0,-MULT)
         ELSEIF(TYPV.EQ.3.OR.TYPV.EQ.1) THEN
C
C     WE ARE AT THE LOWER BOUND
C
           MULT=MAX(0.0D 0,-MULT)
         ELSE
           MULT=-MULT
         END IF
         S(J)=MULT
         TEMP=ABS(MULT)
         EL=MAX(EL,TEMP)
         IF(TEMP.LE.EPS13) THEN
           S(J)=0.0D 0
           GO TO 250
         END IF
         IBLAND=J
         JBLAND=I
         NUMP=NUMP+1
         IF(ABS(MULT).GT.XNORM) XNORM=ABS(MULT)
         IER=1
C
C     JUMP LABEL WHEN THE SOLUTION IS NOT OPTIMAL
C
250   CONTINUE
C
C     END OF CYCLE
C
300   CONTINUE
C
C     SUPERBASIC PART OF THE DIRECTION
C
      DO 330 I=SPAN+1,LINP
      J=BCOL(I)
      S(J)=-LAM(I)
      IBLAND=J
      JBLAND=I
      IF(ABS(S(J)).GE.EPS19) IER=1
330   CONTINUE
C
C     OPTIMALITY CHECK
C
      IF(IER.EQ.0) THEN
        IER=1
        GO TO 900
      ELSE
        IER=0
      END IF
C
C     DECIDE WHETHER WE WILL DEVOTE SPECIAL CARE TO DEGENERACY
C     OR NOT. THE LATTER CASE WILL BE WHEN MLP1.EQ.0 OR WHEN
C     NUMBER OF PIVOTS IS  TOO LARGE.
C
      IF(MLP1.NE.0.AND.NUMP.LT.ETAP*MLP1.AND.MQP.GT.0.
     *                OR.KINP.EQ.1)                   THEN
C
C     INTO WN01 ARE PUSHED THE RESTRICTIONS OF THE FOLLOWING SHAPE
C               +J  .....BASIC VARIABLE J IS LOWER BOUNDED DEGENERATE
C               -J  .....BASIC VARIABLE J IS UPPER BOUNDED DEGENERATE
C
      NSOLD=LINP
      IND=0
      XNORM=0.0D 0
      DO 500 I=1,M
        J=BCOL(I)
        TYPV=TYPE(J)
        IF(TYPV.EQ.-1.OR.TYPV.EQ.-3) THEN
C
C LOWER BOUNDED BASIC VARIABLE
C
          IF(ABS(X(J)-LB(J)).LE.EPS9) THEN
            IND=IND+1
            WN01(IND)=J
          END IF
        END IF
        IF(TYPV.EQ.-2.OR.TYPV.EQ.-3) THEN
C
C UPPER BOUNDED BASIC VARIABLE
C
          IF(ABS(X(J)-UB(J)).LE.EPS9) THEN
            IND=IND+1
            WN01(IND)=-J
          END IF
      END IF
500   CONTINUE
C
C     DEGENERACY RULE IS SELECTED
C
      IF(MQP.EQ.1) THEN
      IF(KINP.EQ.1) THEN
        ISTRT=JBLAND
        ISTOP=JBLAND
        DO 560 I=SPAN+1,N
          IF(I.NE.JBLAND) THEN
            S(BCOL(I))=0.0D 0
          END IF
560     CONTINUE
      ELSE
        ISTRT=SPAN+1
        ISTOP=N
      END IF
      DO 700 I=ISTRT,ISTOP
      J1=BCOL(I)
      TK=S(J1)
      IF(TK.EQ.0.0D 0) GO TO 700
C
C     FORM B**-1 * B(-) * EI
C
      INEW=BCOL(I)
      CALL UKSGLK(M,MROWS,MCOLS,NZA,INEW,RNR,PRNR,A,WR01)
C
C     FTRAN
C
      JOB=0
      CALL UXSGFB(M,NZ,NAU,NEL,NUP,PTL,LENL,CNR,PTR,LENR,ORDC,ORDR,
     *            ORDL,LI,LJ,CM,AU,WR01,WR02,JOB)
C
C     DECIDE, WHETHER THE COMPONENT IS ZERO OR NOT
C
      DO 600 J=1,IND
      K=WN01(J)
      KA=ABS(K)
      K1=INBCOL(KA)
      IF(KA.NE.K) THEN
        IF(TK*WR01(K1).LT.0.0D 0)   THEN
          S(J1)=0.0D 0
          IOLD=K1
          GO TO 700
        END IF
      ELSE
        IF(TK*WR01(K1).GT.0.0D 0)   THEN
          S(J1)=0.0D 0
          IOLD=K1
          GO TO 700
        END IF
      END IF
600   CONTINUE
C
C     IF WE ARE HERE, THEN THERE EXIST S(I) <> 0,ABS(S(I)).GT.EPS9.
C
      XNORM=MAX(XNORM,ABS(TK))
C
C
C     CHANGE LINP,BCOL WHEN SUPERBASIS IS INCREASED
C
      IF(JBLAND.GT.LINP) THEN
        LINP=LINP+1
        L=BCOL(LINP)
        BCOL(LINP)=INEW
        BCOL(I)=L
        INBCOL(J1)=LINP
        IF(I.GT.LINP) THEN
          INBCOL(L)=0
        ELSE
          INBCOL(L)=I
        END IF
      END IF
C
C     ADD THE COLUMN
C
      IF(TK.NE.0) THEN
        DO 650 J=1,M
        K1=BCOL(J)
        S(K1)=S(K1)-TK*WR01(J)
650     CONTINUE
      END IF
700   CONTINUE
C
C     NORM TEST, CALL UKQGLX FOR SETTING RMAX
C
      CALL UKQGLX(MCOLS,M,TYPE,X,LB,UB,S,BCOL,NSB,NBB,WN01,ALF1,ALF2,
     & EPS9,RMIN,RMAX,DMAX,XMAX,GNORM,XNORM,LINP)
      IF(RMAX.LE.EPS9) THEN
        DO 710 I=NSOLD+1,LINP
          INBCOL(I)=0
  710   CONTINUE
        LINP=NSOLD
        KINP=1
        XNORM=0.0D 0
      ELSE
        IF(XNORM.LE.EPS13) THEN
          MLP1=MLP1+1
          INEW=IBLAND
        ELSE
          MLP1=0
          KINP=0
        END IF
      END IF
C
C     END IF OF MQP.EQ.1
C
      END IF
                                                        ELSE
C
C     FILL WR01 WITH NONBASIC PART OF P
C
      XNORM=0.0D 0
      DO 800 I=SPAN+1,N
        J=BCOL(I)
        WR02(I-SPAN)=-S(J)
        XNORM=MAX(ABS(S(J)),XNORM)
800   CONTINUE
C
C     COMPUTE N*PN
C
      JOB=0
      CALL UXSGMM(M,N-SPAN,NZA,RNR,PRNR,ORDR,BCOL(SPAN+1),A,WR02,
     *            WR01,JOB)
      JOB=0
      CALL UXSGFB(M,NZ,NAU,NEL,NUP,PTL,LENL,CNR,PTR,LENR,ORDC,ORDR,
     *            ORDL,LI,LJ,CM,AU,WR01,WR02,JOB)
      INEW=IBLAND
      DO 950 I=1,M
        J=BCOL(I)
        S(J)=WR01(I)
        XNORM=MAX(ABS(S(J)),XNORM)
950   CONTINUE
C
C ADD ELEMENT INTO SUPERBASIS
C
      NSOLD=LINP
      DO 1000 I=LINP+1,N
      J=BCOL(I)
      IF(S(J).EQ.0.0D 0) GO TO 1000
      LINP=LINP+1
      L=BCOL(LINP)
      BCOL(LINP)=J
      BCOL(I)=L
      INBCOL(J)=LINP
      IF(I.GT.LINP) THEN
                  INBCOL(L)=0
                  ELSE
                  INBCOL(L)=I
      END IF
1000  CONTINUE
C
C XNORM IS THE MAXIMUM ONE, CALL UKQGLX
C
      CALL UKQGLX(MCOLS,M,TYPE,X,LB,UB,S,BCOL,NSB,NBB,WN01,ALF1,ALF2,
     & EPS9,RMIN,RMAX,DMAX,XMAX,GNORM,XNORM,LINP)
      IF(RMAX.LE.EPS9) THEN
        DO 1010 I=NSOLD+1,LINP
          INBCOL(I)=0
 1010   CONTINUE
        LINP=NSOLD
        KINP=1
        XNORM=0.0D 0
      ELSE
        IF(XNORM.LE.EPS13) THEN
          KINP=1
        ELSE
          MLP1=0
          KINP=0
        END IF
      END IF
C
C    END IF OF THE MAIN DEGENERACY BRANCH
C
      END IF
C
C     OUTPUT DISCUSSION: POSSIBILITIES: XNORM=0,DEGENERACY PART WAS USED
C     AND WE HAVE INEW,IOLD - EXCHANGE OF COLUMNS
C                                      XNORM<>0,DEGENERACY PART WAS USED
C     AND LINSEARCH WILL BE USED - OUTPUT IN P.
C     XNORM CAN BE ZERO OR NONZERO , DEGENERACY PART WAS NOT USED ,
C     INEW IS DEFINED AND ULSGLX FIND IOLD ACC. TO BLAND RULE -
C     RMAX=0,NBB<>0.
C
900   CONTINUE
C
C SET ITERD
C
      IF(IER.EQ.1.AND.TO.GT.0) THEN
        ITERD=1
      ELSEIF(IER.EQ.1.AND.TO.EQ.0) THEN
        ITERD=2
      ELSEIF(XNORM.EQ.0.AND.IOLD.NE.0) THEN
        ITERD=3
C      ELSEIF(XNORM.EQ.0.AND.IOLD.EQ.0) THEN
C        ITERD=4
      ELSE
        ITERD=0
      END IF
      SNORM=XNORM
      RETURN
      END
