* SUBROUTINE ULLPA2             ALL SYSTEMS                   97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE INITIAL FEASIBLE POINT AND THE LINEAR PROGRAMMING
* SUBROUTINE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  IO  N  DIMENSION OF THE MANIFOLD DEFINED BY ACTIVE CONSTRAINTS.
*  II  NA  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IXA(NF)  VECTOR CONTAINING INFORMATION ON TRUST REGION ACTIVITY.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  XO(NF)  SAVED VECTOR OF VARIABLES.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  AF(2*NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFD(2*NA)  VECTOR CONTAINING INCREMENTS OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IA(2*NA)  VECTOR CONTAINING TYPES OF DEVIATIONS.
*  II  IAA(NF+1)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RI  AG(NF*NA)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         APPROXIMATED FUNCTIONS.
*  RI  AR((NF+1)*(NF+2)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RO  AZ(NF+1)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RO  G(NF+1)  GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RI  S(NF+1)  DIRECTION VECTOR.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  EPS9  TOLERANCE FOR ACTIVITY OF CONSTRAINTS.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  XNORM  VALUE OF LINEARIZED MINIMAX FUNCTION.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  II  JOB  SPECIFICATION OF THE PROBLEM. JOB=0-QUADRATIC PROGRAMMING
*         PROBLEM. JOB=1-QUADRATIC PROGRAMMING SUBPROBLEM FOR DIRECTION
*         DETERMINATION.
*
* COMMON DATA :
*  II  LCS  VARIABLE WHICH SPECIFIES TRUST REGION BOUNDS.
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IO  ITERL  TYPE OF FEASIBLE POINT. ITERL=1-ARBITRARY FEASIBLE POINT.
*         ITERL=2-OPTIMUM FEASIBLE POINT. ITERL=-1 FEASIBLE POINT DOES
*         NOT EXISTS. ITERL=-2 OPTIMUM FEASIBLE POINT DOES NOT EXISTS.
*  II  IEXT  TYPE OF EXTREMUM. IEXT<0-MINIMAX. IEXT=0-CHEBYSHEV
*         APPROXIMATION, IEXT>0-MAXIMIN.
*
* SUBPROGRAMS USED :
*  S   ULINIT  DETERMINATION OF INITIAL POINT SATISFYING SIMPLE BOUNDS.
*  S   ULAMAX  DETERMINATION OF THE MAXIMUM DEVIATION.
*  S   ULMAXA  MAXIMUM STEPSIZE USING LINEAR APPROXIMATED FUNCTIONS.
*  S   ULMAXS  MAXIMUM STEPSIZE USING SIMPLE BOUNDS.
*  S   ULMAXT  MAXIMUM STEPSIZE USING TRUST REGION BOUNDS.
*  S   ULNEWA  IDENTIFICATION OF ACTIVE LINEAR APPROXIMATED FUNCTIONS.
*  S   ULNEWS  IDENTIFICATION OF ACTIVE SIMPLE BOUNDS.
*  S   ULNEWT  IDENTIFICATION OF ACTIVE TRUST REGION BOUNDS.
*  S   ULDIRA  NEW VALUES OF APPROXIMATED FUNCTIONS.
*  S   ULDIRS  NEW VALUES OF VARIABLES.
*  S   ULSETA  INITIAL VALUES OF APPROXIMATED FUNCTIONS.
*  S   ULCLAG  VECTOR OF LAGRANGE MULTIPLIERS IS DETERMINED.
*  S   ULSLAG  NEGATIVE PROJECTED GRADIENT IS DETERMINED.
*  S   ULTLAG  THE OPTIMUM LAGRANGE MULTIPLIER IS DETERMINED.
*  S   ULVLAG  AN AUXILIARY VECTOR IS DETERMINED.
*  S   ULADF1  CONSTRAINT ADDITION.
*  S   ULRMF0  CONSTRAINT DELETION.
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXDSMI  DETERMINATION OF THE INITIAL UNIT DENSE SYMMETRIC
*         MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVINA  ABSOLUTE VALUES OF ELEMENTS OF AN INTEGER VECTOR.
*  S   UXVINC  UPDATE OF AN INTEGER VECTOR.
*  S   UXVIND  CHANGE OF THE INTEGER VECTOR FOR CONSTRAINT ADDITION.
*  S   UXVINT  CHANGE OF THE INTEGER VECTOR FOR TRUST REGION BOUND
*         ADDITION.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UOLLP1  PRINT OF ENTRY TO QUADRATIC PROGRAMMING.
*  S   UOLLP2  PRINT OF EXIT FROM QUADRATIC PROGRAMMING.
*  S   UOLQP5  PRINT OF INFORMATION DURING QUADRATIC PROGRAMMING.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE ULLPA2(NF,N,NA,X,IX,IXA,XN,XO,XL,XU,AF,AFD,IA,IAA,
     & AG,AR,AZ,G,S,ETA9,EPS7,EPS9,XDEL,XNORM,UMAX,GMAX,JOB)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,NA,IX(NF),IXA(NF),IA(2*NA),IAA(NF+1),JOB
      REAL*8 X(NF),XN(NF),XO(NF),XL(NF),XU(NF),AF(2*NA),AFD(2*NA),
     & AG(NF*NA),AR((NF+1)*(NF+2)/2),AZ(NF+1),G(NF+1),S(NF+1),ETA9,
     & EPS7,EPS9,XDEL,XNORM,UMAX,GMAX
      REAL*8 POM,CON,E,T,DMAX,SNORM
      INTEGER NAD,I,IOLD,JOLD,KOLD,INEW,JNEW,KNEW,MODE,IER,KA,KREM
      CALL UOLLP1('ULLPA2')
      N=NF
      T=ONE
      CON=ETA9
      IF (IEXT.EQ.0) THEN
      NAD=2*NA
      ELSE
      NAD=NA
      ENDIF
C
C     INITIATION
C
      CALL UXVCOP(NF,X,XO)
      JOLD=0
      JNEW=0
      NRED=0
      KREM=0
      ITERL=0
      DMAX=ZERO
      IF (LCS.GT.0) CALL UXVINC(NF,IX,IXA)
      IF (KBF.GT.0) THEN
C
C     SHIFT OF VARIABLES FOR SATISFYING SIMPLE BOUNDS
C
      CALL UXVINA(NF,IX)
      CALL ULINIT(NF,X,IX,XN,XO,XL,XU,XDEL,EPS9,INEW,ITERL)
      IF (ITERL.LT.0) THEN
      CALL UOERR1('ULLPA2',10-ITERL)
      GO TO 6
      ENDIF
      DO 1 I=1,NF
      IF (IX(I).LT.0) THEN
      N=N-1
      IAA(NF-N)=-I
      ENDIF
    1 CONTINUE
      CALL UXDSMI(NF-N,AR)
      ENDIF
      CALL UXVINA(NAD,IA)
    3 IF (ITERL.EQ.0) THEN
      ITERL=1
      IF (NF.GT.N) CALL ULSETA(NF,NA,X,XN,XO,AF,IA,AG,S,NORMF)
C
C     INITIAL MINIMAX VARIABLE
C
      CALL ULAMAX(NA,NAD,AF,IA,XNORM,ETA9)
C
C     ADDITION OF ACTIVE FUNCTIONS
C
      DO 11 KA=1,NAD
      IF (IA(KA).GT.0) THEN
      INEW=0
      CALL ULNEWA(NA,0,KA,AF,IA,EPS9,XNORM,INEW,KNEW)
      CALL ULADF0(NF,N,NA,NA,0,IA,IAA,AG,AR,AG,S,EPS7,GMAX,UMAX,E,T,
     & INEW,KNEW,NADD,IER)
      CALL UXVIND(NAD,IA,KA,IER)
      ENDIF
   11 CONTINUE
      ENDIF
    5 IF (NF.LE.N) THEN
C
C     IMPOSSIBLE SITUATION
C
      ITERL=-5
      CALL UOERR1('ULLPA2',10-ITERL)
      GO TO 6
      ENDIF
C
C     LAGRANGE MULTIPLIERS DETERMINATION
C
      CALL ULCLAG(NF,N,NA,0,IA,IAA,AR,AZ,SNORM,KOLD)
      IF (KOLD.EQ.0) THEN
C
C     IMPOSSIBLE SITUATION
C
      ITERL=-6
      CALL UOERR1('ULLPA2',10-ITERL)
      GO TO 6
      ENDIF
      CALL ULTLAG(NF,N,NA,NA,0,IX,IA,IAA,AZ,IA,EPS7,UMAX,IOLD)
C
C     PROJECTED GRADIENT FOR THE SECOND PHASE
C
      CALL UXVSET(NF,ZERO,S)
      CALL ULSLAG(NF,N,NA,NA,0,IAA,AG,AZ,AG,S,EPS7,GMAX)
      MODE=1
      CALL UOLQP5(NF,N,0,NA,X,IX,IA,IA,IAA,AF,AR,AZ,DMAX,GMAX,UMAX,
     & XNORM,JOLD,JNEW,MODE)
      INEW=0
      IF (GMAX.EQ.ZERO) THEN
C
C     OPTIMUM ON A LINEAR MANIFOLD OBTAINED
C
      IF (IOLD.EQ.0) THEN
C
C     OPTIMAL SOLUTION ACHIEVED
C
      IF (JOB.GT.0) THEN
      CALL UXVDIF(NF,X,XO,S)
      CALL UXVCOP(NF,XO,X)
      CALL UXVSET(NF,ZERO,G)
      ENDIF
      ITERL= 2
      GO TO 6
      ELSE
C
C     CONSTRAINT DELETION
C
      CALL ULRMF0(NF,N,NA,NA,0,IX,IXA,IA,IAA,AR,IA,S,LCS,IOLD,KREM,
     & NREM,IER)
      DMAX=ZERO
      JOLD=IOLD
      JNEW=0
      GO TO 5
      ENDIF
      ELSE
C
C     STEPSIZE SELECTION
C
      POM=CON
      CALL ULMAXA(NF,NA,NAD,0,AF,AFD,IA,AG,S,POM,XNORM,SNORM,KREM,INEW)
      IF (NORMF.GT.0) CALL UXVMUL(NF,XN,S,S,1)
      IF (LCS.GT.0) CALL ULMAXT(NF,X,IX,IXA,XN,XO,S,POM,XDEL,NORMF,
     & KREM,INEW)
      CALL ULMAXS(NF,X,IX,XL,XU,S,POM,KBF,KREM,INEW)
      IF (INEW.EQ.0) THEN
C
C     BOUNDED SOLUTION DOES NOT EXIST
C
      ITERL=-2
      CALL UOERR1('ULLPA2',10-ITERL)
      GO TO 6
      ELSE
C
C     STEP REALIZATION
C
      CALL ULDIRS(NF,X,IX,S,POM,KBF)
      CALL ULDIRA(NA,NAD,AF,AFD,IA,POM,XNORM,SNORM)
C
C     CONSTRAINT ADDITION
C
      IF (INEW.GT.0) THEN
      KA=INEW
      INEW=0
      CALL ULNEWA(NA,0,KA,AF,IA,EPS9,XNORM,INEW,KNEW)
      CALL ULADF0(NF,N,NA,NA,0,IA,IAA,AG,AR,AG,S,EPS7,GMAX,UMAX,E,T,
     & INEW,KNEW,NADD,IER)
      CALL UXVIND(NAD,IA,KA,IER)
      ELSE IF (INEW+NF.GE.0) THEN
      I=-INEW
      INEW=0
      CALL ULNEWS(NF,X,IX,XN,XL,XU,EPS9,NORMF,I,INEW)
      CALL ULADF0(NF,N,NA,NA,0,IA,IAA,AG,AR,AG,S,EPS7,GMAX,UMAX,E,T,
     & INEW,KNEW,NADD,IER)
      CALL UXVIND(NF,IX,I,IER)
      ELSE
      I=-(INEW+NF)
      INEW=0
      CALL ULNEWT(NF,X,IX,IXA,XN,XO,XDEL,EPS9,NORMF,I,INEW)
      CALL ULADF0(NF,N,NA,NA,0,IA,IAA,AG,AR,AG,S,EPS7,GMAX,UMAX,E,T,
     & INEW,KNEW,NADD,IER)
      CALL UXVIND(NF,IX,I,IER)
C      CALL UXVINT(NF,IX,IXA,I,IER)
      ENDIF
      DMAX=POM
      NRED=NRED+1
      JNEW=INEW
      JOLD=0
      GO TO 3
      ENDIF
      ENDIF
    6 CALL UOLLP2
      RETURN
      END
