* SUBROUTINE ULLPB2             ALL SYSTEMS                   91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE INITIAL FEASIBLE POINT AND THE LINEAR PROGRAMMING
* SUBROUTINE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  IO  N  DIMENSION OF THE MANIFOLD DEFINED BY ACTIVE CONSTRAINTS.
*  II  NC  NUMBER OF LINEAR CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IXA(NF)  VECTOR CONTAINING INFORMATION ON TRUST REGION ACTIVITY.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  XO(NF)  SAVED VECTOR OF VARIABLES.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  CF(NF)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCYIONS.
*  RO  CFD(NF)  VECTOR CONTAINING INCREMENTS OF THE CONSTRAINT
*         FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RO  CZ(NF)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GO(NF)  SAVED GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(NF)  DIRECTION VECTOR.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  EPS9  TOLERANCE FOR ACTIVITY OF CONSTRAINTS.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  II  MFP  TYPE OF FEASIBLE POINT. MFP=1-ARBITRARY FEASIBLE POINT.
*         MFP=2-OPTIMUM FEASIBLE POINT. MFP=3-REPEATED SOLUTION.
*  II  JOB  SPECIFICATION OF THE PROBLEM. JOB=0-LINEAR PROGRAMMING
*         PROBLEM. JOB=1-LINEAR PROGRAMMING SUBPROBLEM FOR DIRECTION
*         DETERMINATION.
*
* COMMON DATA :
*  II  LCS  VARIABLE WHICH SPECIFIES TRUST REGION BOUNDS.
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IO  ITERL  TYPE OF FEASIBLE POINT. ITERL=1-ARBITRARY FEASIBLE POINT.
*         ITERL=2-OPTIMUM FEASIBLE POINT. ITERL=-1 FEASIBLE POINT DOES
*         NOT EXISTS. ITERL=-2 OPTIMUM FEASIBLE POINT DOES NOT EXISTS.
*
* SUBPROGRAMS USED :
*  S   ULINIT  DETERMINATION OF INITIAL POINT SATISFYING SIMPLE BOUNDS.
*  S   ULMAXL  MAXIMUM STEPSIZE USING LINEAR CONSTRAINTS.
*  S   ULMAXS  MAXIMUM STEPSIZE USING SIMPLE BOUNDS.
*  S   ULMAXT  MAXIMUM STEPSIZE USING TRUST REGION BOUNDS.
*  S   ULNEWL  IDENTIFICATION OF ACTIVE LINEAR CONSTRAINTS.
*  S   ULNEWS  IDENTIFICATION OF ACTIVE SIMPLE BOUNDS.
*  S   ULNEWT  IDENTIFICATION OF ACTIVE TRUST REGION BOUNDS.
*  S   ULDIRL  NEW VALUES OF CONSTRAINT FUNCTIONS.
*  S   ULDIRS  NEW VALUES OF VARIABLES.
*  S   ULSETC  INITIAL VALUES OF CONSTRAINT FUNCTIONS.
*  S   ULSETG  DETERMINATION OF THE FIRST PHASE GRADIENT VECTOR.
*  S   ULGLAG  GRADIENT OF THE LAGRANGIAN FUNCTION IS DETERMINED.
*  S   ULSLAG  NEGATIVE PROJECTED GRADIENT IS DETERMINED.
*  S   ULTLAG  THE OPTIMUM LAGRANGE MULTIPLIER IS DETERMINED.
*  S   ULVLAG  AN AUXILIARY VECTOR IS DETERMINED.
*  S   ULADF1  CONSTRAINT ADDITION.
*  S   ULRMF0  CONSTRAINT DELETION.
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXDSMI  DETERMINATION OF THE INITIAL UNIT DENSE SYMMETRIC
*         MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVINA  ABSOLUTE VALUES OF ELEMENTS OF AN INTEGER VECTOR.
*  S   UXVINC  UPDATE OF AN INTEGER VECTOR.
*  S   UXVIND  CHANGE OF THE INTEGER VECTOR FOR CONSTRAINT ADDITION.
*  S   UXVINT  CHANGE OF THE INTEGER VECTOR FOR TRUST REGION BOUND
*         ADDITION.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UOLLP1  PRINT OF ENTRY TO QUADRATIC PROGRAMMING.
*  S   UOLLP2  PRINT OF EXIT FROM QUADRATIC PROGRAMMING.
*  S   UOLQP4  PRINT OF INFORMATION DURING QUADRATIC PROGRAMMING.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE ULLPB2(NF,N,NC,X,IX,IXA,XN,XO,XL,XU,CF,CFD,IC,ICA,
     & CL,CU,CG,CR,CZ,G,GO,S,ETA9,EPS7,EPS9,XDEL,UMAX,GMAX,MFP,JOB)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,NC,IX(NF),IXA(NF),IC(NC),ICA(NF),MFP,JOB
      REAL*8 X(NF),XN(NF),XO(NF),XL(NF),XU(NF),CF(NC),CFD(NC),CL(NC),
     & CU(NC),CG(NF*NC),CR(NF*(NF+1)/2),CZ(NF),G(NF),GO(NF),S(NF),
     & ETA9,EPS7,EPS9,XDEL,UMAX,GMAX
      REAL*8 POM,CON,DMAX
      INTEGER IPOM,I,IOLD,JOLD,INEW,JNEW,MODE,IER,KREM,KC
      CALL UOLLP1('ULLPB2')
      CON=ETA9
C
C     INITIATION
C
      CALL UXVCOP(NF,X,XO)
      IF (JOB.EQ.0) CALL UXVCOP(NF,G,GO)
      IPOM=0
      JOLD=0
      JNEW=0
      NRED=0
      KREM=0
      ITERL=1
      DMAX=ZERO
      IF (MFP.EQ.3) GO TO 5
      N=NF
      ITERL=0
      IF (LCS.GT.0) CALL UXVINC(NF,IX,IXA)
      IF (KBF.GT.0) THEN
C
C     SHIFT OF VARIABLES FOR SATISFYING SIMPLE BOUNDS
C
      CALL UXVINA(NF,IX)
      CALL ULINIT(NF,X,IX,XN,XO,XL,XU,XDEL,EPS9,INEW,ITERL)
      IF (ITERL.LT.0) THEN
      CALL UOERR1('ULLPB2',10-ITERL)
      GO TO 6
      ENDIF
      DO 1 I=1,NF
      IF (IX(I).LT.0) THEN
      N=N-1
      ICA(NF-N)=-I
      ENDIF
    1 CONTINUE
      CALL UXDSMI(NF-N,CR)
      ENDIF
      IF (NC.GT.0) THEN
C
C     ADDITION OF ACTIVE CONSTRAINTS AND INITIAL CHECK OF FEASIBILITY
C
      CALL UXVINA(NC,IC)
      IF (NF.GT.N) CALL ULSETC(NF,NC,X,XN,XO,CF,IC,CG,S,ISNC,NORMF)
      DO 2 KC=1,NC
      IF (IC(KC).NE.0) THEN
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL ULADR0(NF,N,NC,ICA,CG,CR,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NC,IC,KC,IER)
      IF (IC(KC).LT.-10) IPOM=1
      ENDIF
    2 CONTINUE
      ENDIF
    3 IF (IPOM.EQ.1) THEN
C
C     CHECK OF FEASIBILITY AND UPDATE OF THE FIRST PHASE OBJECTIVE
C     FUNCTION
C
      CALL ULSETG(NF,NC,IC,CG,G,INEW)
      IF (INEW.EQ.0) IPOM=0
      ENDIF
      IF (IPOM.EQ.0.AND.ITERL.EQ.0) THEN
C
C     FEASIBILITY ACHIEVED
C
      ITERL=1
      CALL UXVCOP(NF,GO,G)
      IF (MFP.EQ.1) GO TO 6
      ENDIF
C
C     LAGRANGE MULTIPLIERS DETERMINATION
C
    5 IF (NF.GT.N) THEN
      CALL ULVLAG(NF,N,NC,NC,NC,ICA,CG,CG,G,CZ)
      CALL UXDPRB(NF-N,CR,CZ,0)
      CALL ULTLAG(NF,N,NC,NC,NC,IX,IC,ICA,CZ,IC,EPS7,UMAX,IOLD)
      ELSE
      IOLD=0
      UMAX=ZERO
      ENDIF
C
C     PROJECTED GRADIENT DETERMINATION
C
      IF (N.GT.0) THEN
      CALL UXVNEG(NF,G,S)
      CALL ULSLAG(NF,N,NC,NC,NC,ICA,CG,CZ,CG,S,EPS7,GMAX)
      ELSE
      GMAX=ZERO
      ENDIF
      MODE=1-IPOM
      CALL UOLQP4(NF,N,NC,X,IX,IC,ICA,CF,CR,CZ,DMAX,GMAX,UMAX,JOLD,JNEW,
     & MODE)
      INEW=0
      IF (GMAX.EQ.ZERO) THEN
C
C     OPTIMUM ON A LINEAR MANIFOLD OBTAINED
C
      IF (IOLD.EQ.0) THEN
      IF (IPOM.EQ.0) THEN
C
C     OPTIMAL SOLUTION ACHIEVED
C
      IF (JOB.GT.0) THEN
      CALL UXVDIF(NF,X,XO,S)
      CALL UXVCOP(NF,XO,X)
      CALL UXVCOP(NF,GO,G)
      CALL ULGLAG(NF,N,NC,NC,NC,ICA,CG,CZ,CG,G)
      ENDIF
      ITERL= 2
      GO TO 6
      ELSE
      IPOM=0
      DO 22 KC=1,NC
      IF (IC(KC).LT.-10) THEN
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      IF (IC(KC).LT.-10) IPOM=1
      ENDIF
   22 CONTINUE
      IF (IPOM.EQ.0) THEN
C
C     OPTIMAL SOLUTION ACHIEVED
C
      CALL UXVCOP(NF,GO,G)
      IF (JOB.GT.0) THEN
      CALL UXVDIF(NF,X,XO,S)
      CALL UXVCOP(NF,XO,X)
      CALL ULGLAG(NF,N,NC,NC,NC,ICA,CG,CZ,CG,G)
      ENDIF
      ITERL= 2
      GO TO 6
      ELSE
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      CALL UXVCOP(NF,GO,G)
      ITERL=-1
      CALL UOERR1('ULLPB2',10-ITERL)
      GO TO 6
      ENDIF
      ENDIF
      ELSE
C
C     CONSTRAINT DELETION
C
      CALL ULRMF0(NF,N,NC,NC,NC,IX,IXA,IC,ICA,CR,IC,S,LCS,IOLD,KREM,
     & NREM,IER)
      DMAX=ZERO
      JOLD=IOLD
      JNEW=0
      GO TO 5
      ENDIF
      ELSE
C
C     STEPSIZE SELECTION
C
      POM=CON
      CALL ULMAXL(NF,NC,CF,CFD,IC,CL,CU,CG,S,POM,KBC,KREM,INEW)
      IF (NORMF.GT.0) CALL UXVMUL(NF,XN,S,S,1)
      IF (LCS.GT.0) CALL ULMAXT(NF,X,IX,IXA,XN,XO,S,POM,XDEL,NORMF,
     & KREM,INEW)
      CALL ULMAXS(NF,X,IX,XL,XU,S,POM,KBF,KREM,INEW)
      IF (INEW.EQ.0) THEN
      IF (IPOM.EQ.0) THEN
C
C     BOUNDED SOLUTION DOES NOT EXIST
C
      ITERL=-2
      CALL UOERR1('ULLPB2',10-ITERL)
      ELSE
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      ITERL=-3
      CALL UOERR1('ULLPB2',10-ITERL)
      ENDIF
      GO TO 6
      ELSE
C
C     STEP REALIZATION
C
      CALL ULDIRS(NF,X,IX,S,POM,KBF)
      CALL ULDIRL(NC,CF,CFD,IC,POM,KBC)
C
C     CONSTRAINT ADDITION
C
      IF (INEW.GT.0) THEN
      KC=INEW
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL ULADR0(NF,N,NC,ICA,CG,CR,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NC,IC,KC,IER)
      ELSE IF (INEW+NF.GE.0) THEN
      I=-INEW
      INEW=0
      CALL ULNEWS(NF,X,IX,XN,XL,XU,EPS9,NORMF,I,INEW)
      CALL ULADR0(NF,N,NC,ICA,CG,CR,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NF,IX,I,IER)
      ELSE
      I=-(INEW+NF)
      INEW=0
      CALL ULNEWT(NF,X,IX,IXA,XN,XO,XDEL,EPS9,NORMF,I,INEW)
      CALL ULADR0(NF,N,NC,ICA,CG,CR,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NF,IX,I,IER)
C      CALL UXVINT(NF,IX,IXA,I,IER)
      ENDIF
      DMAX=POM
      NRED=NRED+1
      JNEW=INEW
      JOLD=0
      GO TO 3
      ENDIF
      ENDIF
    6 CALL UOLLP2
      RETURN
      END
