* SUBROUTINE ULQPB1             ALL SYSTEMS                   97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* PRIMAL NULL SPACE QUADRATIC PROGRAMMING METHOD.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  IO  N  DIMENSION OF THE MANIFOLD DEFINED BY ACTIVE CONSTRAINTS.
*  II  NC  NUMBER OF LINEAR CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  II  IXA(NF)  VECTOR CONTAINING INFORMATION ON TRUST REGION ACTIVITY.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  XO(NF)  SAVED VECTOR OF VARIABLES.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  CF(NF)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RO  CFD(NF)  VECTOR CONTAINING INCREMENTS OF THE CONSTRAINT
*         FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RO  GO(NF)  SAVED GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  H(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OR INVERSION OF
*         THE HESSIAN MATRIX APPROXIMATION.
*  RI  S(NF)  DIRECTION VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RI  EPS9  TOLERANCE FOR ACTIVITY OF CONSTRAINTS.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  II  MFP  TYPE OF FEASIBLE POINT. MFP=1-ARBITRARY FEASIBLE POINT.
*         MFP=2-OPTIMUM FEASIBLE POINT. MFP=3-REPEATED SOLUTION.
*
* COMMON DATA :
*  II  LCS  VARIABLE WHICH SPECIFIES TRUST REGION BOUNDS.
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  IU  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*         IDECF=1-GILL-MURRAY DECOMPOSITION. IDECF=9-INVERSION.
*         IDECF=10-DIAGONAL MATRIX.
*  IU  NDECF  NUMBER OF DECOMPOSITIONS.
*  IO  ITERQ  TYPE OF FEASIBLE POINT. ITERQ=1-ARBITRARY FEASIBLE POINT.
*         ITERQ=2-OPTIMUM FEASIBLE POINT. ITERQ=-1 FEASIBLE POINT DOES
*         NOT EXISTS. ITERQ=-2 OPTIMUM FEASIBLE POINT DOES NOT EXISTS.
*
* SUBPROGRAMS USED :
*  S   ULINIT  DETERMINATION OF INITIAL POINT SATISFYING SIMPLE BOUNDS.
*  S   ULMAXL  MAXIMUM STEPSIZE USING LINEAR CONSTRAINTS.
*  S   ULMAXS  MAXIMUM STEPSIZE USING SIMPLE BOUNDS.
*  S   ULMAXT  MAXIMUM STEPSIZE USING TRUST REGION BOUNDS.
*  S   ULNEWL  IDENTIFICATION OF ACTIVE LINEAR CONSTRAINTS.
*  S   ULNEWS  IDENTIFICATION OF ACTIVE SIMPLE BOUNDS.
*  S   ULNEWT  IDENTIFICATION OF ACTIVE TRUST REGION BOUNDS.
*  S   ULDIRL  NEW VALUES OF CONSTRAINT FUNCTIONS.
*  S   ULDIRS  NEW VALUES OF VARIABLES.
*  S   ULSETC  INITIAL VALUES OF CONSTRAINT FUNCTIONS.
*  S   ULSETG  DETERMINATION OF THE FIRST PHASE GRADIENT VECTOR.
*  S   ULTRBG  DETERMINATION OF LAGRANGE MULTIPLIERS AND COMPUTATION
*         OF THE TRANSFORMED GRADIENT OF THE OBJECTIVE FUNCTION.
*  S   ULADB0  CONSTRAINT ADDITION.
*  S   ULRMB0  CONSTRAINT DELETION.
*  S   ULRMZ1  G-ORTOGONALIZATION OF A NEW BASIC VECTOR FROM THE
*         REDUCED SUBSPACE.
*  S   UXDPGF  GILL-MURRAY DECOMPOSITION OF A DENSE SYMMETRIC MATRIX.
*  S   UXDPGM  MATRIX-VECTOR PRODUCT AFTER GILL-MURRAY DECOMPOSITION.
*  S   UXDCMM  PREMULTIPLICATION OF A VECTOR BY A DENSE RECTANGULAR
*  S   UXDSMI  DETERMINATION OF THE INITIAL UNIT DENSE SYMMETRIC
*         MATRIX.
*  S   UXDSMM  MATRIX VECTOR PRODUCT.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXUDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  S   UXVINA  ABSOLUTE VALUES OF ELEMENTS OF AN INTEGER VECTOR.
*  S   UXVINC  UPDATE OF AN INTEGER VECTOR.
*  S   UXVIND  CHANGE OF THE INTEGER VECTOR FOR CONSTRAINT ADDITION.
*  S   UXVINT  CHANGE OF THE INTEGER VECTOR FOR TRUST REGION BOUND
*         ADDITION.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UXVSUM  SUM OF TWO VECTORS.
*  S   UOLQP1  PRINT OF ENTRY TO QUADRATIC PROGRAMMING.
*  S   UOLQP2  PRINT OF EXIT FROM QUADRATIC PROGRAMMING.
*  S   UOLQP3  PRINT OF INFORMATION DURING QUADRATIC PROGRAMMING.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE ULQPB1(NF,N,NC,X,IX,IXA,XN,XO,XL,XU,CF,CFD,IC,ICA,CL,
     & CU,CG,CR,CZ,G,GO,H,S,ETA0,ETA9,EPS7,EPS9,XDEL,UMAX,GMAX,MFP)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,N,NC,IX(NF),IXA(NF),IC(NC),ICA(NF),MFP
      REAL*8 X(NF),XN(NF),XO(NF),XL(NF),XU(NF),CF(NC),CFD(NC),CL(NC),
     & CU(NC),CG(NF*NC),CR(NF*(NF+1)/2),CZ(NF*NF),G(NF),GO(NF),
     & H(NF*(NF+1)/2),S(NF),ETA0,ETA9,EPS7,EPS9,XDEL,UMAX,GMAX
      REAL*8 POM,CON,DMAX
      INTEGER NCA,NCR,NCZ,IPOM,I,K,L,IOLD,JOLD,INEW,JNEW,INF,MODE,IER,
     & KREM,KC
      CALL UOLQP1('ULQPB1')
      L=0
      CON=ETA9
      IF (IDECF.LT.0) IDECF=1
      IF (IDECF.EQ.0) THEN
C
C     GILL-MURRAY DECOMPOSITION
C
      POM=ETA0
      CALL UXDPGF(NF,H,INF,POM,DMAX)
      NDECF=NDECF+1
      IDECF=1
      ENDIF
      IF (IDECF.NE.1) THEN
      ITERM=-1
      TDXX='BAD DEC1'
      CALL UOERR1('ULQPB1',1)
      GO TO 6
      ENDIF
C
C     INITIATION
C
      CALL UXVCOP(NF,X,XO)
      CALL UXVCOP(NF,G,GO)
      IPOM=0
      JOLD=0
      JNEW=0
      NRED=0
      KREM=0
      ITERQ=1
      DMAX=ZERO
      IF (MFP.EQ.3) GO TO 5
      IF (LCS.GT.0) CALL UXVINC(NF,IX,IXA)
      IF (KBF.GT.0) CALL UXVINA(NF,IX)
C
C     SHIFT OF VARIABLES FOR SATISFYING SIMPLE BOUNDS
C
      CALL ULINIT(NF,X,IX,XN,XO,XL,XU,XDEL,EPS9,INEW,ITERQ)
      IF (ITERQ.LT.0) THEN
      CALL UOERR1('ULQPB1',10-ITERQ)
      GO TO 6
      ENDIF
      N=0
      NCA=0
      NCZ=0
      DO 1 I=1,NF
      IF (KBF.GT.0.AND.IX(I).LT.0) THEN
      NCA=NCA+1
      ICA(NCA)=-I
      ELSE
      N=N+1
      CALL UXVSET(NF,ZERO,CZ(NCZ+1))
      CZ(NCZ+I)=ONE
      NCZ=NCZ+NF
      CALL ULRMZ1(NF,N,CR,CZ,H,IDECF,IER)
      ENDIF
    1 CONTINUE
      CALL UXDSMI(NCA,CR)
      IF (NC.GT.0) THEN
C
C     ADDITION OF ACTIVE CONSTRAINTS AND INITIAL CHECK OF FEASIBILITY
C
      CALL UXVINA(NC,IC)
      IF (NF.GT.N) CALL ULSETC(NF,NC,X,XN,XO,CF,IC,CG,S,ISNC,NORMF)
      DO 2 KC=1,NC
      IF (IC(KC).NE.0) THEN
      INEW=0
      CALL ULNEWL(KC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL ULADB0(NF,N,NC,ICA,CG,CR,CZ,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NC,IC,KC,IER)
      IF (IC(KC).LT.-10) IPOM=1
      ENDIF
    2 CONTINUE
      ENDIF
    3 IF (IPOM.EQ.1) THEN
C
C     CHECK OF FEASIBILITY AND UPDATE OF THE FIRST PHASE OBJECTIVE
C     FUNCTION
C
      CALL ULSETG(NF,NC,IC,CG,GO,INEW)
      IF (INEW.EQ.0) IPOM=0
      ENDIF
      IF (IPOM.EQ.0.AND.ITERQ.EQ.0) THEN
C
C     FEASIBILITY ACHIEVED
C
      ITERQ=1
      IF (MFP.EQ.1) GO TO 6
      IF (IDECF.EQ.1) THEN
      CALL UXVDIF(NF,X,XO,S)
      IF (NORMF.GT.0) CALL UXVMUL(NF,XN,S,S,-1)
      CALL UXDPGM(NF,H,S,0)
      ELSE
      CALL UXVDIF(NF,X,XO,GO)
      IF (NORMF.GT.0) CALL UXVMUL(NF,XN,GO,GO,-1)
      CALL UXDSMM(NF,H,GO,S)
      ENDIF
      CALL UXVSUM(NF,S,G,GO)
      ENDIF
C
C     LAGRANGE MULTIPLIERS AND REDUCED GRADIENT DETERMINATION
C
    5 CALL ULTRBG(NF,N,NC,IX,IC,ICA,CG,CR,CZ,GO,S,EPS7,GMAX,UMAX,IOLD)
      MODE=1-IPOM
      IF (L.GT.2) GMAX=ZERO
      CALL UOLQP3(NF,N,NC,X,IX,IC,ICA,CF,CR,CZ,DMAX,GMAX,UMAX,JOLD,JNEW,
     & MODE)
      INEW=0
      IF (GMAX.EQ.ZERO) THEN
C
C     OPTIMUM ON A LINEAR MANIFOLD OBTAINED
C
      IF (IOLD.EQ.0) THEN
      IF (IPOM.EQ.0) THEN
C
C     OPTIMAL SOLUTION ACHIEVED
C
      ITERQ= 2
      GO TO 6
      ELSE
      IPOM=0
      DO 22 KC=1,NC
      IF (IC(KC).LT.-10) THEN
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      IF (IC(KC).LT.-10) IPOM=1
      ENDIF
   22 CONTINUE
      IF (IPOM.EQ.0) THEN
C
C     OPTIMAL SOLUTION ACHIEVED
C
      CALL UXVCOP(NF,GO,G)
      ITERL= 2
      GO TO 6
      ELSE
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      CALL UXVCOP(NF,GO,G)
      ITERQ=-1
      CALL UOERR1('ULQPB1',10-ITERL)
      GO TO 6
      ENDIF
      ENDIF
      ELSE
C
C     CONSTRAINT DELETION
C
      CALL ULRMB0(NF,N,NC,XN,ICA,CG,CR,CZ,GO,S,NORMF,IOLD,KREM,NREM,IER)
      CALL ULRMZ1(NF,N,CR,CZ,H,IDECF,IER)
      KC=ICA(NF-N+1)
      IF (KC.GT.0) THEN
      IC(KC)=-IC(KC)
      ELSE
      K=-KC
      IX(K)=-IX(K)
      IF (LCS.GT.0) THEN
      IF (IXA(K).GE.0) THEN
      IX(K)=IXA(K)
      IXA(K)=-1
      ENDIF
      ENDIF
      ENDIF
      JNEW=0
      JOLD=IOLD
      DMAX=ZERO
      GO TO 5
      ENDIF
      ELSE
C
C     DIRECTION DETERMINATION
C
      NCA=NF-N
      NCR=NCA*(NCA+1)/2
      CALL UXDCMM(NF,N,CZ,S,CR(NCR+1))
      CALL UXVNEG(NF,CR(NCR+1),S)
C
C     STEPSIZE SELECTION
C
      POM=CON
      CALL ULMAXL(NF,NC,CF,CFD,IC,CL,CU,CG,S,POM,KBC,KREM,INEW)
      IF (NORMF.GT.0) CALL UXVMUL(NF,XN,S,S,1)
      IF (LCS.GT.0) CALL ULMAXT(NF,X,IX,IXA,XN,XO,S,POM,XDEL,NORMF,
     & KREM,INEW)
      CALL ULMAXS(NF,X,IX,XL,XU,S,POM,KBF,KREM,INEW)
      IF (IPOM.EQ.0) THEN
      IF (POM.GT.ONE) THEN
C
C     OPTIMAL STEPSIZE
C
      INEW=0
      POM=ONE
      L=L+1
      ELSE
      L=0
      ENDIF
      ELSE
      IF (INEW.EQ.0) THEN
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      ITERQ=-3
      CALL UOERR1('ULQPB1',10-ITERQ)
      GO TO 6
      ENDIF
      ENDIF
C
C     STEP REALIZATION
C
      CALL ULDIRS(NF,X,IX,S,POM,KBF)
      CALL ULDIRL(NC,CF,CFD,IC,POM,KBC)
      IF (IPOM.EQ.0) THEN
      IF (NORMF.GT.0) CALL UXVNEG(NF,CR(NCR+1),S)
      IF (IDECF.EQ.1) THEN
      CALL UXDPGM(NF,H,S,0)
      CALL UXVDIR(NF,POM,S,GO,GO)
      ELSE
      CALL UXDSMM(NF,H,S,CR(NCR+1))
      CALL UXVDIR(NF,POM,CR(NCR+1),GO,GO)
      ENDIF
      ENDIF
C
C     CONSTRAINT ADDITION
C
      IF (INEW.EQ.0) THEN
      ELSE IF (INEW.GT.0) THEN
      KC=INEW
      INEW=0
      CALL ULNEWL(NC,KC,CF,IC,CL,CU,EPS9,INEW)
      CALL ULADB0(NF,N,NC,ICA,CG,CR,CZ,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NC,IC,KC,IER)
      ELSE IF (INEW+NF.GE.0) THEN
      I=-INEW
      INEW=0
      CALL ULNEWS(NF,X,IX,XN,XL,XU,EPS9,NORMF,I,INEW)
      CALL ULADB0(NF,N,NC,ICA,CG,CR,CZ,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NF,IX,I,IER)
      ELSE
      I=-(INEW+NF)
      INEW=0
      CALL ULNEWT(NF,X,IX,IXA,XN,XO,XDEL,EPS9,NORMF,I,INEW)
      CALL ULADB0(NF,N,NC,ICA,CG,CR,CZ,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      CALL UXVIND(NF,IX,I,IER)
C      CALL UXVINT(NF,IX,IXA,I,IER)
      ENDIF
      DMAX=POM
      IF(DMAX.GT.ZERO) NRED=NRED+1
      JNEW=INEW
      JOLD=0
      GO TO 3
      ENDIF
    6 CALL UOLQP2
      RETURN
      END
