* SUBROUTINE UNNEQ1                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SINGLE NONLINEAR EQUATION.
*
* PARAMETERS :
*  RI  AA  LEFT ENDPOINT OF THE INTERVAL.
*  RI  BB  RIGHT ENDPOINT OF THE INTERVAL.
*  RO  X  COMPUTED SOLUTION POINT.
*  RO  F  COMPUTED VALUE OF THE NONLINEAR FUNCTION.
*  RI  REQUIRED PRECISION FOR THE SOLUTION POINT.
*  RI  REQUIRED PRECISION FOR THE NONLINEAR FUNCTION.
*  IO  IC NUMBER OF ITERATIONS.
*  IO  IE ERROR SPECIFICATION.
*
* METHOD :
* D.LEE: THREE NEW RAPIDLY CONVERGENT ALGORITHMS FOR FINDING A ZERO
* OF A FUNCTION, SIAM J. SCI. STAT. COMPUT. 6 (1985) 193-208.
*
      SUBROUTINE UNNEQ1(AA,BB,X,F,FUN,EPSX,EPSF,IC,IE)
      REAL*8 AA,BB,X,F,FUN,EPSX,EPSF
      INTEGER IC,IE
      INTEGER ITER,ITMAX,K,L
      REAL*8 FA,FB,X1,X2,X3,F1,F2,F3,R,R1,RA,RB,D,D1,A,B,C,Z,W,FW,GW
      REAL*8 DEL,DDL,F21,F32
      REAL*8 ZERO,ONE,TWO,THREE,FOUR,HALF,CON
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0,TWO=2.0D 0,THREE=3.0D 0,
     & FOUR=4.0D 0,HALF=0.5D 0,CON=0.1D 0)
      EXTERNAL FUN
      IE=0
      ITMAX=IC-3
      IF (ITMAX.LE.0) ITMAX=100
      IC=2
      FA=FUN(AA)
      IF (ABS(FA).LE.EPSF) THEN
      X=AA
      F=FA
      RETURN
      ENDIF
      FB=FUN(BB)
      IF (ABS(FB).LE.EPSF) THEN
      X=BB
      F=FB
      RETURN
      ENDIF
      IF (FA*FB.GT.ZERO) THEN
      X=AA
      F=FA
      IE=-2
      RETURN
      ENDIF
      X1=AA
      F1=FA
      X2=HALF*(AA+BB)
      F2=FUN(X2)
      IC=IC+1
      IF (F1*F2.GT.ZERO) THEN
      X3=X1
      F3=F1
      X1=BB
      F1=FB
      ELSE
      X3=BB
      F3=FB
      ENDIF
      L=0
      D=ZERO
      R=ZERO
      DO 20 ITER=1,ITMAX
      D1=D
      R1=R
      D=ABS(X1-X2)
      IF (ABS(F1).LT.ABS(F2)) THEN
      X=X1
      F=F1
      ELSE
      X=X2
      F=F2
      ENDIF
      DEL=EPSX*(ABS(X)+ONE)
      IF (ABS(F).LE.EPSF.OR.D.LE.TWO*DEL) RETURN
      Z=X1+HALF*(X2-X1)
      DDL=MAX(CON*D,DEL)
      IF (THREE*D.LE.TWO*D1) THEN
      K=0
      ELSE
      K=1
      ENDIF
      IF (X2.EQ.X1) THEN
      F21=0.0D 0
      ELSE
      F21=(F2-F1)/(X2-X1)
      ENDIF
      IF (X3.EQ.X2) THEN
      F32=0.0D 0
      ELSE
      F32=(F3-F2)/(X3-X2)
      ENDIF
      A=(F32-F21)/(X3-X1)
      B=A*(X2+X1)-F21
      C=F2-(A*X2-B)*X2
      IF (ABS(A).LE.1.0D-10) THEN
      R=(F2*X1-F1*X2)/(F2-F1)
      ELSE
      R=B*B-FOUR*A*C
      IF (R.LT.ZERO) THEN
      R=(F2*X1-F1*X2)/(F2-F1)
      ELSE
      R=SQRT(R)
      RA=HALF*(B+R)/A
      RB=HALF*(B-R)/A
      IF (ABS(RA-Z).LE.ABS(RB-Z)) THEN
      R=RA
      ELSE
      R=RB
      ENDIF
      IF (R.LE.MIN(X1,X2).OR.R.GE.MAX(X1,X2)) THEN
      R=(F2*X1-F1*X2)/(F2-F1)
      ENDIF
      ENDIF
      ENDIF
      IF (L.GE.2) THEN
      W=R
      IF (ABS(W-X).LT.DEL) W=X+DEL*SIGN(ONE,Z-X)
      ELSE IF (K.EQ.1.OR.ABS(R-X).GE.ABS(Z-X)) THEN
      W=Z
      ELSE
      W=R+HALF*ABS(R-R1)*SIGN(ONE,R-X)
      IF (ABS(W-X).LT.DDL) W=X+DDL*SIGN(ONE,Z-X)
      IF (ABS(W-X).GE.ABS(Z-X)) W=Z
      ENDIF
      FW=FUN(W)
      IC=IC+1
      GW=(A*W-B)*W+C
      IF (ABS(FW-GW).LE.1.0D-1*ABS(F).OR.ABS(F).LE.1.0D-3*
     *MAX(ABS(F1),ABS(F2)).AND.L.GE.2) THEN
      L=L+1
      ELSE
      L=0
      ENDIF
      IF (FW*SIGN(ONE,F1).GE.ZERO) THEN
      IF (D.LE.ABS(X3-W)) THEN
      X3=X1
      F3=F1
      X1=X2
      F1=F2
      X2=W
      F2=FW
      ELSE
      X1=W
      F1=FW
      ENDIF
      ELSE
      X3=X2
      F3=F2
      X2=W
      F2=FW
      ENDIF
   20 CONTINUE
      IE=-1
      RETURN
      END
