* FUNCTION UNRAN5                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 SI : ORIGINAL VERSION
*
* PURPOSE :
*  RANDOM NUMBER GENERATOR. RETURNS A UNIFORM DEVIATE BETWEEN 0.0
*  AND 1.0. (FOR COMPUTER WITHOUT FAST 32-BIT INTEGER ARITHMETIC)
*
* PARAMETERS :
*  IU  IDUM  SET IDUM TO ANY NEGATIVE VALUE TO INITIALIZE OR
*            REINITIALIZE THE SEQUANCE.
*
* METHOD :
*  KNUTH,DONALD E,1981,"SEMINUMERICAL ALGORITHMS",
*  VOL.2 OF "THE ART OF COMPUTER PROGRAMMING".
*
      FUNCTION UNRAN5(IDUM)
      REAL*4 UNRAN5
      INTEGER IDUM
      REAL*4 MBIG,MSEED,MZ
      REAL*4 FAC
      PARAMETER (MBIG=1000000000.0,MSEED=161803398.0,MZ=0.0,FAC=1.E-9)
C
C MA(55) - THIS VALUE IS SPECIAL AND SHOULD NOT BE MODIFIED: SEE KNUTH
C
      REAL*4 MA(55),MJ,MK
      INTEGER IFF,I,II,INEXT,INEXTP,K
      SAVE MA,IFF,INEXT,INEXTP
      DATA IFF /0/
C
C INITIALIZATION
C
      IF(IDUM.LT.0.OR.IFF.EQ.0)THEN
        IFF=1
C
C INITIALIZE MA(55) USING THE SEED IDUM AND THE LARGE NUMBER MSEED
C
        MJ=MSEED-IABS(IDUM)
        MJ=MOD(MJ,MBIG)
        DO 1 I=1,54
          MA(I)=0.0
    1   CONTINUE
        MA(55)=MJ
        MK=1
C
C NOW INITIALIZE THE REST OF THE TABLE IN A SLIGHTLY RANDOM ORDER
C WITH NUMBERS THAT ARE NOT ESPECIALLY RANDOM
C
        DO 10 I=1,54
          II=1+MOD(21*I,55)
          MA(II)=MK
          MK=MJ-MK
          IF(MK.LT.MZ)MK=MK+MBIG
          MJ=MA(II)
   10   CONTINUE
C
C WE RANDOMIZE THEM BY "WARMING UP THE GENERATOR"
C
        DO 30 K=1,4
          DO 20 I=1,55
            MA(I)=MA(I)-MA(1+MOD(I+30,55))
            IF(MA(I).LT.MZ) MA(I)=MA(I)+MBIG
   20     CONTINUE
   30   CONTINUE
C
C PREPARE INDICES FOR OUR FIRST GENERATED NUMBER.
C CONSTANT 31 IS SPECIAL: SEE KNUTH
C
        INEXT=0
        INEXTP=31
        IDUM=1
      ENDIF
C
C START (EXCEPT OF INITIALIZATION).
C INCREMENT INEXT, WRAPPING AROUND 56 TO 1.
C
      INEXT=INEXT+1
      IF(INEXT.EQ.56) INEXT=1
C
C DTTO FOR INEXTP
C
      INEXTP=INEXTP+1
      IF(INEXTP.EQ.56) INEXTP=1
C
C NOW GENERATE A NEW RANDOM NUMBER SUBTRACTIVELY
C
      MJ=MA(INEXT)-MA(INEXTP)
C
C BE SURE IT IS IN RANGE
C
      IF(MJ.LT.MZ) MJ=MJ+MBIG
C
C STORE IT
C
      MA(INEXT)=MJ
C
C OUTPUT THE DERIVED UNIFORM DEVIATE
C
      UNRAN5=MJ*FAC
      END
