* SUBROUTINE UO2FN4                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* PRINT OF ITERATIONS IN UNCONSTRAINED MINIMIZATION.
*
* PARAMETERS:
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  AF(NA)  VALUES OF THE APPROXIMATED FUNCTIONS.
*  RI  CF(NC)  VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*  RO  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*  RI  CMAX  VALUE OF THE CONSTRAINT VIOLATION.
*
* COMMON DATA :
*  II  KD  DEGREE OF REQUIRED DERIVATIVES.
*  II  ISNA  LEVEL OF PRINT. IF ISNA>0 VALUES OF THE APPROXIMATED
*         FUNCTIONS ARE PRINTED.
*  II  ISNC  LEVEL OF PRINT. IF ISNC>0 VALUES OF CONSTRAINTS ARE
*         PRINTED.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF MATRIX CORRECTION.
*         KOUT=5-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=6-ADDITIONAL PRINT OF DEBUGGING.
*  II  LOUT  PRINT OF ITERATIONS. LOUT=0-STANDARD PAPER SAVING PRINT.
*         LOUT=1-ADDITIONAL PRINT OF VARIABLES. LOUT=2-ADDITIONAL
*         PRINT OF THE GRADIENT. LOUT=3-ADDITIONAL PRINT OF THE HESSIAN
*         MATRIX.
*  II  MOUT  LEVEL OF PRINT ON THE TERMINAL DEVICE.
*         MOUT=0 - NO PRINT.
*         MOUT=1 - PRINT OF FINAL RESULTS.
*         MOUT=2 - ADDITIONAL PRINT OF ITERATIONS.
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  RI  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*  RO  GMAX  NORM OF THE TRANSFORMED GRADIENT.
*
      SUBROUTINE UO2FN4(NF,NX,NA,NC,IX,X,XL,XU,XSL,XSU,XZL,XZU,CSL,CSU,
     & CZL,CZU,G,AF,IC,CF,CL,CU,F,DMAX,GMAX,CMAX)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NDEM
      COMMON /UMCOMD/ NDEM
      INTEGER NF,NX,NA,NC,IX(*),IC(*)
      REAL*8 X(NF),XL(NF),XU(NF),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CSL(NC),CSU(NC),CZL(NC),CZU(NC),G(NF),AF(*),CF(*),CL(*),CU(*),
     & F,DMAX,GMAX,CMAX
      INTEGER I
      LOGICAL LL
      NDEM=NF
      LL=KOUT.EQ.0.AND.LOUT.GT.0
      IF(ABS(MOUT).GT.1) THEN
      IF(KD.LE.0) THEN
      WRITE(ITR,15) NIC,NIT,NFV,F,CMAX
      IF(LL) WRITE(IWR,15) NIC,NIT,NFV,F,CMAX
      ELSE
      WRITE(ITR,16) NIC,NIT,NFV,NFG,F,CMAX,GMAX
      IF(LL) WRITE(IWR,16) NIC,NIT,NFV,NFG,F,CMAX,GMAX
      ENDIF
      ENDIF
      IF(JOUT.EQ.0) RETURN
      IF (ABS(KOUT).LE.1.AND.LOUT.EQ.0) THEN
      IF (KD.LE.0) THEN
      WRITE (IWR,11) NIC,NIT,NFV,F,CMAX,DMAX,TXFU,TDXX,TSXX,TPXX
   11 FORMAT (' ','NIC=',I3,1X,'NIT=',I5,1X,'NFV=',I5,1X,'F=',
     & D 9.3,1X,'C=',D 9.3,1X,'D=',D 9.3,1X,A8,1X,A8,1X,A8,1X,A8)
      ELSE
      WRITE (IWR,12) NIC,NIT,NFV,NFG,F,CMAX,GMAX,DMAX,TXFU,TDXX,TSXX,
     & TPXX
   12 FORMAT (' ','NIC=',I3,1X,'NIT=',I5,1X,'NFV=',I5,1X,'NFG=',I5,1X,
     & 'F=',D 9.3,1X,'C=',D 9.3,1X,'G=',D 9.3,1X,'D=',D 9.3,1X,A8,
     & 1X,A8,1X,A8,1X,A8)
      ENDIF
      ELSE
      IF (KD.LE.0) THEN
      WRITE (IWR,1) NIC,NIT,NFV,F,CMAX,TXFU,TDXX,TSXX,TPXX,DMAX
    1 FORMAT (' ','NIC=',I4,1X,'NIT=',I5,1X,'NFV=',I5,1X,'F=',D 9.3,
     & 1X,'C=',D 9.3,1X,'D=',D 9.3/1X,A8,1X,A8,2X,A8,2X,A8)
      ELSE
      WRITE (IWR,2) NIC,NIT,NFV,NFG,F,CMAX,GMAX,TXFU,TDXX,TSXX,TPXX,DMAX
    2 FORMAT (' ','NIC=',I4,1X,'NIT=',I5,1X,'NFV=',I5,1X,'NFG=',I5,2X,
     & 'F=',D 9.3,1X,'C=',D 9.3,1X,'G=',D 9.3/1X,A8,1X,A8,2X,A8,2X,
     & A8,3X,'D=',D 9.3)
      ENDIF
      ENDIF
      IF (ABS(LOUT).GE.1) THEN
      IF (LOUT.LT.0.AND.KBF.GT.0) WRITE (IWR,30) (ABS(IX(I)),
     & I=1,MIN(25,NF))
      WRITE ( IWR, 3)  (X(I), I=1,MIN(25,NF))
      IF (LOUT.LT.0.AND.KBF.GT.0) WRITE (IWR,31) (XL(I),I=1,MIN(25,NF))
      IF (LOUT.LT.0.AND.KBF.GT.0) WRITE (IWR,32) (XU(I),I=1,MIN(25,NF))
    3 FORMAT ( ' ', 'X   =', 5D 14.5:/( 6X, 5D 14.5))
   30 FORMAT ( ' ', 'IX  =', 5I14:/( 6X, 5I14))
   31 FORMAT ( ' ', 'XL  =', 5D 14.5:/( 6X, 5D 14.5))
   32 FORMAT ( ' ', 'XU  =', 5D 14.5:/( 6X, 5D 14.5))
      ENDIF
      IF (LOUT.LT.0.AND.ISNA.GT.0)
     *WRITE ( IWR, 4)  (AF(I), I=1,MIN(25,NA))
    4 FORMAT ( ' ', 'AF  =', 5D 14.5:/( 6X, 5D 14.5))
      IF (LOUT.LT.0.AND.ISNC.GT.0) THEN
      IF (KBC.GT.0) WRITE (IWR,50) (ABS(IC(I)),I=1,MIN(25,NC))
      WRITE ( IWR, 5)  (CF(I), I=1,MIN(25,NC))
      IF (KBC.GT.0) WRITE (IWR,51) (CL(I),I=1,MIN(25,NC))
      IF (KBC.GT.0) WRITE (IWR,52) (CU(I),I=1,MIN(25,NC))
    5 FORMAT ( ' ', 'CF  =', 5D 14.5:/( 6X, 5D 14.5))
   50 FORMAT ( ' ', 'IC  =', 5I14:/( 6X, 5I14))
   51 FORMAT ( ' ', 'CL  =', 5D 14.5:/( 6X, 5D 14.5))
   52 FORMAT ( ' ', 'CU  =', 5D 14.5:/( 6X, 5D 14.5))
      ENDIF
      IF (ABS(LOUT).GE.2) THEN
      IF (KD.GT.0) WRITE(IWR,6) (G(I),I=1,MIN(25,NF))
    6 FORMAT ( ' ', 'G   =', 5D 14.5:/( 6X, 5D 14.5))
      ENDIF
      IF (ABS(LOUT).GE.3) THEN
      IF (KBF.GE.1) WRITE(IWR,7) 'XSL =',(XSL(I),I=1,MIN(25,NX))
      IF (KBF.GE.2) WRITE(IWR,7) 'XSU =',(XSU(I),I=1,MIN(25,NX))
      IF (KBC.GE.1) WRITE(IWR,7) 'CSL =',(CSL(I),I=1,MIN(25,NC))
      IF (KBC.GE.2) WRITE(IWR,7) 'CSU =',(CSU(I),I=1,MIN(25,NC))
      IF (KBF.GE.1) WRITE(IWR,7) 'XZL =',(XZL(I),I=1,MIN(25,NX))
      IF (KBF.GE.2) WRITE(IWR,7) 'XZU =',(XZU(I),I=1,MIN(25,NX))
      IF (KBC.GE.1) WRITE(IWR,7) 'CZL =',(CZL(I),I=1,MIN(25,NC))
      IF (KBC.GE.2) WRITE(IWR,7) 'CZU =',(CZU(I),I=1,MIN(25,NC))
    7 FORMAT ( ' ', A5, 5D 14.5:/( 6X, 5D 14.5))
      ENDIF
      IF(LOUT.NE.0.AND.ABS(KOUT).LE.1) WRITE(IWR,8)
      IF(ABS(KOUT).GT.1 .OR.LOUT.LT.0) WRITE(IWR,9)
    8 FORMAT(' ')
    9 FORMAT(' ',43('-'))
   15 FORMAT(' ',4X,'NIC=',I3,1X,'NIT=',I4,1X,'NFV=',I5,1X,'F=',
     *            D9.3,1X,'C=',D9.3)
   16 FORMAT(' ',4X,'NIC=',I3,1X,'NIT=',I4,1X,'NFV=',I5,1X,'NFG=',
     *            I5,1X,'F=',D9.3,1X,'C=',D9.3,1X,'G=',D9.3)
      RETURN
      END
