* SUBROUTINE UP1AP7             ALL SYSTEMS                 06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION.
* COMPUTATION OF THE VALUES AND THE GRADIENTS OF THE CONSTRAINT
* FUNCTIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  KA  INDEX OF THE APPROXIMATED FUNCTION.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE SPARSE JACOBIAN MATRIX.
*  IU  KC  INDEX OF THE CONSTRAINT.
*  II  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  GA(NF)  GRADIENT OF THE APPROXIMATED FUNCTION.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  RI  GC(NF)  GRADIENT OF THE SELECTED FUNCTION.
*  II  ICG(NC+1)  POSITIONS OF THE FIRST ROWS ELEMENTS IN THE SPARSE
*         JACOBIAN STRUCTURE.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE SPARSE JACOBIAN
*         STRUCTURE.
*  RI  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RI  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RO  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  R  STEPSIZE  PARAMETER.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RO  FA  VALUE OF THE APPROXIMATED FUNCTION.
*  RO  FC  VALUE OF THE CONSTRAINT FUNCTION.
*  RO  F  VALUE OF THE PENALTY FUNCTION.
*  RO  CMAX  MAXIMUM CONSTRAINT VIOLATION.
*  RI  EPS7  SMOOTHING PARAMETER.
*  RI  REXP  EXPONENT IN A SUM OF POWERS.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  KD  DEGREE OF REQUIRED DERIVATIVES.
*  II  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  II  IEXT  TYPE OF EXTREMUM. IEXT=0-MINIMUM. IEXT=1-MAXIMUM.
*  IU  NAV  NUMBER OF APPROXIMATED FUNCTION VALUES COMPUTED.
*  IU  NAG  NUMBER OF APPROXIMATED FUNCTION GRADIENTS COMPUTED.
*  IU  NFV  NUMBER OF OBJECTIVE FUNCTION VALUES COMPUTED.
*  IU  NFG  NUMBER OF OBJECTIVE FUNCTION GRADIENTS COMPUTED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  RF  UXUDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE UP1AP7(NF,NX,NA,NAL,MA,KA,NC,MC,KC,X,IX,XL,XU,XZL,XZU,
     & GA,IAG,JAG,IC,CL,CU,CZL,CZU,GC,ICG,JCG,XO,GO,G,R,FF,FA,FC,F,
     & CMAX,EPS7,REXP)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NX,NA,NAL,MA,KA,NC,MC,KC,IX(NF),IAG(NA+1),JAG(MA),
     & IC(NC),ICG(NC+1),JCG(MC)
      REAL*8 X(NF),XL(NX),XU(NX),XZL(NX),XZU(NX),GA(NF),CL(NC),CU(NC),
     & CZL(NC),CZU(NC),GC(NF),XO(NX+NC),GO(NX+NC),G(NF),R,FF,FA,FC,F,
     & CMAX,EPS7,REXP
      LOGICAL L1
      INTEGER I,J,K,KK,L,JP
      REAL*8 TEMP,TEMP1,TEMP2,FTEMP,UXUDOT
      SAVE FTEMP,KK
      IF (TSS(NS).NE.'UFXX') CALL UYPRO1('UFXX',-1)
      GO TO (1,3,5,10,12) ISP(NS)
    1 CONTINUE
      IF (REXP.LE.ONE) REXP=TWO
      IF (KD.LE.LD) GO TO 15
      L1=.FALSE.
      IF (R.GT.ZERO) L1=.TRUE.
      IF (LD.LT.0) THEN
      CMAX=ZERO
      F=ZERO
      ENDIF
      IF (KD.GE.1.AND.LD.LT.1) CALL UXVSET(NF,ZERO,G)
      KA=1
    2 IF (KA.GT.NA) GO TO 20
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
      FTEMP=ABS(FA)**REXP
      IF (LD.GE.0) GO TO 4
      F=F+FA*FA
    4 IF (KD.LT.1) GO TO 19
      IF (LD.GE.1) GO TO 19
      CALL UYSET0(2,3)
      RETURN
    5 CONTINUE
      IF (FA.NE.ZERO) FTEMP=FTEMP/FA
      K=IAG(KA)
      L=IAG(KA+1)-K
      DO 18 J=1,L
      JP=ABS(JAG(K))
      G(JP)=G(JP)+FTEMP*GA(JP)
      K=K+1
   18 CONTINUE
   19 KA=KA+1
      GO TO 2
   20 IF (KD.GE.0.AND.LD.LT.0) F=F/REXP
      IF (NA.GT.NAL) THEN
      NFV=NAV/(NA-NAL)
      NFG=NAG/(NA-NAL)
      ELSE
      NFV=NFV+1
      NFG=NFG+1
      ENDIF
      IF (LD.LT.0) THEN
      FF=F
      F=ZERO
      ENDIF
      IF (KD.GE.1.AND.LD.LT.1) CALL UXVCOP(NF,G,GA)
      IF (KBF.LE.0) GO TO 8
      IF (LD.LT.0) THEN
      DO 6 I=1,NX
      KK=ABS(IX(I))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      TEMP1=XL(I)-X(I)
      IF (L1) THEN
      TEMP2=XZL(I)+R*XO(I)
      ELSE
      TEMP2=XZL(I)
      ENDIF
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      CMAX=MAX(CMAX,TEMP1)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      TEMP1=X(I)-XU(I)
      IF (L1) THEN
      TEMP2=XZU(I)+R*GO(I)
      ELSE
      TEMP2=XZU(I)
      ENDIF
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      CMAX=MAX(CMAX,TEMP1)
      ENDIF
    6 CONTINUE
      ENDIF
      IF (KD.GE.1.AND.LD.LT.1) THEN
      DO 7 I=1,NX
      KK=ABS(IX(I))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (L1) THEN
      TEMP2=XZL(I)+R*XO(I)
      ELSE
      TEMP2=XZL(I)
      ENDIF
      G(I)=G(I)-TEMP2
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (L1) THEN
      TEMP2=XZU(I)+R*GO(I)
      ELSE
      TEMP2=XZU(I)
      ENDIF
      G(I)=G(I)+TEMP2
      ENDIF
    7 CONTINUE
      ENDIF
    8 IF (KBC.LE.0) GO TO 15
      KC=1
    9 IF (KC.GT.NC) GO TO 15
      KK=ABS(IC(KC))
      IF (KK.EQ.0) GO TO 14
      IF (LD.GE.0) GO TO 11
      CALL UYSET0(3,4)
      RETURN
   10 CONTINUE
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      TEMP1=CL(KC)-FC
      IF (L1) THEN
      TEMP2=CZL(KC)+R*XO(NX+KC)
      ELSE
      TEMP2=CZL(KC)
      ENDIF
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      CMAX=MAX(CMAX,TEMP1)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      TEMP1=FC-CU(KC)
      IF (L1) THEN
      TEMP2=CZU(KC)+R*GO(NX+KC)
      ELSE
      TEMP2=CZU(KC)
      ENDIF
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      CMAX=MAX(CMAX,TEMP1)
      ENDIF
      IF (KK.EQ.5) THEN
      F=F+FC**2
      CMAX=MAX(CMAX,ABS(FC))
      ENDIF
   11 IF (KD.LT.1) GO TO 14
      IF (LD.GE.1) GO TO 14
      CALL UYSET0(4,5)
      RETURN
   12 CONTINUE
      IF (KK.EQ.3.OR.KK.EQ.4) THEN
      IF (L1) THEN
      TEMP= CZU(KC)-CZL(KC)+R*(GO(NX+KC)-XO(NX+KC))
      ELSE
      TEMP= CZU(KC)-CZL(KC)
      ENDIF
      ELSE IF (KK.EQ.1) THEN
      IF (L1) THEN
      TEMP=-CZL(KC)-R*XO(NX+KC)
      ELSE
      TEMP=-CZL(KC)
      ENDIF
      ELSE IF (KK.EQ.2) THEN
      IF (L1) THEN
      TEMP= CZU(KC)+R*GO(NX+KC)
      ELSE
      TEMP= CZU(KC)
      ENDIF
      ELSE IF (KK.EQ.5) THEN
      IF (L1) THEN
      TEMP= CZL(KC)+R*XO(NX+KC)
      ELSE
      TEMP= CZL(KC)
      ENDIF
      ENDIF
      K=ICG(KC)
      L=ICG(KC+1)-K
      DO 13 J=1,L
      JP=ABS(JCG(K))
      G(JP)=G(JP)+TEMP*GC(JP)
      K=K+1
   13 CONTINUE
   14 KC=KC+1
      GO TO 9
   15 CONTINUE
      IF (KD.GE.1.AND.LD.LT.0) THEN
      F=HALF*(F+UXUDOT(NF,G,G,IX,KBF))
      ENDIF
      CALL UYEPI1(-1)
      RETURN
      END
