* SUBROUTINE UP2LQ7             ALL SYSTEMS                 00/12/01
C PORTABILITY : ALL SYSTEMS
C 00/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE VALUE AND THE GRADIENT OF THE OBJECTIVE FUNCTION.
* COMPUTATION OF THE VALUES AND THE GRADIENTS OF THE CONSTRAINT
* FUNCTIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE SPARSE JACOBIAN MATRIX.
*  IU  KC  INDEX OF THE CONSTRAINT.
*  II  M  SIZE OF THE SPARSE MATRIX WORK SPACE.
*  II  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  GF(NF)  GRADIENT OF THE MODEL FUNCTION.
*  RI  HF(M)  HESSIAN ELEMENTS OF THE QUADRATIC MODEL FUNCTION.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  RI  GC(NF)  GRADIENT OF THE SELECTED FUNCTION.
*  II  ICG(NC+1)  POSITIONS OF THE FIRST ROWS ELEMENTS IN THE SPARSE
*         JACOBIAN STRUCTURE.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE SPARSE JACOBIAN
*         STRUCTURE.
*  RI  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RI  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RO  G(NF)  GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RO  H(M)  SPARSE HESSIAN MATRIX OF THE LAGRANGIAN FUNCTION.
*  II  IH(NF+1)  POINTERS OF THE DIAGONAL ELEMENTS OF HESSIAN MATRIX.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF HESSIAN MATRIX.
*  RI  EPS7  INITIAL VALUE OF THE SLACK VARIABLES.
*  RI  FF  VALUE OF THE MODEL FUNCTION.
*  RO  FC  VALUE OF THE PENALTY TERM.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RO  CMAX  MAXIMUM CONSTRAINT VIOLATION.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  KD  DEGREE OF REQUIRED DERIVATIVES.
*  II  LD  DEGREE OF PREVIOUSLY COMPUTED DERIVATIVES.
*  II  IEXT  TYPE OF EXTREMUM. IEXT=0-MINIMUM. IEXT=1-MAXIMUM.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*
      SUBROUTINE UP2LQ7(NF,NX,NC,MC,KC,M,X,IX,XL,XU,XZL,XZU,GF,HF,CF,
     & IC,CL,CU,CZL,CZU,CG,GC,ICG,JCG,XO,GO,G,H,IH,JH,EPS7,R,FF,FC,F,
     & CMAX)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER NF,NX,NC,MC,KC,M,IX(NF),IC(NC),ICG(NC+1),JCG(MC),IH(NF+1),
     & JH(M)
      REAL*8 X(NF),XL(NX),XU(NX),XZL(NX),XZU(NX),GF(NF),HF(M),CF(NC+1),
     & CL(NC),CU(NC),CZL(NC),CZU(NC),CG(MC),GC(NF),XO(NX+NC),GO(NX+NC),
     & G(NF),H(M),EPS7,R,FF,FC,F,CMAX
      LOGICAL L1
      INTEGER I,J,K,KK,L,JP
      REAL*8 TEMP,TEMP1,TEMP2,UXUDOT
      IF (KD.LE.LD) GO TO 20
      L1=.FALSE.
      IF (R.GT.ZERO) L1=.TRUE.
      IF (LD.GE.0) GO TO 3
      CALL UXSSMM(NF,IH(NF+1)-1,HF,IH,JH,X,G)
      CALL UXVSUM(NF,GF,G,G)
      F=FF+HALF*(UXUDOT(NF,X,GF,IX,KBF)+UXUDOT(NF,X,G,IX,KBF))
      IF (IEXT.GT.0) F=-F
      CF(NC+1)=F
      NFV=NFV+1
      NFG=NFG+1
      CMAX=ZERO
    3 IF (KD.LT.1) GO TO 7
      IF (LD.GE.1) GO TO 5
      IF (LD.GE.0) THEN
      CALL UXSSMM(NF,IH(NF+1)-1,HF,IH,JH,X,G)
      CALL UXVSUM(NF,GF,G,G)
      NFG=NFG+1
      ENDIF
      IF (IEXT.GT.0) CALL UXVNEG(NF,G,G)
    5 IF (KD.LT.2) GO TO 7
      IF (LD.GE.2) GO TO 7
      IDECF=0
      IF (IEXT.GT.0) THEN
      CALL UXVNEG(M,HF,H)
      ELSE
      CALL UXVCOP(M,HF,H)
      ENDIF
      IF (LD.LT.0) THEN
      FF=F
      F=ZERO
      ENDIF
    7 IF (KBF.LE.0) GO TO 10
      IF (LD.LT.0) THEN
      DO 8 I=1,NX
      KK=ABS(IX(I))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      TEMP1=XL(I)-X(I)
      IF (L1) THEN
      TEMP2=XZL(I)+R*XO(I)
      ELSE
      TEMP2=XZL(I)
      ENDIF
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      CMAX=MAX(CMAX,TEMP)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      TEMP1=X(I)-XU(I)
      IF (L1) THEN
      TEMP2=XZU(I)+R*GO(I)
      ELSE
      TEMP2=XZU(I)
      ENDIF
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      CMAX=MAX(CMAX,TEMP)
      ENDIF
    8 CONTINUE
      ENDIF
      IF (KD.GE.1.AND.LD.LT.1) THEN
      DO 9 I=1,NX
      KK=ABS(IX(I))
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (L1) THEN
      TEMP2=XZL(I)+R*XO(I)
      ELSE
      TEMP2=XZL(I)
      ENDIF
      G(I)=G(I)-TEMP2
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      IF (L1) THEN
      TEMP2=XZU(I)+R*GO(I)
      ELSE
      TEMP2=XZU(I)
      ENDIF
      G(I)=G(I)+TEMP2
      ENDIF
    9 CONTINUE
      ENDIF
   10 IF (KBC.LE.0) GO TO 20
      KC=1
   11 IF (KC.GT.NC) GO TO 20
      KK=ABS(IC(KC))
      IF (KK.EQ.0) GO TO 19
      IF (LD.GE.0) GO TO 13
      CALL UALNF3(NF,NC,MC,KC,X,X,CG,ICG,JCG,FC,0,0)
      CF(KC)=FC
      IF (KK.EQ.1.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      TEMP1=CL(KC)-FC
      IF (L1) THEN
      TEMP2=CZL(KC)+R*XO(NX+KC)
      ELSE
      TEMP2=CZL(KC)
      ENDIF
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      CMAX=MAX(CMAX,TEMP)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.3.OR.KK.EQ.4) THEN
      TEMP1=FC-CU(KC)
      IF (L1) THEN
      TEMP2=CZU(KC)+R*GO(NX+KC)
      ELSE
      TEMP2=CZU(KC)
      ENDIF
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      F=F+TEMP**2
      CMAX=MAX(CMAX,TEMP)
      ENDIF
      IF (KK.EQ.5) THEN
      FC=FC-CL(KC)
      F=F+FC**2
      CMAX=MAX(CMAX,ABS(FC))
      CF(KC)=FC
      ENDIF
   13 IF (KD.LT.1) GO TO 19
      IF (LD.GE.1) GO TO 19
      CALL UALNG3(NF,NC,MC,KC,X,CG,ICG,JCG,GC,0,0)
      IF (KK.EQ.3.OR.KK.EQ.4) THEN
      IF (L1) THEN
      TEMP= CZU(KC)-CZL(KC)+R*(GO(NX+KC)-XO(NX+KC))
      ELSE
      TEMP= CZU(KC)-CZL(KC)
      ENDIF
      ELSE IF (KK.EQ.1) THEN
      IF (L1) THEN
      TEMP=-CZL(KC)-R*XO(NX+KC)
      ELSE
      TEMP=-CZL(KC)
      ENDIF
      ELSE IF (KK.EQ.2) THEN
      IF (L1) THEN
      TEMP= CZU(KC)+R*GO(NX+KC)
      ELSE
      TEMP= CZU(KC)
      ENDIF
      ELSE IF (KK.EQ.5) THEN
      IF (L1) THEN
      TEMP= CZL(KC)+R*XO(NX+KC)
      ELSE
      TEMP= CZL(KC)
      ENDIF
      ENDIF
      K=ICG(KC)
      L=ICG(KC+1)-K
      DO 15 J=1,L
      JP=ABS(JCG(K))
      G(JP)=G(JP)+TEMP*GC(JP)
      K=K+1
   15 CONTINUE
   19 KC=KC+1
      GO TO 11
   20 CONTINUE
      IF (KD.GE.1.AND.LD.LT.0) THEN
      F=HALF*(F+UXUDOT(NF,G,G,IX,KBF))
      ENDIF
      LD=KD
      RETURN
      END
