* SUBROUTINE US0SM3                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  SIMPLEX TRANSFORMATION
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RU  X(N)  VECTOR OF VARIABLES.
*  RA  XO(N)  AUXILIARY VECTOR.
*  RA  XP(N)  AUXILIARY VECTOR.
*  RU  S(N)  VECTOR OF SIMPLICIAL VALUES.
*  RU  H(N*(N+1))  MATRIX OF SIMPLICIAL POINTS.
*  RA  N(N*N)  AUXILIARY MATRIX.
*  IA  IB(N)  AUXILIARY VECTOR.
*  RI  ALF1  COEFFICIENT FOR STEPSIZE EXPANSION.
*  RI  BET1  LOWER BOUND FOR STEPSIZE REDUCTION.
*  RI  GAM1  LOWER BOUND FOR STEPSIZE EXPANSION.
*  RU  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RU  FP  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RU  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RU  FF  PRINTED VALUE OF THE OBJECTIVE FUNCTION.
*  RU  XDEL  CURRENT STEPSIZE.
*  RA  DMAX  AUXILIARY VARIABLE.
*  RA  EL  AUXILIARY VARIABLE.
*  RA  EU  AUXILIARY VARIABLE.
*  IA  MODE  MODE OF LOCAL SEARCH. MODE=0-UNSUCCESSFUL STEP.
*         MODE>0-FORWARD STEP. MODE<0-BACKWARD STEP.
*  IA  IVAR  INDEX VARIABLE.
*  IA  JVAR  INDEX VARIABLE.
*  IU  INF  DECOMPOSITION INDICATOR. INF=0 IF THE MATRIX H IS
*         NONSINGULAR.
*  II  IRES1  RESTART SPECIFICATION. RESTART IS PERFORMED AFTER
*         IRES1*N+IRES2 ITERATIONS.
*  II  IRES2  RESTART SPECIFICATION. RESTART IS PERFORMED AFTER
*         IRES1*N+IRES2 ITERATIONS.
*
* COMMON DATA :
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  II  IREST  RESTART SPECIFICATION. IF IREST>0 THEN RESTART IS
*         PERFORMED.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UXDGGF  LU DECOMPOSITION OF A DENSE SQUARE MATRIX.
*  S   UXDCMC  CENTER OF COLUMNS OF A DENSE RECTANGULAR MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDDR  VECTOR AUGMENTED BY THE SCALED DIFFERENCE OF
*         TWO VECTORS.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVEXT  EXTREMAL ELEMENTS OF A VECTOR.
*  S   UXVMED  MEDIAN OF TWO VECTORS.
*  RF  UXVNOR  EUCLIDEAN NORM OF A VECTOR.
*  RF  UXVNM2  EUCLIDEAN NORM OF A VECTOR DIFFERENCE.
*  S   UOS0A1  PRINT OF ENTRY TO LOCAL SEARCH.
*  S   UOS0A2  PRINT OF EXIT FROM LOCAL SEARCH.
*
* METHOD :
*  NELDER, MEAD : SIMPLEX METHOD
*  AND STATISTICAL PROBLEMS, J. OF THE ACM 8 (1961), PP 212-229
*
      SUBROUTINE US0SM3(N,X,XO,XP,S,H,B,IB,ALF1,BET1,GAM1,FO,FP,F,FF,
     & XDEL,DMAX,EL,EU,MODE,IVAR,JVAR,INF,IRES1,IRES2)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,IB(N),MODE,IVAR,JVAR,INF,IRES1,IRES2
      REAL*8 X(N),XO(N),XP(N),S(N+1),H(N*(N+1)),B(N*N)
      REAL*8 ALF1,BET1,GAM1,FO,FP,F,FF,XDEL,DMAX,EL,EU
      REAL*8 UXVNM2,UXVNOR,TEMP
      INTEGER I,KL,KU,LL,LU
      SAVE KL,KU,LL,LU
      REAL*8 CON
      PARAMETER (CON=0.1D 0)
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',0)
      GOTO (1,2,3,6,8) ISP(NS)
    1 CONTINUE
      CALL UOS0A1
      MODE=0
      IF (NIT.LE.LIT+1) THEN
      CALL UXVEXT(N+1,S,EL,EU,KL,KU)
      LL=(KL-1)*N+1
      LU=(KU-1)*N+1
      ENDIF
C
C     CENTROID DETERMINATION
C
      CALL UXDCMC(N,N+1,H,XO,KU)
C
C     REFLECTION
C
      CALL UXVDDR(N,XO,-ALF1,H(LU),XO,X)
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    2 IF (F.GE.EL) GO TO 4
C
C     EXPANSION
C
      FP=F
      CALL UXVCOP(N,X,XP)
      CALL UXVDDR(N,XO,-GAM1,H(LU),XO,X)
      CALL UYSET2
      CALL UYSET0(1,3)
      RETURN
    3 IF (F.LT.EL) THEN
      MODE=1
      CALL UXVCOP(N,X,H(LU))
      S(KU)=F
      ELSE
      CALL UXVCOP(N,XP,H(LU))
      S(KU)=FP
      ENDIF
      GO TO 10
    4 DO 5 I=1,N+1
      IF (I.EQ.KU) GO TO 5
      IF (F.LE.S(I)) THEN
      CALL UXVCOP(N,X,H(LU))
      S(KU)=F
      GO TO 10
      ENDIF
    5 CONTINUE
C
C     CONTRACTION
C
      IF (F.LE.EU) THEN
      CALL UXVDDR(N,XO,-BET1,H(LU),XO,X)
      ELSE
      CALL UXVDDR(N,XO, BET1,H(LU),XO,X)
      ENDIF
      CALL UYSET2
      CALL UYSET0(1,4)
      RETURN
    6 IF (F.LE.EU) THEN
      CALL UXVCOP(N,X,H(LU))
      S(KU)=F
      GO TO 10
      ENDIF
C
C     RESTART
C
      IF (N.GT.0.AND.NIT-KIT.GE.IRES1*N+IRES2) IREST=1
      CALL UXVCOP(N,H(LL),X)
      F=S(KL)
      XDEL=ONE
      IVAR=N*N+1
      JVAR=1
      DO 16 I=1,N
      CALL UXVDIF(N,H(JVAR),H(IVAR),B(JVAR))
      XDEL=XDEL*UXVNOR(N,B(JVAR))
      JVAR=JVAR+N
   16 CONTINUE
      XDEL=XDEL**(ONE/DBLE(N))
      CALL UXDGGF(N,B,IB,INF,ZERO)
      TEMP=ONE
      JVAR=1
      DO 17 I=1,N
      IVAR=IB(I)
      TEMP=TEMP*ABS(B(JVAR+IVAR))
   17 CONTINUE
      TEMP=TEMP**(ONE/DBLE(N))
      IF (IREST.GT.0.OR.TEMP.LE.CON*XDEL) THEN
      CALL UYRES1('NORMAL  ')
      GO TO 11
      ENDIF
C
C     REDUCTION
C
      IVAR=1
      JVAR=1
    7 IF (IVAR.GT.N) GO TO 10
      IF (IVAR.EQ.KL) GO TO 9
      CALL UXVMED(N,H(JVAR),H(LL),X)
      CALL UYSET2
      CALL UYSET0(1,5)
      RETURN
    8 CONTINUE
      CALL UXVCOP(N,X,H(JVAR))
      S(IVAR)=F
    9 IVAR=IVAR+1
      JVAR=JVAR+N
      GO TO 7
   10 CALL UXVEXT(N+1,S,EL,EU,KL,KU)
      LL=(KL-1)*N+1
      LU=(KU-1)*N+1
      IF (MODE.EQ.0) THEN
      CALL UXVCOP(N,H(LL),X)
      F=EL
      ENDIF
      DMAX=UXVNM2(N,X,H(LU))
      FO=EU
   11 CALL UOS0A2(FO,FF)
      CALL UYEPI1(0)
      RETURN
      END
