* SUBROUTINE US1L08                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SPECIAL LINE SEARCH FOR NONSMOOTH NONCONVEX VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  MB  DECLARED NUMBER OF LINEAR APPROXIMATED FUNCTIONS
*  II  NB  CURRENT NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  RU  X(N)  VECTOR OF VARIABLES.
*  RI  S(N)  DIRECTION VECTOR.
*  RI  AF(4*MB)  VECTOR OF BUNDLE FUNCTIONS VALUES.
*  RI  AG(N*MB)  MATRIX WHOSE COLUMNS ARE BUNDLE SUBGRADIENTS.
*  RI  AY(N*MB)  MATRIX WHOSE COLUMNS ARE VARIABLE VECTORS.
*  RO  T  VALUE OF THE STEPSIZE PARAMETER.
*  RO  TB  BUNDLE PARAMETER FOR MATRIX SCALING.
*  RO  FO  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RU  PO  PREVIOUS DIRECTIONAL DERIVATIVE.
*  RU  P  DIRECTIONAL DERIVATIVE.
*  RI  TMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  TMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  GNORM  NORM OF THE GRADIENT VECTOR.
*  RI  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RI  WK  STOPPING PARAMETER.
*  RI  EPS1  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE FUNCTION VALUE).
*  RI  EPS2  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         DIRECTIONAL DERIVATIVE).
*  RI  ETA5  DISTANCE MEASURE PARAMETER.
*  RI  ETA9  MAXIMUM FOR REAL NUMBERS.
*  II  JE  EXTRAPOLATION INDICATOR.
*  RI  MOS3   LOCALITY MEASURE PARAMETER.
*
* COMMON DATA :
*  IO  ITERS  NULL STEP INDICATOR. ITERS=0-NULL STEP. ITERS=1-DESCENT
*         STEP.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UNSTP4  STEPSIZE DETERMINATION FOR DESCENT STEPS.
*  S   UNSTP5  STEPSIZE DETERMINATION FOR NULL STEPS.
*  S   UNINT1  EXTRAPOLATION OR INTERPOLATION FOR LINE SEARCH
*              WITH DIRECTIONAL DERIVATIVES.
*
* METHOD :
* SPECIAL METHOD OF STEP LENGTH DETERMINATION.
*
      SUBROUTINE US1L08(N,MB,NB,X,S,AF,AG,AY,T,TB,FO,F,PO,P,
     & TMIN,TMAX,GNORM,SNORM,WK,EPS1,EPS2,ETA5,ETA9,JE,MES1,MOS3)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,MB,NB,JE,MES1,MOS3
      REAL*8 S(N),X(N),AF(MB*4),AG(N*MB),AY(N*MB),T,TB,FO,
     & F,PO,P,TMIN,TMAX,GNORM,SNORM,WK,EPS1,EPS2,ETA5,ETA9
      REAL*8 BET,FL,FU,PL,PU,TL,TU
      INTEGER IER,NPOM
      LOGICAL LPOM
      SAVE FL,FU,PL,PU,TL,TU
      SAVE NPOM
      SAVE LPOM
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',1)
      GOTO (1,8) ISP(NS)
   1  CONTINUE
      IF (MES1.GE.0) THEN
      LPOM=ITERS.NE.0
      ELSE
      LPOM=ITERS.EQ.0
      ENDIF
      NPOM=0
      CALL UOS1L1(ZERO,FO,PO,GNORM,SNORM,TMAX)
      IF (JE.GT.0) T=DBLE(2-JE/99)*T
      IF (JE.LE.0) T=MIN(ONE,TMAX)
      IF (JE.GT.0) TSXX='EXTRAPOL'
      IF (PO.EQ.ZERO) TSXX='P - ZERO'
      IF (PO.EQ.ZERO.OR.JE.GT.0) GO TO 10
      IF (ITERS.EQ.1) THEN
        CALL UNSTP4(N,MB,NB,X,AF,AG,AY,S,F,PO,T,TB,ETA5,ETA9,MOS3)
        TSXX='INIT-DES'
      ELSE
        CALL UNSTP5(N,MB,NB,X,AF,AG,AY,S,F,PO,T,TB,ETA5,ETA9,MOS3)
        TSXX='INIT-ZER'
      ENDIF
   10 T=MIN(MAX(T,TMIN),TMAX)
      TL=ZERO
      TU=T
      FL=FO
      PL=PO
C
C     FUNCTION AND GRADIENT EVALUATION AT A NEW POINT
C
    7 CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    8 CONTINUE
      CALL UOS1L3(T,F,P,SNORM,JE)
C
C     NULL/DESCENT STEP TEST (ITERS=0/1)
C
      ITERS=1
      TSXX='DESCENT '
      IF (F.LE.FO-T*(EPS1+EPS1)*WK) THEN
        TL=T
        FL=F
        PL=P
      ELSE
        TU=T
        FU=F
        PU=P
      ENDIF
      BET=MAX(ABS(FO-F+P*T),ETA5*(SNORM*T)**MOS3)
      IF (F.LE.FO-T*EPS1*WK.AND.(T.GE.TMIN.OR.BET.GT.EPS1*WK))
     & GO TO 5
      IF (P-BET.GE.-EPS2*WK.OR.TU-TL.LT.TMIN*1.0D-1) THEN
        IF (LPOM.OR.NPOM.GE.ABS(MES1)) GO TO 4
        NPOM=NPOM+1
      ENDIF
      IF (TL.EQ.ZERO.AND.PL.LT.ZERO) THEN
        CALL UNINT1(TL,TU,FL,FU,PL,PU,T,2,2,IER,TSXX)
      ELSE
        T=HALF*(TU+TL)
      ENDIF
      GO TO 7
    4 ITERS=0
      TSXX='NULL  ST'
    5 IF (T.GE.TMAX) NMAX=NMAX+1
      CALL UOS0L2(T,F,SNORM)
      CALL UYEPI1(1)
      RETURN
      END
