* SUBROUTINE US1Q01                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  STANDARD LINE SEARCH WITH DIRECTIONAL DERIVATIVES SPECIALIZED FOR
*  SPARSE QP USING UNFACTORIZED MATRIX H.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  N NUMBER OF VARIABLES.
*  II  MCOLS  DECLARED NUMBER OF VARIABLES.
*  RI  X(MCOLS)  NEW VECTOR OF VARIABLES.
*  RI  WR01(MCOLS)  OLD VECTOR OF VARIABLES.
*  RI  G(MCOLS)  NEW GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GF(MCOLS) GRADIENT OF THE MODEL FUNCTION.
*  RI  H(M)  HESSIAN ELEMENTS OF THE QUADRATIC MODEL FUNCTION.
*  II  IH(MCOLS+1)  POINTERS OF THE DIAGONAL ELEMENTS OF HESSIAN MATRIX.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF HESSIAN MATRIX.
*  RI  S(MCOLS)  DIRECTION VECTOR.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  RO  P  VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-PERFECT
*         LINE SEARCH. ITERS=2 GOLDSTEIN STEPSIZE. ITERS=3-WEAK WOLFE
*         STEPSIZE. ITERS=4-STRONG WOLFE STEPSIZE.
*         ITERS=5-ARMIJO STEPSIZE. ITERS=6-FIRST STEPSIZE.
*         ITERS=7-MAXIMUM STEPSIZE. ITERS=8-UNBOUNDED FUNCTION.
*         ITERS=-1-MRED REACHED. ITERS=-2-POSITIVE DIRECTIONAL
*         DERIVATIVE. ITERS=-3-ERROR IN INTERPOLATION.
*
* SUBPROGRAM USED :
*  S   UNINT1  EXTRAPOLATION OR INTERPOLATION WITH DIRECTIONAL
*         DERIVATIVES.
*  S   UF1FQ2  QUADRARIC FUNCTION EVALUATION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*
* METHOD :
* SAFEGUARDED EXTRAPOLATION AND INTERPOLATION WITH STANDARD TERMINATION
* CRITERIA.
*
      SUBROUTINE US1Q01(NF,N,M,MCOLS,MMAX,X,WR01,G,GF,H,IH,JH,S,R,FF,F,
     & FO,P,PO,RMIN,RMAX,TU,ITERS)
      INTEGER NF,N,M,MCOLS,MMAX,IH(MCOLS+1),JH(MMAX),ITERS
      REAL*8 X(MCOLS),WR01(MCOLS),G(MCOLS),GF(MCOLS),H(MMAX),S(MCOLS),
     & R,FF,F,FO,P,PO,RMIN,RMAX,TU
      INTEGER MTYP,MERR,MES,MES1,MES2,INITS,IEST,MODE,NRED,MRED,MAXST
      REAL*8 RU,FU,PU,RL,FL,PL,UXVDOT
      REAL*8 ACCUR,FFF,EPS1,EPS2,FMIN
      CHARACTER*8 TDUMMY
      PARAMETER(ACCUR=1.0D-9)
      LOGICAL L1,L2,L3,L4
C
C     SET DEFAULT VALUES
C
      MES=2
      MES1=1
      MES2=1
      EPS1=0.0001D 0
      EPS2=0.9D 0
      INITS=1
      IEST=0
      NRED=0
      MRED=20
      FMIN=-1.0D 60
      IF (PO.GE.0.0D 0) THEN
                      R = 0.0D 0
                      ITERS=-2
      GO TO 4
      ENDIF
C
C     SET INITIAL CRASH HELP
C
      FFF=F
      TU=0
C
C     INITIAL STEPSIZE SELECTION
C
C      RP = 0.0D 0
C      FP = FO
C      PP = PO
      IF (INITS.LE.0) GO TO 2
      IF (INITS.EQ.1.OR.IEST.EQ.0) THEN
      R = 1.0D 0
      ENDIF
      R = MAX(R,RMIN)
      R=MIN(R,RMAX)
      MODE = 0
      RU = 0.0D 0
      FU = FO
      PU = PO
C
C     NEW STEPSIZE SELECTION (EXTRAPOLATION OR INTERPOLATION)
C
    2 CALL UNINT1(RL,RU,FL,FU,PL,PU,R,MODE,MTYP,MERR,TDUMMY)
      IF (MERR.GT.0) THEN
      ITERS = -3
      GO TO 4
      ELSE IF (MODE.EQ.1) THEN
      NRED = NRED  - 1
      R = MIN(R,RMAX)
      ELSE IF (MODE.EQ.2) THEN
      NRED = NRED + 1
      ENDIF
C
C     COMPUTATION OF THE NEW FUNCTION VALUE AND THE NEW DIRECTIONAL
C     DERIVATIVE
C
      CALL UXVDIR(N,R,S,WR01,X)
C
C     COMPUTE FUNCTIONAL VALUE / GRADIENT
C
      CALL UYSET2
      CALL UF1FQ2(NF,IH(NF+1)-1,X,GF,G,H,IH,JH,FF,F)
C
C     CRASH HELP - UPDATE OF TU - CRASH STEPSIZE
C
      IF(F.LT.FFF) THEN
        FFF=F
        TU=R
      END IF
      P = UXVDOT(N,G,S)
      IF (F.LE.FMIN) THEN
      ITERS = 6
      GO TO 4
      ELSE
      L1 = R.LE.RMIN
      L2 = R.GE.RMAX
      L3 = F - FO.LE.EPS1*R*PO
      L4 = P.GE.EPS2*PO
      MAXST = 0
      IF (L2) MAXST = 1
      ENDIF
C
C     TEST ON TERMINATION
C
      IF (L1.AND..NOT.L3) THEN
      ITERS = 0
      GO TO 4
      ELSE IF (L2.AND.L3.AND..NOT.L4)  THEN
      ITERS = 5
      GO TO 4
      ELSE IF (L3.AND.MES1.EQ.3) THEN
      ITERS = 4
      GO TO 4
      ELSE IF (L3.AND.L4.AND.(MES2.EQ.1.OR.MODE.NE.0)) THEN
      ITERS = 3
      GO TO 4
      ELSE IF (ABS(NRED).GE.MRED) THEN
      ITERS = -1
      GO TO 4
      ELSE
C      RP = R
C      FP = F
C      PP = P
      MODE = MAX(MODE,1)
      MTYP = MES
      ENDIF
      IF (MODE.EQ.1) THEN
C
C     INTERVAL CHANGE AFTER EXTRAPOLATION
C
      RL = RU
      FL = FU
      PL = PU
      RU = R
      FU = F
      PU = P
      IF (.NOT.L3) THEN
      NRED = 0
      MODE = 2
      ELSE IF ( MES1 .EQ. 1) THEN
      MTYP = 2
      ENDIF
      ELSE
C
C     INTERVAL CHANGE AFTER INTERPOLATION
C
      IF (.NOT.L3) THEN
      RU = R
      FU = F
      PU = P
      ELSE
      RL = R
      FL = F
      PL = P
      ENDIF
      ENDIF
      GO TO 2
    4 CONTINUE
      IF((RMAX-R).LE.ACCUR) ITERS=7
      RETURN
      END
