* SUBROUTINE UUDVI2                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  NONSMOOTH VARIABLE METRIC UPDATE OF THE INVERSE HESSIAN MATRIX.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  H(N*(N+1)/2)  POSITIVE DEFINITE APPROXIMATION OF THE INVERSE
*         HESSIAN MATRIX.
*  RI  S(N)  DIRECTION VECTOR.
*  RI  GV(N)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  U(N)  DIFFERENCE OF CURRENT AND PREVIOUS GRADIENTS.
*  RO  V(N)  AUXILIARY VECTOR.
*  RI  Z  DOT PRODUCT OF VECTORS S, U.
*  RI  RO  CORRECTION PARAMETER.
*  II  JC  CORRECTION INDICATOR.
*  II  JU  UPDATING INDICATOR.
*  II  NNK  CONSECUTIVE NULL STEPS COUNTER.
*  II  JOB  TYPE OF MINIMIZATION. JOB=0 - CONVEX MINIMIZATION.
*         JOB=1 - NONCONVEX MINIMIZATION.
*
* COMMON DATA :
*
* SUBPROGRAMS USED :
*  S   UXDSMM  MULTIPLICATION OF A DENSE SYMMETRIC MATRIX BY A VECTOR.
*  S   UXDSMU  UPDATE OF A DENSE SYMMETRIC MATRIX.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXVDEL  SQUARED NORM OF A SHIFTED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVLIN  LINEAR COMBINATION OF TWO VECTORS.
*
      SUBROUTINE UUDVI2(N,H,S,GV,U,V,Z,RO,JC,JU,NNK,JOB)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,JC,JU,NNK,JOB
      REAL*8 H(N*(N+1)/2),S(N),GV(N),U(N),V(N),Z,RO
      REAL*8 GAM,P,P1,P2,P3,Q,W,UXVDOT,UXVDEL
      INTEGER I,J,L
      LOGICAL LB,LR
      IF(JOB.GT.0)JU=0
      IF(Z.LE.ZERO) THEN
        TUXX='B NEG.  '
        RETURN
      ENDIF
      CALL UXDSMM(N,H,U,V)
      W=UXVDOT(N,U,V)
      GAM=ONE
      IF(NIT.EQ.1)THEN
        Q=ONE
        IF(W.NE.ZERO)Q=Z/W
        IF((Q-2.5D-1)*(Q-THREE).GT.ZERO)GAM=MIN(THREE,MAX(2.0D-2,Q))
      ENDIF
      P1=UXVDEL(N,-ONE,S,V)
      P2=UXVDOT(N,GV,V)-UXVDOT(N,GV,S)
      P3=UXVDOT(N,GV,GV)
      P=W-Z
      LB=NNK.EQ.0
      LR=NNK.NE.0.AND.P2.LT.ZERO
      IF(JC.EQ.1.AND.(P1.LT.RO*P*N.OR.P2*P2.LT.RO*P*P3))LR=.FALSE.
      IF(LB)THEN
        CALL UXVLIN(N,ONE/Z,V,-HALF*(ONE/GAM+W/Z)/Z,S,U)
        L=1
        IF(JOB.GT.0)JU=1
        DO 9 I=1,N
        DO 9 J=1,I
        H(L)=(H(L)-U(I)*S(J)-S(I)*U(J))*GAM
    9   L=L+1
        TUXX='BFGS    '
        NUPD=NUPD+1
      ELSEIF(LR)THEN
        CALL UXVDIF(N,V,S,U)
        JU=1
        CALL UXDSMU(N,H,-ONE/P,U)
        TUXX='RANK ONE'
        NUPD=NUPD+1
      ELSE
        TUXX='NO UPDT.'
      ENDIF
      RETURN
      END
