* SUBROUTINE UULVI2                ALL SYSTEMS                04/12/01
C PORTABILITY : ALL SYSTEMS
C 04/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* RANK-TWO LIMITED-STORAGE VARIABLE-METRIC METHODS IN THE PRODUCT FORM.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES (NUMBER OF ROWS OF XM).
*  II  M  NUMBER OF COLUMNS OF XM.
*  RI  XM(N*M)  RECTANGULAR MATRIX IN THE PRODUCT FORM SHIFTED BROYDEN
*         METHOD (STORED COLUMNWISE): H-SIGMA*I=XM*TRANS(XM)
*  RO  XR(M)  VECTOR TRANS(XM)*H**(-1)*XO.
*  RO  GR(M)  MATRIX TRANS(XM)*GO.
*  RI  G(N)  NEW GRADIENT VECTOR.
*  RA  S(N)  AUXILIARY VECTORS (H**(-1)*XO AND U).
*  RA  SO(N)  AUXILIARY VECTORS ((H-SIGMA*I)*H**(-1)*XO AND V).
*  RU  XO(N)  VECTORS OF VARIABLES DIFFERENCE XO AND VECTOR XO-TILDE.
*  RU  GO(N)  GRADIENT DIFFERENCE GO AND VECTOR XM*TRANS(XM)*GO.
*  RI  R  STEPSIZE PARAMETER.
*  RI  PO  OLD DIRECTIONAL DERIVATIVE (MULTIPLIED BY R)
*  RU  SIG  SCALING PARAMETER (ZETA AND SIGMA).
*  II  MET3  CHOICE OF SIGMA (1-CONSTANT, 2-QUADRATIC EQUATION).
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDRMM  MULTIPLICATION OF A ROWWISE STORED DENSE RECTANGULAR
*         MATRIX BY A VECTOR.
*  S   UXDCMM  MULTIPLICATION OF A COLUMNWISE STORED DENSE RECTANGULAR
*         MATRIX BY A VECTOR.
*  S   UXDCMU  UPDATE OF A COLUMNWISE STORED DENSE RECTANGULAR MATRIX.
*         WITH CONTROLLING OF POSITIVE DEFINITENESS.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVDIR  VECTOR AUGMENTED BY A SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
* METHOD :
* RANK-ONE LIMITED-STORAGE VARIABLE-METRIC METHOD IN THE PRODUCT FORM.
*
      SUBROUTINE UULVI2(N,M,XM,XMO,XR,GR,G,S,SO,XO,GO,GV,R,PO,SIG,MET2,
     & MET3,MET5,NNK)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER N,M,MET2,MET3,MET5,NNK
      REAL*8 XM(N*M),XMO(N*M),XR(M),GR(M),G(N),S(N),SO(N),XO(N),GO(N),
     & GV(N),R,PO,SIG
      REAL*8 UXVDOT,POM1,POM2
      REAL*8 DEN,POM,A,B,C,AA,BB,CC,AH,PAR,ZET
      CALL UOU1D1(N,XO,GO)
      ZET=SIG
C
C     COMPUTATION OF B
C
      B=UXVDOT(N,XO,GO)
      IF (B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      GO TO 22
      ENDIF
      IF (NNK.GT.0) THEN
C
C     COMPUTATION OF POM1=TRANS(GV)*H*GV
C
      CALL UXVCOP(N*M,XM,XMO)
      CALL UXDRMM(N,M,XM,GV,XR)
      POM1=SIG*UXVDOT(N,GV,GV)+UXVDOT(M,XR,XR)
      ENDIF
C
C     COMPUTATION OF GR=TRANS(XM)*GO, XR=TRANS(XM)*H**(-1)*XO
C     AND S=H**(-1)*XO, SO=(H-SIGMA*I)*H**(-1)*XO. COMPUTATION
C     OF AA=GR*GR, BB=GR*XR, CC=XR*XR. COMPUTATION OF A AND C.
C
      CALL UXDRMM(N,M,XM,GO,GR)
      CALL UXVDIF(N,GO,G,S)
      CALL UXVSCL(N,R,S,S)
      CALL UXDRMM(N,M,XM,S,XR)
      CALL UXVDIR(N,-SIG,S,XO,SO)
      AH=UXVDOT(N,GO,GO)
      AA=UXVDOT(M,GR,GR)
      BB=UXVDOT(M,GR,XR)
      CC=UXVDOT(M,XR,XR)
      A=AA+AH*SIG
      C=-R*PO
C
C     DETERMINATION OF THE PARAMETER SIG (SHIFT)
C
      POM=B/AH
      IF (A.GT.ZERO) THEN
      DEN=UXVDOT(N,XO,XO)
      IF (MET3.LE.4) THEN
      SIG=SQRT(MAX(ZERO,ONE-AA/A))/(ONE+SQRT(MAX(ZERO,ONE-B*B/
     & (DEN*AH))))*POM
      ELSE
      SIG=SQRT(MAX(ZERO,SIG*AH/A))/(ONE+SQRT(MAX(ZERO,ONE-B*B/
     & (DEN*AH))))*POM
      ENDIF
      SIG=MAX(SIG,2.0D-1*POM)
      SIG=MIN(SIG,8.0D-1*POM)
      ELSE
      SIG=2.5D-1*POM
      ENDIF
C
C     COMPUTATION OF SHIFTED XO AND SHIFTED B
C
      DEN=B
      B=B-AH*SIG
      CALL UXVDIR(N,-SIG,GO,XO,XO)
C
C     COMPUTATION OF THE PARAMETER RHO (CORRECTION)
C
      IF (MET2.LE.1) THEN
      PAR=ONE
      ELSE IF (MET2.EQ.2) THEN
      PAR=SIG*AH/B
      ELSE IF (MET2.EQ.3) THEN
      PAR=SQRT(ONE-AA/A)
      ELSE IF (MET2.EQ.4) THEN
      PAR=SQRT(SQRT(ONE-AA/A)*(SIG*AH/B))
      ELSE IF (MET2.EQ.5) THEN
      PAR=SQRT(SQRT(HALF*ZET/(SIG+ZET))*SIG*AH/DEN)
      ELSE
      PAR=ZET/(ZET+SIG)
      ENDIF
C
C     COMPUTATION OF THE PARAMETER THETA (BFGS)
C
      POM=SIGN(SQRT(PAR*B/CC),BB)
C
C     COMPUTATION OF Q AND P
C
      IF (MET5.EQ.1) THEN
C
C     RANK ONE UPDATE OF XM
C
      CALL UXVDIR(M,POM,XR,GR,XR)
      CALL UXVLIN(N,PAR,XO,POM,SO,S)
      CALL UXDCMU(N,M,XM,-ONE/(PAR*B+POM*BB),S,XR)
      ELSE
C
C     RANK TWO UPDATE OF XM
C
      CALL UXVDIR(N,PAR/POM-BB/B,XO,SO,S)
      CALL UXDCMV(N,M,XM,-ONE/B,XO,GR,-ONE/CC,S,XR)
      ENDIF
      IF (NNK.GT.0) THEN
C
C     COMPUTATION OF POM2=TRANS(GV)*NEW(H)*GV
C
      CALL UXDRMM(N,M,XM,GV,XR)
      POM2=SIG*UXVDOT(N,GV,GV)+UXVDOT(M,XR,XR)
      IF (POM1.LT.POM2) THEN
      CALL UXVCOP(N*M,XMO,XM)
      NUPD=NUPD-1
      ENDIF
      ENDIF
      NUPD=NUPD+1
   22 CONTINUE
      ITERH=0
C      CALL UOU1D3(N,H,S,RHO,GAM,PAR,A,B,C,POM,ETA9)
      RETURN
      END
