* SUBROUTINE UULVP4                ALL SYSTEMS                06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED-STORAGE VARIABLE-METRIC UPDATE IN THE PRODUCT FORM.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES (NUMBER OF ROWS OF THE MATRIX XM).
*  II  M  NUMBER OF COLUMNS OF THE MATRIX XM.
*  RU  XM(N*M)  RECTANGULAR MATRIX FOR THE PRODUCT FORM VM
*         METHOD (STORED COLUMNWISE): BAR(H)=XM*TRANS(XM).
*  RI  XO(N)  VECTOR OF VARIABLES DIFFERENCE.
*  RI  GO(N)  GRADIENT DIFFERENCE.
*  RU  XS(N)  ON INPUT VECTOR H**(-1)*XO AND THEN AUXILIARY VECTOR.
*  RA  GS(N)  AUXILIARY VECTOR.
*  RA  HR(M*3) AUXILIARY VECTORS (TRANS(XM)*GO, TRANS(XM)*B*XO).
*  RI  RHO  CORRECTION PARAMETER.
*  RO  AP  OUTPUT PARAMETER BAR(A)=TRANS(GO)*BAR(H)*GO.
*  RO  B  OUTPUT PARAMETER B=TRANS(GO)*XO.
*
* SUBPROGRAMS USED :
*  S   UXDRMM  MULTIPLICATION OF A ROWWISE STORED DENSE RECTANGULAR
*         MATRIX BY A VECTOR.
*  S   UXDCMM  MULTIPLICATION OF A COLUMNWISE STORED DENSE RECTANGULAR
*         MATRIX BY A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*
* METHOD :
* J.VLCEK, L.LUKSAN: NEW CLASS OF LIMITED-MEMORY VARIATIONALLY-DERIVED
* VARIABLE METRIC METHODS. ICS REPORT V973-06, ICS AVCR, PRAGUE 2006.
*
      SUBROUTINE UULVP4(N,M,XM,XO,GO,XS,GS,HR,RHO,AP,B)
      INTEGER NS,ISB,ISP(20),ISD(20),KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI(7)
      COMMON /UMOPTI/ NS,ISB,ISP,ISD,KD,KDF,KDA,KDC,KDE,KDY,LD,LDA,LDC,
     & LDE,LDY,KBF,KBA,KBC,ISNF,ISNA,ISNC,NORMF,NORMA,NORMC,IDECF,IDECA,
     & IDECC,IDECE,NDECF,NDECA,NDECC,NDECE,MTESX,NTESX,MTESF,NTESF,LCS,
     & IEST,IEXT,NEXT,IDIR,ITRAN,IREST,ITERM,ITERS,ITERH,ITERD,ITERL,
     & ITERQ,MAXST,KIC,LIC,MIC,NIC,KIT,LIT,MIT,NIT,KFV,LFV,MFV,NFV,NAV,
     & NCV,NEV,NFG,NAG,NCG,NEG,NFH,NAH,NCH,NEH,NREM,NADD,NRES,NREF,NRED,
     & NCGR,NLNS,MSTP,NSTP,IRD,IWR,ITR,IOUT,JOUT,KOUT,KOUT1,KOUT2,KOUT3,
     & LOUT,MOUT,NOUT,NOUT1,NOUT2,NOUT3,LFG,MFG,NSCL,NUPD,NMAX,NTIME,
     & MCGR,NCYC,IMOPTI
      REAL*8 TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR(12)
      COMMON /UMOPTR/ TOLX,TOLF,TOLB,TOLG,TOLC,TOLA,TOLR,RAN,RMOPTR
      REAL*8 ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS(2)
      COMMON /UMCONS/ ZERO,HALF,ONE,TWO,THREE,FOUR,FIVE,TEN,RMCONS
      CHARACTER*4 TSS(20)
      CHARACTER*8 TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,TUXX,
     & TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT(3)
      CHARACTER*80 TER
      COMMON /UMOPTT/ TSS,TER,TGXN,TGXU,TXAN,TXAU,TXFN,TXFU,TDXX,TSXX,
     & TUXX,TRXX,TPFX,TGFX,TLPX,TQPX,TPXX,TEDX,TENX,TMOPTT
      CHARACTER*6 NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN
      CHARACTER*14 NAME
      COMMON /UMOPTN/ NGXN,NGXU,NXAN,NXAU,NXFN,NXFU,NGXX,NHXX,NDXX,NSXX,
     & NUXX,NFXX,NFGX,NFHX,NAXX,NAGX,NAHX,NCXX,NCGX,NCHX,NPXX,NGFX,NLPX,
     & NQPX,NSGX,NEDX,NEXX,NPFX,NBFX,TMOPTN,NAME
      INTEGER M,N
      REAL*8 XM(N*M),XO(N),GO(N),XS(N),GS(N),HR(M*3),RHO,AP,B
      INTEGER I,J,L
      REAL*8 GAM,ETA,BP,CP,TMP,TEMP,UXVDOT
      GAM=1.0D 0
      ETA=0.5D 0
      CALL UXDRMM(N,M,XM,GO,HR)
      CALL UXDRMM(N,M,XM,XS,HR(M+1))
      AP=UXVDOT(M,HR,HR)
      BP=UXVDOT(M,HR,HR(M+1))
      CP=UXVDOT(M,HR(M+1),HR(M+1))
      IF (AP*(AP*CP-BP*BP).LE.0.0D 0) RETURN
      DO 1 I=1,M
      HR(M+M+I)=(AP*HR(M+I)-BP*HR(I))/SQRT(AP*(AP*CP-BP*BP))
  1   CONTINUE
      CALL UXDCMM(N,M,XM,HR(M+M+1),XS)
      CALL UXDCMM(N,M,XM,HR,GS)
      DO 3 I=1,N
      L=I
      TMP=SQRT(RHO/(GAM*B))*XO(I)-XS(I)
      TEMP=SQRT(ETA)/B*XO(I)+(1.0D 0-SQRT(ETA))/AP*GS(I)
      DO 2 J=1,M
      XM(L)=SQRT(GAM)*(XM(L)-TEMP*HR(J)+TMP*HR(J+M+M))
      L=L+N
   2  CONTINUE
   3  CONTINUE
      TUXX='PROJ-1  '
      NUPD=NUPD+1
      ITERH=0
      RETURN
      END
