* SUBROUTINE UXDCPF               ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* QR DECOMPOSITION OF ROWWISE STORED DENSE RECTANGULAR MATRIX Q USING
* HOUSEHOLDER TRANSFORMATIONS WITHOUT PIVOTING.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX Q.
*  II  M  NUMBER OF ROWS OF THE MATRIX Q.
*  RU  Q(M*N)  RECTANGULAR MATRIX STORED ROWWISE IN THE
*         ONE-DIMENSIONAL ARRAY.
*  RO  R(N*(N+1)/2)  UPPER TRIANGULAR MATRIX STORED IN THE PACKED FORM.
*
* SUBPROGRAMS USED :
*  S   UXDRMN  EUCLIDEAN NORM OF A PART OF THE ROWWISE STORED
*         RECTANGULAR MATRIX COLUMN.
*
* METHOD :
* P.A.BUSSINGER, G.H.GOLUB : LINEAR LEAST SQUARES SOLUTION BY
* HOUSEHOLDER TRANSFORMATION. NUMER. MATH. 7 (1965) 269-276.
*
      SUBROUTINE UXDCPF(N,M,Q,R)
      INTEGER N,M
      REAL*8 Q(M*N),R(N*(N+1)/2)
      REAL*8 ALF,POM,UXVDOT,UXVNOR
      INTEGER I,J,K,L,IP,KP,NM
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      NM=MIN(N,M)
C
C     QR DECOMPOSITION
C
      L=0
      KP=0
      DO 6 K=1,NM
      POM=UXVNOR(M-K+1,Q(KP+K))
      IF (Q(KP+K).NE.ZERO) POM=SIGN(POM,Q(KP+K))
      R(L+K)=-POM
      DO 1 J=1,K-1
      R(L+J)=Q(KP+J)
      Q(KP+J)=ZERO
    1 CONTINUE
      IF (POM.NE.ZERO) THEN
C
C     HOUSEHOLDER TRANSFORMATION
C
      CALL UXVSCL(M-K+1,ONE/POM,Q(KP+K),Q(KP+K))
      Q(KP+K)=Q(KP+K)+ONE
      IP=KP
      DO 5 I=K+1,N
      IP=IP+M
      ALF=UXVDOT(M-K+1,Q(KP+K),Q(IP+K))/Q(KP+K)
      CALL UXVDIR(M-K+1,-ALF,Q(KP+K),Q(IP+K),Q(IP+K))
    5 CONTINUE
      ENDIF
      L=L+K
      KP=KP+M
    6 CONTINUE
C
C     EXPLICIT FORMULATION OF THE ORTHOGONAL MATRIX
C
      KP=NM*M
      DO 11 K=NM,1,-1
      IP=KP
      KP=KP-M
      IF (Q(KP+K).NE.ZERO) THEN
      DO 9 I=K+1,NM
      ALF=UXVDOT(M-K+1,Q(KP+K),Q(IP+K))/Q(KP+K)
      CALL UXVDIR(M-K+1,-ALF,Q(KP+K),Q(IP+K),Q(IP+K))
      IP=IP+M
    9 CONTINUE
      CALL UXVNEG(M-K+1,Q(KP+K),Q(KP+K))
      ENDIF
      Q(KP+K)=Q(KP+K)+ONE
   11 CONTINUE
      RETURN
      END
