* SUBROUTINE UXDGGF                ALL SYSTEMS                94/12/01
C PORTABILITY : ALL SYSTEMS
C 94/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* FACTORIZATION A*P=L*U OF A DENSE GENERAL SQUARE MATRIX A WHERE P IS
* A PERMUTATION MATRIX, L IS A LOWER TRIANGULAR MATRIX AND U IS AN UPPER
* TRIANGULAR MATRIX,
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*N) ON INPUT A GIVEN DENSE GENERAL SQUARE MATRIX A STORED BY
*         ROWS. ON OUTPUT THE COMPUTED FACTORIZATION A*P=L*U.
*  IO  IA(N) A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  IO  INF AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS SUFFICIENTLY NONSINGULAR. IF INF>0 THEN INF
*         IS A NUMBER OF THE ELIMINATION STEP IN WHICH A SINGULARITY
*         WAS DETECTED.
*  RI  TOL ON INPUT A DESIRED TOLERANCE FOR POSITIVE DEFINITENESS. ON
*         OUTPUT THE MOST NEGATIVE DIAGONAL ELEMENT USED IN THE
*         FACTORIZATION PROCESS (IF INF>0).
*
* METHOD :
* GAUSSIAN ELIMINATION WITH PARTIAL PIVOTING.
*
      SUBROUTINE UXDGGF(N,A,IA,INF,TOL)
      INTEGER N,IA(N),INF
      REAL*8 A(N*N),TOL
      REAL*8 APIV,APOM
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      INTEGER I,J,JP,K,KP,KP1
      INF=0
      DO 1 I=1,N
      IA(I)=(I-1)*N
    1 CONTINUE
      DO 10 K=1,N
      KP1=K+1
      KP=IA(K)
      I=K
      APIV=ABS(A(KP+K))
      DO 3 J=KP1,N
      JP=IA(J)
      APOM=ABS(A(JP+K))
      IF (APIV.LT.APOM) THEN
      APIV=APOM
      I=J
      ENDIF
    3 CONTINUE
      IF (APIV.EQ.ZERO) THEN
      INF=K
      A(KP+K)=TOL
      ELSE
      IA(K)=IA(I)
      IA(I)=KP
      KP=IA(K)
      IF (APIV.LE.TOL) THEN
      INF=K
      A(KP+K)=SIGN(TOL,A(KP+K))
      ENDIF
      ENDIF
      APIV=ONE/A(KP+K)
      DO 9 J=KP1,N
      JP=IA(J)
      APOM=APIV*A(JP+K)
      A(JP+K)=APOM
      DO 8 I=KP1,N
      A(JP+I)=A(JP+I)-A(KP+I)*APOM
    8 CONTINUE
    9 CONTINUE
   10 CONTINUE
      RETURN
      END
