* SUBROUTINE UXDRFP             ALL SYSTEMS                   05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE FEASIBLE POINT
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NC  NUMBER OF LINEAR CONSTRAINTS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CB(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RO  X(NF)  VECTOR OF VARIABLES.
*  RA  CF(NF)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RA  CFD(NF)  VECTOR CONTAINING INCREMENTS OF THE CONSTRAINT
*         FUNCTIONS.
*  IA  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  IA  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RA  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RA  CZ(NF)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RA  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RA  S(NF)  DIRECTION VECTOR.
*
* SUBPROGRAMS USED :
*  S   UXDRFM  MAXIMUM STEPSIZE USING LINEAR CONSTRAINTS.
*  S   UXDRFN  IDENTIFICATION OF ACTIVE LINEAR CONSTRAINTS.
*  S   UXDRFD  NEW VALUES OF CONSTRAINT FUNCTIONS.
*  S   UXDRFG  DETERMINATION OF THE FIRST PHASE GRADIENT VECTOR.
*  S   UXDRFL  GRADIENT OF THE LAGRANGIAN FUNCTION IS DETERMINED.
*  S   UXDRFS  NEGATIVE PROJECTED GRADIENT IS DETERMINED.
*  S   UXDRFT  THE OPTIMUM LAGRANGE MULTIPLIER IS DETERMINED.
*  S   UXDRFV  AN AUXILIARY VECTOR IS DETERMINED.
*  S   UXDRFA  CONSTRAINT ADDITION.
*  S   UXDRFR  CONSTRAINT DELETION.
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXDSMI  DETERMINATION OF THE INITIAL UNIT DENSE SYMMETRIC
*         MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVINA  ABSOLUTE VALUES OF ELEMENTS OF AN INTEGER VECTOR.
*  S   UXVINC  UPDATE OF AN INTEGER VECTOR.
*  S   UXVIND  CHANGE OF THE INTEGER VECTOR FOR CONSTRAINT ADDITION.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE UXDRFP(NF,NC,CG,CB,X,CF,CFD,IC,ICA,CR,CZ,G,S,JOB,INF)
      INTEGER NF,NC,IC(NC),ICA(NF),JOB,INF
      REAL*8 X(NF),CF(NC),CFD(NC),CB(NC),CG(NF*NC),CR(NF*(NF+1)/2),
     & CZ(NF),G(NF),S(NF)
      REAL*8 POM,UMAX,GMAX,DMAX
      INTEGER N,IPOM,IOLD,JOLD,INEW,JNEW,IER,KREM,KC
      REAL*8 CON,CON7,CON9
      PARAMETER(CON=1.0D120,CON7=1.0D-10,CON9=1.0D-10)
      INF=0
C
C     INITIATION
C
      IPOM=0
      JOLD=0
      JNEW=0
      KREM=0
      DMAX=0.0D 0
      CALL UXVSET(NF,0.0D 0,X)
      CALL UXVSET(NC,0.0D 0,CF)
      N=NF
      CALL UXVINS(NC,JOB,IC)
      DO 2 KC=1,NC
      INEW=0
      CALL UXDRFL(NC,KC,CF,IC,CB,CON9,INEW)
      CALL UXDRFA(NF,N,NC,ICA,CG,CR,S,CON7,GMAX,UMAX,INEW,IER)
C      CALL UXVIND(NC,IC,KC,IER)
      IF (IER.LE.1) CALL UXVIND(NC,IC,KC,0)
      IF (IC(KC).LT.-10) IPOM=1
    2 CONTINUE
    3 IF (IPOM.EQ.1) THEN
C
C     CHECK OF FEASIBILITY AND UPDATE OF THE FIRST PHASE OBJECTIVE
C     FUNCTION
C
      CALL UXDRFG(NF,NC,IC,CG,G,INEW)
      IF (INEW.EQ.0) IPOM=0
      ENDIF
      IF (IPOM.EQ.0) THEN
C
C     FEASIBILITY ACHIEVED
C
      INF=1
      GO TO 6
      ENDIF
C
C     LAGRANGE MULTIPLIERS DETERMINATION
C
    5 IF (NF.GT.N) THEN
      CALL UXDRFV(NF,N,NC,ICA,CG,G,CZ)
      CALL UXDPRB(NF-N,CR,CZ,0)
      CALL UXDRFT(NF,N,NC,IC,ICA,CZ,CON7,UMAX,IOLD)
      ELSE
      IOLD=0
      UMAX=0.0D 0
      ENDIF
C
C     PROJECTED GRADIENT DETERMINATION
C
      IF (N.GT.0) THEN
      CALL UXVNEG(NF,G,S)
      CALL UXDRFS(NF,N,NC,ICA,CG,CZ,S,CON7,GMAX)
      ELSE
      GMAX=0.0D 0
      ENDIF
      INEW=0
      IF (GMAX.EQ.0.0D 0) THEN
C
C     OPTIMUM ON A LINEAR MANIFOLD OBTAINED
C
      IF (IOLD.EQ.0) THEN
      IPOM=0
      DO 22 KC=1,NC
      IF (IC(KC).LT.-10) THEN
      INEW=0
      CALL UXDRFL(NC,KC,CF,IC,CB,CON9,INEW)
      IF (IC(KC).LT.-10) IPOM=1
      ENDIF
   22 CONTINUE
      IF (IPOM.EQ.0) THEN
C
C     OPTIMAL SOLUTION ACHIEVED
C
      INF= 2
      GO TO 6
      ELSE
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      INF=-1
      GO TO 6
      ENDIF
      ELSE
C
C     CONSTRAINT DELETION
C
      CALL UXDRFR(NF,N,NC,IC,ICA,CR,S,IOLD,KREM,IER)
      DMAX=0.0D 0
      JOLD=IOLD
      JNEW=0
      GO TO 5
      ENDIF
      ELSE
C
C     STEPSIZE SELECTION
C
      POM=CON
      CALL UXDRFM(NF,NC,CF,CFD,IC,CB,CG,S,POM,KREM,INEW)
      IF (INEW.EQ.0) THEN
C
C     FEASIBLE SOLUTION DOES NOT EXIST
C
      INF=-3
      GO TO 6
      ELSE
C
C     STEP REALIZATION
C
      CALL UXVDIR(NF,POM,S,X,X)
      CALL UXDRFD(NC,CF,CFD,IC,POM)
C
C     CONSTRAINT ADDITION
C
      IF (INEW.GT.0) THEN
      KC=INEW
      INEW=0
      CALL UXDRFL(NC,KC,CF,IC,CB,CON9,INEW)
      CALL UXDRFA(NF,N,NC,ICA,CG,CR,S,CON7,GMAX,UMAX,INEW,IER)
C      CALL UXVIND(NC,IC,KC,IER)
      IF (IER.LE.1) CALL UXVIND(NC,IC,KC,0)
      ENDIF
      DMAX=POM
      JNEW=INEW
      JOLD=0
      GO TO 3
      ENDIF
      ENDIF
    6 CONTINUE
      RETURN
      END
