* SUBROUTINE UXDSBU                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* CORRECTION OF A DENSE SYMMETRIC MATRIX A IN THE FACTORED FOR
* A=P*L*D*TRANS(L)*TRANS(P) OBTAINED BY THE SUBROUTINE UXDSBF. THE
* CORRECTION IS DEFINED AS A:=A+ALF*X*TRANS(X) WHERE ALF IS A GIVEN
* SCALING FACTOR AND X IS A GIVEN VECTOR.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2)  FACTORIZATION A=P*L*D*TRANS(L)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDSBF.
*  II  IA(N)  PERMUTATION FIELD OBTAINED BY THE SUBROUTINE UXDSBF.
*  RI  ALF  SCALING FACTOR IN THE CORRECTION TERM.
*  RI  X(N)  VECTOR IN THE CORRECTION TERM.
*  RA  Y(N) AUXILIARY VECTOR.
*
* METHOD :
* D.C.SORENSEN : UPDATING THE SYMMETRIC INDEFINITE FACTORIZATION WITH
* APPLICATION IN A MODIFIED NEWTON'S METHOD. ARGONNE NATIONAL LAB.
* REPORT NO. ANL-77-49, ARGONNE, ILLINOIS, 1977.
*
      SUBROUTINE UXDSBU(N,A,IA,ALF,X,Y)
      INTEGER N,IA(N)
      REAL*8 A(N*(N+1)/2),ALF,X(N),Y(N)
      REAL*8 SIG,D11,D21,D31,D22,D32,D33,B1,B2,B3,U1,U0,T,T1,T2,DET,
     * L1,L2,S
      INTEGER IP,IP1,IP2,I,I1,I2,JP,J,II,I1I,I2I,I1I1,I1I2,I2I2,IJ,I1J,
     * I2J,IND
      REAL*8 TWO, CON
      PARAMETER(TWO=2.0D 0, CON=0.64D 0)
      IND(I,J) = MAX(I,J)*(MAX(I,J)-1)/2 + MIN(I,J)
      SIG = ALF
      CALL UXVCOP(N,X,Y)
      IP = 1
    1 CONTINUE
      I = IA(IP)
      IF (I.LT.0) GOTO 9
C
C     THE DIAGONAL BLOCK BEGINNING AT ENTRY IP IS 1*1
C
      II = IND(I,I)
      T = Y(I)
      B1 = SIG*T
      D11 = A(II)+B1*T
      DO 2 JP = IP+1,N
      J=ABS(IA(JP))
      IJ = IND(I,J)
      Y(J) = Y(J) - A(IJ)*T
    2 CONTINUE
    3 CONTINUE
C
C     ENTER 1*1
C     THE 1*1 BLOCK WILL BE PROCESSED
C
      IF (IP.GE.N) THEN
      A(II) = D11
      RETURN
      ENDIF
      IP1 = IP + 1
      I1 = IA(IP1)
      U1 = ABS(D11)
      U0 = ABS(B1)
      IF (U1 .GE. CON*U0 .OR. U1*ABS(SIG) .GT. CON*U0*U0) THEN
      SIG = SIG - B1*B1/D11
      B1 = B1/D11
      A(II) = D11
      DO 4 JP = IP1,N
      J=ABS(IA(JP))
      IJ = IND(I,J)
      A(IJ) = A(IJ) + B1*Y(J)
    4 CONTINUE
      IP=IP1
      GO TO 1
      ELSE IF (I1 .GT. 0) THEN
C
C     A 2*2 BLOCK IS FORMED BY COMBINING THE NEXT 1*1 BLOCK WITH
C     BLOCK IP
C
      I1I = IND(I1,I)
      I1I1 = IND(I1,I1)
      B2 = Y(I1)
      T = B2
      D21 = B2*B1
      B2 = SIG*B2
      D22 = A(I1I1) + B2*T
      L1 = A(I1I)
      D22 = D22 + L1*D21
      B2 = B2 + L1*B1
      D21 = D21 + L1*D11
      D22 = D22 + L1*D21
      DO 5 JP = IP1+1, N
      J=ABS(IA(JP))
      IJ = IND(I,J)
      I1J = IND(I1,J)
      Y(J) = Y(J) - A(I1J)*T
      A(IJ) = A(IJ) - A(I1J)*L1
    5 CONTINUE
      GO TO 11
      ELSE
C
C     IF THIS PORTION OF THE CODE IS REACHED WE ARE IN THE CASE OF A
C     1*1 SINGULAR BLOCK FOLLOWED BY A 2*2 BLOCK.
C     THIS 2*2 BLOCK IS JOINED TO THE 1*1 BLOCK TO FORM A 3*3 MATRIX D
C
      I1=-I1
      IP2 = IP1+1
      I2 = -IA(IP2)
      I1I = IND(I1,I)
      I2I = IND(I2,I)
      I1I1 = IND(I1,I1)
      I1I2 = IND(I1,I2)
      I2I2 = IND(I2,I2)
      T1 = Y(I1)
      T2 = Y(I2)
      B2 = SIG*T1
      B3 = SIG*T2
      D22 = A(I1I1) + B2*T1
      D32 = A(I1I2) + B3*T1
      D33 = A(I2I2) + B3*T2
      D21 = T1*B1
      D31 = T2*B1
      L1 = A(I1I)
      L2 = A(I2I)
      T = L2*D11
      D33 = D33 + L2*(TWO*D31 + T)
      D31 = D31 + T
      D32 = D32 + L1*D31 + L2*D21
      T = L1*D11
      D22 = D22 + L1*(TWO*D21 + T)
      D21 = D21 + T
      B2 = B2 + L1*B1
      B3 = B3 + L2*B1
      DO 6 JP = IP2+1, N
      J=ABS(IA(JP))
      IJ = IND(I,J)
      I1J = IND(I1,J)
      I2J = IND(I2,J)
      Y(J) = Y(J) - (A(I1J)*T1 + A(I2J)*T2)
      A(IJ) = A(IJ) - (A(I1J)*L1 + A(I2J)*L2)
    6 CONTINUE
      U1 = MAX(ABS(D11), ABS(D22), ABS(D33))
      U0 = MAX(ABS(D21), ABS(D31), ABS(D32))
      IF (U1 .GE. CON*U0) THEN
C
C      A 1*1 PIVOT WILL BE USED
C
      IF  ( U1 .EQ. ABS(D11))  THEN
      ELSE  IF  ( U1 .EQ. ABS(D22))  THEN
      T = D11
      D11 = D22
      D22 = T
      T = D32
      D32 = D31
      D31 = T
      T = B2
      B2 = B1
      B1 = T
      J = I1
      I1 = I
      I = J
      J = II
      II = I1I1
      I1I1 = J
      J = I1I2
      I1I2 = I2I
      I2I = J
      ELSE
      T = D11
      D11 = D33
      D33 = T
      T = D21
      D21 = D32
      D32 = T
      T = B1
      B1 = B3
      B3 = T
      J = I
      I = I2
      I2 = J
      J = II
      II = I2I2
      I2I2 = J
      J = I1I
      I1I = I1I2
      I1I2 = J
      ENDIF
      D22 = D22 - (D21 * D21) / D11
      D32 = D32 - (D31 * D21) / D11
      D33 = D33 - (D31 * D31) / D11
      B1 = B1 / D11
      B2 = B2 - B1 * D21
      B3 = B3 - B1 * D31
      D21 = D21 / D11
      D31 = D31 / D11
      SIG = SIG - D11 * B1 * B1
      DO 7  JP = IP2+1, N
      J=ABS(IA(JP))
      IJ = IND(I,J)
      I1J = IND(I1,J)
      I2J = IND(I2,J)
      A(IJ) = A(IJ) + D21 * A(I1J) + D31 * A(I2J) + B1 * Y(J)
    7 CONTINUE
      A(II) = D11
      A(I1I) = D21
      A(I2I) = D31
      IA(IP) = I
      IA(IP1) = -I1
      IA(IP2) = -I2
      D11 = D22
      D22 = D33
      D21 = D32
      B1 = B2
      B2 = B3
      IP = IP1
      IP1 = IP2
      I = I1
      I1 = I2
      II=I1I1
      I1I=I1I2
      I1I1=I2I2
      GO TO  11
      ELSE
C
C     A 2*2 PIVOT WILL BE USED
C
      IF  ( U0 .EQ. ABS(D21))   THEN
      ELSE   IF  ( U0 .EQ. ABS(D31))   THEN
      T = D22
      D22 = D33
      D33 = T
      T = D21
      D21 = D31
      D31 = T
      T = B2
      B2 = B3
      B3 = T
      J = I1
      I1 = I2
      I2 = J
      J = I1I1
      I1I1 = I2I2
      I2I2 = J
      J = I1I
      I1I = I2I
      I2I = J
      ELSE
      T = D11
      D11 = D22
      D22 = D33
      D33 = T
      T = D21
      D21 = D32
      D32 = D31
      D31 = T
      T = B1
      B1 = B2
      B2 = B3
      B3 = T
      J = I
      I = I1
      I1 = I2
      I2 = J
      J = II
      II = I1I1
      I1I1 = I2I2
      I2I2 = J
      J = I1I
      I1I = I1I2
      I1I2 = I2I
      I2I = J
      ENDIF
      DET = D11 * D22 - D21 * D21
      T = ( D22 * D31 - D21 * D32) / DET
      S = ( D11 * D32 - D21 * D31) / DET
      B3 = B3 - ( T*B1 + S*B2)
      D33 = D33 - ( T*D31 + S*D32)
      D31 = T
      D32 = S
      T = ( D22 * B1 - D21 * B2) / DET
      S = ( D11 * B2 - D21 * B1) / DET
      SIG = SIG - ( T*B1 + S*B2)
      B1 = T
      B2 = S
      DO  8   JP = IP2 + 1, N
      J=ABS(IA(JP))
      IJ = IND(I,J)
      I1J = IND(I1,J)
      I2J = IND(I2,J)
      A(IJ) = A(IJ) + D31 * A(I2J) + B1 * Y(J)
      A(I1J) = A(I1J) + D32 * A(I2J) + B2 * Y(J)
    8 CONTINUE
      A(II) = D11
      A(I1I) = D21
      A(I1I1) = D22
      A(I2I) = D31
      A(I1I2) = D32
      IA(IP) = -I
      IA(IP1) = -I1
      IA(IP2) = I2
      D11 = D33
      B1 = B3
      IP=IP2
      I = I2
      II=I2I2
      GO TO  3
      ENDIF
      ENDIF
    9 CONTINUE
C
C     THE DIAGONAL BLOCK BEGINNING AT ENTRY IP IS 2*2
C
      I = -I
      IP1 = IP + 1
      I1 = -IA(IP1)
      II = IND(I,I)
      I1I = IND(I1,I)
      I1I1 = IND(I1,I1)
      T1 = Y(I)
      T2 = Y(I1)
      B1 = SIG * Y(I)
      B2 = SIG * Y(I1)
      D11 = A(II) + B1 * Y(I)
      D21 = A(I1I) + B2 * Y(I)
      D22 = A(I1I1) + B2 * Y(I1)
      DO  10   JP = IP1+1, N
      J=ABS(IA(JP))
      IJ = IND(I,J)
      I1J = IND(I1,J)
      Y(J) = Y(J) - ( A(IJ)*T1 + A(I1J)*T2)
   10 CONTINUE
   11 CONTINUE
C
C     ENTER 2*2
C     THE 2*2 BLOCK WILL BE PROCESSED
C
      U1 = MAX( ABS(D11), ABS(D22))
      IF  ( U1 .LT. CON*ABS(D21))   THEN
C
C     A 2*2 PIVOT WILL BE USED
C
      DET = D11*D22 - D21*D21
      A(II) = D11
      A(I1I) = D21
      A(I1I1) = D22
      IA(IP)=-I
      IA(IP1)=-I1
      IF  ( IP1 .EQ. N)   RETURN
      T1 = (D22*B1 - D21*B2) / DET
      T2 = (D11*B2 - D21*B1) / DET
      DO  12   JP = IP1+1, N
      J=ABS(IA(JP))
      IJ = IND(I,J)
      I1J = IND(I1,J)
      A(IJ) = A(IJ) + T1*Y(J)
      A(I1J) = A(I1J) + T2*Y(J)
   12 CONTINUE
      SIG = SIG - (T1*B1 + T2*B2)
      IP = IP + 2
      GO TO  1
      ELSE
C
C     A 1*1 PIVOT WILL BE USED
C
      IF  ( U1 .NE. ABS(D11))   THEN
      T = D11
      D11 = D22
      D22 = T
      T = B1
      B1 = B2
      B2 = T
      J = I
      I = I1
      I1 = J
      J = II
      II = I1I1
      I1I1 = J
      ENDIF
      IA(IP) = I
      IA(IP1) = I1
      D22 = D22 - (D21*D21) / D11
      D21 = D21 / D11
      B2 = B2 - B1*D21
      B1 = B1 / D11
      DO  13  JP = IP1+1, N
      J=ABS(IA(JP))
      IJ = IND(I,J)
      I1J = IND(I1,J)
      A(IJ) = A(IJ) + D21*A(I1J) + B1*Y(J)
   13 CONTINUE
      A(II) = D11
      A(I1I) = D21
      SIG = SIG - B1 * D11 * B1
      D11 = D22
      B1 = B2
      IP = IP1
      I=I1
      II=I1I1
      GO TO  3
      ENDIF
      END
