* SUBROUTINE UXDSRO                ALL SYSTEMS               92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ESTIMATION OF THE MINIMUM SINGULAR VALUE AND THE CORRESPONDING
* SINGULAR VECTOR OF A PERMUTED DENSE UPPER TRIANGULAR MATRIX A.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) PERMUTED UPPER TRIANGULAR MATRIX.
*  II  IA(N)  PERMUTATION VECTOR.
*  RO  X(N)  ESTIMATED SINGULAR VECTOR.
*  RO  ALF  SQUARE OF THE ESTIMATED SINGULAR VALUE.
*  II  JOB  OPTION. IF JOB=0 THEN ONLY ESTIMATED SINGULAR VALUE IS
*         COMPUTED. IS JOB>0 THEN BOTH ESTIMATED SINGULAR VALUE AND
*         ESTIMATED SINGULAR VECTOR ARE COMPUTED BY JOB ITERATIONS.
*
* SUBPROGRAMS USED :
*  S   UXDSRB  BACK SUBSTITUTION.
*  S   UXDSRD  COMPUTATION OF A DIRECTION OF NEGATIVE CURVATURE.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*
* METHOD :
* C.H.BISCHOF : INCREMENTAL CONDITION ESTIMATION. SIAM J. MATRIX ANAL.
* APPL. 11 (1990) 312-322.
*
      SUBROUTINE UXDSRO(N,A,IA,X,ALF,JOB)
      INTEGER N,IA(N),JOB
      REAL*8 A(N*(N+1)/2),X(N),ALF
      REAL*8 DEN,XNEW,XEST,BET,ETA,EPS,S,C,UXVDOT
      INTEGER I,K,IP,KP,IK,KK,IND
      REAL*8 ZERO,ONE,TWO,HALF
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0,TWO=2.0D 0,HALF=5.0D-1)
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
C
C     COMPUTATION OF THE VECTOR V WITH POSSIBLE MAXIMUM NORM SUCH
C     THAT  TRANS(R)*V=U  WHERE U IS A VECTOR WITH UNIT NORM
C
      XEST=ZERO
      DO 3 K=1,N
      KP=IA(K)
      KK=IND(KP,KP)
      ALF=ZERO
      DO 1 I=1,K-1
      IP=IA(I)
      IK=IND(IP,KP)
      ALF=ALF+X(IP)*A(IK)
    1 CONTINUE
      DEN=A(KK)**2
      IF (DEN.EQ.ZERO) THEN
      ALF=ZERO
      CALL UXDSRD(N,A,IA,X,K)
      RETURN
      ELSE
      IF (ALF.EQ.ZERO) THEN
      IF (XEST*DEN.GT.ONE) THEN
      XNEW=XEST
      S=ONE
      C=ZERO
      ELSE
      XNEW=ONE/DEN
      S=ZERO
      C=ONE
      ENDIF
      ELSE
      BET=DEN*XEST+ALF**2-ONE
      ETA=HALF*BET/ABS(ALF)
      EPS=SQRT(ETA**2+ONE)
      XNEW=(ONE+ABS(ALF)*(ETA+EPS))/DEN
      S=SQRT((ONE+ETA/EPS)/TWO)
      C=SQRT((ONE-ETA/EPS)/TWO)
      ENDIF
      ENDIF
      DO 2 I=1,K-1
      IP=IA(I)
      X(IP)=X(IP)*S
    2 CONTINUE
      X(KP)=(C-ALF*S)/A(KK)
      XEST=XNEW
    3 CONTINUE
C
C     FIRST ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C     ALF=1/(TRANS(V)*V)
C
      IF (JOB.LE.0) THEN
      ALF=ONE/XEST
      RETURN
      ENDIF
C
C     COMPUTATION OF THE VECTOR X SUCH THAT R*X=V
C
      XEST=ZERO
      DO 9 K=N,1,-1
      KP=IA(K)
      DO 8 I=K+1,N
      IP=IA(I)
      IK=IND(IP,KP)
      X(KP)=X(KP)-A(IK)*X(IP)
    8 CONTINUE
      KK=IND(KP,KP)
      X(KP)=X(KP)/A(KK)
      XEST=XEST+X(KP)*X(KP)
    9 CONTINUE
      XEST=SQRT(XEST)
      IF (JOB.LE.1) THEN
C
C     SECOND ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C     ALF=1/SQRT(TRANS(X)*X)
C
      ALF=ONE/XEST
      ELSE
C
C     INVERSE ITERATIONS
C
      DO 11 K=2,JOB
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO 10 I=1,N
      IP=IA(I)
      X(IP)=X(IP)/XEST
   10 CONTINUE
      CALL UXDSRB(N,A,IA,X,0)
      XEST=SQRT(UXVDOT(N,X,X))
   11 CONTINUE
      ALF=ONE/XEST
      ENDIF
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO 12 I=1,N
      IP=IA(I)
      X(IP)=X(IP)/XEST
   12 CONTINUE
      RETURN
      END
