* SUBROUTINE UXDTHR                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  RECURSIVE ORTHOGONAL TRANSFORMATION OF A SET OF LINEAR
*  EQUATIONS USING SQUARE ROOT FREE ROTATIONS.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  IU  M  ACTUAL NUMBER OF ROWS IN THE MATRIX A.
*  RU  A(N*(N+1)/2) UPPER TRIANGULAR MATRIX OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  RU  D(N)  DIAGONAL MATRIX OBTAINED USING ORTHOGONAL TRANSFORMATION
*         OF A SET OF LINEAR EQUATIONS.
*  RU  B(N)  RIGHT HAND SIDE VECTOR OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  RU  X(N)  COEFFICIENTS OF A NEW LINEAR EQUATION.
*  RU  Y  RIGHT HAND SIDE OF A NEW LINEAR EQUATION.
*
* METHOD :
*  S.HAMMARLING: A NOTE ON MODIFICATIONS TO THE GIVENS PLANE
*  ROTATIONS. J. INST. MATHS. APPLICS. (1984),13,215-218.
*
      SUBROUTINE UXDTHR(N,M,A,D,B,X,Y)
      INTEGER N,M
      REAL*8 A(N*(N+1)/2),D(N),B(N),X(N),Y
      REAL*8 A1,A2,A3,B1,B2,B3,C
      INTEGER I,J,II,IJ
      REAL*8 ZERO,ONE
      PARAMETER(ZERO=0.0D 0,ONE=1.0D 0)
      II=0
      C=ONE
      DO 3 I=1,M
      II=II+I
      IF(X(I).NE.ZERO) THEN
      A1=A(II)
      A2=A1*D(I)
      A3=A1*A2
      B1=X(I)
      B2=B1*C
      B3=B1*B2
      IF(B3.LE.A3) THEN
      B1=B1/A1
      B2=B2/A2
      B3=B3/A3
      A3=ONE/(ONE+B3)
      D(I)=A3*D(I)
      C=A3*C
      A(II)=A(II)+B2*X(I)
      IJ=II
      DO 1 J=I+1,N
      IJ=IJ+J-1
      A1=B1*A(IJ)
      A(IJ)=A(IJ)+B2*X(J)
      X(J)=X(J)-A1
    1 CONTINUE
      A1=B1*B(I)
      B(I)=B(I)+B2*Y
      Y=Y-A1
      ELSE
      A1=A1/B1
      A2=A2/B2
      A3=A3/B3
      B3=ONE/(ONE+A3)
      B1=D(I)
      D(I)=B3*C
      C=B3*B1
      A(II)=X(I)+A2*A(II)
      IJ=II
      DO 2 J=I+1,N
      IJ=IJ+J-1
      B1=A(IJ)
      A(IJ)=X(J)+A2*A(IJ)
      X(J)=B1-A1*X(J)
    2 CONTINUE
      B1=B(I)
      B(I)=Y+A2*B(I)
      Y=B1-A1*Y
      ENDIF
      ENDIF
    3 CONTINUE
      IF(M.LT.N) THEN
      IJ=II+1
      DO 4 J=M+1,N
      IJ=IJ+J-1
      A(IJ)=X(J)
    4 CONTINUE
      M=M+1
      D(M)=C
      B(M)=Y
      ENDIF
      RETURN
      END
