* SUBROUTINE UXEGCF                ALL SYSTEMS                94/12/01
C PORTABILITY : ALL SYSTEMS
C 94/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* FACTORIZATION A=P*L*U OF A COMPLEX DENSE GENERAL MATRIX A WHERE P
* IS A PERMUTATION MATRIX, L IS A LOWER TRIANGULAR MATRIX AND U IA AN
* UPPER TRIANGULAR MATRIX.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRICES AR AND AI.
*  II  ND  DECLARED ORDER OF THE MATRICES AR AND AI.
*  RU  AR(ND,N)  ON INPUT A REAL PART OF A GIVEN COMPLEX MATRIX
*         STORED AS A TWO-DIMENSIONAL ARRAY. ON OUTPUT A REAL
*         PART OF THE COMPUTED FACTORIZATION A=P*L*U.
*  RU  AI(ND,N)  ON INPUT AN IMAGINARY PART OF A GIVEN COMPLEX MATRIX
*         STORED AS A TWO-DIMENSIONAL ARRAY. ON OUTPUT AN IMAGINARY
*         PART OF THE COMPUTED FACTORIZATION A=P*L*U.
*  IO  IP(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         THE PERMUTATION MATRIX P.
*  IO  IER  AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS NONSINGULAR. IF INF>0 THEN A IS SINGULAR.
*
* METHOD :
* C.B.MOLER, ALGORITHM 423, LINEAR EQUATION SOLVER, C.A.C.M. 15 (1972),
* P. 274.
*
      SUBROUTINE UXEGCF (N, ND, AR, AI, IP, IER)
      INTEGER N,ND,IP(N),IER,NM1,K,KP1,M,I,J
      REAL*8 AR(ND,N),AI(ND,N),TR,TI,PRODR,PRODI,DEN,ZERO
      PARAMETER (ZERO=0.0D 0)
      IER = 0
      IP(N) = 1
      IF (N .EQ. 1) GO TO 70
      NM1 = N - 1
      DO 60 K = 1,NM1
        KP1 = K + 1
        M = K
        DO 10 I = KP1,N
          IF (ABS(AR(I,K))+ABS(AI(I,K)) .GT.
     &          ABS(AR(M,K))+ABS(AI(M,K))) M = I
 10     CONTINUE
        IP(K) = M
        TR = AR(M,K)
        TI = AI(M,K)
        IF (M .EQ. K) GO TO 20
        IP(N) = -IP(N)
        AR(M,K) = AR(K,K)
        AI(M,K) = AI(K,K)
        AR(K,K) = TR
        AI(K,K) = TI
 20     CONTINUE
        IF (ABS(TR)+ABS(TI) .EQ. ZERO) GO TO 80
        DEN=TR*TR+TI*TI
        TR=TR/DEN
        TI=-TI/DEN
        DO 30 I = KP1,N
          PRODR=AR(I,K)*TR-AI(I,K)*TI
          PRODI=AI(I,K)*TR+AR(I,K)*TI
          AR(I,K)=-PRODR
          AI(I,K)=-PRODI
 30       CONTINUE
        DO 50 J = KP1,N
          TR = AR(M,J)
          TI = AI(M,J)
          AR(M,J) = AR(K,J)
          AI(M,J) = AI(K,J)
          AR(K,J) = TR
          AI(K,J) = TI
          IF (ABS(TR)+ABS(TI) .EQ. ZERO) GO TO 48
          IF (TI .EQ. ZERO) THEN
            DO 40 I = KP1,N
            PRODR=AR(I,K)*TR
            PRODI=AI(I,K)*TR
            AR(I,J) = AR(I,J) + PRODR
            AI(I,J) = AI(I,J) + PRODI
 40         CONTINUE
            GO TO 48
          END IF
          IF (TR .EQ. 0.D0) THEN
            DO 45 I = KP1,N
            PRODR=-AI(I,K)*TI
            PRODI=AR(I,K)*TI
            AR(I,J) = AR(I,J) + PRODR
            AI(I,J) = AI(I,J) + PRODI
 45         CONTINUE
            GO TO 48
          END IF
          DO 47 I = KP1,N
            PRODR=AR(I,K)*TR-AI(I,K)*TI
            PRODI=AI(I,K)*TR+AR(I,K)*TI
            AR(I,J) = AR(I,J) + PRODR
            AI(I,J) = AI(I,J) + PRODI
 47         CONTINUE
 48       CONTINUE
 50       CONTINUE
 60     CONTINUE
 70   K = N
      IF (ABS(AR(N,N))+ABS(AI(N,N)) .EQ. ZERO) GO TO 80
      RETURN
 80   IER = K
      IP(N) = 0
      RETURN
      END
