* SUBROUTINE UXLSMM                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE VARIABLE METRIC UPDATE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  NUMBER OF QUASI-NEWTON STEPS.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  GM(NF*MF)  SET OF GRADIENT DIFFERENCES.
*  RU  GR(MF*(MF+1)/2)  MATRIX TRANS(GM)*XM STORED IN PACKED FORM.
*  RO  HR(MF*(MF+1)/2)  MATRIX IN UPDATE REPRESENTATION.
*  RO  DR(MF)  MODIFIED DIAGONAL OF THE MATRIX GR.
*  RA  U1(MF)  AUXILIARY VECTOR.
*  RA  U2(MF)  AUXILIARY VECTOR.
*  RA  U3(MF)  AUXILIARY VECTOR.
*  RI  U(NF)  INPUT VECTOR.
*  RO  V(NF)  OUTPUT VECTOR.
*  RI  PAR  SCALING PARAMETER.
*  II  MET  METHOD SELECTION. MET=1-BFGS UPDATE. MET=4-RANK ONE UPDATE.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION.
*  S   UXDCMM  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXDCMD  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX AND ADDED TO A SCALED VECTOR.
*  S   UXDRMM  VECTOR MULTIPLIED BY A ROWWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  S   UXVMUL  VECTOR MULTIPLIED BY A DIAGONAL MATRIX STORED AS VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH THE CHANGE OF THE SIGN.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSEL  INITIATION OF A VECTOR.
*
* METHOD :
* LIMITED STORAGE VARIABLE METRIC METHODS.
*
      SUBROUTINE UXLSMM(NF,MF,XM,GM,GR,HR,DR,U1,U2,U3,U,V,PAR,MET)
      INTEGER NF,MF,MET
      REAL*8 XM(NF*MF),GM(NF*MF),GR(MF*(MF+1)/2),HR(MF*(MF+1)/2),
     & DR(MF),U1(MF),U2(MF),U3(MF),U(NF),V(NF),PAR
      INTEGER I,II,J
      IF (MF.LE.0) THEN
      CALL UXVCOP(NF,U,V)
      ELSE IF (MET.EQ.4) THEN
      CALL UXDRMM(NF,MF,XM,U,U1)
      CALL UXDRMD(NF,MF,GM,U,-PAR,U1,U1)
      CALL UXDPRB(MF,HR,U1,0)
      CALL UXDCMM(NF,MF,XM,U1,V)
      CALL UXDCMD(NF,MF,GM,U1,-PAR,V,V)
      CALL UXVDIR(NF,PAR,U,V,V)
      ELSE
      CALL UXDRMM(NF,MF,GM,U,U1)
      CALL UXDRMM(NF,MF,XM,U,U2)
      CALL UXVSCL(MF,PAR,U2,U2)
      CALL UXVMUL(MF,DR,U1,U3,1)
      II=0
      DO 2 I=2,MF
      II=II+1
      DO 1 J=1,I-1
      II=II+1
      U2(I)=U2(I)+GR(II)*U3(J)
    1 CONTINUE
    2 CONTINUE
      CALL UXDPRB(MF,HR,U2,0)
      II=0
      DO 4 I=2,MF
      II=II+1
      DO 3 J=1,I-1
      II=II+1
      U1(J)=U1(J)-GR(II)*U2(I)
    3 CONTINUE
    4 CONTINUE
      CALL UXVMUL(MF,DR,U1,U1,1)
      CALL UXDCMM(NF,MF,XM,U2,V)
      CALL UXDCMD(NF,MF,GM,U1,-PAR,V,V)
      CALL UXVDIR(NF,PAR,U,V,V)
      ENDIF
      RETURN
      END
