* SUBROUTINE UXPRMM                ALL SYSTEMS                10/12/01
C PORTABILITY : ALL SYSTEMS
C 10/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE QUASI-NEWTON PRECONDITIONER.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  NUMBER OF QUASI-NEWTON STEPS.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  GM(NF*MF)  SET OF GRADIENT DIFFERENCES.
*  II  IM(MF)  VECTOR OF INDICES.
*  RU  XR(MF*(MF+1)/2)  MATRIX TRANS(XM)*XM STORED IN PACKED FORM.
*  RA  UM(MF)  AUXILIARY VECTOR.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  U(NF)  INPUT VECTOR.
*  RO  V(NF)  OUTPUT VECTOR.
*  II  JOB  OPTION. JOB=1 - SCHUBERT UPDATE. JOB=2 - COLUMN UPDATE.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION.
*  S   UXDCMM  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXDCMD  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX AND ADDED TO A SCALED VECTOR.
*  S   UXDRMM  VECTOR MULTIPLIED BY A ROWWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH THE CHANGE OF THE SIGN.
*
* METHOD :
* LIMITED STORAGE QUASI-NEWTON METHODS.
*
      SUBROUTINE UXPRMM(NF,MF,XM,GM,IM,XR,UM,S,U,V,JOB)
      INTEGER NF,MF,IM(MF),JOB
      REAL*8 XM(NF*MF),GM(NF*MF),XR(MF*(MF+1)/2),UM(MF),S(NF),
     & U(NF),V(NF)
      INTEGER I
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      IF (MF.LE.0) THEN
      CALL UXVCOP(NF,U,V)
      ELSE
      IF (JOB.EQ.1) THEN
      CALL UXDRMM(NF,MF,XM,U,UM)
      ELSE
      DO 1 I=1,MF
      UM(I)=U(IM(I))
    1 CONTINUE
      ENDIF
      CALL UXDPRB(MF,XR,UM,-1)
      CALL UXDCMD(NF,MF,XM,UM,-ONE,U,S)
      CALL UXDCMM(NF,MF,GM,UM,V)
      CALL UXVDIR(NF,-ONE,V,S,V)
      CALL UXVNEG(NF,V,V)
      ENDIF
      RETURN
      END
