* SUBROUTINE UXSGDS                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DA : ORIGINAL VERSION
*
* PURPOSE :
* SPARSE SYMBOLIC AND NUMERICAL LU FACTORIZATION OF NONSYMMETRIC
* MATRIX FROM UMFPACK.
*
* PARAMETERS :
*  RA  XMEM(MMAX)  AUXILIARY REAL STORAGE.
*  IA  IMEM(MMAX)  AUXILIARY INTEGER STORAGE.
*  IO  PMEM(7,2)  STATUS OF AUXILIARY REAL AND INTEGER STORAGE
*  RI  AVALUE(NZ)  NONZERO ELEMENTS OF A SPARSE MATRIX.
*  II  AINDC(NZ)  COLUMN INDICES OF THE NONZERO ELEMENTS.
*  II  AINDR(NZ)  ROW INDICES OF THE NONZERO ELEMENTS.
*  II  N  ORDER OF A SPARSE MATRIX.
*  II  NZ NUMBER OF NONZERO ELEMENTS OF A SPARSE MATRIX.
*  IO  LU(2,2)  POINTERS OF LU FACTOR IN AN AUXILIARY STORAGE.
*  IO  ERROR  ERROR MESSAGE.
*  RI  RELPT  RELATIVE TOLERANCE FOR A PIVOT.
*  RI  ABSPT  ABSOLUTE TOLERANCE FOR A PIVOT.
*  RI  GRO  POSSIBLE GROW OF A FRONTAL MATRIX (USUALLY 2.0 - 3.0).
*  II  LOSRCH  NUMBER OF COLUMNS OF LOWEST LOWER BOUND DEGREE TO
*          EXAMINE DURING PIVOT SEARCH.
*  II  HISRCH  NUMBER OF COLUMNS OF LOWEST UPPER BOUND DEGREE TO
*          EXAMINE DURING PIVOT SEARCH.
*  II  NB  BLOCK SIZE FOR NUMERICAL FACTORIZATION (USUALLY 16 - 64).
*  II  EXTRA  EXTRA SPACE TO ALLOCATE FOR TUPLE LISTS. (USUALLY 5).
*  II  BLOK  OPTION. BLOK=0 NO PRELIMINARY PERMUTATIONS. BLOK=1
*         PRELIMINARY PERMUTATIONS TO BLOCK TRIANGULAR FORM.
*  II  SYMC  OPTION. SYMC=0 SYMMETRIC PIVOTS ARE NOT PREFERRED.
*         SYMC=1 SYMMETRIC PIVOTS ARE PREFERRED.
*
      SUBROUTINE UXSGDS(XMEM,IMEM,PMEM,AVALUE,AINDC,AINDR,N,NZ,LU,
     & ERROR,RELPT,ABSPT,GRO,LOSRCH,HISRCH,NB,EXTRA,BLOK,SYMC)
      INTEGER N,NZ,IMEM(*),PMEM(7,2),AINDC(NZ),AINDR(NZ),NINVLD,
     & LU(2,2),ERROR,LOSRCH,
     & HISRCH,NB,EXTRA,NPIV,IGAR,XGAR,BLOK,SYMC
      LOGICAL SYMSRC
      REAL*8 XMEM(*),AVALUE(NZ),RELPT,ABSPT,GRO,PMIN
      INTEGER MMEM(7,2),XSIZE,ISIZE,XP,IP,LU2(2,2),K,MAXINT,
     & HISRC2,NB2,EXTRA2,USAGE,BARE,LOSRC2
      LOGICAL TRANS,BLOCK,BLOCK2
      REAL*8 ABSPT2,RELPT2,GRO2
      TRANS=.FALSE.
      BLOCK=.FALSE.
      IF (BLOK.NE.0) BLOCK=.TRUE.
      SYMSRC=.FALSE.
      IF (SYMC.EQ.0) SYMSRC=.TRUE.
      IGAR = 0
      XGAR = 0
      CALL UXIMAX(MAXINT)
      IF (MAXINT .LT. 4*N+5) GOTO 999
      IF (N.LE.0) GOTO 998
      IF (NZ.LT.0) GOTO 997
      CALL UXIMCO(PMEM,ERROR)
      IF (ERROR .NE. 0) GOTO 996
      BLOCK2 = BLOCK
      RELPT2 = RELPT
      IF (RELPT2 .LT. 0.0) RELPT2 = 0.0
      IF (RELPT2 .GT. 1.0) RELPT2 = 1.0
      IF (ABSPT .LE. 0.0) THEN
      ABSPT2 = 0.0
      ELSE
      ABSPT2 = ABSPT
      ENDIF
      HISRC2 = MAX (0, HISRCH)
      HISRC2 = MIN (HISRC2, N)
      LOSRC2 = MAX (0, LOSRCH)
      LOSRC2 = MIN (LOSRC2, N)
      IF (HISRC2 .EQ. 0 .AND. LOSRC2 .EQ. 0) THEN
      HISRC2 = 1
      ENDIF
      EXTRA2 = MAX (0, EXTRA)
      EXTRA2 = MIN (EXTRA2, N)
      NB2 = MAX (1, NB)
      NB2 = MIN (NB2, N)
      GRO2 = GRO
      IF (GRO2 .LT. 1.0) GRO2 = 1.0
      IF (GRO2 .GT. N)   GRO2 = N
C
C     INITALIZE ALLOLCABLE MEMORY: XX/II, WHICH ARE THE SIZE OF
C     THE CURRENTLY AVAILABLE FREE MEMORY IN XMEM AND IMEM
C
      XP = PMEM (1,2)
      IP = PMEM (1,1)
      XSIZE = PMEM (2,2) - XP
      ISIZE = PMEM (2,1) - IP
      CALL UXIINI(MMEM,ERROR,1,ISIZE+1,
     & 1,XSIZE+1)
      IF (ERROR .NE. 0) GOTO 996
C
C     INITIALIZATIONS AND FACTORIZATION
C
      CALL UXSGS1(LU2,AINDC,AINDR,AVALUE,XMEM(XP),IMEM(IP),MMEM,
     & ERROR,MAXINT,HISRC2,NB2,
     & EXTRA2,LOSRC2,NPIV,N,NZ,BLOCK2,
     & ABSPT2,RELPT2,GRO2,PMIN,IGAR,XGAR,SYMSRC,
     & TRANS,NINVLD)
C
C     ADJUST PMEM TO REFLECT USAGE OF IMEM/XMEM DURING FACTORIZATION
C
      DO 10 K = 1, 2
      USAGE = (PMEM(1,K) - PMEM(3,K)) + (PMEM(4,K) - PMEM(2,K))
      BARE = PMEM (6,K)
      PMEM (5,K) = MAX (PMEM (5,K), USAGE + MMEM (5,K))
      PMEM (7,K) = MAX (PMEM (7,K), BARE + MMEM (7,K))
10    CONTINUE
      LU (1,1) = 0
      LU (1,2) = 0
      IF (ERROR .NE. 0) RETURN
C
C     THE LU FACTORS ARE ALLOCATED IN MMEM, BUT NOT IN PMEM
C
      CALL UXIGET(PMEM,ERROR,2,
     & LU2(2,2)-LU2(1,2),1,LU(1,2))
      IF (ERROR .NE. 0) GOTO 996
      IF (LU (1,2) .EQ. 0) GOTO 994
      CALL UXIGET(PMEM,ERROR,1,
     & LU2(2,1)-LU2(1,1),1,LU(1,1))
      IF (ERROR .NE. 0) GOTO 996
      IF (LU (1,1) .EQ. 0) GOTO 995
      LU (2,2) = LU (1,2) + (LU2 (2,2) - LU2 (1,2))
      LU (2,1) = LU (1,1) + (LU2 (2,1) - LU2 (1,1))
      RETURN
C
C     ERROR CONDITIONS
C
  999 ERROR = 68
      CALL UOERR1('UXSGDS',ERROR)
      RETURN
  998 ERROR = 61
      CALL UOERR1('UXSGDS',ERROR)
      RETURN
  997 ERROR = 62
      CALL UOERR1('UXSGDS',ERROR)
      RETURN
  996 ERROR = 64
      CALL UOERR1('UXSGDS',ERROR)
      RETURN
  995 ERROR = 65
      CALL UOERR1('UXSGDS',ERROR)
      RETURN
  994 ERROR = 66
      CALL UOERR1('UXSGDS',ERROR)
      RETURN
      END
