* SUBROUTINE UXSGFI                 ALL SYSTEMS                90/12/01
C PORTABILITY : ALL SYSTEMS
C 90/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* INITIALIZATION OF THE INDEX ARRAY CNR. IT IS FILLED WITH THE ROW
* AND COLUMN SCHEMES ACCORDING TO THE INFORMATION IN THE ARRAY BCOL.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  M  NUMBER OF PROCESSED COLUMNS THAT FORM THE COLUMS OF THE MATRIX
*         THAT DESCRIBES THE ACTIVE SET. M <= N.
*  II  Q  NUMBER OF COLUMNS OF THE MATRIX OF CONSTRAINTS.
*  II  NZA SIZE OF THE ARRAYS A AND RNR THAT STORE THE CONSTRAINTS
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NART NUMBER OF ARTIFICIAL VARIABLES THAT ARE IMPLICITLY
*           PROCESSED DURING FACTORIZATION
*  II  RNR(N) ROW INDICES OF THE STORED COLUMNS OF THE CONSTRAINT MATRIX
*  II  PRNR(N) POINTER VECTOR OF THE ARRAY RNR.
*  IO  LENC(N) LENGTHS OF THE COLUMNS.
*  IO  PTC(N) POINTER VECTOR OF THE COLUMNS OF THE SPARSE MATRIX.
*  IO  PTR(N) POINTER VECTOR OF THE ROWS OF THE SPARSE MATRIX.
*  IO  LENR(N) LENGTHS OF THE ROWS.
*  IO  CNR(NZ) CONTAINS THE INDICES OF THE COLUMN AND ROW SCHEME
*              OF THE PROCESSED MATRIX.
*  II  BCOL(Q)  ARRAY OF WHICH FIRST M POSITIONS SHOW THE INDICES
*               OF THE COLUMNS IN THE ARRAY RNR. POSITIONS M+1,...,
*               M+P CONTAIN INFORMATION ABOUT THE ACTIVE BOUNDS.
*               M+P <= N.
*  RI  A(NZA) ELEMENTS OF THE ORIGINAL CONSTRAINT MATRIX.
*  RO  AU(NAU) ELEMENTS OF THE PROCESSED SPARSE MATRIX.
*  IA  WN01(N) AUXILIARY VECTOR.
*  IO  IBGR BEGINNING OF THE ROW SCHEME IN CNR.
*  IO  IENDLU END OF THE COLUMN SCHEME IN CNR.
*  IO  IFL ERROR FLAG.
*  IO  NFL ERROR INFORMATION IMPORTANT FOR THE NEXT RUN.
*
      SUBROUTINE UXSGFI(N,M,Q,NZA,NZ,NAU,RNR,PRNR,LENC,PTC,PTR,LENR,
     *                CNR,BCOL,A,AU,WN01,IBGR,IENDLU,IFL,NFL,IJOB)
      INTEGER N,M,Q,NZA,NZ,NAU,IFL,NFL,IJOB
      INTEGER RNR(NZA),CNR(NZ)
      INTEGER PTR(N),PRNR(Q+1),PTC(N),LENR(N),LENC(N),BCOL(N)
      INTEGER IBGR,IENDLU
      INTEGER WN01(N)
      REAL*8 A(NZA),AU(NAU)
      INTEGER I,J,K,L,K2,LL,IDIST,JSTRT,JSTOP
C
C     SET CNR TO ZERO EXCLUDING THE LAST POSITION
C
      IF(IJOB.NE.1) THEN
        DO 100 I=1,NZ-1
        CNR(I)=0
100     CONTINUE
      ELSE
        DO 101 I=PRNR(Q+1),NZ-1
        CNR(I)=0
101     CONTINUE
      END IF
      CNR(NZ)=1
C
C     INITIALIZATON OF THE WORKING ARRAY
C
      DO 200 I=1,N
      LENR(I)=0
200   CONTINUE
C
C     NUMBER OF NONZERO ELEMENTS
C
      LL=PRNR(Q+1)-1
C
C     NUMBER OF FREE LOCATIONS
C
      K=NAU-LL
      L=NZ/2-LL
C
C
      IFL=0
      NFL=0
      IF(K.LT.0) THEN
                 IFL=1
                 NFL=-K
                 RETURN
      END IF
      IF(L.LT.0) THEN
                 IFL=2
                 NFL=-2*L
                 RETURN
      END IF
C
C     ROW AND COLUMN SHIFT
C
      IDIST=MIN(K,L)/(4*N)
      IF (IJOB.EQ.1) IDIST=0
C
C     BEGINNING OF THE ROW INDICES
C
      IBGR=NZ-LL-N*IDIST
C
C     LENGTH OF THE ROWS ARE COMPUTED
C
      DO 400 I=1,M
      JSTRT=I
      IF (IJOB.NE.1) JSTRT=BCOL(I)
      JSTOP=PRNR(JSTRT+1)-1
      JSTRT=PRNR(JSTRT)
            DO 300 J=JSTRT,JSTOP
            K=RNR(J)
            LENR(K)=LENR(K)+1
300         CONTINUE
400   CONTINUE
C
C     FILL THE ROW SCHEME
C
      PTR(1)=IBGR
      L=IBGR
      WN01(1)=IBGR
      DO 500 I=2,N
      L=L+LENR(I-1)+IDIST
      PTR(I)=L
      WN01(I)=L
500   CONTINUE
C
C     FILL THE ARRAY OF THE COLUMN INDICES CNR
C
      DO 700 I=1,M
      JSTRT=I
      IF (IJOB.NE.1) JSTRT=BCOL(I)
      JSTOP=PRNR(JSTRT+1)-1
      JSTRT=PRNR(JSTRT)
            DO 600 J=JSTRT,JSTOP
            K=RNR(J)
            L=WN01(K)
            CNR(L)=I
            WN01(K)=L+1
600         CONTINUE
700   CONTINUE
C
C     COLUMN SCHEME IS CREATED
C
      K2=1
      DO 900 I=1,M
      PTC(I)=K2
      JSTRT=I
      IF (IJOB.NE.1) JSTRT=BCOL(I)
      JSTOP=PRNR(JSTRT+1)-1
      JSTRT=PRNR(JSTRT)
            DO 800 K=JSTRT,JSTOP
            AU(K2)=A(K)
            CNR(K2)=RNR(K)
            K2=K2+1
800         CONTINUE
      LENC(I)=K2-PTC(I)
      K2=K2+IDIST
900   CONTINUE
      IENDLU=K2-IDIST-1
      RETURN
      END
