* SUBROUTINE UXSGFO                 ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* FORMING THE OUTPUT OF THE FACTORIZATION
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  M  NUMBER OF PROCESSED COLUMNS THAT FORM THE COLUMS OF THE MATRIX
*         THAT DESCRIBES THE ACTIVE SET. M <= N.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  IU  PTC(N) POINTER VECTOR TO THE COLUMNS OF THE U-FACTOR.
*  IU  LENC(N) LENGTHS OF THE COLUMNS OF THE U-FACTOR.
*  IU  CNR(NZ) CONTAINS THE INDICES OF THE COLUMN AND ROW SCHEME
*              OF THE PROCESSED MATRIX.
*  IO  PTR(N) POINTER VECTOR OF THE ROWS OF THE U-FACTOR.
*  IO  LENR(N) LENGTHS TO THE ROWS OF THE U-FACTOR.
*  IU  PTL(N) ON INPUT:VECTOR POINTING AT THE BEGINNING OF THE L-PART
*                      OF THE MATRIX BEEING FACTORIZED.
*             ON OUTPUT: POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  IO  LENL(N) LENGTHS OF THE COLUMNS OF THE L-FACTOR.
*  II  ORDR(N) ROW PERMUTATION VECTOR.
*  II  ORDC(N) COLUMN PERMUTATION VECTOR.
*  IU  INVC(N) ON INPUT :INVERSE COLUMN PERMUTATION VECTOR,
*              ON OUTPUT:SPIKE REDUCTION VECTOR.
*  RU  AU(NAU) NUMERICAL ELEMENTS OF THE PROCESSED SPARSE MATRIX.
*  IO  ORDL(N) ROW PERMUTATION VECTOR OF THE FACTOR L.
*  RI  DIAG(N) DIAGONAL ELEMENTS OF THE FACTOR U.
*  IO  IENDLU END OF THE ROW SCHEME OF THE FACTOR U IN CNR.
*  IO  IBGR BEGINNING OF THE COLUMN SCHEME OF THE FACTOR U IN CNR.
*  IO  IFL  ERROR FLAG.
*
      SUBROUTINE UXSGFO(N,M,NZ,NAU,PTC,LENC,CNR,PTR,LENR,PTL,LENL,
     *                 ORDR,ORDC,INVC,AU,ORDL,DIAG,IENDLU,IBGR,IFL)
      INTEGER N,M,NZ,NAU
      INTEGER PTC(N),PTR(N),PTL(N),LENC(N),LENR(N),LENL(N)
      INTEGER ORDR(N),ORDC(N),ORDL(N),INVC(N)
      INTEGER CNR(NZ)
      INTEGER IENDLU,IBGR,IFL
      REAL*8 AU(NAU),DIAG(N),ONE
      INTEGER JSTRT,JSTOP,KSTRT,KSTOP,J,J1,J2,L1,P1,IND,IFL1
      INTEGER NUMRZ,NUMCZ,LENS,II,K2,JLEN,I,K,L,LL,K1,ISTRT
      PARAMETER (ONE=1D0)
C     CALL COMPRESSION ROUTINE
C
      CALL UXSGFC(N,M,NZ,NAU,PTC,LENC,CNR,PTR,LENR,AU,IBGR,IENDLU)
C
C     K1 IS THE END OF THE FACTOR U THAT IS BEEING WRITTEN
C
      K1=NAU-1
C
C     SET THE ERROR FLAG
C
      IFL=0
C
C     MARK THE COLUMNS' BEGINNINGS
C
      DO 100 I=1,M
      K=ABS(PTC(I))
      IF(LENC(I).NE.0) THEN
                       L=CNR(K)
                       CNR(K)=-I
                       PTC(I)=L
                       ELSE
                       PTL(I)=1
                       PTC(I)=1
      END IF
100   CONTINUE
C
C     ARRAY OF THE LENGTH OF THE L FACTOR IS INITIALIZED
C
      DO 150 I=1,N
      LENL(I)=0
150   CONTINUE
C
C     MAIN CYCLE
C
      ISTRT=1
      I=1
200   K=CNR(I)
      IF(K.LT.0) THEN
                 K=-K
                 JLEN=PTL(K)
                 CNR(I)=PTC(K)
                 JSTRT=I
                 JSTOP=I+JLEN-1
                 KSTRT=JSTOP+1
                 KSTOP=I+LENC(K)-1
C
C     TEST IF IT IS POSSIBLE PLACE THE COLUMN OF U AT THE END OF
C     THE WORK SPACE . IF NOT ,THEN COLUMNS ARE SHRINKED.
C
C
C     ORIGINAL VERSION SHORTCUT
C
      IF((K1-JLEN).LE.IENDLU) THEN
                              IFL=1
                              RETURN
      END IF
          IF((K1-JLEN).LE.IENDLU) THEN
                     DO 300 J=I,IENDLU
                     CNR(ISTRT+J-I)=CNR(J)
                     AU(ISTRT+J-I)=AU(J)
300                  CONTINUE
                     I=ISTRT
                     JSTRT=I
                     JSTOP=I+PTL(K)-1
                     KSTRT=JSTOP+1
                     KSTOP=I+LENC(K)-1
                     IENDLU=IENDLU-ISTRT+I
          END IF
          IF ((K1-JLEN).LE.IENDLU) THEN
                     J1=JSTOP-JSTRT+1
                     J2=KSTOP-KSTRT+1
                     DO 400 J=0,MIN(J1,J2)-1
                     L1=CNR(JSTRT+J)
                     P1=AU(JSTRT+J)
                     CNR(JSTRT+J)=CNR(KSTOP-J)
                     AU(JSTRT+J)=AU(KSTOP-J)
                     CNR(KSTOP-J)=L1
                     AU(KSTOP-J)=P1
400                  CONTINUE
C
C     COMPARE THE L AND U FRACTIONS IN THE COLUMN
C
                     IF(J1.LE.J2) THEN
                     JSTRT=KSTOP-J1+1
                     KSTOP=JSTRT-1
                                  ELSE
                     KSTOP=JSTRT+J2-1
                     JSTRT=KSTOP+1
                     END IF
                     JSTOP=KSTOP
                     KSTRT=JSTRT
          END IF
C
C     REWRITE THE U-PART OF THE COLUMN
C
                 K1=K1-JLEN-1
                 PTC(K)=K1+1
                 IND=2
                 DO 500 J=JSTRT,JSTOP
                 AU(K1+IND)=AU(J)
                 CNR(K1+IND)=CNR(J)
                 IND=IND+1
500              CONTINUE
C
C
C     SET THE LENGTH LENC FOR THE COLUMN SCHEME OF THE U-FACTOR
C
                 LENC(K)=IND-1
C
C     REWRITE THE L-PART OF THE COLUMN
C
                 IND=0
                 DO 600 J=KSTRT,KSTOP
                 CNR(ISTRT+IND)=CNR(J)
                 AU(ISTRT+IND)=AU(J)
                 IND=IND+1
600              CONTINUE
C
C      PTL IS THE NEW POINTER OF THE L COLUMN
C
                 PTL(K)=ISTRT
                 LENL(K)=IND
                 ISTRT=ISTRT+IND
                 I=I+IND+LENC(K)-1
                 ELSE
                 I=I+1
      END IF
      IF(I.LE.IENDLU) GO TO 200
C
C     THE LAST PART OF PTL IS FORMED
C
      DO 700 I=M+1,N
      PTC(I)=0
      PTL(I)=1
      LENC(I)=0
700   CONTINUE
C
C     SPACE TEST. ACCORDING TO THE TEST TWO MODES OF THE WORK
C     WILL BE SET.
C     SITUATION : FACTOR L : POSITIONS 1,...,ISTRT-1
C                 FACTOR U - COL. SCHEME: POSITIONS K1+1,...,NAU-1.
C
      IF((K1-ISTRT+1).GE.(NAU-K1-1)) THEN
C
C     QUICKER VERSION
C
C
C     INITIALIZATION OF THE ROWS' LENGTHS
C
      DO 800 I=1,N
      LENR(I)=0
800   CONTINUE
C
C     COUNT NUMBER OF ELEMENTS IN ROWS
C
      DO 1000 I=1,N
      JSTRT=PTC(I)+1
      JSTOP=PTC(I)+LENC(I)-1
            DO 900 J=JSTRT,JSTOP
            K=CNR(J)
            LENR(K)=LENR(K)+1
900         CONTINUE
1000  CONTINUE
C
C     FIND THE FILLING POINTERS
C
      PTR(1)=ISTRT
      L=ISTRT
      ORDL(1)=ISTRT
      DO 1100 I=2,N
      L1=L
      L=L+LENR(I-1)
      IF(L.GT.L1) THEN
                  L=L+1
                  ORDL(I-1)=ORDL(I-1)+1
      END IF
      PTR(I)=L
      ORDL(I)=L
1100  CONTINUE
      IF(LENR(N).GT.0) ORDL(N)=ORDL(N)+1
      ISTRT=L+LENR(N)+1
      IF(ISTRT.GE.K1) THEN
                      IFL1=2
                      RETURN
      END IF
C
C     FILL THE SCHEME
C
      DO 1300 I=1,M
      JSTRT=PTC(I)+1
      JSTOP=PTC(I)+LENC(I)-1
            DO 1200 J=JSTRT,JSTOP
            K=CNR(J)
            L=ORDL(K)
C
C     ELEMENTS IN ROWS ARE PERMUTED : INSTEAD OF I IS INVC(I)
C
            CNR(L)=INVC(I)
            AU(L)=AU(J)
            ORDL(K)=L+1
1200        CONTINUE
1300  CONTINUE
C
C     ZERO ROWS AND COLUMNS
C
      NUMRZ=0
      NUMCZ=0
      DO 1350 I=1,N
      IF (LENR(I).EQ.0) THEN
                        NUMRZ=NUMRZ+1
      END IF
      IF(LENC(I).EQ.0) THEN
                       NUMCZ=NUMCZ+1
      END IF
1350  CONTINUE
C
C     TEST IF THE WITHENED FACTOR (USING THE ACTIVE BOUNDS) CAN
C     BE PLACED WITHIN THE GIVEN SPACE
C
C
C     FREE SPACE
C
      IF((ISTRT+NUMRZ).GE.K1)THEN
                             IFL=2
                             RETURN
                             ELSE
          IF ((ISTRT+NUMRZ+NUMCZ).GE.K1) THEN
                                    IFL=-1
                                    ELSE
C
C     SHIFT THE COLUMN SCHEME AT THE END
C
C
C     LENGTH OF THIS SHIFT
C
      LENS=NZ-NAU-NUMCZ
      IND=NUMCZ+1
C
C     DECIDE WHETHER THE SHIFT WAS NEGATIVE OR NOT
C
      IF(LENS.GE.0) THEN
      DO 1400 I=NAU-1,K1+1,-1
      CNR(NZ-IND)=CNR(I)
      IND=IND+1
1400  CONTINUE
      IBGR=NZ-IND+1
                    ELSE
      IND=K1+1+LENS
      IBGR=IND
      DO 1450 I=K1+1,NAU-1
      CNR(IND)=CNR(I)
      IND=IND+1
1450  CONTINUE
      END IF
C
C     PTC UPDATE
C
      DO 1500 I=1,M
      PTC(I)=PTC(I)+LENS
1500  CONTINUE
C
C     ADD THE REMAINING ELEMENTS OF THE FACTOR U
C
      IND=NZ-1
      DO 1600 L=1,M
      I=ORDC(L)
      K=LENC(I)
      IF(K.EQ.0)       THEN
                       PTC(I)=IND
                       CNR(IND)=ORDR(L)
                       LENC(I)=1
                       IND=IND-1
                       ELSE
                       CNR(PTC(I))=ORDR(L)
      END IF
1600  CONTINUE
      DO 1650 L=M+1,N
      I=ORDC(L)
      K=LENC(I)
      IF(K.EQ.0) THEN
                 PTC(I)=IND
                 CNR(IND)=ORDR(L)
                 IND=IND-1
                 LENC(I)=1
                 ELSE
                 CNR(PTC(I))=L
      END IF
1650  CONTINUE
          END IF
      END IF
C
C     DIAGONAL ELEMENTS ARE PERMUTED: IN THIS CYCLE INSTEAD OF
C     CNR(*)=ORDC(L) IS CNR(*)=L.
C
      DO 1700 L=1,M
      I=ORDR(L)
      K=LENR(I)
      IF(K.EQ.0)       THEN
                       PTR(I)=ISTRT
                       CNR(ISTRT)=L
                       AU(ISTRT)=DIAG(I)
                       LENR(I)=1
                       ISTRT=ISTRT+1
                       ELSE
                       CNR(PTR(I))=L
                       AU(PTR(I))=DIAG(I)
                       LENR(I)=LENR(I)+1
      END IF
1700  CONTINUE
      DO 1750 L=M+1,N
      I=ORDR(L)
      K=LENR(I)
      IF(K.EQ.0) THEN
                 PTR(I)=ISTRT
                 CNR(ISTRT)=L
                 AU(ISTRT)=ONE
                 LENR(I)=1
                 ISTRT=ISTRT+1
                 ELSE
                 CNR(PTR(I))=L
                 AU(PTR(I))=ONE
                 LENR(I)=LENR(I)+1
      END IF
1750  CONTINUE
                                     ELSE
                                     IFL=1
                                     RETURN
      END IF
C
C     SET END OF THE FACTORS
C
      IENDLU=ISTRT-1
      IF (IFL.EQ.-1) THEN
                    IBGR=NZ
      END IF
C
C     SET THE BARRIERS
C
      CNR(NZ)=1
      DO 1800 I=IENDLU+1,IBGR-1
      CNR(I)=0
1800  CONTINUE
C
C     PERMUTE PTL
C
      DO 1900 I=1,N
      ORDL(I)=PTL(I)
1900  CONTINUE
      DO 2000 I=1,N
      J=ORDC(I)
      PTL(I)=ORDL(J)
2000  CONTINUE
      DO 2100 I=1,N
      ORDL(I)=LENL(I)
2100  CONTINUE
      DO 2200 I=1,N
      J=ORDC(I)
      LENL(I)=ORDL(J)
2200  CONTINUE
      DO 2300 I=1,N
      ORDL(I)=ORDR(I)
2300  CONTINUE
C
C     SET THE VECTOR RIGHT USEFUL FOR SPIKE REDUCING
C     INTO THE ARRAY INVC. UP IS PLACED INTO DIAG.
C
      DO 2400 I=1,N
      INVC(I)=N+1
      DIAG(I)=0
2400  CONTINUE
      DO 2600 II=1,M
      I=ORDR(II)
      JSTRT=PTR(I)+1
      JSTOP=JSTRT+LENR(I)-2
      DO 2500 J=JSTRT,JSTOP
      K=CNR(J)
      K2=ORDC(K)
      LL=DIAG(K2)
      DIAG(K2)=MAX(LL,II)
      INVC(I)=MIN(INVC(I),CNR(J))
2500  CONTINUE
2600  CONTINUE
      RETURN
      END
