* SUBROUTINE UXSGIS                ALL SYSTEMS                96/12/01
C PORTABILITY : ALL SYSTEMS
C 96/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A SPARSE UNSYMMETRIC
* MATRIX A USING INCOMPLETE FACTORIZATION OBTAINED BY THE SUBROUTINE
* UXSGIF.
*
* PARAMETERS :
*  II  N  MATRIX DIMENSION.
*  II  M  NUMBER OF MATRIX NONZERO ELEMENTS.
*  RU  A(M)  NONZERO ELEMENTS OF THE MATRIX A.
*  II  IA(N+1)  ROW POINTERS OF THE MATRIX A.
*  II  JA(M)  COLUMN INDICES OF THE MATRIX A.
*  IO  IP(N)  PERMUTATION VECTOR.
*  II  ID(N) INDICES OF DIAGONAL ELEMENTS.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  RA  Y(N)  AUXILIARY VECTOR.
*  RI  PAR  RELAXATION PARAMETER.
*  JOB  OPTION. JOB=0 - SOLUTION WITH THE ORIGINAL MATRIX.
*         JOB=1 - SOLUTION WITH THE MATRIX TRANSPOSE.
*
      SUBROUTINE UXSGIS(N,M,A,IA,JA,IP,ID,X,Y,PAR,JOB)
      INTEGER N,M,IA(N+1),JA(M),IP(N),ID(N),JOB
      REAL*8 A(M),X(N),Y(N),PAR
      INTEGER J,K,JJ,JP,KSTRT,KSTOP
      REAL*8 APOM,ATEM,ZERO
      PARAMETER (ZERO=0.0D 0)
      IF (JOB.LE.0) THEN
      DO 2 K=1,N
      ATEM=ZERO
      KSTRT=IA(K)
      KSTOP=IA(K+1)-1
      DO 1 JJ=KSTRT,KSTOP
      J=JA(JJ)
      JP=IP(J)
      IF (JP.LT.K) ATEM=ATEM-A(JJ)*Y(JP)
    1 CONTINUE
      X(K)=X(K)+PAR*ATEM
      Y(K)=X(K)/A(ID(K))
    2 CONTINUE
      DO 4 K=N,1,-1
      ATEM=ZERO
      KSTRT=IA(K)
      KSTOP=IA(K+1)-1
      DO 3 JJ=KSTRT,KSTOP
      J=JA(JJ)
      JP=IP(J)
      IF (JP.GT.K) ATEM=ATEM-A(JJ)*X(JP)
      IF (JP.EQ.K) APOM=A(JJ)
    3 CONTINUE
      X(K)=(X(K)+PAR*ATEM)/APOM
    4 CONTINUE
      CALL UXVSFP(N,IP,X,Y)
      ELSE
      CALL UXVSBP(N,IP,X,Y)
      CALL UXVSET(N,ZERO,Y)
      DO 6 K=1,N
      X(K)=(X(K)+PAR*Y(K))/A(ID(K))
      KSTRT=IA(K)
      KSTOP=IA(K+1)-1
      DO 5 JJ=KSTRT,KSTOP
      J=JA(JJ)
      JP=IP(J)
      IF (JP.GT.K) Y(JP)=Y(JP)-A(JJ)*X(K)
    5 CONTINUE
    6 CONTINUE
      CALL UXVSET(N,ZERO,Y)
      DO 8 K=N,1,-1
      KSTRT=IA(K)
      KSTOP=IA(K+1)-1
      DO 7 JJ=KSTRT,KSTOP
      J=JA(JJ)
      JP=IP(J)
      IF (JP.LT.K) Y(JP)=Y(JP)-A(JJ)*X(K)
    7 CONTINUE
      X(K)=X(K)+PAR*Y(K)/A(ID(K))
    8 CONTINUE
      ENDIF
      RETURN
      END
