* SUBROUTINE UXSGUC                 ALL SYSTEMS                90/12/01
C PORTABILITY : ALL SYSTEMS
C 90/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SYMBOLICAL PART OF THE SPARSE COLUMN EXCHANGE.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  M  NUMBER OF PROCESSED COLUMNS THAT FORM THE COLUMS OF THE MATRIX
*         THAT DESCRIBES THE ACTIVE SET. M <= N.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NEL NUMBER OF L-UPDATES IN THE ARRAYS LI,LJ,CM.
*  II  NUP SIZE OF THE L-UPDATE ARRAYS (LI,LJ,CM)
*  IU  PTR(N) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  IU  LENR(N) LENGTHS OF THE ROWS OF THE FACTOR U.
*  IU  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  IU  ORDR(N) ROW PERMUTATION OF THE FACTOR U.
*  II  IDEB  =1 FOR LUSOL UPDATE - OTHERWISE USES PERMUTATION
*            OF THE BUMP.
*  IU  ORDC(N) COLUMN PERMUTATION OF THE FACTOR U.
*  IU  RIGHT(N) ROW SPIKE VECTOR OF THE FACTOR U.
*  IU  UP(N) COLUMN SPIKE VECTOR FOR THE FACTOR U.
*  IA  WR01(N) AUXILIARY VECTOR.
*  IA  WR02(N) AUXILIARY VECTOR.
*  IU  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  IU  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RU  CM(NUP) UPDATE MULTIPLIERS.
*  RU  AU(NAU) NUMERICAL VALUES OF THE FACTORS.
*  RI  STFL  STABILITY FLAG.
*  RU  MMFL  MAXIMUM MULTIPLIER FOUND.
*  RI  DRFL  DROPPING PARAMETER.
*  RU  MXU   MAXIMUM ELEMENT OF THE U-DIAGONAL.
*  RU  MNU   MINIMUM ELEMENT OF THE U-DIAGONAL.
*  IU  COLUP BEGINNING OF THE BUMP.
*  IU  LAST  END OF THE BUMP.
*  IU  IENDLU  END OF THE ROW INDICES OF THE FACTOR U IN CNR.
*  II  IBEG  BEGINNING OF THE FACTOR U IN CNR.
*  IO  IFL  OUTPUT INFORMATION:
*                              =11 IF NAU IS DECLARED TOO SMALL
*                              =12 IF MATRIX IS SINGULAR
*                              =13 IF FACTORS ARE BADLY CONDITIONED.
*                              =14 LITTLE SPACE FOR L-UPDATES.
*                              =15 MATRIX SINGULARITY IS FACED.
*
      SUBROUTINE UXSGUC(N,M,NZ,NAU,NEL,NUP,PTR,LENR,CNR,ORDR,IDEB,
     *                 ORDC,RIGHT,UP,WR01,WR02,LI,LJ,CM,AU,STFL,MMFL,
     *                 DRFL,MXU,MNU,COLUP,LAST,IENDLU,IBEG,IFL)
      INTEGER N,M,NZ,NAU,NEL,NUP,IDEB,COLUP,LAST,IBEG
      INTEGER IENDLU,IFL
      INTEGER PTR(N),LENR(N),ORDR(N),ORDC(N)
      INTEGER RIGHT(N),UP(N)
      INTEGER CNR(NZ),LI(NUP),LJ(NUP)
      INTEGER I,J,I1,I2,ROWUP,L,IND
      REAL*8 STFL,MMFL,DRFL,MXU,MNU
      REAL*8 WR01(N),WR02(N)
      REAL*8 CM(NUP),AU(NAU)
      INTEGER IMODE,I3,JND,IUP,IRIGH,K,K1,K2
      INTEGER RA,RL,PTRL,JSTRT,JSTOP
C
C     DECIDE WHETHER MORE REDUCING ITERATIONS IS USED
C     IMODE = 0 : NO . IMODE <> 0 : YES.
C
      IMODE=0
      IF(IMODE.EQ.0) THEN
                     GO TO 10
                     ELSE
                     CALL UXSGUA(N,NZ,PTR,LENR,CNR,ORDR,
     *                           ORDC,RIGHT,UP,WR01,WR02,
     *                           COLUP,LAST)
      END IF
C
C     INITIALIZATION
C
10    I2=0
      I3=0
      I1=N+1
      IND=LAST
      JND=COLUP
      DO 50 I=COLUP,LAST
      WR01(I)=I
50    CONTINUE
C
C     ROW SCAN. WR01 IS FILLED WITH THE NEW INDICES OF THE BUMP.
C     WR02 CONTAINS AT THE POSITIONS 1,...,I2  "FILLED" COLUMNS THAT
C     WILL BE SHIFTED UP AND AT THE POSITIONS
C     I1,...,N "EMPTY" COLUMNS THAT WILL BE TRANSFERRED AT THE END
C     OF THE BUMP.
C
      IF(IDEB.EQ.1) THEN
                    IRIGH=100000000
                    IUP=-1
                    ELSE
                    IRIGH=LAST
                    IUP=COLUP
      END IF
      DO 100 I=LAST,COLUP,-1
      IF(RIGHT(ORDR(I)).GT.IRIGH) THEN
                           I1=I1-1
                           WR01(I)=IND
                           IND=IND-1
                           WR02(I1)=I
                           ELSE
         IF(UP(ORDC(I)).LT.IUP) THEN
                           I1=I1-1
                           I3=I3+1
                           WR01(I)=JND
                           JND=JND+1
                           WR02(I1)=-I
                              ELSE
                              I2=I2+1
                              WR02(I2)=I
         END IF
      END IF
100   CONTINUE
      IF(I2.NE.0) THEN
                  IND=COLUP+I3
                  DO 150 I=I2-1,1,-1
                  J=WR02(I)
                  WR01(J)=IND
                  IND=IND+1
150               CONTINUE
                  J=WR02(I2)
                  WR01(J)=IND
      END IF
C
C     CHANGE THE INDICES IN THE REDUCED BUMP.
C     RIGHT IS ALSO UPDATED BY COMPARING IT WITH THE
C     NONDIAGONAL ELEMENTS OUTSIDE THE REDUCED BUMP.
C
      DO 300 I=1,I2
         L=ORDR(INT(WR02(I)))
         DO 200 J=PTR(L),PTR(L)+LENR(L)-1
         K=CNR(J)
         IF(K.LE.LAST) CNR(J)=WR01(K)
200      CONTINUE
300   CONTINUE
      DO 320 I=I1,N
         L=ORDR(INT(ABS(WR02(I))))
         DO 310 J=PTR(L),PTR(L)+LENR(L)-1
         K=CNR(J)
         IF(K.LE.LAST) CNR(J)=WR01(K)
310      CONTINUE
320   CONTINUE
C
C     UP IS SET FOR THE COLUMNS OF THE BUMP
C
      DO 110 I=COLUP,LAST
      UP(ORDC(I))=0
110   CONTINUE
C
C     CHANGE THE OTHER INDICES
C
      DO 400 I=1,COLUP-1
      RA=N+1
      L=ORDR(I)
      RL=RIGHT(L)
      PTRL=PTR(L)
      K=CNR(PTRL)
      IF(K.LE.LAST.AND.K.GE.COLUP) CNR(J)=WR01(K)
            DO 350 J=PTRL+1,PTRL+LENR(L)-1
            K=CNR(J)
            K2=ORDC(K)
            IF(K.LE.LAST.AND.K.GE.COLUP) THEN
                                         K1=WR01(K)
                                         CNR(J)=K1
                                         RA=MIN(RA,K1)
                                         UP(K2)=MAX(UP(K2),I)
            END IF
350         CONTINUE
C
C     SETTING OF THE RIGHT
C
      IF (RL.LT.COLUP.OR.RL.GT.LAST) GO TO 400
      RIGHT(L)=RA
400   CONTINUE
C
C     PERMUTE THE ROWS AND COLUMNS.
C     ROWS.
C
      IND=I3
      JND=I3+1
      IF(I2.NE.0) THEN
                  DO 600 I=I2-1,1,-1
                  L=WR02(I)
                  L=ORDR(L)
                  IND=IND+1
                  WR01(IND)=L
600               CONTINUE
                  IND=IND+1
                  WR01(IND)=ORDR(INT(WR02(I2)))
      END IF
      DO 500 I=I1,N
      L=WR02(I)
      IF(L.LT.0) THEN
                 L=-L
                 L=ORDR(L)
                 JND=JND-1
                 WR01(JND)=L
                 ELSE
                 L=ORDR(L)
                 IND=IND+1
                 WR01(IND)=L
      END IF
500   CONTINUE
      DO 700 I=LAST,COLUP,-1
      ORDR(I)=WR01(IND)
      IND=IND-1
700   CONTINUE
C
C     COLUMNS.
C
      IND=I3
      JND=I3+1
      IF(I2.NE.0) THEN
            DO 900 I=I2-1,1,-1
            L=WR02(I)
            L=ORDC(L)
            IND=IND+1
            WR01(IND)=L
900         CONTINUE
            IND=IND+1
            WR01(IND)=ORDC(INT(WR02(I2)))
      END IF
      DO 800 I=I1,N
      L=WR02(I)
      IF(L.LT.0) THEN
                 L=-L
                 L=ORDC(L)
                 JND=JND-1
                 WR01(JND)=L
                 ELSE
                 L=ORDC(L)
                 IND=IND+1
                 WR01(IND)=L
      END IF
800   CONTINUE
      DO 1000 I=LAST,COLUP,-1
      ORDC(I)=WR01(IND)
      IND=IND-1
1000  CONTINUE
C
C     SET ROWUP
C
      ROWUP=LAST+I1-N-1+I3
C
C     ELIMINATE THE ROW SPIKE
C
      CALL UXSGUR(N,M,NZ,NAU,NEL,NUP,ROWUP,PTR,LENR,CNR,ORDR,
     *            WR01,WR02,LI,LJ,CM,AU,STFL,MMFL,DRFL,MXU,MNU,
     *            IENDLU,IBEG,IFL)
C
C TEST IFL
C
      IF(IFL.NE.0) RETURN
C
C    RIGHT AND UP UPDATE OF THE REDUCED BUMP
C
      DO 1200 I=COLUP,LAST
      L=ORDR(I)
      RIGHT(L)=N+1
      JSTRT=PTR(L)+1
      JSTOP=JSTRT+LENR(L)-2
      DO 1100 J=JSTRT,JSTOP
      K=CNR(J)
      K2=ORDC(K)
      UP(K2)=MAX(UP(K2),I)
      RIGHT(L)=MIN(RIGHT(L),CNR(J))
1100  CONTINUE
1200  CONTINUE
      RETURN
      END
