* SUBROUTINE UXSIIN                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL TRANSFORMATION OF THE PARTITIONED KARUSH-KUHN-TUCKER MATRIX
* INTO THE COMPLETE SYMMETRIC STRUCTURE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  RI  H(M)  NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  II  IH(N+1)  POINTERS OF THE HESSIAN MATRIX.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  RI  CG(MC)  NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  ICG(NC+1)  POINTERS OF THE JACOBIAN MATRIX.
*  II  JCG(MC)  COLUMN INDICES OF THE JACOBIAN MATRIX.
*  II  KCG(MC+N)  INDICES OF ELEMENTS OF THE ARRAY CG IN THE ARRAY B.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RO  B(2*(M+MC)-N)  NONZERO ELEMENTS OF THE GENERAL KARUSH-KUHN-TUCKER
*         MATRIX.
*  II  IB(N+NC+1)  POINTERS OF THE GENERAL KARUSH-KUHN-TUCKER MATRIX.
*  RO  D(NCA)  DIAGONAL SOUTH-EAST CORNER.
*
      SUBROUTINE UXSIIN(N,M,H,IH,NC,MC,CG,ICG,JCG,KCG,NCA,ICA,B,IB,D)
      INTEGER N,M,IH(N+1),NC,MC,ICG(NC+1),JCG(MC),KCG(MC+N),NCA,
     & ICA(NCA),IB(N+NCA+1)
      REAL*8 H(M),CG(MC),B(M+MC+N+NCA),D(NCA)
      INTEGER I,J,L,KC
      REAL ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
C
C     ELEMENTS OF THE MATRIX H
C
      DO 2 I=1,N
      L=IB(I)
      DO 1 J=IH(I),IH(I+1)-1
      B(L)=H(J)
      L=L+1
    1 CONTINUE
    2 CONTINUE
C
C     ELEMENTS OF THE MATRIX CG
C
      DO 4 I=1,NCA
      KC=ICA(I)
      IF (KC.LT.0) THEN
      B(KCG(MC-KC))=ONE
      ELSE
      DO 3 J=ICG(KC),ICG(KC+1)-1
      IF (JCG(J).GT.0) THEN
      B(KCG(J))=CG(J)
      ELSE
      B(KCG(J))=ZERO
      ENDIF
    3 CONTINUE
      ENDIF
    4 CONTINUE
C
C     DIAGONAL SUBMATRIX
C
      DO 5 J=1,NCA
      B(IB(N+J))=D(J)
    5 CONTINUE
      RETURN
      END
