* SUBROUTINE UXSIS7                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* AUXILIARY SUBROUTINE FOR UXSIBS.
* STORAGE AND OPERATION COUNT EVALUATION.
*
      SUBROUTINE UXSIS7(N, M, KA, JA, PERM, NA, NE, ND, NS,
     & LSTKI, LSTKR, IW)
C
C     EVALUATE NUMBER OF OPERATIONS AND SPACE REQUIRED BY FACTORIZATION
C     USING UXSIBF.  THE VALUES GIVEN ARE EXACT ONLY IF NO NUMERICAL
C     PIVOTING IS PERFORMED AND THEN ONLY IF KA(1) WAS NOT
C     EQUIVALENCED TO IW(1) BY THE USER BEFORE CALLING UXSIBS.  IF
C     THE EQUIVALENCE HAS BEEN MADE ONLY AN UPPER BOUND FOR NIRNEC
C     AND NRLNEC CAN BE CALCULATED ALTHOUGH THE OTHER COUNTS WILL
C     STILL BE EXACT.
C
      INTEGER N, M, NS
      INTEGER NA(NS),NE(NS),ND(NS),KA(*),JA(*),
     & PERM(N),IW(*)
      INTEGER LSTKR(N), LSTKI(N)
C
C     N MUST BE SET TO THE MATRIX ORDER. IT IS NOT ALTERED.
C     M MUST BE SET TO THE NUMBER OF NON-ZEROS INPUT. IT IS NOT ALTERED.
C     KA,JA.  UNLESS KA(1) HAS BEEN EQUIVALENCED TO IW(1)
C     KA,JA MUST BE SET TO THE ROW AND COLUMN INDICES OF THE
C     NON-ZEROS ON INPUT.  THEY ARE NOT ALTERED BY UXSIS7.
C     PERM MUST BE SET TO THE POSITION IN THE PIVOT ORDER OF EACH ROW.
C     IT IS NOT ALTERED.
C     NA,NE,ND MUST BE SET TO HOLD, FOR EACH TREE NODE, THE NUMBER OF
C     STACK ELEMENTS ASSEMBLED, THE NUMBER OF ELIMINATIONS AND THE SIZE
C     OF THE ASSEMBLED FRONT MATRIX RESPECTIVELY.  THEY ARE NOT ALTERED.
C     NS MUST BE SET TO HOLD THE NUMBER OF TREE NODES. IT IS NOT
C     ALTERED.
C     LSTKI IS USED AS A WORK ARRAY BY UXSIS7.
C     LSTKR.  IF KA(1) IS EQUIVALENCED TO IW(1)  THEN LSTKR(I)
C     MUST HOLD THE TOTAL NUMBER OF OFF-DIAGONAL ENTRIES (INCLUDING
C     DUPLICATES) IN ROW I (I=1,..,N) OF THE ORIGINAL MATRIX.  IT
C     IS USED AS WORKSPACE BY UXSIS7.
C     IW IS A WORKSPACE ARRAY USED BY OTHER SUBROUTINES AND PASSED TO THIS
C     SUBROUTINE ONLY SO THAT A TEST FOR EQUIVALENCE WITH KA CAN BE
C     MADE.
C
      INTEGER I,IERROR,  INEW, IOLD, IORG, IROW, ISTKI, ISTKR,
     &        ITOP, ITREE,
     &        JOLD, JORG, K,
     &        LSTK,
     &        NASSR, NCMPA, NCMPBI, NCMPBR,  NEIG, NELIM, NFR,
     &        NIRADU, NIRBDU, NIRNEC, NIRTOT,
     &        NRLADU, NRLBDU, NTWO,
     &        NRLNEC,NRLTOT,  NSTK, NUMORG,  M1, M2
      DOUBLE PRECISION OPS
      INTRINSIC MIN0, MAX0
C
C     COUNTS FOR OPERATIONS AND STORAGE ARE ACCUMULATED IN VARIABLES
C     OPS,NRLTOT,NIRTOT,NRLNEC,NIRNEC,NRLADU,NRLNEC,NIRNEC.
C     OPS NUMBER OF MULTIPLICATIONS AND ADDITIONS DURING FACTORIZATION.
C     NRLADU,NIRADU REAL AND INTEGER STORAGE RESPECTIVELY FOR THE
C     MATRIX FACTORS.
C     NRLTOT,NIRTOT REAL AND INTEGER STRORAGE RESPECTIVELY REQUIRED
C     FOR THE FACTORIZATION IF NO COMPRESSES ARE ALLOWED.
C     NRLNEC,NIRNEC REAL AND INTEGER STORAGE RESPECTIVELY REQUIRED FOR
C     THE FACTORIZATION IF COMPRESSES ARE ALLOWED.
C     IERROR,NCMPA,NCMPBR,NCMPBI AND NTWO ARE NOT USED IN THIS
C     SUBROUTINE.
C
      COMMON /UMMAT2/ OPS, IERROR, NRLTOT, NIRTOT, NRLNEC, NIRNEC,
     & NRLADU, NIRADU, NRLBDU, NIRBDU, NCMPA, NCMPBR, NCMPBI, NTWO,NEIG
C
C     JUMP IF IW AND KA HAVE NOT BEEN EQUIVALENCED.
C
      IF (KA(1).NE.IW(1)) GO TO 20
C
C     RESET KA(1).
C
      KA(1) = IW(1)-1
C
C     THE TOTAL NUMBER OF OFF-DIAGONAL ENTRIES IS ACCUMULATED IN M2.
C     LSTKI IS SET TO HOLD THE TOTAL NUMBER OF ENTRIES (INCUDING
C     THE DIAGONAL) IN EACH ROW IN PERMUTED ORDER.
C
      M2 = 0
      DO 10 IOLD=1,N
        INEW = PERM(IOLD)
        LSTKI(INEW) = LSTKR(IOLD) + 1
        M2 = M2 + LSTKR(IOLD)
   10 CONTINUE
C
C     M1 IS THE NUMBER OF ENTRIES IN ONE TRIANGLE INCLUDING THE
C     DIAGONAL.
C     M2 IS THE TOTAL NUMBER OF ENTRIES INCLUDING THE DIAGONAL.
C
      M1 = M2/2 + N
      M2 = M2 + N
      GO TO 60
C
C     COUNT (IN LSTKI) NON-ZEROS IN ORIGINAL MATRIX BY PERMUTED ROW
C     (UPPER TRIANGLE ONLY). INITIALIZE COUNTS.
C
   20 DO 30 I=1,N
        LSTKI(I) = 1
   30 CONTINUE
C
C     ACCUMULATE NUMBER OF NON-ZEROS WITH INDICES IN RANGE IN M1
C     DUPLICATES ON THE DIAGONAL ARE IGNORED BUT M1 INCLUDES ANY
C     DIAGONALS NOT PRESENT ON INPUT.
C     ACCUMULATE ROW COUNTS IN LSTKI.
C
      M1 = N
      IF (M.EQ.0) GO TO 50
      DO 40 I=1,M
        IOLD = KA(I)
        JOLD = JA(I)
C
C     JUMP IF INDEX IS OUT OF RANGE.
C
        IF (IOLD.LT.1 .OR. IOLD.GT.N) GO TO 40
        IF (JOLD.LT.1 .OR. JOLD.GT.N) GO TO 40
        IF (IOLD.EQ.JOLD) GO TO 40
        M1 = M1 + 1
        IROW = MIN0(PERM(IOLD)+0,PERM(JOLD)+0)
        LSTKI(IROW) = LSTKI(IROW) + 1
   40 CONTINUE
   50 M2 = M1
C
C     ISTKR,ISTKI CURRENT NUMBER OF STACK ENTRIES IN
C     REAL AND INTEGER STORAGE RESPECTIVELY.
C     OPS,NRLADU,NIRADU,NIRTOT,NRLTOT,NIRNEC,NRLNEC,M2 ARE DEFINED
C     ABOVE.
C     M2 CURRENT NUMBER OF ORIGINAL MATRIX ENTRIES NOT YET PROCESSED.
C     NUMORG CURRENT TOTAL NUMBER OF ROWS ELIMINATED.
C     ITOP CURRENT NUMBER OF ELEMENTS ON THE STACK.
C
   60 ISTKI = 0
      ISTKR = 0
      OPS = 0.0D0
      NRLADU = 0
C
C     ONE LOCATION IS NEEDED TO RECORD THE NUMBER OF BLOCKS
C     ACTUALLY USED.
C
      NIRADU = 1
      NIRTOT = M1
      NRLTOT = M1
      NIRNEC = M2
      NRLNEC = M2
      NUMORG = 0
      ITOP = 0
C
C     EACH PASS THROUGH THIS LOOP PROCESSES A NODE OF THE TREE.
C
      DO 100 ITREE=1,NS
        NELIM = NE(ITREE)
        NFR = ND(ITREE)
        NSTK = NA(ITREE)
C
C     ADJUST STORAGE COUNTS ON ASSEMBLY OF CURRENT FRONTAL MATRIX.
C
        NASSR = NFR*(NFR+1)/2
        IF (NSTK.NE.0) NASSR = NASSR - LSTKR(ITOP) + 1
        NRLTOT = MAX0(NRLTOT,NRLADU+NASSR+ISTKR+M1)
        NIRTOT = MAX0(NIRTOT,NIRADU+NFR+2+ISTKI+M1)
        NRLNEC = MAX0(NRLNEC,NRLADU+NASSR+ISTKR+M2)
        NIRNEC = MAX0(NIRNEC,NIRADU+NFR+2+ISTKI+M2)
C
C     DECREASE M2 BY THE NUMBER OF ENTRIES IN ROWS BEING ELIMINATED AT
C     THIS STAGE.
C
        DO 70 IORG=1,NELIM
          JORG = NUMORG + IORG
          M2 = M2 - LSTKI(JORG)
   70   CONTINUE
        NUMORG = NUMORG + NELIM
C
C     JUMP IF THERE ARE NO STACK ASSEMBLIES AT THIS NODE.
C
        IF (NSTK.LE.0) GO TO 90
C
C     REMOVE ELEMENTS FROM THE STACK.  THERE ARE ITOP ELEMENTS ON THE
C     STACK WITH THE APPROPRIATE ENTRIES IN LSTKR,LSTKI GIVING
C     THE REAL AND INTEGER STORAGE RESPECTIVELY FOR EACH STACK
C     ELEMENT.
C
        DO 80 K=1,NSTK
          LSTK = LSTKR(ITOP)
          ISTKR = ISTKR - LSTK
          LSTK = LSTKI(ITOP)
          ISTKI = ISTKI - LSTK
          ITOP = ITOP - 1
   80   CONTINUE
C
C     ACCUMULATE NON-ZEROS IN FACTORS AND NUMBER OF OPERATIONS.
C
   90   NRLADU = NRLADU + (NELIM*(2*NFR-NELIM+1))/2
        NIRADU = NIRADU + 2 + NFR
        IF (NELIM.EQ.1) NIRADU = NIRADU - 1
       OPS = OPS + ((NFR*NELIM*(NFR+1)-(2*NFR+1)*NELIM*(NELIM+1)/
     &   2+NELIM*(NELIM+1)*(2*NELIM+1)/6)/2)
        IF (ITREE.EQ.NS) GO TO 100
C
C     JUMP IF ALL OF FRONTAL MATRIX HAS BEEN ELIMINATED.
C
        IF (NFR.EQ.NELIM) GO TO 100
C
C     STACK REMAINDER OF ELEMENT.
C
        ITOP = ITOP + 1
        LSTKR(ITOP) = (NFR-NELIM)*(NFR-NELIM+1)/2
        LSTKI(ITOP) = NFR - NELIM + 1
        ISTKI = ISTKI + LSTKI(ITOP)
        ISTKR = ISTKR + LSTKR(ITOP)
C
C     WE DO NOT NEED TO ADJUST THE COUNTS FOR THE REAL STORAGE BECAUSE
C     THE REMAINDER OF THE FRONTAL MATRIX IS SIMPLY MOVED IN THE
C     STORAGE FROM FACTORS TO STACK AND NO EXTRA STORAGE IS REQUIRED.
C
        NIRTOT = MAX0(NIRTOT,NIRADU+ISTKI+M1)
        NIRNEC = MAX0(NIRNEC,NIRADU+ISTKI+M2)
  100 CONTINUE
C
C     ADJUST THE STORAGE COUNTS TO ALLOW FOR THE USE OF THE REAL AND
C     INTEGER STORAGE FOR PURPOSES OTHER THAN PURELY THE
C     FACTORIZATION ITSELF.
C     THE SECOND TWO TERMS ARE THE MINUMUM AMOUNT REQUIRED BY UXSIF1.
C
      NRLNEC = MAX0(NRLNEC,N+MAX0(M,M1))
      NRLTOT = MAX0(NRLTOT,N+MAX0(M,M1))
      NRLNEC = MIN0(NRLNEC,NRLTOT)
      NIRNEC = MAX0(M,NIRNEC)
      NIRTOT = MAX0(M,NIRTOT)
      NIRNEC = MIN0(NIRNEC,NIRTOT)
      RETURN
      END
