* SUBROUTINE UXSKGS                ALL SYSTEMS                96/12/01
C PORTABILITY : ALL SYSTEMS
C 96/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SYMBOLIC TRANSFORMATION OF THE PARTITIONED KARUSH-KUHN-TUCKER MATRIX
* INTO THE COMPLETE GENERAL STRUCTURE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  II  IH(N+1)  POINTERS OF THE HESSIAN MATRIX.
*  II  JH(M)  COLUMN INDICES OF THE HESSIAN MATRIX.
*  IO  KH(M)  INDICES OF ELEMENTS OF THE ARRAY H IN THE ARRAY B.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  ICG(N+1)  POINTERS OF THE JACOBIAN MATRIX.
*  II  JCG(MC)  COLUMN INDICES OF THE JACOBIAN MATRIX.
*  IO  KCG(MC)  INDICES OF ELEMENTS OF THE ARRAY CG IN THE ARRAY B.
*  IO  IB(N+NC+1)  POINTERS OF THE GENERAL KARUSH-KUHN-TUCKER MATRIX.
*  IO  JB(2*(M+MC)-N)  COLUMN INDICES OF THE GENERAL KARUSH-KUHN-TUCKER
*  IO  KB(N+1) POINTERS OF DIAGONAL ELEMENTS IN THE ARRAY B.
*         MATRIX.
*
      SUBROUTINE UXSKGS(N,M,IH,JH,KH,NC,MC,ICG,JCG,KCG,IB,JB,KB)
      INTEGER N,M,IH(N+1),JH(M),KH(M),NC,MC,ICG(NC+1),JCG(MC),KCG(MC),
     & IB(N+NC+1),JB(2*(M+MC)-N),KB(N+1)
      INTEGER I,J,K,KC
C
C     INITIAL COMPUTATION OF THE POINTERS
C
      KB(1)=1
      DO 1 I=1,N
      KB(I+1)=IH(I+1)-IH(I)
    1 CONTINUE
      DO 3 I=1,N
      DO 2 J=IH(I)+1,IH(I+1)-1
      K=JH(J)
      KB(K+1)=KB(K+1)+1
    2 CONTINUE
    3 CONTINUE
      DO 5 KC=1,NC
      DO 4 J=ICG(KC),ICG(KC+1)-1
      K=JCG(J)
      KB(K+1)=KB(K+1)+1
    4 CONTINUE
    5 CONTINUE
      DO 6 I=1,N
      KB(I+1)=KB(I+1)+KB(I)
    6 CONTINUE
C
C     SUBDIAGONAL ELEMENTS OF THE MATRIX H
C
      DO 9 I=1,N
      DO 8 J=IH(I)+1,IH(I+1)-1
      K=JH(J)
      JB(KB(K))=I
      KH(J)=KB(K)
      KB(K)=KB(K)+1
    8 CONTINUE
    9 CONTINUE
C
C     SUPERDIAGONAL ELEMENTS OF THE MATRIX H
C
      DO 11 I=1,N
      IB(I)=KB(I)
      DO 10 J=IH(I),IH(I+1)-1
      JB(IB(I))=JH(J)
      IB(I)=IB(I)+1
   10 CONTINUE
   11 CONTINUE
C
C     SUPERDIAGONAL ELEMENTS OF THE MATRIX CG
C
      DO 13 KC=1,NC
      DO 12 J=ICG(KC),ICG(KC+1)-1
      K=JCG(J)
      JB(IB(K))=KC+N
      KCG(J)=IB(K)
      IB(K)=IB(K)+1
   12 CONTINUE
   13 CONTINUE
C
C     FINAL COMPUTATION OF THE POINTERS
C
      DO 14 I=N,1,-1
      IB(I+1)=IB(I)
   14 CONTINUE
      IB(1)=1
C
C     SUBDIAGONAL ELEMENTS OF THE MATRIX CG
C
      K=IB(N+1)
      DO 16 KC=1,NC
      IB(N+KC)=K
      DO 15 J=ICG(KC),ICG(KC+1)-1
      JB(K)=JCG(J)
      K=K+1
   15 CONTINUE
C
C     ZERO SUBMATRIX
C
      JB(K)=N+KC
      K=K+1
   16 CONTINUE
      IB(N+NC+1)=K
      RETURN
      END
