* SUBROUTINE UXSKIS                ALL SYSTEMS                96/12/01
C PORTABILITY : ALL SYSTEMS
C 96/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SYMBOLIC TRANSFORMATION OF THE PARTITIONED KARUSH-KUHN-TUCKER MATRIX
* INTO THE COMPLETE SYMMETRIC STRUCTURE.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  II  IH(N+1)  POINTERS OF THE HESSIAN MATRIX.
*  II  JH(M)  COLUMN INDICES OF THE HESSIAN MATRIX.
*  II  NC  NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  ICG(N+1)  POINTERS OF THE JACOBIAN MATRIX.
*  II  JCG(MC)  COLUMN INDICES OF THE JACOBIAN MATRIX.
*  IO  KCG(MC)  INDICES OF ELEMENTS OF THE ARRAY CG IN THE ARRAY B.
*  IO  IB(N+NC+1)  POINTERS OF THE INDEFINITE KARUSH-KUHN-TUCKER MATRIX.
*  IO  JB(M+MC+NC)  COLUMN INDICES OF THE INDEFINITE KARUSH-KUHN-TUCKER
*         MATRIX.
*
      SUBROUTINE UXSKIS(N,M,IH,JH,NC,MC,ICG,JCG,KCG,IB,JB)
      INTEGER N,M,IH(N+1),JH(M),NC,MC,ICG(NC+1),JCG(MC),KCG(MC),
     & IB(N+NC+1),JB(M+MC+NC)
      INTEGER I,J,K,KC
C
C     INITIAL COMPUTATION OF POINTERS
C
      IB(1)=1
      DO 1 I=1,N
      IB(I+1)=IH(I+1)-IH(I)
    1 CONTINUE
      DO 5 KC=1,NC
      DO 4 J=ICG(KC),ICG(KC+1)-1
      K=JCG(J)
      IB(K+1)=IB(K+1)+1
    4 CONTINUE
    5 CONTINUE
      DO 6 I=1,N
      IB(I+1)=IB(I+1)+IB(I)
    6 CONTINUE
C
C     ELEMENTS OF THE MATRIX H
C
      DO 11 I=1,N
      DO 10 J=IH(I),IH(I+1)-1
      JB(IB(I))=JH(J)
      IB(I)=IB(I)+1
   10 CONTINUE
   11 CONTINUE
C
C     ELEMENTS OF THE MATRIX CG
C
      DO 13 KC=1,NC
      DO 12 J=ICG(KC),ICG(KC+1)-1
      K=JCG(J)
      JB(IB(K))=KC+N
      KCG(J)=IB(K)
      IB(K)=IB(K)+1
   12 CONTINUE
   13 CONTINUE
C
C     FINAL COMPUTATION OF THE POINTERS
C
      DO 14 I=N,1,-1
      IB(I+1)=IB(I)
   14 CONTINUE
      IB(1)=1
C
C     ZERO SUBMATRIX
C
      K=IB(N+1)
      DO 15 KC=1,NC
      JB(K)=N+KC
      K=K+1
      IB(N+KC+1)=K
   15 CONTINUE
      RETURN
      END
