* SUBROUTINE UXSPCU                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* CORRECTION OF A SPARSE SYMMETRIC POSITIVE DEFINITE MATRIX A+E IN THE
* FACTORED FORM A+E=L*D*TRANS(L) OBTAINED BY THE SUBROUTINE UXSPCN
* THE CORRECTION IS DEFINED AS A+E:=A+E+ALF*X*TRANS(X) WHERE ALF IS A
* GIVEN SCALING FACTOR AND X IS A GIVEN VECTOR. FACTORS ARE STORED IN
* THE COMPACT FORM.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  MMAX  LENGTH OF THE PRINCIPAL MATRIX VECTORS (SL,A)
*  RI  A(MMAX)      FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE
*         SUBROUTINE UXSPGN.IT CONTAINS THE NUMERICAL VALUES OF THE
*         FACTORS STORED IN THE COMPACT FORM ACCORDING TO THE
*         INFORMATION IN THE VECTORS PSL,SL.
*  II  PSL(N+1)  POINTER VECTOR OF THE FACTORIZED MATRIX A
*  II  SL(MMAX)  COMPACT SHEME OF THE FACTORIZED MATRIX A.
*  RI  ALF  SCALING FACTOR IN THE CORRECTION TERM
*  RI  X(N)  VECTOR IN THE CORRECTION TERM
*  RA  Y(N) AUXILIARY VECTOR
*
* METHOD :
* P.E.GILL, W.MURRAY, M.SAUNDERS: METHODS FOR COMPUTING AND MODIFYING
* THE LDV FACTORS OF A MATRIX, MATH. OF COMP. 29 (1974) PP. 1051-1077.
* T.STEIHAUG:LOCAL AND SUPERLINEAR CONVERGENCE FOR TRUNCATED ITERATED
* PROJECTION METHODS,MATH. PROG. 27(1983) PP. 176-190.
*
      SUBROUTINE UXSPCU(N,MMAX,A,PSL,SL,ALF,X,Y)
      INTEGER  N,MMAX
      INTEGER PSL(N+1),SL(MMAX)
      REAL*8  A(MMAX),ALF,X(N),Y(N)
      REAL*8  B, D, P, R, T, TO
      INTEGER  I, J, IS
      REAL*8 ZERO, ONE, FOUR, CON
      PARAMETER  (ZERO = 0.0D0, ONE = 1.0D0, FOUR = 4.0D0, CON=1.0D-8)
      IF (ALF .GE. ZERO) THEN
C
C     FORWARD CORRECTION IN CASE WHEN THE SCALING FACTOR IS NONNEGATIVE
C
      ALF = SQRT(ALF)
      CALL UXVSCL(N, ALF, X, Y)
      TO = ONE
      DO 300 I=1,N
      D = A(PSL(I)+I-1)
      P = Y(I)
      T = TO + P*P/D
      R = TO / T
      A(PSL(I)+I-1)=D/R
      B = P / (D*T)
      IF (A(PSL(I)+I-1).LE.FOUR*D) THEN
C
C     AN EASY FORMULA FOR LIMITED DIAGONAL ELEMENT
C
      IS=SL(I)+N+1
      DO 100 J=PSL(I)+I,PSL(I+1)+I-1
      D = A(J)
      Y(SL(IS))=Y(SL(IS))-P*D
      A(J)=D+B*Y(SL(IS))
      IS=IS+1
100   CONTINUE
      ELSE
C
C     A MORE COMPLICATE BUT NUMERICALLY STABLE FORMULA FOR UNLIMITED
C     DIAGONAL ELEMENT
C
      IS=SL(I)+N+1
      DO 200 J=PSL(I)+I,PSL(I+1)+I-1
      D = A(J)
      A(J)=R*D+B*Y(SL(IS))
      Y(SL(IS))=Y(SL(IS))-P*D
      IS=IS+1
200   CONTINUE
      ENDIF
      TO = T
300   CONTINUE
      ELSE
C
C     BACKWARD CORRECTION IN CASE WHEN THE SCALING FACTOR IS NEGATIVE
C
      ALF = SQRT( -ALF)
      CALL UXVSCL(N, ALF, X, Y)
      TO = ONE
      DO 500 I=1,N
      D = Y(I)
      IS=SL(I)+N+1
C     Y(I)=Y(I)-A(PSL(I)+I-1)*D
      DO 400 J=PSL(I)+I,PSL(I+1)+I-1
      Y(SL(IS))=Y(SL(IS))-A(J)*D
      IS=IS+1
400   CONTINUE
      TO = TO - D*D/A(PSL(I)+I-1)
500   CONTINUE
      IF (TO .LE. ZERO)  TO = CON
      DO 700 I=N,1,-1
      D=A(PSL(I)+I-1)
      P = Y(I)
      T = TO + P*P/D
      A(PSL(I)+I-1)= D * TO / T
      B = -P / (D*TO)
      TO = T
      IS=SL(I)+N+1
      DO 600 J=PSL(I)+I,PSL(I+1)+I-1
      D = A(J)
      A(J) = D + B*Y(SL(IS))
      Y(SL(IS))=Y(SL(IS))+P*D
      IS=IS+1
600   CONTINUE
700   CONTINUE
      ENDIF
      RETURN
      END
