* SUBROUTINE UXSPJB                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A SPARSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A+E USING INCOMPLETE FACTORIZATION
* A+E=L*D*TRANS(L) OBTAINED BY THE SUBROUTINE UXSPJF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  M NUMBER OF NONZERO ELEMENTS OF THE MATRIX A.
*  RI  A(M) INCOMPLETE FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE
*         SUBROUTINE UXSPJF.
*  II  IA(N+1)  POINTERS OF THE DIAGONAL ELEMENTS OF A.
*  II  JA(M)  INDICES OF THE NONZERO ELEMENTS OF A.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 THEN X:=(A+E)**(-1)*X. IF JOB>0 THEN
*         X:=L**(-1)*X. IF JOB<0 THEN X:=TRANS(L)**(-1)*X.
*
* METHOD :
* BACK SUBSTITUTION
*
      SUBROUTINE UXSPJB(N,M,A,IA,JA,X,JOB)
      INTEGER N,M,IA(N+1),JA(M),JOB
      REAL*8 A(M),X(N)
      INTEGER I,K,IK,KK,LL
      REAL*8 ZERO,CON
      PARAMETER (ZERO=0.0D 0,CON=1.0D120)
      IF (JOB .GE. 0) THEN
C
C     PHASE 1 : X:=L**(-1)*X
C
      KK=1
      DO 3 K=1,N
      LL=IA(K+1)
      IF (JA(KK).LE.0) GO TO 2
      DO 1 IK=KK+1,LL-1
      I=JA(IK)
      IF (I.LE.0) GO TO 1
      X(I)=X(I)-A(IK)*X(K)
      IF (ABS(X(I)).GE.CON) X(I)=SIGN(CON,X(I))
    1 CONTINUE
    2 KK=LL
    3 CONTINUE
      ENDIF
      IF (JOB .EQ. 0) THEN
C
C     PHASE 2 : X:=D**(-1)*X
C
      DO 4 K=1,N
      IF (JA(IA(K)).LE.0) GO TO 4
      X(K)=X(K)/A(IA(K))
    4 CONTINUE
      ENDIF
      IF (JOB .LE. 0) THEN
C
C     PHASE 3 : X:=TRANS(L)**(-1)*X
C
      LL=IA(N+1)
      DO 7 K=N,1,-1
      KK=IA(K)
      IF (JA(KK).LE.0) GO TO 6
      DO 5 IK=KK+1,LL-1
      I=JA(IK)
      IF (I.LE.0) GO TO 5
      X(K)=X(K)-A(IK)*X(I)
      IF (ABS(X(K)).GE.CON) X(K)=SIGN(CON,X(K))
    5 CONTINUE
    6 LL=KK
    7 CONTINUE
      ENDIF
      RETURN
      END
