* SUBROUTINE UXSPTF                ALL SYSTEMS                03/12/01
C PORTABILITY : ALL SYSTEMS
C 03/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* INCOMPLETE CHOLESKY FACTORIZATION A+E=L*D*TRANS(L) OF A SPARSE
* SYMMETRIC POSITIVE DEFINITE MATRIX A+E WHERE D AND E ARE DIAGONAL
* POSITIVE DEFINITE MATRICES AND L IS A LOWER TRIANGULAR MATRIX.
* METHOD IS BASED ON THE SIMPLE IC(0) ALGORITHM WITHOUT DIAGONAL
* COMPENSATION. SPARSE RIGHT-LOOKING IMPLEMENTATION.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  M NUMBER OF NONZERO ELEMENTS OF THE MATRIX A.
*  RI  A(M)  SPARSE SYMMETRIC (USUALLY POSITIVE DEFINITE) MATRIX.
*  II  IA(N+1)  POINTERS OF THE DIAGONAL ELEMENTS OF A.
*  II  JA(M)  INDICES OF THE NONZERO ELEMENTS OF A.
*  IA  WN01(N+1)  AUXILIARY ARRAY.
*  IO  INF  AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS SUFFICIENTLY POSITIVE DEFINITE AND E=0. IF
*         INF<0 THEN A IS NOT SUFFICIENTLY POSITIVE DEFINITE AND E>0. IF
*         INF>0 THEN A IS INDEFINITE AND INF IS AN INDEX OF THE
*         MOST NEGATIVE DIAGONAL ELEMENT USED IN THE FACTORIZATION
*         PROCESS.
*  RU  ALF  ON INPUT A DESIRED TOLERANCE FOR POSITIVE DEFINITENESS.
*         ON OUTPUT THE MOST NEGATIVE DIAGONAL ELEMENT USED IN THE
*         FACTORIZATION PROCESS (IF INF>0).
*  RO  TAU  MAXIMUM DIAGONAL ELEMENT OF THE MATRIX E.
*
* METHOD :
* P.E.GILL, W.MURRAY : NEWTON TYPE METHODS FOR UNCONSTRAINED AND
* LINEARLY CONSTRAINED OPTIMIZATION, MATH. PROGRAMMING 28 (1974)
* PP. 311-350.
*
      SUBROUTINE UXSPTF(N,M,A,IA,JA,WN01,INF,ALF,TAU)
      INTEGER N,M,IA(N+1),JA(M),WN01(N+1),INF
      REAL*8 A(M),ALF,TAU
      INTEGER I,J,K,L,II,K1,L1,L2,JSTRT,JSTOP,IDIAG,KSTRT,KSTOP,NJA
      REAL*8 PTOL,BET,GAM,TEMP,DEL,DIAG,NDIAG,INVDIAG
      PTOL=ALF
      NJA=IA(N+1)-1
C
C     INITIALIZE AUXILIARY VECTOR
C
      INF=0
      CALL UXVINS(N,0,WN01)
C
C     GILL-MURRAY MODIFICATION
C
      ALF=0.0D 0
      BET=0.0D 0
      GAM=0.0D 0
      TAU=0.0D 0
      DO 2 I=1,N
        IDIAG=IA(I)
        IF (JA(IDIAG).LE.0) GO TO 2
        TEMP=A(IDIAG)
        BET=MAX(BET,ABS(TEMP))
        DO 1 J=IA(I)+1,IA(I+1)-1
          IF (JA(J).LE.0) GO TO 1
          TEMP=A(J)
          GAM=MAX(GAM,ABS(TEMP))
    1   CONTINUE
    2 CONTINUE
      BET=MAX(PTOL,BET,GAM/DBLE(N))
      DEL=PTOL*BET
C
C     COMPUTE THE PRECONDITIONER
C
      DO 8 K=1,N
        KSTRT=IA(K)
        KSTOP=IA(K+1)-1
        IF (JA(KSTRT).LE.0) GO TO 8
        DIAG=A(KSTRT)
        IF (ALF.GT.DIAG) THEN
          ALF=DIAG
          IF (INF.GE.0) INF=K
        ENDIF
        GAM=0.0D 0
        DO 3 J=KSTRT+1,KSTOP
          IF (JA(J).LE.0) GO TO 3
          TEMP=A(J)
          GAM=MAX(GAM,ABS(TEMP))
    3   CONTINUE
        GAM=GAM*GAM
        INVDIAG=MAX(ABS(DIAG),GAM/BET,DEL)
        IF (INVDIAG.EQ.DEL) INF=-1
        IF (TAU.LT.INVDIAG-DIAG) THEN
          TAU=INVDIAG-DIAG
        ENDIF
        INVDIAG=1.0D 0/INVDIAG
        A(KSTRT)=INVDIAG
C
C     RIGHT-LOOKING UPDATE
C
C
C     SET POINTERS
C
        DO 4 II=KSTRT,KSTOP
          K1=JA(II)
          IF (K1.GT.0) WN01(K1)=II
    4   CONTINUE
C
C     INNER LOOP
C
        DO 6 I=KSTRT+1,KSTOP
          J=JA(I)
          IF (J.LE.0) GO TO 6
          NDIAG=A(I)
          JSTRT=IA(J)
          JSTOP=IA(J+1)-1
          DO 5 L=JSTRT,JSTOP
            L1=JA(L)
            IF (L1.LE.0) GO TO 5
            L2=WN01(L1)
            IF(L2.NE.0) THEN
              A(L)=A(L)-(A(L2)*INVDIAG)*NDIAG
            END IF
    5     CONTINUE
    6   CONTINUE
C
C     CLEAR THE POINTERS
C
        DO 7 II=KSTRT,KSTOP
          K1=JA(II)
          IF (K1.GT.0) WN01(K1)=0
    7   CONTINUE
    8 CONTINUE
      RETURN
      END
