* SUBROUTINE  UXSRSP               ALL SYSTEMS               95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 LU : ORIGINAL VERSION
*
* PURPOSE : CREATE ROW PERMUTATIONS FOR OBTAINING DIAGONAL NONZEROS.
*
*  PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX.
*  II  M  NUMBER OF NONZEROS MEMBERS IN THE MATRIX.
*  II  IA(M+1)  ROW POINTERS OF THE SPARSE MATRIX.
*  II  JA(M)  COLUMN INDICES OF THE SPARSE MATRIX.
*  IO  IP(N)  PERMUTATION VECTOR.
*  II  NR  NUMBER OF STRUCTURALLY INDEPENDENT ROWS.
*  IA  IW1(N) AUXILIARY VECTOR.
*  IA  IW2(N) AUXILIARY VECTOR.
*  IA  IW3(N) AUXILIARY VECTOR.
*  IA  IW4(N) AUXILIARY VECTOR.
*
      SUBROUTINE UXSRSP(N,M,IA,JA,IP,NR,IW1,IW2,IW3,IW4)
      INTEGER M,N,NR
      INTEGER IA(N+1),JA(M),IP(N),IW1(N),IW2(N),IW3(N),IW4(N)
      INTEGER I,II,I1,I2,J,J1,K,KK,L
      DO 10 I=1,N
      IW2(I)=IA(I+1)-IA(I)-1
      IW3(I)=0
      IP(I)=0
   10 CONTINUE
      NR=0
C
C     MAIN LOOP.
C     EACH PASS ROUND THIS LOOP EITHER RESULTS IN A NEW ASSIGNMENT
C     OR GIVES A ROW WITH NO ASSIGNMENT.
C
      DO 100 L=1,N
      J=L
      IW1(J)=-1
      DO 70 K=1,L
C
C     LOOK FOR A CHEAP ASSIGNMENT
C
      I1=IW2(J)
      IF (I1.LT.0) GO TO 30
      I2=IA(J+1)-1
      I1=I2-I1
      DO 20 II=I1,I2
      I=JA(II)
      IF (IP(I).EQ.0) GO TO 80
   20 CONTINUE
C
C     NO CHEAP ASSIGNMENT IN ROW.
C
      IW2(J)=-1
C
C     BEGIN LOOKING FOR ASSIGNMENT CHAIN STARTING WITH ROW J.
C
   30 CONTINUE
      IW4(J)=IA(J+1)-IA(J)-1
C
C     INNER LOOP.  EXTENDS CHAIN BY ONE OR BACKTRACKS.
C
      DO 60 KK=1,L
      I1=IW4(J)
      IF (I1.LT.0) GO TO 50
      I2=IA(J+1)-1
      I1=I2-I1
C
C     FORWARD SCAN.
C
      DO 40 II=I1,I2
      I=JA(II)
      IF (IW3(I).EQ.L) GO TO 40
C
C     COLUMN I HAS NOT YET BEEN ACCESSED DURING THIS PASS.
C
      J1=J
      J=IP(I)
      IW3(I)=L
      IW1(J)=J1
      IW4(J1)=I2-II-1
      GO TO 70
   40 CONTINUE
C
C     BACKTRACKING STEP.
C
   50 CONTINUE
      J=IW1(J)
      IF (J.EQ.-1) GO TO 100
   60 CONTINUE
   70 CONTINUE
C
C     NEW ASSIGNMENT IS MADE.
C
   80 CONTINUE
      IP(I)=J
      IW2(J)=I2-II-1
      NR=NR+1
      DO 90 K=1,L
      J=IW1(J)
      IF (J.EQ.-1) GO TO 100
      II=IA(J+1)-IW4(J)-2
      I=JA(II)
      IP(I)=J
   90 CONTINUE
  100 CONTINUE
C
C     IF MATRIX IS STRUCTURALLY SINGULAR, WE NOW COMPLETE THE
C     PERMUTATION IP.
C
      IF (NR.EQ.N) RETURN
      DO 110 I=1,N
      IW2(I)=0
  110 CONTINUE
      K=0
      DO 130 I=1,N
      IF (IP(I).NE.0) GO TO 120
      K=K+1
      IW4(K)=I
      GO TO 130
  120 CONTINUE
      J=IP(I)
      IW2(J)=I
  130 CONTINUE
      K=0
      DO 140 I=1,N
      IF (IW2(I).NE.0) GO TO 140
      K=K+1
      L=IW4(K)
      IP(L)=I
  140 CONTINUE
      RETURN
      END
