* SUBROUTINE UXVZMZ             ALL SYSTEMS                 02/04/05
C PORTABILITY : ALL SYSTEMS
C 02/04/05 VL : ORIGINAL VERSION
*
* PURPOSE :
*   DERIVATION OF MATRIX TRANS(Z)*M*Z USED IN THE RANK TWO LIMITED
*   MEMORY METHOD FOLLOWED BY COMPUTATION OF ITS EIGENVALUES AND
*   EIGENVECTORS BY THE JACOBI METHOD.
*
* PARAMETERS :
*  II  N  SIZE OF THE VECTORS IN XS.
*  IO  K  NUMBER OF ORTHONORMAL VECTORS IN THE ARRAY XS (1-3).
*  RI  XS(N*3)  ARRAY CONTAINING K ORTHONORMAL VECTORS.
*  RU  QQ(15)  WORKSPACE ARRAY.
*  RI  XO  DIFFERENCE OF TWO VECTORS OF VARIABLES (STEP).
*  RI  GS  VECTOR EQUAL TO XM*TRANS(XM)*GO.
*  RI  S  VECTOR EQUAL TO (ZET*I+XM*TRANS(XM))**(-1)*XO.
*  RI  ZET  OLD VALUE OF THE SHIFT PARAMETER.
*  RI  SIG  NEW VALUE OF THE SHIFT PARAMETER.
*  RI  A  VALUE EQUAL TO TRANS(GO)*(ZET*I+XM*TRANS(XM))*GO.
*  RI  B  VALUE EQUAL TO TRANS(XO)*GO.
*  RI  AH  VALUE EQUAL TO TRANS(GO)*GO.
*  RI  AA  VALUE EQUAL TO TRANS(GO)*XM*TRANS(XM)*GO.
*  RI  BB  VALUE EQUAL TO TRANS(XO)*(ZET*I+XM*TRANS(XM))**(-1)*
*         XM*TRANS(XM)*GO.
*  RI  CC  VALUE EQUAL TO TRANS(XO)*(ZET*I+XM*TRANS(XM))**(-1)*
*         XM*TRANS(XM)*(ZET*I+XM*TRANS(XM))**(-1)*XO.
*  RI  GAM  SCALING PARAMETER.
*  RI  RHO  PARAMETER FOR NONQUADRATIC CORRECTION.
*
* SUBPROGRAMS USED :
*  S   UXVJAC  JACOBI METHOD FOR 3*3 SYMMETRIC MATRICES.
*
      SUBROUTINE UXVZMZ(N,K,XS,QQ,XO,GS,S,ZET,SIG,A,B,AA,BB,AH,POM,
     & PAR,DEN)
      INTEGER N,K
      REAL*8 XS(N*3),QQ(15),XO(N),GS(N),S(N),ZET,SIG,A,B,AA,BB,AH,
     &  POM,PAR,DEN
      REAL*8 W1,W2,W3,W4,W5,W6,W7,W8,W9,UXVDOT
      INTEGER I,J,L
      IF (K.GE.1) THEN
      W1=UXVDOT(N,XS,XO)
      W2=UXVDOT(N,XS,GS)
      W3=UXVDOT(N,XS,S)
      QQ(1)=ZET-PAR*SIG*(1.0D 0+(AH/B)*W1*W1/(B-SIG*AH))+
     & ZET*(AH/A)*W2*W2/AA+(POM/A)*((A/B)*W1-W2)**2+
     & (AA/DEN)*(W1-(BB/AA)*W2-ZET*W3)**2
      IF (K.GE.2) THEN
      W4=UXVDOT(N,XS(1+N),XO)
      W5=UXVDOT(N,XS(1+N),GS)
      W6=UXVDOT(N,XS(1+N),S)
      QQ(2)=ZET-PAR*SIG*(1.0D 0+(AH/B)*W4*W4/(B-SIG*AH))+
     & ZET*(AH/A)*W5*W5/AA+(POM/A)*((A/B)*W4-W5)**2+
     & (AA/DEN)*(W4-(BB/AA)*W5-ZET*W6)**2
      QQ(4)=-PAR*SIG*(AH/B)*W1*W4/(B-SIG*AH)+
     & ZET*(AH/A)*W2*W5/AA+(POM/A)*((A/B)*W1-W2)*((A/B)*W4-W5)+
     & (AA/DEN)*(W1-(BB/AA)*W2-ZET*W3)*(W4-(BB/AA)*W5-ZET*W6)
      IF (K.GE.3) THEN
      W7=UXVDOT(N,XS(1+N+N),XO)
      W8=UXVDOT(N,XS(1+N+N),GS)
      W9=UXVDOT(N,XS(1+N+N),S)
      QQ(3)=ZET-PAR*SIG*(1.0D 0+(AH/B)*W7*W7/(B-SIG*AH))+
     & ZET*(AH/A)*W8*W8/AA+(POM/A)*((A/B)*W7-W8)**2+
     & (AA/DEN)*(W7-(BB/AA)*W8-ZET*W9)**2
      QQ(5)=-PAR*SIG*(AH/B)*W1*W7/(B-SIG*AH)+
     & ZET*(AH/A)*W2*W8/AA+(POM/A)*((A/B)*W1-W2)*((A/B)*W7-W8)+
     & (AA/DEN)*(W1-(BB/AA)*W2-ZET*W3)*(W7-(BB/AA)*W8-ZET*W9)
      QQ(6)=-PAR*SIG*(AH/B)*W4*W7/(B-SIG*AH)+
     & ZET*(AH/A)*W5*W8/AA+(POM/A)*((A/B)*W4-W5)*((A/B)*W7-W8)+
     & (AA/DEN)*(W4-(BB/AA)*W5-ZET*W6)*(W7-(BB/AA)*W8-ZET*W9)
      ENDIF
      ENDIF
      ENDIF
      L=50
      CALL UXVJAC(K,QQ,L)
      J=1
      W1=QQ(1)
      DO 13 I=2,K
      IF (QQ(I).GT.W1) THEN
      J=I
      W1=QQ(I)
      ENDIF
   13 CONTINUE
      IF (W1.LE.0.0D 0) QQ(J)=0.0D 0
      DO 14 I=1,K
      QQ(6+I)=QQ(3+3*J+I)*SQRT(QQ(J))
   14 CONTINUE
      RETURN
      END
