* SUBROUTINE UYADU1                ALL SYSTEMS                04/12/01
C PORTABILITY : ALL SYSTEMS
C 04/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SUBGRADIENT AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  HH(N*(N+1)/2)  GILL-MURRAY DECOMPOSITION OF THE AGGREGATED
*         HESSIAN MATRIX.
*  RI  G(N)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GO(N)  PREVIOUS SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(N)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  H(N*(N+1)/2)  HESSIAN MATRIX OF THE OBJECTIVE FUNCTION.
*  RI  HO(N*(N+1)/2)  PREVIOUS HESSIAN MATRIX OF THE OBJECTIVE
*         FUNCTION.
*  RI  HV(N*(N+1)/2)  AGGREGATED HESSIAN MATRIX OF THE OBJECTIVE
*         FUNCTION.
*  RI  S(N)  DIRECTION VECTOR.
*  RA  U(N)  AUXILIARY VECTOR.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYADU1(N,HH,G,GO,GV,H,HO,HV,S,U,ALF,ALFV)
      INTEGER N
      REAL*8 HH(N*(N+1)/2),G(N),GO(N),GV(N),H(N*(N+1)/2),HO(N*(N+1)/2),
     & HV(N*(N+1)/2),S(N),U(N),ALF,ALFV
      REAL*8 A,B,ALFM,LAM1,LAM2,PQ,PR,PRQR,QQP,QR,RR,RRP,RRQ,W,W1
      INTEGER I
      REAL*8 ZERO,ONE,UXVDOT
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      ALFM=ZERO
C
C     General routine - here always input parameter ALFM=0
C
      RR=ALFV+ALFV
      RRP=ALFV-ALFM
      RRQ=ALFV-ALF
      DO 1 I=1,N
        A=S(I)
        U(I)=GO(I)-GV(I)
        S(I)=G(I)-GV(I)
        RR=RR-A*GV(I)
        RRP=RRP+A*U(I)
        RRQ=RRQ+A*S(I)
    1 CONTINUE
      PQ=ZERO
      PR=ZERO
      QR=ZERO
      QQP=ALF-ALFM
      PRQR=ZERO
      CALL UXDPGB(N,HH,U,1)
      CALL UXDPGB(N,HH,S,1)
      DO 4 I=1,N
        W1=ONE/H(I*(I+1)/2)
        PR=PR+W1*U(I)**2
        PRQR=PRQR+W1*U(I)*S(I)
        QR=QR+W1*S(I)**2
        PQ=PQ+W1*(S(I)-U(I))**2
C        QQP=QQP+W1*G(I)*(S(I)-U(I))
        S(I)=W1*(S(I)-U(I))
    4 CONTINUE
      CALL UXDPGB(N,HH,S,-1)
      QQP=QQP+UXVDOT(N,G,S)
      IF (PR.LE.ZERO.OR.QR.LE.ZERO) GO TO 10
      A=RRQ/QR
      B=PRQR/QR
      W=PRQR*B-PR
      IF (W.EQ.ZERO) GO TO 10
      LAM1=(A*PRQR-RRP)/W
      LAM2=A-LAM1*B
      IF (LAM1*(LAM1-ONE).LT.ZERO.AND.LAM2*(LAM1+LAM2-ONE).LT.ZERO)
     & GO TO 40
C
C     MINIMUM ON THE BOUNDARY
C
   10 LAM1=ZERO
      LAM2=ZERO
      IF (ALF.LE.ALFV) LAM2=ONE
      IF (QR.GT.ZERO) LAM2=MIN(ONE,MAX(ZERO,RRQ/QR))
      W=(LAM2*QR-RRQ-RRQ)*LAM2
      A=ZERO
      IF (ALFM.LE.ALFV) A=ONE
      IF (PR.GT.ZERO) A=MIN(ONE,MAX(ZERO,RRP/PR))
      B=(A*PR-RRP-RRP)*A
      IF(B.LT.W)THEN
        W=B
        LAM1=A
        LAM2=ZERO
      ENDIF
      IF (QQP*(QQP-PQ).GE.ZERO) GO TO 40
      IF (QR-RRQ-RRQ-QQP*QQP/PQ.GE.W) GO TO 40
      LAM1=QQP/PQ
      LAM2=ONE-LAM1
   40 IF (LAM1.EQ.ZERO.AND.LAM2*(LAM2-ONE).LT.ZERO.AND.RRP-LAM2*PRQR
     & .GT.ZERO.AND.PR.GT.ZERO) LAM1=MIN(ONE-LAM2,(RRP-LAM2*PRQR)/PR)
      A=ONE-LAM1-LAM2
      ALFV=LAM1*ALFM+LAM2*ALF+A*ALFV
      DO 5 I=1,N
        GV(I)=LAM1*GO(I)+LAM2*G(I)+A*GV(I)
    5 CONTINUE
      DO 6 I=1,N*(N+1)/2
        HV(I)=LAM1*HO(I)+LAM2*H(I)+A*HV(I)
    6 CONTINUE
      RETURN
      END
