* SUBROUTINE UYAGB1                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SUBGRADIENT AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  H(N*(N+1)/2)  POSITIVE DEFINITE APPROXIMATION OF THE INVERSE
*         HESSIAN MATRIX.
*  RI  G(NF)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GO(NF)  PREVIOUS SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(NF)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GN(N)  REDUCED AGGREGATED SUBGRADIENT.
*  RI  SN(N)  REDUCED DIRECTION VECTOR.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE.
*  RI  S(N)  REDUCED SUBGRADIENT.
*  RI  U(N)  PREVIOUS REDUCED SUBGRADIENT.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYAGB1(NF,N,H,G,GO,GV,GN,SN,CZ,S,U,ALF,ALFV)
      INTEGER NF,N
      REAL*8 H(N*(N+1)/2),G(NF),GO(NF),GV(NF),GN(N),SN(N),CZ(NF*NF),
     & S(N),U(N),ALF,ALFV
      REAL*8 A,B,ALFM,LAM1,LAM2,PQ,PR,PRQR,QQP,QR,RR,RRP,RRQ,W,W1,W2
      INTEGER I,J,L
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      ALFM=ZERO
C
C     General routine - here always input parameter ALFM=0
C
      CALL UXDRMM(NF,N,CZ,G,S)
      CALL UXDRMM(NF,N,CZ,GO,U)
      RR=ALFV+ALFV
      RRP=ALFV-ALFM
      RRQ=ALFV-ALF
      DO 1 I=1,N
        RR=RR-SN(I)*GN(I)
        U(I)=U(I)-GN(I)
        GN(I)=S(I)-GN(I)
        RRP=RRP+SN(I)*U(I)
    1   RRQ=RRQ+SN(I)*GN(I)
      PQ=ZERO
      PR=ZERO
      QR=ZERO
      QQP=ALF-ALFM
      PRQR=ZERO
      DO 4 I=1,N
        L=I*(I-1)/2+1
        W1=ZERO
        W2=ZERO
        DO 2 J=1,I-1
          W=H(L)
          W1=W1+W*GN(J)
          W2=W2+W*U(J)
    2     L=L+1
        DO 3 J=I,N
          W=H(L)
          W1=W1+W*GN(J)
          W2=W2+W*U(J)
    3     L=L+J
        PR=PR+U(I)*W2
        QR=QR+GN(I)*W1
        PQ=PQ+(GN(I)-U(I))*(W1-W2)
        QQP=QQP+S(I)*(W1-W2)
    4   PRQR=PRQR+U(I)*W1
      IF(PR.LE.ZERO.OR.QR.LE.ZERO)GOTO10
      A=RRQ/QR
      B=PRQR/QR
      W=PRQR*B-PR
      IF(W.EQ.ZERO)GOTO10
      LAM1=(A*PRQR-RRP)/W
      LAM2=A-LAM1*B
      IF(LAM1*(LAM1-ONE).LT.ZERO.AND.LAM2*(LAM1+LAM2-ONE).LT.ZERO)
     & GOTO40
C
C     MINIMUM ON THE BOUNDARY
C
   10 LAM1=ZERO
      LAM2=ZERO
      IF(ALF.LE.ALFV)LAM2=ONE
      IF(QR.GT.ZERO)LAM2=MIN(ONE,MAX(ZERO,RRQ/QR))
      W=(LAM2*QR-RRQ-RRQ)*LAM2
      A=ZERO
      IF(ALFM.LE.ALFV)A=ONE
      IF(PR.GT.ZERO)A=MIN(ONE,MAX(ZERO,RRP/PR))
      B=(A*PR-RRP-RRP)*A
      IF(B.LT.W)THEN
        W=B
        LAM1=A
        LAM2=ZERO
      ENDIF
      IF(QQP*(QQP-PQ).GE.ZERO)GOTO40
      IF(QR-RRQ-RRQ-QQP*QQP/PQ.GE.W)GOTO40
      LAM1=QQP/PQ
      LAM2=ONE-LAM1
   40 IF(LAM1.EQ.ZERO.AND.LAM2*(LAM2-ONE).LT.ZERO.AND.RRP-LAM2*PRQR
     &.GT.ZERO.AND.PR.GT.ZERO)LAM1=MIN(ONE-LAM2,(RRP-LAM2*PRQR)/PR)
      A=ONE-LAM1-LAM2
      B=ONE-LAM2
      DO 5 I=1,N
    5   GN(I)=S(I)+LAM1*U(I)-B*GN(I)
      DO 6 I=1,NF
    6   GV(I)=LAM1*GO(I)+LAM2*G(I)+A*GV(I)
      ALFV=LAM1*ALFM+LAM2*ALF+A*ALFV
      RETURN
      END
