* SUBROUTINE UYAGS2                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SIMPLIFIED AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  H(N*(N+1)/2)  POSITIVE DEFINITE APPROXIMATION OF THE INVERSE
*         HESSIAN MATRIX.
*  RI  G(NF)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(NF)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GN(NF)  REDUCED AGGREGATED SUBGRADIENT.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYAGS2(NF,N,IX,H,G,GV,GN,S,ALF,ALFV)
      INTEGER NF,N,IX(NF)
      REAL*8 H(N*(N+1)/2),G(NF),GV(NF),GN(N),S(NF),ALF,ALFV
      REAL*8 LAM,P,Q,W
      INTEGER I,J,K,L
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      P=ALFV-ALF
      K=0
      DO 1 I=1,NF
        IF (IX(I).GE.0) THEN
          K=K+1
          Q=G(I)-GV(I)
          P=P+S(I)*Q
          S(K)=Q
        ENDIF
    1 CONTINUE
      Q=ZERO
      K=0
      DO 4 I=1,NF
        IF (IX(I).GE.0) THEN
          K=K+1
          L=K*(K-1)/2+1
          W=ZERO
          DO 2 J=1,K-1
            W=W+H(L)*S(J)
    2       L=L+1
          DO 3 J=K,N
            W=W+H(L)*S(J)
    3       L=L+J
          Q=Q+W*S(K)
        ENDIF
    4 CONTINUE
      LAM=0.5D 0+SIGN(0.5D 0,P)
      IF(Q.GT.ZERO)LAM=MIN(ONE,MAX(ZERO,P/Q))
      P=ONE-LAM
      K=0
      DO 5 I=1,NF
        GV(I)=LAM*G(I)+P*GV(I)
        IF (IX(I).GE.0) THEN
          K=K+1
          GN(K)=GV(I)
        ENDIF
    5 CONTINUE
      ALFV=LAM*ALF+P*ALFV
      RETURN
      END
