* SUBROUTINE UYAGU1                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SUBGRADIENT AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  H(N*(N+1)/2)  POSITIVE DEFINITE APPROXIMATION OF THE INVERSE
*         HESSIAN MATRIX.
*  RI  G(N)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GO(N)  PREVIOUS SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(N)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(N)  DIRECTION VECTOR.
*  RA  U(N)  AUXILIARY VECTOR.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYAGU1(N,H,G,GO,GV,S,U,ALF,ALFV)
      INTEGER N
      REAL*8 H(N*(N+1)/2),G(N),GO(N),GV(N),S(N),U(N),ALF,ALFV
      REAL*8 A,B,ALFM,LAM1,LAM2,PQ,PR,PRQR,QQP,QR,RR,RRP,RRQ,W,W1,W2
      INTEGER I,J,L
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      ALFM=ZERO
C
C     General routine - here always input parameter ALFM=0
C
      RR=ALFV+ALFV
      RRP=ALFV-ALFM
      RRQ=ALFV-ALF
      DO 1 I=1,N
        RR=RR-S(I)*GV(I)
        U(I)=GO(I)-GV(I)
        GV(I)=G(I)-GV(I)
        RRP=RRP+S(I)*U(I)
        RRQ=RRQ+S(I)*GV(I)
    1 CONTINUE
      PQ=ZERO
      PR=ZERO
      QR=ZERO
      QQP=ALF-ALFM
      PRQR=ZERO
      DO 4 I=1,N
        L=I*(I-1)/2+1
        W1=ZERO
        W2=ZERO
        DO 2 J=1,I-1
          W=H(L)
          W1=W1+W*GV(J)
          W2=W2+W*U(J)
          L=L+1
    2  CONTINUE
        DO 3 J=I,N
          W=H(L)
          W1=W1+W*GV(J)
          W2=W2+W*U(J)
          L=L+J
    3   CONTINUE
        PR=PR+U(I)*W2
        QR=QR+GV(I)*W1
        PRQR=PRQR+U(I)*W1
        PQ=PQ+(GV(I)-U(I))*(W1-W2)
        QQP=QQP+G(I)*(W1-W2)
    4 CONTINUE
      IF (PR.LE.ZERO.OR.QR.LE.ZERO) GO TO 10
      A=RRQ/QR
      B=PRQR/QR
      W=PRQR*B-PR
      IF (W.EQ.ZERO) GO TO 10
      LAM1=(A*PRQR-RRP)/W
      LAM2=A-LAM1*B
      IF (LAM1*(LAM1-ONE).LT.ZERO.AND.LAM2*(LAM1+LAM2-ONE).LT.ZERO)
     & GO TO 40
C
C     MINIMUM ON THE BOUNDARY
C
   10 LAM1=ZERO
      LAM2=ZERO
      IF (ALF.LE.ALFV) LAM2=ONE
      IF (QR.GT.ZERO) LAM2=MIN(ONE,MAX(ZERO,RRQ/QR))
      W=(LAM2*QR-RRQ-RRQ)*LAM2
      A=ZERO
      IF (ALFM.LE.ALFV) A=ONE
      IF (PR.GT.ZERO) A=MIN(ONE,MAX(ZERO,RRP/PR))
      B=(A*PR-RRP-RRP)*A
      IF(B.LT.W)THEN
        W=B
        LAM1=A
        LAM2=ZERO
      ENDIF
      IF (QQP*(QQP-PQ).GE.ZERO) GO TO 40
      IF (QR-RRQ-RRQ-QQP*QQP/PQ.GE.W) GO TO 40
      LAM1=QQP/PQ
      LAM2=ONE-LAM1
   40 IF (LAM1.EQ.ZERO.AND.LAM2*(LAM2-ONE).LT.ZERO.AND.RRP-LAM2*PRQR
     & .GT.ZERO.AND.PR.GT.ZERO) LAM1=MIN(ONE-LAM2,(RRP-LAM2*PRQR)/PR)
      A=ONE-LAM1-LAM2
      DO 5 I=1,N
        GV(I)=LAM1*GO(I)+(ONE-LAM1)*G(I)-A*GV(I)
    5 CONTINUE
      ALFV=LAM1*ALFM+LAM2*ALF+A*ALFV
      RETURN
      END
