* SUBROUTINE UYAHU1                ALL SYSTEMS                05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SUBGRADIENT AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  M  NUMBER OF COLUMNS OF XM.
*  RI  XM(N*M)  RECTANGULAR MATRIX IN THE PRODUCT FORM SHIFTED BROYDEN
*         METHOD (STORED COLUMNWISE): H-SIGMA*I=XM*TRANS(XM)
*  RA  GR(M)  AUXILIARY VECTOR.
*  RI  G(N)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  GO(N)  PREVIOUS SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(N)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(N)  DIRECTION VECTOR.
*  RA  U(N)  AUXILIARY VECTOR.
*  RI  PAR  SHIFT PARAMETER.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYAHU1(N,M,M1,XM,GM,XM1,GM1,GR,HR,HR1,
     & G,GO,GV,S,U,V1,V2,U1,U2,PAR,ALF,ALFV,RHO,JC)
      INTEGER N,M,M1,JC
      REAL*8 XM(N*M),GM(N*M),XM1(N*M1),GM1(N*M1),GR(M*(M+1)/2),
     & HR(M*(M+1)/2),HR1(M1*(M1+1)/2),G(N),GO(N),GV(N),S(N),U(N),
     & V1(N),V2(N),U1(N),U2(N),PAR,ALF,ALFV,RHO
      REAL*8 A,B,ALFM,LAM1,LAM2,PQ,PR,PRQR,QQP,QR,RR,RRP,RRQ,W
      INTEGER I
      REAL*8 ZERO,ONE
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      ALFM=ZERO
C
C     General routine - here always input parameter ALFM=0
C
      RR=ALFV+ALFV
      RRP=ALFV-ALFM
      RRQ=ALFV-ALF
      DO 1 I=1,N
        A=S(I)
        U(I)=GO(I)-GV(I)
        S(I)=G(I)-GV(I)
        RR=RR-A*GV(I)
        RRP=RRP+A*U(I)
        RRQ=RRQ+A*S(I)
    1 CONTINUE
      PQ=ZERO
      PR=ZERO
      QR=ZERO
      PRQR=ZERO
      QQP=ZERO
      IF (JC.GE.1) THEN
      DO 2 I=1,N
      PQ=PQ+RHO*(S(I)-U(I))**2
      PR=PR+RHO*U(I)**2
      QR=QR+RHO*S(I)**2
      PRQR=PRQR+RHO*U(I)*S(I)
      QQP=QQP+RHO+G(I)*(S(I)-U(I))
    2 CONTINUE
      ENDIF
      QQP=QQP+ALF-ALFM
      CALL UXLSIN(N,M,M1,XM,GM,GR,HR,XM1,GM1,HR1,U1,U2,U,V1,PAR)
      CALL UXLSIN(N,M,M1,XM,GM,GR,HR,XM1,GM1,HR1,U1,U2,S,V2,PAR)
      DO 3 I=1,N
        PR=PR+U(I)*V1(I)
        QR=QR+S(I)*V2(I)
        PRQR=PRQR+U(I)*V2(I)
        PQ=PQ+(S(I)-U(I))*(V2(I)-V1(I))
        QQP=QQP+G(I)*(V2(I)-V1(I))
    3 CONTINUE
      IF (PR.LE.ZERO.OR.QR.LE.ZERO) GO TO 10
      A=RRQ/QR
      B=PRQR/QR
      W=PRQR*B-PR
      IF (W.EQ.ZERO) GO TO 10
      LAM1=(A*PRQR-RRP)/W
      LAM2=A-LAM1*B
      IF (LAM1*(LAM1-ONE).LT.ZERO.AND.LAM2*(LAM1+LAM2-ONE).LT.ZERO)
     & GO TO 40
C
C     MINIMUM ON THE BOUNDARY
C
   10 LAM1=ZERO
      LAM2=ZERO
      IF (ALF.LE.ALFV) LAM2=ONE
      IF (QR.GT.ZERO) LAM2=MIN(ONE,MAX(ZERO,RRQ/QR))
      W=(LAM2*QR-RRQ-RRQ)*LAM2
      A=ZERO
      IF (ALFM.LE.ALFV) A=ONE
      IF (PR.GT.ZERO) A=MIN(ONE,MAX(ZERO,RRP/PR))
      B=(A*PR-RRP-RRP)*A
      IF(B.LT.W)THEN
        W=B
        LAM1=A
        LAM2=ZERO
      ENDIF
      IF (QQP*(QQP-PQ).GE.ZERO) GO TO 40
      IF (QR-RRQ-RRQ-QQP*QQP/PQ.GE.W) GO TO 40
      LAM1=QQP/PQ
      LAM2=ONE-LAM1
   40 IF (LAM1.EQ.ZERO.AND.LAM2*(LAM2-ONE).LT.ZERO.AND.RRP-LAM2*PRQR
     & .GT.ZERO.AND.PR.GT.ZERO) LAM1=MIN(ONE-LAM2,(RRP-LAM2*PRQR)/PR)
      A=ONE-LAM1-LAM2
      ALFV=LAM1*ALFM+LAM2*ALF+A*ALFV
      DO 5 I=1,N
        GV(I)=LAM1*GO(I)+LAM2*G(I)+A*GV(I)
    5 CONTINUE
      RETURN
      END
