* SUBROUTINE UYAHU2                ALL SYSTEMS                05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SIMPLIFIED AGGREGATION FOR NONSMOOTH VARIABLE METRIC METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  M  NUMBER OF COLUMNS OF XM.
*  RI  XM(N*M)  RECTANGULAR MATRIX IN THE PRODUCT FORM SHIFTED BROYDEN
*         METHOD (STORED COLUMNWISE): H-SIGMA*I=XM*TRANS(XM)
*  RA  GR(M)  AUXILIARY VECTOR.
*  RI  G(N)  SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GV(N)  AGGREGATED SUBGRADIENT OF THE OBJECTIVE FUNCTION.
*  RA  S(N)  AUXILIARY VECTOR.
*  RI  PAR  SHIFT PARAMETER.
*  RO  ALF  LINEARIZATION TERM.
*  RU  ALFV  AGGREGATED LINEARIZATION TERM.
*
      SUBROUTINE UYAHU2(N,M,M1,XM,GM,XM1,GM1,GR,HR,HR1,G,GV,S,V,
     & U1,U2,PAR,ALF,ALFV,RHO,JC)
      INTEGER N,M,M1,JC
      REAL*8 XM(N*M),GM(N*M),XM1(N*M1),GM1(N*M1),GR(M*(M+1)/2),
     & HR(M*(M+1)/2),HR1(M1*(M1+1)/2),G(N),GV(N),S(N),V(N),
     & U1(N),U2(N),PAR,ALF,ALFV,RHO
      REAL*8 LAM,P,Q,W
      INTEGER I
      REAL*8 ZERO,ONE,UXVDOT
      PARAMETER (ZERO=0.0D 0,ONE=1.0D 0)
      P=ALFV-ALF
      DO 1 I=1,N
      W=S(I)
      P=P+W*S(I)
      S(I)=G(I)-GV(I)
    1 CONTINUE
      Q=ZERO
      IF (JC.GE.1) THEN
      DO 2 I=1,N
      Q=Q+RHO*S(I)**2
    2 CONTINUE
      ENDIF
      CALL UXLSIN(N,M,M1,XM,GM,GR,HR,XM1,GM1,HR1,U1,U2,S,V,PAR)
      Q=Q+UXVDOT(N,S,V)
      LAM=0.5D 0+SIGN(0.5D 0,P)
      IF (Q.GT.ZERO) LAM=MIN(ONE,MAX(ZERO,P/Q))
      P=ONE-LAM
      ALFV=LAM*ALF+P*ALFV
      DO 5 I=1,N
      GV(I)=LAM*G(I)+P*GV(I)
    5 CONTINUE
      RETURN
      END
