$INTERFACE UA0GS2                                             97/12/01

  $HELP
    PRECISION OF THE COMPUTED VALUES
  $CHECK(ETA1,ETA0,REAL,4)
  CALL UA0GS2(NF,NA,NAL,$X,$IX,$XN,$AG,$AF,$AFO,ONE/TWO,$ETA1,KBF,NORMF)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$AG($NF*$NA)\$AF($NA)\$AFO($NA)')
  $ADD(INTEGER,'\$IX($NX)')
$REM  $ADD(INTEGER,'\NF\NA\NAL\KBF\NORMF')
  $ADD(INIT)
    NAGX='UA0GS2'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UA0GS2             ALL SYSTEMS                 97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* NUMERICAL COMPUTATION OF THE DENSE JACOBIAN MATRIX.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  II  NA  NUMBER OF LINEAR FUNCTIONS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  AG(NF*NA)  DENSE JACOBIAN MATRIX.
*  RI  AF(NA)  VECTOR OF FUNCTION VALUES.
*  RA  AFO(NA)  AUXILIARY VECTOR.
*  RI  PREC  PARAMETER FOR THE DIFFERENCE DETERMINATION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*  II  KB  TYPE OF BOUNDS. KB=0-BOUNDS ARE NOT USED. KB=1-ONE SIDED
*         BOUNDS. KB=1-TWO SIDED BOUNDS.
*  II  NORM  SCALING SPECIFICATION. NORM=0-NO SCALING IS PERFORMED.
*         NORM=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORM=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UA0GS2(NF,NA,NAL,X,IX,XN,AG,AF,AFO,PREC,ETA1,KB,NORM)
      $INCLUDE('UMCOMM')
      INTEGER NF,NA,NAL,IX(NF),KB,NORM
      $FLOAT X(NF),XN(NF),AG(NF*NA),AF(NA),AFO(NA),PREC,ETA1
      $FLOAT XSTEP,XTEMP,ETA
      INTEGER KA,KAP,IVAR
      SAVE XSTEP,XTEMP,ETA
      SAVE IVAR
      IF (TSS(NS).NE.'UAGX') CALL UYPRO1('UAGX',-1)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      ETA=ETA1**PREC
      CALL UXVCOP(NA,AF,AFO)
      IVAR=0
    2 CONTINUE
      IVAR=IVAR+1
      IF (IVAR.GT.NF) GO TO 4
      IF (KB.GT.0) THEN
      IF (IX(IVAR).LE.-5) GO TO 2
      ENDIF
C
C     STEP SELECTION
C
      IF (NORM.GT.0) THEN
      XSTEP=XN(IVAR)
      ELSE
      XSTEP=ONE
      ENDIF
      XSTEP=ETA*MAX(ABS(X(IVAR)),XSTEP)*SIGN(ONE,X(IVAR))
      XTEMP=X(IVAR)
      X(IVAR)=X(IVAR)+XSTEP
      XSTEP=X(IVAR)-XTEMP
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
      KAP=NF*NAL
      DO 33 KA=NAL+1,NA
      AG(KAP+IVAR)=(AF(KA)-AFO(KA))/XSTEP
      KAP=KAP+NF
   33 CONTINUE
      X(IVAR)=XTEMP
      GO TO 2
    4 CALL UXVCOP(NA,AFO,AF)
      CALL UYEPI1(-1)
      RETURN
      END
