$INTERFACE UA0GS3                                             98/12/01

  $HELP
    PRECISION OF THE COMPUTED VALUES
  $CHECK(ETA1,ETA0,REAL,4)
  CALL UA0GS3(NF,NA,MA,KA,$X,$IX,$XN,$GA,$IAG,$JAG,FA,$ETA1,KBF,NORMF)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$GA($NF)')
$REM  $ADD(REAL,'\FA')
  $ADD(INTEGER,'\$IX($NX)\$IAG($NA+1)\$JAG($MA)')
$REM  $ADD(INTEGER,'\NF\NA\MA\KA\KBF\NORMF')
  $ADD(INIT)
    NAGX='UA0GS3'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UA0GS3             ALL SYSTEMS                 98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* NUMERICAL COMPUTATION OF GRADIENT OF THE SPARSE SELECTED FUNCTION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  KA  INDEX OF THE SELECTED FUNCTION.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  GA(NF)  GRADIENT OF THE APPROXIMATED FUNCTION.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RI  FA  VALUE OF THE SELECTED FUNCTION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*  II  KB  TYPE OF BOUNDS. KB=0-BOUNDS ARE NOT USED. KB=1-ONE SIDED
*         BOUNDS. KB=1-TWO SIDED BOUNDS.
*  II  NORM  SCALING SPECIFICATION. NORM=0-NO SCALING IS PERFORMED.
*         NORM=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORM=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UA0GS3(NF,NA,MA,KA,X,IX,XN,GA,IAG,JAG,FA,ETA1,KB,
     & NORM)
      $INCLUDE('UMCOMM')
      INTEGER NF,NA,MA,KA,IX(NF),IAG(NA+1),JAG(MA),KB,NORM
      $FLOAT X(NF),XN(NF),GA(NF),FA,ETA1
      $FLOAT XSTEP,XTEMP,FTEMP,ETA
      INTEGER IVAR,KVAR
      SAVE XSTEP,XTEMP,FTEMP,ETA
      SAVE IVAR,KVAR
      IF (TSS(NS).NE.'UAGX') CALL UYPRO1('UAGX',-1)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      FTEMP=FA
      KVAR=IAG(KA)
    2 CONTINUE
      IF (KVAR.GE.IAG(KA+1)) GO TO 4
      IVAR=ABS(JAG(KVAR))
      IF (KB.GT.0) THEN
      IF (IX(IVAR).LE.-5) THEN
      KVAR=KVAR+1
      GO TO 2
      ENDIF
      ENDIF
C
C     STEP SELECTION
C
      IF (NORM.GT.0) THEN
      XSTEP=XN(IVAR)
      ELSE
      XSTEP=ONE
      ENDIF
      XSTEP=ETA*MAX(ABS(X(IVAR)),XSTEP)*SIGN(ONE,X(IVAR))
      XTEMP=X(IVAR)
      X(IVAR)=X(IVAR)+XSTEP
      XSTEP=X(IVAR)-XTEMP
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
      GA(IVAR)=(FA-FTEMP)/XSTEP
      X(IVAR)=XTEMP
      KVAR=KVAR+1
      GO TO 2
    4 FA=FTEMP
      CALL UYEPI1(-1)
      RETURN
      END
