$INTERFACE UA0GS4                                             98/12/01

  $HELP
    PRECISION OF THE COMPUTED VALUES
  $CHECK(ETA1,ETA0,REAL,4)
  $ADD(INTEGER,'\$IAG($NA+1)\$JAG($MA)\$COLA($NF)')
  $ADD(REAL,'\$X($NF)\$XN($NF)\$S($NF)\$AG($MA)\$AF($NA)\$AFO($NA)')
$REM  $ADD(INTEGER,'\NF\NA\NAL\MA\JVAR\NORMF')
  CALL UA0GS4(NF,NA,NAL,MA,$X,$XN,$S,$AG,$IAG,$JAG,$AF,$AFO,$COLA,JVAR&
,$ETA1,NORMF)
  $ADD(INIT)
    NAGX='UA0GS4'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UA0GS4             ALL SYSTEMS                 98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE SPARSE JACOBIAN MATRIX.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  RU  X(NF)  VECTORS OF VARIABLES.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RI  XO(NF)  SAVED VECTORS OF VARIABLES.
*  RO  AG(MA)  SPARSE JACOBIAN MATRIX.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RO  AFO(NA)  SAVED VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  II  IG(NF) VECTOR DEFINING GROUPS CONTAINING JACOBIAN COLUMNS.
*  II  NVAR  NUMBER OF GROUPS IN NUMERICAL DIFFERENTIATION.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*  II  NORM  SCALING SPECIFICATION. NORM=0-NO SCALING IS PERFORMED.
*         NORM=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORM=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S  UYPRO1  PROLOGUE.
*  S  UYEPI1  EPILOGUE.
*  S  UYSET0  STATUS DEFINITION.
*  S  UXVSET  INITIATION OF A VECTOR.
*  S  UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*
      SUBROUTINE UA0GS4(NF,NA,NAL,MA,X,XN,XO,AG,IAG,JAG,AF,AFO,IG,NVAR,
     & ETA1,NORM)
      $INCLUDE('UMCOMM')
      INTEGER NF,NA,NAL,MA,IAG(NA+1),JAG(MA),IG(NF),NVAR,NORM
      $FLOAT X(NF),XN(NF),XO(NF),AG(MA),AF(NA),AFO(NA),ETA1
      INTEGER I,J,KA,IVAR
      $FLOAT ETA,XSTEP
      SAVE IVAR
      SAVE ETA
      IF (TSS(NS).NE.'UAGX') CALL UYPRO1('UAGX',-1)
      GO TO (1,4) ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      CALL UXVCOP(NF,X,XO)
      CALL UXVCOP(NA,AF,AFO)
      IVAR=1
    2 IF (IVAR.GT.NVAR) GO TO 7
      DO 3 I=1,NF
      X(I)=XO(I)
      IF (IG(I).EQ.IVAR) THEN
      IF (NORM.GT.0) THEN
      XSTEP=XN(I)
      ELSE
      XSTEP=ONE
      ENDIF
      XSTEP=ETA*MAX(ABS(X(I)),XSTEP)*SIGN(ONE,X(I))
      X(I)=X(I)+XSTEP
      ENDIF
    3 CONTINUE
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    4 DO 6 KA=NAL+1,NA
      DO 5 J=IAG(KA),IAG(KA+1)-1
      I=ABS(JAG(J))
      IF (IG(I).EQ.IVAR) THEN
      AG(J)=(AF(KA)-AFO(KA))/(X(I)-XO(I))
      ENDIF
    5 CONTINUE
    6 CONTINUE
      IVAR=IVAR+1
      GO TO 2
    7 CALL UXVCOP(NF,XO,X)
      CALL UXVCOP(NA,AFO,AF)
      CALL UYEPI1(-1)
      RETURN
      END
