$INTERFACE UA1HS2                                             97/12/01

  $HELP
    PRECISION OF THE COMPUTED GRADIENTS
  $CHECK(ETA1,ETA0,REAL,4)
  CALL UA1HS2(NF,NA,NAL,$X,$IX,$XN,$AH,$AG,$AGO,$AF,$AFO,$ETA1)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$AH($NF*$NA*($NF+1)/2)\$AG($NF*$NA)&
  \$AGO($NF*$NA)\$AF($NA)\$AFO($NA)')
  $ADD(INTEGER,'\$IX($NX)')
$REM  $ADD(INTEGER,'\NF\NA\NAL')
  $ADD(INIT)
    NFHX='UA1HS2'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UA1HS2                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NUMERICAL COMPUTATION OF THE HESSIAN MATRIX OF THE MODEL FUNCTION
* USING ITS GRADIENTS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NA  NUMBER OF FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR FUNCTIONS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RO  AH(NF*NA*(NF+1)/2) SIMULTANEOUSLY COMPUTED DENSE HESSIAN
*         MATRICES.
*  RI  AG(NF*NA)  DENSE JACOBIAN MATRIX.
*  RA  AGO(NF*NA)  AUXILIARY ARRAY.
*  RI  AF(NA)  VECTOR OF FUNCTION VALUES.
*  RA  AFO(NA)  AUXILIARY VECTOR.
*  RI  ETA1  PRECISION OF THE COMPUTED VALUES.
*
* COMMON DATA :
*  II  KBF  TYPE OF BOUNDS. KBF=0-BOUNDS ARE NOT USED. KBF=1-ONE SIDED
*         BOUNDS. KBF=1-TWO SIDED BOUNDS.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*
      SUBROUTINE UA1HS2(NF,NA,NAL,X,IX,XN,AH,AG,AGO,AF,AFO,ETA1)
      $INCLUDE('UMCOMM')
      INTEGER NF,NA,NAL,IX(NF)
      $FLOAT X(NF),XN(NF),AH(NF*NA*(NF+1)/2),AG(NF*NA),AGO(NF*NA),
     & AF(NA),AFO(NA),ETA1
      $FLOAT XSTEP,XTEMP,ETA
      INTEGER I,J,IJ,KA,K,L,IVAR,IND
      SAVE XSTEP,XTEMP,ETA
      SAVE IVAR
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
      IF (TSS(NS).NE.'UAHX') CALL UYPRO1('UAHX',-1)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      ETA=SQRT(ETA1)
      CALL UXVCOP(NA,AF,AFO)
      CALL UXVCOP(NF*NA,AG,AGO)
      IVAR=0
    2 CONTINUE
      IVAR=IVAR+1
      IF (IVAR.GT.NF) GO TO 5
      IF (KBF.GT.0) THEN
      IF (IX(IVAR).LE.-5) GO TO 2
      ENDIF
C
C     STEP SELECTION
C
      IF (NORMF.GT.0) THEN
      XSTEP=XN(IVAR)
      ELSE
      XSTEP=ONE
      ENDIF
      XTEMP=X(IVAR)
      IF (XTEMP.GE.ZERO) THEN
      XSTEP= ETA*MAX(ABS(XTEMP),XSTEP)
      ELSE
      XSTEP=-ETA*MAX(ABS(XTEMP),XSTEP)
      ENDIF
      X(IVAR)=XTEMP+XSTEP
      XSTEP=X(IVAR)-XTEMP
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
C
C     NUMERICAL DIFFERENTIATION
C
      DO 4 J=1,NF
      IF (KBF.GT.0) THEN
      IF (IX(J).LE.-5) GO TO 4
      ENDIF
      IJ=IND(IVAR,J)
      K=NF*NAL
      L=K*(NF+1)/2
      DO 33 KA=NAL+1,NA
      IF (J.GE.IVAR) THEN
      AH(L+IJ)=(AG(K+J)-AGO(K+J))/XSTEP
      ELSE
      AH(L+IJ)=HALF*(AH(L+IJ)+(AG(K+J)-AGO(K+J))/XSTEP)
      ENDIF
      K=K+NF
      L=L+NF*(NF+1)/2
   33 CONTINUE
    4 CONTINUE
      X(IVAR)=XTEMP
      GO TO 2
    5 CALL UXVCOP(NA,AFO,AF)
      CALL UXVCOP(NF*NA,AGO,AG)
      CALL UYEPI1(-1)
      RETURN
      END
