* SUBROUTINE UCCDER             ALL SYSTEMS                 08/12/01
C PORTABILITY : ALL SYSTEMS
C 08/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF THE DIRECTIONAL DERIVATIVE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  RI  CFD(NC)  VECTOR CONTAINING INCREMENTS OF CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  II  ICCA(NCA)  VECTOR CONTAINING INDICES OF COUPLED COMPLEMENTARITY
*         CONSTRAINTS.
*  II  S(NF+NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RI  GS(NCA)  VECTOR CONTAINING ACTIVE SLACK INCREMENTS.
*  RI  RPF2  PENALTY COEFFICIENT.
*  RI  RPF3  BARRIER COEFFICIENT.
*  RI  RPF4  EXACT PENALTY COEFFICIENT.
*  RU  P  STEPSIZE  PARAMETER.
*
      SUBROUTINE UCCDER(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,CFD,
     & IC,CL,CU,CSL,CSU,CZL,CZU,ICA,ICCA,S,GS,RPF2,RPF3,RPF4,P)
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA),ICCA(NCA)
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CFD(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),
     & S(NF+NCA),GS(NCA),RPF2,RPF3,RPF4,P
      INTEGER I,J,K,KK
      $FLOAT TEMP,TEMP1,TEMP2,TEMP3
      DO 1 I=1,NCA
      TEMP=GS(I)
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=S(NF+I)
      TEMP2=XL(K)-X(K)+XSL(K)
      TEMP3=XSL(K)+TEMP
      P=P-RPF3*TEMP/XSL(K)+TEMP*(XZL(K)+TEMP1)-S(K)*(TEMP1+RPF2*TEMP2)+
     & RPF2*TEMP*TEMP2
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=S(NF+I)
      TEMP2=X(K)-XU(K)+XSU(K)
      TEMP3=XSU(K)+TEMP
      P=P-RPF3*TEMP/XSU(K)+TEMP*(XZU(K)+TEMP1)+S(K)*(TEMP1+RPF2*TEMP2)+
     & RPF2*TEMP*TEMP2
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=S(NF+I)
      TEMP2=CL(K)-CF(K)+CSL(K)
      TEMP3=CSL(K)+TEMP
      P=P-RPF3*TEMP/CSL(K)+TEMP*(CZL(K)+TEMP1)-CFD(K)*(TEMP1+
     & RPF2*TEMP2)+RPF2*TEMP*TEMP2
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=S(NF+I)
      TEMP2=CF(K)-CU(K)+CSU(K)
      TEMP3=CSU(K)+TEMP
      P=P-RPF3*TEMP/CSU(K)+TEMP*(CZU(K)+TEMP1)+CFD(K)*(TEMP1+
     & RPF2*TEMP2)+RPF2*TEMP*TEMP2
      ELSE IF (KK.EQ.5) THEN
      P=P+CFD(K)*(S(NF+I)+RPF2*CF(K))
      ENDIF
      ENDIF
      J=ICCA(I)
      IF (J.GT.0) THEN
      P=P+RPF4*TEMP3*GS(J)
      ENDIF
    1 CONTINUE
      RETURN
      END
