* SUBROUTINE UCFACT             ALL SYSTEMS                 01/12/01
C PORTABILITY : ALL SYSTEMS
C 01/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF ACTIVE CONSTRAINTS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  MC  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE HESSIAN MATRIX.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  II  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RO  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RO  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RO  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RO  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITIONS OF THE FIRST ROWS ELEMENTS IN THE SPARSE
*         JACOBIAN STRUCTURE.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE SPARSE JACOBIAN
*         STRUCTURE.
*  RU  G(NF)  RIGHT HAND SIDE VECTOR.
*  RA  GO(NF)  AUXILIARY ARRAY.
*  RU  H(M)  SPARSE HESSIAN MATRIX OF THE LAGRANGIAN FUNCTION.
*  II  IH(NF+1) POINTERS OF DIAGONAL ELEMENTS OF THE SPARSE HESSIAN
*         MATRIX.
*  II  JH(M)  COLUMN INDICES OF NONZERO ELEMENTS OF THE SPARSE HESSIAN
*         MATRIX.
*  IO  ICA(NX+NC)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RO  CP(NCA)  RIGHT HAND SIDE VECTOR.
*  RO  D(NCA)  DIAGONAL SOUTH-EAST CORNER.
*  RI  EPS7  SMOOTHING PARAMETER.
*  RI  EPS9  TOLERANCE FOR ACTIVITY OF CONSTRAINTS.
*  RO  STOL  TOLERANCE FOR MODIFIED DIAGONAL MATRIX.
*  RO  SIG1  SQUARED NORM OF THE PRECISION-MEASURE VECTOR.
*  II  MOT1  CHOICE OF THE PRECISION-MEASURE VECTOR. MOT1=1-RIGHT-HAND
*         SIDE VECTOR IS USED. MOT1=2-CONSTRAINT-VIOLATION VECTOR IS
*         USED.
*  IO  INF  CHANGE OF ACTIVE CONSTRAINT INDICATOR. INF=0-NO CHANGE.
*         INF=1-CHANGE OF ACTIVE CONSTRAINTS.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCFACT(NF,NX,NC,MC,M,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,
     & CF,IC,CL,CU,CSL,CSU,CZL,CZU,CG,ICG,JCG,G,GO,H,IH,JH,ICA,CP,D,
     & EPS7,EPS9,STOL,SIG1,MOT1,INF)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,MC,M,NCA,IX(NX),IC(NC),ICG(NC+1),JCG(MC),
     & IH(*),JH(*),ICA(NX+NC),MOT1,INF
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CF(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),CG(MC),
     & G(NF),GO(NF),H(*),CP(*),D(*),EPS7,EPS9,STOL,SIG1
      INTEGER I,K,KK,KC,KCA
      $FLOAT DENL,DENU,NUML,NUMU,TEMP,TEMP1
      INF=0
      KCA=NCA
      NCA=0
      IF (KBF.GT.0) THEN
      DO 1 I=1,NX
      IX(I)=ABS(IX(I))
      IF (IX(I).EQ.0) THEN
      ELSE IF (IX(I).EQ.1) THEN
      XSL(I)=SQRT((XL(I)-X(I))**2+XZL(I)**2+EPS7**2)
      NUML=ABS(XSL(I)-XZL(I))
      DENL=ABS(XSL(I)+XL(I)-X(I))
      IF (NUML.LE.EPS9*DENL) THEN
      IX(I)=-1
      NCA=NCA+1
      IF (NCA.GT.KCA) THEN
      ICA(NCA)=-I
      INF=1
      ELSE IF (ICA(NCA).NE.-I) THEN
      ICA(NCA)=-I
      INF=1
      ENDIF
      CP(NCA)= XSL(I)*(ONE-XZL(I)/DENL)
      D(NCA)=-MAX(NUML/DENL,STOL)
      ENDIF
      ELSE IF (IX(I).EQ.2) THEN
      XSU(I)=SQRT((X(I)-XU(I))**2+XZU(I)**2+EPS7**2)
      NUMU=ABS(XSU(I)-XZU(I))
      DENU=ABS(XSU(I)+X(I)-XU(I))
      IF (NUMU.LE.EPS9*DENU) THEN
      IX(I)=-2
      NCA=NCA+1
      IF (NCA.GT.KCA) THEN
      ICA(NCA)=-I
      INF=1
      ELSE IF (ICA(NCA).NE.-I) THEN
      ICA(NCA)=-I
      INF=1
      ENDIF
      CP(NCA)=-XSU(I)*(ONE-XZU(I)/DENU)
      D(NCA)=-MAX(NUMU/DENU,STOL)
      ENDIF
      ELSE IF (IX(I).EQ.3.OR.IX(I).EQ.4) THEN
      XSL(I)=SQRT((XL(I)-X(I))**2+XZL(I)**2+EPS7**2)
      XSU(I)=SQRT((X(I)-XU(I))**2+XZU(I)**2+EPS7**2)
      NUML=ABS(XSL(I)-XZL(I))
      NUMU=ABS(XSU(I)-XZU(I))
      DENL=ABS(XSL(I)+XL(I)-X(I))
      DENU=ABS(XSU(I)+X(I)-XU(I))
      IF (NUML*DENU.LE.NUMU*DENL) THEN
      IF (NUML.LE.EPS9*DENL) THEN
      IX(I)=-3
      NCA=NCA+1
      IF (NCA.GT.KCA) THEN
      ICA(NCA)=-I
      INF=1
      ELSE IF (ICA(NCA).NE.-I) THEN
      ICA(NCA)=-I
      INF=1
      ENDIF
      CP(NCA)= XSL(I)*(ONE-XZL(I)/DENL)
      D(NCA)=-MAX(NUML/DENL,STOL)
      ENDIF
      ELSE
      IF (NUMU.LE.EPS9*DENU) THEN
      IX(I)=-4
      NCA=NCA+1
      IF (NCA.GT.KCA) THEN
      ICA(NCA)=-I
      INF=1
      ELSE IF (ICA(NCA).NE.-I) THEN
      ICA(NCA)=-I
      INF=1
      ENDIF
      CP(NCA)=-XSU(I)*(ONE-XZU(I)/DENU)
      D(NCA)=-MAX(NUMU/DENU,STOL)
      ENDIF
      ENDIF
      ELSE IF (IX(I).EQ.5) THEN
      IX(I)=-5
      ENDIF
      IF(EPS9.GE.1.0$(P)120) GO TO 1
      KK=IX(I)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      H(IH(I))=H(IH(I))+DENL/NUML
      G(I)=G(I)+XSL(I)*(ONE+(XL(I)-X(I))/NUML)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      H(IH(I))=H(IH(I))+DENU/NUMU
      G(I)=G(I)-XSU(I)*(ONE+(X(I)-XU(I))/NUMU)
      ENDIF
      IF (MOT1.EQ.2) THEN
      IF (IX(I).EQ.-1.OR.IX(I).EQ.-3) THEN
      SIG1=SIG1+(XSL(I)+XL(I)-X(I)-XZL(I))**2
      ELSE IF (IX(I).EQ.-2.OR.IX(I).EQ.-4) THEN
      SIG1=SIG1+(XSU(I)+X(I)-XU(I)-XZU(I))**2
      ENDIF
      ENDIF
    1 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 2 KC=1,NC
      IC(KC)=ABS(IC(KC))
      IF (IC(KC).EQ.0) THEN
      ELSE IF (IC(KC).EQ.1) THEN
      CSL(KC)=SQRT((CL(KC)-CF(KC))**2+CZL(KC)**2+EPS7**2)
      NUML=ABS(CSL(KC)-CZL(KC))
      DENL=ABS(CSL(KC)+CL(KC)-CF(KC))
      IF (NUML.LE.EPS9*DENL) THEN
      IC(KC)=-1
      NCA=NCA+1
      IF (NCA.GT.KCA) THEN
      ICA(NCA)=KC
      INF=1
      ELSE IF (ICA(NCA).NE.KC) THEN
      ICA(NCA)=KC
      INF=1
      ENDIF
      CP(NCA)= CSL(KC)*(ONE-CZL(KC)/DENL)
      D(NCA)=-MAX(NUML/DENL,STOL)
      ENDIF
      ELSE IF (IC(KC).EQ.2) THEN
      CSU(KC)=SQRT((CF(KC)-CU(KC))**2+CZU(KC)**2+EPS7**2)
      NUMU=ABS(CSU(KC)-CZU(KC))
      DENU=ABS(CSU(KC)+CF(KC)-CU(KC))
      IF (NUMU.LE.EPS9*DENU) THEN
      IC(KC)=-2
      NCA=NCA+1
      IF (NCA.GT.KCA) THEN
      ICA(NCA)=KC
      INF=1
      ELSE IF (ICA(NCA).NE.KC) THEN
      ICA(NCA)=KC
      INF=1
      ENDIF
      CP(NCA)=-CSU(KC)*(ONE-CZU(KC)/DENU)
      D(NCA)=-MAX(NUMU/DENU,STOL)
      ENDIF
      ELSE IF (IC(KC).EQ.3.OR.IC(KC).EQ.4) THEN
      CSL(KC)=SQRT((CL(KC)-CF(KC))**2+CZL(KC)**2+EPS7**2)
      CSU(KC)=SQRT((CF(KC)-CU(KC))**2+CZU(KC)**2+EPS7**2)
      NUML=ABS(CSL(KC)-CZL(KC))
      NUMU=ABS(CSU(KC)-CZU(KC))
      DENL=ABS(CSL(KC)+CL(KC)-CF(KC))
      DENU=ABS(CSU(KC)+CF(KC)-CU(KC))
      IF (NUML*DENU.LE.NUMU*DENL) THEN
      IF (NUML.LE.EPS9*DENL) THEN
      IC(KC)=-3
      NCA=NCA+1
      IF (NCA.GT.KCA) THEN
      ICA(NCA)=KC
      INF=1
      ELSE IF (ICA(NCA).NE.KC) THEN
      ICA(NCA)=KC
      INF=1
      ENDIF
      CP(NCA)= CSL(KC)*(ONE-CZL(KC)/DENL)
      D(NCA)=-MAX(NUML/DENL,STOL)
      ENDIF
      ELSE
      IF (NUMU.LE.EPS9*DENU) THEN
      IC(KC)=-4
      NCA=NCA+1
      IF (NCA.GT.KCA) THEN
      ICA(NCA)=KC
      INF=1
      ELSE IF (ICA(NCA).NE.KC) THEN
      ICA(NCA)=KC
      INF=1
      ENDIF
      CP(NCA)=-CSU(KC)*(ONE-CZU(KC)/DENU)
      D(NCA)=-MAX(NUMU/DENU,STOL)
      ENDIF
      ENDIF
      ELSE IF (IC(KC).EQ.5) THEN
      IC(KC)=-5
      NCA=NCA+1
      IF (NCA.GT.KCA) THEN
      ICA(NCA)=KC
      INF=1
      ELSE IF (ICA(NCA).NE.KC) THEN
      ICA(NCA)=KC
      INF=1
      ENDIF
      CP(NCA)=-CF(KC)
      D(NCA)=ZERO
      ENDIF
      IF(EPS9.GE.1.0$(P)120) GO TO 2
      KK=IC(KC)
      TEMP=ZERO
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP=DENL/NUML
      TEMP1= CSL(KC)*(ONE+(CL(KC)-CF(KC))/NUML)
      IF (TEMP.NE.ZERO) THEN
      K=ICG(KC)
      CALL UALNG3(NF,NC,MC,KC,X,CG,ICG,JCG,GO,ISNC,0)
      CALL UASSH3(NF,NC,MC,M,H,IH,JH,ICG,JCG,GO,KC,TEMP)
      CALL UXWDIR(ICG(KC+1)-K,NF,JCG(K),TEMP1,CG(K),G,G,1)
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP=DENU/NUMU
      TEMP1=-CSU(KC)*(ONE+(CF(KC)-CU(KC))/NUMU)
      IF (TEMP.NE.ZERO) THEN
      K=ICG(KC)
      CALL UALNG3(NF,NC,MC,KC,X,CG,ICG,JCG,GO,ISNC,0)
      CALL UASSH3(NF,NC,MC,M,H,IH,JH,ICG,JCG,GO,KC,TEMP)
      CALL UXWDIR(ICG(KC+1)-K,NF,JCG(K),TEMP1,CG(K),G,G,1)
      ENDIF
      ENDIF
      IF (MOT1.EQ.2) THEN
      IF (IC(KC).EQ.-1.OR.IC(KC).EQ.-3) THEN
      SIG1=SIG1+(CSL(KC)+CL(KC)-CF(KC)-CZL(KC))**2
      ELSE IF (IC(KC).EQ.-2.OR.IC(KC).EQ.-4) THEN
      SIG1=SIG1+(CSU(KC)+CF(KC)-CU(KC)-CZU(KC))**2
      ENDIF
      ENDIF
    2 CONTINUE
      ENDIF
      IF (NCA.NE.KCA) INF=1
      RETURN
      END
