* SUBROUTINE UCIMAX                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETRMINATION OF THE MAXIMUM STEPSIZE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NX)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NX)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NX)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NX)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NX)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CFD(NC)  VECTOR CONTAINING VALUES OF THE CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NX+NC)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  S(NF+NCA)  DIRECTION VECTOR.
*  RI  GS(NCA)  VECTOR CONTAINING ACTIVE SLACK INCREMENTS.
*  RI  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RI  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RO  RMAXP  MAXIMUM VALUE OF THE PRIMAL STEPSIZE PARAMETER.
*  RO  RMAXD  MAXIMUM VALUE OF THE DUAL STEPSIZE PARAMETER.
*  RI  ETA6  COEFFICIENT FOR COMPUTING PRIMAL AND DUAL MAXIMUM
*            STEPSIZES.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCIMAX(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CFD,IC,
     & CL,CU,CSL,CSU,CZL,CZU,ICA,S,GS,XO,GO,RMAX,RMAXP,RMAXD,ETA6)
      $INCLUDE('UMCOMM')
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA)
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),
     & CFD(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),S(NF+NCA),
     & GS(NCA),XO(NX+NC),GO(NX+NC),RMAX,RMAXP,RMAXD,ETA6
      INTEGER I,K,KK
      $FLOAT TEMP
      RMAXP=RMAX/ETA6
      RMAXD=RMAX/ETA6
      DO 1 I=1,NCA
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      IF (GS(I).LT.ZERO) RMAXP=MIN(RMAXP,-XSL(K)/GS(I))
      IF (S(NF+I).LT.ZERO) RMAXD=MIN(RMAXD,-XZL(K)/S(NF+I))
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      IF (GS(I).LT.ZERO) RMAXP=MIN(RMAXP,-XSU(K)/GS(I))
      IF (S(NF+I).LT.ZERO) RMAXD=MIN(RMAXD,-XZU(K)/S(NF+I))
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      IF (GS(I).LT.ZERO) RMAXP=MIN(RMAXP,-CSL(K)/GS(I))
      IF (S(NF+I).LT.ZERO) RMAXD=MIN(RMAXD,-CZL(K)/S(NF+I))
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      IF (GS(I).LT.ZERO) RMAXP=MIN(RMAXP,-CSU(K)/GS(I))
      IF (S(NF+I).LT.ZERO) RMAXD=MIN(RMAXD,-CZU(K)/S(NF+I))
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      KK=IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP=X(K)+S(K)-XL(K)-XSL(K)
      IF (TEMP.LT.ZERO) RMAXP=MIN(RMAXP,-XSL(K)/TEMP)
      TEMP=XO(K)
      IF (TEMP.LT.ZERO) RMAXD=MIN(RMAXD,-XZL(K)/TEMP)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP=XU(K)-X(K)-S(K)-XSU(K)
      IF (TEMP.LT.ZERO) RMAXP=MIN(RMAXP,-XSU(K)/TEMP)
      TEMP=GO(K)
      IF (TEMP.LT.ZERO) RMAXD=MIN(RMAXD,-XZU(K)/TEMP)
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      KK=IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP=CFD(K)-CL(K)-CSL(K)
      IF (TEMP.LT.ZERO) RMAXP=MIN(RMAXP,-CSL(K)/TEMP)
      TEMP=XO(NX+K)
      IF (TEMP.LT.ZERO) RMAXD=MIN(RMAXD,-CZL(K)/TEMP)
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP=CU(K)-CFD(K)-CSU(K)
      IF (TEMP.LT.ZERO) RMAXP=MIN(RMAXP,-CSU(K)/TEMP)
      TEMP=GO(NX+K)
      IF (TEMP.LT.ZERO) RMAXD=MIN(RMAXD,-CZU(K)/TEMP)
      ENDIF
    3 CONTINUE
      ENDIF
      RMAXP=ETA6*RMAXP
      RMAXD=ETA6*RMAXD
      RETURN
      END
