* SUBROUTINE UCIND1             ALL SYSTEMS                   98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE SPARSE SCHUR COMPLEMENT FOR THE KARUSH-KUHN-TUCKER
* MATRIX WITH THE DIAGONAL HESSIAN BLOCK.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  II  NC  NUMBER OF CONSTRAINT FUNCTIONS.
*  II  MC  NUMBER OF NONZERO ELEMENTS IN THE FIELD CG.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE SCHUR COMPLEMENT.
*  II  ND  ACTUAL NUMBER OF DENSE ROWS.
*  RO  B(M)  ELEMENTS OF THE SPARSE MATRIX B.
*  II  IB(NC+1)  POINTERS OF THE DIAGONAL ELEMENTS OF B.
*  II  JB(M)  INDICES OF THE NONZERO ELEMENTS OF B.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  II  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  II  KCG(NF)  INDICATION OF DENSE ROWS.
*  RO  CGD(NC*ND)  ELEMENTS OF DENSE ROWS.
*  RI  DD(ND)  DENSE PART OF DIAGONAL WEIGHTING MATRIX.
*  RI  D(NF)  DIAGONAL WEIGHTING MATRIX.
*
      SUBROUTINE UCIND1(NF,NCA,NC,MC,M,ND,B,IB,JB,ICA,CG,ICG,JCG,KCG,
     & CGD,DD,D)
      INTEGER NF,NCA,NC,MC,M,ND,IB(NCA+1),JB(M),ICA(NCA),ICG(NC+1),
     & JCG(MC),KCG(NF)
      $FLOAT B(M),CG(MC),CGD(NCA*ND),DD(ND),D(NF+NCA)
      INTEGER I,II,I1,I2,IK,IL,K,KK,K1,K2,L,LL,L1,L2
      IF (ND.GT.0) CALL UXVSET(NCA*ND,0.0$P 0,CGD)
      DO 6 I=1,NCA
      I1=IB(I)
      I2=IB(I+1)-1
      K=ICA(I)
      B(I1)=-D(NF+I)
      DO 4 II=I1,I2
      IF (II.NE.I1) B(II)=0.0$P 0
      L=ICA(JB(II))
      IF (K.LT.0) THEN
      IF (KCG(-K).EQ.0) THEN
      KK=-K
      IF (L.LT.0) THEN
      IF (L.EQ.K) B(II)=B(II)+D(KK)
      ELSE
      L1=ICG(L)
      L2=ICG(L+1)-1
      DO 1 LL=L1,L2
      IF (JCG(LL).EQ.KK) THEN
      B(II)=B(II)+D(KK)*CG(LL)
      GO TO 4
      ENDIF
    1 CONTINUE
      ENDIF
      ENDIF
      ELSE IF (K.GT.0) THEN
      K1=ICG(K)
      K2=ICG(K+1)-1
      IF (L.LT.0) THEN
      IF (KCG(-L).EQ.0) THEN
      LL=-L
      DO 2 KK=K1,K2
      IF (JCG(KK).EQ.LL) THEN
      B(II)=B(II)+D(LL)*CG(KK)
      GO TO 4
      ENDIF
    2 CONTINUE
      ENDIF
      ELSE IF (L.GT.0) THEN
      KK=K1
      LL=ICG(L)
      L2=ICG(L+1)-1
      IK=JCG(KK)
      IL=JCG(LL)
    3 IF (KCG(IK).NE.0.OR.IK.LE.0.OR.IK.LT.IL) THEN
      IF (KK.GE.K2) GO TO 4
      KK=KK+1
      IK=JCG(KK)
      GO TO 3
      ELSE IF (KCG(IL).NE.0.OR.IL.LE.0.OR.IL.LT.IK) THEN
      IF (LL.GE.L2) GO TO 4
      LL=LL+1
      IL=JCG(LL)
      GO TO 3
      ELSE
      B(II)=B(II)+D(IK)*CG(KK)*CG(LL)
      IF(KK.GE.K2.OR.LL.GE.L2) GO TO 4
      KK=KK+1
      LL=LL+1
      IK=JCG(KK)
      IL=JCG(LL)
      GO TO 3
      ENDIF
      ENDIF
      ENDIF
    4 CONTINUE
      IF (ND.GT.0) THEN
      IF (K.LT.0) THEN
      KK=-K
      LL=KCG(KK)
      IF (LL.GT.0) THEN
      CGD((LL-1)*NCA+I)=1.0$P 0
      DD(LL)=1.0$P 0/D(K)
      ENDIF
      ELSE
      DO 5 KK=K1,K2
      L=JCG(KK)
      LL=KCG(L)
      IF (L.GT.0.AND.LL.GT.0) THEN
      CGD((LL-1)*NCA+I)=CG(KK)
      DD(LL)=1.0$P 0/D(L)
      ENDIF
    5 CONTINUE
      ENDIF
      ENDIF
    6 CONTINUE
      RETURN
      END
