* SUBROUTINE UCISTP             ALL SYSTEMS                 01/12/01
C PORTABILITY : ALL SYSTEMS
C 01/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* STEP FOR LAGRANGIAN MULTIPLIERS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XSL(NX)  VECTOR CONTAINING LOVER SLACKS FOR VARIABLES.
*  RI  XSU(NX)  VECTOR CONTAINING UPPER SLACKS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CFD(NC)  VECTOR CONTAINING INCREMENTS OF CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CSL(NC)  VECTOR CONTAINING LOVER SLACKS FOR CONSTRAINTS.
*  RI  CSU(NC)  VECTOR CONTAINING UPPER SLACKS FOR CONSTRAINTS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  S(NF+NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RI  GS(NCA)  VECTOR CONTAINING ACTIVE SLACK INCREMENTS.
*  RI  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RI  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RI  R  STEPSIZE PARAMETER.
*  RI  ETA6  COEFFICIENT FOR COMPUTING PRIMAL AND DUAL MAXIMUM
*            STEPSIZES.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UCISTP(NF,NX,NC,NCA,X,IX,XL,XU,XSL,XSU,XZL,XZU,CF,CFD,
     & IC,CL,CU,CSL,CSU,CZL,CZU,ICA,S,GS,XO,GO,R,ETA6,ICON)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA),ICON
      $FLOAT X(NF),XL(NX),XU(NX),XSL(NX),XSU(NX),XZL(NX),XZU(NX),CF(NC),
     & CFD(NC),CL(NC),CU(NC),CSL(NC),CSU(NC),CZL(NC),CZU(NC),S(NF+NCA),
     & GS(NCA),XO(NF+NC),GO(NF+NC),R,ETA6
      INTEGER I,K,KK
      $FLOAT RP,RD,TEMP
      DO 1 I=1,NCA
      RP=R
      RD=R
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      IF (GS(I).LT.ZERO) RP=MIN(RP,-ETA6*XSL(K)/GS(I))
      IF (S(NF+I).LT.ZERO) RD=MIN(RD,-ETA6*XZL(K)/S(NF+I))
      XZL(K)=XZL(K)+RD*S(NF+I)
      IF (ABS(ICON).EQ.2.AND.X(K).GT.XL(K)) THEN
      XSL(K)=X(K)-XL(K)
      ELSE
      XSL(K)=XSL(K)+RP*GS(I)
      ENDIF
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      IF (GS(I).LT.ZERO) RP=MIN(RP,-ETA6*XSU(K)/GS(I))
      IF (S(NF+I).LT.ZERO) RD=MIN(RD,-ETA6*XZU(K)/S(NF+I))
      XZU(K)=XZU(K)+RD*S(NF+I)
      IF (ABS(ICON).EQ.2.AND.X(K).LT.XU(K)) THEN
      XSU(K)=XU(K)-X(K)
      ELSE
      XSU(K)=XSU(K)+RP*GS(I)
      ENDIF
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      IF (GS(I).LT.ZERO) RP=MIN(RP,-ETA6*CSL(K)/GS(I))
      IF (S(NF+I).LT.ZERO) RD=MIN(RD,-ETA6*CZL(K)/S(NF+I))
      CZL(K)=CZL(K)+RD*S(NF+I)
      IF (ABS(ICON).EQ.2.AND.CF(K).GT.CL(K)) THEN
      CSL(K)=CF(K)-CL(K)
      ELSE
      CSL(K)=CSL(K)+RP*GS(I)
      ENDIF
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      IF (GS(I).LT.ZERO) RP=MIN(RP,-ETA6*CSU(K)/GS(I))
      IF (S(NF+I).LT.ZERO) RD=MIN(RD,-ETA6*CZU(K)/S(NF+I))
      CZU(K)=CZU(K)+RD*S(NF+I)
      IF (ABS(ICON).EQ.2.AND.CF(K).LT.CU(K)) THEN
      CSU(K)=CU(K)-CF(K)
      ELSE
      CSU(K)=CSU(K)+RP*GS(I)
      ENDIF
      ELSE IF (KK.EQ.5) THEN
      CZL(K)=CZL(K)+RD*S(NF+I)
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      RP=R
      RD=R
      KK=IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP=X(K)+S(K)-XL(K)-XSL(K)
      IF (TEMP.LT.ZERO) RP=MIN(RP,-ETA6*XSL(K)/TEMP)
      IF (XO(K).LT.ZERO) RD=MIN(RD,-ETA6*XZL(K)/XO(K))
      XZL(K)=XZL(K)+RD*XO(K)
      IF (ABS(ICON).EQ.2.AND.X(K).GT.XL(K)) THEN
      XSL(K)=X(K)-XL(K)
      ELSE
      XSL(K)=XSL(K)+RP*TEMP
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP=XU(K)-X(K)-S(K)-XSU(K)
      IF (TEMP.LT.ZERO) RP=MIN(RP,-ETA6*XSU(K)/TEMP)
      IF (GO(K).LT.ZERO) RD=MIN(RD,-ETA6*XZU(K)/GO(K))
      XZU(K)=XZU(K)+RD*GO(K)
      IF (ABS(ICON).EQ.2.AND.X(K).LT.XU(K)) THEN
      XSU(K)=XU(K)-X(K)
      ELSE
      XSU(K)=XSU(K)+RP*TEMP
      ENDIF
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      RP=R
      RD=R
      KK=IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP=CFD(K)-CL(K)-CSL(K)
      IF (TEMP.LT.ZERO) RP=MIN(RP,-ETA6*CSL(K)/TEMP)
      IF (XO(NX+K).LT.ZERO) RD=MIN(RD,-ETA6*CZL(K)/XO(NX+K))
      CZL(K)=CZL(K)+RD*XO(NX+K)
      IF (ABS(ICON).EQ.2.AND.CF(K).GT.CL(K)) THEN
      CSL(K)=CF(K)-CL(K)
      ELSE
      CSL(K)=CSL(K)+RP*TEMP
      ENDIF
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP=CU(K)-CFD(K)-CSU(K)
      IF (TEMP.LT.ZERO) RP=MIN(RP,-ETA6*CSU(K)/TEMP)
      IF (GO(NX+K).LT.ZERO) RD=MIN(RD,-ETA6*CZU(K)/GO(NX+K))
      CZU(K)=CZU(K)+RD*GO(NX+K)
      IF (ABS(ICON).EQ.2.AND.CF(K).LT.CU(K)) THEN
      CSU(K)=CU(K)-CF(K)
      ELSE
      CSU(K)=CSU(K)+RP*TEMP
      ENDIF
      ENDIF
    3 CONTINUE
      ENDIF
      RETURN
      END
