$INTERFACE UD1TL5                                             98/12/01

    $ADD(REAL,'\UXVDOT')
    $ADD(REAL,'\$GN($NF)\$SN($NF)')
    IF (KD.GT.0) P=UXVDOT(NF,$GF,$S)
    CALL UD1TL5(NF,$GN,$SN,EPS0,ALF1,ALF2,P,GNORM,SNORM,RMIN,RMAX,XMAX&
,INITD)
$REM  $ADD(INTEGER,'\NF\$N\MES\INITD\INITH')
$REM  $ADD(REAL,'\EPS0\ALF1\ALF2\R\P\GNORM\SNORM\RMIN\RMAX\XMAX\XDEL')
  $INITD=1
  $HELP
    TOLERANCE FOR DESCENT DIRECTION
  $CHECK(EPS0,'1.0$P-16',REAL,4)
  $HELP
    LOWER BOUND FOR STEPSIZE PARAMETER
  $CHECK(ALF1,'1.0$P-10',REAL,4)
  $HELP
    UPPER BOUND FOR STEPSIZE PARAMETER
  $CHECK(ALF2,'1.0$P 10',REAL,4)
  $HELP
    MAXIMUM STEPSIZE
  $CHECK(XMAX,'1.0$P 3',REAL,2)
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UD1TL5                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TEST ON DESCENT DIRECTION AND PREPARATION OF LINE SEARCH.
*
* PARAMETERS :
*  II  NDIM DECLARED NUMBER OF VARIABLES.
*  RI  G(NDIM) GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  S(NDIM) DIRECTION VECTOR.
*  RI  EPS0  TOLERANCE FOR UNIFORM DESCENT DIRECTION.
*  RI  ALF1  LOWER BOUND FOR STEPSIZE PARAMETER.
*  RI  ALF2  UPPER BOUND FOR STEPSIZE PARAMETER.
*  RO  P  DIRECTIONAL DERIVATE.
*  RI  GNORM  EUCLIDEAN NORM OF THE GRADIENT.
*  RI  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RO  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RO  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  XMAX  MAXIMUM STEPSIZE.
*  II  INITD  CHOICE OF THE RESTART CRITERION. INITD<0-CHANGE OF THE
*         SIGN IF P<0. INITD>0-NO CHANGE IF P<0. ABS(INITD)=0-NO
*         RESTART CRITERION. ABS(INITD)=1-UNIFORM DESCENT CRITERION.
*
* COMMON DATA :
*  IU  ITERD  TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  IU  IREST  RESTART SPECIFICATION. IF IREST>0 THEN RESTART IS
*         PERFORMED.
*  IU  ITERM  TERMINATION INDICATOR. TERMINATION IF ITERM .NE. 0.
*  IU  NRED  ITERATION COUNT IN LINE SEARCH.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION VECTOR OBTAINED.
*
* SUBPROGRAMS USED :
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  S   UYRES1  DEFINITION OF THE RESTART VARIABLES.
*  S   UOD1D8  INTERMEDIATE PRINT.
*
      SUBROUTINE UD1TL5(NDIM,G,S,EPS0,ALF1,ALF2,P,GNORM,SNORM,RMIN,RMAX,
     & XMAX,INITD)
      $INCLUDE('UMCOMM')
      $INCLUDE('UMCOMD')
      INTEGER NDIM,INITD
      $FLOAT G(NDIM),S(NDIM),EPS0,ALF1,ALF2,P,GNORM,SNORM,RMIN,RMAX,
     & XMAX
      $FLOAT CON1
      PARAMETER (CON1=5.0$P-1)
      NDEM=NDIM
      IF(ITERD.LT.0) THEN
        ITERM=ITERD
        RETURN
      ENDIF
C
C     TEST ON DESCENT DIRECTION
C
      IF(SNORM.LE.ZERO) THEN
        CALL UYRES1('ZERO DIR')
      ELSEIF(ABS(INITD).EQ.0.OR.P+EPS0*GNORM*SNORM.LE.ZERO) THEN
        IREST=0
      ELSEIF(ABS(INITD).EQ.1) THEN
C
C     UNIFORM DESCENT CRITERION
C
        CALL UYRES1('UNIFORM ')
      ENDIF
      IF (IREST.EQ.0) THEN
C
C     PREPARATION OF LINE SEARCH
C
        NRED = 0
        RMIN=ALF1*GNORM/SNORM
        RMAX=MIN(ALF2*GNORM/SNORM,XMAX/SNORM)
        RMIN=MIN(RMIN,RMAX/TEN)
      ENDIF
      CALL UOD1D8(NDIM,G,S,GNORM,SNORM,P)
      RETURN
      END
