$INTERFACE UEVDP8                                             98/12/01

  $DIME='18*($MEL)'
  $IF(MED<2) $DIME='11*($MEL)'
  $IF(MED=1)
    NAL=$NA
  $ENDIF
  $HELP
    STEP SELECTION
      1 - STEP SELECTION FROM LOCAL TRUNCATION ERROR
      2 - STEP SELECTION FROM PRESCRIBED MESH POINTS
  $CHECK(MED1,1,'1\2',4)
  $IF(MED1=1) $MED2=0
  $HELP
    NUMBER OF RESETING SOLUTIONS
      MED2=0 - NO RESETING SOLUTIONS
      MED2=1 - FIRST NE SOLUTIONS RESETTED
      MED2=2 - FIRST NE SOLUTIONS INTERPOLATED
  $CHECK(MED2,0,'0\1\2',4)
  $HELP
    NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS
  $CHECK(MED3,1000,INT,4)
  $HELP
    MAXIMUM NUMBER OF INTEGRATION STEPS
  $CHECK(MSTP,100000,INT,3)
  $HELP
    PARAMETER FOR STEPSIZE CONTROL
  $CHECK(BET3,'4.0$P-2',REAL,4)
  $HELP
    LOVER PARAMETER FOR STEPSIZE SELECTION
  $CHECK(TET1,'0.2$P 0',REAL,4)
  $HELP
    UPPER PARAMETER FOR STEPSIZE SELECTION
  $CHECK(TET2,'1.0$P 1',REAL,4)
  $HELP
    SAFETY FACTOR FOR STEPSIZE PREDICTION
  $CHECK(TET3,'0.9$P 0',REAL,4)
  $HELP
    INITIAL STEPSIZE
  $CHECK(HE,'0.0$P 0',REAL,4)
  $HELP
    MAXIMUM STEPSIZE
  $CHECK(HEMAX,'0.0$P 0',REAL,4)
  $HELP
    RELATIVE PRECISION
  $CHECK(TOLR,'1.0$P-6',REAL,3)
  $HELP
    ABSOLUTE PRECISION
  $CHECK(TOLA,'1.0$P-6',REAL,3)
  $HELP
    UPPER BOUND FOR SOLUTION NORM
  $CHECK(FMAX,'1.0$P 20',REAL,4)
  CALL UEVDP8(NA,NAL,NE,MEL,KA,$AT,$AY,$YA,$YAO,$EF,$ES,$ETA0,$TET1&
,$TET2,$TET3,$BET3,TA,TAO,TAMAX,HE,HEMAX,FMAX,ERR,MED,MED1,MED2,MED3&
,NACC,NREJ,IER)
  $ADD(REAL,'\$AT($NA)\$AY($NA*$NE)\$YA($MEL)\$YAO($MEL)\$EF($MEL)&
  \$ES($DIME)')
$REM  $ADD(REAL,'\TA\TAO\TAMAX\HE\HEMAX\FMAX\ERR')
$REM  $ADD(INTEGER,'\NA\NAL\NE\MEL\KA\NACC\NREJ\IER')
  GOTO ($LET,$LED) ISB+1
  $ADD(SINID)
    IF (HEMAX.LE.ZERO) HEMAX=TAMAX-TA
  $ENDADD
  $ADD(INIT)
    NEDX='UEVDP8'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UEVDP8                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF THE INITIAL VALUE PROBLEM FOR A SET OF THE FIRST ORDER
* ORDINARY DIFFERENTIAL EQUATIONS USING THE EIGHTH ORDER DORMAN AND
* PRINCE INTEGRATION FORMULA.
*
* PARAMETERS :
*  II  NA  NUMBER OF MESH POINTS.
*  II  NAL  NUMBER OF MESH POINTS.
*  II  NE  DECLARED NUMBER OF DIFFERENTIAL EQUATIONS.
*  II  MEL  ACTUAL NUMBER OF DIFFERENTIAL EQUATIONS.
*  IU  KA  INDEX OF THE MESH POINT.
*  RI  AT(NA)  MESH POINTS.
*  RO  AY(NA*NE)  FIELD OF SOLUTION VECTORS IN THE MESH POINTS.
*  RU  YA(MEL)  SOLUTION VECTOR.
*  RA  YAO(MEL)  OLD SOLUTION VECTOR.
*  RI  EF(MEL)  RIGHT HAND SIDE VECTOR.
*  RA  ES(18*MEL)  AUXILIARY VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  RI  TET1  LOVER PARAMETER FOR STEPSIZE SELECTION.
*  RI  TET2  UPPER PARAMETER FOR STEPSIZE SELECTION.
*  RI  TET3  SAFETY FACTOR FOR STEPSIZE PREDICTION.
*  RI  BET3  PARAMETER FOR STEPSIZE CONTROL.
*  RU  TA  VALUE OF THE INTEGRATION PARAMETER.
*  RU  TAO  OLD VALUE OF THE INTEGRATION PARAMETER.
*  RI  TAMAX  MAXIMUM VALUE OF THE INTEGRATION PARAMETER.
*  RU  HE  ACTUAL STEPSIZE.
*  RU  HEMAX  MAXIMUM STEPSIZE.
*  RI  FMAX  UPPER BOUND FOR SOLUTION NORM.
*  RO  ERR  INTEGRATION ERROR.
*  II  MED  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  II  MED1  CHOICE OF STEP SELECTION.
*  II  MED2  NUMBER OF RESETING SOLUTIONS.
*  II  MED3  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  IO  NACC  NUMBER OF ACCEPTED INTEGRATION STEPS.
*  IO  NREJ  NUMBER OF REJECTED INTEGRATION STEPS.
*  IO  IER  ERROR INDICATOR. IER=0-SOLUTION OBTAINED. IER=-1-SYSTEM
*         IS UNSTABLE. IER=-2-MAXIMUM NUMBER OF STEPS EXCEEDED.
*         IER=-3-TOO SMALL STEP REQUIRED. IER=-4-SYSTEM IS PROBABLY
*         STIFF. IER=-5-JACOBIAN MATRIX IS SINGULAR. IER=-1-LACK
*         OF SPACE FOR OUTPUT DATA.
*
* COMMON DATA :
*  IO  NSTP  ACTUAL  NUMBER OF INTEGRATION STEPS.
*  II  MSTP  MAXIMUM NUMBER OF INTEGRATION STEPS.
*  RI  TOLR  RELATIVE PRECISION.
*  RI  TOLA  ABSOLUTE PRECISION.
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UNVDP8  INTERPOLATION IN MESH POINTS.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVMED  MEDIAN OF TWO VECTORS.
*
* METHOD :
* E.HAIRER, S.P.NORSETT, G.WANNER: SOLVING ORDINARY DIFFERENTIAL
* EQUATIONS I. SPRINGER SERIES IN COMPUTATIONAL MATHEMATICS.
* SPRINGER VERLAG, BERLIN 1980.
*
      SUBROUTINE UEVDP8(NA,NAL,NE,MEL,KA,AT,AY,YA,YAO,EF,ES,ETA0,TET1,
     & TET2,TET3,BET3,TA,TAO,TAMAX,HE,HEMAX,FMAX,ERR,MED,MED1,MED2,MED3,
     & NACC,NREJ,IER)
      $INCLUDE ('UMCOMM')
      INTEGER NA,NAL,NE,MEL,KA,MED,MED1,MED2,MED3,NACC,NREJ,IER
      $FLOAT AT(*),AY(*),YA(MEL),YAO(MEL),EF(MEL),ES(*),ETA0,
     & TET1,TET2,TET3,BET3,TA,TAO,TAMAX,HE,HEMAX,FMAX,ERR
      LOGICAL REJECT,LAST
      INTEGER NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,NE11,NE12,NE13,
     & NE14,NE15,NE16,NE17,NE18,I,IASTI,IORD,NONSTI,NEGPOS
      $FLOAT FACOLD,FAC,FAC11,POSNEG,HLAMB,DNF,DNY,
     & SK,DER2,DER12,HE1,HNEW,STNUM,STDEN,YDIFF,BSPL,ERR2,ERRI,
     & DENO,TAPH,UXVMAX
      $FLOAT C2,C3,C4,C5,C6,C7,C8,C9,C10,C11,C14,C15,C16,B1,B6,B7,B8,
     & B9,B10,B11,B12,BHH1,BHH2,BHH3,ER1,ER6,ER7,ER8,ER9,ER10,ER11,ER12
      $FLOAT A21,A31,A32,A41,A43,A51,A53,A54,A61,A64,A65,A71,A74,A75,
     & A76,A81,A84,A85,A86,A87,A91,A94,A95,A96,A97,A98,A101,A104,A105,
     & A106,A107,A108,A109,A111,A114,A115,A116,A117,A118,A119,A1110,
     & A121,A124,A125,A126,A127,A128,A129,A1210,A1211,A141,A147,A148,
     & A149,A1410,A1411,A1412,A1413,A151,A156,A157,A158,A1511,A1512,
     & A1513,A1514,A161,A166,A167,A168,A169,A1613,A1614,A1615
      $FLOAT D41,D46,D47,D48,D49,D410,D411,D412,D413,D414,D415,D416,
     & D51,D56,D57,D58,D59,D510,D511,D512,D513,D514,D515,D516,D61,D66,
     & D67,D68,D69,D610,D611,D612,D613,D614,D615,D616,D71,D76,D77,D78,
     & D79,D710,D711,D712,D713,D714,D715,D716
      SAVE REJECT,LAST
      SAVE NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,NE11,NE12,NE13,
     & NE14,NE15,NE16,NE17,NE18,IASTI,IORD,NONSTI,NEGPOS
      SAVE FACOLD,POSNEG,HLAMB,DNF,HNEW,TAPH
C
C     PARAMETERS FOR THE EIGHTH ORDER DORMAN-PRINCE FORMULA
C
      PARAMETER (
     &  C2  = 0.526001519587677318785587544488$P-1,
     &  C3  = 0.789002279381515978178381316732$P-1,
     &  C4  = 0.118350341907227396726757197510$P+0,
     &  C5  = 0.281649658092772603273242802490$P+0,
     &  C6  = 0.333333333333333333333333333333$P+0,
     &  C7  = 0.25$P+00,
     &  C8  = 0.307692307692307692307692307692$P+0,
     &  C9  = 0.651282051282051282051282051282$P+0,
     &  C10 = 0.6$P+00,
     &  C11 = 0.857142857142857142857142857142$P+0,
     &  C14 = 0.1$P+00,
     &  C15 = 0.2$P+00,
     &  C16 = 0.777777777777777777777777777778$P+0)
      PARAMETER (
     &  B1 =   5.42937341165687622380535766363$P-2,
     &  B6 =   4.45031289275240888144113950566$P+0,
     &  B7 =   1.89151789931450038304281599044$P+0,
     &  B8 =  -5.8012039600105847814672114227$P+0,
     &  B9 =   3.1116436695781989440891606237$P-1,
     &  B10 = -1.52160949662516078556178806805$P-1,
     &  B11 =  2.01365400804030348374776537501$P-1,
     &  B12 =  4.47106157277725905176885569043$P-2)
      PARAMETER (
     &  BHH1 = 0.244094488188976377952755905512$P+0,
     &  BHH2 = 0.733846688281611857341361741547$P+0,
     &  BHH3 = 0.220588235294117647058823529412$P-1)
      PARAMETER (
     &  ER1  =  0.1312004499419488073250102996$P-1,
     &  ER6  = -0.1225156446376204440720569753$P+1,
     &  ER7  = -0.4957589496572501915214079952$P+0,
     &  ER8  =  0.1664377182454986536961530415$P+1,
     &  ER9  = -0.3503288487499736816886487290$P+0,
     &  ER10 =  0.3341791187130174790297318841$P+0,
     &  ER11 =  0.8192320648511571246570742613$P-1,
     &  ER12 = -0.2235530786388629525884427845$P-1)
      PARAMETER (
     &  A21 =    5.26001519587677318785587544488$P-2,
     &  A31 =    1.97250569845378994544595329183$P-2,
     &  A32 =    5.91751709536136983633785987549$P-2,
     &  A41 =    2.95875854768068491816892993775$P-2,
     &  A43 =    8.87627564304205475450678981324$P-2,
     &  A51 =    2.41365134159266685502369798665$P-1,
     &  A53 =   -8.84549479328286085344864962717$P-1,
     &  A54 =    9.24834003261792003115737966543$P-1,
     &  A61 =    3.7037037037037037037037037037$P-2,
     &  A64 =    1.70828608729473871279604482173$P-1,
     &  A65 =    1.25467687566822425016691814123$P-1,
     &  A71 =    3.7109375$P-2,
     &  A74 =    1.70252211019544039314978060272$P-1,
     &  A75 =    6.02165389804559606850219397283$P-2,
     &  A76 =   -1.7578125$P-2)
      PARAMETER (
     &  A81 =    3.70920001185047927108779319836$P-2,
     &  A84 =    1.70383925712239993810214054705$P-1,
     &  A85 =    1.07262030446373284651809199168$P-1,
     &  A86 =   -1.53194377486244017527936158236$P-2,
     &  A87 =    8.27378916381402288758473766002$P-3,
     &  A91 =    6.24110958716075717114429577812$P-1,
     &  A94 =   -3.36089262944694129406857109825$P+0,
     &  A95 =   -8.68219346841726006818189891453$P-1,
     &  A96 =    2.75920996994467083049415600797$P+1,
     &  A97 =    2.01540675504778934086186788979$P+1,
     &  A98 =   -4.34898841810699588477366255144$P+1,
     &  A101 =   4.77662536438264365890433908527$P-1,
     &  A104 =  -2.48811461997166764192642586468$P+0,
     &  A105 =  -5.90290826836842996371446475743$P-1,
     &  A106 =   2.12300514481811942347288949897$P+1,
     &  A107 =   1.52792336328824235832596922938$P+1,
     &  A108 =  -3.32882109689848629194453265587$P+1,
     &  A109 =  -2.03312017085086261358222928593$P-2)
      PARAMETER (
     &  A111 =  -9.3714243008598732571704021658$P-1,
     &  A114 =   5.18637242884406370830023853209$P+0,
     &  A115 =   1.09143734899672957818500254654$P+0,
     &  A116 =  -8.14978701074692612513997267357$P+0,
     &  A117 =  -1.85200656599969598641566180701$P+1,
     &  A118 =   2.27394870993505042818970056734$P+1,
     &  A119 =   2.49360555267965238987089396762$P+0,
     &  A1110 = -3.0467644718982195003823669022$P+0,
     &  A121 =   2.27331014751653820792359768449$P+0,
     &  A124 =  -1.05344954667372501984066689879$P+1,
     &  A125 =  -2.00087205822486249909675718444$P+0,
     &  A126 =  -1.79589318631187989172765950534$P+1,
     &  A127 =   2.79488845294199600508499808837$P+1,
     &  A128 =  -2.85899827713502369474065508674$P+0,
     &  A129 =  -8.87285693353062954433549289258$P+0,
     &  A1210 =  1.23605671757943030647266201528$P+1,
     &  A1211 =  6.43392746015763530355970484046$P-1)
      PARAMETER (
     &  A141 =  5.61675022830479523392909219681$P-2,
     &  A147 =  2.53500210216624811088794765333$P-1,
     &  A148 = -2.46239037470802489917441475441$P-1,
     &  A149 = -1.24191423263816360469010140626$P-1,
     &  A1410 =  1.5329179827876569731206322685$P-1,
     &  A1411 =  8.20105229563468988491666602057$P-3,
     &  A1412 =  7.56789766054569976138603589584$P-3,
     &  A1413 = -8.298$P-3)
      PARAMETER (
     &  A151 =  3.18346481635021405060768473261$P-2,
     &  A156 =  2.83009096723667755288322961402$P-2,
     &  A157 =  5.35419883074385676223797384372$P-2,
     &  A158 = -5.49237485713909884646569340306$P-2,
     &  A1511 = -1.08347328697249322858509316994$P-4,
     &  A1512 =  3.82571090835658412954920192323$P-4,
     &  A1513 = -3.40465008687404560802977114492$P-4,
     &  A1514 =  1.41312443674632500278074618366$P-1,
     &  A161 = -4.28896301583791923408573538692$P-1,
     &  A166 = -4.69762141536116384314449447206$P+0,
     &  A167 =  7.68342119606259904184240953878$P+0,
     &  A168 =  4.06898981839711007970213554331$P+0,
     &  A169 =  3.56727187455281109270669543021$P-1,
     &  A1613 = -1.39902416515901462129418009734$P-3,
     &  A1614 =  2.9475147891527723389556272149$P+0,
     &  A1615 = -9.15095847217987001081870187138$P+0)
      PARAMETER (
     &  D41  = -0.84289382761090128651353491142$P+1,
     &  D46  =  0.56671495351937776962531783590$P+0,
     &  D47  = -0.30689499459498916912797304727$P+1,
     &  D48  =  0.23846676565120698287728149680$P+1,
     &  D49  =  0.21170345824450282767155149946$P+1,
     &  D410 = -0.87139158377797299206789907490$P+0,
     &  D411 =  0.22404374302607882758541771650$P+1,
     &  D412 =  0.63157877876946881815570249290$P+0,
     &  D413 = -0.88990336451333310820698117400$P-1,
     &  D414 =  0.18148505520854727256656404962$P+2,
     &  D415 = -0.91946323924783554000451984436$P+1,
     &  D416 = -0.44360363875948939664310572000$P+1)
      PARAMETER (
     &  D51  =  0.10427508642579134603413151009$P+2,
     &  D56  =  0.24228349177525818288430175319$P+3,
     &  D57  =  0.16520045171727028198505394887$P+3,
     &  D58  = -0.37454675472269020279518312152$P+3,
     &  D59  = -0.22113666853125306036270938578$P+2,
     &  D510 =  0.77334326684722638389603898808$P+1,
     &  D511 = -0.30674084731089398182061213626$P+2,
     &  D512 = -0.93321305264302278729567221706$P+1,
     &  D513 =  0.15697238121770843886131091075$P+2,
     &  D514 = -0.31139403219565177677282850411$P+2,
     &  D515 = -0.93529243588444783865713862664$P+1,
     &  D516 =  0.35816841486394083752465898540$P+2)
      PARAMETER (
     &  D61 =  0.19985053242002433820987653617$P+2,
     &  D66 = -0.38703730874935176555105901742$P+3,
     &  D67 = -0.18917813819516756882830838328$P+3,
     &  D68 =  0.52780815920542364900561016686$P+3,
     &  D69 = -0.11573902539959630126141871134$P+2,
     &  D610 =  0.68812326946963000169666922661$P+1,
     &  D611 = -0.10006050966910838403183860980$P+1,
     &  D612 =  0.77771377980534432092869265740$P+0,
     &  D613 = -0.27782057523535084065932004339$P+1,
     &  D614 = -0.60196695231264120758267380846$P+2,
     &  D615 =  0.84320405506677161018159903784$P+2,
     &  D616 =  0.11992291136182789328035130030$P+2)
      PARAMETER (
     &  D71  = -0.25693933462703749003312586129$P+2,
     &  D76  = -0.15418974869023643374053993627$P+3,
     &  D77  = -0.23152937917604549567536039109$P+3,
     &  D78  =  0.35763911791061412378285349910$P+3,
     &  D79  =  0.93405324183624310003907691704$P+2,
     &  D710 = -0.37458323136451633156875139351$P+2,
     &  D711 =  0.10409964950896230045147246184$P+3,
     &  D712 =  0.29840293426660503123344363579$P+2,
     &  D713 = -0.43533456590011143754432175058$P+2,
     &  D714 =  0.96324553959188282948394950600$P+2,
     &  D715 = -0.39177261675615439165231486172$P+2,
     &  D716 = -0.14972683625798562581422125276$P+3)
      IF (TSS(NS).NE.'UEDX') CALL UYPRO1('UEDX',-1)
      GO TO (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18) ISP(NS)
    1 CONTINUE
      NE1=0
      NE2=MEL
      NE3=2*MEL
      NE4=3*MEL
      NE5=4*MEL
      NE6=5*MEL
      NE7=6*MEL
      NE8=7*MEL
      NE9=8*MEL
      NE10=9*MEL
      NE11=10*MEL
      NE12=11*MEL
      NE13=12*MEL
      NE14=13*MEL
      NE15=14*MEL
      NE16=15*MEL
      NE17=16*MEL
      NE18=17*MEL
      NSTP=0
      IASTI=0
      NONSTI=0
      HLAMB=ZERO
      FACOLD=1.$P-4
      POSNEG=SIGN(ONE,TAMAX-TA)
      IF (MED1.EQ.1) THEN
      KA=2
      ELSE
      NEGPOS=POSNEG
      IF (NEGPOS.GT.0) KA=1
      IF (NEGPOS.LT.0) KA=NA
      ENDIF
      TAO=TA
C
C     INITIALISATION
C
      CALL UXVCOP(MEL,YA,YAO)
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE1+1))
      HEMAX=ABS(HEMAX)
      IORD=8
      IF (MED1.EQ.2.OR.HE.NE.ZERO) GO TO 24
C
C     INITIAL STEPSIZE
C
      DNF=ZERO
      DNY=ZERO
      DO 21 I=1,MEL
      SK=TOLA+TOLR*ABS(YAO(I))
      DNF=DNF+(ES(NE1+I)/SK)**2
      DNY=DNY+(YAO(I)/SK)**2
  21  CONTINUE
      IF (DNF.LE.1.$P-10.OR.DNY.LE.1.$P-10) THEN
        HE=1.0$P-6
      ELSE
        HE=SQRT(DNY/DNF)*0.01$P 0
      END IF
      HE=MIN(HE,HEMAX)
      HE=SIGN(HE,POSNEG)
      DO 22 I=1,MEL
      YA(I)=YAO(I)+HE*ES(NE1+I)
  22  CONTINUE
      TA=TAO+HE
      CALL UYSET0(1,3)
      RETURN
    3 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE2+1))
      DER2=ZERO
      DO 23 I=1,MEL
      SK=TOLA+TOLR*ABS(YAO(I))
      DER2=DER2+((ES(NE2+I)-ES(NE1+I))/SK)**2
  23  CONTINUE
      DER2=SQRT(DER2)/HE
      DER12=MAX(ABS(DER2),SQRT(DNF))
      IF (DER12.LE.1.$P-15) THEN
        HE1=MAX(1.0$P-6,ABS(HE)*1.0$P-3)
      ELSE
        HE1=(0.01$P 0/DER12)**(ONE/IORD)
      END IF
      HE=MIN(100.0$P 0*ABS(HE),HE1,HEMAX)
      HE=SIGN(HE,POSNEG)
  24  CONTINUE
      REJECT=.FALSE.
      IF (MED.NE.0) CALL UNVDP8(NA,NAL,NE,MEL,KA,AT,AY,YAO,TA,TAO,HE,
     & IER,-MED)
C
C     BASIC INTEGRATION STEP
C
  25  CONTINUE
      IF (UXVMAX(NE,YA).GE.FMAX) GO TO 49
      IF (MED1.EQ.2) THEN
      IER=0
      KA=KA+NEGPOS
      IF (KA.LT.1.OR.KA.GT.NA) GO TO 50
      HE=AT(KA)-TAO
      ELSE
      IF (NSTP.GT.MSTP) GOTO 48
      IF (0.1$P 0*ABS(HE).LE.ABS(TAO)*ETA0)GOTO 47
      IF ((TAO+1.01$P 0*HE-TAMAX)*POSNEG.GT.ZERO) THEN
        HE=TAMAX-TAO
        LAST=.TRUE.
      ELSE
        LAST=.FALSE.
      END IF
      END IF
      NSTP=NSTP+1
C
C     THE FIRST 12 STAGES
C
      DO 26 I=1,MEL
      YA(I)=YAO(I)+HE*A21*ES(NE1+I)
  26  CONTINUE
      TA=TAO+C2*HE
      CALL UYSET0(1,4)
      RETURN
    4 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE2+1))
      DO 27 I=1,MEL
      YA(I)=YAO(I)+HE*(A31*ES(NE1+I)+A32*ES(NE2+I))
  27  CONTINUE
      TA=TAO+C3*HE
      CALL UYSET0(1,5)
      RETURN
    5 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE3+1))
      DO 28 I=1,MEL
      YA(I)=YAO(I)+HE*(A41*ES(NE1+I)+A43*ES(NE3+I))
  28  CONTINUE
      TA=TAO+C4*HE
      CALL UYSET0(1,6)
      RETURN
    6 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE4+1))
      DO 29 I=1,MEL
      YA(I)=YAO(I)+HE*(A51*ES(NE1+I)+A53*ES(NE3+I)+A54*ES(NE4+I))
  29  CONTINUE
      TA=TAO+C5*HE
      CALL UYSET0(1,7)
      RETURN
    7 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE5+1))
      DO 30 I=1,MEL
      YA(I)=YAO(I)+HE*(A61*ES(NE1+I)+A64*ES(NE4+I)+A65*ES(NE5+I))
  30  CONTINUE
      TA=TAO+C6*HE
      CALL UYSET0(1,8)
      RETURN
    8 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE6+1))
      DO 31 I=1,MEL
      YA(I)=YAO(I)+HE*(A71*ES(NE1+I)+A74*ES(NE4+I)+A75*ES(NE5+I)+
     & A76*ES(NE6+I))
  31  CONTINUE
      TA=TAO+C7*HE
      CALL UYSET0(1,9)
      RETURN
    9 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE7+1))
      DO 32 I=1,MEL
      YA(I)=YAO(I)+HE*(A81*ES(NE1+I)+A84*ES(NE4+I)+A85*ES(NE5+I)+
     & A86*ES(NE6+I)+A87*ES(NE7+I))
  32  CONTINUE
      TA=TAO+C8*HE
      CALL UYSET0(1,10)
      RETURN
   10 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE8+1))
      DO 33 I=1,MEL
      YA(I)=YAO(I)+HE*(A91*ES(NE1+I)+A94*ES(NE4+I)+A95*ES(NE5+I)+
     & A96*ES(NE6+I)+A97*ES(NE7+I)+A98*ES(NE8+I))
  33  CONTINUE
      TA=TAO+C9*HE
      CALL UYSET0(1,11)
      RETURN
   11 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE9+1))
      DO 34 I=1,MEL
      YA(I)=YAO(I)+HE*(A101*ES(NE1+I)+A104*ES(NE4+I)+A105*ES(NE5+I)+
     & A106*ES(NE6+I)+A107*ES(NE7+I)+A108*ES(NE8+I)+A109*ES(NE9+I))
  34  CONTINUE
      TA=TAO+C10*HE
      CALL UYSET0(1,12)
      RETURN
   12 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE10+1))
      DO 35 I=1,MEL
      YA(I)=YAO(I)+HE*(A111*ES(NE1+I)+A114*ES(NE4+I)+A115*ES(NE5+I)+
     & A116*ES(NE6+I)+A117*ES(NE7+I)+A118*ES(NE8+I)+A119*ES(NE9+I)+
     & A1110*ES(NE10+I))
  35  CONTINUE
      TA=TAO+C11*HE
      CALL UYSET0(1,13)
      RETURN
   13 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE2+1))
      DO 36 I=1,MEL
      YA(I)=YAO(I)+HE*(A121*ES(NE1+I)+A124*ES(NE4+I)+A125*ES(NE5+I)+
     & A126*ES(NE6+I)+A127*ES(NE7+I)+A128*ES(NE8+I)+A129*ES(NE9+I)+
     & A1210*ES(NE10+I)+A1211*ES(NE2+I))
  36  CONTINUE
      TA=TAO+HE
      CALL UYSET0(1,14)
      RETURN
   14 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE3+1))
      DO 37 I=1,MEL
      ES(NE4+I)=B1*ES(NE1+I)+B6*ES(NE6+I)+B7*ES(NE7+I)+
     & B8*ES(NE8+I)+B9*ES(NE9+I)+B10*ES(NE10+I)+B11*ES(NE2+I)+
     & B12*ES(NE3+I)
      ES(NE5+I)=YAO(I)+HE*ES(NE4+I)
  37  CONTINUE
      IF (MED1.EQ.2) THEN
        NACC=NACC+1
        DO 57 I=1,MEL
        ES(NE1+I)=ES(NE4+I)
        YAO(I)=ES(NE5+I)
  57    CONTINUE
        IF (MED2.EQ.1) CALL UXVCOP(NE,AY((KA-1)*NE+1),YAO)
        IF (MED2.EQ.2) CALL UXVMED(NE,AY((KA-1)*NE+1),YAO,YAO)
        CALL UNVDP8(NA,NAL,NE,MEL,KA,AT,AY,YAO,TA,TAO,HE,IER,3)
        TAO=AT(KA)
        GO TO 25
      ENDIF
C
C     ERROR ESTIMATION
C
      ERR=ZERO
      ERR2=ZERO
      DO 38 I=1,MEL
      SK=TOLA+TOLR*MAX(ABS(YAO(I)),ABS(ES(NE5+I)))
      ERRI=ES(NE4+I)-BHH1*ES(NE1+I)-BHH2*ES(NE9+I)-BHH3*ES(NE3+I)
      ERR2=ERR2+(ERRI/SK)**2
      ERRI=ER1*ES(NE1+I)+ER6*ES(NE6+I)+ER7*ES(NE7+I)+
     & ER8*ES(NE8+I)+ER9*ES(NE9+I)+ER10*ES(NE10+I)+ER11*ES(NE2+I)+
     & ER12*ES(NE3+I)
      ERR=ERR+(ERRI/SK)**2
  38  CONTINUE
      DENO=ERR+0.01$P 0*ERR2
      IF (DENO.LE.ZERO) DENO=ONE
      ERR=ABS(HE)*ERR*SQRT(ONE/(MEL*DENO))
      FAC11=ERR**(1.0$P 0/8.0$P 0-BET3*0.2$P 0)
      FAC=FAC11/FACOLD**BET3
      FAC=MAX(ONE/TET2,MIN(ONE/TET1,FAC/TET3))
      HNEW=HE/FAC
      IF(ERR.GT.ONE) THEN
C
C     STEP IS REJECTED
C
        HNEW=HE/MIN(ONE/TET1,FAC11/TET3)
        REJECT=.TRUE.
        IF(NACC.GE.1)NREJ=NREJ+1
        HE=HNEW
        GOTO 25
      END IF
C
C     STEP IS ACCEPTED
C
      FACOLD=MAX(ERR,1.0$P-4)
      NACC=NACC+1
      CALL UXVCOP(MEL,YA,ES(NE11+1))
      CALL UXVCOP(MEL,ES(NE5+1),YA)
      CALL UYSET0(1,15)
      RETURN
   15 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE4+1))
C
C     STIFFNESS DETECTION
C
      IF (MOD(NACC,MED3).EQ.0.OR.IASTI.GT.0) THEN
        STNUM=ZERO
        STDEN=ZERO
        DO 39 I=1,MEL
        STNUM=STNUM+(ES(NE4+I)-ES(NE3+I))**2
        STDEN=STDEN+(ES(NE5+I)-ES(NE11+I))**2
  39    CONTINUE
        IF (STDEN.GT.ZERO) HLAMB=ABS(HE)*SQRT(STNUM/STDEN)
        IF (HLAMB.GT.6.1$P 0) THEN
          NONSTI=0
          IASTI=IASTI+1
          IF (IASTI.EQ.15) GO TO 46
        ELSE
          NONSTI=NONSTI+1
          IF (NONSTI.EQ.6) IASTI=0
        END IF
      END IF
C
C     SAVE THE FIRST FUNCTION EVALUATIONS
C
      IF (MED.LT.2) GO TO 54
      IF (HE*(TA-AT(KA)).LT.ZERO) GO TO 54
      MEL=NE
      TAPH=TA
      DO 40 I=1,MEL
      ES(NE11+I)=YAO(I)
      YDIFF=ES(NE5+I)-YAO(I)
      ES(NE12+I)=YDIFF
      BSPL=HE*ES(NE1+I)-YDIFF
      ES(NE13+I)=BSPL
      ES(NE14+I)=YDIFF-HE*ES(NE4+I)-BSPL
      ES(NE15+I)=D41*ES(NE1+I)+D46*ES(NE6+I)+D47*ES(NE7+I)+
     &            D48*ES(NE8+I)+D49*ES(NE9+I)+D410*ES(NE10+I)+
     &            D411*ES(NE2+I)+D412*ES(NE3+I)
      ES(NE16+I)=D51*ES(NE1+I)+D56*ES(NE6+I)+D57*ES(NE7+I)+
     &            D58*ES(NE8+I)+D59*ES(NE9+I)+D510*ES(NE10+I)+
     &            D511*ES(NE2+I)+D512*ES(NE3+I)
      ES(NE17+I)=D61*ES(NE1+I)+D66*ES(NE6+I)+D67*ES(NE7+I)+
     &            D68*ES(NE8+I)+D69*ES(NE9+I)+D610*ES(NE10+I)+
     &            D611*ES(NE2+I)+D612*ES(NE3+I)
      ES(NE18+I)=D71*ES(NE1+I)+D76*ES(NE6+I)+D77*ES(NE7+I)+
     &            D78*ES(NE8+I)+D79*ES(NE9+I)+D710*ES(NE10+I)+
     &            D711*ES(NE2+I)+D712*ES(NE3+I)
  40  CONTINUE
C
C     THE NEXT THREE FUNCTION EVALUATIONS
C
      DO 41 I=1,MEL
      YA(I)=YAO(I)+HE*(A141*ES(NE1+I)+A147*ES(NE7+I)+
     &      A148*ES(NE8+I)+A149*ES(NE9+I)+A1410*ES(NE10+I)+
     &      A1411*ES(NE2+I)+A1412*ES(NE3+I)+A1413*ES(NE4+I))
  41  CONTINUE
      TA=TAO+C14*HE
      CALL UYSET0(1,16)
      RETURN
   16 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE10+1))
      DO 42 I=1,MEL
      YA(I)=YAO(I)+HE*(A151*ES(NE1+I)+A156*ES(NE6+I)+
     &      A157*ES(NE7+I)+A158*ES(NE8+I)+A1511*ES(NE2+I)+
     &      A1512*ES(NE3+I)+A1513*ES(NE4+I)+A1514*ES(NE10+I))
  42  CONTINUE
      TA=TAO+C15*HE
      CALL UYSET0(1,17)
      RETURN
   17 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE2+1))
      DO 43 I=1,MEL
      YA(I)=YAO(I)+HE*(A161*ES(NE1+I)+A166*ES(NE6+I)+
     &      A167*ES(NE7+I)+A168*ES(NE8+I)+A169*ES(NE9+I)+
     &      A1613*ES(NE4+I)+A1614*ES(NE10+I)+A1615*ES(NE2+I))
  43  CONTINUE
      TA=TAO+C16*HE
      CALL UYSET0(1,18)
      RETURN
   18 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE3+1))
C
C     FINAL PREPARATION
C
      DO 44 I=1,MEL
      ES(NE15+I)=HE*(ES(NE15+I)+D413*ES(NE4+I)+
     &            D414*ES(NE10+I)+D415*ES(NE2+I)+D416*ES(NE3+I))
      ES(NE16+I)=HE*(ES(NE16+I)+D513*ES(NE4+I)+
     &            D514*ES(NE10+I)+D515*ES(NE2+I)+D516*ES(NE3+I))
      ES(NE17+I)=HE*(ES(NE17+I)+D613*ES(NE4+I)+
     &            D614*ES(NE10+I)+D615*ES(NE2+I)+D616*ES(NE3+I))
      ES(NE18+I)=HE*(ES(NE18+I)+D713*ES(NE4+I)+
     &            D714*ES(NE10+I)+D715*ES(NE2+I)+D716*ES(NE3+I))
  44  CONTINUE
      TA=TAPH
      MEL=NE2
  54  DO 45 I=1,MEL
      ES(NE1+I)=ES(NE4+I)
      YAO(I)=ES(NE5+I)
  45  CONTINUE
      IF (MED.EQ.1) CALL UNVDP8(NA,NAL,NE,MEL,KA,AT,AY,YAO,TA,TAO,HE,
     &      IER,MED)
      IF (MED.GE.2) THEN
      IF (HE*(TA-AT(KA)).GE.ZERO) THEN
        CALL UNVDP8(NA,NAL,NE,MEL,KA,AT,AY,ES(NE11+1),TA,TAO,HE,IER,MED)
      ENDIF
      ENDIF
      IF (IER.EQ.-11) GO TO 50
      TAO=TA
C
C     NORMAL EXIT
C
      IF (LAST) THEN
        HE=HNEW
        IER=0
        GO TO 50
      END IF
      IF(ABS(HNEW).GT.HEMAX)HNEW=POSNEG*HEMAX
      IF(REJECT)HNEW=POSNEG*MIN(ABS(HNEW),ABS(HE))
      REJECT=.FALSE.
      HE=HNEW
      GOTO 25
C
C     FAIL EXIT
C
  46  IER=-4
      TEDX='STIFF EQ'
      GO TO 50
  47  IER=-3
      TEDX='SMALL ST'
      GO TO 50
  48  IER=-2
      TEDX='STEP LIM'
      GO TO 50
  49  IER=-1
      TEDX='UNSTABLE'
  50  CALL UYEPI1(-1)
      RETURN
      END
