$INTERFACE UEVEXA                                             98/12/01

  $HELP
    NUMBER OF STABILITY CHECKS IN SELECTED LINES
    OF THE EXTRAPOLATION TABLE
  $CHECK(MSTAB,1,'1\2\3\4\5',4)
  $HELP
    NUMBER OF CHECKED LINES OF THE EXTRAPOLATION TABLE
  $CHECK(NSTAB,2,'1\2\3\4\5\6\7\8\9',4)
  $IF(NSTAB>MED3) $NSTAB=MED3
  $HELP
    SAFETY FACTOR FOR THE STEPSIZE CONTROL
  $CHECK(SAF1,'0.65$P 0',REAL,4)
  $HELP
    SAFETY FACTOR FOR THE STEPSIZE CONTROL
  $CHECK(SAF2,'0.94$P 0',REAL,4)
  $HELP
    PARAMETER FOR THE STEPSIZE REDUCTION
  $CHECK(SAF3,'0.5$P 0',REAL,4)
  $HELP
    PARAMETER FOR THE STEPSIZE SELECTION
  $CHECK(FAC1,'2.0$P-2',REAL,4)
  $HELP
    PARAMETER FOR THE STEPSIZE SELECTION
  $CHECK(FAC2,'4.0$P 0',REAL,4)
  CALL UEVEXA(MAL,NE,MEL,$YA,$YAO,$EF,$ES,$IES,$SAF1,$SAF2,$SAF3,$FAC1&
,$FAC2,$MSTAB,$NSTAB,L1,L2,$ETA0,TA,TAO,HE,HEMAX,ERR,ERRO,MED,MED3&
,KAP)
$REM  $ADD(REAL,'\TA\TAO\HE\HEMAX\ERR\ERRO')
$REM  $ADD(INTEGER,'\MAL\NE\MEL')

  $EXIT

* IMPLEMENTATION

* SUBROUTINE UEVEXA                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF THE INITIAL VALUE PROBLEM FOR A SET OF THE FIRST ORDER
* ORDINARY DIFFERENTIAL EQUATIONS USING THE FIFTH ORDER DORMAN AND
* PRINCE INTEGRATION FORMULA.
*
* PARAMETERS :
*  IU  MAL
*  II  NE  DECLARED NUMBER OF DIFFERENTIAL EQUATIONS.
*  II  MEL  ACTUAL NUMBER OF DIFFERENTIAL EQUATIONS.
*  RI  AT(NA)  MESH POINTS.
*  RO  AY(NA*NE)  FIELD OF SOLUTION VECTORS IN THE MESH POINTS.
*  RU  YA(MEL)  SOLUTION VECTOR.
*  RA  YAO(MEL)  OLD SOLUTION VECTOR.
*  RI  EF(MEL)  RIGHT HAND SIDE VECTOR.
*  RA  ES(12*MEL)  AUXILIARY VECTOR.
*  RI  ETA0  MACHINE PRECISION.
*  RU  TA  VALUE OF THE INTEGRATION PARAMETER.
*  RU  TAO  OLD VALUE OF THE INTEGRATION PARAMETER.
*  RU  HE  ACTUAL STEPSIZE.
*  RU  HEMAX  MAXIMUM STEPSIZE.
*  RO  ERR  INTEGRATION ERROR.
*  RO  ERRO  OLD INTEGRATION ERROR.
*  II  MED  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  II  MED3  NUMBER OF STEPS PRECEEDING TEST OF STIFTNESS.
*  IU  KAP  AUXILIARY VARIABLE.
*
* COMMON DATA :
*  RI  TOLR  RELATIVE PRECISION.
*  RI  TOLA  ABSOLUTE PRECISION.
*
* SUBPROGRAMS USED :
*  S   UNVDP5  INTERPOLATION IN MESH POINTS.
*  S   UXVCOP  COPYING OF A VECTOR.
*
* METHOD :
* E.HAIRER, S.P.NORSETT, G.WANNER: SOLVING ORDINARY DIFFERENTIAL
* EQUATIONS I. SPRINGER SERIES IN COMPUTATIONAL MATHEMATICS.
* SPRINGER VERLAG, BERLIN 1980.
*
      SUBROUTINE UEVEXA(MAL,NE,MEL,YA,YAO,EF,ES,IES,SAF1,SAF2,SAF3,
     & FAC1,FAC2,MSTAB,NSTAB,ATOV,REJECT,ETA0,TA,TAO,HE,HEMAX,ERR,
     & ERRO,MED,MED3,KAP)
      $INCLUDE('UMCOMM')
      INTEGER MAL,NE,MEL,MED,MED3,IES(MED3),MSTAB,NSTAB,KAP
      $FLOAT ES(*),YA(MEL),YAO(MEL),EF(MEL),SAF1,SAF2,SAF3,FAC1,FAC2,
     & ETA0,TA,TAO,HE,HEMAX,ERR,ERRO
      LOGICAL ATOV,REJECT
      $FLOAT HJ,YS,DELT1,DELT2,QUOT,DBLENJ,FAC,T1I,EXPO,FACMIN
      INTEGER NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,I,L,LFS,M,MM,
     & NJMID
      SAVE NE1,NE2,NE3,NE4,NE5,NE6,NE7,NE8,NE9,NE10,LFS,M,MM,NJMID
      SAVE HJ
C
C     THIS SUBROUTINE COMPUTES THE J-TH LINE OF THE
C     EXTRAPOLATION TABLE AND PROVIDES AN ESTIMATION
C     OF THE OPTIMAL STEPSIZE
C
      IF (TSS(NS).NE.'UEDY') CALL UYPRO1('UEDY',-1)
      GO TO (1,2,3) ISP(NS)
    1 CONTINUE
      LFS=2*MED3*MED3+MED3
      NE1=0
      NE2=MEL
      NE3=2*MEL
      NE4=3*MEL
      NE5=4*MEL
      NE6=NE5+MED3*MEL
      NE7=NE6+MED3
      NE8=NE7+MED3
      NE9=NE8+MED3
      NE10=NE9+LFS*NE
      TAO=TA
      HJ=HE/IES(KAP)
C
C     EULER STARTING STEP
C
      DO 30 I=1,MEL
      ES(NE2+I)=YAO(I)
  30  YA(I)=YAO(I)+HJ*ES(NE3+I)
C
C     EXPLICIT MIDPOINT RULE
C
      M=IES(KAP)-1
      NJMID=IES(KAP)/2
      MM=1
  35  IF (MM.GT.M) GO TO 36
C
C     OUTPUT
C
      IF (MED.GE.2.AND.MM.EQ.NJMID) THEN
        DO 31 I=1,NE
  31    ES(NE10+MED3*(I-1)+KAP)=YA(I)
      END IF
C
      TA=TAO+HJ*MM
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE1+1))
C
C     OUTPUT
C
      IF (MED.GE.2.AND.ABS(MM-NJMID).LE.2*KAP-1) THEN
        MAL=MAL+1
        DO 32 I=1,NE
 32     ES(NE9+LFS*(I-1)+MAL)=ES(NE1+I)
      END IF
C
      DO 34 I=1,MEL
      YS=ES(NE2+I)
      ES(NE2+I)=YA(I)
  34  YA(I)=YS+2.D0*HJ*ES(NE1+I)
      IF (MM.LE.MSTAB.AND.KAP.LE.NSTAB) THEN
C
C     STABILITY CHECK
C
        DELT1=0.D0
        DO 21 I=1,MEL
  21    DELT1=DELT1+(ES(NE3+I)/ES(NE4+I))**2
        DELT2=0.D0
        DO 26 I=1,MEL
  26    DELT2=DELT2+((ES(NE1+I)-ES(NE3+I))/ES(NE4+I))**2
        QUOT=DELT2/MAX(ETA0,DELT1)
        IF (QUOT.GT.4.D0) THEN
          GOTO 79
        END IF
      END IF
      MM=MM+1
      GO TO 35
C
C     FINAL SMOOTHING STEP
C
  36  TA=TAO+HE
      CALL UYSET0(1,3)
      RETURN
    3 CONTINUE
      CALL UXVCOP(MEL,EF,ES(NE1+1))
C
C     OUTPUT
C
      IF (MED.GE.2.AND.NJMID.LE.2*KAP-1) THEN
        MAL=MAL+1
        DO 39 I=1,NE
 39     ES(NE9+LFS*(I-1)+MAL)=ES(NE1+I)
      END IF
C
      DO 40 I=1,MEL
  40  ES(NE5+MED3*(I-1)+KAP)=(ES(NE2+I)+YA(I)+
     &   HJ*ES(NE1+I))/2.D0
C
C     POLYNOMIAL EXTRAPOLATION
C
      IF (KAP.EQ.1)THEN
        TA=TAO
        GO TO 80
      END IF
      DBLENJ=IES(KAP)
      DO 60 L=KAP,2,-1
      FAC=(DBLENJ/IES(L-1))**2-1.D0
      DO 60 I=1,MEL
      ES(NE5+MED3*(I-1)+L-1)=ES(NE5+MED3*(I-1)+L)+
     &  (ES(NE5+MED3*(I-1)+L)-ES(NE5+MED3*(I-1)+L-1))/FAC
  60  CONTINUE
      ERR=0.D0
C
C     SCALING
C
      DO 65 I=1,MEL
      T1I=MAX(ABS(YAO(I)),ABS(ES(NE5+MED3*(I-1)+1)))
      ES(NE4+I)=TOLA + TOLR*T1I
  65  ERR=ERR+((ES(NE5+MED3*(I-1)+1)-ES(NE5+
     &    MED3*(I-1)+2))/ES(NE4+I))**2
      ERR=SQRT(ERR/MEL)
      IF (ERR*ETA0.GE.1.D0) GOTO 79
      IF (KAP.GT.2.AND.ERR.GE.ERRO) GOTO 79
      ERRO=MAX(4*ERR,1.D0)
C
C     COMPUTE OPTIMAL STEPSIZES
C
      EXPO=1.D0/(2*KAP-1)
      FACMIN=FAC1**EXPO
      FAC=MIN(FAC2/FACMIN,MAX(FACMIN,(ERR/SAF1)**EXPO/SAF2))
      FAC=1.D0/FAC
      ES(NE6+KAP)=MIN(HE*FAC,HEMAX)
      ES(NE7+KAP)=ES(NE8+KAP)/ES(NE6+KAP)
      TA=TAO
      GO TO 80
  79  ATOV=.TRUE.
      HE=HE*SAF3
      REJECT=.TRUE.
      TA=TAO
  80  CALL UYEPI1(-1)
      RETURN
      END
