* SUBROUTINE UKQGLH                 ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* THIS SUBROUTINE COMPUTES PRODUCT WR01(1,...,LINP-M)=TRANS(Z)*H*
* Z*C(1,...,LINP-M) WHERE Z=TRANS(-TRANS(S)*TRANS(B)**-1/I).
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE SPARSE MATRIX
*  II  M  NUMBER OF ROWS OF THE SPARSE MATRIX.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  MCOLS DECLARED NUMBER OF VARIABLES.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NZA SIZE OF THE INDEX ARRAY RNR.
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NUP  LENGTH OF THE ARRAYS LI,LJ,CM.
*  II  NEL  NUMBER OF TRIPLES IN ARRAYS (LI,LJ,CM).
*  II  PTL(MROWS) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(MROWS) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  II  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  II  PTR(MROWS) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  II  LENR(MROWS) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  RNR(NZA) COLUMNS OF THE ORIGINAL MATRIX A.
*  II  PRNR(MCOLS+1) INDEX VECTOR FOR THE ARRAY RNR.
*  II  ORDC(MROWS) COLUMN PERMUTATION.
*  II  ORDR(MROWS) ROW PERMUTATION.
*  II  ORDL(MROWS) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  II  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  II  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  RI  AU(NZ) NUMERICAL VALUES OF THE FACTORS.
*  RI  A(NZA) NUMERICAL VALUES OF THE MATRIX.
*  RO  WR01(MCOLS) WR01 = TRANS(Z)*H*Z*C(1,...,LINP-M) : Z IS THE BASIS
*              OF THE SPACE  ORTHOGONAL TO THE ONE FORMED BY THE ROWS
*              OF THE MATRIX A.
*  RA  WR02(MCOLS) AUXILIARY ARRAY.
*  II  BCOL(MCOLS) PERMUTATION ARRAY: FIRST M COMPONENTS FORM BASIS,
*              NEXT LINP-M FORM SUPERBASIS AND REST CORRESPOND TO
*              THE NON-BASIC VARIABLES.
*  II  INBCOL(MCOLS) VECTOR WITH COMPONENTS :
*                INBCOL(K)=I WHEN EXISTS K=BCOL(I) AND I<=LINP,
*                INBCOL(K)=0 OTHERWISE.
*  RI  C(MCOLS) INPUT VECTOR THAT IS MULTIPLIED.
*  II  PH(MROWS+1) POINTER VECTOR OF SH,H.
*  II  SH(MMAX) INDEX VECTOR OF THE HESSIAN
*  RI  H(MMAX) NUMERICAL VALUES OF THE HESSIAN MATRIX.
*  II  LINP NUMBER OF BASICS AND SUPERBASICS.
*
      SUBROUTINE UKQGLH(N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,MMAX,PTL,
     & LENL,CNR,PTR,LENR,RNR,PRNR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,A,WR01,
     & WR02,BCOL,INBCOL,C,PH,SH,H,LINP)
      INTEGER N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,MMAX,PTL(MROWS),
     & LENL(MROWS),CNR(NZ),PTR(MROWS),LENR(MROWS),RNR(NZA),
     & PRNR(MCOLS+1),ORDC(MROWS),ORDR(MROWS),ORDL(MROWS),LI(NUP),
     & LJ(NUP),BCOL(MCOLS),INBCOL(MCOLS),PH(MCOLS+1),SH(MMAX),LINP
      $FLOAT CM(NUP),AU(NAU),A(NZA),WR01(MCOLS),WR02(MCOLS),C(MCOLS),
     & H(MMAX)
      INTEGER INV
C
C     WR01=Z*C
C
      CALL UKQGLG(M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,PTL,LENL,CNR,PTR,
     & LENR,RNR,PRNR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,A,WR01,WR02,BCOL,C,LINP)
C
C     WR02=H*WR01
C
      CALL UKQSMM(N,MMAX,LINP,BCOL,INBCOL,PH,SH,H,WR01,WR02)
C
C     WR01=TRANS(Z)*WR02
C
      INV=1
      CALL UKQGLT(M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,PTL,LENL,CNR,PTR,
     & LENR,RNR,PRNR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,A,WR01,WR02,BCOL,WR02,
     & INV,LINP)
      RETURN
      END
