* SUBROUTINE UKQSMM                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE:
* MULTIPLICATION OF A (NS X NS) SUBMATRIX OF A SYMMETRIX MATRIX H
* BY THE VECTOR WR01(1,...,NS). RESULT IS STORED IN WR02(1,...,NS).
* H HAS STORED ONLY ONE TRIANGLE ( COLUMNS OF LOWER TRIANGLE).
* SUBMATRIX CORRESPONDS TO THE FIRST NS COMPONENTS OF THE VECTOR
* BCOL(1,...,N).
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX.
*  II  NHES SIZE OF THE PRINCIPAL MATRIX ARRAYS (SH,H).
*  II  NS SIZE OF THE SUBMATRIX THAT IS USED.
*  II  BCOL(N) VECTOR THAT DETERMINES THE SUBMATRIX.
*  II  INBCOL(N) VECTOR WITH COMPONENTS :
*                INBCOL(K)=I WHEN EXISTS K=BCOL(I) AND I<=NS,
*                INBCOL(K)=0 OTHERWISE.
*  II  PH(N+1) INDEX VECTOR OF THE INPUT SPARSE MATRIX
*  II  SH(NHES) COLUMN INDICES OF THE INPUT MATRIX
*  RI  H(NHES) ELEMENTS OF THE LOWER HALF OF THE SYMMETRIC MATRIX.
*  RI  WR01(N) INPUT VECTOR THAT IS MULTIPLIED.
*  RO  WR02(N) OUTPUT VECTOR WHERE WR02(NS):= SUBM OF H * WR01(NS)
*
      SUBROUTINE UKQSMM(N,NHES,NS,BCOL,INBCOL,PH,SH,H,WR01,WR02)
      INTEGER N,NHES,NS
      INTEGER BCOL(N),INBCOL(N),PH(N+1),SH(NHES)
      $FLOAT H(NHES),WR01(N),WR02(N)
      INTEGER I,J,INB,JSPH,JLEN,K,JSTRT,JSTOP
      $FLOAT TEMP
C
C     ZERO THE VECTOR WR02
C
      DO 100 I=1,NS
      WR02(I)=0.0$P 0
100   CONTINUE
C
C     MULTIPLYING
C
      DO 300 I=1,NS
      JSTRT=BCOL(I)
      TEMP=0.0$P 0
      JSPH=PH(JSTRT)
      JSTOP=PH(JSTRT+1)-1
      JLEN=JSTOP+1-JSPH
          DO 200 J=JSPH,JSTOP
          K=SH(J)
          INB=INBCOL(K)
          IF(INB.EQ.0) GO TO 200
          IF(INB.GT.NS) GO TO 200
C
C     CONTRIBUTION OF THE UPPER HALF
C
          TEMP=TEMP+WR01(INB)*H(J)
C
C     CONTRIBUTION OF THE LOWER HALF
C
          WR02(INB)=WR02(INB)+H(J)*WR01(I)
200       CONTINUE
C
C     ADD THE CONTRIBUTION AF THE UPPER HALF OF THE MATRIX.
C     SUBTRACT THE DOUBLY ADDED DIAGONAL .
C
      IF(JLEN.GT.0) WR02(I)=WR02(I)+TEMP-H(JSPH)*WR01(I)
300   CONTINUE
      RETURN
      END
