$INTERFACE UKSGLM                                             98/12/01

  $HELP
    PRECISION FOR EITHER LINEAR OR QUADRATIC PROGRAMMING SUBROUTINE
  $CHECK(EPS7,'1.0$P-7',REAL,4)
  $HELP
    TOLERANCE FOR ACTIVE CONSTRAINTS
  $CHECK(EPS9,'1.0$P-9',REAL,4)
  $HELP
    MAXIMUM NUMBER OF SIMPLEX ITRATIONS
  $CHECK(MIS,1000,INT,4,NO)
  CALL UKSGLM(NF,N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,IBEG,IBGR&
,IENDLU,$RNR,$PRNR,$A,$XL,$XU,$IX,$WN01,$BCOL,$G,$X,$PTL,$LENL&
,$PTC,$LENC,$PTR,$LENR,$CNR,$ORDC,$ORDR,$ORDL,$RIGHT,$UP,$GG&
,$WR01,$AU,$LI,$LJ,$CM,$WR02,DEL1,DEL2,EPS7,EPS9,DMAX,GMAX,UMAX&
,FF,MFP,MLP1,$MIS,INEW,IOLD,KOLD,KINP,MINP,IER)
  $ADD(INTEGER,'\$LI($NUP)\$LJ($NUP)')
  $ADD(REAL,'\$GG($MCOLS)\$CM($NUP)')
$REM  $ADD(INTEGER,'NF\N\M\MROWS\MCOLS\NZ\NZA\NAU\NUP\NEL\IBEG&
\IBGR\IENDLU\MFP\MLP1\INEW\IOLD\KOLD\KINP\MINP\IER')
$REM  $ADD(REAL,'\DEL1\DEL2\EPS7\DMAX\GMAX\UMAX\FF')
  $ADD(INIT)
    NDXX='UKSGLM'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UKSGLM                 ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* FIRST PHASE DRIVER.
*
* PARAMETERS :
*  IU  NF INITIAL NUMBER OF VARIABLES. ON OUTPUT IT IS CHANGED BY
*         ADDING SLACKS.
*  II  N NUMBER OF COLUMNS.
*  II  M NUMBER OF ROWS.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  MCOLS DECLARED NUMBER OF COLUMNS.
*  II  NZ  SIZE OF VECTORS CNR/INR.
*  II  NZA  SIZE OF ARRAYS A/RNR.
*  II  NAU SIZE OF VECTORS AI/AU.
*  II  NUP SIZE OF ARRAYS LI,LJ,CM.
*  IO  NEL  NUMBER OF ELEMENTARY ELLIMINATION MATRICES STORED IN THE
*           TRIPLE: LI,LJ,CM.
*  IO  IBEG  BEGINNING OF THE ROW SCHEME OF THE FACTOR U IN CNR.
*  IO  IBGR  BEGINNING OF THE COLUMN SCHEME IN CNR.
*  IO  IENDLU  END OF THE ROW INDICES OF THE FACTOR U IN CNR.
*  IO  RNR(NZA) COLUMNS OF THE MATRIX OF CONSTRAINTS.
*  IO  PRNR(MCOLS+1) POINTER VECTOR FOR RNR.
*  RO  A(NZA) NUMERICAL VALUES OF THE MATRIX OF CONSTRAINTS.
*  RI  LB(N) LOWER BOUNDS ON VARIABLES.
*  RI  UB(N) UPPER BOUNDS ON VARIABLES.
*  IU  TYPE(MCOLS) TYPES OF THE VARIABLES:
*                                    =1 FOR NONBASIC LOWER BOUNDED
*                                    =2 FOR NONBASIC UPPER BOUNDED
*                                    =3 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS LOWER BOUND
*                                    =4 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS UPPER BOUND
*                                    =5 FOR NONBASIC FIXED VARIABLE
*                                    =6 FOR NONBASIC FREE VARIABLE
*                                    =-1 FOR BASIC LOWER BOUNDED
*                                    =-2 FOR BASIC UPPER BOUNDED
*                                    =-3 FOR BASIC LOWER AND UPPER
*                     BOUNDED VARIABLE
*                                    =-6 FOR BASIC FREE VARIABLE.
*  IA  WN01(MCOLS) AUXILIARY VECTOR.
*  IO  BCOL(MCOLS) M FIRST ELEMENTS CONTAIN COLUMN INDICES OF BASIC
*              COLUMNS.
*  RO  C(MCOLS) COST VECTOR USUALLY USED FOR PHASE-1 PRICING.
*  RO  X(MCOLS) VECTOR OF SOLUTION.
*  IU  PTL(MROWS) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  IU  LENL(MROWS) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  IU  PTR(MROWS) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  IU  LENR(MROWS) LENGTHS OF THE ROWS OF THE FACTOR U.
*  IU  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  IO  ORDC(MROWS) COLUMN PERMUTATION OF THE FACTOR U OF THE BASIS.
*  IO  ORDR(MROWS) ROW PERMUTATION OF THE FACTOR U OF THE BASIS.
*  IO  ORDL(MROWS) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  IO  RIGHT(MROWS) RIGHTMOST POSITION OF THE ROW SPIKE FOR BASIS.
*  IO  UP(MROWS) UPPERMOST POSITION OF THE COLUMN SPIKE FOR BASIS.
*  II  GN(MCOLS) COST FUNCTION OF THE LINEAR PROGRAMMING.
*  RA  WR01(MCOLS) AUXILIARY VECTOR
*  RO  AU(NAU) NUMERICAL VALUES OF THE FACTORS.
*  IO  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  IO  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RO  CM(NUP) UPDATE MULTIPLIERS.
*  RA  WR02(MCOLS) AUXILIARY VECTOR.
*  RO  DEL1  MULTIPLIER BOUND FOR PIVOT LOOK-UP.
*  RO  DEL2  RELATIVE BOUND FOR NEGLECTING SMALL ELEMENTS WHEN
*            CONSTRUCTING INITIAL BASIS.
*  RO  EPS7  BOUND FOR NEGLECTING SMALL SPARSE MATRIX ELEMENTS.
*  RO  EPS9  BOUND FOR NEGLECTING SMALL TOLERANCES.
*  RU  DMAX  MINIMUM ELEMENT OF THE U-DIAGONAL (EARLIER MNU).
*  RU  GMAX  ELEMENT WITH THE MAXIMUM ABSOLUTE VALUE OF THE FACTOR U.
*  RU  UMAX  MAXIMUM ELEMENT OF THE U-DIAGONAL (EARLIER MXU).
*  RO  FF  ORIGINAL FUNCTIONAL VALUE.
*  II  MFP  TYPE OF FEASIBLE POINT: ON OUTPUT FROM THE INITIAL
*           TRANSFORMATION IT IS >0  : =1 WHEN WE WANT A FEASIBLE
*           SOLUTION, =2 WHEN WE WANT AN OPTIMAL ONE, =3 WHEN
*           FEASIBLE SOLUTION IS GIVEN AND WE WANT AN OPTIMAL ONE.
*  RO  MLP1  COUNT OF DEGENERATE PIVOTS AFTER THE LAST NONDEGENERATE
*            ONE.
*  IO  MIS  MAXIMUM NUMBER OF SIMPLEX ITERATIONS.
*  II  INEW INPUT COLUMN NUMBER
*  II  IOLD POSITION OF THE OUTPUT COLUMN IN BCOL.
*  IU  KOLD  NUMBER OF UPDATES AFTER LATEST REFACTORIZATION.
*  IA  KINP NUMBER OF ARTIFICIALS.
*  IO  MINP NUMBER OF SLACKS.
*  IO  IER  ERROR INDICATOR.
*
* COMMON DATA:
*
*  IO  NREF  NUMBER OF REFACTORIZATIONS.
*  II  IEXT  TYPE UF EXTREMUM. IEXT > 0 - MAXIMUM. IEXT <= 0 - MINIMUM.
*  IA  ITERL TERMINATION INDICATOR USED FOR INTERNAL COMMUNICATION:
*                              =0 STANDARD VALUE
*                              FACTORIZATION OUTPUT:
*                              =1 IF NAU IS DECLARED TOO SMALL.
*                              =2 IF NZ IS TO BE INCREASED.
*                              =3 IF MATRIX IS TOO UNSTABLE.
*                              =4 IF MATRIX IS SINGULAR.
*                              =5 IF NZ IS TOO SMALL FOR THE FACTORS.
*                              =6 IF NAU IS TOO SMALL FOR FACTORS.
*                              =7 IF NZ IS SMALL FOR THE AFTER-F-MANIP.
*                              =8 LACK OF SPACE IN UXSGFD.
*                              DRIVER THROUGHPUT:
*                              =9 IF COLUMN INTO BASE IS THE SAME AS
*                                 OUT-OF-THE -BASE-COLUMN (INTERNAL).
*                              UPDATE OUTPUT:
*                              =11 IF NAU IS DECLARED TOO SMALL.
*                              =12 IF MATRIX IS SINGULAR.
*                              =13 IF FACTORS ARE BADLY CONDITIONED.
*                              =14 LITTLE SPACE FOR L-UPDATES.
*                              =15 MATRIX SINGULARITY IS FACED.
*                              DRIVER END OUTPUT (UKSGLE):
*                              =16 IF THERE IS NO FEASIBLE SOLUTION.
*  IO  ITERL TYPE OF FEASIBLE POINT. ITERL=1-ARBITRARY FEASIBLE POINT-
*         PHASE 1 OUTPUT,
*         ITERL=2-OPTIMUM FEASIBLE POINT. ITERL=-1 FEASIBLE POINT DOES
*         NOT EXISTS. ITERL=-2 OPTIMUM FEASIBLE POINT DOES NOT EXISTS,
*         ITERL=-3 - SMALL NAU, ITERL=-4 - NZ IS SMALL, ITERL=-5 - BADLY
*         CONDITIONED PROBLEM, ITERL=-6 -  SMALL NUP
*  TO  TLPX  TEXT INFORMATION ABOUT CAUSE OF TERMINATION.
*
      SUBROUTINE UKSGLM(NF,N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,
     & IBEG,IBGR,IENDLU,RNR,PRNR,A,LB,UB,TYPE,WN01,BCOL,C,X,PTL,
     & LENL,PTC,LENC,PTR,LENR,CNR,ORDC,ORDR,ORDL,RIGHT,UP,GN,WR01,
     & AU,LI,LJ,CM,WR02,DEL1,DEL2,EPS7,EPS9,DMAX,GMAX,UMAX,FF,MFP,
     & MLP1,MIS,INEW,IOLD,KOLD,KINP,MINP,IER)
      $INCLUDE('UMCOMM')
      INTEGER NF,N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,IBEG,IBGR,
     & IENDLU,RNR(NZA),PRNR(MCOLS+1),TYPE(MCOLS),WN01(MCOLS),
     & BCOL(MCOLS),PTL(MROWS),LENL(MROWS),PTC(MROWS),LENC(MROWS),
     & PTR(MROWS),LENR(MROWS),CNR(NZ),ORDC(MROWS),ORDR(MROWS),
     & ORDL(MROWS),RIGHT(MROWS),UP(MROWS),LI(NUP),LJ(NUP),
     & MFP,MLP1,MIS,INEW,IOLD,KOLD,KINP,MINP,IER
      INTEGER IDEB,SCOL,LACK,P12
      $FLOAT A(NZA),LB(MCOLS),UB(MCOLS),C(MCOLS),X(MCOLS),GN(MCOLS),
     & WR01(MCOLS),AU(NAU),CM(NUP),WR02(MCOLS),DEL1,DEL2,
     & EPS7,EPS9,DMAX,GMAX,UMAX,FF
      $FLOAT DRFL,GRFL,GSFL,STFL,AMAX
      INTEGER NUMITR,I,JOB,PHASE,NROLD,IJNEW,IJOLD
      INTEGER FIFTY,EIGHTY,HUNDRE
      PARAMETER (FIFTY=50,EIGHTY=80,HUNDRE=100)
      PARAMETER(GRFL=1.0$P 1,GSFL=1.0$P 16)
C
C TEST ITERL AFTER UKLINM
C
      IF(ITERL.NE.0) GO TO 600
      DO I=1,N
        GN(I)=C(I)
      END DO
C
C     INITIALIZE LOCAL AUXILIARY VARIABLES
C
      SCOL=1
      KOLD=0
      NRED=0
      NROLD=0
      NUMITR=0
      IDEB=1
      P12=0
C
C     SET GLOBAL STATUS VARIABLES
C
      CALL UKSGLF(STFL,DEL1,DEL2,EPS7,EPS9,MLP1)
C
C SET PHASE
C
      IF(MFP.EQ.3) THEN
        PHASE=2
        DO 70 I=1,N
          C(I)=GN(I)
   70   CONTINUE
      ELSE
        PHASE=1
      END IF
      CALL UOKLP1('UKSGLM')
C
C     FACTORIZE THE BASIS
C
100   IF(NREF.GT.NROLD) THEN
        NROLD=NREF
        CALL UXSGFD(M,M,NZ,NZA,NAU,NEL,P12,N,PTL,LENL,PTC,LENC,PTR,
     *              LENR,CNR,ORDC,ORDR,ORDL,RNR,PRNR,BCOL,RIGHT,WR01,
     *              A,AU,STFL,GMAX,UMAX,DMAX,IBEG,IBGR,IENDLU,LACK,
     *              GSFL,DRFL,ITERL,0)
        IF (ITERL.NE.0) THEN
          CALL UOERR1('UKSGLM',ITERL+20)
          IF(ITERL.EQ.1.OR.ITERL.EQ.6) THEN
            ITERL=-3
          ELSEIF(ITERL.EQ.2.OR.ITERL.EQ.7) THEN
            ITERL=-4
          ELSE
            ITERL=-5
          ENDIF
          GO TO 500
        END IF
C
C SET VECTOR UP
C
        DO 150 I=1,M
          UP(I)=WR01(I)
150     CONTINUE
C
      END IF
C
C OBJECTIVE FUNCTION UPDATE
C
      IF(PHASE.EQ.1) THEN
        CALL UKLOBJ(N,M,MCOLS,TYPE,BCOL,LB,UB,X,C,EPS9,KINP)
      END IF
C
C     FILL LAM - REPLACED BY WR02
C
      CALL UKSGLL(M,MCOLS,BCOL,WR02,C,PHASE,IEXT)
C
C     BTRAN - LAM REPLACED BY WR02
C
      JOB=0
      CALL UXSGFT(M,NZ,NAU,NEL,NUP,PTL,LENL,CNR,PTR,LENR,ORDC,
     *            ORDR,ORDL,LI,LJ,CM,AU,WR02,WR01,JOB)
C
C     FIND THE INPUT COLUMN - LAM REPLACED BY WR02
C
      CALL UKSGLC(N,MCOLS,NZA,PRNR,RNR,A,C,TYPE,WR02,SCOL,PHASE,EPS9,
     & MLP1,INEW,IEXT,IER)
C
C SET IJNEW - PRINTED VALUE
C
      IF(IER.EQ.0) THEN
        IJNEW=INEW
      ELSE
        IJNEW=0
      ENDIF
      NRED=NRED+1
      NUMITR=NUMITR+1
      IF (NUMITR.GT.MIS) THEN
        ITERL=-7
        TLPX='INEXACT '
        CALL UOERR1('UKSGLM',90)
        GO TO 600
      ENDIF
      IF(IER.EQ.1) THEN
        IF(PHASE.EQ.1) THEN
          ITERL=1
        ELSE
          ITERL=2
        ENDIF
        GO TO 300
      END IF
C
C     FCIN FILLING - FCIN REPLACED BY WR02
C
      CALL UKSGLK(M,MROWS,MCOLS,NZA,INEW,RNR,PRNR,A,WR02)
C
C     FTRAN - FCIN REPLACED BY WR02
C
      JOB=0
      CALL UXSGFB(M,NZ,NAU,NEL,NUP,PTL,LENL,CNR,PTR,LENR,ORDC,
     *            ORDR,ORDL,LI,LJ,CM,AU,WR02,WR01,JOB)
C
C     OUTPUT COLUMN SEARCH - FCIN REPLACED BY WR02
C
      CALL UKSGLO(NF,M,MROWS,MCOLS,TYPE,LB,UB,ORDC,BCOL,WR02,X,AMAX,
     & EPS9,INEW,IOLD,KINP,MINP,IER)
      IF(IER.EQ.1) THEN
C
C CHANGE OF ACTIVE BOUND ONLY
C
        ITERL=9
        IJOLD=IJNEW
      ELSEIF(IER.EQ.2) THEN
C
C UNBOUNDNESS DETECTED
C
        ITERL=-2
        CALL UOERR1('UKSGLM',12)
        GO TO 300
      ELSE
        IJOLD=BCOL(IOLD)
      END IF
C
C     TEST ON THE ZERO STEP DUE TO DEGENERACY
C
      IF(AMAX.LT.EPS9) THEN
        MLP1=MLP1+1
      ELSE
        MLP1=0
      END IF
C
C     COLUMN EXCHANGE
C
      IF(ITERL.NE.9) THEN
        CALL UXSGUM(M,M,NZ,NZA,NAU,NEL,NUP,PTR,LENR,CNR,PRNR,RNR,
     *              ORDR,IDEB,ORDC,BCOL,PTL,LENL,ORDL,WR01,WR02,LI,LJ,
     *              CM,RIGHT,UP,A,AU,STFL,GMAX,DRFL,UMAX,DMAX,INEW,IOLD,
     *              IBGR,IENDLU,IBEG,ITERL)
        KOLD=KOLD+1
      ELSE
        ITERL=0
      END IF
C
C
      CALL UOKLP3(N,M,MROWS,MCOLS,NEL,IJNEW,IJOLD,PHASE,NUMITR,LENR,
     & TYPE,BCOL,X,C,FF,KINP)
C
      IF(ITERL.GT.10) CALL UOERR1('UKSGLM',ITERL+20)
      IF(ITERL.EQ.11) THEN
        ITERL=-3
        GO TO 500
      ELSEIF(ITERL.GT.0) THEN
        NREF=NREF+1
        IF(KOLD.EQ.0) THEN
          IF(ITERL.EQ.14) THEN
            ITERL=-6
          ELSE
            ITERL=-5
          ENDIF
          GO TO 300
        END IF
        KOLD=0
        GO TO 100
      END IF
      IF(KOLD.GE.HUNDRE) THEN
        NREF=NREF+1
        KOLD=0
      END IF
      GO TO 100
C
C OPTIMUM SOLUTION PROCESS FOR THE PHASE 1 - FCIN IS REPLACED
C BY THE VECTOR C THAT IS OF NO USE AT THAT TIME.
C
300   CONTINUE
C
C SET TLXX
C
      IF(ITERL.EQ.2) THEN
        TLPX='OPT SOL '
      ELSEIF(ITERL.EQ.1) THEN
        TLPX='FEAS SOL'
      ELSEIF(ITERL.EQ.-1) THEN
        CALL UOERR1('UKSGLM',11)
        TLPX='NO FEAS '
      ELSEIF(ITERL.EQ.-2) THEN
        CALL UOERR1('UKSGLM',12)
        TLPX='UNBOUND '
      ELSEIF(ITERL.EQ.-3) THEN
        TLPX='SMLL NAU'
      ELSEIF(ITERL.EQ.-4) THEN
        TLPX='SMALL NZ'
      ELSEIF(ITERL.EQ.-5) THEN
        TLPX='BAD COND'
      ELSEIF(ITERL.EQ.-6) THEN
        TLPX='SMLL NUP'
      ENDIF
      IF(PHASE.EQ.1) THEN
        CALL UKSGLE(NF,N,M,MROWS,MCOLS,NZ,NZA,NAU,NUP,NEL,IBEG,IBGR,
     & IENDLU,PTR,LENR,CNR,PRNR,RNR,ORDR,IDEB,ORDC,BCOL,PTL,LENL,LENC,
     & PTC,ORDL,WR01,WR02,LI,LJ,CM,RIGHT,UP,A,AU,STFL,GMAX,UMAX,DMAX,
     & WN01,C,X,C,TYPE,EPS7,EPS9,DMAX,GMAX,UMAX,NREF,INEW,IOLD,KOLD,
     & KINP,MINP,ITERL)
      IF (ITERL.EQ.-1) TLPX='NO FEAS '
      END IF
      CALL UOKLP3(N,M,MROWS,MCOLS,NEL,IJNEW,IJOLD,PHASE,NUMITR,LENR,
     & TYPE,BCOL,X,C,FF,KINP)
      CALL UOKLP2(N,M,MROWS,MCOLS,NEL,PHASE,X,C,TYPE,BCOL,NUMITR,LENR,
     & FF,KINP)
      IF(PHASE.EQ.1.AND.ITERL.NE.1.OR.PHASE.EQ.2.AND.ITERL.NE.2) THEN
        GO TO 600
      ENDIF
C
C PHASE 2
C
      IF(MFP.EQ.1) THEN
        GO TO 600
      ELSEIF(MFP.EQ.3) THEN
        GO TO 500
      ELSE
        IF(PHASE.EQ.2) THEN
          GO TO 500
        ELSEIF(PHASE.EQ.1) THEN
          PHASE=2
          NUMITR=0
          DO 400 I=1,N
            C(I)=GN(I)
  400     CONTINUE
          GO TO 100
        ENDIF
      ENDIF
  500 CONTINUE
C
  600 CONTINUE
C
C CHANGE AND DEFINE SOME IMPORTANT DIMENSIONAL VARIABLES
C NAP WILL SAVE NUMBER OF SLACKS, NF IS CHANGED TO NUMBER OF
C INPUT VARIABLES + NUMBER OF SLACKS
C
C      NAP=MINP
C      NF=N
C
C SHIFT THE VALUES BACK
C
C      DO I=N+1,MROWS
C        X(I)=0.0D0
C      END DO
C      DO I=N,1,-1
C        J=PRMC(I)
C        IF(I.NE.J) THEN
C          X(J)=X(I)
C          X(I)=0.0D0
C        END IF
C      END DO
C
      RETURN
      END
