* SUBROUTINE UKSGLO                 ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* FIND THE OUTPUT COLUMN OUT FOR THE SIMPLEX CYCLE.
*
* PARAMETERS :
*  II  NF NUMBER OF VARIABLES.
*  II  M NUMBER OF ROWS.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  MCOLS DECLARED NUMBER OF COLUMNS.
*  IU  TYPE(N) TYPE OF THE VARIABLES:
*                                    =1 FOR NONBASIC LOWER BOUNDED
*                                    =2 FOR NONBASIC UPPER BOUNDED
*                                    =3 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS LOWER BOUND
*                                    =4 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS UPPER BOUND
*                                    =5 FOR NONBASIC FIXED VARIABLE
*                                    =6 FOR NONBASIC FREE VARIABLE
*                                    =-1 FOR BASIC LOWER BOUNDED
*                                    =-2 FOR BASIC UPPER BOUNDED
*                                    =-3 FOR BASIC LOWER AND UPPER
*                     BOUNDED VARIABLE
*                                    =-6 FOR BASIC FREE
*  RO  LB(N) LOWER BOUNDS ON VARIABLES.
*  RO  UB(N) UPPER BOUNDS ON VARIABLES.
*  II  ORDC(N) COLUMN PERMUTATION OF THE FACTOR U.
*  II  BCOL(N) INDICES OF THE BASIC COLUMNS.
*  RI  FCIN(N) FORWARDLY TRANSFORMED COLUMN USED FOR
*              OUTPUT COLUMN TEST.
*  RA  X(N) AUXILIARY VECTOR - LATER IT WILL BE VECTOR OF SOLUTION.
*  RO  AMAX STEPSIZE.
*  RI  EPS9 TOLERANCE ON  RELATIVE COSTS.
*  II  INEW INPUT COLUMN FOR THE SIMPLEX CHANGE.
*  IO  IOLD OUTPUT COLUMN.
*  II  KINP NUMBER OF ARTIFICIALS.
*  IO  MINP NUMBER OF SLACKS.
*  IO  IER OUTPUT VARIABLE:=0 WHEN OUTPUT COLUMN OTHER THEN INEW
*                             IS FOUND.
*                          =1 WHEN BOUND CHANGE IS ON INEW.
*                             VARIABLE IOLD IS THEN NOT USED.
*                          =2 WHEN PROBLEM IS UNBOUNDED.
*
      SUBROUTINE UKSGLO(NF,M,MROWS,MCOLS,TYPE,LB,UB,ORDC,BCOL,FCIN,X,
     & AMAX,EPS9,INEW,IOLD,KINP,MINP,IER)
      INTEGER NF,M,MROWS,MCOLS,TYPE(MCOLS),ORDC(MROWS),BCOL(MROWS),
     & INEW,IOLD,KINP,MINP,IER
      $FLOAT LB(MCOLS),UB(MCOLS),FCIN(MROWS),X(MCOLS),AMAX,EPS9
      $FLOAT INFT,FCT,FCMAX,TEMP
      INTEGER TYPV,TYPO
      INTEGER I,J,K,L,K1,L1,L2,IMAX
      PARAMETER (INFT=1.0D+74)
C
C     INITIALIZATION
C     IER=1 FOR BOUND CHANGE BY INEW AND IER=2 FOR UNBOUNDED PROBLEM.
C
      IER=0
      IMAX=0
      TYPV=TYPE(INEW)
      AMAX=INFT
      EPS9=1.0$P-12
C
C     TYPE TEST
C
      IF(TYPV.EQ.1.OR.TYPV.EQ.3.OR.TYPV.EQ.61) THEN
C
C VARIABLE INEW IS TO BE INCREASED
C
        DO 100 J=1,M
        I=BCOL(J)
        FCT=FCIN(J)
        IF(FCT.GT.EPS9) THEN
C
C X IS TO BE DECREASED - FOR TYPES -2, -6 TEMP IS NOT DEFINED
C
          IF(TYPE(I).EQ.-2.OR.TYPE(I).EQ.-6) GO TO 100
          TEMP=(X(I)-LB(I))/FCT
          TEMP=MAX(0.0$P 0,TEMP)
          IF(TEMP.GT.AMAX) THEN
            GO TO 100
          ELSEIF(TEMP.LT.AMAX) THEN
            AMAX=TEMP
            IMAX=J
            FCMAX=FCT
          ELSEIF(AMAX.NE.0) THEN
            IF(I.GE.(M-KINP+1)) THEN
              AMAX=TEMP
              IMAX=J
              FCMAX=FCT
            ELSE
              IF(FCT.GT.FCMAX) THEN
                AMAX=TEMP
                IMAX=J
                FCMAX=FCT
              END IF
            END IF
          END IF
        ELSE
C
C X IS TO BE INCREASED - FOR TYPES -1, -6 TEMP IS NOT DEFINED
C
          IF(TYPE(I).EQ.-1.OR.TYPE(I).EQ.-6) GO TO 100
          IF(FCT.LT.-EPS9) THEN
            TEMP=(UB(I)-X(I))/(-FCT)
            TEMP=MAX(0.0$P 0,TEMP)
            IF(TEMP.GT.AMAX) THEN
              GO TO 100
            ELSEIF(TEMP.LT.AMAX) THEN
              AMAX=TEMP
              IMAX=J
              FCMAX=-FCT
            ELSEIF(AMAX.NE.0) THEN
              IF(I.GE.(M-KINP+1)) THEN
                AMAX=TEMP
                IMAX=J
                FCMAX=-FCT
              ELSE
                IF((-FCT).GT.FCMAX) THEN
                  AMAX=TEMP
                  IMAX=J
                  FCMAX=-FCT
                END IF
              END IF
            END IF
          END IF
        END IF
100     CONTINUE
C
C     TEST WHETHER THE COLUMN INEW IS NOT ONLY TO CHANGE VALUE
C     AND STAY NONBASIC. IN THIS CASE IMAX IS SET TO ZERO
C
      IF(TYPV.LE.2.OR.TYPV.EQ.61) GO TO 120
      TEMP=(UB(INEW)-X(INEW))
      IF(AMAX.GT.TEMP) THEN
                       AMAX=TEMP
                       IER=1
      END IF
120   CONTINUE
C
C     CHANGE X
C
      DO 150 J=1,M
      I=BCOL(J)
      X(I)=X(I)-FCIN(J)*AMAX
150   CONTINUE
      X(INEW)=X(INEW)+AMAX
C
C     TYPE CHANGE FOR INEW
C
      IF(IER.EQ.1) THEN
        TYPE(INEW)=4
      ELSEIF(TYPV.EQ.61) THEN
        TYPE(INEW)=-6
      ELSE
        TYPE(INEW)=-TYPV
      END IF
      IF(IER.EQ.1) RETURN
      ELSE
C
C VARIABLE INEW IS TO BE DECREASED
C
        DO 200 J=1,M
          I=BCOL(J)
          FCT=FCIN(J)
          IF(FCT.GT.EPS9) THEN
C
C X IS TO BE INCREASED - FOR TYPES -1, -6 TEMP IS NOT DEFINED
C
            IF(TYPE(I).EQ.-1.OR.TYPE(I).EQ.-6) GO TO 200
            TEMP=(UB(I)-X(I))/FCT
            TEMP=MAX(0.0$P 0,TEMP)
            IF(TEMP.GT.AMAX) THEN
              GO TO 200
            ELSEIF(TEMP.LT.AMAX) THEN
              AMAX=TEMP
              IMAX=J
              FCMAX=FCT
            ELSEIF(AMAX.NE.0) THEN
              IF(I.GE.(M-KINP+1)) THEN
                AMAX=TEMP
                IMAX=J
                FCMAX=FCT
              ELSE
                IF(FCT.GT.FCMAX) THEN
                  AMAX=TEMP
                  IMAX=J
                  FCMAX=FCT
                END IF
              END IF
            END IF
          ELSE
            IF(FCT.LT.-EPS9) THEN
C
C X IS TO BE DECREASED - FOR TYPES -2, -6 TEMP IS NOT DEFINED
C
              IF(TYPE(I).EQ.-2.OR.TYPE(I).EQ.-6) GO TO 200
              TEMP=(X(I)-LB(I))/(-FCT)
              TEMP=MAX(0.0$P 0,TEMP)
              IF(TEMP.GT.AMAX) THEN
                GO TO 200
              ELSEIF(TEMP.LT.AMAX) THEN
                AMAX=TEMP
                IMAX=J
                FCMAX=-FCT
              ELSEIF(AMAX.NE.0) THEN
                IF(I.GE.(M-KINP+1)) THEN
                  AMAX=TEMP
                  IMAX=J
                  FCMAX=-FCT
                ELSE
                  IF((-FCT).GT.FCMAX) THEN
                    AMAX=TEMP
                    IMAX=J
                    FCMAX=-FCT
                  END IF
                END IF
              END IF
            END IF
          END IF
200     CONTINUE
C
C     TEST WHETHER THE COLUMN INEW IS NOT ONLY TO CHANGE VALUE
C     AND STAY NONBASIC. IN THIS CASE IER IS SET TO ONE.
C
      IF(TYPV.LE.2.OR.TYPV.EQ.62) GO TO 220
      TEMP=(X(INEW)-LB(INEW))
      IF(AMAX.GT.TEMP) THEN
                       IER=1
                       AMAX=TEMP
      END IF
220   CONTINUE
C
C     X CHANGE
C
      DO 250 J=1,M
      I=BCOL(J)
      X(I)=X(I)+FCIN(J)*AMAX
250   CONTINUE
      X(INEW)=X(INEW)-AMAX
C
C     TYPE CHANGE FOR INEW
C
      IF(IER.EQ.1) THEN
        TYPE(INEW)=3
      ELSEIF(TYPV.EQ.62) THEN
        TYPE(INEW)=-6
      ELSEIF(TYPV.EQ.4) THEN
        TYPE(INEW)=-3
      ELSE
        TYPE(INEW)=-TYPV
      END IF
      IF(IER.EQ.1) RETURN
      END IF
C
C     COLUMN EXCHANGE
C     WHEN COLUMN CORRESPONDING TO SOME ARTIFICIAL VARIABLE
C     IS TO BE DELETED THEN COLUMN EXCHANGE IS TO BE DONE.
C     CHANGE INFLUENCES ALSO VECTOR OF COLUMN PERMUTATION
C     ORDC.COLUMNS INEW AND IOLD USE DIFFERENT FORMATS.
C     IOLD RETURNS POSITION IN THE VECTOR BCOL INSTEAD OF
C     THE REAL COLUMN NUMBER.
C
C     BCOL EXCHANGE
C
C
C ARTIFICIAL VARIABLE ENTERS BASIS - MUST BE EXCHANGED FOR THE
C VARIABLE WITH INDEX M-KINP+1
C
      J=M-KINP+1
      IF(IMAX.GT.J) THEN
      L=BCOL(J)
      K=BCOL(IMAX)
      BCOL(J)=K
      BCOL(IMAX)=L
C
C    KINP IS DECREASED
C
      KINP=KINP-1
C
C     ORDC UPDATE
C
      L1=0
      L2=0
      DO 300 I=1,M
      K1=ORDC(I)
      IF(K1.EQ.J.OR.K1.EQ.IMAX) THEN
          IF(L1.NE.0) THEN
                      L2=I
                      GO TO 400
                      ELSE
                      L1=I
          END IF
      END IF
300   CONTINUE
400   CONTINUE
C
C     EXCHANGE
C
      IF(ORDC(L1).NE.J) THEN
                        ORDC(L1)=J
                        ORDC(L2)=IMAX
                        ELSE
                        ORDC(L1)=IMAX
                        ORDC(L2)=J
      END IF
C
C     IMAX REDEFINITION
C
      IMAX=J
                     ELSE
C
C     IN CASE THAT J.EQ.IMAX WE ONLY DECREASE KINP
C     ELSE WE DO NOTHING.
C
         IF(IMAX.EQ.J) THEN
                       KINP=KINP-1
         END IF
      END IF
      IF(INEW.GT.(NF+MINP).AND.IMAX.LT.(M-KINP+1)) THEN
        J=M-KINP
        L=BCOL(J)
        K=BCOL(IMAX)
        BCOL(J)=K
        BCOL(IMAX)=L
C
C FICTIVE INCREASE OF KINP
C
        KINP=KINP+1
C
C     ORDC UPDATE
C
        L1=0
        L2=0
        DO 310 I=1,M
        K1=ORDC(I)
        IF(K1.EQ.J.OR.K1.EQ.IMAX) THEN
          IF(L1.NE.0) THEN
            L2=I
            GO TO 410
          ELSE
            L1=I
          END IF
        END IF
310     CONTINUE
410     CONTINUE
C
C     EXCHANGE
C
        IF(ORDC(L1).NE.J) THEN
          IF (L1.GT.0) ORDC(L1)=J
          ORDC(L2)=IMAX
        ELSE
          ORDC(L1)=IMAX
          IF (L2.GT.0) ORDC(L2)=J
        END IF
        IMAX=J
      ELSEIF(INEW.GT.(NF+MINP).AND.IMAX.EQ.J) THEN
        KINP=KINP+1
      ENDIF
C
C     PROBLEM IS UNBOUNDED
C
      IF(AMAX.EQ.INFT) THEN
                      IER=2
                      RETURN
      END IF
C
C     SET IOLD AND ITS TYPE
C
      IOLD=IMAX
      J=BCOL(IOLD)
      TYPO=TYPE(J)
      IF(TYPO.GT.-3) THEN
                     TYPE(J)=-TYPO
      ELSEIF(TYPO.EQ.-3.OR.TYPO.EQ.-4) THEN
        IF(ABS(X(J)-LB(J)).GT.ABS(X(J)-UB(J))) THEN
          TYPE(J)=4
        ELSE
          TYPE(J)=3
        END IF
      ELSEIF(TYPO.EQ.-6) THEN
        TYPE(J)=6
      END IF
      RETURN
      END
