* SUBROUTINE ULADB4               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NEW LINEAR CONSTRAINT OR A NEW SIMPLE BOUND IS ADDED TO THE ACTIVE
* SET. TRANSFORMED HESSIAN MATRIX APPROXIMATION OR ITS INVERSION
* IS UPDATED.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  IU  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE.
*  RU  H(NF*(NF+1)/2)  TRANSFORMED HESSIAN MATRIX APPROXIMATION OR
*         ITS INVERSION.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  EPS7  TOLERANCE FOR LINEAR INDEPENDENCE OF CONSTRAINTS.
*  RO  GMAX  MAXIMUM ABSOLUTE VALUE OF A PARTIAL DERIVATIVE.
*  RO  UMAX  MAXIMUM ABSOLUTE VALUE OF A NEGATIVE LAGRANGE MULTIPLIER.
*  IU  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*         IDECF=1-GILL-MURRAY DECOMPOSITION. IDECF=9-INVERSION.
*         IDECF=10-DIAGONAL MATRIX.
*  II  INEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*  IU  NADD  NUMBER OF CONSTRAINT ADDITIONS.
*  IO  IER  ERROR INDICATOR.
*
* SUBPROGRAMS USED :
*  S   ULADR0  CORRECTION OF KERNEL OF THE ORTHOGONAL PROJECTION
*         AFTER CONSTRAINT ADDITION.
*  S   UXDRMM  PREMULTIPLICATION OF A VECTOR BY A ROWWISE STORED DENSE
*         RECTANGULAR MATRIX.
*  S   UXDRMV  COPY OF THE SELECTED COLUMN OF A ROWWISE STORED DENSE
*         RECTANGULAR MATRIX.
*  S   UXDRGR  PLANE ROTATION OF A TRANSPOSED DENSE RECTANGULAR MATRIX.
*         RECTANGULAR MATRIX.
*  S   UXDSMR  PLANE ROTATION OF A DENSE SYMMETRIC MATRIX.
*  S   UXVORT  DETERMINATION OF AN ELEMENTARY ORTHOGONAL MATRIX FOR
*         PLANE ROTATION.
*
      SUBROUTINE ULADB4(NF,N,NC,ICA,CG,CR,CZ,H,S,EPS7,GMAX,UMAX,IDECF,
     & INEW,NADD,IER)
      INTEGER NF,N,NC,ICA(NF),IDECF,INEW,NADD,IER
      $FLOAT CG(NF*NC),CR(NF*(NF+1)/2),CZ(NF*NF),H(NF*(NF+1)/2),S(NF),
     & EPS7,GMAX,UMAX
      $FLOAT CK,CL
      INTEGER I,J,K,L,N1
      IF (IDECF.NE.0.AND.IDECF.NE.9) THEN
      IER=-2
      RETURN
      ENDIF
      CALL ULADR0(NF,N,NC,ICA,CG,CR,S,EPS7,GMAX,UMAX,INEW,NADD,IER)
      IF (IER.NE.0) RETURN
      IF (N.GT.0) THEN
      N1=N+1
      IF (INEW.GT.0) THEN
      CALL UXDRMM(NF,N1,CZ,CG((INEW-1)*NF+1),S)
      ELSE
      CALL UXDRMV(NF,N1,CZ,S,-INEW)
      ENDIF
      DO 1 L=1,N
      K=L+1
      CALL UXVORT(S(K),S(L),CK,CL,IER)
      CALL UXDRGR(NF,N1,CZ,K,L,CK,CL,IER)
      CALL UXDSMR(N1,H,K,L,CK,CL,IER)
      IF (IER.LT.0) RETURN
    1 CONTINUE
      IF (IDECF.EQ.9) THEN
      L=N*(N+1)/2
      IF (H(L+N1).NE.0.0$P 0) THEN
      CL=1.0$P 0/H(L+N1)
      K=0
      DO 3 I=1,N
      CK=CL*H(L+I)
      DO 2 J=1,I
      K=K+1
      H(K)=H(K)-CK*H(L+J)
    2 CONTINUE
    3 CONTINUE
      ENDIF
      ENDIF
      ENDIF
      IER=0
      RETURN
      END
