$INTERFACE ULLPI1                                             98/12/01
     $HELP
       INTERIOR POINT METHOD
         1 - JIANMING MIAO ALGORITHM 1
         2 - JIANMING MIAO ALGORITHM 2
         3 - MIZUNO ALGORITHM
     $CHECK(MLP,1,'1\2\3',3)
     $HELP
       LOWER BOUND FOR STEPLENGTH
     $CHECK(TOLX,'1.0$P-8',REAL,3)
     $HELP
       LOWER BOUND FOR CONSTRAINT VIOLATION
     $CHECK(TOLC,'1.0$P-6',REAL,3)
     $HELP
       LOWER BOUND FOR GRADIENT OF THE
       LAGRANGIAN FUNCTION
     $CHECK(TOLG,'1.0$P-6',REAL,3)
 $00 CALL ULLPI1(NF,NC,MC,$GF,$CG,$ICG,$JCG,$CL,$X,$CZ,$G,$CF,$D,$SO,$S&
,$XS,$GS,CMAX,GMAX,UMAX,RPF1,$MLP,MRED,INEW)
     GO TO ($20,$10) ISB+1
 $10 CONTINUE
     $IF(NUMBER=2) CALL UXVNEG(NF,$D,$H)
     $SUBST('UDSLL$NUMBER')
     GOTO $00
 $20 CONTINUE
     $ADD(INTEGER,'\$ICG($NC+1)\$JCG($MC)')
     $ADD(REAL,'\$GF($NF)\$CG($MC)\$CL($NC)\$X($NF)\$CZ($NC)\$G($NF)&
     \$CF($NC)\$D($NF)\$SO($NF)\$S(2*$NF+$NC)\$XS(2*$NF)\$GS($NF)')
     $ADD(INIT)
       NLPX='ULLPI1'
     $ENDADD
     $EXIT

* IMPLEMENTATION

* SUBROUTINE ULLPI1                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 KO : ORIGINAL VERSION
*
* PURPOSE :
* INFEASIBLE PREDICTOR-CORRECTOR PRIMAL-DUAL INTERIOR POINT METHOD
* FOR LINEAR PROGRAMMING.
*
* PARAMETERS :
*  II  NF NUMBER OF VARIABLES.
*  II  NC NUMBER OF CONSTRAINTS.
*  II  MC  NUMBER OF ELEMENTS IN THE FIELD CG.
*  RI  C(NF)  VECTOR OF PRICES (GRADIENT OF THE LINEAR FUNCTION).
*  RI  CG(MC)  ELEMENTS OF THE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC)  COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RI  B(NC)  RIGHT HAND SIDE VECTOR.
*  RO  X(NF)  VECTOR OF VARIABLES.
*  RA  Y(NC)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RA  S(NF)  VECTOR OF SLACK VARIABLES.
*  RA  G(NF) GRADIENT OF THE LAGRANGIAN.
*  RA  CF(NC) CONSTRAINT VIOLATION VECTOR.
*  RA  D(NF)  DIAGONAL MATRIX.
*  RA  Z(2*NF+NC) DIRECTION VECTOR.
*  RA  VEC1(2*NF)  AUXILIARY VECTOR.
*  RA  VEC2(NF)  AUXILIARY VECTOR.
*
* COMMON DATA :
*  II  MMAX LENGTH OF THE ARRAYS JB,B.
*  RI  ETA0  MACHINE PRECISION.
*  RI  ETA2  TOLERANCE FOR POSITIVE DEFINITENESS.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION VECTOR OBTAINED.
*
* SUBPROGRAMS USED :
*  S   UXSCMD  MULTIPLICATION OF A DENSE RECTANGULAR MATRIX STORED BY
*         COLUMNS BY A VECTOR WITH EVENTUAL ADDITION OF A SCALED VECTOR.
*  S   UXSRMD  MULTIPLICATION OF A DENSE RECTANGULAR MATRIX STORED BY
*         ROWS BY A VECTOR WITH EVENTUAL ADDITION OF A SCALED VECTOR.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH CHANGE OF THE SIGN.
*  RF  UXVNOR  EUCLIDEAN NORM OF A VECTOR.
*  RF  UXVSAB  SUM OF ABSOLUTE VALUES OF VECTOR ELEMENTS.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UXVSCL  SCALING OF A VECTOR.
*
* METHOD :
*
      SUBROUTINE ULLPI1(NF,NC,MC,C,CG,ICG,JCG,B,X,Y,G,CF,D,S,Z,VEC1,
     & VEC2,CMAX,GMAX,UMAX,RPF1,MLP,MRED,INEW)
      $INCLUDE('UMCOMM')
      INTEGER NF,NC,MC,ICG(NC+1),JCG(MC),MLP,MRED,INEW
      $FLOAT C(NF),CG(MC),B(NC),X(NF),Y(NC),G(NF),CF(NC),D(NF),S(NF),
     & Z(2*NF+NC),VEC1(2*NF),VEC2(NF),CMAX,GMAX,UMAX,RPF1
      $FLOAT POM,POM1,POM2,ALFA,BETA1,BETA2,GAMA1,ETA,RRO,RK
      $FLOAT INFEAS,UXVDOT,UXVNOR,UXVSAB
      INTEGER I
      SAVE BETA1,BETA2,GAMA1,RRO,RK
      IF (TSS(NS).NE.'ULPX') CALL UYPRO1('ULPX',1)
      GOTO (1,2,3) ISP(NS)
    1 CONTINUE
      CALL UOLLP1('ULLPI1')
      NRED=0
      RRO=0.5$P 2
      BETA1=0.25$P 0
      BETA2=0.5$P 0
      GAMA1=0.25$P 0
C
C     INITIAL APPROXIMATION
C
      RK=ONE
      CALL UXVSET(NF,RRO,X)
      CALL UXVSET(NF,RRO,S)
      CALL UXVSET(NC,ZERO,Y)

C
C     MAIN ITERATIVE CYCLE
C
   10 IF (NRED.GT.MRED) GO TO 74
      NRED=NRED+1
C
C     PREDICTOR
C
      RPF1=UXVDOT(NF,X,S)
      IF (MLP.EQ.3) RPF1=BETA1*RPF1
      RPF1=RPF1/$DBLE(NF)
      CALL UXVMUL(NF,X,S,D,-1)
C
C     RIGHT HAND SIDE PREPARATION FOR THE PREDICTOR STEP
C     CF = (CG)*X - B
C     G  = TRANS(CG)*Y - C + RPF1*INV(X)*E
C
      CALL UXSRMD(NF,NC,MC,CG,ICG,JCG,X,-ONE,B,CF)
      CALL UXSCMD(NF,NC,MC,CG,ICG,JCG,Y,-ONE,C,G)
      IF (MLP.EQ.1) THEN
      ELSE IF (MLP.EQ.2) THEN
      DO 11 I=1,NF
      G(I)=G(I)+S(I)-RPF1/X(I)
   11 CONTINUE
      ELSE
      DO 12 I=1,NF
      G(I)=G(I)+(RPF1/X(I))
   12 CONTINUE
      ENDIF
      CALL UYSET0(1,2)
      RETURN
C
C     DEFINITION OF THE NEW VARIABLES (X,Y,S) AFTER THE PREDICTOR STEP
C
    2 IF (MLP.EQ.1) THEN
      DO 13 I=1,NF
      Z(NF+NC+I)=-S(I)*(ONE+Z(I)/X(I))
   13 CONTINUE
      ELSE IF (MLP.EQ.2) THEN
      DO 14 I=1,NF
      Z(NF+NC+I)=-(S(I)*Z(I)+RPF1)/X(I)
   14 CONTINUE
      ELSE
      CALL UXSCMD(NF,NC,MC,CG,ICG,JCG,Z(NF+1),-ONE,C,VEC1)
      CALL UXVNEG(NF,VEC1,VEC1)
      CALL UXSCMD(NF,NC,MC,CG,ICG,JCG,Y,ONE,S,VEC2)
      CALL UXVDIF(NF,VEC1,VEC2,Z(NC+NF+1))
      ENDIF
      CALL UXVMUL(NF,Z(NF+NC+1),Z,VEC1,1)
      IF (MLP.EQ.1) THEN
      POM=UXVNOR(NF,VEC1)/RPF1
      ALFA=TWO/(ONE+SQRT(ABS(ONE+FOUR**2*POM)))
      ELSE IF (MLP.EQ.2) THEN
      POM=FOUR*UXVNOR(NF,VEC1)
      ALFA=ONE/(ONE+SQRT(ABS(ONE+POM/RPF1)))
      ALFA=1.9991$P 0*ALFA
      ELSE
      POM=UXVSAB(NF,VEC1)
      DO 16 I=1,NF
      VEC1(I)=VEC1(I)-POM/$DBLE(NF)
   16 CONTINUE
      POM=ABS(POM)
      POM1=UXVNOR(NF,VEC1)
      ETA=MAX(POM,POM1)
      POM2=UXVDOT(NF,X,S)
      POM=(POM2*GAMA1)/(2*$DBLE(NF)*ETA)
      POM=SQRT(POM)
      POM1=(BETA1*POM2)/ETA
      POM2=(BETA2-BETA1)*POM2/ETA
      ALFA=MIN(HALF,POM,POM1,POM2)
      ALFA=1.99$P 0*ALFA
      ENDIF
      CALL UXVDIR(NF,ALFA,Z,X,X)
      CALL UXVDIR(NC,ALFA,Z(NF+1),Y,Y)
      CALL UXVDIR(NF,ALFA,Z(NF+NC+1),S,S)
      IF (MLP.EQ.3) RK=(1-ALFA)*RK
C
C     CORRECTOR
C
      IF (MLP.LE.2) THEN
      RPF1=(ONE-ALFA)*RPF1
      ELSE
      RPF1=UXVDOT(NF,X,S)/$DBLE(NF)
      ENDIF
      CALL UXVMUL(NF,X,S,D,-1)
C
C     RIGHT HAND SIDE PREPARATION FOR THE CORRECTOR STEP
C     CF = 0
C     G  = - S + RPF1*INV(X)*E
C
      CALL UXVSET(NC,ZERO,CF)
      DO 19 I=1,NF
      G(I)=RPF1/X(I)-S(I)
   19 CONTINUE
      CALL UYSET0(1,3)
      RETURN
    3 CALL UXSCMD(NF,NC,MC,CG,ICG,JCG,Z(NF+1),0.0D 0,S,VEC1)
      CALL UXVNEG(NF,VEC1,Z(NF+NC+1))
C
C     DEFINITION OF THE NEW VARIABLES (X,Y,S) AFTER THE CORRECTOR STEP
C
      CALL UXVDIR(NF,ONE,Z,X,X)
      CALL UXVDIR(NC,ONE,Z(NF+1),Y,Y)
      CALL UXVDIR(NF,ONE,Z(NF+NC+1),S,S)
C
C     TERMINATION CRITERIA
C
      GMAX=UXVDOT(NF,X,S)
      CALL UXSRMD(NF,NC,MC,CG,ICG,JCG,X,-1.0D 0,B,VEC1)
      CMAX=UXVNOR(NC,VEC1)
      CALL UXVDIF(NF,S,C,VEC2)
      CALL UXSCMD(NF,NC,MC,CG,ICG,JCG,Y, 1.0D 0,VEC2,VEC1)
      UMAX=UXVNOR(NF,VEC1)
      CALL UXVCOP(NF,X,VEC1)
      CALL UXVCOP(NF,S,VEC1(NF+1))
      CALL UOLLP3(NF,NC,X,Y,S,G,CF,CMAX,GMAX,UMAX,INEW)
      IF ((GMAX.LE.TOLG).AND.(CMAX.LE.TOLC).AND.(UMAX.LE.TOLC)) THEN
      TUXX='OPTIMUM '
      GOTO 75
      ENDIF
      IF (MLP.EQ.3) THEN
      INFEAS=2*GMAX/(RK*RRO)
      POM=UXVSAB(2*NF,VEC1)
      IF (POM.GT.INFEAS) THEN
      TXFU='INFEAS  '
      GOTO 75
      ENDIF
      ENDIF
      GO TO 10
   74 CONTINUE
   75 CALL UOLLP2
      CALL UYEPI1(1)
      RETURN
      END
