* SUBROUTINE ULRMB0               ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* OLD LINEAR CONSTRAINT OR AN OLD SIMPLE BOUND IS REMOVED FROM THE
* ACTIVE SET.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NC  NUMBER OF CONSTRAINTS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  II  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CG(NF*NC)  MATRIX WHOSE COLUMNS ARE NORMALS OF THE LINEAR
*         CONSTRAINTS.
*  RI  CR(NF*(NF+1)/2)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  RI  CZ(NF*NF)  MATRIX WHOSE COLUMNS ARE BASIC VECTORS FROM THE
*         CURRENT REDUCED SUBSPACE.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GN(NF)  TRANSFORMED GRADIENT OF THE OBJECTIVE FUNCTION.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  II  IOLD  INDEX OF THE OLD ACTIVE CONSTRAINT.
*  IO  KREM  AUXILIARY VARIABLE.
*  IU  NREM NUMBER OF CONSTRAINT DELETION.
*  IO  IER  ERROR INDICATOR.
*
* SUBPROGRAMS USED :
*  S   ULRMR0  CORRECTION OF KERNEL OF THE ORTHOGONAL PROJECTION
*         AFTER CONSTRAINT DELETION.
*  S   UXDPRB  BACK SUBSTITUTION.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVMUL  DIAGONAL PREMULTIPLICATION OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE ULRMB0(NF,N,NC,XN,ICA,CG,CR,CZ,G,GN,NORMF,IOLD,KREM,
     & NREM,IER)
      INTEGER NF,N,NC,ICA(NF),NORMF,IOLD,KREM,NREM,IER
      $FLOAT XN(NF),CG(NF*NC),CR(NF*(NF+1)/2),CZ(NF*NF),G(NF),GN(NF)
      $FLOAT UXVDOT
      INTEGER NCA,NCR,NCZ,I,J,KC
      IER=0
      IF (N.EQ.NF) IER=2
      IF (IOLD.EQ.0) IER=3
      IF (IER.NE.0) RETURN
      NCA=NF-N
      NCR=NCA*(NCA-1)/2
      NCZ=N*NF
      CALL ULRMR0(NF,N,ICA,CR,CZ(NCZ+1),IOLD,KREM,IER)
      CALL UXVSET(NCA,0.0$P 0,CZ(NCZ+1))
      CZ(NCZ+NCA)=1.0$P 0
      CALL UXDPRB(NCA,CR,CZ(NCZ+1),-1)
      CALL UXVCOP(NCA,CZ(NCZ+1),CR(NCR+1))
      N=N+1
      CALL UXVSET(NF,0.0$P 0,CZ(NCZ+1))
      DO 4 J=1,NCA
      KC=ICA(J)
      IF (KC.GT.0) THEN
      CALL UXVDIR(NF,CR(NCR+J),CG((KC-1)*NF+1),CZ(NCZ+1),CZ(NCZ+1))
      ELSE
      I=-KC
      CZ(NCZ+I)=CZ(NCZ+I)+CR(NCR+J)
      ENDIF
    4 CONTINUE
      IF (NORMF.GT.0) THEN
      CALL UXVMUL(NF,XN,G,CR(NCR+1),1)
      GN(N)=UXVDOT(NF,CZ(NCZ+1),CR(NCR+1))
      ELSE
      GN(N)=UXVDOT(NF,CZ(NCZ+1),G)
      ENDIF
      NREM=NREM+1
      IER=0
      RETURN
      END
