* SUBROUTINE UMGLAG               ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* NEGATIVE PROJECTED GRADIENT IS DETERMINED USING LAGRANGE MULTIPLIERS.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF CURRENT LINEAR APPROXIMATED FUNCTIONS.
*  II  NB  NUMBER OF DECLARED CONSTRAINTS OR APPROXIMATED FUNCTIONS.
*  II  NC  NUMBER OF LINEARIZED CONSTRAINTS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  MC  NUMBER OF ELEMENTS IN THE FIELD CG.
*  II  IAA(NF)  VECTOR CONTAINING INDICES OF ACTIVE FUNCTIONS.
*  RI  AG(MA)  SPARSE APPROXIMATED FUNCTION JACOBIAN MATRIX.
*  II  IAG(NA+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD AG.
*  II  JAG(MA) COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RO  AZ(NF+1)  VECTOR OF LAGRANGE MULTIPLIERS.
*  RI  CG(MC)  SPARSE CONSTRAINT JACOBIAN MATRIX.
*  II  ICG(NC+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CG.
*  II  JCG(MC) COLUMN INDICES OF ELEMENTS IN THE FIELD CG.
*  RO  G(NF)  NEGATIVE PROJECTED GRADIENT OF THE QUADRATIC FUNCTION.
*
* SUBPROGRAMS USED :
*  S   UXWDIR  VECTOR AUGMENTED BY THE SCALED SPARSE VECTOR.
*
      SUBROUTINE UMGLAG(NF,N,NA,NB,NC,MA,MC,IAA,AG,IAG,JAG,AZ,CG,ICG,
     & JCG,G)
      INTEGER NF,N,NA,NB,NC,MA,MC,IAA(NF),IAG(NA+1),JAG(MA),ICG(NB+1),
     & JCG(MC)
      $FLOAT AG(MA),AZ(NF),CG(MC),G(NF)
      INTEGER NAA,J,K,L
      NAA=NF-N
      DO 3 J=1,NAA
      L=IAA(J)
      IF (L.GT.NC) THEN
      L=L-NC
      K=IAG(L)
      L=IAG(L+1)-K
      CALL UXWDIR(L,NF,JAG(K),-AZ(J),AG(K),G,G,1)
      ELSE IF (L.GT.0) THEN
      K=ICG(L)
      L=ICG(L+1)-K
      CALL UXWDIR(L,NF,JCG(K),-AZ(J),CG(K),G,G,1)
      ELSE
      L=-L
      G(L)=G(L)-AZ(J)
      ENDIF
    3 CONTINUE
      RETURN
      END
