* SUBROUTINE UMRMR0               ALL SYSTEMS                93/12/01
C PORTABILITY : ALL SYSTEMS
C 93/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TRIANGULAR DECOMPOSITION OF KERNEL OF THE ORTHOGONAL PROJECTION IS
* UPDATED AFTER CONSTRAINT DELETION.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  MMAX  MAXIMUM NUMBER OF NONZERO ELEMENTS OF THE MATRIX CR.
*  IU  ICA(NF)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  CR(MMAX)  TRIANGULAR DECOMPOSITION OF KERNEL OF THE
*         ORTHOGONAL PROJECTION.
*  II  ICR(NF+1)  POSITION OF THE FIRST RORWS ELEMENTS IN THE FIELD CR.
*  II  JCR(MMAX) COLUMN INDICES OF ELEMENTS IN THE FIELD CR.
*  RA  CK(NF)  AUXILIARY VECTOR.
*  RA  CL(NF)  AUXILIARY VECTOR.
*  II  IOLD  INDEX OF THE OLD ACTIVE CONSTRAINT.
*  IO  KREM  AUXILIARY VARIABLE.
*
* COMMON DATA :
*  IO  ITERM  TERMINATION INDICATOR.
*  TO  TDXX  TEXT INFORMATION ON THE DIRECTION VECTOR OBTAINED.
*
* SUBPROGRAMS USED :
*  S   UXVORT  DETERMINATION OF AN ELEMENTARY ORTHOGONAL MATRIX FOR
*         PLANE ROTATION.
*  S   UXVROT  PLANE ROTATION OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*  S   UOERR1  ERROR MESSAGES.
*
      SUBROUTINE UMRMR0(NF,N,MMAX,ICA,CR,ICR,JCR,CK,CL,IOLD,KREM)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,MMAX,ICA(NF),ICR(NF+1),JCR(MMAX),IOLD,KREM
      $FLOAT CR(MMAX),CK(NF),CL(NF)
      INTEGER NCA,I,J,K,K1,K2,L,M1,M2,KC
      NCA=NF-N
      IF (IOLD.LT.NCA) THEN
      KC=ICA(IOLD)
      M1=ICR(NCA+1)
      M2=MMAX+1-NF
      K=M2-M1
      IF (K.LE.0) THEN
      ITERM=-18
      TDXX='LACK SPC'
      CALL UOERR1('UMRMR0',18)
      RETURN
      ENDIF
      L=M1-ICR(IOLD+1)
      DO 1 J=1,L
      CR(M2-J)=CR(M1-J)
      JCR(M2-J)=JCR(M1-J)
    1 CONTINUE
      DO 2 I=IOLD+1,NCA+1
      ICR(I)=ICR(I)+K
    2 CONTINUE
      K2=M2-1
      L=ICR(IOLD)
      DO 6 I=IOLD+1,NCA
      CALL UXVSET(I-IOLD,0.0$P 0,CR(K2+IOLD))
      M1=ICR(I)
      M2=ICR(I+1)-1
      DO 3 J=M1,M2
      K=JCR(J)
      IF (K.LT.IOLD) THEN
      CR(L)=CR(J)
      JCR(L)=K
      L=L+1
      ELSE
      CR(K2+K)=CR(J)
      ENDIF
    3 CONTINUE
      DO 4 K=IOLD,I-1
      K1=K+1
      IF (K1.EQ.I) THEN
      CALL UXVORT(CR(K2+K),CR(K2+K1),CK(K),CL(K),JCR(K2+K))
      ELSE
      CALL UXVROT(CR(K2+K),CR(K2+K1),CK(K),CL(K),JCR(K2+K))
      ENDIF
    4 CONTINUE
      DO 5 K=IOLD,I-1
      IF (CR(K2+K).NE.0.0$P 0) THEN
      CR(L)=CR(K2+K)
      JCR(L)=K
      L=L+1
      ENDIF
    5 CONTINUE
      IF (L-1.GT.M2) THEN
      ITERM=-18
      TDXX='LACK SPC'
      CALL UOERR1('UMRMR0',18)
      RETURN
      ENDIF
      ICR(I)=L
      ICA(I-1)=ICA(I)
    6 CONTINUE
      ICA(NCA)=KC
      ENDIF
      KREM=1
      RETURN
      END
