* SUBROUTINE UNCUR3                ALL SYSTEMS                15/12/01
C PORTABILITY : ALL SYSTEMS
C 15/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ACCEPTABLE ROOTS OF A CUBIC EQUATION.
*
* PARAMETERS :
*  RI  A0  COEFFICIENT OF THE CUBIC EQUATION.
*  RI  A1  COEFFICIENT OF THE CUBIC EQUATION.
*  RI  A2  COEFFICIENT OF THE CUBIC EQUATION.
*  RI  A3  COEFFICIENT OF THE CUBIC EQUATION.
*  RI  B  BOUND FOR ACCEPTABLE ROOTS.
*  RO  X  THE SMALLER ACCEPTABLE ROOT.
*  IO  INF  THE CASE INDICATOR. INF=0-ACCEPTABLE ROOTS DOES NOT EXIST.
*         INF=1-AN ACCEPTABLE ROOT OF THE LINEAR EQUATION, INF=2-AN
*         ACCEPTABLE ROOT OF THE QUADRATIC EQUATION. INF=3-AN 
*         ACCEPTABLE ROOT OF THE CUBIC EQUATION. INF=-1-THE INITIAL
*         FUNCTION VALUE IS NONPOSITIVE, INF=-2-THE INITIAL DERIVATIVE
*         IS NONNEGATIVE.    
*
      SUBROUTINE UNCUR3(A0,A1,A2,A3,B,X,INF)
      REAL*8 A0,A1,A2,A3,B,X
      INTEGER INF
      REAL*8 C,D,E,F,X1,X2,X3
      REAL*8 ZERO,TWO,THREE,FOUR
      REAL*8 PI3
      PARAMETER (ZERO=0.0$P 0,TWO=2.0$P 0,THREE=3.0$P 0,FOUR=4.0$P 0)
      PARAMETER (PI3=1.047197551196597746$P 0)
      INF=ZERO
      IF (A0.LE.ZERO) THEN
      INF=-1
      RETURN 
      ELSE IF (A1.GE.ZERO) THEN
      INF=-2
      RETURN
      ELSE IF (A3.LE.ZERO) THEN
*
*     THE QUADRATIC CASE
*    
      IF (A2.EQ.ZERO) THEN
      X=-A0/A1
      IF (X.GT.B) INF=1
      RETURN
      ENDIF
      D=A1*A1-FOUR*A0*A2
      IF (D.LT.ZERO) RETURN
      D=SQRT(D)
      X=-(A1+D)/(TWO*A2)
      IF (X.GT.B) THEN
      INF=2
      RETURN
      ENDIF
      X=-(A1-D)/(TWO*A2)
      IF (X.GT.B) THEN
      INF=2
      RETURN
      ENDIF
      ELSE
*
*     THE CUBIC CASE
* 
      C=A1/A3-(A2/A3)**2/THREE
      D=A0/A3-(A1*A2)/(A3*A3*THREE)+(A2/A3)**3/13.5$P 0
      IF((D/TWO)**2+(C/THREE)**3.GE.ZERO.OR.C.EQ.ZERO) RETURN
      F=ACOS(-D/SQRT((ABS(C)/THREE)**3*FOUR))/THREE
      E=SQRT(ABS(C)/0.75$P 0)
      X1= E*COS(F)-A2/(A3*THREE)
      X2=-E*COS(F-PI3)-A2/(A3*THREE)
      X3=-E*COS(F+PI3)-A2/(A3*THREE)
      X=MAX(X1,X2,X3)
      IF (X.LE.B) RETURN
      IF (X1.GT.B.AND.X1.LT.X) X=X1
      IF (X2.GT.B.AND.X2.LT.X) X=X2
      IF (X3.GT.B.AND.X3.LT.X) X=X3
      INF=3
      ENDIF
      RETURN
      END   
        
     