* SUBROUTINE UNINT6                ALL SYSTEMS                90/12/01
C PORTABILITY : ALL SYSTEMS
C 90/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* EXTRAPOLATION OR INTERPOLATION FOR LINE SEARCH USED IN CONSTRAINED
* MINIMIZATION USING L1 EXACT PENALTY FUNCTION.
*
* PARAMETERS :
*  II  NC  NUMBER OF CONSTRAINTS.
*  RU  CF(NC+1)  VECTOR CONTAINING NEW VALUES OF THE CONSTRAINT
*         FUNCTIONS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CFO(NC+1)  VECTOR CONTAINING OLD VALUES OF THE CONSTRAINT
*         FUNCTIONS.
*  RI  CFD(NC+1)  VECTOR CONTAINING INCREMENTS OF THE CONSTRAINT
*         FUNCTIONS.
*  RI  CP(NC)  VECTOR CONTAINING PENALTY PARAMETERS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  ETA0  MACHINE PRECISION.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RA  FC  AUXILIARY VARIABLE.
*  IO  MERR  ERROR INDICATOR. MERR=0 FOR NORMAL RETURN.
*  II  MODE  MODE OF INTERPOLATION.
*
* COMMON DATA :
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
      SUBROUTINE UNINT6(NC,CF,CL,CU,CFO,CFD,CP,IC,ETA0,R,FC,MERR,MODE)
      $INCLUDE ('UMCOMM')
      INTEGER NC,IC(NC),MERR,MODE
      $FLOAT CF(NC+1),CL(NC),CU(NC),CFO(NC+1),CFD(NC+1),CP(NC),ETA0,R,FC
      $FLOAT RI,AI,BI,CI,EI,RJ,AJ,BJ,CJ,EJ,RK,AK,BK,CK,A,B,C
      $FLOAT RTEMP
      INTEGER K,KC
      LOGICAL LI,LJ,LK
      $FLOAT CON1,CON2
      PARAMETER (CON1=1.0$P-1,CON2=1.0$P-10)
      MERR=0
      RTEMP=R
      IF (MODE.LE.0) THEN
      RETURN
      ELSE IF (MODE.EQ.1) THEN
      R=TWO*R
      TSXX='BISECT. '
      RETURN
      ELSE
      TSXX='L1 EXACT'
      ENDIF
      C= CFO(NC+1)
      B= CFD(NC+1)
      A= ((CF(NC+1)-CFO(NC+1))/R-CFD(NC+1))/R
      RI=ZERO
      CI=C
      BI=B
      AI=A
      RJ=R
      CJ=C
      BJ=B
      AJ=A
      DO 1 KC=1,NC
      IF (IC(KC).NE.0) THEN
      FC=((CF(KC)-CFO(KC))/R-CFD(KC))/R
      IF (IC(KC).EQ.1.OR.IC(KC).GE.3) THEN
      IF (ZERO.GT.CFO(KC)-CL(KC)) THEN
      CI=CI-CP(KC)*(CFO(KC)-CL(KC))
      BI=BI-CP(KC)*CFD(KC)
      AI=AI-CP(KC)*FC
      ENDIF
      IF (ZERO.GT.CF(KC)-CL(KC)) THEN
      CJ=CJ-CP(KC)*(CFO(KC)-CL(KC))
      BJ=BJ-CP(KC)*CFD(KC)
      AJ=AJ-CP(KC)*FC
      ENDIF
      ENDIF
      IF (IC(KC).EQ.2.OR.IC(KC).GE.3) THEN
      IF (ZERO.LT.CFO(KC)-CU(KC)) THEN
      CI=CI+CP(KC)*(CFO(KC)-CU(KC))
      BI=BI+CP(KC)*CFD(KC)
      AI=AI+CP(KC)*FC
      ENDIF
      IF (ZERO.LT.CF(KC)-CU(KC)) THEN
      CJ=CJ+CP(KC)*(CFO(KC)-CU(KC))
      BJ=BJ+CP(KC)*CFD(KC)
      AJ=AJ+CP(KC)*FC
      ENDIF
      ENDIF
      CF(KC)=FC
      ENDIF
    1 CONTINUE
      IF (TWO*AJ*R+BJ.LT.ZERO) RETURN
      IF (BI.GE.ZERO) THEN
      R =HALF*(RI+RJ)
      TSXX='BISECT. '
      RETURN
      ENDIF
      LK=.TRUE.
      LK=LK.AND.ABS(AI-AJ).LE.CON2*ABS(AI)
      LK=LK.AND.ABS(BI-BJ).LE.CON2*ABS(BI)
      LK=LK.AND.ABS(CI-CJ).LE.CON2*ABS(CI)
    2 RK=HALF*(RI+RJ)
      K=0
      IF (LK) THEN
      K=1
      IF (AI.LE.ETA0*ABS(BI)) THEN
      IF (BI.GE.ZERO) THEN
      R=RI
      ELSE
      R=RJ
      ENDIF
      ELSE
      R=-HALF*BI/AI
      IF (R.LE.CON1*RTEMP) THEN
      R=CON1*RTEMP
      TSXX='LOWER B.'
      RETURN
      ENDIF
      ENDIF
      ELSE
      CK=CJ-CI
      BK=BJ-BI
      AK=AJ-AI
      IF (ABS(AK).LE.ETA0*ABS(BK)) THEN
      IF (ABS(BK).LE.ETA0*ABS(CK)) THEN
      IF (AI.LE.ETA0*ABS(BI)) THEN
      R=RK
      ELSE
      R=-HALF*BI/AI
      ENDIF
      ELSE
      R=-CK/BK
      K=2
      ENDIF
      ELSE
      R=-HALF*BK/AK
      BK=SQRT(MAX(ZERO,R*R-CK/AK))
      AK=R+BK
      CK=R-BK
      IF (AK.GT.RI.AND.AK.LT.RJ) THEN
      R=AK
      K=2
      ELSE IF (CK.GT.RI.AND.CK.LT.RJ) THEN
      R=CK
      K=2
      ELSE
      R=RK
      ENDIF
      ENDIF
      ENDIF
      IF (RJ-RI.LE.CON2*RTEMP) RETURN
      CK=C
      BK=B
      AK=A
      DO 3 KC=1,NC
      IF (IC(KC).NE.0) THEN
      FC=(CF(KC)*R+CFD(KC))*R+CFO(KC)
      IF (IC(KC).EQ.1.OR.IC(KC).GE.3) THEN
      IF (ZERO.GT.FC-CL(KC)) THEN
      CK=CK-CP(KC)*(CFO(KC)-CL(KC))
      BK=BK-CP(KC)*CFD(KC)
      AK=AK-CP(KC)*CF(KC)
      ENDIF
      ENDIF
      IF (IC(KC).EQ.2.OR.IC(KC).GE.3) THEN
      IF (ZERO.LT.FC-CU(KC)) THEN
      CK=CK+CP(KC)*(CFO(KC)-CU(KC))
      BK=BK+CP(KC)*CFD(KC)
      AK=AK+CP(KC)*CF(KC)
      ENDIF
      ENDIF
      ENDIF
    3 CONTINUE
      LI=.TRUE.
      LI=LI.AND.ABS(AK-AI).LE.CON2*ABS(AK)
      LI=LI.AND.ABS(BK-BI).LE.CON2*ABS(BK)
      LI=LI.AND.ABS(CK-CI).LE.CON2*ABS(CK)
      LJ=.TRUE.
      LJ=LJ.AND.ABS(AK-AJ).LE.CON2*ABS(AK)
      LJ=LJ.AND.ABS(BK-BJ).LE.CON2*ABS(BK)
      LJ=LJ.AND.ABS(CK-CJ).LE.CON2*ABS(CK)
      IF ((LI.OR.LJ).AND.K.NE.0) THEN
      IF (LK) RETURN
      EI=TWO*AI*R+BI
      EJ=TWO*AJ*R+BJ
      IF (EI*EJ.LE.ZERO) RETURN
      LK=.TRUE.
      IF (EI.LE.ZERO) THEN
      RI=R
      AI=AJ
      BI=BJ
      CI=CJ
      ELSE
      RJ=R
      AJ=AI
      BJ=BI
      CJ=CI
      ENDIF
      ELSE
      LK=.FALSE.
      IF (TWO*AK*R+BK.LT.ZERO) THEN
      IF (LJ) LK=.TRUE.
      RI=R
      AI=AK
      BI=BK
      CI=CK
      ELSE
      IF (LI) LK=.TRUE.
      RJ=R
      AJ=AK
      BJ=BK
      CJ=CK
      ENDIF
      ENDIF
      GO TO 2
      END



