* SUBROUTINE UNNEQ2                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SINGLE NONLINEAR EQUATION.
*
* PARAMETERS :
*  RI  AA  LEFT ENDPOINT OF THE INTERVAL.
*  RI  BB  RIGHT ENDPOINT OF THE INTERVAL.
*  RO  X  COMPUTED SOLUTION POINT.
*  RO  F  COMPUTED VALUE OF THE NONLINEAR FUNCTION.
*  RI  REQUIRED PRECISION FOR THE SOLUTION POINT.
*  RI  REQUIRED PRECISION FOR THE NONLINEAR FUNCTION.
*  IO  IC NUMBER OF ITERATIONS.
*  IO  IE ERROR SPECIFICATION.
*
* METHOD :
* D.LEE: AN EFFICIENT DERIVATIVE FREE METHOD FOR SOLVING NONLINEAR
* EQUATIONS, ACM TRANS. ON MATH. SOFTWARE 11 (1985) 250-262.
*
      SUBROUTINE UNNEQ2(AA,BB,X,F,FUN,EPSX,EPSF,IC,IE)
      $FLOAT AA,BB,X,F,FUN,EPSX,EPSF
      INTEGER IC,IE
      INTEGER ITER,ITMAX,INT
      $FLOAT FA,FB,X1,X2,X3,F1,F2,F3,D,D1,D2,D3,C,E,G,G1,G3,GE,H,W,Z
      $FLOAT FC,FG,FW,DEL
      $FLOAT ZERO,ONE,TWO,HALF,CON
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0,TWO=2.0$P 0,
     & HALF=0.5$P 0,CON=0.595$P 0)
      EXTERNAL FUN
      IE=0
      ITMAX=IC-3
      IF (ITMAX.LE.0) ITMAX=100
      IC=2
      FA=FUN(AA)
      IF (ABS(FA).LE.EPSF) THEN
      X=AA
      F=FA
      RETURN
      ENDIF
      FB=FUN(BB)
      IF (ABS(FB).LE.EPSF) THEN
      X=BB
      F=FB
      RETURN
      ENDIF
      IF (FA*FB.GT.ZERO) THEN
      X=AA
      F=FA
      IE=-2
      RETURN
      ENDIF
      X1=AA
      F1=FA
      X2=HALF*(AA+BB)
      F2=FUN(X2)
      IC=IC+1
      IF (F1*F2.GT.ZERO) THEN
      X3=X1
      F3=F1
      X1=BB
      F1=FB
      ELSE
      X3=BB
      F3=FB
      ENDIF
      INT=1
      D=ABS(AA-BB)
      D1=D+D
      D2=D1+D1
      DO 20 ITER=1,ITMAX
      D3=D2
      D2=D1
      D1=D
      D=ABS(X1-X2)
      IF (ABS(F1).LT.ABS(F2)) THEN
      X=X1
      F=F1
      ELSE
      X=X2
      F=F2
      ENDIF
      DEL=EPSX*(ABS(X)+ONE)
      IF (ABS(F).LE.EPSF.OR.D.LE.TWO*DEL) RETURN
      Z=X1+HALF*(X2-X1)
      IF (INT.EQ.1) THEN
      C=X1
      FC=F1
      ENDIF
      E=X2+HALF*(C-X2)
      GE=(F2-FC)/(X2-C)
      H=TWO*((F2-F3)/(X2-X3)-GE)/(X3-C)
      G1=GE+H*(X1-E)
      G3=GE+H*(X3-E)
      IF (D.GT.CON*D3.OR.G1*G3.LE.ZERO.AND.INT.EQ.1) GO TO 11
      G=GE+H*(X-E)
      FG=F/G
      W=X-(ONE+HALF*FG*H/G)*FG
      IF ((W-X)*SIGN(ONE,(W-Z)).LE.ZERO) GO TO 12
      W=X-FG
      IF (ABS(W-X).LE.ABS(Z-X)) GO TO 12
   11 W=Z
      GO TO 13
   12 IF (ABS(W-X).LT.DEL) W=X+DEL*SIGN(ONE,Z-X)
   13 FW=FUN(W)
      IC=IC+1
      INT=1
      IF (FW*SIGN(ONE,F1).GE.ZERO) THEN
      IF (D.LE.ABS(X3-W)) THEN
      X3=X1
      F3=F1
      X1=X2
      F1=F2
      X2=W
      F2=FW
      ELSE
      X1=W
      F1=FW
      ENDIF
      ELSE
      IF (D.LE.ABS(X3-W)) THEN
      X3=X2
      F3=F2
      X2=W
      F2=FW
      ELSE
      C=X3
      FC=F3
      X3=X2
      F3=F2
      X2=W
      F2=FW
      INT=0
      ENDIF
      ENDIF
   20 CONTINUE
      IE=-1
      RETURN
      END
