* SUBROUTINE UODLI4                ALL SYSTEMS                99/12/01
C PORTABILITY : ALL SYSTEMS
C 99/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* PRINT OF ITERATIONS IN INTERIOR POINT METHOD.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  D(NF+NCA) DIAGONAL SUBMATRICES.
*  RI  S(NCA)  STEP FOR LAGRANGE MULTIPLIERS CORRESPONDING TO
*         ACTIVE CONSTRAINTS.
*  RI  XO(NX+NC)  STEP FOR LAGRANGE MULTIPLIERS CORRESPONDING TO
*         INACTIVE LOWER CONSTRAINTS.
*  RI  GO(NX+NC)  STEP FOR LAGRANGE MULTIPLIERS CORRESPONDING TO
*         INACTIVE UPPER CONSTRAINTS.
*  RI  GS(NCA)  STEP FOR SLACK VARIABLES CORRESPONDING TO ACTIVE
*         CONSTRAINTS.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  IWR  NUMBER OF THE OUTPUT UNIT.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=5-ADDITIONAL PRINT OF DEBUGGING.
*  II  LOUT  LEVEL OF PRINT. LOUT=0-PRINT OF SCALAR VALUES.
*         LOUT=1-ADDITIONAL PRINT OF BASIC VECTORS.
*         LOUT=2-ADDITIONAL PRINT OF OTHER VECTORS.
*         LOUT=3-ADDITIONAL PRINT OF MATRICES.
*
      SUBROUTINE UODLI4(NF,NX,NC,NCA,D,S,XO,GO,GS)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,NCA
      $FLOAT D(NF+NC),S(NCA),XO(NX+NC),GO(NX+NC),GS(NCA)
      INTEGER I
      IF(JOUT.EQ.0) RETURN
      IF(ABS(KOUT).LE.3) RETURN
      IF (ABS(LOUT).GE.3) WRITE (IWR,1) (D(I),I=1,MIN(NF,25))
    1 FORMAT(' ','KX  =',5$P 14.5:/(6X,5$P 14.5))
      IF (ABS(LOUT).GE.3) WRITE (IWR,8) (D(NF+I),I=1,
     & MIN(NCA,25))
    8 FORMAT(' ','KU  =',5$P 14.5:/(6X,5$P 14.5))
      IF (ABS(LOUT).GE.2) WRITE (IWR,2) (GS(I),I=1,
     & MIN(NCA,25))
    2 FORMAT(' ','DSA =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.2) WRITE(IWR,3) (S(I),I=1,MIN(NCA,25))
    3 FORMAT(' ','DZA =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.3) THEN
      IF (KBF.GE.1) WRITE(IWR,4) (XO(I),I=1,MIN(NX,25))
      IF (KBF.GE.2) WRITE(IWR,5) (GO(I),I=1,MIN(NX,25))
      IF (KBC.GE.1) WRITE(IWR,6) (XO(NX+I),I=1,MIN(NC,25))
      IF (KBC.GE.2) WRITE(IWR,7) (GO(NX+I),I=1,MIN(NC,25))
    4 FORMAT(' ','DXZL=',5$P 14.5:/(6X,5$P 14.5))
    5 FORMAT(' ','DXZU=',5$P 14.5:/(6X,5$P 14.5))
    6 FORMAT(' ','DCZL=',5$P 14.5:/(6X,5$P 14.5))
    7 FORMAT(' ','DCZU=',5$P 14.5:/(6X,5$P 14.5))
      ENDIF
      RETURN
      END
