* SUBROUTINE UODLX4                ALL SYSTEMS                05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* PRINT OF ITERATIONS IN INTERIOR POINT METHOD.
*
* PARAMETERS:
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  D(NF+NCA) DIAGONAL SUBMATRICES.
*  RI  S(NCA)  STEP FOR LAGRANGE MULTIPLIERS CORRESPONDING TO
*         ACTIVE CONSTRAINTS.
*  RI  XO(NX+NC)  STEP FOR LAGRANGE MULTIPLIERS CORRESPONDING TO
*         INACTIVE LOWER CONSTRAINTS.
*  RI  GO(NX+NC)  STEP FOR LAGRANGE MULTIPLIERS CORRESPONDING TO
*         INACTIVE UPPER CONSTRAINTS.
*  RI  GS(NCA)  STEP FOR SLACK VARIABLES CORRESPONDING TO ACTIVE
*         CONSTRAINTS.
*
* COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*  II  IWR  NUMBER OF THE OUTPUT UNIT.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=5-ADDITIONAL PRINT OF DEBUGGING.
*  II  LOUT  LEVEL OF PRINT. LOUT=0-PRINT OF SCALAR VALUES.
*         LOUT=1-ADDITIONAL PRINT OF BASIC VECTORS.
*         LOUT=2-ADDITIONAL PRINT OF OTHER VECTORS.
*         LOUT=3-ADDITIONAL PRINT OF MATRICES.
*
      SUBROUTINE UODLX4(NF,NA,X,G,S,GS,FF,RPF3,INF)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NA,INF
      $FLOAT X(NF+1),G(NF+1),S(NF+1),GS(NF+1),FF,RPF3
      INTEGER I
      LOGICAL LL
      LL=KOUT.EQ.0.AND.LOUT.GT.0
      IF(ABS(MOUT).GT.3) THEN
      WRITE (ITR,10) NF,NA,X(NF+1),G(NF+1),FF
      IF(LL) WRITE (IWR,10) NF,NA,X(NF+1),G(NF+1),FF
      WRITE (ITR,11) NF,INF,S(NF+1),GS(NF+1),RPF3
      IF(LL) WRITE (IWR,11) NF,INF,S(NF+1),GS(NF+1),RPF3
      ENDIF
   10 FORMAT (' ','NF =',I4,2X,'NA =',I4,2X,
     & 'X =',$P 9.3,2X,'G =',$P 9.3,2X,'FF  =',$P 9.3)
   11 FORMAT (' ','NF =',I4,2X,'INF=',I4,2X,
     & 'S =',$P 9.3,2X,'B =',$P 9.3,2X,'RPF3=',$P 9.3)
      IF(JOUT.EQ.0) RETURN
      IF(ABS(KOUT).LE.3) RETURN
      WRITE (IWR,1) NF,NA,X(NF+1),G(NF+1),FF
      WRITE (IWR,2) NF,INF,S(NF+1),GS(NF+1),RPF3
    1 FORMAT ( ' ','NF =',I4,2X,'NA =',I4,2X,
     & 'X =',$P 9.3,2X,'G =',$P 9.3,2X,'FF =',$P 9.3)
    2 FORMAT ( ' ','NF =',I4,2X,'INF=',I4,2X,
     & 'S =',$P 9.3,2X,'B =',$P 9.3,2X,'RPF=',$P 9.3)
      IF (ABS(LOUT).GE.2) WRITE (IWR,3) (G(I),I=1,MIN(NF,25))
    3 FORMAT(' ','G   =',5$P 14.5:/(6X,5$P 14.5))
      IF (ABS(LOUT).GE.2) WRITE (IWR,4) (S(I),I=1,MIN(NF,25))
    4 FORMAT(' ','S   =',5$P 14.5:/(6X,5$P 14.5))
      IF (ABS(LOUT).GE.3) WRITE (IWR,5) (GS(I),I=1,MIN(NF,25))
    5 FORMAT(' ','B   =',5$P 14.5:/(6X,5$P 14.5))
      RETURN
      END
