* SUBROUTINE UOKLQ2                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
*  PRINT OF EXIT FROM QUADRATIC PROGRAMMING SUBROUTINE.
*
* PARAMETERS:
*  II  N NUMBER OF VARIABLES.
*  II  M NUMBER OF CONSTRAINTS.
*  II  MROWS DECLARED NUMBER OF ROWS.
*  II  MCOLS DECLARED NUMBER OF COLUMNS.
*  RI  X(MCOLS) VECTOR OF SOLUTION.
*  RI  C(MCOLS) COST VECTOR.
*  II  TYPE(MCOLS) TYPES OF THE VARIABLES:
*                                    =1 FOR NONBASIC LOWER BOUNDED
*                                    =2 FOR NONBASIC UPPER BOUNDED
*                                    =3 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS LOWER BOUND
*                                    =4 FOR NONBASIC LOWER AND UPPER
*                     BOUNDED VARIABLE SET TO ITS UPPER BOUND
*                                    =5 FOR NONBASIC FIXED VARIABLE
*                                    =6 FOR NONBASIC FREE VARIABLE
*                                    =-1 FOR BASIC LOWER BOUNDED
*                                    =-2 FOR BASIC UPPER BOUNDED
*                                    =-3 FOR BASIC LOWER AND UPPER
*                     BOUNDED VARIABLE
*                                    =-6 FOR BASIC FREE VARIABLE.
*  II  BCOL(MCOLS) M FIRST ELEMENTS CONTAIN COLUMN INDICES OF BASIC
*              COLUMNS.
*  II  LENR(MCOLS) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  F  FUNCTIONAL VALUE.
*  RI  GNORM NORM OF THE REDUCED GRADIENT.
*  II  NIQ MINOR CYCLE COUNT.
*  II  NAQ MAJOR CYCLE COUNT.
*  II  LINP NUMBER OF BASICS AND SUPERBASICS.
*
* COMMON DATA :
*  II  NCGR NUMBER OF CG ITERATIONS.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF MATRIX CORRECTION.
*         KOUT=5-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=6-ADDITIONAL PRINT OF DEBUGGING.
*  II  LOUT  PRINT OF ITERATIONS. LOUT=0-STANDARD PAPER SAVING PRINT.
*         LOUT=1-ADDITIONAL PRINT OF FINAL RESULTS OF SUBROUTINES.
*         LOUT=2-ADDITIONAL PRINT OF VARIABLES. LOUT=3-ADDITIONAL
*         PRINT OF THE GRADIENT. LOUT=4-ADDITIONAL PRINT OF THE HESSIAN
*         MATRIX.
*  II  MOUT  LEVEL OF PRINT ON THE TERMINAL DEVICE.
*         ABS(MOUT)=0 - NO PRINT.
*         ABS(MOUT)=1 - PRINT OF FINAL RESULTS.
*         ABS(MOUT)=2 - ADDITIONAL PRINT OF ITERATIONS.
*
      SUBROUTINE UOKLQ2(N,M,MROWS,MCOLS,X,C,TYPE,BCOL,LENR,F,GNORM,
     & NIQ,NAQ,LINP)
      $INCLUDE('UMCOMM')
      INTEGER N,M,MROWS,MCOLS,TYPE(MCOLS),BCOL(MCOLS),LENR(MROWS),
     & NIQ,NAQ,LINP
      $FLOAT X(MCOLS),C(MCOLS),F,GNORM
      INTEGER NSBSP,I,IU
      LOGICAL LL
      LL=KOUT.EQ.0.AND.LOUT.GT.0
C
C NUMBER OF SUPERBASIC VARIABLES
C
      NSBSP=LINP-M
C
C COMPUTE IU
C
      IU=0
      DO 100 I=1,M
        IU=IU+LENR(I)
  100 CONTINUE
      IF(ABS(MOUT).GT.2) THEN
      WRITE(ITR,9) NAQ,NIQ,NCGR,ITERD,ITERQ,F,GNORM
      IF(LL) WRITE(IWR,9) NAQ,NIQ,NCGR,ITERD,ITERQ,F,GNORM
      ENDIF
    9 FORMAT(' ','NAQ=',I3,1X,'NIQ=',I5,1X,'NCGR=',I5,1X,
     *       'ITERD=',I2,1X,'ITERQ=',I2,2X,'F=',$P 13.7,2X,'G=',$P 9.3)
      IF(JOUT.EQ.0) RETURN
      IF(ABS(KOUT).LE.3) RETURN
      IF(ABS(LOUT).GE.1) WRITE(IWR,5)(X(I),I=1,MIN(N,25))
    5 FORMAT(' ','X   =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.2) WRITE(IWR,8)(C(I),I=1,MIN(N,25))
    8 FORMAT(' ','GF  =',5$P 14.5:/(6X,5$P 14.5))
      IF(ABS(LOUT).GE.2) WRITE(IWR,4)(BCOL(I),I=1,MIN(M,50))
    4 FORMAT(' ','BCOL=',10I7:/(6X,10I7))
      IF(ABS(LOUT).GE.2.AND.KBF.GT.0)WRITE(IWR,6)(TYPE(I),I=1,MIN(N,50))
    6 FORMAT(' ','IX  =',10I7:/(6X,10I7))
      WRITE(IWR,2) NAQ,NIQ,NSBSP,NCGR,IU,F,GNORM
    2 FORMAT(' ','NAQ=',I3,2X,'NIQ=',I5,2X,'NSBSP=',I4,2X,
     *       'NCGR=',I3,2X,'IU=',I5,2X,'F=',$P 9.3,1X,
     *       'G=',$P 9.3)
      WRITE(IWR,1) ITERQ,TQPX
    1 FORMAT(' ','END OF QUADRATIC PROGRAMMING ',
     *       ' : ITERQ = ',I3,'  ',A8)
      RETURN
      END
