$INTERFACE UP0AF7                                             01/12/01

  $HELP
    SMOOTHING PARAMETER
  $CHECK(EPS7,'1.0$P-8',REAL,4)
  CALL UP0AF7(NF,NX,NC,NCA,$X,$IX,$XL,$XU,$XZL,$XZU,$CF,$IC,$CL,$CU&
,$CZL,$CZU,$ICA,$S,$XO(NF+1),$GO(NF+1),EPS7,R,RPF2,FC,F)
  $ADD(REAL,'\$X($NF)\$XL($NX)\$XU($NX)\$XZL($NX)\$XZU($NX)\$CF($NC+1)&
  \$CL($NC)\$CU($NC)\$CZL($NC)\$CZU($NC)\S($NF+$NCA)\XO($NF+$NCA)&
  \GO($NF+$NCA)')
  $ADD(INTEGER,'\$IX($NX)\$IC($NC)\$ICA($NCA)')
$REM  $ADD(REAL,'\EPS7\R\RPF2\FC\F')
$REM  $ADD(INTEGER,'\NF\NX\NC\NCA')
  $ADD(INIT)
    NPFX='UP0AF7'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UP0AF7             ALL SYSTEMS                 01/12/01
C PORTABILITY : ALL SYSTEMS
C 01/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF VALUE OF THE AUGMENTED LAGRANGIAN FUNCTION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  NX  NUMBER OF BOX CONSTRAINTS.
*  II  NC  NUMBER OF GENERAL CONSTRAINTS.
*  II  NCA  NUMBER OF ACTIVE CONSTRAINTS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XL(NF)  VECTOR CONTAINING LOWER BOUNDS FOR VARIABLES.
*  RI  XU(NF)  VECTOR CONTAINING UPPER BOUNDS FOR VARIABLES.
*  RI  XZL(NF)  VECTOR CONTAINING LOVER MULTIPLIERS FOR VARIABLES.
*  RI  XZU(NF)  VECTOR CONTAINING UPPER MULTIPLIERS FOR VARIABLES.
*  RI  CF(NC+1)  VECTOR CONTAINING VALUES OF CONSTRAINT FUNCTIONS.
*  II  IC(NC)  VECTOR CONTAINING TYPES OF CONSTRAINTS.
*  RI  CL(NC)  VECTOR CONTAINING LOWER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CU(NC)  VECTOR CONTAINING UPPER BOUNDS FOR CONSTRAINT FUNCTIONS.
*  RI  CZL(NC)  VECTOR CONTAINING LOVER MULTIPLIERS FOR CONSTRAINTS.
*  RI  CZU(NC)  VECTOR CONTAINING UPPER MULTIPLIERS FOR CONSTRAINTS.
*  II  ICA(NCA)  VECTOR CONTAINING INDICES OF ACTIVE CONSTRAINTS.
*  RI  S(NCA)  VECTOR CONTAINING NEWTON INCREMENTS.
*  RI  XO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR LOWER INACTIVE
*         MULTIPLIERS.
*  RI  GO(NX+NC)  VECTOR CONTAINING INCREMENTS FOR UPPER INACTIVE
*         MULTIPLIERS.
*  RI  EPS7  SMOOTHING PARAMETER.
*  RI  R  STEPSIZE  PARAMETER.
*  RI  RPF2  PENALTY COEFFICIENT.
*  RO  FC  VALUE OF THE PENALTY TERM.
*  RO  F  VALUE OF THE PENALTY FUNCTION.
*
*  COMMON DATA :
*  II  KBF  SPECIFICATION OF SIMPLE BOUNDS. KBF=0-NO SIMPLE BOUNDS.
*         KBF=1-ONE SIDED SIMPLE BOUNDS. KBF=2=TWO SIDED SIMPLE BOUNDS.
*  II  KBC  SPECIFICATION OF LINEAR CONSTRAINTS. KBC=0-NO LINEAR
*         CONSTRAINTS. KBC=1-ONE SIDED LINEAR CONSTRAINTS. KBC=2=TWO
*         SIDED LINEAR CONSTRAINTS.
*
      SUBROUTINE UP0AF7(NF,NX,NC,NCA,X,IX,XL,XU,XZL,XZU,CF,
     & IC,CL,CU,CZL,CZU,ICA,S,XO,GO,EPS7,R,RPF2,FC,F)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NX,NC,NCA,IX(NX),IC(NC),ICA(NCA)
      $FLOAT X(NF),XL(NX),XU(NX),XZL(NX),XZU(NX),CF(NC+1),CL(NC),
     & CU(NC),CZL(NC),CZU(NC),S(*),XO(*),GO(*),EPS7,R,RPF2,FC,F
      INTEGER I,K,KK
      $FLOAT TEMP,TEMP1,TEMP2
      FC=0.0$P 0
      DO 1 I=1,NCA
      K=ICA(I)
      IF (K.LT.0) THEN
      K=-K
      KK=-IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=XL(K)-X(K)
      TEMP2=XZL(K)+R*S(NF+I)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      FC=FC+TEMP1*(XZL(K)+S(NF+I))+HALF*RPF2*TEMP**2
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=X(K)-XU(K)
      TEMP2=XZU(K)+R*S(NF+I)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      FC=FC+TEMP1*(XZU(K)+S(NF+I))+HALF*RPF2*TEMP**2
      ENDIF
      ELSE
      KK=-IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3) THEN
      TEMP1=CL(K)-CF(K)
      TEMP2=CZL(K)+R*S(NF+I)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      FC=FC+TEMP1*(CZL(K)+S(NF+I))+HALF*RPF2*TEMP**2
      ELSE IF (KK.EQ.2.OR.KK.EQ.4) THEN
      TEMP1=CF(K)-CU(K)
      TEMP2=CZU(K)+R*S(NF+I)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      FC=FC+TEMP1*(CZU(K)+S(NF+I))+HALF*RPF2*TEMP**2
      ELSE IF (KK.EQ.5) THEN
      FC=FC+(CZL(K)+S(NF+I)+HALF*RPF2*CF(K))*CF(K)
      ENDIF
      ENDIF
    1 CONTINUE
      IF (KBF.GT.0) THEN
      DO 2 K=1,NX
      KK=IX(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP1=XL(K)-X(K)
      TEMP2=XZL(K)+R*XO(K)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      FC=FC+TEMP1*(XZL(K)+XO(K))+HALF*RPF2*TEMP**2
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP1=X(K)-XU(K)
      TEMP2=XZU(K)+R*GO(K)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      FC=FC+TEMP1*(XZU(K)+GO(K))+HALF*RPF2*TEMP**2
      ENDIF
    2 CONTINUE
      ENDIF
      IF (KBC.GT.0) THEN
      DO 3 K=1,NC
      KK=IC(K)
      IF (KK.EQ.1.OR.KK.EQ.3.OR.ABS(KK).EQ.4) THEN
      TEMP1=CL(K)-CF(K)
      TEMP2=CZL(K)+R*XO(NX+K)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      FC=FC+TEMP1*(CZL(K)+XO(NX+K))+HALF*RPF2*TEMP**2
      ENDIF
      IF (KK.EQ.2.OR.KK.EQ.4.OR.ABS(KK).EQ.3) THEN
      TEMP1=CF(K)-CU(K)
      TEMP2=CZU(K)+R*GO(NX+K)
      TEMP=SQRT(TEMP1**2+TEMP2**2+EPS7**2)+TEMP1-TEMP2
      FC=FC+TEMP1*(CZU(K)+GO(NX+K))+HALF*RPF2*TEMP**2
      ENDIF
    3 CONTINUE
      ENDIF
      F=CF(NC+1)+FC
      RETURN
      END
