$INTERFACE US0C01                                             97/12/01

  $HELP
    STRATEGY FOR STEPSIZE REDUCTION
      1 - QUADRATIC  INTERPOLATION IS USED
      2 - FLETCHER'S INTERPOLATION IS USED
  $CHECK(MOS1,1,'1\2',4)
  $HELP
    STRATEGY FOR PROJECTION STEPS
      1 - PROJECTION STEP IS CHOSEN ONLY IF
          THE BASIC STEP IS UNSUCCESSFUL
      2 - PROJECTION STEP IS CHOSEN WHENEVER THE
          BASIC STEP DOES NOT IMPLY STEPSIZE EXPANSION
  $CHECK(MOS2,1,'1\2',4)
  $HELP
    FIRST TOLERANCE FOR RATIO DF/DFPRED
  $CHECK(EPS3,'0.10$P 0',REAL,4)
  $HELP
    SECOND TOLERANCE FOR RATIO DF/DFPRED
  $CHECK(EPS4,'0.25$P 0',REAL,4)
  $HELP
    LOWER BOUND FOR STEPSIZE REDUCTION
  $CHECK(BET1,'0.5$P-1',REAL,4)
  $HELP
    UPPER BOUND FOR STEPSIZE REDUCTION
  $CHECK(BET2,'0.5$P 0',REAL,4)
  $HELP
    LOWER BOUND FOR STEPSIZE EXPANSION
  $CHECK(GAM1,'2.0$P 0',REAL,4)
  $HELP
    MAXIMUM STEPSIZE
  $CHECK(XMAX,'1.0$P 3',REAL,2)
  CALL US0C01(N,$G,$GO,EPS3,EPS4,BET1,BET2,GAM1,RO,RP,R,FO,F,PO,P&
,XDEL,XNORM,XMAX,SNORM,RMAX,IPOM3,MOS1,MOS2)
  $ADD(REAL,'\$G($NF)\$GO($NF)')
$REM $ADD(REAL,'\EPS3\EPS4\BET1\BET2\GAM1\RO\RP\R\FO\F\PO\P\XDEL\XNORM&
  \XMAX\SNORM\RMAX')
$REM $ADD(INTEGER,'\N\IPOM3\MOS1\MOS2')
  $ADD(INIT)
    NSXX='US0C01'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US0C01                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* CORRECTED SEARCH WITH TRUST REGION UPDATE.
*
* PARAMETERS :
*  II  N  DECLARED NUMBER OF VARIABLES.
*  RI  G(N)  GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RO  GO(N)  SAVED GRADIENT OF THE LAGRANGIAN FUNCTION.
*  RI  EPS3  FIRST TOLERANCE FOR RATIO DF/DFPRED.
*  RI  EPS4  SECOND TOLERANCE FOR RATIO DF/DFPRED.
*  RI  BET1  LOWER BOUND FOR STEPSIZE REDUCTION.
*  RI  BET2  UPPER BOUND FOR STEPSIZE REDUCTION.
*  RI  GAM1  LOWER BOUND FOR STEPSIZE EXPANSION.
*  RO  RO  INITIAL VALUE OF THE STEPSIZE PARAMETER.
*  RO  RP  PREVIOUS VALUE OF THE STEPSIZE PARAMETER.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  P  VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RU  XDEL  TRUST REGION BOUND.
*  RA  XNORM  AUXILIARY VARIABLE.
*  RI  XMAX MAXIMUM STEPSIZE.
*  RI  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  I   IPOM3
*  II  MOS1  CHOICE OF THE SCALING MATRIX. MOS1=1-UNIT MATRIX.
*         MOS1=2-SQUARE OF THE MATRIX H.
*  II  MOS2  CHOICE OF THE WEIGHTING MATRIX. MOS2=1-UNIT MATRIX.
*         MOS1=2-SQUARE OF THE MATRIX TRANS(L)*L.
*
* COMMON DATA :
*  IO  MAXST  MAXIMUM STEPSIZE INDICATOR. MAXST=0 OR MAXST=1 IF MAXIMUM
*         STEPSIZE WAS NOT OR WAS REACHED.
*  IU  IDIR INDICATOR FOR DIRECTION DETERMINATION.
*         IDIR=0-BASIC DETERMINATION. IDIR=1-DETERMINATION
*         AFTER STEPSIZE REDUCTION. IDIR=2-DETERMINATION AFTER
*         STEPSIZE EXPANSION.
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-STEP
*         BOUND WAS DECREASED. ITERS=2-STEP BOUND WAS UNCHANGED.
*         ITERS=3-STEP BOUND WAS INCREASED. ITERS=6-FIRST STEPSIZE.
*  II  ITERD TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UOS0G1  PRINT OF ENTRY TO STEPSIZE SELECTION.
*  S   UOS0G2  PRINT OF EXIT FROM STEPSIZE SELECTION.
*  S   UOS0G4  PRINT OF INFORMATION DURING STEPSIZE SELECTION.
*  S   UOERR1  ERROR MESSAGES.
*
* METHOD :
*
      SUBROUTINE US0C01(N,G,GO,EPS3,EPS4,BET1,BET2,GAM1,RO,RP,R,FO,F,
     & PO,P,XDEL,XNORM,XMAX,SNORM,RMAX,IPOM3,MOS1,MOS2)
      $INCLUDE('UMCOMM')
      INTEGER N,IPOM3,MOS1,MOS2
      $FLOAT G(N),GO(N)
      $FLOAT EPS3,EPS4,BET1,BET2,GAM1,RO,RP,R,FO,F,PO,P,XDEL,XNORM,
     & XMAX,SNORM,RMAX
      $FLOAT RF,DFPRED
      SAVE RF,DFPRED
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',0)
      GO TO (1,2,3,4) ISP(NS)
    1 CONTINUE
      CALL UOS0G1(RO,FO,XDEL,RMAX)
      IDIR=0
C
C     COMPUTATION OF THE NEW FUNCTION VALUE
C
      R=MIN(ONE,RMAX)
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      IPOM3=1
      DFPRED=FO-XNORM+HALF*PO
      IF (DFPRED.LE.ZERO) THEN
      R=ZERO
      ITERS=-4
      TSXX='BAD PRED'
      CALL UOERR1('US0C01',5-ITERS)
      GO TO 7
      ENDIF
      RF=(FO-F)/DFPRED
      CALL UOS0G4(R,F,RF,XDEL,SNORM,IPOM3)
      IF (RF.GT.ONE-EPS4) THEN
      IF (ITERD.EQ.3) THEN
      ITERS=3
      XDEL=MIN(GAM1*XDEL,XMAX)
      TSXX='SMALL B2'
      IF (RF.GT.ONE-EPS3) THEN
      XDEL=MIN(GAM1*XDEL,XMAX)
      TSXX='SMALL B4'
      ENDIF
      ELSE
      ITERS=2
      TSXX='GOOD  B '
      ENDIF
      GO TO 6
      ELSE IF (RF.GE.EPS4.AND.MOS2.LE.1) THEN
      ITERS=2
      TSXX='GOOD  B '
      GO TO 6
      ENDIF
      CALL UYSET0(2,3)
      RETURN
    3 CONTINUE
      IPOM3=2
      RP=(FO-XNORM+HALF*(P-PO))/DFPRED
      CALL UOS0G4(R,F,RP,XDEL,SNORM,IPOM3)
      IF (RF.GE.EPS4) THEN
      IF (ABS(RP-ONE).LE.EPS3.AND.ITERD.EQ.3) THEN
      ITERS=3
      XDEL=MIN(GAM1*XDEL,XMAX)
      TSXX='SMALL T2'
      ELSE
      ITERS=2
      TSXX='GOOD  T '
      ENDIF
      GO TO 6
      ENDIF
      IF (ABS(RP-ONE).LE.EPS4) THEN
      R=MIN(ONE,RMAX)
      CALL UYSET2
      CALL UYSET0(1,4)
      RETURN
      ELSE
      GO TO 5
      ENDIF
    4 IPOM3=3
      RF=(FO-F)/DFPRED
      CALL UOS0G4(R,F,RF,XDEL,SNORM,IPOM3)
      CALL UXVCOP(N,G,GO)
      IF (RF.GT.ONE-EPS4) THEN
      IF (ITERD.EQ.3) THEN
      ITERS=3
      XDEL=MIN(GAM1*XDEL,XMAX)
      TSXX='SMALL P2'
      IF (RF.GT.ONE-EPS3) THEN
      XDEL=MIN(GAM1*XDEL,XMAX)
      TSXX='SMALL P4'
      ENDIF
      ENDIF
      GO TO 6
      ELSE IF(ABS(RF-HALF).LE.EPS4) THEN
      ITERS=2
      TSXX='GOOD  P '
      GO TO 6
      ENDIF
    5 IF (MOS1.LE.1) THEN
      RP=HALF*P/(P+FO-F)
      ELSE IF (ITERD.NE.3) THEN
      RP=HALF*(ONE-HALF*RF)
      ELSE
      RP=HALF*(ONE-RF)
      ENDIF
      RP=MAX(RP,BET1)
      RP=MIN(RP,BET2)
      XDEL=RP*MIN(XDEL,SNORM)
      ITERS=1
      TSXX='LARGE   '
    6 IF (FO.LE.F) THEN
      IDIR=1
      ITERS=0
      TSXX='ZERO ST '
      ELSE
      MAXST=0
      IF (XDEL.GE.XMAX) MAXST=1
      IF (IPOM3.EQ.2) CALL UYSET2
      ENDIF
    7 CALL UOS0G2(R,F,XDEL)
      CALL UYEPI1(0)
      RETURN
      END
