$INTERFACE US0G08                                             97/12/01

  $HELP
    STRATEGY FOR STEP DECREASING
      1 - FLETCHER
      2 - POWELL
      3 - SAFEGUARDED INTERPOLATION
  $CHECK(MES1,3,'1\2\3')
  $HELP
    STRATEGY FOR STEP INCREASING
      1 - FLETCHER
      2 - POWELL
  $CHECK(MES2,2,'1\2')
  $HELP
    STRATEGY FOR RESTART
      0 - RESTART AFTER UNSUCCESSFUL STEP
      1 - RESTART AFTER LARGE STEP
      2 - RESTART AFTER LARGE BOUNDARY STEP
  $CHECK(MES3,1,'0\1\2')
  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST
    ON THE CHANGE OF THE FUNCTION VALUE)
  $CHECK(EPS1,'1.0$P-4',REAL,4)
  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST
    ON THE CHANGE OF THE CONSTRAINT VIOLATION)
  $CHECK(EPS2,'1.0$P 0-1.0$P-4',REAL,4)
  $HELP
    FIRST TOLERANCE FOR RATIO DF/DFPRED. STEP BOUND IS
    DECREASED IF DF/DFPRED<EPS3
  $CHECK(EPS3,'0.10$P 0',REAL,4)
  $HELP
    SECOND TOLERANCE FOR RATIO DF/DFPRED. STEP BOUND IS
    INCREASED IF IT IS ACTIVE AND DF/DFPRED>EPS4
  $CHECK(EPS4,'0.90$P 0',REAL,4)
  $HELP
    LOWER BOUND FOR STEPSIZE REDUCTION
  $CHECK(BET1,'0.05$P 0',REAL,4)
  $DEFAULT='0.50$P 0'
  $IF(MES1=3) $DEFAULT='0.75$P 0'
  $HELP
    UPPER BOUND FOR STEPSIZE REDUCTION
  $CHECK(BET2,DEFAULT,REAL,4)
  $HELP
    LOWER BOUND FOR STEPSIZE EXPANSION
  $CHECK(GAM1,'2.0$P 0',REAL,4)
  $HELP
    UPPER BOUND FOR STEPSIZE EXPANSION
  $CHECK(GAM2,'1.0$P 6',REAL,4)
  $HELP
    MAXIMUM STEPSIZE
  $CHECK(XMAX,'1.0$P 3',REAL,2)
  $HELP
    MAXIMUM NUMBER OF ZERO STEPS IN ONE ITERATION
  $CHECK(MRED,10,INT,4)
  CALL US0G08($FS,EPS3,EPS4,BET1,BET2,GAM1,GAM2,RO,R,FO,F,PO,PP,XDEL&
,XDELO,XMAX,SNORM,PAR1,RMAX,KTERS,MES1,MES2,MES3,MRED,$KDS)
  $ADD(REAL,'\$FS(2)')
$REM  $ADD(REAL,'\EPS3\EPS4\BET1\BET2\GAM1\GAM2\RO\R\FO\F\PO\PP\XDEL&
$REM  \XDELO\XMAX\SNORM\PAR1\RMAX')
$REM  $ADD(INTEGER,'\KTERS\MES1\MES2\MES3\MRED')
  $ADD(INIT)
    NSXX='US0G08'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US0G08                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SIMPLE SEARCH WITH TRUST REGION UPDATE.
*
* PARAMETERS :
*  RU  FS(2)  MARKOV FILTER SAMPLES.
*  RI  EPS1  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE FUNCTION VALUE).
*  RI  EPS2  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE CONSTRAINT VIOLATION).
*  RI  EPS3  FIRST TOLERANCE FOR RATIO DF/DFPRED. STEP BOUND IS
*         DECREASED IF DF/DFPRED<EPS3.
*  RI  EPS4  SECOND TOLERANCE FOR RATIO DF/DFPRED. STEP BOUND IS
*         INCREASED IF IT IS ACTIVE AND DF/DFPRED>EPS4.
*  RI  BET1  LOWER BOUND FOR STEPSIZE REDUCTION.
*  RI  BET2  UPPER BOUND FOR STEPSIZE REDUCTION.
*  RI  GAM1  LOWER BOUND FOR STEPSIZE EXPANSION.
*  RI  GAM2  UPPER BOUND FOR STEPSIZE EXPANSION.
*  RO  RO  INITIAL VALUE OF THE STEPSIZE PARAMETER.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PP  QUADRATIC PART OF THE PREDICTED FUNCTION VALUE.
*  RU  XDEL  TRUST REGION BOUND.
*  RO  XDELO  PREVIOUS TRUST REGION BOUND.
*  RI  XMAX MAXIMUM STEPSIZE.
*  RI  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  II  KTERS  TERMINATION SELECTION. KTERS=1-NORMAL TERMINATION.
*         KTERS=6-FIRST STEPSIZE.
*  II  MES1  SWITCH FOR EXTRAPOLATION. MES1=1-CONSTANT INCREASING OF
*         THE INTERVAL. MES1=2-EXTRAPOLATION SPECIFIED BY THE PARAMETER
*         MES. MES1=3 SUPPRESSED EXTRAPOLATION.
*  II  MES2  SWITCH FOR TERMINATION. MES2=1-NORMAL TERMINATION.
*         MES2=2-TERMINATION AFTER AT LEAST TWO STEPS (ASYMPTOTICALLY
*         PERFECT LINE SEARCH).
*  II  MES3  SAFEGUARD AGAINST ROUNDING ERRORS. MES3=0-SAFEGUARD
*         SUPPRESSED. MES3=1-FIRST LEVEL OF SAFEGUARD. MES3=2-SECOND
*         LEVEL OF SAFEGUARD.
*  II  MRED  MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*
* COMMON DATA :
*  IO  MAXST  MAXIMUM STEPSIZE INDICATOR. MAXST=0 OR MAXST=1 IF MAXIMUM
*         STEPSIZE WAS NOT OR WAS REACHED.
*  IU  IDIR INDICATOR FOR DIRECTION DETERMINATION.
*         IDIR=0-BASIC DETERMINATION. IDIR=1-DETERMINATION
*         AFTER STEPSIZE REDUCTION. IDIR=2-DETERMINATION AFTER
*         STEPSIZE EXPANSION.
*  IO  NRED  ACTUAL NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-STEP
*         BOUND WAS DECREASED. ITERS=2-STEP BOUND WAS UNCHANGED.
*         ITERS=3-STEP BOUND WAS INCREASED. ITERS=6-FIRST STEPSIZE.
*  II  ITERD TERMINATION INDICATOR. ITERD<0-BAD DECOMPOSITION.
*         ITERD=0-DESCENT DIRECTION. ITERD=1-NEWTON LIKE STEP.
*         ITERD=2-INEXACT NEWTON LIKE STEP. ITERD=3-BOUNDARY STEP.
*         ITERD=4-DIRECTION WITH THE NEGATIVE CURVATURE.
*         ITERD=5-MARQUARDT STEP.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UOS0G1  PRINT OF ENTRY TO STEPSIZE SELECTION.
*  S   UOS0G2  PRINT OF EXIT FROM STEPSIZE SELECTION.
*  S   UOS0G3  PRINT OF INFORMATION DURING STEPSIZE SELECTION.
*  S   UOERR1  ERROR MESSAGES.
*
* METHOD :
* G.A.SCHULTZ, R.B.SCHNABEL, R.H.BYRD: A FAMILY OF TRUST-REGION-BASED
* ALGORITHMS FOR UNCONSTRAINED MINIMIZATION WITH STRONG GLOBAL
* CONVERGENCE PROPERTIES, SIAM J. NUMER.ANAL. 22 (1985) PP. 47-67.
*
      SUBROUTINE US0G08(FS,EPS1,EPS2,EPS3,EPS4,BET1,BET2,GAM1,GAM2,RO,
     & R,FO,F,FC,PO,PP,XDEL,XDELO,XMAX,SNORM,RMAX,KTERS,MES1,MES2,MES3,
     & MRED)
      $INCLUDE('UMCOMM')
      $FLOAT FS(2),EPS1,EPS2,EPS3,EPS4,BET1,BET2,GAM1,GAM2,RO,R,FO,F,
     & FC,PO,PP,XDEL,XDELO,XMAX,SNORM,RMAX
      $FLOAT DF,DFPRED
      INTEGER KTERS,MES1,MES2,MES3,MRED
      INTEGER NRED1,NRED2
      LOGICAL L1,L2
      SAVE NRED1,NRED2
      $FLOAT CON
      PARAMETER(CON=1.0$P-6)
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',0)
      GO TO (1,2) ISP(NS)
    1 CONTINUE
      CALL UOS0G1(RO,FO,XDEL,RMAX)
      IF (IDIR.EQ.0) THEN
      NRED1=0
      NRED2=0
      ENDIF
      IDIR=0
      XDELO=XDEL
C
C     COMPUTATION OF THE NEW FUNCTION VALUE
C
      R=MIN(ONE,RMAX)
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      IF(KTERS.LT.0.OR.KTERS.GT.5) THEN
      ITERS=6
      TSXX='FIRST ST'
      ELSE
      DF=FO-F
      DFPRED=-R*(PO+R*PP)
      L1=F.GT.FS(1)-EPS1*FS(2).AND.FC.GT.EPS2*FS(2)
      L2=DFPRED.GE.CON*FC*FC
      IF (L1.OR.L2.AND.DF.LT.EPS3*DFPRED) THEN
C
C     STEP IS TOO LARGE, IT HAS TO BE REDUCED
C
      IF (MES1.EQ.1) THEN
      XDEL=BET2*SNORM
      ELSE IF (MES1.EQ.2) THEN
      XDEL=BET2*MIN(HALF*XDEL,SNORM)
      ELSE
      XDEL=HALF*PO*SNORM/(PO+DF)
      XDEL=MAX(XDEL,BET1*SNORM)
      XDEL=MIN(XDEL,BET2*SNORM)
      ENDIF
      ITERS=1
      TSXX='LARGE   '
      IF (MES3.LE.1) THEN
      NRED2=NRED2+1
      ELSE
      IF (ITERD.GT.2) NRED2=NRED2+1
      ENDIF
      ELSE IF (DF.LE.EPS4*DFPRED) THEN
C
C     STEP IS SUITABLE
C
      ITERS=2
      TSXX='GOOD    '
      ELSE
C
C     STEP IS TOO SMALL, IT HAS TO BE ENLARGED
C
      IF (MES2.EQ.2) THEN
      XDEL=MAX(XDEL,GAM1*SNORM)
      ELSE IF (ITERD.GT.2) THEN
      XDEL=GAM1*XDEL
      ENDIF
      ITERS=3
      TSXX='SMALL   '
      ENDIF
      XDEL=MIN(XDEL,XMAX,GAM2*SNORM)
      IF (FO.LE.F) THEN
      IF (NRED1.GE.MRED) THEN
      ITERS=-1
      TSXX='MAX  RED'
      CALL UOERR1('US0G08',5-ITERS)
      ELSE
      IDIR=1
      ITERS=0
      TSXX='ZERO  ST'
      NRED1=NRED1+1
      ENDIF
      ENDIF
      ENDIF
      MAXST=0
      IF (XDEL.GE.XMAX) MAXST=1
      CALL UOS0G3(R,F,FO,PO,PP,XDEL,SNORM)
      IF (MES3.EQ.0) THEN
      NRED=NRED1
      ELSE
      NRED=NRED2
      ENDIF
      CALL UOS0G2(R,F,XDEL)
      CALL UYEPI1(0)
      RETURN
      END

