$INTERFACE US0L10                                             05/12/01

  $HELP
    UPPER BOUND FOR FUNCTION VALUE
  $CHECK(FMAX,'1.0$P 20',REAL,4)
  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH (IN THE
    ARMIJO CONDITION)
  $CHECK(EPS1,'1.0$P-4',REAL,4)
  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST
    ON THE CHANGE OF THE CONSTRAINT VIOLATION)
  $CHECK(EPS2,'1.0$P-4',REAL,4)
  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST
    ON THE CHANGE OF THE FUNCTION VALUE)
  $CHECK(EPS3,'1.0$P-4',REAL,4)
  $HELP
    RESTART CRITERION
  $CHECK(ETA3,'1.0$P 3',REAL,4)
  $HELP
    RESRTART CRITERION
  $CHECK(ETA4,'1.0$P 3',REAL,4)
  $HELP
    RESTART CRITERION
  $CHECK(ETA5,'1.0$P 6',REAL,4)
  $HELP
    MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS
  $CHECK(MRED,20,INT,4)
  CALL US0L10(KS,$MS,$FS,EPS1,EPS2,EPS3,PO,RO,R,F,FO,FC,FCO,RMIN&
,RMAX,SNORM,PAR,$ETA3,$ETA4,$ETA5,SWITCH,MODE,INITS,MRED)
$REM  $ADD(REAL,'\EPS1\EPS2\EPS3\PO\RO\R\FF\FC\RMIN\RMAX\SNORM&
$REM  \PAR')
$REM  $ADD(INTEGER,'\MODE\INITS\MRED')
  $ADD(INIT)
    NSXX='US0L10'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US0L10                ALL SYSTEMS                05/12/01
C PORTABILITY : ALL SYSTEMS
C 05/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  LINE SEARCH WITHOUT DIRECTIONAL DERIVATIVES BASED ON A SQP FILTER.
*
* PARAMETERS :
*  IU  KS  NUMBER OF FILTER SAMPLES.
*  II  MS  MAXIMUM NUMBER OF FILTER SAMPLES.
*  RU  FS(MS,2)  ARRAY CONTAINING FILTER SAMPLES.
*  RI  EPS1  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE FUNCTION VALUE).
*  RI  EPS2  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE CONSTRAINT VIOLATION).
*  RO  RO  INITIAL VALUE OF THE STEPSIZE PARAMETER.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  FF  VALUE OF THE OBJECTIVE FUNCTION.
*  RO  FFO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RO  FC  NORM OF THE CONSTRAINT VIOLATION.
*  RO  FCO  OLD NORM OF THE CONSTRAINT VIOLATION.
*  RI  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  SNORM  NORM OF THE DIRECTION VECTOR.
*  RI  FR  RESTART CRITERION.
*  RI  FCR  RESTART CRITERION.
*  RI  SWITCH  SWITCHING CRITERION FOR STEP-LENGTH SELECTION STRATEGY.
*  IO  MODE  MODE OF LINE SEARCH. MODE=0-INITIAL STEPSIZE.
*         MODE=1-EXTRAPOLATION. MODE=2-INTERPOLATION.
*  II  INITS  CHOICE OF THE INITIAL STEPSIZE. INITS=0-INITIAL STEPSIZE
*         IS SPECIFIED IN THE CALLING PROGRAM. INITS=1-UNIT INITIAL
*         STEPSIZE. INITS=2-COMBINED UNIT AND QUADRATICALLY ESTIMATED
*         INITIAL STEPSIZE. INITS=3-QUADRATICALLY ESTIMATED INITIAL
*         STEPSIZE.
*  II  MRED  MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*
* COMMON DATA :
*  IO  MAXST  MAXIMUM STEPSIZE INDICATOR. MAXST=0 OR MAXST=1 IF MAXIMUM
*         STEPSIZE WAS NOT OR WAS REACHED.
*  IO  NRED  ACTUAL NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-PERFECT
*         LINE SEARCH. ITERS=2 GOLDSTEIN STEPSIZE. ITERS=3-WEAK WOLFE
*         STEPSIZE. ITERS=4-STRONG WOLFE STEPSIZE.
*         ITERS=5-ARMIJO STEPSIZE. ITERS=6-FIRST STEPSIZE.
*         ITERS=7-MAXIMUM STEPSIZE. ITERS=8-UNBOUNDED FUNCTION.
*         ITERS=-1-MRED REACHED. ITERS=-2-POSITIVE DIRECTIONAL
*         DERIVATIVE. ITERS=-3-ERROR IN INTERPOLATION.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UOS0F1  PRINT OF ENTRY TO STEPSIZE SELECTION.
*  S   UOS0F2  PRINT OF EXIT FROM STEPSIZE SELECTION.
*  S   UOS0F3  PRINT OF INFORMATION DURING STEPSIZE SELECTION.
*  S   UOERR1  ERROR MESSAGES.
*
* METHOD :
* SAFEGUARDED EXTRAPOLATION AND INTERPOLATION WITH EXTENDED TERMINATION
* CRITERIA.
*
      SUBROUTINE US0L10(KS,MS,FS,EPS1,EPS2,EPS3,PO,RO,R,FF,FFO,FC,FCO,
     & RMIN,RMAX,SNORM,PAR,ETA3,ETA4,SIGMA,SWITCH,MODE,INITS,MRED)
      $INCLUDE ('UMCOMM')
      INTEGER KS,MS,MODE,INITS,MRED
      $FLOAT FS(MS,2),EPS1,EPS2,EPS3,PO,RO,R,FF,FFO,FC,FCO,RMIN,RMAX,
     & SNORM,PAR,ETA3,ETA4,SIGMA,SWITCH
      INTEGER I,J
C      INTEGER NR,NM
      $FLOAT FFP,FCP,RH,FR,FCR,FR1
      SAVE FFP,FCP,RH,FR,FCR
      REAL*8 CON
      PARAMETER(CON=1.2$P 0)
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',0)
      GO TO (1,3) ISP(NS)
    1 CONTINUE
      CALL UOS0F1(RO,FF,FC,SNORM,RMAX)
      IF(RMAX.LE.ZERO) THEN
      TSXX='RMAX=0'
      ITERS=0
      GO TO 10
      ENDIF
      RH=ZERO
      FR=ETA3
      FCR=ETA4
C
C     MINIMUM STEPSIZE SELECTION
C
C      FFO=FF
C      FCO=FC
      FCP=FCO*CON
      RMIN=EPS2
      IF (PO.LT.ZERO) RMIN=MIN(RMIN,HALF*EPS2/(ONE-PAR),
     & -EPS3*MIN(FCO,FCP)/PO)
C
C     INITIAL STEPSIZE SELECTION
C
      TSXX='INITIAL '
      IF (INITS.EQ.0) THEN
      ELSE
      R=ONE
      TSXX='INIT-1  '
      ENDIF
      MAXST=0
      R=MAX(R,RMIN)
      R=MIN(R,RMAX)
      IF (R.GE.RMAX) MAXST=1
      MODE=0
C
C     NEW STEPSIZE SELECTION (EXTRAPOLATION OR INTERPOLATION)
C
    2 IF (MODE.NE.0) THEN
      IF (NRED.GE.MRED) THEN
      ITERS=-1
      TSXX='MAX  INT'
      CALL UOERR1('US0L10',5-ITERS)
      GO TO 10
      ENDIF
      R=HALF*R
      NRED=NRED+1
      ENDIF
      IF (R.LT.RMIN) THEN
      ITERS=-1
      TSXX='FR PHASE'
      CALL UOERR1('US0L10',103)
      GO TO 10
      ENDIF
      IF (INITS.EQ.0.AND.MODE.EQ.0) GO TO 3
C
C     COMPUTATION OF THE NEW FUNCTION VALUE
C
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    3 CONTINUE
      MODE=2
      CALL UOS0F3(R,FF,FC,SNORM,MODE)
      FR1=FF
      IF (FR1.LT.FR-EPS2*FCR.OR.FC.LT.FCR-EPS3*FCR) THEN
C      NR=NR+1
      KS=0
      IF(FR1.LT.FR-EPS2*FCR) THEN
      FR=FR1
      ELSE
      FCR=FC
      ENDIF
      ENDIF
C      KS=0
      DO 4 I=1,KS
      IF (FF.GE.FS(I,1).AND.FC.GE.FS(I,2)) GO TO 2
    4 CONTINUE
      IF (PO.LT.ZERO.AND.-R*PO.GT.EPS3*FCP.AND.FF-FFO.LE.EPS1*R*PO)
     & THEN
      TSXX='ARMIJO  '
      ITERS=5
      GO TO 10
      ELSE
      IF((SWITCH-SIGMA*FC)/FC.GT.ONE/R) THEN
      IF (RH.EQ.ZERO.AND.FC.LT.(ONE-EPS2)*FCO) THEN
      RH=R
      GO TO 2
      ENDIF
      ENDIF
      ENDIF
      IF((SWITCH-SIGMA*FC)/FC.LT.ONE/R.AND.RH.GT.ZERO) THEN
      R=RH
      GO TO 10
      ENDIF
      IF (FC.GT.(ONE-EPS2)*FCO.AND.FF.GT.FFO-EPS3*FCO) GO TO 2
      IF (FF.GT.FFO-EPS3*FCO) THEN
      FCP=FCO-EPS2*FCO
      FFP=FFO-EPS3*FCO
      I=1
    5 IF (I.GT.KS) GO TO 7
      IF (FS(I,1).GE.FFP.AND.FS(I,2).GE.FCP) THEN
      DO 6 J=I,KS-1
      FS(J,1)=FS(J+1,1)
      FS(J,2)=FS(J+1,2)
    6 CONTINUE
      KS=KS-1
      ENDIF
      I=I+1
      GO TO 5
    7 IF (KS.GE.MS) THEN
C      NM=NM+1
      DO 8 J=1,KS-1
      FS(J,1)=FS(J+1,1)
      FS(J,2)=FS(J+1,2)
    8 CONTINUE
      KS=KS-1
      ENDIF
      KS=KS+1
      FS(KS,1)=FFP
      FS(KS,2)=FCP
      ENDIF
      ITERS=4
      TSXX='SUCCESS '
   10 IF (MAXST.EQ.1) NMAX=NMAX+1
      CALL UOS0F2(R,FF,FC,SNORM)
      CALL UYEPI1(0)
      RETURN
      END
