$INTERFACE US0PS1                                             97/12/01

  $HELP
    COEFFICIENT FOR STEPSIZE INCREASE
  $CHECK(ALF1,'1.05$P 0',REAL,4)
  $HELP
    COEFFICIENT FOR STEPSIZE INCREASE
  $CHECK(BET1,'0.25$P 0',REAL,4)
  $HELP
    COEFFICIENT FOR STEPSIZE DECREASE
  $CHECK(GAM1,'2.00$P 0',REAL,4)
  CALL US0PS1(N,$X,$XO,$S,ALF1,BET1,GAM1,FO,F,FF,XDEL,DMAX,MODE)
  $ADD(REAL,'\$X($NF)\$XO($NF)\$S($NF)')
$REM $ADD(REAL,'\ALF1\BET1\GAM1\FO\F\FF\XDEL\DMAX')
$REM $ADD(INTEGER,'\MODE')
  $ADD(INIT)
    NSXX='US0PS1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US0PS1                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  ACCELERATION STEP.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  RU  X(N)  NEW VECTOR OF VARIABLES.
*  RU  XO(N)  OLD VECTOR OF VARIABLES.
*  RU  S(N)   VECTOR OF STEPSIZES.
*  RI  ALF1  COEFFICIENT FOR STEPSIZE EXPANSION.
*  RI  BET1  LOWER BOUND FOR STEPSIZE REDUCTION.
*  RI  GAM1  LOWER BOUND FOR STEPSIZE EXPANSION.
*  RU  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RU  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RU  FF  PRINTED VALUE OF THE OBJECTIVE FUNCTION.
*  RU  XDEL  CURRENT STEPSIZE.
*  RA  DMAX  AUXILIARY VARIABLE.
*  IO  MODE  MODE OF LINE SEARCH.
*
* COMMON DATA :
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP.
*          ITERS=1-ACCELERATION STEP.
*  TO  TDXX  TEXT INFORMATION ON THE TERMINATION.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  RF UXVACC  ACCELERATION STEP.
*  S  UXVCOP  COPYING OF A VECTOR.
*  S  UXVSCL  SCALING OF A VECTOR.
*  S  UYPRO1  PROLOGUE.
*  S  UYEPI1  EPILOGUE.
*  S  UYSET0  STATUS DEFINITION.
*  S  UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S  UOS0A1  PRINT OF ENTRY TO ACCELERATION STEP.
*  S  UOS0A2  PRINT OF EXIT FROM ACCELERATION STEP.
*
* METHOD :
*  R.HOOKE, T.A.JEEVES : DIRECT SEARCH SOLUTION OF NUMERICAL
*  AND STATISTICAL PROBLEMS. J. OF THE ACM 8 (1961), PP 212-229
*
      SUBROUTINE US0PS1(N,X,XO,S,ALF1,BET1,GAM1,FO,F,FF,XDEL,DMAX,MODE)
      $INCLUDE('UMCOMM')
      INTEGER N,MODE
      $FLOAT X(N),XO(N),S(N)
      $FLOAT ALF1,BET1,GAM1,FO,F,FF,XDEL,DMAX
      $FLOAT CON,UXVACC
      PARAMETER (CON=1.0$P 2)
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',0)
      GOTO (1,2) ISP(NS)
    1 CONTINUE
      CALL UOS0A1
      IF (F.GE.FO-(ABS(FO)+ONE)*TOLF*CON) THEN
        CALL UXVCOP(N,XO,X)
        F=FO
        TSXX='ZERO  ST'
        IF (XDEL.LE.TOLX) THEN
          TDXX='STEP LIM'
          ITERM=1
        ELSEIF(ITERS.EQ.0) THEN
          XDEL=BET1*XDEL
          CALL UXVSCL(N,BET1,S,S)
        ENDIF
        DMAX=XDEL
        ITERS=0
        GOTO 3
      ENDIF
      DMAX=UXVACC(N,ALF1,XO,X)
      FO=F
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    2 CONTINUE
      IF(MODE.EQ.1) THEN
        XDEL=GAM1*XDEL
        CALL UXVSCL(N,GAM1,S,S)
      ENDIF
      ITERS=1
      TSXX='ACC STEP'
    3 CALL UOS0A2(FO,FF)
      CALL UYEPI1(0)
      RETURN
      END
