$INTERFACE US0SM2                                             97/12/01

  $HELP
    COEFFICIENT FOR REFLECTION
  $CHECK(ALF1,'0.95$P 0',REAL,4)
  $HELP
    COEFFICIENT FOR EXPANSION
  $CHECK(BET1,'0.50$P 0',REAL,4)
  $HELP
    COEFFICIENT FOR CONTRACTION
  $CHECK(GAM1,'2.00$P 0',REAL,4)
  CALL US0SM2(N,$X,$XO,$XP,$S,$H,ALF1,BET1,GAM1,FO,FP,F,FF,DMAX,EL,EU&
,MODE,IVAR,JVAR)
  $ADD(REAL,'\$X($NF)\$XO($NF)\$XP($NF)\$S($NF+1)\$H($NF*$NF)')
$REM  $ADD(REAL,'\ALF1\BET1\GAM1\FO\FP\F\FF\DMAX\EL\EU')
$REM  $ADD(INTEGER,'\MODE\IVAR\JVAR')
  $ADD(INIT)
    NDXX='US0SM2'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US0SM2                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  SIMPLEX TRANSFORMATION
*
* PARAMETERS :
*  II  N  DECLARED NUMBER OF VARIABLES.
*  RU  X(N)  VECTOR OF VARIABLES.
*  RA  XO(N)  AUXILIARY VECTOR.
*  RA  XP(N)  AUXILIARY VECTOR.
*  RU  S(N)   VECTOR OF SIMPLICIAL VALUES.
*  RU  H(N*(N+1))  MATRIX OF SIMPLICIAL POINTS.
*  RI  ALF1  COEFFICIENT FOR STEPSIZE EXPANSION.
*  RI  BET1  LOWER BOUND FOR STEPSIZE REDUCTION.
*  RI  GAM1  LOWER BOUND FOR STEPSIZE EXPANSION.
*  RU  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RU  FP  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RU  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RU  FF  PRINTED VALUE OF THE OBJECTIVE FUNCTION.
*  RA  DMAX  AUXILIARY VARIABLE.
*  RA  EL  AUXILIARY VARIABLE.
*  RA  EU  AUXILIARY VARIABLE.
*  IA  MODE  MODE OF LOCAL SEARCH. MODE=0-UNSUCCESSFUL STEP.
*         MODE>0-FORWARD STEP. MODE<0-BACKWARD STEP.
*  IA  IVAR  INDEX VARIABLE.
*  IA  JVAR  INDEX VARIABLE.
*
* COMMON DATA :
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAMS USED :
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UXDCMC  CENTER OF COLUMNS OF A DENSE RECTANGULAR MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDDR  VECTOR AUGMENTED BY THE SCALED DIFFERENCE OF
*         TWO VECTORS.
*  S   UXVEXT  EXTREMAL ELEMENTS OF A VECTOR.
*  S   UXVMED  MEDIAN OF TWO VECTORS.
*  RF  UXVNM2  EUCLIDEAN NORM OF A VECTOR DIFFERENCE.
*  S   UOS0A1  PRINT OF ENTRY TO LOCAL SEARCH.
*  S   UOS0A2  PRINT OF EXIT FROM LOCAL SEARCH.
*
* METHOD :
*  NELDER, MEAD : SIMPLEX METHOD
*  AND STATISTICAL PROBLEMS, J. OF THE ACM 8 (1961), PP 212-229
*
      SUBROUTINE US0SM2(N,X,XO,XP,S,H,ALF1,BET1,GAM1,FO,FP,F,FF,DMAX,
     & EL,EU,MODE,IVAR,JVAR)
      $INCLUDE('UMCOMM')
      INTEGER N,MODE,IVAR,JVAR
      $FLOAT X(N),XO(N),XP(N),S(N+1),H(N*(N+1))
      $FLOAT ALF1,BET1,GAM1,FO,FP,F,FF,DMAX,EL,EU
      $FLOAT UXVNM2
      INTEGER I,KL,KU,LL,LU
      SAVE KL,KU,LL,LU
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',0)
      GOTO (1,2,3,6,8) ISP(NS)
    1 CONTINUE
      CALL UOS0A1
      MODE=0
      IF (NIT.LE.LIT+1) THEN
      CALL UXVEXT(N+1,S,EL,EU,KL,KU)
      LL=(KL-1)*N+1
      LU=(KU-1)*N+1
      ENDIF
C
C     CENTROID DETERMINATION
C
      CALL UXDCMC(N,N+1,H,XO,KU)
C
C     REFLECTION
C
      CALL UXVDDR(N,XO,-ALF1,H(LU),XO,X)
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
    2 IF (F.GE.EL) GO TO 4
C
C     EXPANSION
C
      FP=F
      CALL UXVCOP(N,X,XP)
      CALL UXVDDR(N,XO,-GAM1,H(LU),XO,X)
      CALL UYSET2
      CALL UYSET0(1,3)
      RETURN
    3 IF (F.LT.EL) THEN
      MODE=1
      CALL UXVCOP(N,X,H(LU))
      S(KU)=F
      ELSE
      CALL UXVCOP(N,XP,H(LU))
      S(KU)=FP
      ENDIF
      GO TO 10
    4 DO 5 I=1,N+1
      IF (I.EQ.KU) GO TO 5
      IF (F.LE.S(I)) THEN
      CALL UXVCOP(N,X,H(LU))
      S(KU)=F
      GO TO 10
      ENDIF
    5 CONTINUE
C
C     CONTRACTION
C
      IF (F.LE.EU) THEN
      CALL UXVDDR(N,XO,-BET1,H(LU),XO,X)
      ELSE
      CALL UXVDDR(N,XO, BET1,H(LU),XO,X)
      ENDIF
      CALL UYSET2
      CALL UYSET0(1,4)
      RETURN
    6 IF (F.LE.EU) THEN
      CALL UXVCOP(N,X,H(LU))
      S(KU)=F
      GO TO 10
      ENDIF
C
C     REDUCTION
C
      IVAR=1
      JVAR=1
    7 IF (IVAR.GT.N) GO TO 10
      IF (IVAR.EQ.KL) GO TO 9
      CALL UXVMED(N,H(JVAR),H(LL),X)
      CALL UYSET2
      CALL UYSET0(1,5)
      RETURN
    8 CONTINUE
      CALL UXVCOP(N,X,H(JVAR))
      S(IVAR)=F
    9 IVAR=IVAR+1
      JVAR=JVAR+N
      GO TO 7
   10 CALL UXVEXT(N+1,S,EL,EU,KL,KU)
      LL=(KL-1)*N+1
      LU=(KU-1)*N+1
      IF (MODE.EQ.0) THEN
      CALL UXVCOP(N,H(LL),X)
      F=EL
      ENDIF
      DMAX=UXVNM2(N,X,H(LU))
      FO=EU
      CALL UOS0A2(FO,FF)
      CALL UYEPI1(0)
      RETURN
      END
