$INTERFACE US1L05                                             97/12/01

  $HELP
    METHOD OF LINE SEARCH:
      1 - BISECTION
      2 - QUADRATIC INTERPOLATION
          (WITH ONE DIRECTIONAL DERIVATE)
  $CHECK(MES,4,'1\2',3)
  $HELP
    SWITCH FOR EXTRAPOLATION:
      1 - CONSTANT INCREASING OF THE INTERVAL
      2 - EXTRAPOLATION SPECIFIED BY THE PARAMETER MES
      3 - SUPPRESSED EXTRAPOLATION
  $CHECK(MES1,2,'1\2\3',4)
  $HELP
    SWITCH FOR WEIGHT UPDATE:
      1 - USE OF THE CURVATURE
      2 - USE OF THE UNIDIRECTIONAL MINIMUM
  $CHECK(MES2,1,'1\2',4)
  $HELP
    UPPER BOUND FOR FUNCTION VALUE
  $CHECK(FMAX,'1.0$P 20',REAL,4)
  $HELP
    PRECISION OF THE COMPUTED VALUES
  $CHECK(ETA1,'1.0$P 2*ETA0',REAL,4)
  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH
    (IN TEST ON THE CHANGE OF THE FUNCTION VALUE)
  $CHECK(EPS1,'1.0$P-2',REAL,4)
  $HELP
    TERMINATION TOLERANCE FOR LINE SEARCH
    (IN TEST ON THE CHANGE OF THE DIRECTIONAL DERIVATES)
  $CHECK(EPS2,'0.5$P 0',REAL,4)
  $HELP
    MAXIMUM NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS
  $CHECK(MRED,10,INT,4)
  CALL US1L05(EPS1,EPS2,ETA5,RO,RP,R,FO,FP,F,PO,PP,P,FMIN,FMAX,GNORM&
,SNORM,PAR1,PAR2,RMAX,RMIN,TO,MODE,MES,MES1,MES2,INITS)
$REM  $ADD(REAL,'\EPS1\EPS2\ETA5\RO\RP\R\FO\FP\F\PO\PP\P\FMIN\FMAX&
$REM  \GNORM\SNORM\RMAX\RMIN\TO')
$REM  $ADD(INTEGER,'\MODE\MES\MES1\MES2\INITS')
  $ADD(INIT)
    NSXX='US1L05'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE US1L05                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  SPECIAL LINE SEARCH WITH DIRECTIONAL DERIVATIVES FOR BUNDLE METHODS.
*
* PARAMETERS :
*  RI  EPS1  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE FUNCTION VALUE).
*  RI  EPS2  TERMINATION TOLERANCE FOR LINE SEARCH (IN TEST ON THE
*         CHANGE OF THE DIRECTIONAL DERIVATIVE).
*  RI  ETA5  DISTANCE MEASURE PARAMETER.
*  RO  RO  INITIAL VALUE OF THE STEPSIZE PARAMETER.
*  RO  RP  PREVIOUS VALUE OF THE STEPSIZE PARAMETER.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  RO  FP  AUXILIARY VALUE OF THE OBJECTIVE FUNCTION.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  PP  PREVIOUS VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  P  VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  FMIN  LOWER BOUND FOR VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FMAX  UPPER BOUND FOR VALUE OF THE OBJECTIVE FUNCTION.
*  RI  GNORM  NORM OF THE GRADIENT VECTOR.
*  RI  SNORM  NORM OF THE DIRECTION VECTOR.
*  RA  PAR1  AUXILIARY VARIABLE FOR CONTROLLED SCALING OF VARIABLE
*            METRIC UPDATES.
*  RA  PAR2  AUXILIARY VARIABLE FOR CONTROLLED SCALING OF VARIABLE
*            METRIC UPDATES.
*  RI  RMAX  MAXIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  RMIN  MINIMUM VALUE OF THE STEPSIZE PARAMETER.
*  RI  TO    WEIGHT PARAMETER.
*  IO  MODE  MODE OF LINE SEARCH. MODE=0-INITIAL STEPSIZE.
*         MODE=1-EXTRAPOLATION. MODE=2-INTERPOLATION.
*  II  MES  METHOD SELECTION. MES=1-BISECTION. MES=2-QUADRATIC
*         INTERPOLATION (WITH ONE DIRECTIONAL DERIVATIVE).
*  II  MES1  SWITCH FOR EXTRAPOLATION. MES1=1-CONSTANT INCREASING OF
*         THE INTERVAL. MES1=2-EXTRAPOLATION SPECIFIED BY THE PARAMETER
*         MES. MES1=3 SUPPRESSED EXTRAPOLATION.
*  II  MES2  SWITCH FOR TERMINATION. MES2=1-NORMAL TERMINATION.
*         MES2=2-TERMINATION AFTER AT LEAST TWO STEPS (ASYMPTOTICALLY
*         PERFECT LINE SEARCH).
*  II  INITS  CHOICE OF THE INITIAL STEPSIZE. INITS=0-INITIAL STEPSIZE
*         IS SPECIFIED IN THE CALLING PROGRAM. INITS=1-UNIT INITIAL
*         STEPSIZE. INITS=2-COMBINED UNIT AND QUADRATICALLY ESTIMATED
*         INITIAL STEPSIZE. INITS=3-QUADRATICALLY ESTIMATED INITIAL
*         STEPSIZE.
*
* COMMON DATA :
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  IO  NRED  ACTUAL NUMBER OF EXTRAPOLATIONS OR INTERPOLATIONS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-PERFECT
*         LINE SEARCH. ITERS=2 GOLDSTEIN STEPSIZE. ITERS=3-WEAK WOLFE
*         STEPSIZE. ITERS=4-STRONG WOLFE STEPSIZE.
*         ITERS=5-ARMIJO STEPSIZE. ITERS=6-FIRST STEPSIZE.
*         ITERS=7-MAXIMUM STEPSIZE. ITERS=8-UNBOUNDED FUNCTION.
*         ITERS=9-SHORT STEP. ITERS=10-ZERO STEP.
*         ITERS=-1-MRED REACHED. ITERS=-2-POSITIVE DIRECTIONAL
*         DERIVATIVE. ITERS=-3-ERROR IN INTERPOLATION.
*  II  IEST  LOWER BOUND SPECIFICATION. IEST=0 OR IEST=1 IF LOWER BOUND
*         IS NOT OR IS GIVEN.
*  TO  TSXX  TEXT INFORMATION ON THE STEPSIZE OBTAINED.
*
* STATUS VARIABLES :
*  NS,ISP,TSS
*
* SUBPROGRAM USED :
*  S   UNINT1  EXTRAPOLATION OR INTERPOLATION WITH DIRECTIONAL
*         DERIVATIVES.
*  S   UYPRO1  PROLOGUE.
*  S   UYEPI1  EPILOGUE.
*  S   UYSET0  STATUS DEFINITION.
*  S   UYSET2  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UOS1L1  PRINT OF ENTRY TO STEPSIZE SELECTION.
*  S   UOS0L2  PRINT OF EXIT FROM STEPSIZE SELECTION.
*  S   UOS1L3  PRINT OF INFORMATION DURING STEPSIZE SELECTION.
*
* METHOD :
* SAFEGUARDED EXTRAPOLATION AND INTERPOLATION WITH SPECIAL TERMINATION
* CRITERIA.
*
      SUBROUTINE US1L05(EPS1,EPS2,ETA5,RO,RP,R,FO,FP,F,PO,PP,P,FMIN,
     & FMAX,GNORM,SNORM,PAR1,PAR2,RMAX,RMIN,TO,MODE,MES,MES1,MES2,
     & INITS)
      $INCLUDE ('UMCOMM')
      INTEGER MODE,MES,MES1,MES2,INITS
      $FLOAT EPS1,EPS2,ETA5,RO,RP,R,FO,FP,F,PO,PP,P,FMIN,FMAX,GNORM
      $FLOAT SNORM,PAR1,PAR2,RMAX,RMIN,TO
      $FLOAT RL,FL,PL,RU,FU,PU,RTEMP,TOP,TOP1,TOP2,TOPB,TOPC,TOAUX,FAUX
      $FLOAT CON1
      INTEGER MTYP,MERR,INIT1,IPOC,IPOCN,IPOCM
      LOGICAL L3,L5,M0,M3,M4
      PARAMETER (CON1=2$P-3)
      SAVE M0,M4
      SAVE MTYP,IPOC,IPOCN,IPOCM
      SAVE RL,FL,PL,RU,FU,PU,TOP,TOP1,TOP2,TOAUX,FAUX
      IF (TSS(NS).NE.'USXX') CALL UYPRO1('USXX',1)
      GOTO (1,3) ISP(NS)
   1  CONTINUE
      IF(NIT-LIT.LE.1) THEN
        IPOC=0
        IPOCN=0
        TOAUX=ONE
        M0=.FALSE.
      ENDIF
      IF(NIT.EQ.NIT/5*5+1)THEN
        FAUX=ZERO
        IPOCM=IPOCN
      ENDIF
      CALL UOS1L1(RO,FO,PO,GNORM,SNORM,RMAX)
      M4=RP.EQ.R
      IF(.NOT.M4)PP=P
      IF(RMAX.LE.ZERO) THEN
        TSXX='RMAX = 0'
        ITERS=0
        GOTO 4
      ENDIF
C
C     INITIAL STEPSIZE SELECTION
C
      IF (INITS.GT.0) THEN
      RTEMP=FMIN-F
      ELSE IF (IEST.EQ.0) THEN
      RTEMP=F - FP
      ELSE
      RTEMP=MAX(F - FP,TEN*(FMIN-F))
      ENDIF
      INIT1=ABS(INITS)
      RP=ZERO
      FP=FO
      TSXX='INITIAL '
      IF (INIT1.EQ.0) THEN
      ELSE IF (INIT1.EQ.1.OR.INITS.GE.1.AND.IEST.EQ.0) THEN
      R=ONE
      TSXX='INIT-1  '
      ELSE IF (INIT1.EQ.2) THEN
      R=MIN(ONE,FOUR*RTEMP/PO)
      TSXX='INIT-2  '
      ELSE IF (INIT1.EQ.3) THEN
      R=MIN(ONE, TWO*RTEMP/PO)
      TSXX='INIT-3  '
      ELSE IF (INIT1.EQ.4) THEN
      R=TWO*RTEMP/PO
      TSXX='INIT-4  '
      ENDIF
      R=MIN(MAX(R,RMIN),RMAX)
      MODE=0
      RL=ZERO
      FL=FO
      PL=PO
      RU=ZERO
      FU=FO
      PU=PO
C
C     NEW STEPSIZE SELECTION (EXTRAPOLATION OR INTERPOLATION)
C
   2  IF(RL.EQ.ZERO) THEN
        IF (MODE.NE.0) MTYP=MIN(MTYP,2)
        PL=PP
        CALL UNINT1(RL,RU,FL,FU,PL,PU,R,MODE,MTYP,MERR,TSXX)
      ELSE
        MERR=0
        R=HALF*(RU+RL)
      ENDIF
      IF (MERR.GT.0) THEN
        ITERS=-MERR
        CALL UOERR1('US1L05',5-ITERS)
        GOTO 4
      ELSEIF (MODE.EQ.1) THEN
        NRED=NRED-1
        R=MIN(R,RMAX)
      ELSEIF (MODE.EQ.2) THEN
        NRED=NRED+1
      ENDIF
C
C     NEW FUNCTION VALUE AND NEW DIRECTIONAL DERIVATIVE
C
      CALL UYSET2
      CALL UYSET0(1,2)
      RETURN
   3  CONTINUE
      CALL UOS1L3(R,F,P,SNORM,MODE)
      IF (MODE.EQ.0) THEN
        PAR1=P/PO
        PAR2=F-FO
      ENDIF
      IF (F.LE.FMIN) THEN
        ITERS=7
        TSXX='F BOUND '
        GOTO 4
      ENDIF
      L3=(F-FO.LE.EPS1*R*PO).OR.(F.LE.FO-ONE)
      MODE=MAX(MODE,1)
      IF (MODE.EQ.1) THEN
C
C     INTERVAL CHANGE AFTER EXTRAPOLATION
C
        TOP2=TWO*(P*R+FO-F)/(R*SNORM)**2
        TOP=TOP2
        TOP1=-ONE
        IF(M4)TOP1=TWO*(F-FO-PP*R)/(R*SNORM)**2
        IF(TOP.LT.-CON1) THEN
          TOP=TOP1
        ELSE
          IF(TOP1.GE.-CON1)TOP=MIN(TOP1,TOP)
        ENDIF
        IF(MES2.EQ.2.AND.PP.LT.ZERO) THEN
          IF(P.GE.ZERO)TOP=MIN(TOP,TO/R*(ONE-P/PP))
          IF(P.LT.ZERO)TOP=MIN(TOP,TO/(ONE+TEN*P/PP))
        ENDIF
        M0=M4.OR.IPOC.GE.6.OR.L3
        RL=RU
        FL=FU
        PL=PU
        RU=R
        FU=F
        PU=P
        IF(.NOT.L3) THEN
          NRED=0
          MODE=2
        ELSEIF ( MES1.EQ.1) THEN
          MTYP=1
        ENDIF
      ELSE
C
C     INTERVAL CHANGE AFTER INTERPOLATION
C
        TOPB=TWO*(P*R+FO-F)/(R*SNORM)**2
        TOPC=TWO*(FU-F-P*(RU-R))/((RU-R)*SNORM)**2
        IF(TOPB.LT.-CON1) THEN
          TOP=TOPC
        ELSE
          IF(TOPC.GE.-CON1)TOP=MIN(TOPB,TOPC)
          IF(TOPC.LT.-CON1)TOP=TOPB
        ENDIF
        IF(L3) THEN
          RL=R
          FL=F
          PL=P
        ELSE
          RU=R
          FU=F
          PU=P
        ENDIF
      ENDIF
C
C     TEST ON TERMINATION
C
      IF(F.LT.FP) THEN
        RP=R
        FP=F
      ENDIF
      RTEMP=MAX(ABS(FP-F+P*(R-RP)),ETA5*((R-RP)*SNORM)**2)
      M3=P-RTEMP.GE. EPS2*PO
      L5=RL.EQ.ZERO
      MTYP=1
      IF(L5) MTYP=MES
      IF(F.GE.FMAX) MTYP=1
      IF(MODE.EQ.1.AND.L3) L5=.FALSE.
      IF(L3.AND.RP.GT.RMIN) GOTO41
      IF(M3.AND.(R.LT.1$P 0.OR.(IPOC.LE.6.AND.NIT.GT.1))) GOTO40
      IF(M3.AND..NOT.M0) GOTO40
      IF(RU-RL.GE.RMIN) GOTO 2
 40   IF(RP.EQ.ZERO) THEN
        TSXX='ZERO  ST'
        ITERS=10
        IPOC=IPOC+1
      ELSE
        TSXX='SHORT ST'
        ITERS=9
        IPOC=0
      ENDIF
      GOTO4
 41   ITERS=5
      TSXX='SERIOUS '
      IPOC=0
      IF(MODE.EQ.1)IPOCN=IPOCN+1
      R=RP
      F=FP
  4   IF (R.GE.RMAX) NMAX=NMAX+1
      CALL UOS0L2(R,F,SNORM)
      CALL UYEPI1(1)
C
C     WEIGHT UPDATING
C
      FAUX=MAX(FAUX,F-FP)
      IF(NIT.EQ.NIT/5*5.AND.MES2.EQ.2) THEN
        IF(IPOCN-IPOCM.GE.4) THEN
          TOAUX=MAX(TOAUX/1.75$P 0,CON1)
        ELSEIF(IPOCN.EQ.IPOCM.AND.FAUX.GE.-R*PO.AND.NIT.GT.10) THEN
          TOAUX=MIN(TOAUX*1.25$P 0,ONE/CON1)
        ENDIF
      ENDIF
      TOP=TOP*TOAUX
      RU=(TOP+CON1*1$P-1)*(TOP*CON1*1$P-2-ONE)
      IF(M0.AND.RU.LE.ZERO.AND.MES2.EQ.1)
     & TO=MIN(MAX(TOP,CON1    ,TO/TEN),ONE/CON1,TO*TEN)
      IF(M0.AND.RU.LE.ZERO.AND.MES2.EQ.2)
     & TO=MIN(MAX(TOP,CON1/TEN,TO/TEN),ONE/CON1,TO*TEN)
      RETURN
      END

