$INTERFACE UUBFM1                                             97/12/01

  $HELP
    SELECTION OF SELF SCALING:
      1 - SELF SCALING SUPPRESSED
      2 - SELF SCALING IN THE FIRST ITERATION AFTER RESTART
      3 - SELF SCALING IN EACH ITERATION
  $CHECK(MET1,3,'1\2\3\4',4)
  $HELP
    TOLERANCE FOR POSITIVE DEFINITENESS
  $CHECK(ETA2,ETA0,REAL,4)
  $IF(TYPE='L') $DEFAULT='2.0D-2'
  $HELP
    TOLERANCE FOR APPLICATION OF THE BFGS METHOD
  $CHECK(ETA5,DEFAULT,REAL,4)
  $ADD(REAL,'\$AB($MAH)\$S($NF)\$XO($NF)\$AGO($MA)')
$REM  $ADD(REAL,'\F\FO\ETA5')
  $ADD(INTEGER,'\$IA($NA)\$IAG($NA+1)\$JAG($MA)')
$REM  $ADD(INTEGER,'\NF\NA\NAL\MA\MAH\IPOM1\IPOM2')
  CALL UUBFM1(NF,NA,NAL,MA,MAH,$IA,$AB,$IAG,$JAG,$S,$XO,$AGO,F,FO&
,ETA5,$ETA9,IPOM1,IPOM2,$MET,$MET1)
  $DEFAULT='5.0D-4'
  $ADD(INIT)
    NUXX='UUBFM1'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUBFM1                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* VARIABLE METRIC UPDATE OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX
* USING THE FACTORIZATION H=L*D*TRANS(L).
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF BLOCKS OF THE MATRIX H.
*  II  NAL  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE MATRIX H.
*  II  IA(NA)  TYPES OF APPROXIMATED FUNCTIONS.
*  RU  AB(MB)  FACTORIZATION B=L*D*TRANS(L) OF A POSITIVE
*         DEFINITE APPROXIMATION OF THE HESSIAN MATRIX.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RU  XO(NF)  VECTORS OF VARIABLES DIFFERENCE.
*  RI  AGO(MA)  GRADIENTS DIFFERENCE.
*  RI  F  CURRENT VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  PREVIOUS VALUE OF THE OBJECTIVE FUNCTION.
*  RI  ETA5  TOLERANCE FOR A HYBRID METHOD.
*  RI  ETA9  MAXIMUM MACHINE NUMBER.
*  II  IPOM1  METHOD INDICATOR.
*  IO  IPOM2  INDICATOR FOR SCALING.
*  II  MET  METHOD SELECTION. MET=0-NO UPDATE. MET=1-BFGS UPDATE.
*  II  MET1  SELECTION OF SELF SCALING.  MET1=1-SELF SCALING SUPPRESSED.
*         MET1=2 SELF SCALING IN THE FIRST ITERATION AFTER RESTART.
*         MET1=3-SELF SCALING IN EACH ITERATION.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXBSBM  MULTIPLICATION OF A PARTITIONED MATRIX BY A VECTOR.
*  S   UXBSBU  UPDATE OF A PARTITIONED MATRIX.
*  S   UXDSMS  SCALING OF A DENSE SYMMETRIC MATRIX.
*  RF  UXWDOT  DOT PRODUCT OF TWO SPARSE VECTORS.
*  S   UOERR1  ERROR MESAGES.
*
* METHOD :
*  FLETCHER'S COMBINATION OF THE GAUSS-NEWTON AND THE BFGS METHODS.
*
      SUBROUTINE UUBFM1(N,NA,NAL,MA,MB,IA,AB,IAG,JAG,S,XO,AGO,F,FO,
     & ETA5,ETA9,IPOM1,IPOM2,MET,MET1)
      $INCLUDE('UMCOMM')
      INTEGER N,NA,NAL,MA,MB,IA(NA),IAG(NA+1),JAG(MA),IPOM1,IPOM2,
     & MET,MET1
      $FLOAT AB(MB),S(N),XO(N),AGO(MA),F,FO,ETA5,ETA9
      $FLOAT UXWDOT
      $FLOAT AH,BH,CH,GAM
      INTEGER K,L,KA,NB
      LOGICAL L1
      CHARACTER*8 TXXX
      IF (MET.LE.0) RETURN
      IF(IDECF.NE.0) THEN
      ITERH=-1
      TUXX='BAD DEC0'
      CALL UOERR1('UUBFM1',1)
      RETURN
      ENDIF
      IF (FO-F.GE.ETA5*FO) THEN
      IPOM1=0
      ELSE
      IPOM1=1
      ENDIF
      L1=MET1.EQ.4.OR.MET1.EQ.3.AND.IPOM2.GE.1.OR.MET1.EQ.2
     & .AND.IPOM2.EQ.1
      TXXX='        '
      TUXX='        '
      NB=0
      DO 6 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      IF (KA.LE.NAL) GO TO 5
      IA(KA)=-ABS(IA(KA))
C
C     DETERMINATION OF THE PARAMETERS B, C
C
      BH=UXWDOT(L,N,JAG(K),AGO(K),XO,2)
      IF (BH.LE.ONE/ETA9) GO TO 5
      AH=ZERO
      CALL UXBSBM(L,N,AB(NB+1),JAG(K),XO,S,1)
      CH=UXWDOT(L,N,JAG(K),XO,S,1)
      IF (CH.LE.ONE/ETA9) GO TO 5
      IF (L1) THEN
C
C     DETERMINATION OF THE PARAMETER GAM (SELF SCALING)
C
      GAM=CH/BH
      ELSE
      GAM=ONE
      ENDIF
C
C     BFGS UPDATE
C
      CALL UXBSBU(L,N,AB(NB+1),JAG(K),GAM/BH,AGO(K),2)
      CALL UXBSBU(L,N,AB(NB+1),JAG(K),-ONE/CH,S,1)
      IF (TUXX.EQ.TXXX) TUXX = 'BFGS    '
      NUPD=NUPD+1
      ITERH=0
      IPOM2=0
      IA(KA)= ABS(IA(KA))
      IF (GAM.EQ.ONE) GO TO 5
      CALL UXDSMS(L,AB(NB+1),ONE/GAM)
      NSCL=NSCL+1
    5 NB=NB+L*(L+1)/2
    6 CONTINUE
      IF (TUXX.EQ.TXXX) TUXX='NONE UP.'
      RETURN
      END
