$INTERFACE UUBVI2                                             04/12/01

  $HELP
    METHOD CORRECTION:
      1 - NO CORRECTION - ANGLE TEST SUPPRESSED
      2 - NO CORRECTION - ANGLE TEST USED
      3 - POWELL'S CORRECTION
  $CHECK(MET3,1,'1\2\3',4)
  CALL UUBVI2(NF,NA,NAL,MA,MAH,$AH,$IAG,$JAG,$AG,$AGO,$XO,$S,$GS&
,ETA0,ETA9,MET3,NNK)
  $ADD(INTEGER,'\$IAG($NA)\$JAG($MA)')
  $ADD(REAL,'\$AH($MAH)\$AG($MA)\$AGO($MA)\$XO($NF)\$S($NF)\$GS($NF)')
  $ADD(INTEGER,'\NNK')
$REM  $ADD(INTEGER,'\NF\NA\NAL\MA\MAH')
$REM  $ADD(REAL,'\ETA0\ETA9')
  $ADD(INIT)
    NUXX='UUBVI2'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUBVI2                ALL SYSTEMS                04/12/01
C PORTABILITY : ALL SYSTEMS
C 04/12/01 VL : ORIGINAL VERSION
*
* PURPOSE :
*  NONSMOOTH VARIABLE METRIC UPDATE OF THE INVERSE HESSIAN MATRIX.
*
* PARAMETERS :
*  II  NF  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  NAL  NUMBER OF LINEAR APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF ELEMENTS IN THE FIELD AG.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE PARTITIONED HESSIAN MATRIX.
*  RU  AH(MB)  NUMERICAL VALUES OF ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX.
*  II  IAG(NA+1)  POINTERS OF THE JACOBIAN MATRIX.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RI  AG(NF)  NEW GENERALIZED JACOBIAN MATRIX.
*  RI  AGO(NF)  OLD GENERALIZED JACOBIAN MATRIX.
*  RI  XO(N)  VECTOR OF VARIABLES DIFERENCE.
*  RO  S(NF)  AUXILIARY VECTOR.
*  RO  U(NF)  AUXILIARY VECTOR.
*  II  MET3  METHOD CORRECTION. MET3=1-NO CORRECTION. MET3=2-POWELL'S
*         CORRECTION.
*  II  NNK  CONSECUTIVE NULL STEPS COUNTER.
*
* COMMON DATA :
*
* SUBPROGRAMS USED :
*  S   UXBSBM  MULTIPLICATION OF A DENSE SYMMETRIC MATRIX BY A VECTOR.
*  S   UXBSBU  UPDATE OF A PARTITIONED SYMMETRIC MATRIX.
*  S   UXDSMS  SCALING OF A DENSE SYMMETRIC MATRIX.
*  S   UXWDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  RF  UXWDOT  DOT PRODUCT OF VECTORS.
*
      SUBROUTINE UUBVI2(NF,NA,NAL,MA,MB,AH,IAG,JAG,AG,AGO,XO,S,U,
     & ETA0,ETA9,MET3,NNK)
      $INCLUDE('UMCOMM')
      INTEGER NF,NA,NAL,MA,MB,IAG(NA+1),JAG(MA),MET3,NNK
      $FLOAT AH(MB),AG(MA),AGO(MA),XO(NF),S(NF),U(NF),ETA0,ETA9
      $FLOAT GAM,A,B,C,Q,DEN,POM,UXWDOT
      INTEGER KA,K,L,NB,ICOR
      LOGICAL LB,LR
      $FLOAT CON0,CON1,CON2
      PARAMETER (CON0=1.0$P-1,CON1=1.0$P-6,CON2=1.0$P-8)
      CHARACTER*8 TXXX
      TXXX='        '
      TUXX='        '
      NB=0
      ICOR=0
      DO 21 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      IF (KA.LE.NAL) GO TO 20
      CALL UXVDIF(L,AG(K),AGO(K),U)
*
*     DETERMINATION OF THE PARAMETERS B, C
*
      B=UXWDOT(L,NF,JAG(K),U,XO,2)
      IF (ABS(B).LE.ONE/ETA9) GO TO 20
      A=ZERO
      CALL UXBSBM(L,NF,AH(NB+1),JAG(K),XO,S,2)
      C=UXWDOT(L,NF,JAG(K),S,XO,2)
      IF (ABS(C).LE.ONE/ETA9) GO TO 20
      GAM=ONE
      IF(NIT.EQ.1) THEN
        Q=ONE
        IF (C.NE.ZERO) Q=C/B
        IF ((Q-2.5$P-1)*(Q-THREE).GT.ZERO) GAM=MIN(THREE,MAX(2.0$P-2,Q))
      ENDIF
      LB=NNK.EQ.0
      LR=NNK.NE.0.AND.C.LT.GAM*B
      IF (NIT.EQ.1) LR=LR.AND.GAM.NE.Q
      IF(LB)THEN
        IF (MET3.GE.3) THEN
*
*     POWELL'S CORRECTION
*
          IF (B.LT.CON1*ABS(C)) THEN
            Q=(ONE-CON0)*C/(C-B)
            CALL UXVDIF(L,U,S,U)
            CALL UXVDIR(L,Q,U,S,U)
            B=C+Q*(B-C)
          ENDIF
        ELSE IF (MET3.EQ.2) THEN
          IF (B.LT.CON2*ABS(C)) GO TO 20
        ELSE
          IF (B.LT.ZERO) GO TO 20
        ENDIF
*
*     BFGS UPDATE
*
        POM=ZERO
        CALL UXBSBU(L,NF,AH(NB+1),JAG(K),GAM/B,U,2)
        CALL UXBSBU(L,NF,AH(NB+1),JAG(K),-ONE/C,S,2)
        IF (TUXX.EQ.TXXX) TUXX='BFGS    '
        NUPD=NUPD+1
        ITERH=0
        IF (GAM.NE.ONE) THEN
          CALL UXDSMS(L,AH(NB+1),ONE/GAM)
          NSCL=NSCL+1
        ENDIF
      ELSEIF(LR)THEN
        DEN=GAM*B-C
        POM=GAM*B/DEN
        CALL UXVDIR(L,-GAM,U,S,U)
        CALL UXBSBU(L,NF,AH(NB+1),JAG(K), ONE/DEN,U,2)
        IF (TUXX.EQ.TXXX) TUXX='RANK ONE'
        NUPD=NUPD+1
      ENDIF
   20 CONTINUE
      NB=NB+L*(L+1)/2
   21 CONTINUE
      IF (TUXX.EQ.TXXX) TUXX='NONE UP.'
      RETURN
      END
