$INTERFACE UUDGA1                                             97/12/01
$REM LU

  $POMD='D'
  $POMH=H
  $POMN='N'
  $HELP
    METHOD OF UPDATE OF HESSIAN MATRIX:
      1 - BFGS UPDATE
      2 - DFP UPDATE
      3 - RANK ONE UPDATE
      4 - RANK ONE UPDATE WITH RESIDUAL CORRECTION 
  $CHECK(MET,1,'1\2\3\4',3)
  $HELP
    TOLERANCE FOR POSITIVE DEFINITENESS
  $CHECK(ETA2,'ETA0*1.0D-3',REAL,4)
$REM  $N='N'
  $ADD(REAL,'\$AG($NF*$NA)\$AF($NA)\$S($NF)\$XO($NF)\$AFO($NA)&
  \$AFD($NA)')
  $IF(DEC='A')
    $POM1='$N,NA'
    $POM2='N,NA'
  $ELSEIF(DEC='E')
    $POM1='$N,NA,ETA2,IAG'
    $POM2='N,NA,ETA2,IA'
    $ADD(INTEGER,'\$IAG($NF)')
    $HELP
      RANK ONE UPDATE OF THE LU DECOMPOSITION:
        1 - BENNETT'S METHOD WITHOUT PERMUTATIONS 
        2 - SCHWETLICK'S METHOD WITH PERMUTATIONS
        3 - COMBINATION OF THE PREVIOUS METHODS
    $CHECK(MOS3,1,'1\2\3',3)
    $IF(LAPACK='NO')
      $ADD(POM1,',$GO')
      $ADD(POM2,',GO')
      $HELP
        STORING PERMUTATIONS IN THE LU DECOMPOSITION:
          1 - CHANGES OF MATRIX ROWS 
          2 - TWO LEVEL INDICES
      $CHECK(MOS2,1,'1\2',3)
    $ELSEIF(MOS3>1)
      $ADD(POM1,',$GO')
      $ADD(POM2,',GO')
      $MOS2=1
    $ELSE
      $MOS2=1    
    $ENDIF
    $IF(MOS2=1)
      $IF(MOS3>1)
        $IBG='IBG'
        $ADD(POM1,',$IBG')
        $ADD(POM2,',IB')
        $ADD(INTEGER,'\$IBG($NF)')
      $ENDIF
    $ENDIF
  $ELSEIF(DEC='Q')
    $IF(MOS2=1)
      $POMD='E'
    $ELSE
      $POMD='D' 
    $ENDIF  
    $IF(MOS3=1)
      $POM1='$N,NA,$H,$XS,ETA2'
      $POM2='N,NA,H,XS,ETA2'
      $ADD(REAL,'\$H($NA*$NF)\$XS($NF)')
    $ELSE
      $POM1='$N,NA,$H,ETA2'
      $POM2='N,NA,H,ETA2'
      $ADD(REAL,'\$H($NF*($NF+1)/2)')
    $ENDIF
  $ENDIF
  $ADD(POM1,',$AG,$AF,$S,$XO,$AFO,$AFD,$MET')
  $ADD(POM2,',AG,AF,S,XO,AFO,AFD,MET')
  $NUMSUB=''
  $IF(NMET>1) $NUMSUB='_$NMET'
  CALL UUDG$(DEC)1$(NUMSUB)($POM1)
  $ADD(SUBROUTINES)
*     ----------------------------------
*     FACTORIZED VARIABLE METRIC UPDATES
*     TEMPLATE : UUDGA1
*     ----------------------------------
  SUBROUTINE UUDG$(DEC)1$(NUMSUB)($POM2)
  $$INCLUDE('UMCOMM')
  INTEGER N,NA,MET
  $$FLOAT AG(N*NA),AF(NA),S(N),XO(N),AFO(NA),AFD(NA)
  $$IF(DEC='Q')
    INTEGER INF
    $$IF(MOS2=1)
      $$FLOAT H(NA,N),XS(N),ETA2
    $$ELSEIF(MOS3=1)  
      $$FLOAT H(NA*N),XS(N),ETA2
    $$ELSE
      $$FLOAT H(N*(N+1)/2),ETA2
    $$ENDIF
  $$ENDIF
  $$FLOAT UXVDOT
  $$FLOAT A,B,C,POM
  IF (MET.LE.0) GO TO 22
*  CALL UOU1D1(N,XO,GO)
  $$IF(DEC='A')
    $$I=0
    IF (IDECA.NE.0) THEN
  $$ELSEIF(DEC='E')
    $$I=2
    IF (IDECA.EQ.0) THEN
*
*     LU DECOMPOSITION
*
    DEN=ETA2
    $$IF(LAPACK='YES')
      CALL UXDGLF(N,AG,IA,INF,DEN)
    $$ELSEIF(MOS2=1)
      CALL UXEGGF(N,N,AG,IA,INF,DEN)
    $$ELSE
      CALL UXDGGF(N,AG,IA,INF,DEN)
    $$ENDIF
    NDECA=NDECA+1
    IDECA=2
    ELSE IF (IDECA.NE.2) THEN
  $$ELSEIF(DEC='Q')
    $$I=1
    IF (IDECA.EQ.0) THEN
*
*     QR DECOMPOSITION
*
    DEN=ETA2
    $$IF(MOS3=1)
      CALL UX$(POMD)$(JASA)QF(N,NA,AG,H,XS)
      CALL UX$(POMD)CRC(N,NA,AG,INF,DEN)    
    $$ELSE
      CALL UXD$(JASA)PF(N,NA,AG,H)
      CALL UXDPRC(N,H,INF,DEN)
    $$ENDIF
    NDECA=NDECA+1
    IDECA=1
    ELSE IF (IDECA.NE.1) THEN
  $$ENDIF
  ITERH=-1
  TUXX='BAD DEC$$I'
  CALL UOERR1('UUDG$(DEC)1',1)
  GO TO 22
  ENDIF
  IF (ITERD.NE.1) THEN
  $$IF(DEC='A')
    CALL UXD$(JASA)MM($POMA,AG,XO,AFD)
  $$ELSEIF(DEC='E')
    CALL UXVCOP(N,XO,AFD)
    JOB=0
    $$IF(LAPACK='YES')
      $$IF (JASA='R') JOB=1
      CALL UXDGLM(N,AG,IA,AFD,JOB)
    $$ELSEIF(MOS2=1)
      $$IF (JASA='R') JOB=1
      CALL UXEGGM(N,N,AG,IA,AFD,JOB)
    $$ELSE
      $$IF (JASA='C') JOB=1
      CALL UXDGGM(N,AG,IA,AFD,S,JOB)
    $$ENDIF
  $$ELSEIF(DEC='Q')
    CALL UXVCOP(N,XO,S)
    $$IF(MOS3=1)
      CALL UX$(POMD)CRM(N,NA,AG,S,1)
      CALL UXD$(JASA)MM($POMA,H,S,AFD)
      CALL UX$(POMD)CQM(N,NA,AG,AFD,XS)
    $$ELSE
      CALL UXDPRM(N,H,S,1)
      CALL UXD$(JASA)MM($POMA,AG,S,AFD)
    $$ENDIF
  $$ENDIF
  ELSE
  CALL UXVDIF(N,AFO,AF,AFD)
  CALL UXVSCL(N,R,AFD,AFD)
  ENDIF
  B=UXVDOT(NA,AFO,AFD)
  IF (B.LE.ZERO) THEN
  ITERH=2
  TUXX='B - NEG'
  GO TO 22
  ENDIF
  A=UXVDOT(NA,AFO,AFO)
  IF (A.LE.ZERO) THEN
  ITERH=2
  TUXX='A - NEG'
  GO TO 22
  ENDIF
  C=UXVDOT(NA,AFD,AFD)
  IF (C.LE.ZERO) THEN
  ITERH=2
  TUXX='C - NEG'
  GO TO 22
  ENDIF
  $$U17=0
  $$IF($MET=1)
    $$U17=1
    NUPD=NUPD+1
  $$ELSEIF($MET=2)
*
*     THE DFP UPDATE
*
    POM=SQRT(MAX(ZERO,B/A))
    CALL UXVDIR(NA,-POM,AFO,AFD,AFD)
    $$IF(DEC='A')
      CALL UXD$(JASB)MM($POMA,AG,AFO,S)
      CALL UXD$(JASA)MU($POMA,AG,-ONE/B,AFD,S)
    $$ELSEIF(DEC='E')
      CALL UXVCOP(N,AFO,S)
      JOB=1
      $$IF(LAPACK='YES')
        $$IF (JASA='R') JOB=0
        CALL UXDGLM(N,AG,IA,S,JOB)
      $$ELSEIF(MOS2=1)
        $$IF (JASA='R') JOB=0
        CALL UXEGGM(N,N,AG,IA,S,JOB)
      $$ELSE
        $$IF (JASA='C') JOB=0
        CALL UXDGGM(N,AG,IA,S,AFO,JOB)
      $$ENDIF
    $$ELSEIF(DEC='Q')
      $$IF(MOS3=1)
        CALL UXECQB(N,NA,AG,AFO,XS)
        CALL UXD$(JASB)MM($POMA,H,AFO,S)
        CALL UX$(POMD)CRM(N,NA,AG,S,-1)
      $$ELSE
        CALL UXD$(JASB)MM($POMA,AG,AFO,S)
        CALL UXDPRM(N,H,S,-1)
      $$ENDIF  
      $$IF(MOS3=1)
        CALL UX$(POMD)$(JASA)QU(N,NA,H,AG,XS,-ONE/B,AFD,S,XO,INF)
      $$ELSE
        CALL UXD$(JASA)PU(N,NA,AG,H,-ONE/B,AFD,S,XO,INF)
      $$ENDIF  
    $$ENDIF
    TUXX='DFP     '
    NUPD=NUPD+1
  $$ELSEIF($MET=3)
    $$U17=1
*
*     THE RANK ONE UPDATE
*
    IF (B-C.NE.ZERO) THEN
    POM=(A-B)/(B-C)
    IF (POM.GE.ZERO) THEN
    IF (A+C-TWO*B.NE.ZERO) THEN
    POM=(SQRT(POM)-ONE)/(A+C-TWO*B)
    CALL UXVDIR(NA,-ONE,AFO,AFD,AFD)
    $$IF(DEC='A')
      CALL UXD$(JASB)MM($POMA,AG,AFD,S)
      CALL UXD$(JASA)MU($POMA,AG,POM,AFD,S)
    $$ELSEIF(DEC='E')
      CALL UXVCOP(N,AFD,S)
      JOB=1
      $$IF(LAPACK='YES')
        $$IF (JASA='R') JOB=0
        CALL UXDGLM(N,AG,IA,S,JOB)
      $$ELSEIF(MOS2=1)
        $$IF (JASA='R') JOB=0
        CALL UXEGGM(N,N,AG,IA,S,JOB)
      $$ELSE
        $$IF (JASA='C') JOB=0
        CALL UXDGGM(N,AG,IA,S,AFO,JOB)
      $$ENDIF
    $$ELSEIF(DEC='Q')
      $$IF(MOS3=1)
        CALL UXVCOP(NA,AFD,AFO)
        CALL UX$(POMD)CQB(N,NA,AG,AFO,XS)
        CALL UXD$(JASB)MM($POMA,H,AFO,S)
        CALL UX$(POMD)CRM(N,NA,AG,S,-1)
      $$ELSE
$REM        CALL UXD$(JASB)MM($POMA,AG,AFO,S)
        CALL UXD$(JASB)MM($POMA,AG,AFD,S)
        CALL UXDPRM(N,H,S,-1)
      $$ENDIF  
      $$IF(MOS3=1)
        CALL UX$(POMD)$(JASA)QU(N,NA,H,AG,XS,POM,AFD,S,XO,INF)
      $$ELSE
        CALL UXD$(JASA)PU(N,NA,AG,H,POM,AFD,S,XO,INF)
      $$ENDIF  
    $$ENDIF
    TUXX='RANK ONE'
    NUPD=NUPD+1
    GO TO 21
    ENDIF
    ENDIF
    ENDIF
  $$ENDIF
  $$IF(U17>0)
17 CONTINUE
*
*     THE BFGS UPDATE
*
    POM=SQRT(MAX(ZERO,C/B))
    CALL UXVDIR(NA,-POM,AFO,AFD,AFO)
    $$IF(DEC='A')
      CALL UXD$(JASB)MM($POMA,AG,AFO,S)
      CALL UXD$(JASA)MU($POMA,AG,-ONE/C,AFD,S)
    $$ELSEIF(DEC='E')
      CALL UXVCOP(N,AFO,S)
      JOB=1
      $$IF(LAPACK='YES')
        $$IF (JASA='R') JOB=0
        CALL UXDGLM(N,AG,IA,S,JOB)
      $$ELSEIF(MOS2=1)
        $$IF (JASA='R') JOB=0
        CALL UXEGGM(N,N,AG,IA,S,JOB)
      $$ELSE
        $$IF (JASA='C') JOB=0
        CALL UXDGGM(N,AG,IA,S,AFO,JOB)
      $$ENDIF
    $$ELSEIF(DEC='Q')
      $$IF(MOS3=1)
        CALL UX$(POMD)CQB(N,NA,AG,AFO,XS)
        CALL UXD$(JASB)MM($POMA,H,AFO,S)
        CALL UX$(POMD)CRM(N,NA,AG,S,-1)
      $$ELSE
        CALL UXD$(JASB)MM($POMA,AG,AFO,S)
        CALL UXDPRM(N,H,S,-1)
      $$ENDIF  
      $$IF(MOS3=1)
        CALL UX$(POMD)$(JASA)QU(N,NA,H,AG,XS,-ONE/C,AFD,S,XO,INF)
      $$ELSE
        CALL UXD$(JASA)PU(N,NA,AG,H,-ONE/C,AFD,S,XO,INF)
      $$ENDIF  
    $$ENDIF
    TUXX='BFGS    '
  $$ENDIF
21 CONTINUE
  ITERH=0
22 CONTINUE
*  CALL UOU1D2(N,H,S,RHO,GAM,PAR,A,B,C,POM,ETA9)
  RETURN
  END
  $ENDADD
  $ADD(INIT)
    NUXX='UUDG$(DEC)2'
  $ENDADD
  $EXIT
