$INTERFACE UULBM3                                             10/12/01

  $HELP
    SELECTION OF SELF SCALING:
      1 - SELF SCALING SUPPRESSED
      2 - INITIAL SELF SCALING
  $CHECK(MET1,2,'1\2',4)
  $HELP
    SELECTION OF THE LINE SEARCH MODEL:
      MET2=1 - QUADRATIC MODEL
      MET2=2 - BACKWARD TAYLOR EXPANSION (3 TERMS)
      MET2=3 - BACKWARD TAYLOR EXPANSION (4 TERMS)
      MET2=4 - HOMOGENEOUS MODEL
  $CHECK(MET2,1,'1\2\3\4',4)
    $ADD(REAL,'\$XM(2*$NF*$MF)\$XR($MF*($MF+1))\$HR($MF*(2*$MF+1))&
    \$XO($NF)\$GO($NF)\$U2(2*$MF)\$U3(2*$MF)')
$REM  $ADD(REAL,'\PAR3')
$REM  $ADD(INTEGER,'\NF\MF')
  CALL UULBM3(NF,MF,MX,$XM,$XR,$HR,$XO,$GO,$U2,$U3,F,FO,P,PO&
,ETA3,PAR3,MET,MET1,MET2)
  $ADD(INIT)
    NUXX='UULBM3'
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UULBM3                ALL SYSTEMS                10/12/01
C PORTABILITY : ALL SYSTEMS
C 10/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE VARIABLE METRIC UPDATE.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  IU  MM  ACTUAL NUMBER OF VARIABLE METRIC STEPS.
*  RU  XM(2*NF*MF)  SET OF DIFFERENCES.
*  RU  XR(MF*(MF+1))  MATRIX TRANS(XM)*XM STORED IN PACKED FORM.
*  RO  HR(MF*(2*MF+1))  MATRIX IN UPDATE REPRESENTATION.
*  RI  XO(NF)  VECTOR OF VARIABLES DIFFERENCE.
*  RI  GO(NF)  GRADIENT DIFFERENCE.
*  RU  U(2*MF)  VECTOR TRANS(XM)*G.
*  RU  UO(2*MF)  VECTOR TRANS(XM)*GO.
*  RA  V(2*MF)  AUXILIARY VECTOR.
*  RO  F  VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  INITIAL VALUE OF THE OBJECTIVE FUNCTION.
*  RO  P  VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  PAR  SCALING PARAMETER.
*  RU  ETA PARAMETER ETA FOR VARIABLE METRIC UPDATES.
*  II  MET  METHOD SELECTION. MET=1-BFGS UPDATE. MET=2-DFP UPDATE.
*         MET=3-HOSHINO UPDATE. MET=4-RANK ONE UPDATE. MET=5-CONSTANT
*         PARAMETER UPDATE.
*  II  MET1  SCALING OPTION. MET1=1-NO SCALING. MET1=2-PRELIMINARY
*         SCALING.
*
* COMMON DATA :
*  IO  ITERH  TERMINATION INDICATOR. ITERH<0-BAD DECOMPOSITION.
*         ITERH=0-SUCCESSFUL UPDATE. ITERH>0-NONPOSITIVE PARAMETERS.
*  II  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  KIT  NUMBER OF THE ITERATION AFTER LAST RESTART.
*  TO  TUXX  TEXT INFORMATION ON THE CORRECTION USED.
*
* SUBPROGRAMS USED :
*  S   UXDPGT  TRANSFORMATION OF THE GILL-MURRAY DECOMPOSITION.
*  S   UXDSDP  GENERALIZED DOT PRODUCT OF TWO VECTORS.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UOERR1  ERROR MESAGES.
*
* METHOD :
* LIMITED STORAGE VARIABLE METRIC METHODS.
*
      SUBROUTINE UULBM3(NF,MF,MM,XM,XR,HR,XO,GO,UO,V,F,FO,P,PO,BETA,
     & PAR,MET,MET1,MET2)
      $INCLUDE('UMCOMM')
      INTEGER NF,MF,MM,MET,MET1,MET2
      $FLOAT XM(2*NF*MF),XR(MF*(MF+1)),HR(MF*(2*MF+1)),XO(NF),GO(NF),
     & UO(2*MF),V(2*MF),F,FO,P,PO,BETA,PAR
      INTEGER I,II,J,JJ,KK,LL,METS
      $FLOAT UXVDOT
      $FLOAT B,C,RHO,BI,CI,RHOI,HR1,HR2,HR3,POM1,POM2
      $FLOAT CON4
      PARAMETER (CON4=1.0$P-2)
      METS=MET
      IF (MET.GT.4.AND.MET.NE.7) MET=1
      ITERH=0
      C=UXVDOT(NF,XO,XO)
      B=UXVDOT(NF,XO,GO)
      IF (B.LE.ZERO) THEN
      ITERH=2
      TUXX='B - NEG.'
      CALL UOERR1('UULBM3',1)
      GO TO 6
      ENDIF
      PAR=ONE
      IF (MET1.EQ.2) PAR=UXVDOT(NF,GO,GO)/B
      IF (MET1.EQ.3) PAR=B/C
      IF (MET2.EQ.1) THEN
      RHO=ONE
      ELSE
      POM1=(F-FO)/PO
      POM2=P/PO
      IF (MET2.EQ.2) THEN
      IF (POM1.EQ.POM2.OR.POM2.EQ.ONE) THEN
      RHO=ONE
      ELSE
      RHO=HALF*(POM2-ONE)/(POM2-POM1)
      ENDIF
      ELSE IF (MET2.EQ.3) THEN
      IF (TWO*(POM2-ONE)-THREE*(POM1-ONE).EQ.ZERO) THEN
      RHO=ONE
      ELSE
      RHO=HALF*(POM2-ONE)/(TWO*(POM2-ONE)-THREE*(POM1-ONE))
      ENDIF
      ELSE IF (MET2.EQ.4) THEN
      IF (POM1.EQ.ONE.OR.POM2.EQ.POM1) THEN
      RHO=ONE
      ELSE
      RHO=(POM1-ONE)/(POM2-POM1)
      ENDIF
      ENDIF
      IF(RHO.LE.CON4) RHO=ONE
      IF(RHO*CON4.GE.ONE) RHO=ONE
      ENDIF
      RHO=ONE/RHO
      IF (MM.GE.MF) THEN
      II=2*NF
      DO 1 I=1,2*NF*(MM-1)
      XM(I)=XM(I+II)
    1 CONTINUE
      II=2
      JJ=1
      DO 2 I=1,MM-1
      J=JJ+II
      CALL UXVCOP(II,XR(J+2),XR(JJ))
      JJ=J
      II=II+2
    2 CONTINUE
      II=2*(MM-1)*NF+1
      JJ=JJ-1
      ELSE
      II=2*MM*NF+1
      JJ=MM*(MM+1)
      MM=MM+1
      ENDIF
      CALL UXVCOP(NF,GO,XM(II))
      CALL UXVCOP(NF,XO,XM(II+NF))
      CALL UXDRMM(NF,2*MM-2,XM,XO,XR(JJ+1))
      JJ=MM*(MM+1)
      XR(JJ)=C
      XR(JJ-1)=B
      II=1
      JJ=0
      KK=0
      LL=0
      DO 5 I=1,MM
      IF (KK.NE.0) THEN
      CALL UXVCOP(LL,XR(II),UO)
      IF (MET1.GT.1) THEN
      DO 4 J=1,I-1
      UO(2*J)=PAR*UO(2*J)
    4 CONTINUE
      ENDIF
      CALL UXDSMM(LL,HR,UO,V)
      ENDIF
      CI=PAR*XR(JJ+2)+UXVDOT(LL,UO,V)
      BI=XR(JJ+1)
      RHOI=ONE
      IF (I.GE.MM) RHOI=RHO
      IF (MET.EQ.1) THEN
      BETA=ZERO
      TUXX='BFGS    '
      ELSE IF (MET.EQ.2) THEN
      BETA=ONE
      TUXX='DFP     '
      ELSE IF (MET.EQ.3) THEN
      BETA=RHOI*BI/(RHOI*BI+CI)
      TUXX='HOSHINO '
      ELSE IF (MET.EQ.4) THEN
      BETA=ZERO
      TUXX='BFGS    '
      IF (RHOI*BI.GT.CI) THEN
      BETA=RHOI*BI/(RHOI*BI-CI)
      TUXX='RANK ONE'
      ENDIF
      ELSE IF (MET.EQ.7) THEN
      IF (BETA.GE.ONE) THEN
      BETA=ZERO
      TUXX='BFGS    '
      ELSE IF (BETA.EQ.TWO) THEN
      TUXX='PRECON 2'
      ELSE
      TUXX='GENERAL '
      ENDIF
      ENDIF
      HR1=(BETA*CI/BI+RHOI)/BI
      HR2=-BETA/BI
      HR3=(BETA-ONE)/CI
      HR(KK+2*I-1)=HR1
      HR(KK+4*I-2)=HR2
      HR(KK+4*I-1)=HR3
      IF (KK.NE.0) THEN
      CALL UXDSMU(LL,HR,HR3,V)
      CALL UXVSCL(LL,HR2,V,HR(KK+1))
      CALL UXVSCL(LL,HR3,V,HR(KK+2*I))
      ENDIF
      II=II+2*I
      JJ=JJ+2*I+2
      KK=KK+4*I-1
      LL=LL+2
    5 CONTINUE
      NUPD=NUPD+1
    6 CONTINUE
      METS=MET
      RETURN
      END
