$INTERFACE UUSSD7                                             97/12/01

  $HELP
    SWITCH FOR THE HYBRID METHOD:
      0 - THE FLETCHER AND XU SWITCH WITH GENERAL UPDATE
      1 - THE FLETCHER AND XU SWITCH WITH SPECIAL UPDATE
      2 - THE DENNIS AND WELSCH SWITCH
      3 - THE RAMSIN AND WEDIN SWITCH
  $CHECK(MOT2,0,'0\1\2\3',3)
  $N='NF'
  $ADD(REAL,'\$AF($NA)\$H($M)\$AH($MAH)\$AG($MA)\$S($NF)\PPJ\PPB')
$REM  $ADD(REAL,'\PP')
  $ADD(INTEGER,'\$IA($NA)\$IH($NF+1)\$JH($M)\$IAG($NA+1)\$JAG($MA)&
  \IPOM')
$REM  $ADD(INTEGER,'\$N\NA\M\MA\MAH')
  CALL UUSSD7($N,NA,M,MA,MAH,$IA,$AF,$H,$IH,$JH,$AH,$AG,$IAG,$JAG,$S&
,PP,PPJ,PPB,IPOM,$MOT2)
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UUSSD7                ALL SYSTEMS                97/12/01
C PORTABILITY : ALL SYSTEMS
C 97/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* DETERMINATION OF QUADRATIC TERMS FOR THE HYBRID METHOD.
*
* PARAMETERS :
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS OF THE JACOBIAN MATRIX.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE SPARSE HESSIAN MATRIX.
*  II  MB  NUMBER OF NONZERO ELEMENTS OF THE PARTITIONED HESSIAN
*         MATRIX.
*  II  IA(NA)  TYPES OF APPROXIMATED FUNCTIONS.
*  RI  AF(NA)  VECTOR CONTAINING VALUES OF THE APPROXIMATED
*         FUNCTIONS.
*  RU  H(M)  POSITIVE DEFINITE APPROXIMATION OF THE SPARSE HESSIAN
*         MATRIX
*  II  IH(NF+1)  POINTERS OF THE DIAGONAL ELEMENTS OF THE SPARSE
*         HESSIAN MATRIX.
*  II  JH(M)  INDICES OF THE NONZERO ELEMENTS OF THE SPARSE HESSIAN
*             MATRIX IN THE PACKED ROW FORM.
*  RU  AH(MB)  POSITIVE DEFINITE APPROXIMATION OF THE PARTITIONED
*         HESSIAN MATRIX.
*  RI  AG(MA)  NONZERO ELEMENTS OF THE SPARSE JACOBIAN MATRIX.
*  II  IAG(NA+1)  POINTERS OF THE SPARSE JACOBIAN MATRIX.
*  II  JAG(MA)  COLUMN INDICES OF THE SPARSE JACOBIAN MATRIX.
*  RI  S(NF)  DIRECTION VECTOR.
*  RO  PP  QUADRATIC PART OF THE PREDICTED FUNCTION VALUE.
*  RU  PPJ  QUADRATIC TERM WITH THE NORMAL EQUATION MATRIX.
*  RU  PPB  QUADRATIC TERM WITH A SECOND ORDER INFORMATION.
*  II  IPOM  INDICATION FOR SECOND ORDER INFORMATION.
*  II  MOT2  SWITCH FOR THE HYBRID METHOD. MOT2=0-FLETCHER AND XU
*         SWITCH WITH GENERAL UPDATE. MOT2=1-FLETCHER AND XU SWITCH
*         WITH SPECIAL UPDATE. MOT2=2-DENNIS AND WELSCH SWITCH.
*         MOT2=3-RAMSIN AND WEDIN SWITCH.
*
* SUBPROGRAMS USED :
*  RF  UXSSMQ  VALUE OF SPARSE QUADRATIC FORM.
*  RF  UXBSBQ  VALUE OF PARTITIONED QUADRATIC FORM.
*  RF  UXWDOT  DOT PRODUCT OF SPARSE VECTORS.
*
      SUBROUTINE UUSSD7(N,NA,M,MA,MB,IA,AF,H,IH,JH,AH,AG,IAG,JAG,S,PP,
     & PPJ,PPB,IPOM,MOT2)
      $INCLUDE('UMCOMM')
      INTEGER N,NA,M,MA,MB,IA(NA),IH(N+1),JH(M),IAG(NA+1),JAG(MA),IPOM,
     & MOT2
      $FLOAT AF(NA),H(M),AH(MB),AG(MA),S(N),PP,PPJ,PPB
      $FLOAT UXBSBQ,UXSSMQ,UXWDOT
      $FLOAT RHO
      INTEGER K,L,LL,KA,NB
      RHO=PPJ
      IF (IPOM.EQ.0) THEN
      PPJ=UXSSMQ(N,M,H,IH,JH,S,S)
      IF (MOT2.LE.2) THEN
      PPB=PPJ
      NB=0
      DO 1 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      LL=L*(L+1)/2
      IF (IA(KA).GT.0) THEN
      PPB=PPB+AF(KA)*UXBSBQ(L,N,AH(NB+1),JAG(K),S,S,1)
      ENDIF
      NB=NB+LL
    1 CONTINUE
      ENDIF
      ELSE
      PPJ=ZERO
      DO 2 KA=1,NA
      K=IAG(KA)
      L=IAG(KA+1)-K
      IF (IA(KA).GT.0) THEN
      PPJ=PPJ+UXWDOT(L,N,JAG(K),AG(K),S,2)
      ENDIF
    2 CONTINUE
      PPB=UXSSMQ(N,M,H,IH,JH,S,S)
      ENDIF
      PPJ=PPJ*HALF
      PPB=PPB*HALF
      IF (IPOM.EQ.0) THEN
      PP=PPJ
      ELSE
      PP=PPB
      ENDIF
      IF (MOT2.EQ.3) PPB=RHO
      RETURN
      END
