* SUBROUTINE UXDGLB                ALL SYSTEMS                00/12/01
C PORTABILITY : ALL SYSTEMS
C 94/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A DENSE UNSYMMETRIC
* MATRIX A USING THE FACTORIZATION P*A=L*U OBTAINED BY THE SUBROUTINE
* UXDGLF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*N) THE FACTORIZATION P*A=L*U OBTAINED BY THE SUBROUTINE
*         UXDGLF.
*  II  IA(N) A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RU  X(N) ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  RA  Y(N)  AUXILIARY VECTOR.
*  JOB  OPTION. JOB=0 - SOLUTION WITH THE ORIGINAL MATRIX.
*         JOB=1 - SOLUTION WITH THE MATRIX TRSANSPOSE.
*
* METHOD :
* BACK SUBSTITUTION.
*
      SUBROUTINE UXDGLB(N,A,IPIV,B,JOB)
      INTEGER N,IPIV(N),JOB
      $FLOAT A(N*N),B(N)
      $FLOAT  ONE
      PARAMETER (ONE=1.0$P 0)
*
*     Quick return if possible
*
      IF(N.EQ.0) RETURN
      IF(JOB.EQ.0) THEN
*
*        Solve A * X = B.
*
*        Apply row interchanges to the right hand sides.
*
C         CALL DLASWP(1,B,N,1,N,IPIV,1)
         CALL UXVPER(N,IPIV,B,1)
*
*        Solve L*X = B, overwriting B with X.
*
         CALL DTRSM('L','L','N','U',N,1,ONE,A,N,B,N)
*
*        Solve U*X = B, overwriting B with X.
*
         CALL DTRSM('L','U','N','N',N,1,ONE,A,N,B,N)
      ELSE
*
*        Solve A' * X = B.
*
*        Solve U'*X = B, overwriting B with X.
*
         CALL DTRSM('L','U','T','N',N,1,ONE,A,N,B,N)
*
*        Solve L'*X = B, overwriting B with X.
*
         CALL DTRSM('L','L','T','U',N,1,ONE,A,N,B,N)
*
*        Apply row interchanges to the solution vectors.
*
C         CALL DLASWP(1,B,N,1,N,IPIV,-1 )
         CALL UXVPER(N,IPIV,B,-1)
      END IF
      RETURN
      END
