* SUBROUTINE UXDLBD                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF A DIRECTION OF NEGATIVE CURVATURE WITH RESPECT TO
* A DENSE SYMMETRIC MATRIX A USING THE FACTORIZATION
* A=P*U*D*TRANS(U)*TRANS(P) OBTAINED BY THE SUBROUTINE UXDLBF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2)  FACTORIZATION A=P*U*D*TRANS(U)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDLBF.
*  II  IA(N)  PERMUTATION FIELD OBTAINED BY THE SUBROUTINE UXDLBF.
*  RO  X(N)  COMPUTED DIRECTION OF NEGATIVE CURVATURE (I.E.
*         TRANS(X)*A*X<0) IF IT EXISTS.
*  RO  ALF  VALUE OF TRANS(X)*A*X.
*  II  INF  INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. THE
*         DIRECTION OF NEGATIVE CURVATURE EXISTS ONLY IF INF>0.
*
* METHOD :
* R.FLETCHER, T.L.FREEMAN: A MODIFIED NEWTON METHOD FOR MINIMIZATION,
* J.OPTIMIZATION THEORY APPL. 23 (1977) PP. 357-372.
*
      SUBROUTINE UXDLBD(N,A,IA,X,ALF,INF)
      $FLOAT ZERO, HALF, ONE
      PARAMETER (ZERO=0.0$P 0, HALF=5.0$P-1, ONE=1.0$P 0)
      INTEGER N, IA(N), INF
      $FLOAT  A(N*(N+1)/2), X(N), ALF
      INTEGER K, KC
      $FLOAT TEMP, UXVNOR, UXDLBQ
*
      IF (INF .LE. 0) RETURN
      K = N
      KC = N*( N+1 ) / 2
      X = ZERO
   10 CONTINUE
      IF( K.LT.1 ) GO TO 30
      IF( IA( K ).GT.0 ) THEN
*
*         1 x 1 diagonal block.
*
          IF (A(KC).LT.ZERO) X(K) = ONE
          KC = KC - K
          K = K - 1
      ELSE
*
*         2 x 2 diagonal block.
*
          IF (A(KC-K).LT.ZERO.OR.A(KC).LT.ZERO.OR.
     &        (A(KC-K)*A(KC)-A(KC-1)*A(KC-1)).LT.ZERO) THEN
              TEMP = HALF*(A(KC) - A(KC-K))
              X(K-1) = A(KC-1)
              X(K) = TEMP - SQRT(TEMP*TEMP + A(KC-1)*A(KC-1))
              TEMP = UXVNOR(2,X(K-1))
              X(K-1) = X(K-1)/TEMP
              X(K) = X(K)/TEMP
          END IF
          KC = KC - 2*K + 1
          K = K - 2
      END IF
      GOTO 10
   30 CONTINUE
C
C     BACK SUBSTITUTION
C
      ALF=UXDLBQ(N,A,IA,X)
      CALL UXDLBB(N,A,IA,X,-1)
      RETURN
      END


