* SUBROUTINE UXDLGB                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A DENSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A+E USING THE FACTORIZATION
* A+E=P*U*D*TRANS(U)*TRANS(P) OBTAINED BY THE SUBROUTINE UXDLGF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) FACTORIZATION A+E=P*U*D*TRANS(U)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDLGF.
*  II  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 THEN X:=(A+E)**(-1)*X. IF JOB>0 THEN
*         X:=(P*U)**(-1)*X. IF JOB<0 THEN X:=(TRANS(U)*TRANS(P))**(-1)*X
*
* METHOD :
* BACK SUBSTITUTION.
*
      SUBROUTINE UXDLGB(N,A,IA,X,JOB)
      INTEGER  N, IA(N), JOB
      $FLOAT  A(N*(N+1)/2), X(N)
      INTEGER K, KC, KP
*
*     Solve A*X = B, where A = U*D*U**T.
*
      IF (JOB .GE. 0) THEN
*
*         First solve U*D*X = B, overwriting B with X.
*         K is the main loop index, decreasing from N to 1.
*
          KC = N*( N+1 ) / 2 + 1
          DO K = N, 1, -1
              KC = KC - K
*
*             interchange elements K <-> IA(K).
*
              KP = IA( K )
              IF( KP.NE.K ) CALL UNSWAP(X(K),X(KP))
*
*             Multiply by inv(U(K)), where U(K) is the transformation
*             stored in column K of A.
*
              IF (K.GT.1) CALL UXVDIR(K-1,-X(K),A(KC),X(1),X(1))
*
*             Multiply by the inverse of the diagonal block.
*
              IF (JOB .EQ. 0) X(K) = X(K) / A( KC+K-1 )
          ENDDO
      END IF
      IF (JOB .LE. 0) THEN
*
*         Next solve U**T*X = B, overwriting B with X.
*         K is the main loop index.
*
          KC = 1
          DO K = 1, N
*
*             Multiply by inv(U**T(K)), where U(K) is the transformation
*             stored in column K of A.
*
C              IF (K.GT.1) X(K) = X(K) - UXVDOT(K-1,X,A(KC))
              IF (K.GT.1) X(K) = X(K)-DOT_PRODUCT(X(1:K-1),A(KC:KC+K-2))
*
*             Interchange elements K <-> IA(K).
*
              KP = IA( K )
              IF( KP.NE.K ) CALL UNSWAP(X(K),X(KP))
              KC = KC + K
          ENDDO
      END IF
      RETURN
      END