* SUBROUTINE UXDLGN                ALL SYSTEMS               92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* ESTIMATION OF THE MINIMUM EIGENVALUE AND THE CORRESPONDING EIGENVECTOR
* OF A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX A+E USING THE
* FACTORIZATION A+E=U*D*TRANS(U) OBTAINED BY THE SUBROUTINE UXDLGF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) FACTORIZATION A+E=U*D*TRANS(U) OBTAINED BY THE
*         SUBROUTINE UXDLGF.
*  II  IA(N)  PERMUTATION VECTOR WHICH CONTAINS INFORMATION OBTAINED
*         BY THE SUBROUTINE UXDLGF.
*  RO  X(N)  ESTIMATED EIGENVECTOR.
*  RO  ALF  ESTIMATED EIGENVALUE.
*  II  JOB  OPTION. IF JOB=0 THEN ONLY ESTIMATED EIGENVALUE IS
*         COMPUTED. IS JOB>0 THEN BOTH ESTIMATED EIGENVALUE AND
*         ESTIMATED EIGENVECTOR ARE COMPUTED BY JOB ITERATIONS.
*
* METHOD :
* A.K.CLINE, C.B.MOLER, G.W.STEWART, J.H.WILKINSON : AN ESTIMATE
* FOR THE CONDITION NUMBER OF A MATRIX. SIAM J. NUMER. ANAL. 16
* (1979) 368-373.
*
      SUBROUTINE UXDLGN(N,A,IA,X,ALF,JOB)
      INTEGER N, IA(N), JOB
      $FLOAT A(N*(N+1)/2), X(N), ALF
      INTEGER K, KC, KD, KP, I
      $FLOAT XP, XM, FP, FM, UXVDOT
      $FLOAT ZERO, ONE
      PARAMETER (ZERO=0.0$P 0, ONE=1.0$P 0)
C
C     COMPUTATION OF THE VECTOR V WITH POSSIBLE MAXIMUM NORM SUCH
C     THAT  U*D**(1/2)*V=L  WHERE L HAS ELEMENTS +1 OR -1
C
      X = ZERO
      KD = N*(N+1)/2
      DO K = N, 1, -1
          KP = IA(K)
          IF (KP.NE.K) CALL UNSWAP(X(KP),X(K))
          XP = -X(K) + ONE
          XM = -X(K) - ONE
          FP = ABS(XP)
          FM = ABS(XM)
          KD = KD - K
          DO I = 1, K-1
              FP = FP + ABS( X(I) + A(KD+I)*XP )
              FM = FM + ABS( X(I) + A(KD+I)*XM )
          ENDDO
          IF (FP.GE.FM) THEN
              X(K) = XP
              CALL UXVDIR(K-1,XP,A(KD+1),X,X)
          ELSE
              X(K) = XM
              CALL UXVDIR(K-1,XM,A(KD+1),X,X)
          ENDIF
      ENDDO
C
C     COMPUTATION OF THE VECTOR X SUCH THAT
C     D**(1/2)*TRANS(U)*X=V
C
      DO K = 1, N/2
          I = N - K + 1
          CALL UNSWAP(X(K),X(I))
      ENDDO
      FM = ZERO
      KD = N*(N+1)/2
      DO K = N, 1, -1
          KP = IA(K)
          IF (KP.NE.K) CALL UNSWAP(X(KP),X(K))
          IF (JOB.LE.0) THEN
              FP = SQRT(A(KD))
              X(K) = X(K)/FP
              FM = FM + X(K)*X(K)
          ELSE
              X(K) = X(K)/A(KD)
          ENDIF
          KD = KD - K
      ENDDO
      DO K = 1, N
          KP = IA(K)
          IF (KP.NE.K) CALL UNSWAP(X(KP),X(K))
      ENDDO
      DO K = 1, N/2
          I = N - K + 1
          CALL UNSWAP(X(K),X(I))
      ENDDO
      FP = DBLE(N)
      IF (JOB.LE.0) THEN
C
C         FIRST ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C         ALF=(TRANS(L)*L)/(TRANS(V)*V)
C
          ALF=FP/FM
          RETURN
      ENDIF
      FM = ZERO
      KC = 1
      DO K = 1, N
          IF (K.GT.1) X(K) = X(K) - UXVDOT(K-1,A(KC),X)
          FM = FM + X(K)*X(K)
          KP = IA(K)
          IF (KP.NE.K) CALL UNSWAP(X(KP),X(K))
          KC = KC + K
      ENDDO
      FM = SQRT(FM)
      IF (JOB.LE.1) THEN
C
C         SECOND ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C         ALF=SQRT(TRANS(L)*L)/SQRT(TRANS(X)*X)
C
          ALF = SQRT(FP)/FM
      ELSE
C
C         INVERSE ITERATIONS
C
          DO I = 2, JOB
C
C             SCALING THE VECTOR X BY ITS NORM
C
              DO K = 1, N
                  X(K) = X(K)/FM
              ENDDO
              CALL UXDLGB(N,A,IA,X,0)
              FM = SQRT(UXVDOT(N,X,X))
          ENDDO
          ALF = ONE/FM
      ENDIF
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO K = 1, N
          X(K) = X(K)/FM
      ENDDO
C
      RETURN
      END
