* SUBROUTINE UXDLGX                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* SCALING FACTORS FOR A DENSE SYMMETRIC POSITIVE DEFINITE MATRIX A+E
* IN THE FACTORED FORM A+E=P*U*D*TRANS(U)*TRANS(P) OBTAINED BY THE
* SUBROUTINE UXDLGF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RU  A(N*(N+1)/2) FACTORIZATION A+E=P*U*D*TRANS(U)*TRANS(P) OBTAINED
*         BY THE SUBROUTINE UXDLGF.
*  II  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RO  X(N)  VECTOR OF SCALING FACTORS.
*
      SUBROUTINE UXDLGX(N, A, IA, X)
      INTEGER N, IA(N)
      $FLOAT  A(N*(N+1)/2), X(N)
      INTEGER K, KD, KP, I
      $FLOAT ONE, CON1, CON2
      PARAMETER (ONE=1.0$P 0, CON1=2.0$P-5, CON2=1.0$P 5)
C
      X = ONE
      KD = N*(N+1)/2
      DO K = N, 2, -1
          KD = KD - K
          DO I = 1, K-1
              X(K) = X(K) + A(KD+I)**2
          ENDDO
          X(K) = SQRT(X(K))
          X(K) = MAX(X(K),CON1)
          X(K) = MIN(X(K),CON2)
      ENDDO
C
      KD = 0
      DO K = 1, N
          KD = KD + K
          A(KD) = A(KD) * X(K)**2
      ENDDO
C
      DO K = 2, N
          KP = IA(K)
          IF (KP.NE.K) CALL UNSWAP(X(K),X(KP))
      ENDDO
C
      RETURN
      END
