* SUBROUTINE UXDPGC                ALL SYSTEMS               91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
*  DECOMPOSITION A+ALF*I=L*D*TRANS(L) OF A DENSE SYMMETRIC POSITIVE
*  DEFINITE MATRIX A+ALF*I WHERE I IS UNIT MATRIX, D IS DIAGONAL
*  POSITIVE DEFINITE MATRIX AND L IS LOWER TRIANGULAR MATRIX. IF A IS
*  SUFFICIENTLY POSITIVE DEFINITE THEN ALF=0.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) GIVEN DENSE SYMMETRIC MATRIX A STORED IN THE PACKED
*         FORM.
*  RO  B(N*(N+1)/2) COMPUTED FACTORIZATION A+ALF*I=L*D*TRANS(L).
*  RI  TOL  TOLERANCE FOR POSITIVE DEFINITENESS.
*  RU  ALF  SCALAR PARAMETER (SHIFT) IN THE FACTORIZATION.
*  IU  M  COUNT FOR DECOMPOSITIONS.
*
* SUBPROGRAMS USED :
*  S   UXDPGF  GILL-MURRAY DECOMPOSITION OF A DENSE SYMMETRIC MATRIX.
*  S   UXDSDA  ADDITION OF A SCALED UNIT MATRIX TO A DENSE SYMMETRIC
*         MATRIX.
*  S   UXDSMC  COPYING OF A DENSE SYMMETRIC MATRIX.
*  S   UXDSMG  GERSHGORIN BOUNDS OF A DENSE SYMMETRIC MATRIX.
*
* METHOD :
*  J.E. DENNIS, R.B.SCHNABEL: NUMERICAL METHODS FOR UNCONSTRAINED
*  OPTIMIZATION AND NONLINEAR EQUATIONS. PRENTICE HALL, ENGLEWOOD
*  CLIFFS, NEW JERSEY 1983.
*
      SUBROUTINE UXDPGC(N,A,B,TOL,ALF,M)
      INTEGER N,M
      $FLOAT A(N*(N+1)/2), B(N*(N+1)/2), TOL, ALF
      $FLOAT ETA, BET, AMAX, AMIN, TEMP
      INTEGER I, J, K, INF
      $FLOAT ZERO, ONE, TWO
      PARAMETER (ZERO=0.0$P 0, ONE=1.0$P 0, TWO=2.0$P 0)
      ETA=TWO*TOL
      IF(ALF.LE.ZERO) THEN
C
C     COMPUTATION OF THE FIRST SHIFT FROM THE CONDITION
C     NUMBER ESTIMATION
C
      ALF=ZERO
      BET=ZERO
      AMAX=A(1)
      AMIN=A(1)
      K=1
      DO 2 J=1,N
      DO 1 I=1,J
      IF (I.EQ.J) THEN
      AMAX=MAX(AMAX,A(K))
      AMIN=MIN(AMIN,A(K))
      ELSE
      BET=MAX(BET,ABS(A(K)))
      ENDIF
      K=K+1
    1 CONTINUE
    2 CONTINUE
      IF (AMIN.LT.ETA*AMAX) THEN
      TEMP=TWO*ETA*(AMAX-AMIN)-AMIN
      ELSE
      TEMP=ZERO
      ENDIF
      IF (TEMP.GT.ZERO) THEN
      ALF=ALF+TEMP
      AMAX=AMAX+TEMP
      ENDIF
      TEMP=BET*(ONE+TWO*ETA)-AMAX
      IF (TEMP.GT.ZERO) THEN
      ALF=ALF+TEMP
      AMAX=AMAX+TEMP
      ENDIF
      IF (AMAX.EQ.ZERO) THEN
      ALF=ONE
      AMAX=ONE
      ENDIF
      ENDIF
C
C     DECOMPOSITION OF THE SHIFTED MATRIX
C
      CALL UXDSMC(N,A,B)
      IF (ALF.GT.ZERO) CALL UXDSDA(N,B,ALF)
      M=M+1
      TEMP=TOL
      CALL UXDPGF(N,B,INF,TEMP,BET)
      IF (INF.EQ.0) RETURN
C
C     COMPUTATION OF THE SECOND SHIFT FROM THE PREVIOUS DECOMPOSITION
C     AND THE GERSHGORIN BOUNDS
C
      CALL UXDSMC(N,A,B)
      IF (ALF.GT.ZERO) CALL UXDSDA(N,B,ALF)
      CALL UXDSMG(N,B,AMIN,AMAX)
      IF (AMIN.LT.ETA*AMAX) THEN
      TEMP=TWO*ETA*(AMAX-AMIN)-AMIN
      ELSE
      TEMP=ZERO
      ENDIF
      TEMP=MIN(TEMP,BET)
      IF (TEMP.GT.ZERO) THEN
      ALF=ALF+TEMP
      AMAX=AMAX+TEMP
      ENDIF
C
C     DECOMPOSITION OF THE SHIFTED MATRIX
C
      CALL UXDSMC(N,A,B)
      IF (ALF.GT.ZERO) CALL UXDSDA(N,B,ALF)
      M=M+1
      TEMP=TOL
      CALL UXDPGF(N,B,INF,TEMP,BET)
      RETURN
      END
