* SUBROUTINE UXDPRD                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF A DIRECTION OF NEGATIVE CURVATURE WITH RESPECT TO A
* DENSE SYMMETRIC POSITIVE DEFINITE MATRIX A USING THE FACTORIZATION
*         A=TRANS(R)*R.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE
*         SUBROUTINE UXDPGF.
*  RO  X(N)  COMPUTED DIRECTION OF NEGATIVE CURVATURE (I.E.
*         TRANS(X)*A*X<0) IF IT EXISTS.
*  II  INF  INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. THE
*         DIRECTION OF NEGATIVE CURVATURE EXISTS ONLY IF INF>0.
*
* METHOD :
* P.E.GILL, W.MURRAY : NEWTON TYPE METHODS FOR UNCONSTRAINED AND
* LINEARLY CONSTRAINED OPTIMIZATION, MATH. PROGRAMMING 28 (1974)
* PP. 311-350.
*
      SUBROUTINE UXDPRD(N,A,X,INF)
      INTEGER N, INF
      $FLOAT  A(N*(N+1)/2), X(N)
      INTEGER  I, J, II, IJ
      $FLOAT ZERO, ONE
      PARAMETER (ZERO = 0.0D0, ONE = 1.0D0)
C
C     RIGHT HAND SIDE FORMATION
C
      DO 1 I=1,N
      X(I) = ZERO
    1 CONTINUE
      IF (INF .LE. 0) RETURN
      X(INF) = ONE
C
C     BACK SUBSTITUTION
C
      II = INF*(INF-1)/2
      DO  3  I = INF-1, 1, -1
      IJ = II
      DO  2  J = I+1, INF
      IJ = IJ + J - 1
      X(I) = X(I) - A(IJ)*X(J)
    2 CONTINUE
      X(I) = X(I)/A(II)
      II = II - I
    3 CONTINUE
      RETURN
      END
