* SUBROUTINE UXDPRO                ALL SYSTEMS               91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* ESTIMATION OF THE MINIMUM SINGULAR VALUE AND THE CORRESPONDING
* SINGULAR VECTOR OF A DENSE UPPER TRIANGULAR MATRIX A.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) UPPER TRIANGULAR MATRIX.
*  RO  X(N)  ESTIMATED SINGULAR VECTOR.
*  RO  ALF  SQUARE OF THE ESTIMATED SINGULAR VALUE.
*  II  JOB  OPTION. IF JOB=0 THEN ONLY ESTIMATED SINGULAR VALUE IS
*         COMPUTED. IS JOB>0 THEN BOTH ESTIMATED SINGULAR VALUE AND
*         ESTIMATED SINGULAR VECTOR ARE COMPUTED BY JOB ITERATIONS.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION.
*  S   UXDPRD  COMPUTATION OF A DIRECTION OF NEGATIVE CURVATURE.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*
* METHOD :
* C.H.BISCHOF : INCREMENTAL CONDITION ESTIMATION. SIAM J. MATRIX ANAL.
* APPL. 11 (1990) 312-322.
*
      SUBROUTINE UXDPRO(N,A,X,ALF,JOB)
      INTEGER N,JOB
      $FLOAT A(N*(N+1)/2),X(N),ALF
      $FLOAT DEN,XNEW,XEST,BET,ETA,EPS,S,C,UXVDOT
      INTEGER I,K,L,IK,KK
      $FLOAT ZERO,ONE,TWO,HALF
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0,TWO=2.0$P 0,HALF=5.0$P-1)
C
C     COMPUTATION OF THE VECTOR V WITH POSSIBLE MAXIMUM NORM SUCH
C     THAT  TRANS(R)*V=U  WHERE U IS A VECTOR WITH UNIT NORM
C
      XEST=ZERO
      L=0
      DO 3 K=1,N
      ALF=ZERO
      DO 1 I=1,K-1
      ALF=ALF+X(I)*A(I+L)
    1 CONTINUE
      L=L+K
      DEN=A(L)**2
      IF (DEN.EQ.ZERO) THEN
      ALF=ZERO
      CALL UXDPRD(N,A,X,K)
      RETURN
      ELSE
      IF (ALF.EQ.ZERO) THEN
      IF (XEST*DEN.GT.ONE) THEN
      XNEW=XEST
      S=ONE
      C=ZERO
      ELSE
      XNEW=ONE/DEN
      S=ZERO
      C=ONE
      ENDIF
      ELSE
      BET=DEN*XEST+ALF**2-ONE
      ETA=HALF*BET/ABS(ALF)
      EPS=SQRT(ETA**2+ONE)
      XNEW=(ONE+ABS(ALF)*(ETA+EPS))/DEN
      S=SQRT((ONE+ETA/EPS)/TWO)
      C=SQRT((ONE-ETA/EPS)/TWO)
      ENDIF
      ENDIF
      DO 2 I=1,K-1
      X(I)=X(I)*S
    2 CONTINUE
      X(K)=(C-ALF*S)/A(L)
      XEST=XNEW
    3 CONTINUE
C
C     FIRST ESTIMATION OF THE MINIMUM SINGULAR VALUE BY THE FORMULA
C     ALF=1/(TRANS(V)*V)
C
      IF (JOB.LE.0) THEN
      ALF=ONE/XEST
      RETURN
      ENDIF
C
C     COMPUTATION OF THE VECTOR X SUCH THAT R*X=V
C
      XEST=ZERO
      KK=N*(N+1)/2
      DO 9 K=N,1,-1
      IK=KK
      DO 8 I=K+1,N
      IK=IK+I-1
      X(K)=X(K)-A(IK)*X(I)
    8 CONTINUE
      X(K)=X(K)/A(KK)
      XEST=XEST+X(K)*X(K)
      KK=KK-K
    9 CONTINUE
      XEST=SQRT(XEST)
      IF (JOB.LE.1) THEN
C
C     SECOND ESTIMATION OF THE MINIMUM EIGENVALUE BY THE FORMULA
C     ALF=1/SQRT(TRANS(X)*X)
C
      ALF=ONE/XEST
      ELSE
C
C     INVERSE ITERATIONS
C
      DO 11 K=2,JOB
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO 10 I=1,N
      X(I)=X(I)/XEST
   10 CONTINUE
      CALL UXDPRB(N,A,X,0)
      XEST=SQRT(UXVDOT(N,X,X))
   11 CONTINUE
      ALF=ONE/XEST
      ENDIF
C
C     SCALING THE VECTOR X BY ITS NORM
C
      DO 12 I=1,N
      X(I)=X(I)/XEST
   12 CONTINUE
      RETURN
      END
