* SUBROUTINE UXDSRB                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A DENSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A USING THE FACTORIZATION
* A=TRANS(P)*TRANS(R)*R*P.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  RI  A(N*(N+1)/2) FACTORIZATION A=TRANS(P)*TRANS(R)*R*P.
*  II  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX P.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 THEN X:=A**(-1)*X. IF JOB>0 THEN
*         X:=(TRANS(P)*TRANS(R))**(-1)*X. IF JOB<0 THEN X:(P*R)**(-1)*X.

*
* METHOD :
* BACK SUBSTITUTION.
*
      SUBROUTINE UXDSRB(N,A,IA,X,JOB)
      INTEGER N,IA(N),JOB
      $FLOAT  A(N*(N+1)/2),X(N)
      INTEGER  I,J,IP,JP,II,IJ,IND
      IND(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
      IF (JOB.GE.0) THEN
C
C     PHASE 1 : X:=(TRANS(P)*TRANS(R))**(-1)*X
C
      DO 3 I=1,N
      IP=IA(I)
      DO 2 J=1,I-1
      JP=IA(J)
      IJ=IND(IP,JP)
      X(IP)=X(IP)-A(IJ)*X(JP)
    2 CONTINUE
      II=IND(IP,IP)
      X(IP)=X(IP)/A(II)
    3 CONTINUE
      ENDIF
      IF (JOB.LE.0) THEN
C
C     PHASE 2 : X:=(P*R)**(-1)*X
C
      DO 6 I=N,1,-1
      IP=IA(I)
      DO 5 J=I+1,N
      JP=IA(J)
      IJ=IND(IP,JP)
      X(IP)=X(IP)-A(IJ)*X(JP)
    5 CONTINUE
      II=IND(IP,IP)
      X(IP) = X(IP)/A(II)
    6 CONTINUE
      ENDIF
      RETURN
      END
