* SUBROUTINE UXDTGB                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS AFTER ORTHOGONAL
* TRANSFORMATION USING GIVENS ROTATIONS.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  IO  M  INFORMATION OBTAINED DURING THE BACK SUBSTITUTION. IF M=0
*         OR M<0 THEN A IS NOT OR IS ALMOST SINGULAR.
*  RI  A(N*(N+1)/2) UPPER TRIANGULAR MATRIX OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  RU  B(N)  ON INPUT THE RIGHT HAND SIDE VECTOR OBTAINED USING
*         ORTHOGONAL TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*         ON OUTPUT THE SOLUTION B:=A**(-1)*B.
*  RI  TOL  TOLERANCE FOR PSEUDORANK DETERMINATION.
*
* METHOD :
*  BACK SUBSTITUTION.
*
      SUBROUTINE UXDTGB(N,M,A,B,TOL)
      INTEGER N,M
      $FLOAT A(N*(N+1)/2),B(N),TOL
      $FLOAT TOL1,TEMP
      INTEGER I,J,II,IJ
      $FLOAT ZERO
      PARAMETER (ZERO=0.0$P 0)
      M=0
      TOL1=SQRT(TOL)
      TEMP=TOL1
      DO 3 I=1,N*(N+1)/2
      TEMP=MAX(TEMP,ABS(A(I)))
    3 CONTINUE
      TEMP=TEMP*TOL1
      II=N*(N+1)/2
      DO 2 I=N,1,-1
      IF(ABS(A(II)).LE.TEMP) THEN
      M=-1
      A(II)=SIGN(TEMP,A(II))
      ENDIF
      IJ=II
      DO 1 J=I+1,N
      IJ=IJ+J-1
      B(I)=B(I)-A(IJ)*B(J)
    1 CONTINUE
      B(I)=B(I)/A(II)
      II=II-I
    2 CONTINUE
      RETURN
      END

