* SUBROUTINE UXDTGF                ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* FINAL ORTHOGONAL TRANSFORMATION OF A SET OF LINEAR
* EQUATIONS USING GIVENS ROTATIONS.
*
* PARAMETERS :
*  II  N  ORDER OF THE MATRIX A.
*  IU  M  ON INPUT THE ACTUAL NUMBER OF ROWS IN THE MATRIX A.
*         ON OUTPUT AN INFORMATION OBTAINED DURING THE FINAL
*         ORTHOGONAL TRANSFORMATION. IF M=0 OR M<0 THEN A IS NOT
*         OR IS ALMOST SINGULAR.
*  RU  A(N*(N+1)/2) UPPER TRIANGULAR MATRIX OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  RU  B(N)  RIGHT HAND SIDE VECTOR OBTAINED USING ORTHOGONAL
*         TRANSFORMATION OF A SET OF LINEAR EQUATIONS.
*  RI  TOL  TOLERANCE FOR PSEUDORANK DETERMINATION.
*
* SUBPROGRAMS USED :
*  S   UXVORT  COMPUTATION OF THE PLANE ROTATION MATRIX.
*  S   UXVROT  PLANE ROTATION IS APPLIED TO TWO NUMBERS.
*
* METHOD :
*  GIVENS ROTATIONS WITH SQUARE ROOTS.
*
      SUBROUTINE UXDTGF(N,M,A,B,TOL)
      INTEGER N,M
      $FLOAT A(N*(N+1)/2),B(N),TOL
      $FLOAT C,S,TEMP,TOL1
      INTEGER I,J,K,II,IJ,JJ,IK,JK,KK
      $FLOAT ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
      IF(M.LT.N) THEN
      II=M*(M+1)/2
      DO 2 I=M+1,N
      IJ=II+1
      II=II+I
      DO 1 J=I,N
      IJ=IJ+J-1
      A(IJ)=ZERO
    1 CONTINUE
    2 CONTINUE
      ENDIF
      TOL1=SQRT(TOL)
      TEMP=TOL1
      DO 3 I=1,N*(N+1)/2
      TEMP=MAX(TEMP,ABS(A(I)))
    3 CONTINUE
      TEMP=TEMP*TOL1
      M=0
      II=0
      DO 7 I=1,N
      II=II+I
      IF(ABS(A(II)).LE.TEMP) THEN
      M=-1
      JJ=II
      DO 6 J=I+1,N
      IJ=JJ+I
      JJ=JJ+J
      IF(A(IJ).NE.ZERO) THEN
      CALL UXVORT(A(JJ),A(IJ),C,S,KK)
      IK=IJ
      JK=JJ
      DO 5 K=J,N-1
      IK=IK+K
      JK=JK+K
      CALL UXVROT(A(JK),A(IK),C,S,KK)
    5 CONTINUE
      CALL UXVROT(B(J),B(I),C,S,KK)
      ENDIF
    6 CONTINUE
      ENDIF
    7 CONTINUE
      RETURN
      END
