* SUBROUTINE UXECHF               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* QR DECOMPOSITION OF A ROWWISE STORED DENSE RECTANGULAR MATRIX A USING
* HOUSEHOLDER TRANSFORMATIONS WITH PIVOTING.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX A.
*  II  M  NUMBER OF ROWS OF THE MATRIX A.
*  RU  A(M,N)  RECTANGULAR MATRIX STORED IN THE TWO-DIMENSIONAL ARRAY.
*  IO  IA(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         PERMUTATION MATRIX.
*  RO  D(N)  VECTOR CONTAINING INFORMATION ABOUT HOUSEHOLDER
*         TRANSFORMATIONS.
*  RA  E(N)  AUXILIARY VECTOR.
*  RA  V(N)  AUXILIARY VECTOR.
*
* SUBPROGRAMS USED :
*  S   UXDRMN  EUCLIDEAN NORM OF A PART OF THE ROWWISE STORED
*         RECTANGULAR MATRIX COLUMN.
*
* METHOD :
* P.A.BUSSINGER, G.H.GOLUB : LINEAR LEAST SQUARES SOLUTION BY
* HOUSEHOLDER TRANSFORMATION. NUMER. MATH. 7 (1965) 269-276.
*
      SUBROUTINE UXECHF(M,N,A,IA,D,E,V,NM)
      INTEGER M,N,NM
      INTEGER IA(N)
      $FLOAT A(M,N),D(N),E(N),V(N)
      $FLOAT ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
      INTEGER I,IP,J,L,UXVIMX
      $FLOAT AII,POM,TEMP,UXVNOR,UXVDOT
      NM=MIN(N,M)
*
*     INITIALIZE PARTIAL COLUMN NORMS - THE FIRST ENTRIES OF WORK STORE
*     THE EXACT COLUMN NORMS
*     
      DO 20 I=1,N
        IA(I)=I
        E(I)=UXVNOR(M,A(1,I))
        V(I)=E(I)
   20 CONTINUE
*
*     COMPUTE FACTORIZATION
*
      DO 40 I=1,NM
*
*     DETERMINE I-TH PIVOT COLUMN AND SWAP IF NECESSARY
*
        L=(I-1)+UXVIMX(N-I+1,E(I),POM)
        IF (L.NE.I) THEN
          CALL UXVSWP(M,A(1,L),A(1,I),1)
          IP=IA(L)
          IA(L)=IA(I)
          IA(I)=IP
          E(L)=E(I)
          V(L)=V(I)
        END IF
*
*     GENERATE ELEMENTARY REFLECTOR H(I)
*
        IF(I.LT.M) THEN
          CALL UXVORG(M-I+1,A(I,I),A(I+1,I),1,D(I))
        ELSE
          CALL UXVORG(1,A(M,M),A(M,M),1,D(M))
        END IF
        IF (I.LT.N) THEN
*
*     APPLY H(I) TO A(I:M,I+1:N) FROM THE LEFT
*
          AII=A(I,I)
          A(I,I)=ONE
          DO 25 J=I+1,N
            POM=UXVDOT(M-I+1,A(I,I),A(I,J))
            CALL UXVDIR(M-I+1,-D(I)*POM,A(I,I),A(I,J),A(I,J))
   25     CONTINUE    
          A(I,I)=AII
        END IF
*
*     UPDATE PARTIAL COLUMN NORMS
*
        DO 30 J=I+1,N
          IF (E(J).NE.ZERO) THEN
            POM=ONE-(ABS(A(I,J))/E(J))**2
            POM=MAX(POM,ZERO)
            TEMP=ONE+0.05$P 0*POM*(E(J)/V(J))**2
            IF (TEMP.EQ.ONE) THEN
              IF (M-I.GT.0) THEN
                E(J)=UXVNOR(M-I,A(I+1,J))
                V(J)=E(J)
              ELSE
                E(J)=ZERO
                V(N)=ZERO
              END IF
            ELSE
              E(J)=E(J)*SQRT(POM)
            END IF
          END IF
   30   CONTINUE
   40 CONTINUE
      RETURN
      END
