* SUBROUTINE UXECHP               ALL SYSTEMS                16/12/01
C PORTABILITY : ALL SYSTEMS
C 16/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* QR DECOMPOSITION OF A ROWWISE STORED DENSE RECTANGULAR MATRIX A USING
* HOUSEHOLDER TRANSFORMATIONS WITH PIVOTING.
*
* PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX A.
*  II  M  NUMBER OF ROWS OF THE MATRIX A.
*  RU  A(M,N)  RECTANGULAR MATRIX STORED IN THE TWO-DIMENSIONAL ARRAY.
*  RI  TOL  TOLERANCE FOR PSEUDORANK DETERMINATION.
*  RA  E(N)  AUXILIARY VECTOR.
*  RA  V(N)  AUXILIARY VECTOR.
*  IO  NM  ASSUMED RANK OF THE MATRIX A. 
*  IO  NP  PSEUDORANK OF THE MATRIX A.
*
* SUBPROGRAMS USED :
*  S   UXDRMN  EUCLIDEAN NORM OF A PART OF THE ROWWISE STORED
*         RECTANGULAR MATRIX COLUMN.
*
* METHOD :
* P.A.BUSSINGER, G.H.GOLUB : LINEAR LEAST SQUARES SOLUTION BY
* HOUSEHOLDER TRANSFORMATION. NUMER. MATH. 7 (1965) 269-276.
*
      SUBROUTINE UXECHP(M,N,A,TOL,E,V,NM,NP)
      INTEGER M,N,NM,NP
      $FLOAT A(M,N),TOL,E(NM),V(NM)
      INTEGER I
      $FLOAT SMIN,SMAX,SMINPR,SMAXPR,S1,S2,C1,C2
      $FLOAT ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
      E(1)=ONE
      V(1)=ONE
      SMAX=ABS(A(1,1))
      SMIN=SMAX
      IF (SMAX.EQ.ZERO) THEN
        NP=0
      ELSE
        NP=1
   10   CONTINUE
        IF (NP.LT.NM) THEN
          I=NP+1
          CALL UXVCON(NP,E,A(1,I),A(I,I),SMIN,SMINPR,S1,C1,2)
          CALL UXVCON(NP,V,A(1,I),A(I,I),SMAX,SMAXPR,S2,C2,1)
          IF (SMAXPR*TOL.LE.SMINPR) THEN
            DO 20 I = 1,NP
              E(I)=S1*E(I)
              V(I)=S2*V(I)
   20       CONTINUE
            NP=NP+1
            E(NP)=C1
            V(NP)=C2
            SMIN=SMINPR
            SMAX=SMAXPR
            GO TO 10
          END IF
        END IF
      END IF
      RETURN
      END