* SUBROUTINE UXEGCB                ALL SYSTEMS                94/12/01
C PORTABILITY : ALL SYSTEMS
C 94/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A COMPLEX DENSE GENERAL
* MATRIX A USING THE FACTORIZATION A=P*L*U OBTAINED BY THE SUBROUTINE
* UXEGCF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRICES AR AND AI.
*  II  ND  DECLARED ORDER OF THE MATRICES AR AND AI.
*  RU  AR(ND,N)  A REAL PART OF THE FACTORIZATION A=P*L*U OF A
*         COMPLEX DENSE GENERAL MATRIX A OBTAINED BY THE SUBROUTINE
*         UXEGCF.
*  RU  AI(ND,N)  AN IMAGINARY PART OF THE FACTORIZATION A=P*L*U OF A
*         COMPLEX DENSE GENERAL MATRIX A OBTAINED BY THE SUBROUTINE
*         UXEGCF.
*  RI  BR(N)  ON INPUT A REAL PART OF A RIGHT HAND SIDE VECTOR.
*         ON OUTPUT A REAL PART OF A SOLUTION VECTOR.
*  RI  BI(N)  ON INPUT AN IMAGINARY PART OF A RIGHT HAND SIDE VECTOR.
*         ON OUTPUT AN IMAGINARY PART OF A SOLUTION VECTOR.
*  IO  IP(N)  A PERMUTATION FIELD WHICH CONTAINS AN INFORMATION ABOUT
*         THE PERMUTATION MATRIX P.
*
* METHOD :
* C.B.MOLER, ALGORITHM 423, LINEAR EQUATION SOLVER, C.A.C.M. 15 (1972),
* P. 274.
*
      SUBROUTINE UXEGCB (N, ND, AR, AI, BR, BI, IP)
      INTEGER N,ND,IP(N),NM1,K,KP1,M,I,KB,KM1
      $FLOAT AR(ND,N),AI(ND,N),BR(N),BI(N),TR,TI,PRODR,PRODI,DEN
      IF (N .EQ. 1) GO TO 50
      NM1 = N - 1
      DO 20 K = 1,NM1
        KP1 = K + 1
        M = IP(K)
        TR = BR(M)
        TI = BI(M)
        BR(M) = BR(K)
        BI(M) = BI(K)
        BR(K) = TR
        BI(K) = TI
        DO 10 I = KP1,N
          PRODR=AR(I,K)*TR-AI(I,K)*TI
          PRODI=AI(I,K)*TR+AR(I,K)*TI
          BR(I) = BR(I) + PRODR
          BI(I) = BI(I) + PRODI
 10       CONTINUE
 20     CONTINUE
      DO 40 KB = 1,NM1
        KM1 = N - KB
        K = KM1 + 1
        DEN=AR(K,K)*AR(K,K)+AI(K,K)*AI(K,K)
        PRODR=BR(K)*AR(K,K)+BI(K)*AI(K,K)
        PRODI=BI(K)*AR(K,K)-BR(K)*AI(K,K)
        BR(K)=PRODR/DEN
        BI(K)=PRODI/DEN
        TR = -BR(K)
        TI = -BI(K)
        DO 30 I = 1,KM1
          PRODR=AR(I,K)*TR-AI(I,K)*TI
          PRODI=AI(I,K)*TR+AR(I,K)*TI
          BR(I) = BR(I) + PRODR
          BI(I) = BI(I) + PRODI
 30       CONTINUE
 40     CONTINUE
 50     CONTINUE
        DEN=AR(1,1)*AR(1,1)+AI(1,1)*AI(1,1)
        PRODR=BR(1)*AR(1,1)+BI(1)*AI(1,1)
        PRODI=BI(1)*AR(1,1)-BR(1)*AI(1,1)
        BR(1)=PRODR/DEN
        BI(1)=PRODI/DEN
      RETURN
      END
