* SUBROUTINE UXLSIN                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE VARIABLE METRIC DIRECTION DETERMINATION.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  II  MF  NUMBER OF QUASI-NEWTON STEPS.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  GM(NF*MF)  SET OF GRADIENT DIFFERENCES.
*  RU  GR(MF*(MF+1)/2)  MATRIX TRANS(GM)*XM STORED IN PACKED FORM.
*  RO  HR(MF*(MF+1)/2)  MATRIX IN UPDATE REPRESENTATION.
*  RA  U1(MF)  AUXILIARY VECTOR.
*  RA  U2(NF)  AUXILIARY VECTOR.
*  RI  U(NF)  INPUT VECTOR.
*  RO  V(NF)  OUTPUT VECTOR.
*  RI  PAR  SCALING PARAMETER.
*  II  MET  METHOD SELECTION. MET=1-BFGS UPDATE. MET=4-RANK ONE UPDATE.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION.
*  S   UXDCMM  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXDCMD  VECTOR MULTIPLIED BY A COLUMNWISE STORED RECTANGULAR
*         MATRIX AND ADDED TO A SCALED VECTOR.
*  S   UXDRMM  VECTOR MULTIPLIED BY A ROWWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXVDIR  VECTOR AUGMENTED BY THE SCALED VECTOR.
*  S   UXVMUL  VECTOR MULTIPLIED BY A DIAGONAL MATRIX STORED AS VECTOR.
*  S   UXVNEG  COPYING OF A VECTOR WITH THE CHANGE OF THE SIGN.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
* METHOD :
* LIMITED STORAGE VARIABLE METRIC METHODS.
*
      SUBROUTINE UXLSIN(NF,MF,MF1,XM,GM,GR,HR,XM1,GM1,HR1,U1,U2,U,V,
     & PAR)
      INTEGER NF,MF,MF1
      $FLOAT XM(NF*MF),GM(NF*MF),GR(MF*(MF+1)/2),HR(MF*(MF+1)/2),
     & XM1(NF*MF1),GM1(NF*MF1),HR1(MF1*(MF1+1)/2),U1(NF),U2(NF),
     & U(NF),V(NF),PAR
      IF (MF.LE.0) THEN
      CALL UXVCOP(NF,U,V)
      ELSE
      CALL UXDRMM(NF,MF,XM,U,U1)
      CALL UXDRMM(NF,MF,GM,U,U2)
      CALL UXVSCL(MF,PAR,U2,U2)
      CALL UXDPRB(MF,GR,U1,-1)
      CALL UXDSMD(MF,HR,U1,-1.0$P 0,U2,U2)
      CALL UXDPRB(MF,GR,U2, 1)
      CALL UXDCMM(NF,MF,GM,U1,V)
      CALL UXDCMD(NF,MF,XM,U2,-PAR,V,V)
      CALL UXVDIR(NF,PAR,U,V,V)
      ENDIF
      IF (MF1.GT.0) THEN
      CALL UXDRMM(NF,MF1,GM1,U,U1)
      CALL UXDRMD(NF,MF1,XM1,U,-1.0$P 0,U1,U1)
      CALL UXDPRB(MF1,HR1,U1,0)
      CALL UXDCMM(NF,MF1,GM1,U1,U2)
      CALL UXDCMD(NF,MF1,XM1,U1,-1.0$P 0,U2,U2)
      CALL UXVDIR(NF,-1.0$P 0,U2,V,V)
      ENDIF
      RETURN
      END
