* SUBROUTINE UXPPGF                ALL SYSTEMS                10/12/01
C PORTABILITY : ALL SYSTEMS
C 10/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* CHOLESKI DECOMPOSITION OF A TRIDIAGONAL MATRIX.
*
* PARAMETERS :
*  II  N  ORDER OF THE TRIDIAGONAL MATRIX T.
*  RU  D(N)  ON INPUT DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T.
*         ON OUTPUT ELEMENTS OF THE DIAGONAL MATRIX D IN THE
*         DECOMPOSITION T=L*D*TRANS(L).
*  RU  E(N)  ON INPUT THE FIRST SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
*         MATRIX T. ON OUTPUT THE FIRST SUBDIAGONAL ELEMENTS OF THE
*         LOWER TRIANGULAR MATRIX L IN THE DECOMPOSITION T=L*D*TRANS(L).
*  RU  F(N)  ON INPUT THE SECOND SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
*         MATRIX T. ON OUTPUT THE SECOND SUBDIAGONAL ELEMENTS OF THE
*         LOWER TRIANGULAR MATRIX L IN THE DECOMPOSITION T=L*D*TRANS(L).
*  IO  INF  AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS SUFFICIENTLY POSITIVE DEFINITE AND E=0. IF
*         INF<0 THEN A IS NOT SUFFICIENTLY POSITIVE DEFINITE AND E>0. IF
*         INF>0 THEN A IS INDEFINITE AND INF IS AN INDEX OF THE
*         MOST NEGATIVE DIAGONAL ELEMENT USED IN THE FACTORIZATION
*         PROCESS.
*  RU  ALF  ON INPUT A DESIRED TOLERANCE FOR POSITIVE DEFINITENESS. ON
*         OUTPUT THE MOST NEGATIVE DIAGONAL ELEMENT USED IN THE
*         FACTORIZATION PROCESS (IF INF>0).
*  RO  TAU  MAXIMUM DIAGONAL ELEMENT OF THE MATRIX E.
*
      SUBROUTINE UXPPGF(N,D,E,F,INF,ALF,TAU)
      INTEGER N,INF
      $FLOAT  D(N),E(N),F(N),ALF,TAU
      $FLOAT  DI,EI,FI,BET,GAM,DEL,TOL
      INTEGER I,L
      $FLOAT ZERO,ONE,TWO
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0,TWO=2.0$P 0)
      L=0
      INF=0
      TOL=ALF
C
C     ESTIMATION OF THE MATRIX NORM
C
      ALF=ZERO
      GAM=ZERO
      TAU=ZERO
      BET=ABS(D(N))
      DO 1 I=1,N-1
      BET=MAX(BET,ABS(D(I)))
      GAM=MAX(GAM,ABS(E(I)))
      IF (I.LT.N-1) GAM=MAX(GAM,ABS(F(I)))
    1 CONTINUE
      BET=MAX(TOL,TWO*BET,GAM/MAX(ONE,$DBLE(N-1)))
      DEL=TOL*MAX(BET,ONE)
      DO 2 I=1,N
      EI=D(I)
      IF (ALF.GT.EI) THEN
      ALF=EI
      L=I
      ENDIF
      GAM=ZERO
      IF (I.LT.N) GAM=ABS(E(I))
      IF (I.LT.N-1) GAM=MAX(GAM,ABS(F(I)))
      GAM=GAM*GAM
      DI=MAX(ABS(EI),GAM/BET,DEL)
      IF (TAU.LT.DI-EI) THEN
      TAU=DI-EI
      INF=-1
      ENDIF
C
C     GAUSSIAN ELIMINATION
C
      D(I)=DI
      IF (I.LT.N-1) THEN
      EI=E(I)
      FI=F(I)
      E(I)=EI/DI
      F(I)=FI/DI
      D(I+1)=D(I+1)-E(I)*EI
      E(I+1)=E(I+1)-E(I)*FI
      D(I+2)=D(I+2)-F(I)*FI
      ELSE IF (I.LT.N) THEN
      EI=E(I)
      E(I)=EI/DI
      D(I+1)=D(I+1)-E(I)*EI
      ENDIF
    2 CONTINUE
      IF (L.GT.0.AND.ABS(ALF).GT.DEL) INF = L
      RETURN
      END
