* SUBROUTINE UXPSMC                ALL SYSTEMS                10/12/01
C PORTABILITY : ALL SYSTEMS
C 10/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* CHOLESKI DECOMPOSITION OF A TRIDIAGONAL MATRIX.
*
* PARAMETERS :
*  II  N  ORDER OF THE TRIDIAGONAL MATRIX T.
*  RU  D(N)  ON INPUT DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T.
*         ON OUTPUT ELEMENTS OF THE DIAGONAL MATRIX D IN THE
*         DECOMPOSITION T=L*D*TRANS(L).
*  RU  E(N)  ON INPUT THE FIRST SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
*         MATRIX T. ON OUTPUT THE FIRST SUBDIAGONAL ELEMENTS OF THE
*         LOWER TRIANGULAR MATRIX L IN THE DECOMPOSITION T=L*D*TRANS(L).
*  RU  F(N)  ON INPUT THE SECOND SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
*         MATRIX T. ON OUTPUT THE SECOND SUBDIAGONAL ELEMENTS OF THE
*         LOWER TRIANGULAR MATRIX L IN THE DECOMPOSITION T=L*D*TRANS(L).
*  II  JOB  OPTION. JOB=0-NO CORRECTION. JOB=1-FIRST SUPERDIAGONAL
*         ELEMENTS ARE MULTIPLIED BY 2/3 AND SECOND SUPERDIAGONAL
*         ELEMENTS ARE MULTIPLIUED BY 1/3. JOB=2-FIRST SUPERDIAGONAL
*         ELEMENTS ARE MULTIPLIED BY 2/3 AND SECOND SUPERDIAGONAL
*         ELEMENTS ARE MULTIPLIUED BY 1/3 ONLY IF THE CORRESPONDING
*         SUBDETERMINANTS ARE NEGATIVE. JOB=3-SUPERDIAGONAL ELEMENTS
*         ARE MODIFIED IN SUCH A WAY THAT THE NEGAGIVE SUBDETERMINANTS
*         ARE CHANGED TO BE ZEROS.
*
      SUBROUTINE UXPSMC(N,D,E,F,JOB)
      INTEGER N,JOB
      $FLOAT  D(N),E(N),F(N)
      INTEGER I
      $FLOAT D1,D2,DET
      IF (JOB.LE.0) THEN
      ELSE IF (JOB.EQ.1) THEN
      DO 1 I=1,N-2
      E(I)=2.0$P 0*E(I)/3.0$P 0
      E(I+1)=2.0$P 0*E(I+1)/3.0$P 0
      F(I)=F(I)/3.0$P 0
    1 CONTINUE
      ELSE IF (JOB.EQ.2) THEN
      DO 2 I=1,N-2
      D1=D(I)*D(I+1)-2.25$P 0*E(I)**2
      D2=D(I+1)*D(I+2)-2.25$P 0*E(I+1)**2
      DET=D(I+1)*(D(I)*D(I+2)-9.0D0*F(I)**2)-2.25D 0*(D(I)*E(I+1)**2+
     & D(I+2)*E(I)**2-6.0D 0*E(I)*E(I+1)*F(I))
      IF (D1.LT.0.0$P 0.OR.D2.LT.0.0$P 0.OR.DET.LT.0.0$P 0) THEN
      E(I)=2.0$P 0*E(I)/3.0$P 0
      E(I+1)=2.0$P 0*E(I+1)/3.0$P 0
      F(I)=F(I)/3.0$P 0
      ENDIF
    2 CONTINUE
      ELSE IF (JOB.EQ.3) THEN
      DO 3 I=1,N-1
      IF (D(I)*D(I+1).LT.2.25$P 0*E(I)**2) THEN
      E(I)=2.0$P 0*SIGN(SQRT(ABS(D(I)*D(I+1))),E(I))/3.0$P 0
      ENDIF
    3 CONTINUE
      DO 4 I=1,N-2
      DET=D(I+1)*(D(I)*D(I+2)-9.0D0*F(I)**2)-2.25D 0*(D(I)*E(I+1)**2+
     & D(I+2)*E(I)**2-6.0D 0*E(I)*E(I+1)*F(I))
      IF (DET.LT.0.0$P 0) THEN
      F(I)=7.5$P-1*E(I)*E(I+1)/D(I+1)
      ENDIF
    4 CONTINUE
      ENDIF
      RETURN
      END
