* SUBROUTINE UXSGDB                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DA : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH SPARSE NONSYMMETRIC
* MATRIX USING THE FACTORIZATION A=P*L*U*Q OBTAINED BY THE SUBROUTINE
* UXSGDF.
*
* PARAMETERS :
*  RA  XMEM(MMAX)  AUXILIARY REAL STORAGE.
*  IA  IMEM(MMAX)  AUXILIARY INTEGER STORAGE.
*  IO  PMEM(7,2)  STATUS OF AUXILIARY REAL AND INTEGER STORAGE
*  IO  LU(2,2)  POINTERS OF LU FACTOR IN AN AUXILIARY STORAGE.
*  IO  ERROR  ERROR MESSAGE.
*  RI  B(N)  RIGHT HAND SIDE VECTOR.
*  RO  X(N)  SOLUTION VECTOR.
*  RA  W(N)  AUXILIARY VECTOR.
*  RI  ABSPT  ABSOLUTE TOLERANCE FOR A PIVOT.
*  II  VL  DESIRED VECTOR LENGTH FOR BACKWARD SOLVE (USUALLY 16-64).
*  II  JOB  OPTION. JOB=0 - SOLUTION WITH THE ORINAL MATRIX. JOB=1 -
*         SOLUTION WITH THE TRANSPOSED MATRIX.
*
      SUBROUTINE UXSGDB(XMEM,IMEM,PMEM,N,LU,ERROR,B,X,W,ABSPT,VL,JOB)
      INTEGER IMEM(*),PMEM(7,2),N,LU(2,2),ERROR,VL,NPIV,JOB
      LOGICAL TRANS
      $FLOAT XMEM(*),ABSPT,B(N),X(N),W(2*N)
      $FLOAT ABSPT2
      INTEGER AN,ABLKS,LUNS,ANZOFF,ANZ2,ANZDIA,LUPP,AOFFXP,
     & PERMP, LUBLPP, ABLKPP, AOFFPP, AOFFIP, VL2
      IF (LU (1,1) .LE. 0 .OR. (LU(2,1)-LU(1,1)).LT.8 .OR.
     & LU (1,2) .LE. 0 .OR. (LU(2,2)-LU(1,2)).LT.0 ) GOTO 999
      TRANS=.FALSE.
      IF (JOB.GT.0) TRANS=.TRUE.
      CALL UXIMCO(PMEM,ERROR)
      IF (ERROR .NE. 0) GOTO 998
      IF (ABSPT .LE. 0.0) THEN
      ABSPT2 = 0.0
      ELSE
      ABSPT2 = ABSPT
      ENDIF
      CALL UXSGF7(IMEM(LU(1,1)),AN,ABLKS,LUNS,ANZOFF,ANZ2,
     & ANZDIA,ERROR,LUPP,AOFFXP,PERMP,LUBLPP,
     & ABLKPP,AOFFPP,AOFFIP,LU)
      IF (ERROR .NE. 0) GOTO 999
      VL2 = MAX (1, VL)
      VL2 = MIN (VL2, AN)
      IF (TRANS) THEN
      CALL UXSGD2(NPIV,B,XMEM(LU(1,2)),IMEM (LU(1,1)),W,X,
     & IMEM(LU(1,1)-1+ABLKPP),IMEM(LU(1,1)-1+AOFFPP),
     & IMEM(LU(1,1)-1+AOFFIP),IMEM(LU(1,1)-1+LUBLPP),
     & IMEM(LU(1,1)-1+LUPP),IMEM(LU(1,1)-1+PERMP),
     & IMEM(LU(1,1)-1+PERMP+AN),
     & XMEM(LU(1,2)-1+AOFFXP),AN,ABLKS,LUNS,ANZOFF,
     & ABSPT2,W(AN+1),VL2)
      ELSE
      CALL UXSGD1(NPIV,B,XMEM(LU(1,2)),IMEM(LU(1,1)),W,X,
     & IMEM(LU(1,1)-1+ABLKPP),IMEM(LU(1,1)-1+AOFFPP),
     & IMEM(LU(1,1)-1+AOFFIP),IMEM(LU(1,1)-1+LUBLPP),
     & IMEM(LU(1,1)-1+LUPP),IMEM(LU(1,1)-1+PERMP),
     & IMEM(LU(1,1)-1+PERMP+AN),
     & XMEM(LU(1,2)-1+AOFFXP),AN,ABLKS,LUNS,ANZOFF,
     & ABSPT2,W(AN+1),VL2)
      ENDIF
      RETURN
C
C     ERROR CONDITIONS
C
  999 ERROR = 67
      CALL UOERR1('UXSGDB',ERROR)
      RETURN
  998 ERROR = 64
      CALL UOERR1('UXSGDB',ERROR)
      RETURN
      END
