* SUBROUTINE UXSGFF                 ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* MAIN BODY OF THE FACTORIZATION.
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  M  NUMBER OF PROCESSED COLUMNS THAT FORM THE COLUMS OF THE MATRIX
*         THAT DESCRIBES THE ACTIVE SET. M <= N.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  P  NUMBER OF ACTIVE BOUNDS.
*  IO  PTC(N) POINTER VECTOR OF THE COLUMNS OF THE SPARSE MATRIX.
*  IO  LENC(N) LENGTHS OF THE COLUMNS.
*  IA  LENCH(N) LENGTHS OF THE ACTIVE PARTS OF COLUMNS.
*  IO  CNR(NZ) CONTAINS THE INDICES OF THE COLUMN AND ROW SCHEME
*              OF THE PROCESSED MATRIX.
*  IA  PTR(N) POINTER VECTOR OF THE ROWS OF THE SPARSE MATRIX.
*  IU  LENR(N) ON INPUT: LENGTHS OF THE ROWS IN THE SCHEMA INR/AI.
*              ON OUTPUT:NOT USED.
*  IA  PORDR(N) INDEX VECTOR OF THE ROW ORDERING.
*  IO  ORDR(N) ORDER OF THE ROWS THAT WILL BE FURTHER UPDATED.
*  IA  PORDC(N) INDEX VECTOR OF THE COLUMN ORDERING.
*  IO  ORDC(N) ORDER OF THE COLUMNS THAT WILL BE FURTHER UPDATED.
*  RI  AU(NAU) ELEMENTS OF THE PROCESSED SPARSE MATRIX.
*  RO  DIAG(N)  DIAGONAL OF THE FACTOR U.
*  IO  IBGR  BEGINNING OF THE ROW INDICES IN CNR.
*  IO  IENDLU  END OF THE COLUMN INDICES PART IN CNR
*  RI  STFL  STABILITY FLAG.
*  RI  GSFL  BOUND ON THE ELEMENT GROWTH.
*  RI  DRFL  DROPPING PARAMETER.
*  RO  MMFL  MAXIMUM MULTIPLIER FOUND.
*  II  JOB   JOB SPECIFICATION:
*            HIGHER VALUE INCLUDES POSSIBILITIES OF THE LOWER VALUES.
*            JOB=0   :   STANDARD VALUE.
*            JOB=1   :   DROPPING OF SMALL ELEMENTS.
*            JOB=2   :   ELEMENT GROWTH SAFEGUARDING.
*            JOB=3   :   SETTING OF THE OWN PARAMETERS GSFL,DRFL.
*  IO  IFL  ERROR FLAG INTERPRETED IN UXSGFD.
*
      SUBROUTINE UXSGFF(N,M,NZ,NAU,P,PTC,LENC,LENCH,CNR,PTR,LENR,PORDR,
     *                ORDR,PORDC,ORDC,AU,DIAG,IBGR,IENDLU,STFL,
     *                GSFL,DRFL,MMFL,JOB,IFL)
      INTEGER N,M,NZ,NAU,P,IBGR,IENDLU
      INTEGER PTC(N),PTR(N),LENC(N),LENR(N),LENCH(N)
      INTEGER PORDR(*),ORDR(N),PORDC(*),ORDC(N)
      INTEGER CNR(NZ)
      INTEGER CLSTRT,ISTRT,JSTRT,KSTRT,ISTOP,JSTOP,KSTOP
      INTEGER JOB,IFL
      $FLOAT AU(NAU)
      $FLOAT DIAG(N)
      $FLOAT STFL,GRFL,GSFL,AMX,TAU,AI,DRFL
      $FLOAT MMFL,MIBEST,ABEST,EPS
      INTEGER I,J,K,L,J1,J2,J3,K1,K2,L1,L2,L3,JOB1,JOB2
      INTEGER IBEST,JBEST,IFL1,IFL2,NUMPIV,IQCMPR,IE,IDIST
      PARAMETER (EPS=1.0$P-20)
C     INITIALIZATION
C
C
C     MAXIMAL PIVOT USED
C
      MMFL=0
C
C     INITIALIZE THE ERROR FLAG
C
      IFL=0
C
C     IQCMPR DENOTES THAT THE GARBAGE COLLECTION WAS NOW DONE
C
      IQCMPR=0
C
C     IFL2 IS USED DURING INSERTING OF THE ROW FILL-IN
C
      IFL2=0
C
C     PARAMETER SETTING REGULAR GARBAGE COLLECTION
C
      IE=2*IENDLU
C
C     INTEGER FROM THE SET (1/STFL,1) INFLUENCING THE STABILITY
C
      TAU=.2
C
C     PARAMETER SHOWING THE DISTANCE IN THE ARRAY CNR
C     OF THE LINES THAT CAN BE LENGTHENED
C
      IDIST=0
      DO 100 I=1,N
      DIAG(I)=0
100   CONTINUE
C
C     MAXIMUM ELEMENT OF THE MATRIX IS COMPUTED IF NEEDED
C
      IF(JOB.GT.1) THEN
                   K=0
                   CALL UXSGFG(N,M,K,NAU,PTC,LENC,LENCH,ORDC,AU,AMX)
      END IF
      IF(JOB.LE.2) THEN
                   DRFL=1.0$P-10
      END IF
C
C     MAIN CYCLE
C
      DO 3700 K=1,M
C
C     FIND THE PIVOT
C
C
C     SET THE JOB SPECIFICATION FOR QPIV
C
      JOB1=MIN(210,M+1-PORDC(1))
      JOB2=MIN(210,N-P+1-PORDR(1))
      CALL UXSGFP(N,NZ,NAU,PTC,LENC,LENCH,CNR,PTR,LENR,PORDR,
     *          ORDR,PORDC,ORDC,AU,STFL,MIBEST,IBEST,JBEST,
     *          TAU,JOB1,JOB2,IFL1)
C
C     TEST WHETHER PIVOT WAS FOUND
C
      IF(IFL1.EQ.1) THEN
                    IFL=2
                    RETURN
      END IF
C
C     FIND THE MAXIMUM MULTIPLIER
C
      MMFL=MAX(MIBEST,MMFL)
C
C     UPDATE PIVOT COLUMN
C     STORE ABEST IN THE ARRAY DIAG - ON OUTPUT
C     THIS WILL BE DIAGONAL OF THE FACTOR U
C
      ISTRT=PTC(JBEST)
      ISTOP=ISTRT+LENC(JBEST)-1
      ISTRT=ISTRT+LENCH(JBEST)
      DO 200 I=ISTRT,ISTOP
      L=CNR(I)
      IF(L.EQ.IBEST) THEN
                     DIAG(IBEST)=AU(I)
                     ABEST=AU(I)
                     CNR(I)=CNR(ISTOP)
                     AU(I)=AU(ISTOP)
                     CNR(ISTOP)=0
                     GO TO 300
      END IF
200   CONTINUE
300   CONTINUE
C
C     LENGTH UPDATE
C
      LENC(JBEST)=LENC(JBEST)-1
      ISTOP=ISTOP-1
C
C     AU UPDATE IN THE PIVOT COLUMN AND ROW LENGTHS UPDATE
C     FOR THE NON-FILL-IN POSITIONS
C
      DO 400 I=ISTRT,ISTOP
      L=CNR(I)
      L1=PTR(L)
      DO 350 J=L1,L1+LENR(L)-1
      IF(CNR(J).EQ.JBEST) THEN
                          CNR(J)=CNR(L1)
                          CNR(L1)=0
                          PTR(L)=L1+1
      END IF
350   CONTINUE
      AU(I)=AU(I)/ABEST
      LENR(CNR(I))=LENR(CNR(I))-1
400   CONTINUE
C
C     NUMBER OF NONZERO ELEMENTS IN THE ARRAY PIVOT
C
      NUMPIV=ISTOP-ISTRT+1
C
C     EXCLUDE COLUMN JBEST FROM THE ORDERED LIST
C
      DO 700 L2=PORDC(NUMPIV+1),PORDC(NUMPIV+2)-1
      IF(ORDC(L2).EQ.JBEST) THEN
           K1=L2
           DO 500 L3=1,NUMPIV+1
           PORDC(L3)=PORDC(L3) +1
500        CONTINUE
           DO 600 L3=NUMPIV+1,1,-1
           L1=PORDC(L3)-1
           ORDC(K1)=ORDC(L1)
           K1=L1
600        CONTINUE
           ORDC(PORDC(1)-1)=ORDC(K)
           ORDC(K)=JBEST
           GO TO 800
      END IF
700   CONTINUE
800   CONTINUE
C
C     EXCLUDE ROW IBEST FROM THE ROW LIST
C
      IF(LENR(IBEST).EQ.N) THEN
        K2=N
      ELSE
        K2=PORDR(LENR(IBEST)+1)-1
      END IF
C
      DO 1100 L2=PORDR(LENR(IBEST)),K2
      IF(ORDR(L2).EQ.IBEST) THEN
           K1=L2
           DO 900 L3=1,LENR(IBEST)
           PORDR(L3)=PORDR(L3) +1
900        CONTINUE
           DO 1000 L3=LENR(IBEST),1,-1
           L1=PORDR(L3)-1
           ORDR(K1)=ORDR(L1)
           K1=L1
1000       CONTINUE
           ORDR(PORDR(1)-1)=ORDR(K)
           ORDR(K)=IBEST
           GO TO 1200
      END IF
1100  CONTINUE
1200  CONTINUE
C
C     COLUMN CYCLE
C
      CLSTRT=PTR(IBEST)
      DO 2200 I=0,LENR(IBEST)-1
C
C     TEMPORARY COLUMN
C
      L=CNR(I+CLSTRT)
C
C     COLUMN JBEST CANNOT BE USED
C
      IF(L.EQ.JBEST) GO TO 2200
C
C     FILL THE ARRAY PIVOT
C
      DO 1300 J=ISTRT,ISTOP
      DIAG(CNR(J))=AU(J)+EPS
1300  CONTINUE
C
C     NUMBER OF NONZERO ELEMENTS IN THE ARRAY PIVOT
C
      NUMPIV=ISTOP-ISTRT+1
C
C     FIND THE COLUMN MULTIPLIER AI
C
      JSTRT=PTC(L)
      JSTOP=JSTRT+LENC(L)-1
      JSTRT=JSTRT+LENCH(L)
      DO 1400 J=JSTRT,JSTOP
      L1=CNR(J)
      IF(L1.EQ.IBEST) THEN
                      AI=AU(J)
                      CNR(J)=CNR(JSTRT)
                      AU(J)=AU(JSTRT)
                      CNR(JSTRT)=IBEST
                      AU(JSTRT)=AI
                      LENCH(L)=LENCH(L)+1
                      JSTRT=JSTRT+1
                      GO TO 1500
      END IF
1400  CONTINUE
1500  CONTINUE
C
C     UPDATE THE COLUMN L
C
      DO 1600 J=JSTRT,JSTOP
      L1=CNR(J)
      IF(DIAG(L1).NE.0) THEN
                         AU(J)=AU(J)-AI*DIAG(L1)
                         IF(AU(J).EQ.0) AU(J)=EPS
                         DIAG(L1)=0
                         NUMPIV=NUMPIV-1
      END IF
1600  CONTINUE
C
C     DROPPING BLOCK IS OPTIONALLY TRAVERSED
C
      IF(JOB.GT.0.AND.JSTRT.LE.JSTOP) THEN
                   J=JSTRT
                   K1=JSTRT
1650               L1=CNR(J)
                   IF(ABS(AU(J)).GT.DRFL) THEN
                                          AU(K1)=AU(J)
                                          CNR(K1)=L1
                                          K1=K1+1
                                          ELSE
                                          L2=PTR(L1)
                                          DO 1660 J1=L2,L2+N
                                          IF(CNR(J1).EQ.L) THEN
                                                     CNR(J1)=CNR(L2)
                                                     CNR(L2)=0
                                                     PTR(L1)=L2+1
                                                     GO TO 1670
                                          END IF
1660                                      CONTINUE
1670                                      CONTINUE
                                          LENR(L1)=LENR(L1)-1
                   END IF
      J=J+1
      IF(J.LE.JSTOP) GO TO 1650
      LENC(L)=LENC(L)-JSTOP
      JSTOP=K1-1
      DO 333 J1=K1,JSTOP
        CNR(J1)=0
 333  CONTINUE
      LENC(L)=LENC(L)+JSTOP
      END IF
C
C     ADD FILL-IN INTO THE COLUMN L AND ROW L1
C
      LENC(L)=LENC(L)+NUMPIV
      DO 2100 J=0,ISTOP-ISTRT
      L1=CNR(ISTRT+J)
1700  IF(DIAG(L1).NE.0) THEN
          IF(CNR(JSTOP+1).EQ.0) THEN
                                IQCMPR=0
                                JSTOP=JSTOP+1
                                CNR(JSTOP)=L1
                                AU(JSTOP)=-AI*DIAG(L1)
                                DIAG(L1)=0
                                NUMPIV=NUMPIV-1
                                IENDLU=MAX(IENDLU,JSTOP)
                                ELSE
                                J1=JSTOP-PTC(L)+1
              IF((IENDLU+J1+NUMPIV).LT.MIN(IBGR,NAU)) THEN
                                       DO 1800 J2=1,J1
                                       J3=PTC(L)+J2-1
                                       CNR(J2+IENDLU)=CNR(J3)
                                       AU(J2+IENDLU)=AU(J3)
                                       CNR(J3)=0
1800                                   CONTINUE
                                       PTC(L)=IENDLU+1
                                       JSTOP=IENDLU+J1
                                       GO TO 1700
                                                     ELSE
                                       IF(IQCMPR.EQ.1) THEN
                                           IF(NAU.LT.IBGR) THEN
                                                           IFL=4
                                                           ELSE
                                                           IFL=5
                                           END IF
                                           RETURN
                                       END IF
                                       IQCMPR=1
                                       JSTOP=JSTOP-PTC(L)
                                       LENC(L)=LENC(L)-NUMPIV
                                       CALL UXSGFC(N,M,NZ,NAU,PTC,LENC,
     *                                 CNR,PTR,LENR,AU,IBGR,IENDLU)
                                       JSTOP=PTC(L)+JSTOP
                                       ISTRT=PTC(JBEST)+LENCH(JBEST)
                                       ISTOP=PTC(JBEST)+LENC(JBEST)-1
                                       LENC(L)=LENC(L)+NUMPIV
                                       CLSTRT=PTR(IBEST)
                                       GO TO 1700
              END IF
          END IF
C
C     INSERT THE ROW FILL-IN
C
          IBGR=IBGR-IDIST
          KSTRT=PTR(L1)
          KSTOP=KSTRT+LENR(L1)-1
C
C     TEST OF FREE SPACE AT THE BEGINNING AND THE END
C     OF THE ROW LIST
C
1900      IF(CNR(KSTOP+1).EQ.0) THEN
                                IQCMPR=0
                                KSTOP=KSTOP+1
                                CNR(KSTOP)=L
                                LENR(L1)=LENR(L1)+1
                                ELSE
             IF(CNR(KSTRT-1).EQ.0) THEN
                                IQCMPR=0
                                KSTRT=KSTRT-1
                                PTR(L1)=KSTRT
                                CNR(KSTRT)=L
                                LENR(L1)=LENR(L1)+1
                                IBGR=MIN(IBGR,KSTRT)
                                   ELSE
                                K1=KSTOP-KSTRT+1
                                IF((IENDLU+NUMPIV+K1+1).LT.IBGR)    THEN
                                        DO 2000 K2=1,K1
                                        CNR(IBGR-K2)=CNR(KSTRT+K2-1)
                                        CNR(KSTRT+K2-1)=0
2000                                    CONTINUE
                                        KSTOP=IBGR-1
                                        KSTRT=IBGR-K1
                                        PTR(L1)=KSTRT
                                        IFL2=1
                                        GO TO 1900
                                                                  ELSE
                                    IF(IQCMPR.EQ.1) THEN
                                                    IFL=3
                                                    RETURN
                                                    ELSE
                                         LENC(L)=LENC(L)-NUMPIV
                                         JSTOP=JSTOP-PTC(L)
                                         CALL UXSGFC(N,M,NZ,NAU,PTC,LENC
     *                                   ,CNR,PTR,LENR,AU,IBGR,IENDLU)
                                         KSTRT=PTR(L1)
                                         LENC(L)=LENC(L)+NUMPIV
                                         ISTRT=PTC(JBEST)+LENCH(JBEST)
                                         ISTOP=PTC(JBEST)+LENC(JBEST)-1
                                         CLSTRT=PTR(IBEST)
                                         KSTOP=KSTRT+LENR(L1)-1
                                         JSTOP=JSTOP+PTC(L)
                                         IQCMPR=1
                                    END IF
                                    GO TO 1900
                                END IF
             END IF
          END IF
      END IF
      IF(IFL2.EQ.1) THEN
                    IFL2=0
                    IBGR=KSTRT
      END IF
2100  CONTINUE
C
C     NEW END OF COLUMN INDICES
C
      IENDLU=MAX(IENDLU,PTC(L)+LENC(L)-1)
C
C     END OF THE CYCLE OF THE PIVOTAL COLUMN
C
2200  CONTINUE
C
C     POINTER UPDATE TO EXPRESS THAT THESE INDICES WERE USED
C
      PTC(JBEST)=-PTC(JBEST)
      PTR(IBEST)=0
C
C     UPDATE OF THE ARRAYS PORDC,ORDC,PORDR,ORDR
C
      CALL UXSGFU(N,M,P,K,PORDC,ORDC,LENC,LENCH,PORDR,ORDR,LENR)
C
C     TEST WHETHER ANY OTHER COLUMN WAS ZEROED
C     THIS SITUATION CAN BE FACED !
C
      IF(PORDC(1).GT.(K+1)) THEN
                            IFL=2
                            RETURN
      END IF
C
C     GROWTH FACTOR TEST IF DEMANDED
C
      IF(JOB.GT.1) THEN
                   CALL UXSGFG(N,M,K,NAU,PTC,LENC,LENCH,ORDC,AU,GRFL)
                   GRFL=GRFL/AMX
                   IF(GRFL.GE.GSFL) THEN
                                    IFL=1
                                    RETURN
                   END IF
      END IF
C
C     TEST IF GARBAGE COLLECTION IS TO BE DONE
C
      IF(IE.LT.IENDLU) THEN
                       CALL UXSGFC(N,M,NZ,NAU,PTC,LENC,CNR,PTR,
     *                            LENR,AU,IBGR,IENDLU)
                       IE=2*IENDLU
      END IF
3700  CONTINUE
C
C     COMPLETING OF THE ARRAY ORDC
C     ITS POSITIONS M+1,...,N WILL BE FILLED WITH
C     THE CORRESPONDING INDICES
C
      DO 3800 I=M+1,N
      ORDC(I)=I
3800  CONTINUE
      RETURN
      END
