* SUBROUTINE UXSGFL                 ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A SPARSE TRIANGULAR
* FACTOR L : L*ORDR(X)=ORDL(RHS)
*
* PARAMETERS :
*  II  N  NUMBER OF ROWS OF THE SPARSE MATRIX
*  II  M  NUMBER OF PROCESSED COLUMNS THAT FORM THE COLUMS OF THE MATRIX
*         THAT DESCRIBES THE ACTIVE SET. M <= N.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX.
*  II  NEL  NUMBER OF TRIPLES IN ARRAYS (LI,LJ,CM).
*  II  NUP  LENGTH OF THE ARRAYS LI,LJ,CM.
*  II  PTL(N) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(N) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  II  CNR(NZ) CONTAINS THE INDICES OF THE FACTORS.
*  II  ORDR(N) ROW PERMUTATION.
*  II  ORDL(N) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  II  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  II  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  RI  AU(NZ) NUMERICAL VALUES OF THE FACTORS.
*  RU  RHS(N) ON INPUT : RIGHT HAND SIDE OF THE SYSTEM. ON OUTPUT
*             SYSTEM SOLUTION.
*  RA  X(N)  AUXILIARY VECTOR.
*
* METHOD :
* BACK SUBSTITUTION
*
      SUBROUTINE UXSGFL(N,NZ,NAU,NEL,NUP,PTL,LENL,CNR,ORDR,ORDL,LI,LJ,
     & CM,AU,RHS,X)
      INTEGER N,NZ,NAU,NEL,NUP
      INTEGER PTL(N),LENL(N),ORDR(N),ORDL(N)
      INTEGER CNR(NZ)
      INTEGER LI(NUP),LJ(NUP)
      $FLOAT CM(NUP)
      $FLOAT AU(NAU)
      $FLOAT RHS(N),X(N)
      INTEGER I,J,K,L
C
C     FTRANL
C
      DO 300 I=1,N
      L=ORDL(I)
            DO 200 J=PTL(I),PTL(I)+LENL(I)-1
            K=CNR(J)
            RHS(K)=RHS(K)-RHS(L)*AU(J)
200         CONTINUE
300   CONTINUE
C
C     L-UPDATES ARE USED
C
      DO 500 I=1,NEL
      RHS(LI(I))=RHS(LI(I))-CM(I)*RHS(LJ(I))
500   CONTINUE
C
C     TRANSFORMATION TO ORDR
C
      DO 370 I=1,N
      X(I)=RHS(ORDR(I))
370   CONTINUE
      RETURN
      END
