* SUBROUTINE UXSGFM                 ALL SYSTEMS                91/12/01
C PORTABILITY : ALL SYSTEMS
C 91/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* MULTIPLICATION Y=LU*X WITH A SPARSE NONSYMMETRIC
* MATRIX A USING THE FACTORIZATION P A Q = L*U.
*
* PARAMETERS:
*  II  N  SIZE OF THE SYSTEM SOLVED.
*  II  NZ SIZE OF THE INDEX ARRAY CNR
*  II  NAU SIZE OF THE ARRAY THAT STORES THE NUMERICAL ELEMENTS OF
*          THE PROCESSED MATRIX - OF THE ARRAY AU.
*  II  NEL  NUMBER OF TRIPLES IN ARRAYS (LI,LJ,CM).
*  II  NUP  DECLARED LENGTH OF THE ARRAYS LI,LJ,CM.
*  II  PTL(N) POINTER VECTOR OF THE COLUMNS OF THE L-FACTOR.
*  II  LENL(N) LENGTHS OF THE COLUMNS OF THE FACTOR L.
*  II  CNR(NZ) ARRAY THAT CONTAINS THE INDICES OF THE FACTORS.
*  II  PTR(N) POINTER VECTOR OF THE ROWS OF THE FACTOR U.
*  II  LENR(N) LENGTHS OF THE ROWS OF THE FACTOR U.
*  II  ORDC(N) COLUMN PERMUTATION OF THE FACTOR U.
*  II  ORDR(N) ROW PERMUTATION OF THE FACTOR U.
*  II  ORDL(N) ROW PERMUTATION OF THE ORIGINAL FACTOR L.
*  II  LI(NUP) ROW INDICES OF THE L-UPDATES.
*  II  LJ(NUP) COLUMN INDICES OF THE L-UPDATES.
*  RI  CM(NUP) UPDATE MULTIPLIERS.
*  RI  AU(NZ) NUMERICAL VALUES OF THE FACTORS.
*  RU  X(N) INPUT VECTOR.
*  RA  Y(N) OUTPUT VECTOR.
*
* METHOD :
* BACK SUBSTITUTION
*
      SUBROUTINE UXSGFM(N,NZ,NAU,NEL,NUP,PTL,LENL,CNR,PTR,
     *                  LENR,ORDC,ORDR,ORDL,LI,LJ,CM,AU,X,Y)
      INTEGER N,NZ,NAU,NEL,NUP
      INTEGER PTL(N),PTR(N),LENL(N),LENR(N),ORDR(N),ORDC(N),ORDL(N)
      INTEGER CNR(NZ)
      INTEGER LI(NUP),LJ(NUP)
      $FLOAT CM(NUP)
      $FLOAT AU(NAU)
      $FLOAT X(N),Y(N)
      INTEGER I,J,K,L,I1
C
      DO 100 I=1,N
        Y(I)=0.0D0
 100  CONTINUE
C
C     BTRANU
C
      DO 800 I=N,1,-1
        I1=ORDR(I)
        DO 700 J=PTR(I1)+1,PTR(I1)+LENR(I1)-1
          K=CNR(J)
          Y(I1)=Y(I1)+AU(J)*X(K)
 700  CONTINUE
        X(I)=Y(I1)/AU(PTR(I1))
 800  CONTINUE
      DO 850 I=1,N
        Y(ORDC(I))=X(I)
 850  CONTINUE
C
C     JUMP LABEL IF JOB.GT.0
C
C
C     L-UPDATES ARE USED
C
      DO 500 I=1,NEL
        Y(LI(I))=Y(LI(I))+CM(I)*X(LJ(I))
 500  CONTINUE
C
C     FTRANL
C
      DO 300 I=1,N
        L=ORDL(I)
        DO 200 J=PTL(I),PTL(I)+LENL(I)-1
          K=CNR(J)
          Y(K)=Y(K)+X(L)*AU(J)
200     CONTINUE
300   CONTINUE
      RETURN
      END
