* SUBROUTINE UXSIB1                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* AUXILIARY SUBROUTINE FOR UXSIBB.
* THIS SUBROUTINE PERFORMS FORWARD ELIMINATION
* USING THE FACTOR U TRANSPOSE STORED IN A/IA AFTER UXSIBF.
*
      SUBROUTINE UXSIB1(N, A, LA, IW, MW, W, MAXFNT, RHS, IW2, NBLK,
     & LATOP)
      INTEGER N, LA, MW, MAXFNT, NBLK, LATOP
      INTEGER IW2(NBLK), APOS
      INTEGER IW(MW)
      DOUBLE PRECISION A(LA), RHS(N), W(MAXFNT), W1, W2
C     N - MUST BE SET TO THE ORDER OF THE MATRIX. NOT ALTERED
C         BY UXSIB1.
C     A - MUST BE SET TO HOLD THE REAL VALUES
C         CORRESPONDING TO THE FACTORS OF DINVERSE AND U.  THIS MUST BE
C         UNCHANGED SINCE THE PRECEDING CALL TO UXSIBF.  NOT ALTERED
C         BY UXSIB1.
C     LA- LENGTH OF ARRAY A.  NOT ALTERED BY UXSIB1.
C     IW- HOLDS THE INTEGER INDEXING
C         INFORMATION FOR THE MATRIX FACTORS IN A.  THIS MUST BE
C         UNCHANGED SINCE THE PRECEDING CALL TO UXSIBF.  NOT ALTERED
C         BY UXSIB1.
C     MW- LENGTH OF ARRAY IW.  NOT ALTERED BY UXSIB1.
C     W - USED
C         AS WORKSPACE BY UXSIB1 TO HOLD THE COMPONENTS OF THE RIGHT
C         HAND SIDES CORRESPONDING TO CURRENT BLOCK PIVOTAL ROWS.
C     MAXFNT- MUST BE SET TO THE LARGEST NUMBER OF
C         VARIABLES IN ANY BLOCK PIVOT ROW.  THIS VALUE WILL HAVE
C         BEEN OUTPUT BY UXSIBF.  NOT ALTERED BY UXSIB1.
C     RHS - ON INPUT,
C         MUST BE SET TO HOLD THE RIGHT HAND SIDES FOR THE EQUATIONS
C         WHICH THE USER DESIRES TO SOLVE.  ON OUTPUT, RHS WILL HOLD
C         THE MODIFIED VECTORS CORRESPONDING TO PERFORMING FORWARD
C         ELIMINATION ON THE RIGHT HAND SIDES.
C     IW2 - NEED NOT BE SET ON ENTRY. ON EXIT IW2(I) (I = 1,NBLK)
C         WILL HOLD POINTERS TO THE
C         BEGINNING OF EACH BLOCK PIVOT IN ARRAY IW.
C     NBLK - NUMBER OF BLOCK PIVOT ROWS. NOT ALTERED BY UXSIB1.
C     LATOP- NEED NOT BE SET ON ENTRY. ON EXIT, IT IS THE POSITION IN
C         A OF THE LAST ENTRY IN THE  FACTORS. IT MUST BE PASSED
C         UNCHANGED TO UXSIB2.
      INTEGER IBLK, IFR, IFRLVL, ILVL, IOVFLO, IPIV, IPOS, IRHS,
     &        IROW, IST,
     &        J, J1, J2, J3, JJ, JPIV, K, K1, K2, K3, LIELL,
     &        NEMIN, NPIV
      INTRINSIC MIN0, IABS
C
C     IFRLVL IS USED TO CONTROL WHETHER DIRECT OR INDIRECT ACCESS IS
C     USED BY UXSIBB.  INDIRECT ACCESS IS EMPLOYED
C     IN FORWARD AND BACK SUBSTITUTION RESPECTIVELY IF THE SIZE OF
C     A BLOCK IS LESS THAN IFRLVL(MIN0(10,NPIV)) AND
C     IFRLVL(10+MIN0(10,NPIV)) RESPECTIVELY, WHERE NPIV IS THE NUMBER
C     OF PIVOTS IN THE BLOCK.
C
      COMMON /UMMAT3/ IOVFLO, NEMIN, IFRLVL(20)
C
C     APOS. RUNNING POINTER TO CURRENT PIVOT POSITION IN ARRAY A.
C     IPOS. RUNNING POINTER TO BEGINNING OF BLOCK PIVOT ROW IN IW.
C
      APOS = 1
      IPOS = 1
      J2 = 0
      IBLK = 0
      NPIV = 0
      DO 140 IROW=1,N
        IF (NPIV.GT.0) GO TO 90
        IBLK = IBLK + 1
        IF (IBLK.GT.NBLK) GO TO 150
        IPOS = J2 + 1
C
C     SET UP POINTER IN PREPARATION FOR BACK SUBSTITUTION.
C
        IW2(IBLK) = IPOS
C
C     IABS(LIELL) IS NUMBER OF VARIABLES (COLUMNS) IN BLOCK PIVOT ROW.
C
        LIELL = -IW(IPOS)
C
C     NPIV IS NUMBER OF PIVOTS (ROWS) IN BLOCK PIVOT.
C
        NPIV = 1
        IF (LIELL.GT.0) GO TO 10
        LIELL = -LIELL
        IPOS = IPOS + 1
        NPIV = IW(IPOS)
   10   J1 = IPOS + 1
        J2 = IPOS + LIELL
        ILVL = MIN0(NPIV,10)
        IF (LIELL.LT.IFRLVL(ILVL)) GO TO 90
C
C     PERFORM OPERATIONS USING DIRECT ADDRESSING.
C     LOAD APPROPRIATE COMPONENTS OF RIGHT HAND SIDES INTO ARRAY W.
C
        IFR = 0
        DO 20 JJ=J1,J2
          J = IABS(IW(JJ)+0)
          IFR = IFR + 1
          W(IFR) = RHS(J)
   20   CONTINUE
C
C     JPIV IS USED AS A FLAG SO THAT IPIV IS INCREMENTED CORRECTLY AFTER
C     THE USE OF A 2 BY 2 PIVOT.
C
        JPIV = 1
        J3 = J1
C
C     PERFORM OPERATIONS.
C
        DO 70 IPIV=1,NPIV
          JPIV = JPIV - 1
          IF (JPIV.EQ.1) GO TO 70
C
C     JUMP IF WE HAVE A 2 BY 2 PIVOT.
C
          IF (IW(J3).LT.0) GO TO 40
C
C     PERFORM FORWARD SUBSTITUTION USING 1 BY 1 PIVOT.
C
          JPIV = 1
          J3 = J3 + 1
          APOS = APOS + 1
          IST = IPIV + 1
          IF (LIELL.LT.IST) GO TO 70
          W1 = W(IPIV)
          K = APOS
          DO 30 J=IST,LIELL
            W(J) = W(J) + A(K)*W1
            K = K + 1
   30     CONTINUE
          APOS = APOS + LIELL - IST + 1
          GO TO 70
C
C     PERFORM OPERATIONS WITH 2 BY 2 PIVOT.
C
   40     JPIV = 2
          J3 = J3 + 2
          APOS = APOS + 2
          IST = IPIV + 2
          IF (LIELL.LT.IST) GO TO 60
          W1 = W(IPIV)
          W2 = W(IPIV+1)
          K1 = APOS
          K2 = APOS + LIELL - IPIV
          DO 50 J=IST,LIELL
            W(J) = W(J) + W1*A(K1) + W2*A(K2)
            K1 = K1 + 1
            K2 = K2 + 1
   50     CONTINUE
   60     APOS = APOS + 2*(LIELL-IST+1) + 1
   70   CONTINUE
C
C     RELOAD W BACK INTO RHS.
C
        IFR = 0
        DO 80 JJ=J1,J2
          J = IABS(IW(JJ)+0)
          IFR = IFR + 1
          RHS(J) = W(IFR)
   80   CONTINUE
        NPIV = 0
        GO TO 140
C
C     PERFORM OPERATIONS USING INDIRECT ADDRESSING.
C
C     JUMP IF WE HAVE A 2 BY 2 PIVOT.
C
   90   IF (IW(J1).LT.0) GO TO 110
C
C     PERFORM FORWARD SUBSTITUTION USING 1 BY 1 PIVOT.
C
        NPIV = NPIV - 1
        APOS = APOS + 1
        J1 = J1 + 1
        IF (J1.GT.J2) GO TO 140
        IRHS = IW(J1-1)
        W1 = RHS(IRHS)
        K = APOS
        DO 100 J=J1,J2
          IRHS = IABS(IW(J)+0)
          RHS(IRHS) = RHS(IRHS) + A(K)*W1
          K = K + 1
  100   CONTINUE
        APOS = APOS + J2 - J1 + 1
        GO TO 140
C
C     PERFORM OPERATIONS WITH 2 BY 2 PIVOT
C
  110   NPIV = NPIV - 2
        J1 = J1 + 2
        APOS = APOS + 2
        IF (J1.GT.J2) GO TO 130
        IRHS = -IW(J1-2)
        W1 = RHS(IRHS)
        IRHS = IW(J1-1)
        W2 = RHS(IRHS)
        K1 = APOS
        K3 = APOS + J2 - J1 + 2
        DO 120 J=J1,J2
          IRHS = IABS(IW(J)+0)
          RHS(IRHS) = RHS(IRHS) + W1*A(K1) + W2*A(K3)
          K1 = K1 + 1
          K3 = K3 + 1
  120   CONTINUE
  130   APOS = APOS + 2*(J2-J1+1) + 1
  140 CONTINUE
  150 LATOP = APOS - 1
      RETURN
      END
