* SUBROUTINE UXSIBF                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* THIS SUBROUTINE COMPUTES THE FACTORISATION OF THE MATRIX INPUT IN
* A,KA,JA USING INFORMATION (IN IW2) FROM UXSIBS.
*
* PARAMETERS :
*  II  N  MUST BE SET TO THE ORDER OF THE MATRIX. IT IS NOT ALTERED.
*  II  M  MUST BE SET TO THE NUMBER OF NON-ZEROS INPUT. IT IS NOT
*         ALTERED.
*  II  KA  ENTRY K (K=1,M) OF KA MUST BE SET TO THE ROW INDEX OF
*         THE NON-ZERO IN A. KA IS UNALTERED BY UXSIBF.
*  II  JA  ENTRY K (K=1,M) OF JA MUST BE SET TO THE COLUMN INDEX OF
*         THE NON-ZERO IN A. JA IS UNALTERED BY UXSIBF.
*  RU  A(LA)  ON EXIT, ENTRIES 1 TO NRLBDU OF A HOLD REAL INFORMATION
*         ON THE FACTORS AND SHOULD BE PASSED UNCHANGED TO UXSIBB.
*  II  LA  LENGTH OF ARRAY A. AN INDICATION OF A SUITABLE VALUE,
*         SUFFICIENT FOR FACTORIZATION OF A DEFINITE MATRIX, WILL
*         HAVE BEEN PROVIDED IN NRLNEC AND NRLTOT BY UXSIBS.
*         IT IS NOT ALTERED BY UXSIBF.
*  IO  IW(MW)  ENTRIES 1 TO NIRBDU HOLD INTEGER INFORMATION ON THE
*         FACTORS AND SHOULD BE PASSED UNCHANGED TO UXSIBB.
*  II  MW  LENGTH OF ARRAY IW. AN INDICATION OF A SUITABLE VALUE WILL
*         HAVE BEEN PROVIDED IN NIRNEC AND NIRTOT BY UXSIBS.
*         IT IS NOT ALTERED BY UXSIBF.
*  IA  IW1(N)  USED AS WORKSPACE BY UXSIBF.
*  II  IW2(N,3)  MUST BE UNCHANGED SINCE THE CALL TO UXSIBS.
*         IT IS NOT ALTERED BY UXSIBF.
*  II  NS   MUST BE UNCHANGED SINCE THE CALL TO UXSIBS.
*         IT IS NOT ALTERED BY UXSIBF.
*  IO  MF  IT IS THE MAXIMUM SIZE OF THE FRONTAL MATRIX GENERATED
*         DURING THE DECOMPOSITION.
*  IO  ITERM  A ZERO VALUE INDICATES NO ERROR HAS BEEN DETECTED.
*
      SUBROUTINE UXSIBF(N, M, KA, JA, A, LA, IW, MW, IW1, IW2, NS,
     & MF, ITERM)
      INTEGER N, M, LA, MW, NS, MF, ITERM
      INTEGER KA(*),JA(*),IW(MW),IW1(N), IW2(N,3)
      DOUBLE PRECISION A(LA)
      INTEGER IERROR, IFRLVL, IOVFLO, LDIAG, LP, MP,
     &        NCMPA, NCMPBI, NCMPBR, NEIG, NEMIN,
     &        NIRADU,  NIRBDU, NIRNEC, NIRTOT, NRLADU,
     &        NRLBDU, NTWO, M1, NRLNEC, NRLTOT, INF
      DOUBLE PRECISION U, OPS
      INTRINSIC MIN0, IABS
      COMMON /UMMAT1/ U, LP, MP, LDIAG
      COMMON /UMMAT2/ OPS, IERROR, NRLTOT, NIRTOT, NRLNEC, NIRNEC,
     & NRLADU, NIRADU, NRLBDU, NIRBDU, NCMPA, NCMPBR, NCMPBI, NTWO,NEIG
      COMMON /UMMAT3/ IOVFLO, NEMIN, IFRLVL(20)
      IF (N.LT.1 .OR. N.GT.IOVFLO) GO TO 70
      IF (M.LT.0) GO TO 100
      IF (MW.LT.M) GO TO 120
      IF (LA.LT.M+N) GO TO 150
C
C     SORT
C
      CALL UXSIF1(N, M, M1, A, LA, KA, JA, IW, MW, IW2, IW1,
     & INF)
      IF (INF.EQ.-3) GO TO 130
      IF (INF.EQ.-4) GO TO 160
C
C     FACTORIZE
C
      CALL UXSIF2(N, M1, A, LA, IW, MW, IW2, IW2(1,3), NS,
     & MF, IW2(1,2), IW1, INF)
      IF (INF.EQ.-3) GO TO 130
      IF (INF.EQ.-4) GO TO 160
      IF (INF.EQ.-5) GO TO 180
      IF (INF.EQ.-6) GO TO 200
C
C     WARNING MESSAGE
C
      IF (INF.EQ.3) CALL UOERR1('UXSIBF',72)
      RETURN
C
C     ERROR RETURNS
C
   70 ITERM = -61
      CALL UOERR1('UXSIBF',61)
      RETURN
  100 ITERM = -62
      CALL UOERR1('UXSIBF',62)
      RETURN
  120 IERROR = M+3*N+1
      GO TO 140
  130 IERROR = 2*M+3*N+1
  140 ITERM = -65
      CALL UOERR1('UXSIBF',65)
      RETURN
  150 IERROR = M+N
  160 CALL UOERR1('UXSIBF',66)
      ITERM=-66
      RETURN
  180 CALL UOERR1('UXSIBF',70)
      ITERM=-70
      RETURN
  200 CALL UOERR1('UXSIBF',71)
      ITERM=-71
      RETURN
      END
