* SUBROUTINE UXSIIB                ALL SYSTEMS                96/12/01
C PORTABILITY : ALL SYSTEMS
C 96/12/01 DR : ORIGINAL VERSION
*
* PURPOSE :
* BACK SUBSTITUTION AFTER INCOMPLETE BUNCH-PARLETT FACTORIZATION OF
* A SYMMETRIC INDEFINITE SPARSE MATRIX A OBTAINED BY THE SUBROUTINE
* UXSIIF.
*
* PARAMETERS :
*  II  N  MATRIX DIMENSION.
*  II  M  NUMBER OF MATRIX NONZERO ELEMENTS.
*  RU  A(M)  NONZERO ELEMENTS OF THE MATRIX A.
*  II  IA(N+1)  ROW POINTERS OF THE MATRIX A.
*  II  JA(M)  COLUMN INDICES OF THE MATRIX A.
*  RO  X(N)  SOLUTION VECTOR.
*  RI  B(N)  RIGHT HAND SIDE.
*  IO  XDEG(N)  INDICATION WHETHER 1x1 OR 2x2 PIVOT WAS CHOSEN.
*         XDEG(I)=0: 1x1 PIVOT WAS CHOSEN. XDEG(R)=S AND XDEG(S)=R:
*         2x2 PIVOT WAS CHOSEN.
*  IO  PERM(N)  PERMUTATION VECTOR.
*  IO  PERMC(N)  INVERSE PERMUTATION VECTOR.
*
      SUBROUTINE UXSIIB(N,M,A,IA,JA,X,B,XDEG,PERM,INVP,Y,COLR,COLS)
      INTEGER N,M,IA(N+1),JA(M),PERM(N),INVP(N),XDEG(N)
      $FLOAT A(M),B(N),X(N),Y(N)
      $FLOAT TEMP,SUM,DDET,DIAG,NDIAG,SUM1,SUM2
      $FLOAT COLR(N),COLS(N)
      INTEGER I,IND,JSTRT,JSTOP,J,K,L,II,IIND,JJSTRT,JJSTOP
      INTEGER JJ,KK,LL,I55,IND1,IND2,JSTRT1,JSTOP1,J1,K1,L1
      INTEGER II1,IIND1,JJSTRT1,JJSTOP1,JJ1,KK1,LL1,JSTRT2,JSTOP2
      INTEGER J2,K2,L2,II2,IIND2,JJSTRT2,JJSTOP2,JJ2,KK2,LL2,I9
      $FLOAT ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
C
C     SYSTEM L*Y=B IS SOLVED. LOWER TRIANGULAR MATRIX L*D
C     FROM INCOMPLETE BUNCH-PARLETT FACTORIZATION OF SPARSE SYMMETRIC
C     INDEFINITE MATRIX MUST BE TRANSFORMED INTO L BEFORE SOLVING
C     SYSTEM.
C
      DO 12 I=1,N
         Y(I)=B(I)
 12   CONTINUE
      I=1
C
C     MAIN LOOP FOR SOLVING SYSTEM L*Y=B
C
 10   CONTINUE
      IF (I.LE.N) THEN
C
C     NUMBER OF COLUMN WHICH IS NOW IN I-TH COLUMN
C
      IND=PERM(I)
      JSTRT=IA(IND)+1
      JSTOP=IA(IND+1)-1
      IF (XDEG(IND).EQ.0) THEN
C
C     1x1 PIVOT WAS FOUND IN (ORIGINAL) COLUMN IND
C
      DIAG=A(IA(IND))
      TEMP=ONE/DIAG
C
C     IND-TH ELEMENT OF SOLUTION
C
      X(IND)=Y(IND)
C
C     MULTIPLES OF X(IND) ARE SUBTRACTED FROM RELATED ELEMENTS
C     OF RIGHT HAND SIDE
C
      DO 200 J=JSTRT,JSTOP
      K=JA(J)
      L=INVP(K)
      IF (L.GT.I) THEN
      Y(PERM(L))=Y(PERM(L))-(A(J)*TEMP*X(IND))
      END IF
 200  CONTINUE
      IF (I.LE.IND) THEN
C
C     SHORTER COLUMN IS IN I-TH COLUMN, NEXT RELATED ELEMENTS
C     ARE USED WHEN SUBTRACTION FROM RIGHT HAND SIDE IS DONE
C
      DO 300 II=(I+1),N
      IIND=PERM(II)
      JJSTRT=IA(IIND)+1
      JJSTOP=IA(IIND+1)-1
      DO 400 JJ=JJSTRT,JJSTOP
      KK=JA(JJ)
      LL=INVP(KK)
      IF (KK.EQ.IND) THEN
      Y(PERM(II))=Y(PERM(II))-(A(JJ)*TEMP*X(IND))
      END IF
  400 CONTINUE
  300 CONTINUE
      END IF
      I=I+1
      GO TO 10
      ELSE IF (XDEG(IND).NE.0) THEN
C
C     2x2 PIVOT WAS FOUND IN (ORIGINAL) COLUMN IND
C
C     WORKING ARRAYS WHERE COLUMNS WITH 2x2 PIVOT WILL BE SAVED
C     ARE INITIALIZED
C
      DO 55 I55=1,N
      COLR(I55)=ZERO
      COLS(I55)=ZERO
 55   CONTINUE
      IND1=IND
      IND2=XDEG(IND)
      JSTRT1=IA(IND1)
      JSTOP1=IA(IND1+1)-1
C
C     ELEMENTS FROM IND1-TH COLUMN ARE SAVED INTO WORKING ARRAY COLR(N)
C
      DO 801 J1=JSTRT1,JSTOP1
      K1=JA(J1)
      L1=INVP(K1)
      IF (L1.GE.I) THEN
      COLR(L1)=A(J1)
      END IF
  801 CONTINUE
      IF (I.LE.IND1) THEN
C
C     SHORTER COLUMN WAS IN I-TH COLUMN, NEXT RELATED ELEMENTS ARE ADDED
C     INTO WORKING ARRAY COLR(N)
C
      DO 301 II1=(I+1),N
      IIND1=PERM(II1)
      JJSTRT1=IA(IIND1)+1
      JJSTOP1=IA(IIND1+1)-1
      DO 401 JJ1=JJSTRT1,JJSTOP1
      KK1=JA(JJ1)
      LL1=INVP(KK1)
      IF (KK1.EQ.IND1) THEN
      COLR(II1)=A(JJ1)
      END IF
 401  CONTINUE
 301  CONTINUE
      END IF
C
C     ORIGINAL IND2-TH COLUMN IS USED FOR 2x2 PIVOT, WORKING ARRAY
C     COLS(N) IS FILLED
C
      JSTRT2=IA(IND2)
      JSTOP2=IA(IND2+1)-1
      DO 802 J2=JSTRT2,JSTOP2
      K2=JA(J2)
      L2=INVP(K2)
      IF (L2.GE.I) THEN
      COLS(L2)=A(J2)
      END IF
  802 CONTINUE
      IF ((I+1).LE.IND2) THEN
C
C     SHORTER COLUMN WAS IN IND2-TH COLUMN, REALTED ELEMENTS ARE
C     ADDED INTO WORKING ARRAY COLR(N)
C
      DO 302 II2=(I+2),N
      IIND2=PERM(II2)
      JJSTRT2=IA(IIND2)+1
      JJSTOP2=IA(IIND2+1)-1
      DO 402 JJ2=JJSTRT2,JJSTOP2
      KK2=JA(JJ2)
      LL2=INVP(KK2)
      IF (KK2.EQ.IND2) THEN
      COLS(II2)=A(JJ2)
      END IF
 402  CONTINUE
 302  CONTINUE
      END IF
C
C     IND1-TH AND IND2-TH ELEMENTS OF SOLUTION
C
      X(IND1)=Y(IND1)
      X(IND2)=Y(IND2)
C
C     DETERMINANT OF 2x2 PIVOT IS COMPUTED
C
      DDET=(COLR(I)*COLS(I+1))-(COLR(I+1)**2)
C
C     MULTIPLES OF Y(IND1) AND Y(IND2) ARE SUBTRACTED FROM RELATED
C     RIGHT HAND SIDES. IN COLR(N),COLS(N) ARE SAVED ELEMENTS OF
C     MATRIX D*L, BEFORE SUBTRACTION RELATED ELEMNTS OF L ARE
C     COMPUTED
C
      DO 900 I9=(I+2),N
      IF ((COLR(I9).NE.ZERO).OR.( COLS(I9).NE.ZERO)) THEN
      Y(PERM(I9))=Y(PERM(I9))-((1./DDET)*((COLR(I9)*
     &            COLS(I+1))-(COLS(I9)*COLR(I+1)))*X(IND1))
      Y(PERM(I9))=Y(PERM(I9))-((1./DDET)*((COLS(I9)*
     &            COLR(I))-(COLR(I9)*COLR(I+1)))*X(IND2))
      END IF
 900  CONTINUE
      I=I+2
      GO TO 10
      END IF
      END IF
C
C     SYSTEM D*L(T)*X=Y WITH "UPPER" TRIANGULAR MATRIX IS SOLVED
C
      I=N
C
C     MAIN LOOP
C
 20   CONTINUE
      IF (I.GE.1) THEN
C
C     ORIGINAL IND-TH COLUMN IS IN I-TH COLUMN
C
      IND=PERM(I)
      JSTRT=IA(IND)+1
      JSTOP=IA(IND+1)-1
      IF (XDEG(IND).EQ.0) THEN
C
C     1x1 PIVOT WAS CHOSEN IN IND-TH COLUMN
C
      DIAG=A(IA(IND))
      TEMP=ONE/DIAG
      SUM=ZERO
C
C     MULTIPLES OF RELATED ELEMENTS OF SOLUTION X ARE ADDED INTO
C     WORKING VALUE SUM
C
      DO 1200 J=JSTRT,JSTOP
      K=JA(J)
      L=INVP(K)
      IF (L.GT.I) THEN
      SUM=SUM+(A(J)*X(PERM(L)))
      END IF
 1200 CONTINUE
      IF (I.LE.IND) THEN
C
C     SHORTER COLUMN WAS IN I-TH COLUMN, RELATED ELEMNTS ARE ADDED
C     INTO SUM
C
      DO 1300 II=(I+1),N
      IIND=PERM(II)
      JJSTRT=IA(IIND)+1
      JJSTOP=IA(IIND+1)-1
      DO 1400 JJ=JJSTRT,JJSTOP
      KK=JA(JJ)
      LL=INVP(KK)
      IF (KK.EQ.IND) THEN
      SUM=SUM+(A(JJ)*X(PERM(II)))
      END IF
 1400 CONTINUE
 1300 CONTINUE
      END IF
C
C     IND-TH ELEMENT OF SOLUTION X IS COMPUTED
C
      X(IND)=X(IND)-SUM
      X(IND)=X(IND)*TEMP
      I=I-1
      GO TO 20
      ELSE IF (XDEG(IND).NE.0) THEN
C
C     IND-TH COLUMN WAS USED WITH 2x2 PIVOT
C
      IND1=IND
      IND2=XDEG(IND)
      JSTRT1=IA(IND1)+1
      JSTOP1=IA(IND1+1)-1
      SUM1=ZERO
      SUM2=ZERO
      DO 1801 J1=JSTRT1,JSTOP1
      K1=JA(J1)
      L1=INVP(K1)
      IF (K1.EQ.IND2) THEN
      NDIAG=A(J1)
      ELSE IF (L1.GT.I) THEN
      SUM1=SUM1+(A(J1)*X(PERM(L1)))
      END IF
 1801 CONTINUE
      IF (I.LE.IND1) THEN
C
C     SHORTER COLUMN WAS IN I-TH COLUMN, ELEMENTS OF SOLUTION X
C     MULTIPLIED BY RELATED ELEMENTS OF D*L ARE ADDED
C
      DO 1301 II1=(I+1),N
      IIND1=PERM(II1)
      JJSTRT1=IA(IIND1)+1
      JJSTOP1=IA(IIND1+1)-1
      DO 1401 JJ1=JJSTRT1,JJSTOP1
      KK1=JA(JJ1)
      LL1=INVP(KK1)
      IF (KK1.EQ.IND1) THEN
      SUM1=SUM1+(A(JJ1)*X(PERM(II1)))
      END IF
 1401 CONTINUE
 1301 CONTINUE
      END IF
C
C     IND2-TH COLUMN WAS USED FOR 2x2 PIVOT, ELEMENTS OF SOLUTION X
C     MULTIPLIED BY RELATED ELEMENTS OF D*L ARE ADDED
C
      JSTRT2=IA(IND2)+1
      JSTOP2=IA(IND2+1)-1
      DO 1802 J2=JSTRT2,JSTOP2
      K2=JA(J2)
      L2=INVP(K2)
      IF (L2.GT.I) THEN
      SUM2=SUM2+(A(J2)*X(PERM(L2)))
      END IF
 1802 CONTINUE
C
C     SHORTER COLUMN WAS IN (I-1)-TH COLUMN, ELEMENTS MULTIPLIED BY
C     RELATED ELEMENTS OF D*L ARE ADDED
C
      IF ((I-1).LE.IND2) THEN
      DO 1302 II2=(I+1),N
      IIND2=PERM(II2)
      JJSTRT2=IA(IIND2)+1
      JJSTOP2=IA(IIND2+1)-1
      DO 1402 JJ2=JJSTRT2,JJSTOP2
      KK2=JA(JJ2)
      LL2=INVP(KK2)
      IF (KK2.EQ.IND2) THEN
      SUM2=SUM2+(A(JJ2)*X(PERM(II2)))
      END IF
 1402 CONTINUE
 1302 CONTINUE
      END IF
C
C     DETERMINANT OF 2x2 PIVOT IS COMPUTED
C
      DDET=(A(IA(IND1))*A(IA(IND2)))-(NDIAG**2)
C
C     IND1-TH AND IND2-TH ELEMENTS OF SOLUTION ARE COMPUTED
C
      X(IND1)=X(IND1)-SUM1
      X(IND2)=X(IND2)-SUM2
      X(IND2)=((X(IND2)*A(IA(IND1)))-(NDIAG*X(IND1)))*(1./DDET)
      X(IND1)=((A(IA(IND2))*X(IND1))-(X(IND2)*NDIAG))*(1./DDET)
      I=I-2
      GO TO 20
      END IF
      END IF
      RETURN
      END
