* SUBROUTINE UXSIIF                ALL SYSTEMS                96/12/01
C PORTABILITY : ALL SYSTEMS
C 96/12/01 DR : ORIGINAL VERSION
*
* PURPOSE :
* INCOMPLETE BUNCH-PARLETT FACTORIZATION OF A SYMMETRIC INDEFINITE
* SPARSE MATRIX A.
*
* PARAMETERS :
*  II  N  MATRIX DIMENSION.
*  II  M  NUMBER OF MATRIX NONZERO ELEMENTS.
*  RU  A(M)  NONZERO ELEMENTS OF THE MATRIX A.
*  II  IA(N+1)  ROW POINTERS OF THE MATRIX A.
*  II  JA(M)  COLUMN INDICES OF THE MATRIX A.
*  IO  XDEG(N)  INDICATION WHETHER 1x1 OR 2x2 PIVOT WAS CHOSEN.
*         XDEG(I)=0: 1x1 PIVOT WAS CHOSEN. XDEG(R)=S AND XDEG(S)=R:
*         2x2 PIVOT WAS CHOSEN.
*  IO  PERM(N)  PERMUTATION VECTOR.
*  IO  INVP(N)  INVERSE PERMUTATION VECTOR.
*  RO  COL(N)  THE VECTOR WHERE THE COLUMN IN WHICH THE PIVOT WAS
*         CHOSEN IS STORED.
*  RO  COLR(N)  THE VECTOR WHERE THE R-TH COLUMN IS STORED.
*  RO  COLS(N)  THE VECTOR WHERE THE S-TH COLUMN IS STORED.
*
      SUBROUTINE UXSIIF(N,M,A,IA,JA,XDEG,PERM,INVP,COL,COLR,COLS)
      INTEGER N,M,IA(N+1),JA(M),XDEG(N),PERM(N),INVP(N)
      $FLOAT  A(M),COL(N),COLR(N),COLS(N)
      INTEGER I,J,K,INDI,INDK,IR,IS,INDR,INDS,KSTEP,ISEM,INDP,
     & KSTRT,KSTOP,POM,POM1,POM2
      $FLOAT ALFA,RMI1,RMI0
      $FLOAT ZERO,ONE
      PARAMETER (ZERO=0.0$P 0,ONE=1.0$P 0)
C
C     ARRAYS PERM,INVP,DEG ARE INITIALIZED
C
      DO 100 I=1,N
      PERM(I)=I
      INVP(I)=I
      XDEG(I)=0
 100  CONTINUE
      ISEM=1
C
C     ITERATION LOOP
C
      DO 40 KSTEP=1,N
      IF (ISEM.EQ.2) THEN
      ISEM=1
      GO TO 40
      END IF
      ALFA=(ONE+SQRT(17.0$P 0))/18.0$P 0
      RMI1=ZERO
C
C     THE DIAGONAL ELEMENT WITH MAXIMUM ABSOLUTE VALUE IS CHOSEN
C
      DO  10 I=KSTEP,N
      INDI=IA(PERM(I))
      IF (ABS(A(INDI)).GT.RMI1) THEN
      RMI1=ABS(A(INDI))
      INDK=PERM(I)
      INDP=I
      END IF
 10   CONTINUE
      RMI0=ZERO
C
C     THE OFF-DIAGONAL ELEMENT WITH MAXIMUM ABSOLUTE VALUE IS CHOSEN
C
      DO 30 J=KSTEP,N
      KSTRT=IA(PERM(J))+1
      KSTOP=IA(PERM(J)+1)-1
      DO 20 K= KSTRT ,KSTOP
      IF (INVP(JA(K)).GT.KSTEP) THEN
      IF (ABS(A(K)).GT.RMI0) THEN
      RMI0=ABS(A(K))
      INDR=JA(K)
      INDS=PERM(J)
      IR=INVP(INDR)
      IS=J
      END IF
      END IF
 20   CONTINUE
 30   CONTINUE
C
C     DECISION IF 1x1 OR 2x2 PIVOT WILL BE CHOSEN IS DONE
C
      IF (RMI1.GT.(ALFA*RMI0)) THEN
C
C     1x1 PIVOT WAS CHOSEN
C
      XDEG(INDK)=0
      POM=PERM(KSTEP)
      PERM(KSTEP)=INDK
      PERM(INDP)=POM
      INVP(PERM(KSTEP))=KSTEP
      INVP(PERM(INDP))=INDP
      ELSE
C
C     2x2 PIVOT WAS CHOSEN
C
      XDEG(INDR)=INDS
      XDEG(INDS)=INDR
      POM1=PERM(KSTEP)
      POM2=PERM(KSTEP+1)
      PERM(KSTEP)=INDR
      PERM(KSTEP+1)=INDS
      PERM(IR)=POM1
      PERM(IS)=POM2
      INVP(PERM(KSTEP))=KSTEP
      INVP(PERM(KSTEP+1))=KSTEP+1
      INVP(PERM(IR))=IR
      INVP(PERM(IS))=IS
      END IF
      IF (XDEG(PERM(KSTEP)).EQ.0) THEN
C
C     1x1 PIVOT WAS CHOSEN
C
      CALL UXSII1(N,M,IA,JA,A,PERM,INVP,KSTEP,COL)
      CALL UXSII2(N,M,IA,JA,A,PERM,INVP,KSTEP,COL)
      ISEM=1
      ELSE
C
C     2x2 PIVOT WAS CHOSEN
C
      CALL UXSII1(N,M,IA,JA,A,PERM,INVP,KSTEP,COLR)
      CALL UXSII1(N,M,IA,JA,A,PERM,INVP,KSTEP+1,COLS)
      CALL UXSII3(N,M,IA,JA,A,PERM,INVP,KSTEP,COLR,COLS)
      ISEM=2
      END IF
 40   CONTINUE
      END
