* SUBROUTINE UXSIS1                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* AUXILIARY SUBROUTINE FOR UXSIBS.
* SORT PRIOR TO CALLING ANALYSIS ROUTINE UXSIS2.
*
      SUBROUTINE UXSIS1(N, M, KA, JA, IW, LW, IPE, IQ, FLAG, IWFR,
     & INF)
C
C     GIVEN THE POSITIONS OF THE OFF-DIAGONAL NON-ZEROS OF A SYMMETRIC
C     MATRIX, CONSTRUCT THE SPARSITY PATTERN OF THE OFF-DIAGONAL
C     PART OF THE WHOLE MATRIX (UPPER AND LOWER TRIANGULAR PARTS).
C     EITHER ONE OF A PAIR (I,J),(J,I) MAY BE USED TO REPRESENT
C     THE PAIR. DIAGONAL ELEMENTS AND DUPLICATES ARE IGNORED.
C
      INTEGER N, M, LW, IWFR, INF
      INTEGER KA(*),JA(*),IW(LW),FLAG(N)
      INTEGER IPE(N), IQ(N)
C     N MUST BE SET TO THE MATRIX ORDER. IT IS NOT ALTERED.
C     M MUST BE SET TO THE NUMBER OF NON-ZEROS INPUT. IT IS NOT
C     ALTERED.
C     KA(I),I=1,2,...,M MUST BE SET TO THE ROW NUMBERS OF THE
C     NON-ZEROS ON INPUT. IT IS NOT ALTERED UNLESS IT IS EQUIVALENCED
C     TO IW (SEE DESCRIPTION OF IW).
C     JA(I),I=1,2,...,M MUST BE SET TO THE COLUMN NUMBERS OF THE
C     NON-ZEROS ON INPUT. IT IS NOT ALTERED UNLESS IT IS EQUIVALENCED
C     TO IW (SEE DESCRIPTION OF IW).
C     IW NEED NOT BE SET ON INPUT. ON OUTPUT IT CONTAINS LISTS OF
C     COLUMN INDICES, EACH LIST BEING HEADED BY ITS LENGTH.
C     KA(1) MAY BE EQUIVALENCED TO IW(1) AND JA(1) MAY BE
C     EQUIVALENCED TO IW(K), WHERE K.GT.M.
C     LW MUST BE SET TO THE LENGTH OF IW. IT MUST BE AT LEAST 2*M+N.
C     IT IS NOT ALTERED.
C     IPE NEED NOT BE SET ON INPUT. ON OUTPUT IPE(I) POINTS TO THE START
C     OF THE ENTRY IN IW FOR ROW I, OR IS ZERO IF THERE IS NO ENTRY.
C     IQ NEED NOT BE SET.  ON OUTPUT IQ(I),I=1,N CONTAINS THE NUMBER OF
C     OFF-DIAGONAL N0N-ZEROS IN ROW I INCLUDING DUPLICATES.
C     FLAG IS USED FOR WORKSPACE TO HOLD FLAGS TO PERMIT DUPLICATE
C     ENTRIES TO BE IDENTIFIED QUICKLY.
C     IWFR NEED NOT BE SET ON INPUT. ON OUTPUT IT POINTS TO THE FIRST
C     UNUSED LOCATION IN IW.
C
      INTEGER I,ID, IERROR,
     &        J, JN, K, K1, K2,
     &        L, LAST, LDIAG,  LP, LR,
     &        MP,
     &        N1,  NCMPA, NCMPBI, NCMPBR, NDUP,
     &        NEIG, NIRADU,  NIRBDU, NIRNEC, NIRTOT,   NRLADU,
     &        NRLBDU,
     &        NRLNEC, NRLTOT,  NTWO
      DOUBLE PRECISION U, OPS
      COMMON /UMMAT1/ U, LP, MP, LDIAG
      COMMON /UMMAT2/ OPS, IERROR, NRLTOT, NIRTOT, NRLNEC, NIRNEC,
     & NRLADU, NIRADU, NRLBDU, NIRBDU, NCMPA, NCMPBR, NCMPBI, NTWO,NEIG
C
C     INITIALIZE IERROR AND COUNT IN IPE THE
C     NUMBERS OF NON-ZEROS IN THE ROWS AND MOVE ROW AND COLUMN
C     NUMBERS INTO IW.
C
      INF = 0
      IERROR = 0
      DO 10 I=1,N
        IPE(I) = 0
   10 CONTINUE
      LR = M
      IF (M.EQ.0) GO TO 120
      DO 110 K=1,M
        I = KA(K)
        J = JA(K)
        IF (I.LT.J) THEN
        IF (I.GE.1 .AND. J.LE.N) GO TO 90
        ELSE IF (I.GT.J) THEN
        IF (J.GE.1 .AND. I.LE.N) GO TO 90
        ELSE
        IF (I.GE.1 .AND. I.LE.N) GO TO 80
        ENDIF
        IERROR = IERROR + 1
        INF = 1
   80   I = 0
        J = 0
        GO TO 100
   90   IPE(I) = IPE(I) + 1
        IPE(J) = IPE(J) + 1
  100   IW(K) = J
        LR = LR + 1
        IW(LR) = I
  110 CONTINUE
C
C     ACCUMULATE ROW COUNTS TO GET POINTERS TO ROW STARTS IN BOTH IPE
C     AND IQ AND INITIALIZE FLAG
C
  120 IQ(1) = 1
      N1 = N - 1
      IF (N1.LE.0) GO TO 140
      DO 130 I=1,N1
        FLAG(I) = 0
        IF (IPE(I).EQ.0) IPE(I) = -1
        IQ(I+1) = IPE(I) + IQ(I) + 1
        IPE(I) = IQ(I)
  130 CONTINUE
  140 LAST = IPE(N) + IQ(N)
      FLAG(N) = 0
      IF (LR.GE.LAST) GO TO 160
      K1 = LR + 1
      DO 150 K=K1,LAST
        IW(K) = 0
  150 CONTINUE
  160 IPE(N) = IQ(N)
      IWFR = LAST + 1
      IF (M.EQ.0) GO TO 230
C
C     RUN THROUGH PUTTING THE MATRIX ELEMENTS IN THE RIGHT PLACE
C     BUT WITH SIGNS INVERTED. IQ IS USED FOR HOLDING RUNNING POINTERS
C     AND IS LEFT HOLDING POINTERS TO ROW ENDS.
C
      DO 220 K=1,M
        J = IW(K)
        IF (J.LE.0) GO TO 220
        L = K
        IW(K) = 0
        DO 210 ID=1,M
          IF (L.GT.M) GO TO 170
          L = L + M
          GO TO 180
  170     L = L - M
  180     I = IW(L)
          IW(L) = 0
          IF (I.LT.J) GO TO 190
          L = IQ(J) + 1
          IQ(J) = L
          JN = IW(L)
          IW(L) = -I
          GO TO 200
  190     L = IQ(I) + 1
          IQ(I) = L
          JN = IW(L)
          IW(L) = -J
  200     J = JN
          IF (J.LE.0) GO TO 220
  210   CONTINUE
  220 CONTINUE
C
C     RUN THROUGH RESTORING SIGNS, REMOVING DUPLICATES AND SETTING THE
C     MATE OF EACH NON-ZERO.
C     NDUP COUNTS THE NUMBER OF DUPLICATE ELEMENTS.
C
  230 NDUP = 0
      DO 280 I=1,N
        K1 = IPE(I) + 1
        K2 = IQ(I)
        IF (K1.LE.K2) GO TO 240
C ROW IS EMPTY. SET POINTER TO ZERO.
        IPE(I) = 0
        IQ(I) = 0
        GO TO 280
C     ON ENTRY TO THIS LOOP FLAG(J).LT.I FOR J=1,2,...,N. DURING THE
C     LOOP FLAG(J) IS SET TO I IF A NON-ZERO IN COLUMN J IS FOUND.
C     THIS PERMITS DUPLICATES TO BE RECOGNIZED QUICKLY.
C
  240   DO 260 K=K1,K2
          J = -IW(K)
          IF (J.LE.0) GO TO 270
          L = IQ(J) + 1
          IQ(J) = L
          IW(L) = I
          IW(K) = J
          IF (FLAG(J).NE.I) GO TO 250
          NDUP = NDUP + 1
          IW(L) = 0
          IW(K) = 0
  250     FLAG(J) = I
  260   CONTINUE
  270   IQ(I) = IQ(I) - IPE(I)
        IF (NDUP.EQ.0) IW(K1-1) = IQ(I)
  280 CONTINUE
      IF (NDUP.EQ.0) GO TO 310
C
C     COMPRESS IW TO REMOVE DUMMY ENTRIES CAUSED BY DUPLICATES.
C
      IWFR = 1
      DO 300 I=1,N
        K1 = IPE(I) + 1
        IF (K1.EQ.1) GO TO 300
        K2 = IQ(I) + IPE(I)
        L = IWFR
        IPE(I) = IWFR
        IWFR = IWFR + 1
        DO 290 K=K1,K2
          IF (IW(K).EQ.0) GO TO 290
          IW(IWFR) = IW(K)
          IWFR = IWFR + 1
  290   CONTINUE
        IW(L) = IWFR - L - 1
  300 CONTINUE
  310 RETURN
      END
