* SUBROUTINE UXSIS4                 ALL SYSTEMS                95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 DU : ORIGINAL VERSION
*
* PURPOSE :
* AUXILIARY SUBROUTINE FOR UXSIBS.
* SORT PRIOR TO CALLING ANALYSIS ROUTINE UXSIS5.
*
      SUBROUTINE UXSIS4(N, M, KA, JA, PERM, IW, LW, IPE, IQ, FLAG,
     & IWFR, INF)
C
C     GIVEN THE POSITIONS OF THE OFF-DIAGONAL NON-ZEROS OF A SYMMETRIC
C     MATRIX AND A PERMUTATION, CONSTRUCT THE SPARSITY PATTERN
C     OF THE STRICTLY UPPER TRIANGULAR PART OF THE PERMUTED MATRIX.
C     EITHER ONE OF A PAIR (I,J),(J,I) MAY BE USED TO REPRESENT
C     THE PAIR. DIAGONAL ELEMENTS ARE IGNORED. NO CHECK IS MADE
C     FOR DUPLICATE ELEMENTS UNLESS ANY ROW HAS MORE THAN IOVFLO
C     NON-ZEROS, IN WHICH CASE DUPLICATES ARE REMOVED.
C
      INTEGER N, M, LW, IWFR, INF
      INTEGER KA(*),JA(*),IW(LW),PERM(N),FLAG(N)
      INTEGER IPE(N), IQ(N)
C
C     N MUST BE SET TO THE MATRIX ORDER. IT IS NOT ALTERED.
C     M MUST BE SET TO THE NUMBER OF NON-ZEROS INPUT. IT IS NOT
C     ALTERED.
C     KA(I),I=1,2,...,M MUST BE SET TO THE ROW INDICES OF THE
C     NON-ZEROS ON INPUT. IT IS NOT ALTERED UNLESS EQUIVALENCED WITH IW.
C     KA(1) MAY BE EQUIVALENCED WITH IW(1).
C     JA(I),I=1,2,...,M MUST BE SET TO THE COLUMN INDICES OF THE
C     NON-ZEROS ON INPUT. IT IS NOT ALTERED UNLESS EQUIVALENCED
C     WITH IW.JA(1) MAY BE EQUIVELENCED WITH IW(K),K.GT.M.
C     PERM(I) MUST BE SET TO HOLD THE POSITION OF VARIABLE I IN THE
C     PERMUTED ORDER. IT IS NOT ALTERED.
C     IW NEED NOT BE SET ON INPUT. ON OUTPUT IT CONTAINS LISTS OF
C     COLUMN NUMBERS, EACH LIST BEING HEADED BY ITS LENGTH.
C     LW MUST BE SET TO THE LENGTH OF IW. IT MUST BE AT LEAST
C     MAX(M,N+(NO. OF OFF-DIAGONAL NON-ZEROS)). IT IS NOT ALTERED.
C     IPE NEED NOT BE SET ON INPUT. ON OUTPUT IPE(I) POINTS TO THE START OF
C     THE ENTRY IN IW FOR ROW I, OR IS ZERO IF THERE IS NO ENTRY.
C     IQ NEED NOT BE SET. ON OUTPUT IQ(I) CONTAINS THE NUMBER OF
C     OFF-DIAGONAL NON-ZEROS IN ROW I, INCLUDING DUPLICATES.
C     FLAG IS USED FOR WORKSPACE TO HOLD FLAGS TO PERMIT DUPLICATE
C     ENTRIES TO BE IDENTIFIED QUICKLY.
C     IWFR NEED NOT BE SET ON INPUT. ON OUTPUT IT POINTS TO THE FIRST
C     UNUSED LOCATION IN IW.
C     INF NEED NOT BE SET. ON OUTPUT IT IS SET TO 1 IF ANY OUT OF RANGE
C     INDICES ARE FOUND AND TO 0 OTHERWISE.
C
      INTEGER I,ID,IERROR,  IFRLVL,  IN, IOVFLO,
     &        J, JDUMMY, K, K1,K2,
     &        L, LBIG, LDIAG, LEN, LP,
     &        MP,
     &        NCMPA, NCMPBI, NCMPBR,  NEIG, NEMIN,
     &        NIRADU, NIRBDU, NIRNEC, NIRTOT,
     &        NRLADU, NRLBDU,
     &        NRLNEC, NRLTOT, NTWO
      DOUBLE PRECISION U, OPS
      INTRINSIC MAX0
      COMMON /UMMAT1/ U, LP, MP, LDIAG
      COMMON /UMMAT2/ OPS, IERROR, NRLTOT, NIRTOT, NRLNEC, NIRNEC,
     & NRLADU, NIRADU, NRLBDU, NIRBDU, NCMPA, NCMPBR, NCMPBI, NTWO,NEIG
      COMMON /UMMAT3/ IOVFLO, NEMIN, IFRLVL(20)
C
C     INITIALIZE INF, IERROR AND IQ
C
      INF = 0
      IERROR = 0
      DO 10 I=1,N
        IQ(I) = 0
   10 CONTINUE
C
C     COUNT THE NUMBERS OF NON-ZEROS IN THE ROWS, PRINT WARNINGS ABOUT
C     OUT-OF-RANGE INDICES AND TRANSFER GENUINE ROW NUMBERS
C     (NEGATED) INTO IW.
C
      IF (M.EQ.0) GO TO 110
      DO 100 K=1,M
        I = KA(K)
        J = JA(K)
        IW(K) = -I
        IF (I.LT.J) THEN
        IF (I.GE.1 .AND. J.LE.N) GO TO 80
        ELSE IF (I.GT.J) THEN
        IF (J.GE.1 .AND. I.LE.N) GO TO 80
        ELSE
        IW(K) = 0
        IF (I.GE.1 .AND. I.LE.N) GO TO 100
        ENDIF
        IERROR = IERROR + 1
        INF = 1
        IW(K) = 0
        GO TO 100
   80   IF (PERM(J).GT.PERM(I)) GO TO 90
        IQ(J) = IQ(J) + 1
        GO TO 100
   90   IQ(I) = IQ(I) + 1
  100 CONTINUE
C
C     ACCUMULATE ROW COUNTS TO GET POINTERS TO ROW ENDS
C     IN IPE.
C
  110 IWFR = 1
      LBIG = 0
      DO 120 I=1,N
        L = IQ(I)
        LBIG = MAX0(L,LBIG)
        IWFR = IWFR + L
        IPE(I) = IWFR - 1
  120 CONTINUE
C
C     PERFORM IN-PLACE SORT
C
      IF (M.EQ.0) GO TO 250
      DO 160 K=1,M
        I = -IW(K)
        IF (I.LE.0) GO TO 160
        L = K
        IW(K) = 0
        DO 150 ID=1,M
          J = JA(L)
          IF (PERM(I).LT.PERM(J)) GO TO 130
          L = IPE(J)
          IPE(J) = L - 1
          IN = IW(L)
          IW(L) = I
          GO TO 140
  130     L = IPE(I)
          IPE(I) = L - 1
          IN = IW(L)
          IW(L) = J
  140     I = -IN
          IF (I.LE.0) GO TO 160
  150   CONTINUE
  160 CONTINUE
C
C     MAKE ROOM IN IW FOR ROW LENGTHS AND INITIALIZE FLAG.
C
      K = IWFR - 1
      L = K + N
      IWFR = L + 1
      DO 190 I=1,N
        FLAG(I) = 0
        J = N + 1 - I
        LEN = IQ(J)
        IF (LEN.LE.0) GO TO 180
        DO 170 JDUMMY=1,LEN
          IW(L) = IW(K)
          K = K - 1
          L = L - 1
  170   CONTINUE
  180   IPE(J) = L
        L = L - 1
  190 CONTINUE
      IF (LBIG.GE.IOVFLO) GO TO 210
C
C     PLACE ROW LENGTHS IN IW
C
      DO 200 I=1,N
        K = IPE(I)
        IW(K) = IQ(I)
        IF (IQ(I).EQ.0) IPE(I) = 0
  200 CONTINUE
      GO TO 250
C
C     REMOVE DUPLICATE ENTRIES
C
  210 IWFR = 1
      DO 240 I=1,N
        K1 = IPE(I) + 1
        K2 = IPE(I) + IQ(I)
        IF (K1.LE.K2) GO TO 220
        IPE(I) = 0
        GO TO 240
  220   IPE(I) = IWFR
        IWFR = IWFR + 1
        DO 230 K=K1,K2
          J = IW(K)
          IF (FLAG(J).EQ.I) GO TO 230
          IW(IWFR) = J
          IWFR = IWFR + 1
          FLAG(J) = I
  230   CONTINUE
        K = IPE(I)
        IW(K) = IWFR - K - 1
  240 CONTINUE
  250 RETURN
      END
