* SUBROUTINE UXSPCB                ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A SPARSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A+E USING THE FACTORIZATION A+E=L*D*TRANS(L)
* STORED IN THE COMPACT SCHEME. THIS FACTORIZATION CAN BE OBTAINED
* USING THE SUBROUTINE UXSSFN.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  MMAX  LENGTH OF THE PRINCIPAL MATRIX ARRAYS (SL,A).
*  RI  A(MMAX)  FACTORS L,D OF THE FACTORIZATION A+E=L*D*TRANS(L)
*                STORED USING THE COMPACT SCHEME OF STORING.
*  II  PSL(N+1) POINTER ARRAY OF THE FACTORIZED SPARSE MATRIX
*  II  SL(MMAX)  ARRAY OF COLUMN INDICES OF THE FACTORS L AND D
*         STORED USING THE COMPACT SCHEME.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 THEN X:=(A+E)**(-1)*X. IF JOB>0 THEN
*         X:=L**(-1)*X. IF JOB<0 THEN X:=TRANS(L)**(-1)*X.
*
* METHOD :
* BACK SUBSTITUTION
*
      SUBROUTINE UXSPCB(N,MMAX,A,PSL,SL,X,JOB)
      INTEGER N,MMAX
      $FLOAT  A(MMAX),X(N)
      INTEGER PSL(N+1),SL(MMAX),JOB
      INTEGER I,J,IS
C
C     FIRST PHASE
C
      IF (JOB.GE.0) THEN
                    DO 300 I=1,N
                    IS=SL(I)+N+1
                    DO 200 J=PSL(I)+I,PSL(I+1)+I-1
                    X(SL(IS))=X(SL(IS)) - A(J)*X(I)
                    IS=IS+1
200                 CONTINUE
300                 CONTINUE
      END IF
C
C     SECOND PHASE
C
      IF (JOB.EQ.0) THEN
                    DO 400 I=1,N
                    X(I) = X(I)/A(PSL(I)+I-1)
400   CONTINUE
      END IF
C
C     THIRD PHASE
C
      IF (JOB.LE.0) THEN
                    DO 600 I=N,1,-1
                    IS=SL(I)+N+1
                    DO 500 J=PSL(I)+I,PSL(I+1)+I-1
                    X(I)=X(I)-A(J)*X(SL(IS))
                    IS=IS+1
500                 CONTINUE
600                 CONTINUE
      END IF
      RETURN
      END
