* SUBROUTINE UXSPT2                ALL SYSTEMS                94/12/01
C PORTABILITY : ALL SYSTEMS
C 94/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* INCOMPLETE FACTORIZATION A+E=L*D*TRANS(L) OF A SPARSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A+E WHERE D AND E ARE DIAGONAL POSITIVE
* DEFINITE MATRICES AND L IS A LOWER TRIANGULAR MATRIX.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  M NUMBER OF NONZERO ELEMENTS OF THE MATRIX A.
*  II  MMAX DIMENSION OF THE ARRAY AP.
*  RI  A(M)  SPARSE SYMMETRIC (USUALLY POSITIVE DEFINITE) MATRIX.
*  II  IA(N+1)  POINTERS OF THE DIAGONAL ELEMENTS OF A.
*  II  JA(M)  INDICES OF THE NONZERO ELEMENTS OF A.
*  RO  AP(MMAX)  INCOMPLETE FACTORIZATION A+E=L*D*TRANS(L).
*  IO  IP(N+1)  POINTERS OF THE DIAGONAL ELEMENTS OF A.
*  IO  JP(MMAX)  INDICES OF THE NONZERO ELEMENTS OF A.
*  IA  LIST(N+1)  AUXILIARY ARRAY.
*  IA  STRT(N+1)  AUXILIARY ARRAY.
*  IA  WN01(N+1)  AUXILIARY ARRAY.
*  IA  WN02(N+1)  AUXILIARY ARRAY.
*  RA  WR01(N+1)  AUXILIARY ARRAY.
*  RI  DRFL  DROPPING PARAMETER.
*  II  DRTP  TYPE OF DROPPING. DRTP=0-ABSOLUTE DROPPING.
*         DRTP=1-RELATIVE DROPPING.
*  II  LSIZE  MAXIMUM FILL-IN IN A ROW.
*  IO  INF  AN INFORMATION OBTAINED IN THE FACTORIZATION PROCESS. IF
*         INF=0 THEN A IS SUFFICIENTLY POSITIVE DEFINITE AND E=0. IF
*         INF<0 THEN A IS NOT SUFFICIENTLY POSITIVE DEFINITE AND E>0. IF
*         INF>0 THEN A IS INDEFINITE AND INF IS AN INDEX OF THE
*         MOST NEGATIVE DIAGONAL ELEMENT USED IN THE FACTORIZATION
*         PROCESS.
*  RU  ALF  ON INPUT A DESIRED TOLERANCE FOR POSITIVE DEFINITENESS.
*         ON OUTPUT THE MOST NEGATIVE DIAGONAL ELEMENT USED IN THE
*         FACTORIZATION PROCESS (IF INF>0).
*  RO  TAU  MAXIMUM DIAGONAL ELEMENT OF THE MATRIX E.
*
* METHOD :
* P.E.GILL, W.MURRAY : NEWTON TYPE METHODS FOR UNCONSTRAINED AND
* LINEARLY CONSTRAINED OPTIMIZATION, MATH. PROGRAMMING 28 (1974)
* PP. 311-350.
*
      SUBROUTINE UXSPT2(N,M,MMAX,A,IA,JA,AP,IP,JP,LIST,STRT,WN01,WN02,
     & WR01,DRFL,DRTP,LSIZE,INF,ALF,TAU)
      INTEGER N,M,MMAX,IA(N+1),JA(M),IP(N+1),JP(MMAX),LIST(N+1),
     & STRT(N+1),WN01(N+1),WN02(N+1),DRTP,LSIZE,INF
      $FLOAT A(M),AP(MMAX),WR01(N+1),DRFL,ALF,TAU
      INTEGER IDIAG,JNDIAG,INDOLD,COLNEX,COLCUR,NXTROW,ITMP,
     & I,J,K,II,LL,IND,IND2,LENL0,LEN,ISTRT,ISTOP,TOP,NJA,FIRST,MID,LAST
      $FLOAT DIAG,NDIAG,DEL,TEMP,BET,GAM,TMP,DRFLRL,AVGSIZ,PTOL
      PTOL=ALF
      NJA=IA(N+1)-1
      IF(MMAX.LE.NJA) THEN
        INF=-11
        RETURN
      END IF
C
C     INITIALIZE AUXILIARY VECTOR
C
      INF=0
      CALL UXVINS(N,0,WN01)
C
C     MOVE PRECONDITIONER INTO THE FINAL DATA STRUCTURES
C
      CALL UXVICP(N+1,IA,IP)
      CALL UXVICP(NJA,JA,JP)
      CALL UXVCOP(NJA,A,AP)
C
C     GILL-MURRAY MODIFICATION
C
      ALF=0.0$P 0
      BET=0.0$P 0
      GAM=0.0$P 0
      TAU=0.0$P 0
      DO 60 I=1,N
        IDIAG=IA(I)
        TEMP=A(IDIAG)
        BET=MAX(BET,ABS(TEMP))
        DO 50 J=IA(I)+1,IA(I+1)-1
          TEMP=A(J)
          GAM=MAX(GAM,ABS(TEMP))
 50     CONTINUE
 60   CONTINUE
      BET=MAX(PTOL,BET,GAM/$DBLE(N))
      DEL=PTOL*BET
C
C     INTIALIZE WORKING VECTORS
C
      CALL UXVSET(N,0.0$P 0,WR01)
      CALL UXVINS(N,0,LIST)
C
C     START OF THE COLUMN A PROCESSED IN THE NEXT
C     STEP TO BE KEPT DURING SHRINKING OPERATION
C
      TOP=IP(1)
C
C     CYCLE OF COLUMNS FOR THE LEFT-LOOKING ALGORITHM
C
      DO 400 J=1,N
        DIAG=0.0$P 0
        IND2=0
        COLNEX=LIST(J)
200     COLCUR=COLNEX
        IF(COLCUR.NE.0) THEN
          COLNEX=LIST(COLCUR)
          JNDIAG=STRT(COLCUR)
          NDIAG=AP(JNDIAG)
          TEMP=NDIAG*AP(IP(COLCUR))
          DIAG=DIAG+NDIAG*TEMP
          ISTRT=JNDIAG+1
          ISTOP=IP(COLCUR+1)-1
C
C     UPDATE LIST FOR THE ROW NXTROW
C
          IF(ISTRT.LE.ISTOP) THEN
            STRT(COLCUR)=ISTRT
            NXTROW=JP(ISTRT)
            LIST(COLCUR)=LIST(NXTROW)
            LIST(NXTROW)=COLCUR
          END IF
C
C     GATHER UPDATES FROM PREVIOUS COLUMNS
C     MUST BE TESTED WHETHER USING A VECTOR IN NODAL VERSION
C     IS NECESSARY FOR MORE EFFICIENT IMPLEMENTATION SINCE
C     CACHE CAN ADOPT IN THE SAME WAY A PART OF VECTOR A.
C
          DO 300 II=ISTRT,ISTOP
            K=JP(II)
            IF(WN01(K).EQ.0) THEN
              IND2=IND2+1
              WN02(IND2)=K
              WN01(K)=IND2
            END IF
            WR01(K)=WR01(K)-AP(II)*TEMP
 300      CONTINUE
C
C     BACK TO NEXT COLUMN HAVING A NONZERO IN A ROW I
C
          GO TO 200
        END IF
        DIAG=A(IA(J))-DIAG
C
C     ADD CONTRIBUTIONS FROM THE ORIGINAL J-TH COLUMN
C     COMPUTE AVERAGE SIZE OF AN ORIGINAL NONZERO
C
        AVGSIZ=0.0$P 0
        DO 500 II=IA(J)+1,IA(J+1)-1
          K=JA(II)
          TEMP=A(II)
          AVGSIZ=AVGSIZ+ABS(TEMP)
          WR01(K)=WR01(K)+TEMP
          IF(WN01(K).EQ.0) THEN
            IND2=IND2+1
            WN02(IND2)=K
            WN01(K)=IND2
          END IF
 500    CONTINUE
        LEN=IA(J+1)-IA(J)
        AVGSIZ=AVGSIZ+A(IA(J))
        IF(LEN.NE.0) THEN
          AVGSIZ=AVGSIZ/(DBLE(LEN)*DBLE(LEN))
        END IF
        LENL0=IA(J+1)-IA(J)
C
C     DEFINE NEXT UNUSED POSITION
C
        IND=IP(J)+1
C
C     A QUICKER BUT SOMETIMES SLIGHTLY GENEROUS SPACE TEST
C
        IF(IND+IND2.GE.MMAX) THEN
          INF=-12
          RETURN
        END IF
C
C     GET DROP TOLERANCE
C
        IF(DRTP.EQ.1) THEN
          DRFLRL=DRFL*AVGSIZ
        ELSE
          DRFLRL=DRFL
        END IF
C
C     DROP ELEMENTS SMALLER THAN A GIVEN DROP TOLERANCE
C     KEEPING AT MOST LENL0+LSIZE-1 (DIAGONAL ELEMENT IS NOT
C     COUNTED) ELEMENTS IN THE FACTOR COLUMN.
C
        INDOLD=IND
        DO 550 II=1,IND2
          TEMP=WR01(WN02(II))
          WN01(WN02(II))=0
          WR01(WN02(II))=0.0$P 0
          IF(ABS(TEMP).GE.DRFLRL) THEN
            AP(IND)=TEMP
            JP(IND)=WN02(II)
            IND=IND+1
          ELSE
            IND2=IND2-1
          END IF
  550   CONTINUE
        LEN=MIN(IND2,LENL0+LSIZE-1)
C
C     TEST WHETHER WE SHOULD EXCLUDE SOME ELEMENTS FROM THE COLUMN OF L
C
        IF(LEN.LT.IND-INDOLD) THEN
          IF(LEN.NE.0) THEN
            FIRST=INDOLD
            LAST=IND-1
 250        CONTINUE
            MID=FIRST
            TEMP=ABS(AP(MID))
            DO 260 II=FIRST+1,LAST
              IF(ABS(AP(II)).GT.TEMP) THEN
                MID=MID+1
                TMP=AP(MID)
                ITMP=JP(MID)
                AP(MID)=AP(II)
                JP(MID)=JP(II)
                JP(II)=ITMP
                AP(II)=TMP
              END IF
 260        CONTINUE
              TMP=AP(MID)
              ITMP=JP(MID)
              AP(MID)=AP(FIRST)
              JP(MID)=JP(FIRST)
              JP(FIRST)=ITMP
              AP(FIRST)=TMP
              IF(MID.EQ.LEN+INDOLD-1) THEN
                GO TO 255
              ELSEIF(MID.LT.LEN+INDOLD-1) THEN
                FIRST=MID+1
              ELSEIF(MID.GT.LEN+INDOLD-1) THEN
                LAST=MID-1
              END IF
              GO TO 250
          END IF
 255      CONTINUE
        END IF
        IND=INDOLD+LEN
        IP(J+1)=IND
C
C     SORT CONTRIBUTIONS IN WN02 IN ORDER THE SEARCH FOR UPDATING COLUMNS
C     WORK PROPERLY
C
        CALL UXVSR4(LEN,JP(INDOLD),AP(INDOLD))
C
C     RECOMPUTE MAXIMUM ELEMENT IN COLUMN
C
        GAM=0.0$P 0
        DO 350 K=INDOLD,IND-1
          GAM=MAX(GAM,ABS(AP(K)))
 350    CONTINUE
        GAM=GAM*GAM
C
C     TEST DIAGONAL ELEMENT SIZE
C
        IF (ALF.GT.DIAG) THEN
          ALF=DIAG
          LL=K
        ENDIF
        TEMP=MAX(ABS(DIAG),GAM/BET,DEL)
        IF (TAU.LT.TEMP-DIAG) THEN
          TAU=TEMP-DIAG
          INF=-1
        ENDIF
        DIAG=1.0$P 0/TEMP
C
C     INVERSE DIAGONAL ELEMENT INTO DECOMPOSED AP
C
        AP(IP(J))=DIAG
        JP(IP(J))=J
C
C     DEFINE STRT,TOP,IP(J+1)
C
        IF(IND.NE.IP(J)+1) THEN
          STRT(J)=IP(J)+1
          COLNEX=JP(IP(J)+1)
          LIST(J)=LIST(COLNEX)
          LIST(COLNEX)=J
        END IF
        TOP=IP(J+1)
  400 CONTINUE
      IF (LL.GT.0.AND.ABS(ALF).GT.DEL) INF=LL
      RETURN
      END

