* SUBROUTINE UXSPTM                ALL SYSTEMS                96/12/01
C PORTABILITY : ALL SYSTEMS
C 96/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS WITH A SPARSE SYMMETRIC
* POSITIVE DEFINITE MATRIX A+E USING INCOMPLETE FACTORIZATION
* A+E=L*D*TRANS(L) OBTAINED BY THE SUBROUTINE UXSPIF.
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  II  M NUMBER OF NONZERO ELEMENTS OF THE MATRIX A.
*  RI  A(M) INCOMPLETE FACTORIZATION A+E=L*D*TRANS(L) OBTAINED BY THE
*         SUBROUTINE UXSPGF.
*  II  IA(N+1)  POINTERS OF THE DIAGONAL ELEMENTS OF A.
*  II  JA(M)  INDICES OF THE NONZERO ELEMENTS OF A.
*  RU  X(N)  ON INPUT THE RIGHT HAND SIDE OF A SYSTEM OF LINEAR
*         EQUATIONS. ON OUTPUT THE SOLUTION OF A SYSTEM OF LINEAR
*         EQUATIONS.
*  II  JOB  OPTION. IF JOB=0 THEN X:=(A+E)**(-1)*X. IF JOB>0 THEN
*         X:=L**(-1)*X. IF JOB<0 THEN X:=TRANS(L)**(-1)*X.
*
* METHOD :
* BACK SUBSTITUTION
*
      SUBROUTINE UXSPTM(N,M,A,IA,JA,X,Y,JOB)
      INTEGER N,M,IA(N+1),JA(M),JOB
      $FLOAT A(M),X(N),Y(N)
      INTEGER I,J,K
      $FLOAT TEMP,SUM
      IF (JOB.GE.0) THEN
C
C     PHASE 1 : X:=TRANS(L)*X
C
      DO 3 I=1,N
      K=IA(I)
      IF (K.LE.0) GO TO 3
      TEMP=A(K)
      SUM=0.0$P 0
      DO 2 J=IA(I)+1,IA(I+1)-1
      K=JA(J)
      IF (K.GT.0) SUM=SUM+A(J)*X(K)
    2 CONTINUE
      SUM=SUM*TEMP
      X(I)=X(I)+SUM
    3 CONTINUE
      ENDIF
      IF (JOB.EQ.0) THEN
C
C     PHASE 2 : X:=D*X
C
      DO 4 I=1,N
      K=IA(I)
      IF (K.LE.0) GO TO 4
      X(I)=X(I)/A(K)
    4 CONTINUE
      ENDIF
      IF (JOB.LE.0) THEN
C
C     PHASE 3 : X:=L*X
C
      CALL UXVSET(N,0.0$P 0,Y)
      DO 6 I=1,N
      K=IA(I)
      IF (K.LE.0) GO TO 6
      TEMP=X(I)*A(K)
      DO 5 J=IA(I)+1,IA(I+1)-1
      K=JA(J)
      IF (K.GT.0) Y(K)=Y(K)+A(J)*TEMP
    5 CONTINUE
      X(I)=X(I)+Y(I)
    6 CONTINUE
      ENDIF
      RETURN
      END
