* SUBROUTINE UXSRD2              ALL SYSTEMS                92/12/01
C PORTABILITY : ALL SYSTEMS
C 92/12/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SPARSE FRONTAL QR SOLVER - MULTIFRONTAL VARIANT.
*
* PARAMETERS :
*  II  N NUMBER OF MATRIX ROWS.
*  II  NRHS NUMBER OF RIGHT HAND SIDES.
*  II  M NUMBER OF MATRIX ROWS.
*  II  NJB DIMENSION OF THE ARRAYS JB.
*  II  NJBR DIMENSION OF THE ARRAY B.
*  II  IA(N+1) POINTER VECTOR TO COLUMNS OF A STORED IN (JB,B).
*  II  IB(M+1) POINTER VECTOR TO ROWS OF A STORED IN (JB,B).
*  IU  JB(NJB) INTEGER ARRAY COVERING INDICES OF MATRIX ELEMENTS,
*        STACK AND FRONTAL MATRIX.
*  RU  B(NJBR) REAL ARRAY COVERING MATRIX ELEMENTS, STACK AND
*        FRONTAL MATRIX.
*  II  IPRMC(N) INVERSE COLUMN PERMUTATION..
*  II  PERM(M) COLUMN PERMUTATION.
*  IA  WN01(N) AUXILIARY ARRAY.
*  IA  WN02(N) AUXILIARY ARRAY.
*  IA  WN03(N) AUXILIARY ARRAY.
*  II  MASK(M) MASKING VECTOR OF THE MATRIX BLOCKS.
*  II  LABEL(NN) POSTORDER OF BLOCKS.
*  IO  IER ERROR FLAG.
*  II  RHS(M*NRHS) RIGHT HAND SIDE VECTOR.
*
      SUBROUTINE UXSRD2(N,NRHS,M,NJB,NJBR,IA,IB,JB,B,IPRMC,PERM,
     *           WN01,WN02,WN03,MASK,IER,RHS)
C
      INTEGER N,NRHS,M,NJB,NJBR,JB(NJB),IPRMC(N),MASK(M)
      INTEGER PERM(N)
      $FLOAT B(NJBR),RHS(M*NRHS)
      INTEGER IB(M+1),IA(N+1)
      INTEGER WN01(N),WN02(N),WN03(N)
      INTEGER IER
C
      INTEGER FTOP,FTOPR,FBT,FBTR,PI,PIVAL,NBNBL,LMM,INDMM
      INTEGER STOP,STOPR,SBT,SBTR
      INTEGER J1,J2,K,NB,MB
      INTEGER RITOP,I,PTR,II,KK
      INTEGER J,WN01IND,PND,QND
C
C DEFINE FRONTAL AND STACK VALUES
C
      IER=0
      N=N+NRHS
      PI=0
      PIVAL=1
      JB(NJB)=10*NJB
C
C SPACE FOR POINTERS TO ROWS AND COLUMNS
C FIRST - ROWS AND THEN - COLUMNS
C
      FTOP=IB(M+1)-1+IB(M+1)+N+1
      RITOP=IB(M+1)-1+IB(M+1)
      JB(RITOP)=FTOP
      FTOPR=IB(M+1)+N+1
      IF(FTOP.GE.(NJB-2)) THEN
        IER=56
        CALL UOERR1('UXSRD2',56)
        RETURN
      END IF
      IF(FTOPR.GE.(NJBR-2)) THEN
        IER=57
        CALL UOERR1('UXSRD2',57)
        RETURN
      END IF
      FBT=NJB-1
      FBTR=NJBR-1
C
C DEFINE WORKING STACK TOP STOP
C DEFINE WORKING REAL STACK TOP STOPR
C DEFINE WORKING REAL STACK BOTTOM
C
      STOP=FTOP
      STOPR=FTOPR
      SBTR=FBTR
      SBT=FBT
      DO 98 I=1,M
        MASK(I)=0
 98   CONTINUE
      DO 99 I=1,N+1
        WN02(I)=0
        WN03(I)=0
 99   CONTINUE
C
C CYCLE OF THE COLUMNS BLOCKS - BLOCKS OF THE SIZE 1
C
      DO 100 II=1,N-NRHS
        PI=PI+1
        CALL UXSRQ2(N,NRHS,M,NJB,NJBR,WN01IND,JB,B,SBT,SBTR,
     *       STOP,STOPR,IA,IB,IPRMC,PERM,
     *       PI,NBNBL,LMM,INDMM,WN01,WN02,WN03,
     *       PND,QND,MASK,IER,RHS)
        IF(IER.NE.0) RETURN
C
C MOVE R AND SET POINTERS
C
        KK=WN01IND
        MB=JB(KK)
        KK=KK+1
        NB=JB(KK)
        KK=KK+1
        PTR=JB(KK)
        KK=KK+1
C
C TEMPORARY SAVE OF THE INDICES
C
        DO 270 J1=1,NB
          WN02(J1)=JB(KK+J1-1)
 270    CONTINUE
        DO 300 J=1,PIVAL
          K=WN03(PI-PIVAL+J)
          IF(K.EQ.0) THEN
            IER=58
            CALL UOERR1('UXSRD2',58)
            RETURN
          END IF
          DO 400 J1=J,NB
            JB(FTOP)=WN02(J1)
            FTOP=FTOP+1
            B(FTOPR)=B(PTR+J1-1)
            FTOPR=FTOPR+1
 400      CONTINUE
          PTR=PTR+NB
          RITOP=RITOP+1
          JB(RITOP)=FTOP
 300    CONTINUE
C
C RESTORE WN02
C
        DO 271 J1=1,NB
          WN02(J1)=0
 271    CONTINUE
C
        STOP=FTOP
        STOPR=FTOPR
        IF(NBNBL.GT.0) THEN
          JB(SBT)=NBNBL
          SBT=SBT-1
          JB(SBT)=LMM+3
          SBT=SBT-1
          JB(SBT)=INDMM
          SBT=SBT-1
          IF(SBT.LE.STOP) THEN
            IER=59
            CALL UOERR1('UXSRD2',59)
            RETURN
          END IF
        END IF
C
C CLEAR WN02,WN03
C
      DO 411 J1=1,PND
        J2=WN01(J1)
        WN01(J1)=0
        WN03(J2)=0
 411  CONTINUE
      DO 412 J1=QND,N
        J2=WN01(J1)
        WN01(J1)=0
        WN03(J2)=0
 412  CONTINUE
 100  CONTINUE
C
C RETURN
C
      N=N-NRHS
      RETURN
      END
