* SUBROUTINE  UXSRSB               ALL SYSTEMS               95/12/01
C PORTABILITY : ALL SYSTEMS
C 95/12/01 LU : ORIGINAL VERSION
*
* PURPOSE : CREATE ROW PERMUTATIONS FOR OBTAINING UPPER BLOCK
* TRIANGULAR MATRIX.
*
*  PARAMETERS :
*  II  N  NUMBER OF COLUMNS OF THE MATRIX.
*  II  M  NUMBER OF NONZEROS MEMBERS IN THE MATRIX.
*  II  IA(M+1)  ROW POINTERS OF THE SPARSE MATRIX.
*  II  JA(M)  COLUMN INDICES OF THE SPARSE MATRIX.
*  II  LA(N)  LENGTHS OF ROWS.
*  IO  IP(N)  PERMUTATION VECTOR.
*  II  NR  NUMBER OF BLOCKS.
*  IA  IW1(N) AUXILIARY VECTOR.
*  IA  IW2(N) AUXILIARY VECTOR.
*  IA  IW3(N) AUXILIARY VECTOR.
*  IA  IW4(N) AUXILIARY VECTOR.
*
      SUBROUTINE UXSRSB(N,M,IA,JA,LA,IP,NR,IW1,IW2,IW3,IW4)
      INTEGER N,M,NR,IA(N+1),JA(M),LA(N),IP(N),IW1(N),IW2(N),IW3(N),
     & IW4(N)
      INTEGER DUMMY,I,I1,I2,ICNT,II,ISN,IST,IST1,IV,IW,J,K,LCNT,NNM1,STP
      ICNT = 0
C
C     NR IS THE NUMBER OF BLOCKS THAT HAVE BEEN FOUND.
C
      NR = 0
      NNM1 = N + N - 1
C
C     INITIALIZATION OF ARRAYS.
C
      DO 20 J = 1,N
      IW3(J) = 0
      IP(J) = LA(J) - 1
   20 CONTINUE
      DO 120 ISN = 1,N
C
C     LOOK FOR A STARTING NODE
C
      IF (IW3(ISN).NE.0) GO TO 120
      IV = ISN
C
C     IST IS THE NUMBER OF NODES ON THE STACK. IT IS THE STACK POINTER.
C
      IST = 1
C
C     PUT NODE IV AT BEGINNING OF STACK.
C
      IW2(IV) = 1
      IW3(IV) = 1
      IW1(N) = IV
C
C     THE BODY OF THIS LOOP PUTS A NEW NODE ON THE STACK OR BACKTRACKS.
C
      DO 110 DUMMY = 1,NNM1
      I1 = IP(IV)
C
C     HAVE ALL EDGES LEAVING NODE IV BEEN SEARCHED.
C
      IF (I1.LT.0) GO TO 60
      I2 = IA(IV) + LA(IV) - 1
      I1 = I2 - I1
C
C     LOOK AT EDGES LEAVING NODE IV UNTIL ONE ENTERS A NEW NODE OR
C     ALL EDGES ARE EXHAUSTED.
C
      DO 50 II = I1,I2
      IW = JA(II)
C
C     HAS NODE IW BEEN ON STACK ALREADY.
C
      IF (IW3(IW).EQ.0) GO TO 100
C
C     UPDATE VALUE OF IW2(IV) IF NECESSARY.
C
   50 IW2(IV) = MIN(IW2(IV),IW2(IW))
C
C     THERE ARE NO MORE EDGES LEAVING NODE IV.
C
      IP(IV) = -1
C
C       NODE IV THE ROOT OF A BLOCK.
C
   60 IF (IW2(IV).LT.IW3(IV)) GO TO 90
C
C     ORDER NODES IN A BLOCK.
C
      NR = NR + 1
      IST1 = N + 1 - IST
      LCNT = ICNT + 1
C
C     PEEL BLOCK OFF THE TOP OF THE STACK STARTING AT THE TOP AND
C     WORKING DOWN TO THE ROOT OF THE BLOCK.
C
      DO 70 STP = IST1,N
      IW = IW1(STP)
      IW2(IW) = N + 1
      ICNT = ICNT + 1
      IW3(IW) = ICNT
      IF (IW.EQ.IV) GO TO 80
   70 CONTINUE
   80 IST = N - STP
      IW1(NR) = LCNT
C
C     ARE THERE ANY NODES LEFT ON THE STACK.
C
      IF (IST.NE.0) GO TO 90
C
C     HAVE ALL THE NODES BEEN ORDERED.
C
      IF (ICNT.LT.N) GO TO 120
      GO TO 130
C
C     BACKTRACK TO PREVIOUS NODE ON PATH.
C
   90 IW = IV
      IV = IW4(IV)
C
C     UPDATE VALUE OF IW2(IV) IF NECESSARY.
C
      IW2(IV) = MIN(IW2(IV),IW2(IW))
      GO TO 110
C
C     PUT NEW NODE ON THE STACK.
C
  100 IP(IV) = I2 - II - 1
      IW4(IW) = IV
      IV = IW
      IST = IST + 1
      IW2(IV) = IST
      IW3(IV) = IST
      K = N + 1 - IST
      IW1(K) = IV
  110 CONTINUE
  120 CONTINUE
C
C     PUT PERMUTATION IN THE REQUIRED FORM.
C
  130 DO 140 I = 1,N
      II = IW3(I)
  140 IP(II) = I
      RETURN
      END
