* SUBROUTINE UXSSTM                ALL SYSTEMS                86/11/01
C PORTABILITY : ALL SYSTEMS
C 86/11/01 TU : ORIGINAL VERSION
*
* PURPOSE :
* SPARSE MATRIX GENERATOR
*
* PARAMETERS :
*  II  N ORDER OF THE MATRIX A.
*  IO  M NUMBER OF THE NONZERO ELEMENTS OF THE MATRIX TO BE GENERATED
*  IO  PA(N+1) POINTER VECTOR OF THE OUTPUT MATRIX
*  IO  PD(N+1) POINTER VECTOR OF THE OUTPUT MATRIX, THAT POINTS OUT
*       THE DIAGONAL ELEMENTS
*  IO  SA(MMAX) VECTOR OF THE COLUMN INDICES OF THE OUTPUT MATRIX
*  IO  WN11(N+1) WORK VECTOR
*  RO  A(MMAX)   ELEMENTS OF THE GENERATED MATRIX.
*  RO  X(N)   GENERATED RIGHT-HAND SIDE VECTOR.
*  II  ID     PARAMETER OF THE MATRIX DENSITY
*             ID=0   .......     DIAGONAL MATRIX
*             ID INCREASES ..    DENSITY INCREASES.
*  II  MODE  TYPE OF THE GENERATED MATRIX.
*            =0     GENERAL SPARSE SYMMETRIC MATRIX
*            =1     POSITIVE DEFINITE SPARSE SYMMETRIC MATRIX.
*            =2     BANDED POSITIVE DEFINITE MATRIX WITH THE BANDWITH
*                   WIDTH.
*  II  WIDTH  WIDTH OF THE BAND PROVIDING THAT THE MODE=2.
*  II  MMAX  LENGTH OF THE PRINCIPAL SPARSE MATRIX VECTORS (SA,A).
*
      SUBROUTINE UXSSTM(N,M,PA,PD,SA,WN11,A,X,ID,MODE,WIDTH,MMAX)
      INTEGER N,M,MMAX
      INTEGER WN11(N),PA(N+1),PD(N+1),SA(MMAX)
      INTEGER ID,SUM,NUM,I,J,K,L,NSQR,MODE,WIDTH,K1,LO,KK,LL,LN
      REAL*8 A(MMAX),DM,HM,CK,X(N),RANDOM
      REAL*8 HALF,ZERO
      PARAMETER (HALF = 0.5D0,ZERO = 0.0D0, DM=-10,HM= 10)
      L=1
      NSQR=N*N
      NUM=NSQR*ID/100
      SUM=ZERO
      DO 100 I=1,N
      CK=RANDOM(L,.1D0,1.0D0)
      PA(I)=CK*N
      SUM=SUM+PA(I)
100   CONTINUE
      SUM=MAX(SUM,1)
      CK=$DBLE(NUM)/SUM
      CK=HALF*CK
      DO 200 I=1,N
      PA(I)=PA(I)*CK
200   CONTINUE
      K=1
      SUM=1
      DO 500 I=1,N
      DO 300 J=1,PA(I)-1
      CK=RANDOM(L,.1D0,1.0D0)
      KK=CK
      IF(MODE.EQ.2) THEN
      SA(K)=MIN(WIDTH+I,I+KK*N+1,N)
      ELSE
      SA(K)=MIN(N,I+KK*N+1)
      END IF
      K=K+1
300   CONTINUE
      SA(K)=I
      PA(I)=SUM
      K1=K-PA(I)+1
      CALL HPSORT(K1,SA(SUM))
      LO=SA(SUM)
      LL=SUM+1
      DO 400 L=SUM+1,SUM+K1-1
      LN=SA(L)
      IF(LN.NE.LO) THEN
                   SA(LL)=LN
                   LL=LL+1
      END IF
      LO=LN
400   CONTINUE
      SUM=LL
      K=SUM
500   CONTINUE
      PA(N+1)=SUM
      PD(N+1)=SUM
      DO 700 I=1,N
      DO 600 J=PA(I),PA(I+1)-1
      CK=RANDOM(L,.1D0,1.0D0)
      A(J)=DM+CK*(HM-DM)
      IF(SA(J).EQ.I) THEN
                     IF(MODE.EQ.1.OR.MODE.EQ.2) THEN
                     A(J)=100*ABS(A(J))
                     END IF
                     PD(I)=J
      END IF
600   CONTINUE
700   CONTINUE
      DO 800 I=1,N
      CK=RANDOM(L,.1D0,1.0D0)
      X(I)=(100+N*CK)*RANDOM(L,.1D0,1.0D0)
800   CONTINUE
C
C     SYMMETRIC PART IS TO BE FORMED
C
      LN=PA(2)
      LO=PA(1)
      WN11(1)=LN-LO
      DO 900 I=2,N
      LO=LN
      LN=PA(I+1)
      WN11(I)=LN-LO
900   CONTINUE
      DO 1100 I=1,N
      DO 1000 J=PA(I)+1,PA(I+1)-1
      WN11(SA(J))=WN11(SA(J))+1
1000  CONTINUE
1100  CONTINUE
      SUM=0
      DO 1200 I=2,N
      WN11(I)=WN11(I)+WN11(I-1)
1200  CONTINUE
      M=WN11(N)
      PD(1)=1
      K=SUM
      DO 1400 I=N,1,-1
      SUM=WN11(I)
      PD(I+1)=SUM+1
      DO 1300 J=PA(I+1)-1,PA(I),-1
      A(SUM)=A(J)
      SA(SUM)=SA(J)
      SUM=SUM-1
1300  CONTINUE
1400  CONTINUE
      DO 1700 I=1,N
      DO 1600 J=WN11(I)+PA(I)+2-PA(I+1),WN11(I)
      SA(PD(SA(J)))=I
      A(PD(SA(J)))=A(J)
      PD(SA(J))=PD(SA(J))+1
1600  CONTINUE
1700  CONTINUE
      DO 1800 I=1,N
      PA(I+1)=WN11(I)+1
1800  CONTINUE
      RETURN
      END
