* SUBROUTINE UXTQRV                ALL SYSTEMS                06/12/01
C PORTABILITY : ALL SYSTEMS
C 06/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* COMPUTATION OF THE EIGENVALUES OF A SYMMETRIC TRIDIAGONAL MATRIX
* BY THE RATIONAL QR METHOD.
*
* PARAMETERS :
*  II  N  ORDER OF THE TRIDIAGONAL MATRIX T.
*  RU  D(N)  ON INPUT DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T.
*         ON OUTPUT EIGENVALUES OF THE TRIDIAGONAL MATRIX T.
*  RU  E(N)  ON INPUT SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX T
*         ON POSITIONS FROM 2 TO N (E(1) IS ARBITRARY).
*  RO  IER  ERROR INDICATOR.
*
      SUBROUTINE UXTQRV(N,D,E,IER)
      INTEGER N,IER
      $FLOAT D(N),E(N)
      INTEGER I,II,J,L,L1,M,MML
      $FLOAT B,C,F,G,H,P,R,S,PARM
      IER=0
      PARM=1.0$P-16
      IF (N.LE.1) RETURN
      DO 102 I=2,N
      E(I-1)=E(I)
  102 CONTINUE
      F=0.0D0
      B=0.0D0
      E(N)=0.0D0
      DO 290 L=1,N
      J=0
      H=PARM*(ABS(D(L))+SQRT(E(L)))
      IF (B.LE.H) THEN
      B=H
      C=B*B
      ENDIF
      DO 110 M=L,N
      IF (E(M).LE.C) GO TO 120
  110 CONTINUE
  120 IF (M.LE.L) GO TO 210
  130 IF (J.GE.30) THEN
      IER=L
      RETURN
      ENDIF
      J=J+1
      L1=L+1
      S=SQRT(E(L))
      G=D(L)
      P=(D(L1)-G)/(2.0D0*S)
      R=SQRT(1.0D0+P*P)
      D(L)=S/(P+SIGN(R,P))
      H=G-D(L)
      DO 140 I=L1,N
      D(I)=D(I)-H
  140 CONTINUE
      F=F+H
      G=D(M)
      IF (G.EQ.0.0$P 0) G=B
      H=G
      S=0.0D0
      MML=M-L
      DO 200 II=1,MML
      I=M-II
      P=G*H
      R=P+E(I)
      E(I+1)=S*R
      S=E(I)/R
      D(I+1)=H+S*(H+D(I))
      G=D(I)-E(I)/G
      IF (G.EQ.0.0$P 0) G=B
      G=B
      H=G*P/R
  200 CONTINUE
      E(L)=S*G
      D(L)=H
      IF (H.EQ.0.0$P 0) GO TO 210
      IF (ABS(E(L)).LE.ABS(C/H)) GO TO 210
      E(L)=H*E(L)
      IF (E(L).NE.0.0$P 0) GO TO 130
  210 P=D(L)+F
      DO 230 II=2,L
      I=L+2-II
      IF (P.GE.D(I-1)) GO TO 270
      D(I)=D(I-1)
  230 CONTINUE
      I=1
  270 D(I)=P
  290 CONTINUE
      RETURN
      END
