* SUBROUTINE UXVJAC             ALL SYSTEMS                 02/04/04
C PORTABILITY : ALL SYSTEMS
C 02/04/04 VL : ORIGINAL VERSION
*
* PURPOSE :
*   JACOBI METHOD FOR 3*3 SYMMETRIC MATRICES.
*
* PARAMETERS :
*  II  N  DIMENSION OF MATRIX A (1, 2, OR 3).
*  RU  AQ(15)  WORKSPACE ARRAY:
*        ON INPUT:  AQ(1-3) N DIAGONAL ELEMENTS OF A.
*                   AQ(4-6) OFF-DIAGONAL ELEMENTS OF A (12,13,23).
*        ON OUTPUT: AQ(1-3) N EIGENVALUES OF A.
*                   AQ(7-15) N EIGENVECTORS OF A (N ELEMENTS EACH).
*  IU  K  ON INPUT: MAXIMUM NUMBER OF ITERATIONS.
*         ON OUTPUT: ACTUAL NUMBER OF ITERATIONS.
*
      SUBROUTINE UXVJAC(N,AQ,K)
      $FLOAT AQ(15),C,S,Q,W
      INTEGER N,K,I,L
      IF (K.LE.0) K=50
      L=K
      K=0
      DO 10 I=8,14
      AQ(I)=0.0$P 0
 10   CONTINUE
      AQ(7)= 1.0$P 0
      AQ(11)=1.0$P 0
      AQ(15)=1.0$P 0
      GOTO(1,2,3),N
  1   AQ(7)= 1.0$P 0
      RETURN
  2   CALL SC(AQ(1),AQ(4),AQ(2),S,C)
      AQ(7)= C
      AQ(8)=-S
      AQ(9)= S
      AQ(10)=C
      RETURN
  3   K=K+1
      W=MAX(ABS(AQ(1)),ABS(AQ(2)),ABS(AQ(3)))
      Q=MAX(ABS(AQ(4)),ABS(AQ(5)),ABS(AQ(6)))
      IF (Q.LT.1.0$P-16*W.OR.K.GT.L.OR.W.EQ.0.0$P 0) RETURN
      IF (Q.EQ.ABS(AQ(4))) THEN
        CALL SC(AQ(1),AQ(4),AQ(2),S,C)
        CALL ROTP(S,C,AQ(5),AQ(6),AQ(7),AQ(10))
      ELSE IF (Q.EQ.ABS(AQ(5))) THEN
        CALL SC(AQ(1),AQ(5),AQ(3),S,C)
        CALL ROTP(S,C,AQ(4),AQ(6),AQ(7),AQ(13))
      ELSE
        CALL SC(AQ(2),AQ(6),AQ(3),S,C)
        CALL ROTP(S,C,AQ(4),AQ(5),AQ(10),AQ(13))
      ENDIF
      GO TO 3
      END
      SUBROUTINE ROTP(S,C,A1,A2,Q1,Q2)
      REAL*8 A1,A2,C,S,Q1(3),Q2(3),W
      INTEGER I
      W=A1*C-A2*S
      A2=A1*S+A2*C
      A1=W
      DO 1 I=1,3
      W=Q1(I)*C-Q2(I)*S
      Q2(I)=Q1(I)*S+Q2(I)*C
      Q1(I)=W
   1  CONTINUE
      RETURN
      END
      SUBROUTINE SC(A1,A2,A3,S,C)
      REAL*8 A1,A2,A3,C,Q,S
      Q=(A3-A1)/(A2+A2)
      C=SQRT(0.5$P 0+0.5$P 0*SQRT(Q*Q/(1.0$P 0+Q*Q)))
      S=SIGN(0.5$P 0/(C*SQRT(1.0$P 0+Q*Q)),Q)
      Q= A1*C*C-A2*C*S*2.0$P 0+A3*S*S
      A3=A1*S*S+A2*C*S*2.0$P 0+A3*C*C
      A1=Q
      A2=0.0$P 0
      RETURN
      END
