$INTERFACE UY1DG1                                             98/12/01

  CALL UY1DG1(NF,N,$H,$GN,$XO,$GO,$S,$GAM1,GNORM,SNORM,XDEL,XMAX,PO&
,MES3)
  $ADD(REAL,'\$H($NF*($NF+1)/2)\$GN($NF)\$XO($NF)\$GO($NF)\$S($NF)')
$REM  $ADD(REAL,'\GNORM\SNORM\XDEL\XMAX\PO')
$REM  $ADD(INTEGER,'\NF\N')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UY1DG1                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* STEPSIZE CONTROL BY THE METHOD OF DENIS AND MEI.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  N  ACTUAL NUMBER OF VARIABLES.
*  RI  H(NF*(NF+1)/2)  DENSE SYMMETRIC MATRIX WHICH IS USED FOR
*         DETERMINATION OF THE DIRECTION VECTOR. IF IDECF .NE. 1 THEN
*         H IS FACTORED USING THE SUBROUTINE UXDPGF.
*  RI  G(NF) GRADIENT OF THE OBJECTIVE FUNCTION.
*  RI  XO(NF)  VECTOR OF VARIABLES DIFFERENCE.
*  RI  GO(NF)  GRADIENTS DIFFERENCE.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  GAM1  LOWER BOUND FOR STEPSIZE EXPANSION.
*  RO  GNORM  EUCLIDEAN NORM OF THE GRADIENT.
*  RO  SNORM  EUCLIDEAN NORM OF THE DIRECTION VECTOR.
*  RU  XDEL  TRUST REGION BOUND.
*  RI  XMAX  MAXIMUM STEPSIZE.
*  RI  PO  INITIAL VALUE OF THE DIRECTIONAL DERIVATIVE.
*  II  MES3 METHOD SPECIFICATION.
*
* COMMON DATA :
*  IO  ITERS  TERMINATION INDICATOR. ITERS=0-ZERO STEP. ITERS=1-STEP
*         BOUND WAS DECREASED. ITERS=2-STEP BOUND WAS UNCHANGED.
*         ITERS=3-STEP BOUND WAS INCREASED. ITERS=6-FIRST STEPSIZE.
*  IU  IDECF  DECOMPOSITION INDICATOR. IDECF=0-NO DECOMPOSITION.
*
* SUBPROGRAMS USED :
*  S   UXDPGM  MATRIX VECTOR PRODUCT AFTER GILL-MURRAY FACTORIZATION.
*  S   UXDSMM  MATRIX-VECTOR PRODUCT.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  RF  UXVDOT  DOT PRODUCT OF VECTORS.
*
* METHOD :
* J.E.DENNIS, H.H.W.MEI: TWO NEW UNCONSTRAINED OPTIMIZATION ALGORITHMS
* WHICH USE FUNCTION AND GRADIENT VALUES, J. OPTIMIZATION THEORY APPL.
* 28 (1979) PP. 453-482.
*
      SUBROUTINE UY1DG1(NF,N,H,G,XO,GO,S,GAM1,GNORM,SNORM,XDEL,XMAX,
     & PO,MES3)
      $INCLUDE ('UMCOMM')
      INTEGER NF,N,MES3
      $FLOAT H(NF*(NF+1)/2),G(NF),XO(NF),GO(NF),S(NF),GAM1,GNORM,SNORM,
     & XDEL,XMAX,PO
      $FLOAT UXVDOT
      $FLOAT CON
      PARAMETER (CON=5.0$P-1)
      IF (MES3.GE.1.AND.ITERS.EQ.2) THEN
      IF (UXVDOT(N,G,XO).LE.CON*PO) THEN
      XDEL=MAX(XDEL,GAM1*SNORM)
      ITERS = 3
      TSXX = 'SMALL   '
      ENDIF
      ENDIF
      IF (MES3.GE.2.AND.ITERS.EQ.2) THEN
      IF(IDECF.NE.1) THEN
        CALL UXDSMM(N,H,XO,S)
      ELSE
        CALL UXVCOP(N,XO,S)
        CALL UXDPGM(N,H,S,0)
      ENDIF
      CALL UXVDIF(N,GO,S,S)
      IF (UXVDOT(N,S,S).LE.(CON*GNORM)**2) THEN
      XDEL=MAX(XDEL,GAM1*SNORM)
      ITERS = 3
      TSXX = 'SMALL   '
      ENDIF
      ENDIF
      XDEL=MIN(XDEL,XMAX)
      RETURN
      END
