$INTERFACE UYFUT5                                             98/12/01

  $HELP
    RESTART SPECIFICATION. RESTART IS PERFORMED AFTER
    IRES1*N+IRES2 ITERATIONS.
  $CHECK(IRES1,999,INT,4)
  $HELP
    RESTART SPECIFICATION. RESTART IS PERFORMED AFTER
    IRES1*N+IRES2 ITERATIONS.
  $CHECK(IRES2,0,INT,4)
  $HELP
    MAXIMUM NUMBER OF TESTS ON STEPLENGTH
  $CHECK(MTESX,2,INT,4)
  $HELP
    MAXIMUM NUMBER OF TESTS ON FUNCTION DECREASE
  $CHECK(MTESF,2,INT,4)
  $HELP
    MAXIMUM NUMBER OF ITERATIONS
  $CHECK(MIT,1000,INT,3)
  $HELP
    MAXIMUM NUMBER OF COMPUTED FUNCTION VALUES
  $CHECK(MFV,1000,INT,3)
  $HELP
    MAXIMUM NUMBER OF COMPUTED GRADIENT VALUES
  $CHECK(MFG,10000,INT,3)
  $HELP
    LOWER BOUND FOR FINAL PRECISION
  $CHECK(TOLG,'1.0$P-6',REAL,3)
  $HELP
    LOWER BOUND FOR FUNCTION DECREASE
  $CHECK(TOLX,'1.0$P-12',REAL,3)
  $HELP
    LOWER BOUND FOR FUNCTION DECREASE
  $CHECK(TOLF,'1.0$P-12',REAL,3)
  $DEFAULT='-$ETA9'
  $IF($DEF(FMIN))
    $IF(FMIN='0.0$P 0') $DEFAULT='1.0$P-16'
  $ENDIF
  $HELP
    LOWER BOUND FOR FUNCTION VALUE
  $CHECK(TOLB,DEFAULT,REAL,3)
  CALL UYFUT5(NF,R,F,FO,SNORM,ITES,IRES1,IRES2,INEW)
$REM  $ADD(INTEGER,'\NF\ITES\INEW')
$REM  $ADD(REAL,'\R\F\FO\SNORM')
  $ADD(INIT)
    ITES=1
  $ENDADD
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYFUT5                ALL SYSTEMS                98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* TERMINATION CRITERIA AND TEST ON RESTART.
*
* PARAMETERS :
*  II  NF  DECLARED NUMBER OF VARIABLES.
*  RI  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  R  VALUE OF THE STEPSIZE PARAMETER.
*  RI  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RI  SNORM  NORM OF THE DIRECTION VECTOR.
*  II  ITES  SYSTEM VARIBLE WHICH SPECIFIES TERMINATION. IF ITES=0
*         THEN TERMINATION IS SUPPRESSED.
*  II  IRES1  RESTART SPECIFICATION. RESTART IS PERFORMED AFTER
*         IRES1*N+IRES2 ITERATIONS.
*  II  IRES2  RESTART SPECIFICATION. RESTART IS PERFORMED AFTER
*         IRES1*N+IRES2 ITERATIONS.
*  II  INEW  INDEX OF THE NEW ACTIVE CONSTRAINT.
*
* COMMON DATA :
*  II  ITERS  TERMINATION INDICATOR FOR STEPLENGTH DETERMINATION.
*         ITERS=0 FOR ZERO STEP.
*  IO  ITERM  TERMINATION INDICATOR. ITERM=1-TERMINATION AFTER MTESX
*         UNSUFFICIENT STEPLENGTHS. ITERM=2-TERMINATION AFTER MTESF
*         UNSUFFICIENT FUNCTION DECREASES. ITERM=3-TERMINATION ON LOWER
*         BOUND FOR FUNCTION VALUE. ITERM=4-TERMINATION ON LOWER BOUND
*         FOR GRADIENT. ITERM=11-TERMINATION AFTER MAXIMUM NUMBER OF
*         ITERATIONS. ITERM=12-TERMINATION AFTER MAXIMUM NUMBER OF
*         COMPUTED FUNCTION VALUES.
*  IU  IREST  RESTART INDICATOR. RESTART IS PERFORMED IF IREST>0.
*  II  MTESX  MAXIMUM NUMBER OF TESTS ON STEPLENGTH.
*  IU  NTESX  ACTUAL NUMBER OF TESTS ON STEPLENGTH.
*  II  MTESF  MAXIMUM NUMBER OF TESTS ON FUNCTION DECREASE.
*  IU  NTESF  ACTUAL NUMBER OF TESTS ON FUNCTION DECREASE.
*  II  LIT  NUMBER OF THE FIRST ITERATION.
*  II  MIT  MAXIMUM NUMBER OF ITERATIONS.
*  IU  NIT  ACTUAL NUMBER OF ITERATIONS.
*  II  LFV  NUMBER OF THE FIRST COMPUTED FUNCTION VALUE.
*  II  MFV  MAXIMUM NUMBER OF COMPUTED FUNCTION VALUES.
*  IU  NFV  ACTUAL NUMBER OF COMPUTED FUNCTION VALUES.
*  II  LFG  NUMBER OF THE FIRST COMPUTED GRADIENT VALUE.
*  II  MFG  MAXIMUM NUMBER OF COMPUTED GRADIENT VALUES.
*  IU  NFG  ACTUAL NUMBER OF COMPUTED GRADIENT VALUES.
*  II  KD  DEGREE OF REQUIRED DERIVATIVES.
*  II  JOUT  SYSTEM VARIABLE WHICH SPECIFIES PRINT OF INFORMATION
*         DURING THE OPTIMIZATION PROCESS. JOUT=0-NO PRINT.
*         JOUT=1-PRINT IS REQUIRED.
*  II  KOUT  LEVEL OF PRINT. KOUT=0-NO PRINT. KOUT=1-PRINT OF INITIAL
*         VALUES AND FINAL RESULTS TOGETHER WITH PRINT OF ITERATIONS.
*         KOUT=2-ADDITIONAL PRINT OF STEPSIZE SELECTION.
*         KOUT=3-ADDITIONAL PRINT OF DIRECTION DETERMINATION.
*         KOUT=4-ADDITIONAL PRINT OF MATRIX CORRECTION.
*         KOUT=5-ADDITIONAL PRINT OF NUMERICAL DIFFERENTIATION.
*         KOUT=6-ADDITIONAL PRINT OF DEBUGGING.
*  II  KOUT1  LOWER ITERATION IN WHICH INFORMATION IS PRINTED.
*  II  KOUT2  UPPER ITERATION IN WHICH INFORMATION IS PRINTED.
*  II  KOUT3  NUMBER OF ITERATIONS AFTER THAT INFORMATION IS PRINTED.
*  RI  TOLX  LOWER BOUND FOR STEPLENGTH.
*  RI  TOLC  LOWER BOUND FOR CONSTRAINT VIOLATION.
*  RI  TOLB  LOWER BOUND FOR FUNCTION VALUE.
*  RI  TOLG  LOWER BOUND FOR GRADIENT OF THE LAGRANGIAN FUNCTION.
*  TU  TPXX  TEXT INFORMATION ON THE CAUSE OF RESTART.
*  TO  TXFU  TEXT INFORMATION ON THE CAUSE OF TERMINATION.
*
* SUBPROGRAMS USED :
*  S   UYRES1  DEFINITION OF THE RESTART VARIABLES.
*
      SUBROUTINE UYFUT5(NF,R,F,FO,SNORM,ITES,IRES1,IRES2,INEW)
      $INCLUDE ('UMCOMM')
      INTEGER NF,ITES,IRES1,IRES2,INEW
      $FLOAT  R,F,FO,SNORM
      IF (ITERM.LT.0) RETURN
      IF (ITES.EQ.0) GOTO 2
      IF (ITES.LT.0) GOTO 1
      IF (NIT.LE.LIT.OR.INEW.NE.0) THEN
      NTESX = 0
      NTESF = 0
      ELSE
      IF (ITERS.GT.0) THEN
      IF (FO-F.LE.MAX(ABS(F),ONE)*TOLF) THEN
      ITERM=2
      TXFU='FV   TOL'
      NTESF=NTESF+1
      IF (NTESF.GE.MTESF) RETURN
      ELSE
      NTESF=0
      ENDIF
      ENDIF
      IF (SNORM*MIN(ONE,MAX(R,HALF)).LT.TOLG) THEN
      ITERM=4
      TXFU='GRAD TOL'
      NTESX=NTESX+1
      IF (ITERS.GT.0.AND.FO-F.LT.TEN*MAX(ABS(F),ONE)*TOLF) RETURN
      IF (NTESX.GE.2.AND.ITERS.EQ.0) RETURN
      ELSE
      NTESX=0
      ENDIF
      ENDIF
    1 IF (F.LE.TOLB) THEN
      ITERM=3
      TXFU='FV BOUND'
      RETURN
      ENDIF
      IF (NIT-LIT.GE.MIT) THEN
      ITERM=11
      TXFU='IT LIMIT'
      CALL UOERR1('UYFUT5',-1)
      RETURN
      ENDIF
      IF (NFV-LFV.GE.MFV) THEN
      ITERM=12
      TXFU='FV LIMIT'
      CALL UOERR1('UYFUT5',-2)
      RETURN
      ENDIF
      IF ( NFG-LFG .GE. MFG)  THEN
      ITERM = 13
      TXFU = 'FG LIMIT'
      CALL UOERR1('UYFUT1',-3)
      RETURN
      ENDIF
    2 ITERM=0
      TPXX='        '
      IF (NF.NE.0.AND.NIT-KIT.GE.IRES1*NF+IRES2) THEN
      CALL UYRES1('NORMAL  ')
      ENDIF
      NIT=NIT+1
      JOUT=0
      IF (KOUT.EQ.0.OR.KOUT3.LE.0) RETURN
      IF (KOUT1.LE.KOUT2) THEN
      IF(NIT-LIT.GE.KOUT1.AND.NIT-LIT.LE.KOUT2) JOUT=1
      ELSE
      IF(NIT-LIT.GE.KOUT1.OR. NIT-LIT.LE.KOUT2) JOUT=1
      ENDIF
      IF(NIT-LIT-(INT((NIT-LIT)/KOUT3)*KOUT3).NE.0) JOUT=0
      RETURN
      END
