* SUBROUTINE UYSORA               ALL SYSTEMS                08/10/01
C PORTABILITY : ALL SYSTEMS
C 08/10/01 MA : ORIGINAL VERSION
*
* PURPOSE :
* LSTRS METHOD: ADJUSTS THE PARAMETER ALPHA UNTIL A SUITABLE
* INTERPOLATION POINT CAN BE COMPUTED.
*
* PARAMETERS :
*  II  N  NUMBER OF VARIABLES.
*  II  M  NUMBER OF NONZERO ELEMENTS IN HESSIAN MATRIX.
*  II  NCV  NUMBER OF LANCZOS VECTORS.
*  RI  H(M)  ELEMENTS OF THE SPARSE SYMMETRIC MATRIX STORED IN THE
*      PACKED FORM.
*  II  IH(N+1) POINTERS OF THE HESSIAN MATRIX STORED IN A COMPACT FORM.
*  II  JH(M) COLUMN INDICES OF NONZEROES IN THE MATRIX H.
*  RI  G(N)  GRADIENT.
*  RI  GNORM  THE NORM OF A GRADIENT.
*  RU  ALPHA  SCALAR PARAMETER IN LSTRS METHOD.
*  RU  LAMBDA1  THE 1-ST SMALLEST EIGENVALUE OF THE BORDERED MATRIX.
*  RU  LAMBDA2  THE 2-ND SMALLEST EIGENVALUE OF THE BORDERED MATRIX.
*  RU  NU1  THE FIRST COMPONENT OF THE 1-ST EIGENVECTOR.
*  RU  NU2  THE FIRST COMPONENT OF THE 2-ND EIGENVECTOR.
*  RU  U1(N)  COMPONENTS (2,...,N+1) OF THE 1-ST EIGENVECTOR.
*  RU  U2(N)  COMPONENTS (2,...,N+1) OF THE 2-ND EIGENVECTOR.
*  RU  U1NORM  THE NORM OF THE VECTOR U1.
*  RU  U2NORM  THE NORM OF THE VECTOR U2.
*  RU  ALPHAL  LOWER BOUND FOR ALPHA.
*  RU  ALPHAU  UPPER BOUND FOR ALPHA.
*  RU  DELTAU  UPPER BOUND FOR THE SMALLEST EIGENVALUE OF H.
*  IU  NCONV  NUMBER OF CONVERGED EIGENVALUES OF THE BORDERED MATRIX.
*  IU  NMV  NUMBER OF MATRIX-VECTOR MULTIPLICATIONS.
*  LO  ISZ  IF WE HAVE A CORRECTION TERM.
*  RO  Z(N)  CORRECTION TERM.
*  RI  ETA4  STOPPING TOLERANCE FOR EIGENVALUES OF THE BORDERED MATRIX.
*  RI  ETA5  TOLERANCE DETERMINING SMALL INTERVAL FOR ALPHA.
*  RI  ETA6  TOLERANCE DETERMINING SMALL VALUES OF NU1 AND NU2.
*  RI  ETA7  TOLERANCE DETERMINING A POTENTIAL HARD CASE.
*  IO  ITER  WHICH EIGENPAIR WILL BE CONSIDERED (1-ST, 2-ND, NONE).
*  RA  RESID(N+1)  AUXILIARY ARRAY.
*  RA  V(N+1,NCV)  AUXILIARY ARRAY.
*  RA  WORKD(3*(N+1))  AUXILIARY ARRAY.
*  RA  WORKL(NCV*(NCV+8))  AUXILIARY ARRAY.
*  LA  SELECT(NCV)  AUXILIARY ARRAY.
*  IO  INFO  INFO<>0 - WRONG OUTPUT FROM ARPACK.
*
* SUBPROGRAMS USED :
*  S   UYSORE  COMPUTATION OF EIGENPAIRS USING ARPACK.
*  S   UXVCOP  COPYING OF A VECTOR.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*
      SUBROUTINE UYSORA(N,M,NCV,H,IH,JH,G,GNORM,ALPHA,LAMBDA1,LAMBDA2,
     & NU1,NU2,U1,U2,U1NORM,U2NORM,ALPHAL,ALPHAU,DELTAU,NCONV,NMV,ISZ,
     & Z,ETA4,ETA5,ETA6,ETA7,ITER,RESID,V,WORKD,WORKL,SELECT,INFO)
      $FLOAT TWO
      PARAMETER (TWO=2.0D0)
      INTEGER N,M,NCV,IH(N+1),JH(M),NCONV,NMV,ITER,INFO
      $FLOAT H(M),G(N),GNORM,ALPHA,LAMBDA1,LAMBDA2,NU1,NU2,U1(N),U2(N),
     & U1NORM,U2NORM,ALPHAL,ALPHAU,DELTAU,Z(N),ETA4,ETA5,ETA6,ETA7,
     & RESID(N+1),V(N+1,NCV),WORKD(3*(N+1)),WORKL(NCV*(NCV+8))
      LOGICAL ISZ,SELECT(NCV)
      $FLOAT DELTAN,UXVDOT
      LOGICAL ADJUST,SMALLA
      ITER = 0
      IF (NCONV.GT.0) THEN
      IF (ABS(NU1)*GNORM.LE.ETA7*U1NORM.AND..NOT.ISZ) THEN
      CALL UXVCOP(N,U1,Z)
      ISZ = .TRUE.
      ENDIF
      IF (ABS(NU1).LE.ETA6) THEN
      ADJUST = .TRUE.
      ELSE
      ADJUST = .FALSE.
      ITER = 1
      ENDIF
      IF (NCONV.GT.1.AND.ADJUST.AND.ABS(NU2).GT.ETA6) THEN
      ADJUST = .FALSE.
      ITER = 2
      ENDIF
      ELSE
      ADJUST = .TRUE.
      ENDIF
      IF (ABS(ALPHAU-ALPHAL).LE.
     & ETA5*MAX(ABS(ALPHAL),ABS(ALPHAU))) THEN
      SMALLA = .TRUE.
      ELSE
      SMALLA = .FALSE.
      ENDIF
      DO WHILE (ADJUST.AND..NOT.SMALLA)
      ALPHAU = ALPHA
      ALPHA = (ALPHAL + ALPHAU)/TWO
      IF (ABS(ALPHAU-ALPHAL).LE.
     & ETA5*MAX(ABS(ALPHAL),ABS(ALPHAU))) THEN
      SMALLA = .TRUE.
      ELSE
      SMALLA = .FALSE.
      ENDIF
*
*     EIGENPAIRS LAMBDA12,NU12,U12,U12NORM,NCONV
*
      CALL UYSORE(N+1,M,NCV,H,IH,JH,G,ALPHA,LAMBDA1,LAMBDA2,NU1,NU2,
     & U1,U2,U1NORM,U2NORM,NCONV,NMV,RESID,V,WORKD,WORKL,SELECT,ETA4,
     & 2,INFO)
      IF (NCONV.GT.0) THEN
      DELTAN = LAMBDA1 - UXVDOT(N,G,U1)*NU1/U1NORM
      DELTAU = MIN(DELTAU,DELTAN)
      IF (ABS(NU1)*GNORM.LE.ETA7*U1NORM.AND..NOT.ISZ) THEN
      CALL UXVCOP(N,U1,Z)
      ISZ = .TRUE.
      ENDIF
      IF (ABS(NU1).LE.ETA6) THEN
      ADJUST = .TRUE.
      ELSE
      ADJUST = .FALSE.
      ITER = 1
      ENDIF
      IF (NCONV.GT.1.AND.ADJUST.AND.ABS(NU2).GT.ETA6) THEN
      ADJUST = .FALSE.
      ITER = 2
      ENDIF
      ELSE
      ADJUST = .TRUE.
      ENDIF
      ENDDO
      RETURN
      END
