$INTERFACE UYTACD                                             98/12/01

  CALL UYTACD(NF,NA,MA,$X,$IX,$XN,$XO,$G,$GO,$AG,$AGO,$IAG,$JAG&
,R,F,FO,P,PO,DMAX)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$XO($NF)\$G($NF)\$GO($NF)&
  \$AG($MA)\$AGO($MA)')
  $ADD(INTEGER,'\$IX($NF)\$IAG($NA+1)\$JAG($MA)')
$REM  $ADD(REAL,'\R\FO\F\PO\P\DMAX')
$REM  $ADD(INTEGER,'\NF')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYTACD             ALL SYSTEMS                   98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* VECTORS OF VARIABLES DIFFERENCE AND GRADIENTS DIFFERENCE ARE COMPUTED
* AND SCALED. TEST VALUE DMAX IS DETERMINED.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS IN THE FIELD AG.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  II  IX(NF)  VECTOR CONTAINING TYPES OF BOUNDS.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RU  XO(NF)  VECTORS OF VARIABLES DIFFERENCE.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GO(NF)  GRADIENTS DIFFERENCE.
*  RI  AG(MA)  JACOBIAN MATRIX OF THE APPROXIMATED FUNCTIONS.
*  RO  AGO(MA)  SAVED JACOBIAN MATRIX OF THE APPROXIMATED FUNCTIONS.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RO  P  NEW VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PO  OLD VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*
* COMMON DATA :
*  II  ISNA  SAVING SPECIFICATION. ISNA=0-NO SAVING. ISNA=1-FUNCTION
*         VALUES ARE SAVED. ISNA=2-FUNCTION VALUES AND GRADIENTS ARE
*         SAVED.
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  II  ITERS  TERMINATION INDICATOR FOR STEPLENGTH DETERMINATION.
*         ITERS=0 FOR ZERO STEP.
*
* SUBPROGRAMS USED :
*  S   UYSET1  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVSAV  DIFFERENCE OF TWO VECTORS WITH COPYING AND SAVING THE
*         SUBSTRACTED ONE.
*
      SUBROUTINE UYTACD(NF,NA,MA,X,IX,XN,XO,G,GO,AG,AGO,IAG,JAG,
     & R,F,FO,P,PO,DMAX)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NA,MA,IX(NF),IAG(NA+1),JAG(MA)
      $FLOAT X(NF),XN(NF),XO(NF),G(NF),GO(NF),AG(MA),AGO(MA),
     & R,F,FO,P,PO,DMAX
      INTEGER I,J,K,L,JP
      IF (ITERS.GT.0) THEN
      CALL UXVDIF(NF,X,XO,XO)
      CALL UXVDIF(MA,AG,AGO,AGO)
      PO=R*PO
      P=R*P
      ELSE
      F = FO
      P = PO
      CALL UXVSAV(NF,X,XO)
      CALL UXVSAV(NF,G,GO)
      CALL UXVSAV(MA,AG,AGO)
      CALL UYSET1
      ENDIF
      DMAX = ZERO
      DO 1 I=1,NF
      IF (IX(I).LT.0) GO TO 1
      IF (NORMF.GT.0) THEN
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),XN(I)))
      XO(I)=XO(I)/XN(I)
      ELSE
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),ONE))
      ENDIF
    1 CONTINUE
      IF (NORMF.GT.0.AND.ISNA.LE.2) THEN
      DO 3 I=1,NA
      K=IAG(I)
      L=IAG(I+1)-K
      DO 2 J=1,L
      JP=JAG(K)
      IF (JP.GT.0) AGO(K)=AGO(K)*XN(JP)
      K=K+1
    2 CONTINUE
    3 CONTINUE
      ENDIF
      RETURN
      END
