$INTERFACE UYTBUB                                             98/12/01

  CALL UYTBUB(NF,NA,MA,MAH,$X,$XN,$XO,$G,$GO,$S,$AH,$IAG,$JAG,R,F&
,FO,P,PO,DMAX)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$XO($NF)\$G($NF)\$GO($NF)\$S($NF)&
\$AH($MAH)')
  $ADD(INTEGER,'\$IAG($NA+1)\$JAG($MA)')
$REM  $ADD(REAL,'\R\FO\F\PO\P\DMAX')
$REM  $ADD(INTEGER,'\NF\NA\MA\MAH')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYTBUB             ALL SYSTEMS                   98/12/01
C PORTABILITY : ALL SYSTEMS
C 98/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* VECTORS OF VARIABLES DIFFERENCE AND GRADIENTS DIFFERENCE ARE COMPUTED
* AND SCALED AND REDUCED. TEST VALUE DMAX IS DETERMINED.
*
* PARAMETERS :
*  II  NF DECLARED NUMBER OF VARIABLES.
*  II  NA  NUMBER OF APPROXIMATED FUNCTIONS.
*  II  MA  NUMBER OF NONZERO ELEMENTS IN THE FIELD AG.
*  II  M  NUMBER OF NONZERO ELEMENTS OF THE MATRIX.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RU  XO(NF)  VECTORS OF VARIABLES DIFFERENCE.
*  RI  G(NF)  GRADIENT OF THE OBJECTIVE FUNCTION.
*  RU  GO(NF)  GRADIENTS DIFFERENCE.
*  RA  S(NF)  AUXILIARY VECTOR.
*  RI  H(M)  HESSIAN MATRIX OF THE OBJECTIVE FUNCTION OR ITS
*         APPROXIMATION.
*  RI  IAG(NA+1)  POSITION OF THE FIRST ROWS ELEMENTS IN THE FIELD AG.
*  RI  JAG(MA)  COLUMN INDICES OF ELEMENTS IN THE FIELD AG.
*  RO  R  VALUE OF THE STEPSIZE PARAMETER.
*  RO  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RO  P  NEW VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PO  OLD VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*
* COMMON DATA :
*  II  NORMF  SCALING SPECIFICATION. NORMF=0-NO SCALING IS PERFORMED.
*         NORMF=1-SCALING FACTORS ARE DETERMINED AUTOMATICALLY.
*         NORMF=2-SCALING FACTORS ARE SUPPLIED BY USER.
*  II  ITERS  TERMINATION INDICATOR FOR STEPLENGTH DETERMINATION.
*         ITERS=0 FOR ZERO STEP.
*
* SUBPROGRAMS USED :
*  S   UYSET1  DEGREE DEFINITION OF THE COMPUTED DERIVATIVES.
*  S   UXBSMM  MATRIX VECTOR PRODUCT.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  S   UXVSAV  DIFFERENCE OF TWO VECTORS WITH COPYING AND SAVING THE
*         SUBSTRACTED ONE.
*
      SUBROUTINE UYTBUB(NF,NA,MA,M,X,XN,XO,G,GO,S,H,IAG,JAG,R,F,FO,
     & P,PO,DMAX)
      $INCLUDE ('UMCOMM')
      INTEGER NF,NA,MA,M,IAG(NA+1),JAG(MA)
      $FLOAT X(NF),XN(NF),XO(NF),G(NF),GO(NF),S(NF),H(M),R,F,FO,P,PO,
     & DMAX
      INTEGER I
      IF (ITERS.GT.0) THEN
      CALL UXVDIF(NF,X,XO,XO)
      CALL UXVDIF(NF,G,GO,GO)
      PO=R*PO
      P=R*P
      ELSE
      R = ZERO
      F = FO
      P = PO
      CALL UXVSAV(NF,X,XO)
      CALL UXVSAV(NF,G,GO)
      CALL UYSET1
      ENDIF
      CALL UXBSMM(NF,M,NA,MA,H,IAG,JAG,XO,S)
      DMAX = ZERO
      DO 1 I=1,NF
      IF ( NORMF .GT. 0) THEN
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),XN(I)))
      XO(I)=XO(I)/XN(I)
      GO(I)=GO(I)*XN(I)
      S(I)=S(I)*XN(I)
      ELSE
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),ONE))
      ENDIF
    1 CONTINUE
      CALL UXVDIF(NF,GO,S,GO)
      RETURN
      END
