$INTERFACE UYTHUD                                             02/12/01

  $HELP
    TOLERANCE FOR LINEAR INDEPENDENCE.
  $CHECK(EPS7,'1.0$P-5',REAL,4)
  $ADD(REAL,'\$X($NF)\$XN($NF)\$XM($NF*$MF)\$XR($MF*($MF+1)/2)&
  \$HR($MF*($MF+1)/2)\$G($NF)\$GR($MF)\$SR($MF)\$XO($NF)&
  \$GO($NF)')
$REM  $ADD(REAL,'\R\FO\F\PO\P\DMAX')
$REM  $ADD(INTEGER,'\NF\MF\INF')
  CALL UYTHUD(NF,NN,MF,$X,$XN,$XM,$XR,$HR,$G,$GR,$SR,$XO,$GO,R,F,FO&
,P,PO,DMAX,GNORM,$EPS7,MOS,MOS1,MOS2,INF,'$DECOMP')
  $EXIT

* IMPLEMENTATION

* SUBROUTINE UYTHUD                ALL SYSTEMS                02/12/01
C PORTABILITY : ALL SYSTEMS
C 02/12/01 LU : ORIGINAL VERSION
*
* PURPOSE :
* LIMITED STORAGE REDUCED HESSIAN VARIABLE METRIC TRANSFORMATIONS.
*
* PARAMETERS :
*  II  NF  NUMBER OF VARIABLES.
*  IU  N  CURRENT NUMBER OF VARIABLE METRIC STEPS.
*  II  MF  MAXIMUM NUMBER OF VARIABLE METRIC STEPS.
*  RI  X(NF)  VECTOR OF VARIABLES.
*  RI  XN(NF)  VECTOR OF SCALING FACTORS.
*  RU  XM(NF*MF)  SET OF VECTOR OF VARIABLES DIFFERENCES.
*  RU  XR(MF*(MF+1)/2)  CHOLESKI DECOMPOSITION OF THE REDUCED
*         HESSIAN MATRIX.
*  RU  HR(MF*(MF+1)/2)  TRIANGULAR TRANFORMATION MATRIX.
*  RI  G(NF) GRADIENT VECTOR.
*  RO  GR(MF) REDUCED GRADIENT VECTOR.
*  RI  SR(MF) REDUCED DIRECTION VECTOR.
*  RU  XO(NF)  REDUCED VECTOR OF VARIABLES DIFFERENCE.
*  RU  GO(NF)  REDUCED GRADIENT DIFFERENCE.
*  RI  R  STEPSIZE PARAMETER.
*  RO  F  NEW VALUE OF THE OBJECTIVE FUNCTION.
*  RI  FO  OLD VALUE OF THE OBJECTIVE FUNCTION.
*  RO  P  NEW VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RI  PO  OLD VALUE OF THE DIRECTIONAL DERIVATIVE.
*  RO  DMAX  MAXIMUM RELATIVE DIFFERENCE OF VARIABLES.
*  RI  GNORM  NORM OF THE GRADIENT VECTOR.
*  RI  EPS7  TOLERANCE FOR LINEAR DEPENDENCE.
*  II  MOS  OPTION. MOS=1-INVERSION OF REDUCED HESSIAN MATRIX.
*         MOS=2-REDUCED INVERSION OF HESSIAN MATRIX.
*  II  MOS1  OPTION. MOS1=1-UPDATE AFTER ORTHOGONAL TRANSFORMATIONS.
*         MOS1=2-UPDATE BEFORE ORTHOGONAL TRANSFORMATIONS.
*         MOS1=3-ORTHOGONAL TRANSFORMATIONS SUPPRESSED.
*  IO  INF  LINEAR DEPENDENCE INDICATOR. INF=0 IF THE CURRENT GRADIENT
*         IS A LINEAR COMBINATION OF BASIC VECTORS. INF=1 OTHERWISE.
*  CO  TEXT  TYPE OF DECOMPOSITION. TEXT='R'-RECURSIVE QR DECOMPOSITION
*         WITHOUT PERMUTATIONS. TEXT='I'-INVERSION.
*
* SUBPROGRAMS USED :
*  S   UXDPRB  BACK SUBSTITUTION AFTER CHOLESKI DECOMPOSITION.
*  S   UXDRMM  VECTOR MULTIPLIED BY A ROWWISE STORED RECTANGULAR
*         MATRIX.
*  S   UXVCOP  COPYING OF A VECTOR.
*  S   UXVDIF  DIFFERENCE OF TWO VECTORS.
*  RF  UXVDOT  DOT PRODUCT OF TWO VECTORS.
*  S   UXVSCL  SCALING OF A VECTOR.
*  S   UXVSET  INITIATION OF A VECTOR.
*
      SUBROUTINE UYTHUD(NF,N,MF,X,XN,XM,XR,HR,G,GR,SR,XO,GO,R,F,FO,
     & P,PO,DMAX,GNORM,EPS7,MOS,MOS1,MOS2,INF,TEXT)
      $INCLUDE('UMCOMM')
      INTEGER NF,N,MF,MOS,MOS1,MOS2,INF
      $FLOAT X(NF),XN(NF),XM(NF*MF),XR(MF*(MF+1)/2),HR(MF*(MF+1)/2),
     & G(NF),GR(MF),SR(MF),XO(NF),GO(NF),R,F,FO,P,PO,DMAX,GNORM,EPS7
      CHARACTER*1 TEXT
      $FLOAT RHO,TEMP,UXVDOT
      INTEGER I,IER
      IF (ITERS.GT.0) THEN
      CALL UXVDIF(NF,X,XO,XO)
      CALL UXVDIF(NF,G,GO,GO)
      PO=R*PO
      P=R*P
      ELSE
      F = FO
      P = PO
      CALL UXVSAV(NF,X,XO)
      CALL UXVSAV(NF,G,XO)
      CALL UYSET1
      ENDIF
      DMAX = ZERO
      DO 1 I=1,NF
      IF ( NORMF .GT. 0) THEN
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),XN(I)))
      XO(I)=XO(I)/XN(I)
      GO(I)=GO(I)*XN(I)
      ELSE
      DMAX=MAX(DMAX,ABS(XO(I))/MAX(ABS(X(I)),ONE))
      ENDIF
    1 CONTINUE
      TEMP=UXVDOT(NF,G,GO)
      CALL UXVSCL(N,R,SR,XO)
      CALL UXVCOP(N,GR,GO)
      CALL UXDRMM(NF,N,XM,G,GR)
      CALL UXDPRB(N,HR,GR,1)
      CALL UXVDIF(N,GR,GO,GO)
      IF (ITERS.LE.0.OR.MOS1.GE.2) RETURN
      RHO=SQRT(ABS(UXVDOT(NF,G,G)-UXVDOT(N,GR,GR)))
      IF (RHO.LE.EPS7*GNORM) THEN
      INF=0
      ELSE
      INF=1
      IF (TEXT.EQ.'R') THEN
      CALL UXDPRH(N,XR,HR,GR,XO,GO,IER,3)
      ELSE
      CALL UXDSMH(N,XR,HR,GR,XO,GO,IER,MOS,3)
      ENDIF
      DO 2 I=1,NF*(N-1)
      XM(I)=XM(I+NF)
    2 CONTINUE
      IF (MOS2.EQ.1) THEN
      RHO=SQRT(ABS(UXVDOT(NF,G,G)-UXVDOT(N-1,GR,GR)))
      ELSE
      CALL UXVCOP(N-1,GR,HR(N*(N-1)/2+1))
      CALL UXDPRB(N-1,HR,HR(N*(N-1)/2+1),-1)
      CALL UXDCMD(NF,N-1,XM,HR(N*(N-1)/2+1),-ONE,G,XM((N-1)*NF+1))
      RHO=SQRT(UXVDOT(NF,XM((N-1)*NF+1),XM((N-1)*NF+1)))
      ENDIF
      TEMP=TEMP-UXVDOT(N-1,GR,GO)
      GR(N)=RHO
      XO(N)=ZERO
      GO(N)=TEMP/RHO
      CALL UXVCOP(NF,G,XM((N-1)*NF+1))
      CALL UXVCOP(N,GR,HR(N*(N-1)/2+1))
      ENDIF
      RETURN
      END
